@@ -125,6 +125,8 @@ namespace Commons.Xml.Relaxng.Derivative
 				return p2;
 			if (p2.PatternType == RelaxngPatternType.NotAllowed)
 				return p1;
+			if (p1 == p2)
+				return p1;
 			// choice-leaves support
 			if (p1.PatternType == RelaxngPatternType.Empty)
 				return MakeChoiceLeaf (p2);
@@ -290,6 +292,14 @@ namespace Commons.Xml.Relaxng.Derivative
 
 		public abstract bool Nullable { get; }
 
+		internal virtual bool IsTextValueDependent {
+			get { return false; }
+		}
+
+		internal virtual bool IsContextDependent {
+			get { return false; }
+		}
+
 		// fills QName collection
 		public void GetLabels (LabelList elements, LabelList attributes)
 		{
@@ -322,16 +332,36 @@ namespace Commons.Xml.Relaxng.Derivative
 			return RdpNotAllowed.Instance;
 		}
 
+		internal virtual RdpPattern TextDeriv (string s, XmlReader reader, MemoizationStore memo)
+		{
+			return TextDeriv (s, reader);
+		}
+
+		internal virtual RdpPattern EmptyTextDeriv (MemoizationStore memo)
+		{
+			return TextDeriv (String.Empty, null, memo);
+		}
+
 		internal virtual RdpPattern TextOnlyDeriv ()
 		{
 			return this;
 		}
 
+		internal virtual RdpPattern TextOnlyDeriv (MemoizationStore store)
+		{
+			return this;
+		}
+
 		internal virtual RdpPattern MixedTextDeriv ()
 		{
 			return RdpNotAllowed.Instance;
 		}
 
+		internal virtual RdpPattern MixedTextDeriv (MemoizationStore memo)
+		{
+			return RdpNotAllowed.Instance;
+		}
+
 		public RdpPattern ListDeriv (string [] list, int index, XmlReader reader)
 		{
 			return listDerivInternal (list, 0, reader);
@@ -407,6 +437,11 @@ namespace Commons.Xml.Relaxng.Derivative
 			return RdpNotAllowed.Instance;
 		}
 
+		internal virtual RdpPattern StartTagOpenDeriv (string name, string ns, MemoizationStore memo)
+		{
+			return StartTagOpenDeriv (name, ns);
+		}
+
 		// attDeriv(ctx, this, att)
 		// attDeriv _ _ _ = NotAllowed
 		public virtual RdpPattern AttDeriv (string name, string ns, string value, XmlReader reader)
@@ -414,6 +449,26 @@ namespace Commons.Xml.Relaxng.Derivative
 			return RdpNotAllowed.Instance;
 		}
 
+		public virtual RdpPattern StartAttDeriv (string name, string ns)
+		{
+			return RdpNotAllowed.Instance;
+		}
+
+		internal virtual RdpPattern StartAttDeriv (string name, string ns, MemoizationStore memo)
+		{
+			return StartAttDeriv (name, ns);
+		}
+
+		public virtual RdpPattern EndAttDeriv ()
+		{
+			return RdpNotAllowed.Instance;
+		}
+
+		internal virtual RdpPattern EndAttDeriv (MemoizationStore memo)
+		{
+			return EndAttDeriv ();
+		}
+
 		public bool ValueMatch (string s, XmlReader reader)
 		{
 			return Nullable && Util.IsWhitespace (s) ||
@@ -425,6 +480,11 @@ namespace Commons.Xml.Relaxng.Derivative
 			return this;
 		}
 
+		internal virtual RdpPattern StartTagCloseDeriv (MemoizationStore memo)
+		{
+			return StartTagCloseDeriv ();
+		}
+
 		public RdpPattern OneOrMore ()
 		{
 			if (PatternType == RelaxngPatternType.NotAllowed)
@@ -437,6 +497,11 @@ namespace Commons.Xml.Relaxng.Derivative
 		{
 			return RdpNotAllowed.Instance;
 		}
+
+		internal virtual RdpPattern EndTagDeriv (MemoizationStore memo)
+		{
+			return EndTagDeriv ();
+		}
 		#endregion
 	}
 
@@ -453,6 +518,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return true; }
 		}
 
+		internal override bool IsTextValueDependent {
+			get { return false; }
+		}
+
+		internal override bool IsContextDependent {
+			get { return false; }
+		}
+
 		static RdpEmpty instance;
 		public static RdpEmpty Instance {
 			get { return instance; }
@@ -506,6 +579,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return false; }
 		}
 
+		internal override bool IsTextValueDependent {
+			get { return false; }
+		}
+
+		internal override bool IsContextDependent {
+			get { return false; }
+		}
+
 		public override RdpPattern ApplyAfter (RdpApplyAfterHandler h)
 		{
 			return RdpNotAllowed.Instance;
@@ -558,6 +639,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return true; }
 		}
 
+		internal override bool IsTextValueDependent {
+			get { return false; }
+		}
+
+		internal override bool IsContextDependent {
+			get { return false; }
+		}
+
 		public override RelaxngPatternType PatternType {
 			get { return RelaxngPatternType.Text; }
 		}
@@ -571,11 +660,21 @@ namespace Commons.Xml.Relaxng.Derivative
 			return this;
 		}
 
+		internal override RdpPattern EmptyTextDeriv (MemoizationStore memo)
+		{
+			return this;
+		}
+
 		internal override RdpPattern MixedTextDeriv ()
 		{
 			return this;
 		}
 
+		internal override RdpPattern MixedTextDeriv (MemoizationStore memo)
+		{
+			return this;
+		}
+
 		internal override void MarkReachableDefs () 
 		{
 			// do nothing
@@ -712,6 +811,32 @@ namespace Commons.Xml.Relaxng.Derivative
 			}
 		}
 
+		bool isTextValueDependentComputed;
+		bool isTextValueDependent;
+
+		internal override bool IsTextValueDependent {
+			get {
+				if (!isTextValueDependentComputed) {
+					isTextValueDependent = LValue.IsTextValueDependent || RValue.IsTextValueDependent;
+					isTextValueDependentComputed = true;
+				}
+				return isTextValueDependent;
+			}
+		}
+
+		bool isContextDependentComputed;
+		bool isContextDependent;
+
+		internal override bool IsContextDependent {
+			get {
+				if (!isContextDependentComputed) {
+					isContextDependent = LValue.IsContextDependent || RValue.IsContextDependent;
+					isContextDependentComputed = true;
+				}
+				return isContextDependent;
+			}
+		}
+
 		public override RelaxngPatternType PatternType {
 			get { return RelaxngPatternType.Choice; }
 		}
@@ -774,18 +899,41 @@ namespace Commons.Xml.Relaxng.Derivative
 			return LValue.TextDeriv (s, reader).Choice (RValue.TextDeriv (s, reader));
 		}
 
+		internal override RdpPattern TextDeriv (string s, XmlReader reader, MemoizationStore memo)
+		{
+			return memo.TextDeriv (LValue, s, reader).Choice (memo.TextDeriv (RValue, s, reader));
+		}
+
+		internal override RdpPattern EmptyTextDeriv (MemoizationStore memo)
+		{
+			return memo.EmptyTextDeriv (LValue)
+				.Choice (memo.EmptyTextDeriv (RValue));
+		}
+
 		internal override RdpPattern TextOnlyDeriv ()
 		{
 			return LValue.TextOnlyDeriv ().Choice (
 				RValue.TextOnlyDeriv ());
 		}
 
+		internal override RdpPattern TextOnlyDeriv (MemoizationStore memo)
+		{
+			return memo.TextOnlyDeriv (LValue).Choice (
+				memo.TextOnlyDeriv (RValue));
+		}
+
 		internal override RdpPattern MixedTextDeriv ()
 		{
 			return LValue.MixedTextDeriv ().Choice (
 				RValue.MixedTextDeriv ());
 		}
 
+		internal override RdpPattern MixedTextDeriv (MemoizationStore memo)
+		{
+			return memo.MixedTextDeriv (LValue).Choice (
+				memo.MixedTextDeriv (RValue));
+		}
+
 		public override RdpPattern ApplyAfter (RdpApplyAfterHandler handler)
 		{
 //			return handler (LValue).Choice (handler (RValue));
@@ -804,6 +952,12 @@ namespace Commons.Xml.Relaxng.Derivative
 #endif
 		}
 
+		internal override RdpPattern StartTagOpenDeriv (string name, string ns, MemoizationStore memo)
+		{
+			RdpPattern lDeriv = memo.StartTagOpenDeriv (LValue, name, ns);
+			return lDeriv.Choice (memo.StartTagOpenDeriv (RValue, name, ns));
+		}
+
 		// attDeriv cx (Choice p1 p2) att =
 		//  choice (attDeriv cx p1 att) (attDeriv cx p2 att)
 		public override RdpPattern AttDeriv (string name, string ns, string value, XmlReader reader)
@@ -812,6 +966,28 @@ namespace Commons.Xml.Relaxng.Derivative
 				.Choice (RValue.AttDeriv (name, ns, value, reader));
 		}
 
+		public override RdpPattern StartAttDeriv (string name, string ns)
+		{
+			RdpPattern lDeriv = LValue.StartAttDeriv (name, ns);
+			return lDeriv.Choice (RValue.StartAttDeriv (name, ns));
+		}
+
+		internal override RdpPattern StartAttDeriv (string name, string ns, MemoizationStore memo)
+		{
+			return memo.StartAttDeriv (LValue, name, ns)
+				.Choice (memo.StartAttDeriv (RValue, name, ns));
+		}
+
+		public override RdpPattern EndAttDeriv ()
+		{
+			return LValue.EndAttDeriv ().Choice (RValue.EndAttDeriv ());
+		}
+
+		internal override RdpPattern EndAttDeriv (MemoizationStore memo)
+		{
+			return memo.EndAttDeriv (LValue).Choice (memo.EndAttDeriv (RValue));
+		}
+
 		// startTagCloseDeriv (Choice p1 p2) =
 		//  choice (startTagCloseDeriv p1) (startTagCloseDeriv p2)
 		public override RdpPattern StartTagCloseDeriv ()
@@ -820,11 +996,22 @@ namespace Commons.Xml.Relaxng.Derivative
 				.Choice (RValue.StartTagCloseDeriv ());
 		}
 
+		internal override RdpPattern StartTagCloseDeriv (MemoizationStore memo)
+		{
+			return memo.StartTagCloseDeriv (LValue)
+				.Choice (memo.StartTagCloseDeriv (RValue));
+		}
+
 		public override RdpPattern EndTagDeriv ()
 		{
 			return LValue.EndTagDeriv ().Choice (RValue.EndTagDeriv ());
 		}
 
+		internal override RdpPattern EndTagDeriv (MemoizationStore memo)
+		{
+			return memo.EndTagDeriv (LValue).Choice (memo.EndTagDeriv (RValue));
+		}
+
 		internal override void CheckConstraints (bool attribute, bool oneOrMore, bool oneOrMoreGroup, bool oneOrMoreInterleave, bool list, bool dataExcept)
 		{
 			LValue.CheckConstraints (attribute, oneOrMore, oneOrMoreGroup, oneOrMoreInterleave, list, dataExcept);
@@ -850,6 +1037,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			}
 		}
 
+		internal override bool IsTextValueDependent {
+			get { return LValue.IsTextValueDependent || RValue.IsTextValueDependent; }
+		}
+
+		internal override bool IsContextDependent {
+			get { return LValue.IsContextDependent || RValue.IsContextDependent; }
+		}
+
 		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
 			LValue.GetLabels (elements, attributes, collectNameClass);
@@ -879,16 +1074,40 @@ namespace Commons.Xml.Relaxng.Derivative
 				.Choice (LValue.Interleave (RValue.TextDeriv (s, reader)));
 		}
 
+		internal override RdpPattern TextDeriv (string s, XmlReader reader, MemoizationStore memo)
+		{
+			return memo.TextDeriv (LValue, s, reader).Interleave (RValue)
+				.Choice (LValue.Interleave (memo.TextDeriv (RValue, s, reader)));
+		}
+
+		internal override RdpPattern EmptyTextDeriv (MemoizationStore memo)
+		{
+			return memo.EmptyTextDeriv (LValue).Interleave (RValue)
+				.Choice (LValue.Interleave (memo.EmptyTextDeriv (RValue)));
+		}
+
 		internal override RdpPattern TextOnlyDeriv ()
 		{
 			return LValue.TextOnlyDeriv ().Interleave (
 				RValue.TextOnlyDeriv ());
 		}
 
+		internal override RdpPattern TextOnlyDeriv (MemoizationStore memo)
+		{
+			return memo.TextOnlyDeriv (LValue).Interleave (
+				memo.TextOnlyDeriv (RValue));
+		}
+
 		internal override RdpPattern MixedTextDeriv ()
 		{
-			return MakeInterleave (LValue.MixedTextDeriv (), RValue).Choice (
-				MakeInterleave (LValue, RValue.MixedTextDeriv ()));
+			return LValue.MixedTextDeriv ().Interleave (RValue).Choice (
+				LValue.Interleave (RValue.MixedTextDeriv ()));
+		}
+
+		internal override RdpPattern MixedTextDeriv (MemoizationStore memo)
+		{
+			return memo.MixedTextDeriv (LValue).Interleave (RValue).Choice (
+				LValue.Interleave (memo.MixedTextDeriv (RValue)));
 		}
 
 		// => choice (applyAfter (flip interleave p2) (startTagOpenDeriv p1 qn)) (applyAfter (interleave p1) (startTagOpenDeriv p2 qn)
@@ -903,6 +1122,16 @@ namespace Commons.Xml.Relaxng.Derivative
 			return choiceL.Choice (choiceR);
 		}
 
+		internal override RdpPattern StartTagOpenDeriv (string name, string ns, MemoizationStore memo)
+		{
+			RdpPattern handledL = memo.StartTagOpenDeriv (LValue, name, ns);
+			RdpPattern handledR = memo.StartTagOpenDeriv (RValue, name, ns);
+			RdpFlip flipL = MakeFlip (RdpUtil.InterleaveFunction, RValue);
+			RdpPattern choiceL = handledL.ApplyAfter (new RdpApplyAfterHandler (flipL.Apply));
+			RdpPattern choiceR = handledR.ApplyAfter (new RdpApplyAfterHandler (LValue.Interleave));
+			return choiceL.Choice (choiceR);
+		}
+
 		// attDeriv cx (Interleave p1 p2) att =
 		//  choice (interleave (attDeriv cx p1 att) p2)
 		//         (interleave p1 (attDeriv cx p2 att))
@@ -914,6 +1143,26 @@ namespace Commons.Xml.Relaxng.Derivative
 					RValue.AttDeriv (name, ns, value, reader)));
 		}
 
+		public override RdpPattern StartAttDeriv (string name, string ns)
+		{
+			RdpPattern handledL = LValue.StartAttDeriv (name, ns);
+			RdpPattern handledR = RValue.StartAttDeriv (name, ns);
+			RdpFlip flipL = MakeFlip (RdpUtil.InterleaveFunction, RValue);
+			RdpPattern choiceL = handledL.ApplyAfter (new RdpApplyAfterHandler (flipL.Apply));
+			RdpPattern choiceR = handledR.ApplyAfter (new RdpApplyAfterHandler (LValue.Interleave));
+			return choiceL.Choice (choiceR);
+		}
+
+		internal override RdpPattern StartAttDeriv (string name, string ns, MemoizationStore memo)
+		{
+			RdpPattern handledL = memo.StartAttDeriv (LValue, name, ns);
+			RdpPattern handledR = memo.StartAttDeriv (RValue, name, ns);
+			RdpFlip flipL = MakeFlip (RdpUtil.InterleaveFunction, RValue);
+			RdpPattern choiceL = handledL.ApplyAfter (new RdpApplyAfterHandler (flipL.Apply));
+			RdpPattern choiceR = handledR.ApplyAfter (new RdpApplyAfterHandler (LValue.Interleave));
+			return choiceL.Choice (choiceR);
+		}
+
 		// startTagCloseDeriv (Interleave p1 p2) =
 		//  interleave (startTagCloseDeriv p1) (startTagCloseDeriv p2)
 		public override RdpPattern StartTagCloseDeriv ()
@@ -922,6 +1171,12 @@ namespace Commons.Xml.Relaxng.Derivative
 				.Interleave (RValue.StartTagCloseDeriv ());
 		}
 
+		internal override RdpPattern StartTagCloseDeriv (MemoizationStore memo)
+		{
+			return memo.StartTagCloseDeriv (LValue)
+				.Interleave (memo.StartTagCloseDeriv (RValue));
+		}
+
 		public override RelaxngPatternType PatternType {
 			get { return RelaxngPatternType.Interleave; }
 		}
@@ -964,6 +1219,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			}
 		}
 
+		internal override bool IsTextValueDependent {
+			get { return LValue.IsTextValueDependent || (LValue.Nullable ? RValue.IsTextValueDependent : false); }
+		}
+
+		internal override bool IsContextDependent {
+			get { return LValue.IsContextDependent || (LValue.Nullable ? RValue.IsContextDependent : false); }
+		}
+
 		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
 			LValue.GetLabels (elements, attributes, collectNameClass);
@@ -975,15 +1238,43 @@ namespace Commons.Xml.Relaxng.Derivative
 
 		public override RdpPattern TextDeriv (string s, XmlReader reader)
 		{
-			RdpPattern p = LValue.TextDeriv (s, reader).Group (RValue);
+			RdpPattern p = LValue.TextDeriv (s, reader);
+			p = (p.PatternType == RelaxngPatternType.NotAllowed) ?
+				p : p.Group (RValue);
 			return LValue.Nullable ?
 				p.Choice (RValue.TextDeriv (s, reader)) : p;
 		}
 
+		internal override RdpPattern TextDeriv (string s, XmlReader reader, MemoizationStore memo)
+		{
+			RdpPattern p = memo.TextDeriv (LValue, s, reader);
+			p = (p.PatternType == RelaxngPatternType.NotAllowed) ?
+				p : p.Group (RValue);
+			return LValue.Nullable ?
+				p.Choice (memo.TextDeriv (RValue, s, reader)) : p;
+		}
+
+		internal override RdpPattern EmptyTextDeriv (MemoizationStore memo)
+		{
+			RdpPattern p = memo.EmptyTextDeriv (LValue);
+			p = p.PatternType == RelaxngPatternType.NotAllowed ?
+				p : p.Group (RValue);
+			return LValue.Nullable ?
+				p.Choice (memo.EmptyTextDeriv (RValue)) : p;
+		}
+
 		internal override RdpPattern TextOnlyDeriv ()
 		{
-			return LValue.TextOnlyDeriv ().Group (
-				RValue.TextOnlyDeriv ());
+			RdpPattern p = LValue.TextOnlyDeriv ();
+			return p.PatternType == RelaxngPatternType.NotAllowed ?
+				p : p.Group (RValue.TextOnlyDeriv ());
+		}
+
+		internal override RdpPattern TextOnlyDeriv (MemoizationStore memo)
+		{
+			RdpPattern p = memo.TextOnlyDeriv (LValue);
+			return p.PatternType == RelaxngPatternType.NotAllowed ?
+				p : p.Group (memo.TextOnlyDeriv (RValue));
 		}
 
 		internal override RdpPattern MixedTextDeriv ()
@@ -993,6 +1284,13 @@ namespace Commons.Xml.Relaxng.Derivative
 				p.Choice (RValue.MixedTextDeriv ()) : p;
 		}
 
+		internal override RdpPattern MixedTextDeriv (MemoizationStore memo)
+		{
+			RdpPattern p = memo.MixedTextDeriv (LValue).Group (RValue);
+			return LValue.Nullable ?
+				p.Choice (memo.MixedTextDeriv (RValue)) : p;
+		}
+
 		// startTagOpenDeriv (Group p1 p2) qn =
 		//  let x = applyAfter (flip group p2) (startTagOpenDeriv p1 qn)
 		//  in if nullable p1 then
@@ -1010,6 +1308,17 @@ namespace Commons.Xml.Relaxng.Derivative
 				return x;
 		}
 
+		internal override RdpPattern StartTagOpenDeriv (string name, string ns, MemoizationStore memo)
+		{
+			RdpPattern handled = memo.StartTagOpenDeriv (LValue, name, ns);
+			RdpFlip f = MakeFlip (RdpUtil.GroupFunction, RValue);
+			RdpPattern x = handled.ApplyAfter (new RdpApplyAfterHandler (f.Apply));
+			if (LValue.Nullable)
+				return x.Choice (memo.StartTagOpenDeriv (RValue, name, ns));
+			else
+				return x;
+		}
+
 		// attDeriv cx (Group p1 p2) att =
 		//  choice (group (attDeriv cx p1 att) p2)
 		//         (group p1 (attDeriv cx p2 att))
@@ -1020,12 +1329,41 @@ namespace Commons.Xml.Relaxng.Derivative
 					RValue.AttDeriv (name, ns, value, reader)));
 		}
 
+		// startAttDeriv (group p1 p2) == startAttDeriv (interleave p1 p2)
+		public override RdpPattern StartAttDeriv (string name, string ns)
+		{
+			RdpPattern handledL = LValue.StartAttDeriv (name, ns);
+			RdpPattern handledR = RValue.StartAttDeriv (name, ns);
+			RdpFlip flipL = MakeFlip (RdpUtil.GroupFunction, RValue);
+			RdpPattern choiceL = handledL.ApplyAfter (new RdpApplyAfterHandler (flipL.Apply));
+			RdpPattern choiceR = handledR.ApplyAfter (new RdpApplyAfterHandler (LValue.Group));
+			return choiceL.Choice (choiceR);
+		}
+
+		internal override RdpPattern StartAttDeriv (string name, string ns, MemoizationStore memo)
+		{
+			RdpPattern handledL = memo.StartAttDeriv (LValue, name, ns);
+			RdpPattern handledR = memo.StartAttDeriv (RValue, name, ns);
+			RdpFlip flipL = MakeFlip (RdpUtil.GroupFunction, RValue);
+			RdpPattern choiceL = handledL.ApplyAfter (new RdpApplyAfterHandler (flipL.Apply));
+			RdpPattern choiceR = handledR.ApplyAfter (new RdpApplyAfterHandler (LValue.Group));
+			return choiceL.Choice (choiceR);
+		}
+
 		// startTagCloseDeriv (Group p1 p2) =
 		//  group (startTagCloseDeriv p1) (startTagCloseDeriv p2)
 		public override RdpPattern StartTagCloseDeriv ()
 		{
-			return LValue.StartTagCloseDeriv ()
-				.Group (RValue.StartTagCloseDeriv ());
+			RdpPattern p = LValue.StartTagCloseDeriv ();
+			return p.PatternType == RelaxngPatternType.NotAllowed ?
+				p : p.Group (RValue.StartTagCloseDeriv ());
+		}
+
+		internal override RdpPattern StartTagCloseDeriv (MemoizationStore memo)
+		{
+			RdpPattern p = memo.StartTagCloseDeriv (LValue);
+			return p.PatternType == RelaxngPatternType.NotAllowed ?
+				p : p.Group (memo.StartTagCloseDeriv (RValue));
 		}
 
 		public override RelaxngPatternType PatternType {
@@ -1101,6 +1439,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return Child.Nullable; }
 		}
 
+		internal override bool IsTextValueDependent {
+			get { return Child.IsTextValueDependent; }
+		}
+
+		internal override bool IsContextDependent {
+			get { return Child.IsContextDependent; }
+		}
+
 		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
 			Child.GetLabels (elements, attributes, collectNameClass);
@@ -1125,7 +1471,23 @@ namespace Commons.Xml.Relaxng.Derivative
 
 		public override RdpPattern TextDeriv (string s, XmlReader reader)
 		{
-			return Child.TextDeriv (s, reader).Group (this.Choice (RdpEmpty.Instance));
+			RdpPattern p = Child.TextDeriv (s, reader);
+			return p.PatternType == RelaxngPatternType.NotAllowed ?
+				p : p.Group (this.Choice (RdpEmpty.Instance));
+		}
+
+		internal override RdpPattern TextDeriv (string s, XmlReader reader, MemoizationStore memo)
+		{
+			RdpPattern p = memo.TextDeriv (Child, s, reader);
+			return p.PatternType == RelaxngPatternType.NotAllowed ?
+				p : p.Group (this.Choice (RdpEmpty.Instance));
+		}
+
+		internal override RdpPattern EmptyTextDeriv (MemoizationStore memo)
+		{
+			RdpPattern p = memo.EmptyTextDeriv (Child);
+			return p.PatternType == RelaxngPatternType.NotAllowed ?
+				p : p.Group (this.Choice (RdpEmpty.Instance));
 		}
 
 		internal override RdpPattern TextOnlyDeriv ()
@@ -1133,24 +1495,32 @@ namespace Commons.Xml.Relaxng.Derivative
 			return Child.TextOnlyDeriv ().OneOrMore ();
 		}
 
+		internal override RdpPattern TextOnlyDeriv (MemoizationStore memo)
+		{
+			return memo.TextOnlyDeriv (Child).OneOrMore ();
+		}
+
 		internal override RdpPattern MixedTextDeriv ()
 		{
-			return Child.MixedTextDeriv ().Group (
-				this.Choice (RdpEmpty.Instance));
+			RdpPattern p = Child.MixedTextDeriv ();
+			return p.PatternType == RelaxngPatternType.NotAllowed ?
+				p : p.Group (this.Choice (RdpEmpty.Instance));
+		}
+
+		internal override RdpPattern MixedTextDeriv (MemoizationStore memo)
+		{
+			RdpPattern p = memo.MixedTextDeriv (Child);
+			return p.PatternType == RelaxngPatternType.NotAllowed ?
+				p : p.Group (this.Choice (RdpEmpty.Instance));
 		}
 
 		// attDeriv cx (OneOrMore p) att =
 		//  group (attDeriv cx p att) (choice (OneOrMore p) Empty)
 		public override RdpPattern AttDeriv (string name, string ns, string value, XmlReader reader)
 		{
-#if UseStatic
-			return RdpUtil.Group (
-				RdpUtil.AttDeriv (ctx, children, att),
-				RdpUtil.Choice (RdpUtil.OneOrMore (children), RdpEmpty.Instance));
-#else
-			return Child.AttDeriv (name, ns, value, reader)
-				.Group (Choice (RdpEmpty.Instance));
-#endif
+			RdpPattern p = Child.AttDeriv (name, ns, value, reader);
+			return p.PatternType == RelaxngPatternType.NotAllowed ?
+				p : p.Group (Choice (RdpEmpty.Instance));
 		}
 
 		// startTagOpenDeriv (OneOrMore p) qn =
@@ -1164,6 +1534,30 @@ namespace Commons.Xml.Relaxng.Derivative
 			return handled.ApplyAfter (new RdpApplyAfterHandler (f.Apply));
 		}
 
+		internal override RdpPattern StartTagOpenDeriv (string name, string ns, MemoizationStore memo)
+		{
+			RdpPattern rest = RdpEmpty.Instance.Choice (Child.OneOrMore ());
+			RdpPattern handled = memo.StartTagOpenDeriv (Child, name, ns);
+			RdpFlip f = MakeFlip (RdpUtil.GroupFunction, rest);
+			return handled.ApplyAfter (new RdpApplyAfterHandler (f.Apply));
+		}
+
+		public override RdpPattern StartAttDeriv (string name, string ns)
+		{
+			RdpPattern rest = RdpEmpty.Instance.Choice (Child.OneOrMore ());
+			RdpPattern handled = Child.StartAttDeriv (name, ns);
+			RdpFlip f = MakeFlip (RdpUtil.GroupFunction, rest);
+			return handled.ApplyAfter (new RdpApplyAfterHandler (f.Apply));
+		}
+
+		internal override RdpPattern StartAttDeriv (string name, string ns, MemoizationStore memo)
+		{
+			RdpPattern rest = RdpEmpty.Instance.Choice (Child.OneOrMore ());
+			RdpPattern handled = memo.StartAttDeriv (Child, name, ns);
+			RdpFlip f = MakeFlip (RdpUtil.GroupFunction, rest);
+			return handled.ApplyAfter (new RdpApplyAfterHandler (f.Apply));
+		}
+
 		// startTagCloseDeriv (OneOrMore p) =
 		//  oneOrMore (startTagCloseDeriv p)
 		public override RdpPattern StartTagCloseDeriv ()
@@ -1176,6 +1570,11 @@ namespace Commons.Xml.Relaxng.Derivative
 #endif
 		}
 
+		internal override RdpPattern StartTagCloseDeriv (MemoizationStore memo)
+		{
+			return memo.StartTagCloseDeriv (Child).OneOrMore ();
+		}
+
 		internal override void CheckConstraints (bool attribute, bool oneOrMore, bool oneOrMoreGroup, bool oneOrMoreInterleave, bool list, bool dataExcept)
 		{
 			if (dataExcept)
@@ -1210,6 +1609,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return false; }
 		}
 
+		internal override bool IsTextValueDependent {
+			get { return true; }
+		}
+
+		internal override bool IsContextDependent {
+			get { return Child.IsContextDependent; }
+		}
+
 		public override RelaxngPatternType PatternType {
 			get { return RelaxngPatternType.List; }
 		}
@@ -1261,6 +1668,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return false; }
 		}
 
+		internal override bool IsTextValueDependent {
+			get { return true; }
+		}
+
+		internal override bool IsContextDependent {
+			get { return dt.IsContextDependent; }
+		}
+
 		public override RelaxngPatternType PatternType {
 			get { return RelaxngPatternType.Data; }
 		}
@@ -1381,6 +1796,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return false; }
 		}
 
+		internal override bool IsTextValueDependent {
+			get { return true; }
+		}
+
+		internal override bool IsContextDependent {
+			get { return dt.IsContextDependent; }
+		}
+
 		public override RelaxngPatternType PatternType {
 			get { return RelaxngPatternType.Value; }
 		}
@@ -1394,7 +1817,19 @@ namespace Commons.Xml.Relaxng.Derivative
 			// do nothing
 		}
 
+		string cachedValue;
+		RdpPattern cachedPattern;
+
 		public override RdpPattern TextDeriv (string s, XmlReader reader)
+		{
+			if (s == cachedValue && !IsContextDependent)
+				return cachedPattern;
+			cachedPattern = TextDerivCore (s, reader);
+			cachedValue = s;
+			return cachedPattern;
+		}
+
+		RdpPattern TextDerivCore (string s, XmlReader reader)
 		{
 			if (dt.IsTypeEqual (value, s, reader))
 				return RdpEmpty.Instance;
@@ -1442,6 +1877,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return false; }
 		}
 
+		internal override bool IsTextValueDependent {
+			get { return false; }
+		}
+
+		internal override bool IsContextDependent {
+			get { return false; }
+		}
+
 		public override RelaxngPatternType PatternType {
 			get { return RelaxngPatternType.Attribute; }
 		}
@@ -1505,12 +1948,29 @@ namespace Commons.Xml.Relaxng.Derivative
 #endif
 		}
 
+		public override RdpPattern StartAttDeriv (string name, string ns)
+		{
+			return nameClass.Contains (name, ns) ?
+				children.After (RdpEmpty.Instance) : RdpNotAllowed.Instance;
+		}
+
+		internal override RdpPattern StartAttDeriv (string name, string ns, MemoizationStore memo)
+		{
+			return nameClass.Contains (name, ns) ?
+				children.After (RdpEmpty.Instance) : RdpNotAllowed.Instance;
+		}
+
 		// startTagCloseDeriv (Attribute _ _) = NotAllowed
 		public override RdpPattern StartTagCloseDeriv ()
 		{
 			return RdpNotAllowed.Instance;
 		}
 
+		internal override RdpPattern StartTagCloseDeriv (MemoizationStore memo)
+		{
+			return RdpNotAllowed.Instance;
+		}
+
 		internal override void MarkReachableDefs () 
 		{
 			children.MarkReachableDefs ();
@@ -1561,6 +2021,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return false; }
 		}
 
+		internal override bool IsTextValueDependent {
+			get { return false; }
+		}
+
+		internal override bool IsContextDependent {
+			get { return false; }
+		}
+
 		public override RelaxngPatternType PatternType {
 			get { return RelaxngPatternType.Element; }
 		}
@@ -1618,6 +2086,11 @@ namespace Commons.Xml.Relaxng.Derivative
 			return RdpNotAllowed.Instance;
 		}
 
+		internal override RdpPattern TextOnlyDeriv (MemoizationStore memo)
+		{
+			return RdpNotAllowed.Instance;
+		}
+
 		public override RdpPattern StartTagOpenDeriv (string name, string ns)
 		{
 #if UseStatic
@@ -1632,6 +2105,13 @@ namespace Commons.Xml.Relaxng.Derivative
 #endif
 		}
 
+		internal override RdpPattern StartTagOpenDeriv (string name, string ns, MemoizationStore memo)
+		{
+			return nameClass.Contains (name, ns) ?
+				children.After (RdpEmpty.Instance) :
+				RdpNotAllowed.Instance;
+		}
+
 		internal override void MarkReachableDefs () 
 		{
 			children.MarkReachableDefs ();
@@ -1665,6 +2145,14 @@ namespace Commons.Xml.Relaxng.Derivative
 			get { return false; }
 		}
 
+		internal override bool IsTextValueDependent {
+			get { return LValue.IsTextValueDependent; }
+		}
+
+		internal override bool IsContextDependent {
+			get { return LValue.IsContextDependent; }
+		}
+
 		public override void GetLabels (LabelList elements, LabelList attributes, bool collectNameClass)
 		{
 			LValue.GetLabels (elements, attributes, collectNameClass);
@@ -1675,16 +2163,36 @@ namespace Commons.Xml.Relaxng.Derivative
 			return LValue.TextDeriv (s, reader).After (RValue);
 		}
 
+		internal override RdpPattern TextDeriv (string s, XmlReader reader, MemoizationStore memo)
+		{
+			return memo.TextDeriv (LValue, s, reader).After (RValue);
+		}
+
+		internal override RdpPattern EmptyTextDeriv (MemoizationStore memo)
+		{
+			return memo.EmptyTextDeriv (LValue).After (RValue);
+		}
+
 		internal override RdpPattern TextOnlyDeriv ()
 		{
 			return LValue.TextOnlyDeriv ().After (RValue);
 		}
 
+		internal override RdpPattern TextOnlyDeriv (MemoizationStore memo)
+		{
+			return memo.TextOnlyDeriv (LValue).After (RValue);
+		}
+
 		internal override RdpPattern MixedTextDeriv ()
 		{
 			return LValue.MixedTextDeriv ().After (RValue);
 		}
 
+		internal override RdpPattern MixedTextDeriv (MemoizationStore memo)
+		{
+			return memo.MixedTextDeriv (LValue).After (RValue);
+		}
+
 		// startTagOpenDeriv (After p1 p2) qn =
 		//   applyAfter (flip after p2) (startTagOpenDeriv p1 qn)
 		public override RdpPattern StartTagOpenDeriv (string name, string ns)
@@ -1695,6 +2203,14 @@ namespace Commons.Xml.Relaxng.Derivative
 				f.Apply));
 		}
 
+		internal override RdpPattern StartTagOpenDeriv (string name, string ns, MemoizationStore memo)
+		{
+			RdpPattern handled = memo.StartTagOpenDeriv (LValue, name, ns);
+			RdpFlip f = MakeFlip (RdpUtil.AfterFunction, RValue);
+			return handled.ApplyAfter (new RdpApplyAfterHandler (
+				f.Apply));
+		}
+
 		public override RdpPattern ApplyAfter (RdpApplyAfterHandler handler)
 		{
 			return LValue.After (handler (RValue));
@@ -1707,11 +2223,37 @@ namespace Commons.Xml.Relaxng.Derivative
 			return LValue.AttDeriv (name, ns, value, reader).After (RValue);
 		}
 
+		public override RdpPattern StartAttDeriv (string name, string ns)
+		{
+			RdpPattern handled = LValue.StartAttDeriv (name, ns);
+			RdpFlip f = MakeFlip (RdpUtil.AfterFunction, RValue);
+			return handled.ApplyAfter (new RdpApplyAfterHandler (
+				f.Apply));
+		}
+
+		internal override RdpPattern StartAttDeriv (string name, string ns, MemoizationStore memo)
+		{
+			RdpPattern handled = memo.StartAttDeriv (LValue, name, ns);
+			RdpFlip f = MakeFlip (RdpUtil.AfterFunction, RValue);
+			return handled.ApplyAfter (new RdpApplyAfterHandler (
+				f.Apply));
+		}
+
+		public override RdpPattern EndAttDeriv ()
+		{
+			return LValue.Nullable ? RValue : RdpNotAllowed.Instance;
+		}
+
 		public override RdpPattern StartTagCloseDeriv ()
 		{
 			return LValue.StartTagCloseDeriv ().After (RValue);
 		}
 
+		internal override RdpPattern StartTagCloseDeriv (MemoizationStore memo)
+		{
+			return memo.StartTagCloseDeriv (LValue).After (RValue);
+		}
+
 		public override RdpPattern EndTagDeriv ()
 		{
 			return LValue.Nullable ? RValue : RdpNotAllowed.Instance;
