@@ -201,11 +201,10 @@ namespace Commons.Xml.Relaxng
 			// see RELAX NG 4.17.
 			AssembleCombine ();
 
-			// FIXME: It should not return NotAllowed
-			if (assembledStart != null)
-				compiledStart = assembledStart.Compile (this);
-			else
-				return RdpNotAllowed.Instance;
+			// 4.18 : <grammar> must have at least one <start>.
+			if (assembledStart == null)
+				throw new RelaxngException ("A grammar elements must contain at least one start element.");
+			compiledStart = assembledStart.Compile (this);
 
 			// Assemble all define components into top grammar and
 			// return start patterns for descendant grammars.
@@ -240,9 +239,6 @@ namespace Commons.Xml.Relaxng
 
 			Hashtable ht = new Hashtable ();
 			startPattern.setInternTable (ht);
-			RdpNotAllowed.Instance.setInternTable (ht);
-			RdpEmpty.Instance.setInternTable (ht);
-			RdpText.Instance.setInternTable (ht);
 
 			// Check Constraints: RELAX NG spec 7
 			// 7.1.1-4, 7.3, 7.4
@@ -250,12 +246,12 @@ namespace Commons.Xml.Relaxng
 			// 7.1.5
 			CheckStartPatternContent (startPattern);
 
-			// 7.2
-			RdpContentType ct = startPattern.ContentType;
-
 			// 4.19 (c) expandRef - actual replacement
 			startPattern = compiledStart.ExpandRef (assembledDefs);
 
+			// 7.2
+			RdpContentType ct = startPattern.ContentType;
+
 			// return its start pattern.
 			IsCompiled = true;
 			return startPattern;
