@@ -3,6 +3,7 @@
 //
 // Author:
 //   Alan Tam Siu Lung (Tam@SiuLung.com)
+//   Atsushi Enomoto  <atsushi@ximian.com>
 //
 
 using System;
@@ -23,7 +24,39 @@ namespace I18N.CJK
 
 		internal override DbcsConvert GetConvert ()
 		{
-			return Big5Convert.Convert;
+			return DbcsConvert.Big5;
+		}
+
+		// Get the bytes that result from encoding a character buffer.
+		public unsafe override int GetByteCountImpl (char* chars, int count)
+		{
+			DbcsConvert convert = GetConvert ();
+			int index = 0;
+			int length = 0;
+#if NET_2_0
+			EncoderFallbackBuffer buffer = null;
+#endif
+
+			while (count-- > 0) {
+				char c = chars[index++];
+				if (c <= 0x80 || c == 0xFF) { // ASCII
+					length++;
+					continue;
+				}
+				byte b1 = convert.u2n[((int)c) * 2 + 1];
+				byte b2 = convert.u2n[((int)c) * 2];
+				if (b1 == 0 && b2 == 0) {
+#if NET_2_0
+					// FIXME: handle fallback for GetByteCountImpl().
+					length++;
+#else
+					length++;
+#endif
+				}
+				else
+					length += 2;
+			}
+			return length;
 		}
 
 		// Get the bytes that result from encoding a character buffer.
@@ -66,6 +99,7 @@ namespace I18N.CJK
 		public override int GetChars(byte[] bytes, int byteIndex, int byteCount,
 					     char[] chars, int charIndex)
 		{
+			/*
 			DbcsConvert convert = GetConvert ();
 			// A1 40 - FA FF
 			base.GetChars(bytes, byteIndex, byteCount, chars, charIndex);
@@ -99,6 +133,9 @@ namespace I18N.CJK
 				chars[charIndex++] = '?';
 
 			return charIndex - origIndex;
+			*/
+
+			return GetDecoder ().GetChars (bytes, byteIndex, byteCount, chars, charIndex);
 		}
 		
 		// Get a decoder that handles a rolling Big5 state.
@@ -144,34 +181,107 @@ namespace I18N.CJK
 		{
 			// Constructor.
 			public CP950Decoder(DbcsConvert convert) : base(convert) {}
-			
+			int last_byte_count, last_byte_conv;
+
+			public override int GetCharCount (byte[] bytes, int index, int count)
+			{
+				return GetCharCount (bytes, index, count, false);
+			}
+
+#if NET_2_0
+			public override
+#endif
+			int GetCharCount (byte[] bytes, int index, int count, bool refresh)
+			{
+				CheckRange (bytes, index, count);
+
+				int lastByte = last_byte_count;
+				last_byte_count = 0;
+				int length = 0;
+				while (count-- > 0) {
+					int b = bytes[index++];
+					if (lastByte == 0) {
+						if (b <= 0x80 || b == 0xFF) { // ASCII
+							length++;
+						} else if (b < 0xA1 || b >= 0xFA) {
+							// incorrect first byte.
+							length++;
+							count--; // cut one more byte.
+						} else {
+							lastByte = b;
+						}
+						continue;
+					}
+					int ord = ((lastByte - 0xA1) * 191 + b - 0x40) * 2;
+					char c1 = ord < 0 || ord > convert.n2u.Length ?
+						'\0' :
+						(char)(convert.n2u[ord] + convert.n2u[ord + 1] * 256);
+					if (c1 == 0)
+						// FIXME: fallback
+						length++;
+					else
+						length++;
+					lastByte = 0;
+				}
+
+				if (lastByte != 0) {
+					if (refresh)
+						// FIXME: fallback
+						length++;
+					else
+						last_byte_count = lastByte;
+				}
+				return length;
+			}
+
 			public override int GetChars(byte[] bytes, int byteIndex, int byteCount,
 						     char[] chars, int charIndex)
 			{
-				base.GetChars(bytes, byteIndex, byteCount, chars, charIndex);
+				return GetChars (bytes, byteIndex, byteCount, chars, charIndex, false);
+			}
+
+#if NET_2_0
+			public override
+#endif
+			int GetChars(byte[] bytes, int byteIndex, int byteCount,
+						     char[] chars, int charIndex, bool refresh)
+			{
+				CheckRange (bytes, byteIndex, byteCount, chars, charIndex);
+
 				int origIndex = charIndex;
+				int lastByte = last_byte_conv;
+				last_byte_conv = 0;
 				while (byteCount-- > 0) {
 					int b = bytes[byteIndex++];
 					if (lastByte == 0) {
 						if (b <= 0x80 || b == 0xFF) { // ASCII
 							chars[charIndex++] = (char)b;
-							continue;
 						} else if (b < 0xA1 || b >= 0xFA) {
-							continue;
+							// incorrect first byte.
+							chars[charIndex++] = '?';
+							byteCount--; // cut one more byte.
 						} else {
 							lastByte = b;
-							continue;
 						}
+						continue;
 					}
 					int ord = ((lastByte - 0xA1) * 191 + b - 0x40) * 2;
-					char c1 = (char)(convert.n2u[ord] + convert.n2u[ord + 1] * 256);
-					if (c1 == 0) {
+					char c1 = ord < 0 || ord > convert.n2u.Length ?
+						'\0' :
+						(char)(convert.n2u[ord] + convert.n2u[ord + 1] * 256);
+					if (c1 == 0)
 						chars[charIndex++] = '?';
-					} else {
+					else
 						chars[charIndex++] = c1;
-					}
 					lastByte = 0;
 				}
+
+				if (lastByte != 0) {
+					if (refresh)
+						chars [charIndex++] = '?';
+					else
+						last_byte_conv = lastByte;
+				}
 				return charIndex - origIndex;
 			}
 		}
