@@ -16,17 +16,12 @@ using System.Runtime.InteropServices;
 
 // General Information about the System.Windows.Forms assembly
 
-#if (NET_1_0)
-	[assembly: AssemblyVersion("1.0.3300.0")]
-	[assembly: SatelliteContractVersion("1.0.3300.0")]
-#elif (NET_2_0)
-	[assembly: AssemblyVersion ("2.0.3600.0")]
-	[assembly: SatelliteContractVersion ("2.0.3600.0")]
-#elif (NET_1_1)
-	[assembly: AssemblyVersion("1.0.5000.0")]
-	[assembly: SatelliteContractVersion("1.0.5000.0")]
-	[assembly: ComCompatibleVersion(1, 0, 3300, 0)]
-	[assembly: TypeLibVersion(1, 10)]
+[assembly: AssemblyVersion (Consts.FxVersion)]
+[assembly: SatelliteContractVersion (Consts.FxVersion)]
+
+#if (ONLY_1_1)
+[assembly: ComCompatibleVersion (1, 0, 3300, 0)]
+[assembly: TypeLibVersion (1, 10)]
 #endif
 
 [assembly: AssemblyTitle("System.Windows.Forms.dll")]
