@@ -105,6 +105,7 @@ namespace System.Windows.Forms.RTF {
 			destination_callbacks = new DestinationCallback();
 			class_callbacks = new ClassCallback();
 
+			destination_callbacks [Minor.OptDest] = new DestinationDelegate (HandleOptDest);
 			destination_callbacks[Minor.FontTbl] = new DestinationDelegate(ReadFontTbl);
 			destination_callbacks[Minor.ColorTbl] = new DestinationDelegate(ReadColorTbl);
 			destination_callbacks[Minor.StyleSheet] = new DestinationDelegate(ReadStyleSheet);
@@ -244,12 +245,12 @@ namespace System.Windows.Forms.RTF {
 
 		/// <summary>Read the next character from the input</summary>
 		private char GetChar() {
-			char	c;
+			int	c;
 			bool	old_bump_line;
 
 SkipCRLF:
-			if ((c = (char)source.Read()) != EOF) {
-				this.text_buffer.Append(c);
+			if ((c = source.Read()) != -1) {
+				this.text_buffer.Append((char) c);
 			}
 
 			if (this.prev_char == EOF) {
@@ -278,8 +279,8 @@ SkipCRLF:
 				this.line_pos = 1;
 			}
 
-			this.prev_char = c;
-			return c;
+			this.prev_char = (char) c;
+			return (char) c;
 		}
 
 		/// <summary>Parse the RTF stream</summary>
@@ -540,6 +541,8 @@ SkipCRLF:
 			obj = key_table[token.Substring(1)];
 			if (obj == null) {
 				rtf_class = TokenClass.Unknown;
+				major = (Major) -1;
+				minor = (Minor) -1;
 				return;
 			}
 
@@ -575,6 +578,12 @@ SkipCRLF:
 		#endregion	// Methods
 
 		#region Default Delegates
+
+		private void HandleOptDest (RTF rtf)
+		{
+			rtf.SkipGroup ();
+		}
+
 		private void ReadFontTbl(RTF rtf) {
 			int	old;
 			Font	font;
@@ -679,7 +688,7 @@ SkipCRLF:
 
 						sb = new StringBuilder();
 
-						while ((rtf.rtf_class != TokenClass.EOF) && (!rtf.CheckCM(TokenClass.Text, (Major)';')) && (!rtf.CheckCM(TokenClass.Group, Major.EndGroup))) {
+						while ((rtf.rtf_class != TokenClass.EOF) && (!rtf.CheckCM(TokenClass.Text, (Major)';')) && (!rtf.CheckCM(TokenClass.Group, Major.EndGroup)) && (!rtf.CheckCM(TokenClass.Group, Major.BeginGroup))) {
 							sb.Append((char)rtf.major);
 							rtf.GetToken();
 						}
@@ -789,9 +798,6 @@ SkipCRLF:
 					}
 
 					if (rtf.rtf_class == TokenClass.Control) {
-						if (rtf.CheckMM(Major.SpecialChar, Minor.OptDest)) {
-							continue;
-						}
 						if (rtf.CheckMM(Major.ParAttr, Minor.StyleNum)) {
 							style.Num = rtf.param;
 							style.Type = StyleType.Paragraph;
