@@ -48,6 +48,7 @@ namespace System.Windows.Forms {
 			#region Fields
 			private ApplicationContext	context;
 			private bool			messageloop_started;
+			private bool                    handling_exception;
 			private int			thread_id;
 
 			private static Hashtable	threads = new Hashtable();
@@ -60,25 +61,20 @@ namespace System.Windows.Forms {
 
 			#region Properties
 			public ApplicationContext Context {
-				get {
-					return context;
-				}
-
-				set {
-					context = value;
-				}
+				get { return context; }
+				set { context = value; }
 			}
 
 			public bool MessageLoop {
-				get {
-					return messageloop_started;
-				}
-
-				set {
-					messageloop_started = value;
-				}
+				get { return messageloop_started; }
+				set { messageloop_started = value; }
 			}
 
+			public bool HandlingException {
+				get { return handling_exception; }
+				set { handling_exception = value; }
+
+			}
 
 			public static int LoopCount {
 				get {
@@ -135,7 +131,9 @@ namespace System.Windows.Forms {
 						Application.ApplicationExit(null, EventArgs.Empty);
 					}
 				}
-				threads[thread_id] = null;
+				lock (threads) {
+					threads[thread_id] = null;
+				}
 			}
 			#endregion	// Methods
 		}
@@ -188,7 +186,7 @@ namespace System.Windows.Forms {
 		#region Public Static Properties
 		public static bool AllowQuit {
 			get {
-				return browser_embedded;
+				return !browser_embedded;
 			}
 		}
 
@@ -345,7 +343,9 @@ namespace System.Windows.Forms {
 
 		#region Public Static Methods
 		public static void AddMessageFilter(IMessageFilter value) {
-			message_filters.Add(value);
+			lock (message_filters) {
+				message_filters.Add(value);
+			}
 		}
 
 		public static void DoEvents() {
@@ -390,6 +390,16 @@ namespace System.Windows.Forms {
 		}
 
 		public static void OnThreadException(Exception t) {
+			if (MWFThread.Current.HandlingException) {
+				/* we're already handling an exception and we got
+				   another one?  print it out and exit, this means
+				   we've got a runtime/SWF bug. */
+				Console.WriteLine (t);
+				Application.Exit();
+			}
+
+			MWFThread.Current.HandlingException = true;
+
 			if (Application.ThreadException != null) {
 				Application.ThreadException(null, new ThreadExceptionEventArgs(t));
 				return;
@@ -400,11 +410,16 @@ namespace System.Windows.Forms {
 				form.ShowDialog ();
 			} else {
 				Console.WriteLine (t.ToString ());
+				Application.Exit ();
 			}
+
+			MWFThread.Current.HandlingException = false;
 		}
 
 		public static void RemoveMessageFilter(IMessageFilter filter) {
-			message_filters.Remove(filter);
+			lock (message_filters) {
+				message_filters.Remove(filter);
+			}
 		}
 
 		public static void Run() {
@@ -471,8 +486,11 @@ namespace System.Windows.Forms {
 					}
 				}
 				// FIXME - need activate?
-
-				XplatUI.SetModal(context.MainForm.Handle, true);
+				/* make sure the MainForm is enabled */
+				if (context.MainForm != null) {
+					XplatUI.EnableWindow (context.MainForm.Handle, true);
+					XplatUI.SetModal(context.MainForm.Handle, true);
+				}
 			} else {
 				toplevels = null;
 			}
@@ -481,46 +499,46 @@ namespace System.Windows.Forms {
 			thread.MessageLoop = true;
 
 			while (XplatUI.GetMessage(queue_id, ref msg, IntPtr.Zero, 0, 0)) {
-				if ((message_filters != null) && (message_filters.Count > 0)) {
-					Message	m;
-					bool	drop;
+				lock (message_filters) {
+					if (message_filters.Count > 0) {
+						Message	m;
+						bool	drop;
 
-					drop = false;
-					m = Message.Create(msg.hwnd, (int)msg.message, msg.wParam, msg.lParam);
-					for (int i = 0; i < message_filters.Count; i++) {
-						if (((IMessageFilter)message_filters[i]).PreFilterMessage(ref m)) {
-							// we're dropping the message
-							drop = true;
-							break;
+						drop = false;
+						m = Message.Create(msg.hwnd, (int)msg.message, msg.wParam, msg.lParam);
+						for (int i = 0; i < message_filters.Count; i++) {
+							if (((IMessageFilter)message_filters[i]).PreFilterMessage(ref m)) {
+								// we're dropping the message
+								drop = true;
+								break;
+							}
+						}
+						if (drop) {
+							continue;
 						}
-					}
-					if (drop) {
-						continue;
 					}
 				}
 
 				switch((Msg)msg.message) {
-					case Msg.WM_KEYDOWN:
-					case Msg.WM_SYSKEYDOWN:
-					case Msg.WM_CHAR:
-					case Msg.WM_SYSCHAR:
-					case Msg.WM_KEYUP:
-					case Msg.WM_SYSKEYUP: {
-						Message m;
-						Control c;
+				case Msg.WM_KEYDOWN:
+				case Msg.WM_SYSKEYDOWN:
+				case Msg.WM_CHAR:
+				case Msg.WM_SYSCHAR:
+				case Msg.WM_KEYUP:
+				case Msg.WM_SYSKEYUP:
+					Message m;
+					Control c;
 
-						m = Message.Create(msg.hwnd, (int)msg.message, msg.wParam, msg.lParam);
-						c = Control.FromHandle(msg.hwnd);
-						if ((c != null) && !c.PreProcessMessage(ref m)) {
-							goto default;
-						}
-						break;
-					}
-					default: {
-						XplatUI.TranslateMessage(ref msg);
-						XplatUI.DispatchMessage(ref msg);
-						break;
+					m = Message.Create(msg.hwnd, (int)msg.message, msg.wParam, msg.lParam);
+					c = Control.FromHandle(msg.hwnd);
+					if ((c != null) && !c.PreProcessMessage(ref m)) {
+						goto default;
 					}
+					break;
+				default:
+					XplatUI.TranslateMessage(ref msg);
+					XplatUI.DispatchMessage(ref msg);
+					break;
 				}
 
 				// Handle exit, Form might have received WM_CLOSE and set 'closing' in response
@@ -540,29 +558,33 @@ namespace System.Windows.Forms {
 			XplatUI.EndLoop(Thread.CurrentThread);
 
 			if (Modal) {
-				Control c;
+				Form c;
 
-				context.MainForm.Hide();
-				context.MainForm.is_modal = false;
+				Form old = context.MainForm;
+
+				context.MainForm = null;
 
 				while (toplevels.Count>0) {
 					#if DebugRunLoop
 						Console.WriteLine("      Re-Enabling form form {0}", toplevels.Peek());
 					#endif
-					c = (Control)toplevels.Dequeue();
+					c = (Form)toplevels.Dequeue();
 					if (c.IsHandleCreated) {
 						XplatUI.EnableWindow(c.window.Handle, true);
+						context.MainForm = c;
 					}
 				}
 				#if DebugRunLoop
 					Console.WriteLine("   Done with the re-enable");
 				#endif
-				if (context.MainForm.IsHandleCreated) {
+				if (context.MainForm != null && context.MainForm.IsHandleCreated) {
 					XplatUI.SetModal(context.MainForm.Handle, false);
 				}
 				#if DebugRunLoop
 					Console.WriteLine("   Done with the SetModal");
 				#endif
+				old.Close();
+				old.is_modal = false;
 			}
 
 			#if DebugRunLoop
@@ -570,6 +592,7 @@ namespace System.Windows.Forms {
 			#endif
 			if (context.MainForm != null) {
 				context.MainForm.context = null;
+				context.MainForm = null;
 			}
 
 			if (!Modal) {
