@@ -58,7 +58,9 @@ namespace System.Windows.Forms {
 						main_form.HandleDestroyed -= new EventHandler(OnMainFormClosed);
 					}
 					main_form = value;
-					main_form.HandleDestroyed += new EventHandler(OnMainFormClosed);
+					if (main_form != null) {
+						main_form.HandleDestroyed += new EventHandler(OnMainFormClosed);
+					}
 				}
 			}
 		}
@@ -87,7 +89,8 @@ namespace System.Windows.Forms {
 		}
 
 		protected virtual void OnMainFormClosed(object sender, EventArgs e) {
-			ExitThreadCore();
+			if (!MainForm.RecreatingHandle)
+				ExitThreadCore();
 		}
 		#endregion	// Public Instance Methods
 
