@@ -46,7 +46,7 @@ namespace System.Windows.Forms
 		private DrawMode draw_mode = DrawMode.Normal;
 		private ComboBoxStyle dropdown_style = (ComboBoxStyle)(-1);
 		private int dropdown_width = -1;		
-		private object selected_item = null;
+		private int selected_index = -1;
 		internal ObjectCollection items = null;
 		private bool suspend_ctrlupdate;
 		private int maxdrop_items = 8;			
@@ -223,10 +223,12 @@ namespace System.Windows.Forms
 	
 				if (dropdown_style != ComboBoxStyle.DropDownList && textbox_ctrl == null) {
 					textbox_ctrl = new ComboTextBox (this);
+					object selected_item = SelectedItem;
 					if (selected_item != null)
 						textbox_ctrl.Text = GetItemText (selected_item);
 					textbox_ctrl.BorderStyle = BorderStyle.None;
 					textbox_ctrl.TextChanged += new EventHandler (OnTextChangedEdit);
+					textbox_ctrl.Click += new EventHandler (OnTextBoxClick);
 
 					if (IsHandleCreated == true) {
 						Controls.AddImplicit (textbox_ctrl);
@@ -381,16 +383,31 @@ namespace System.Windows.Forms
 		[Browsable (false)]
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 		public override int SelectedIndex {
-			get { return selected_item == null ? -1 : Items.IndexOf (selected_item); }
+			get { return selected_index; }
 			set {
+				if (selected_index == value)
+					return;
+
 				if (value <= -2 || value >= Items.Count)
 					throw new ArgumentOutOfRangeException ("Index of out range");
+    				selected_index = value;
 
-				object item = null;
-				if (value != -1)
-					item = Items [value];
+    				if (dropdown_style != ComboBoxStyle.DropDownList) {
+					if (value == -1)
+						SetControlText("");
+					else
+						SetControlText (GetItemText (Items [value]));
+    				}
 
-    				SelectedItem = item;
+				if (DropDownStyle == ComboBoxStyle.DropDownList)
+    					Invalidate ();
+
+				if (listbox_ctrl != null)
+					listbox_ctrl.HighlightedIndex = value;
+
+				OnSelectedValueChanged (new EventArgs ());
+				OnSelectedIndexChanged (new EventArgs ());
+				OnSelectedItemChanged (new EventArgs ());
 			}
 		}
 
@@ -398,28 +415,13 @@ namespace System.Windows.Forms
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 		[Bindable(true)]
 		public object SelectedItem {
-			get { return selected_item; }
+			get { return selected_index == -1 ? null : Items [selected_index]; }
 			set {				
-				if (selected_item == value)
+				object item = selected_index == -1 ? null : Items [selected_index];
+				if (item == value)
 					return;
 
-				selected_item = value;
-    				
-    				if (dropdown_style != ComboBoxStyle.DropDownList) {
-					if (selected_item == null)
-						SetControlText("");
-					else
-						SetControlText (GetItemText (selected_item));
-    				}
-
-    				OnSelectedValueChanged (new EventArgs ());
-    				OnSelectedIndexChanged  (new EventArgs ());
-    				OnSelectedItemChanged (new EventArgs ());
-				if (DropDownStyle == ComboBoxStyle.DropDownList)
-    					Invalidate ();
-
-				if (listbox_ctrl != null)
-					listbox_ctrl.HighlightedItem = value;
+				SelectedIndex = Items.IndexOf (value);
 			}
 		}
 		
@@ -489,6 +491,11 @@ namespace System.Windows.Forms
 					return;
 
     				sorted = value;
+				SelectedIndex = -1;
+				if (sorted) {
+					Items.Sort ();
+					Layout ();
+				}
     			}
     		}
 
@@ -991,8 +998,7 @@ namespace System.Windows.Forms
 		private void CreateComboListBox ()
 		{			
 			listbox_ctrl = new ComboListBox (this);			
-			if (selected_item != null)
-				listbox_ctrl.HighlightedItem = selected_item;
+			listbox_ctrl.HighlightedIndex = SelectedIndex;
 		}
 		
 		internal void Draw (Rectangle clip, Graphics dc)
@@ -1051,8 +1057,6 @@ namespace System.Windows.Forms
     			button_state = ButtonState.Pushed;				
     			if (dropdown_style == ComboBoxStyle.DropDownList)
     				Invalidate (text_area);
-
-			OnDropDown (EventArgs.Empty);
 		}
 		
 		internal void DropDownListBoxFinished ()
@@ -1189,11 +1193,18 @@ namespace System.Windows.Forms
 			Draw (ClientRectangle, pevent.Graphics);			
 		}
 		
+		private void OnTextBoxClick (object sender, EventArgs e)
+		{
+			OnClick (e);
+		}
+
 		private void OnTextChangedEdit (object sender, EventArgs e)
 		{
 			if (process_textchanged_event == false)
 				return; 
 				
+			OnTextChanged (EventArgs.Empty);
+
 			int item = FindStringCaseInsensitive (textbox_ctrl.Text);
 			
 			if (item == -1)
@@ -1205,7 +1216,7 @@ namespace System.Windows.Forms
 			
 			if (listbox_ctrl != null) {
 				listbox_ctrl.SetTopItem (item);
-				listbox_ctrl.HighlightedItem = Items [item];
+				listbox_ctrl.HighlightedIndex = item;
 			}
 
 			base.Text = textbox_ctrl.Text;
@@ -1221,7 +1232,7 @@ namespace System.Windows.Forms
 		void UpdateBounds ()
 		{
 			if (requested_height != -1)
-				SetBoundsCore (0, 0, 0, requested_height, BoundsSpecified.Height);
+				SetBounds (0, 0, 0, requested_height, BoundsSpecified.Height);
 		}
 
 		private void UpdatedItems ()
@@ -1272,6 +1283,14 @@ namespace System.Windows.Forms
 						throw new ArgumentNullException ("value");
 
 					object_items[index] = value;
+					if (owner.listbox_ctrl != null)
+						owner.listbox_ctrl.InvalidateItem (index);
+					if (index == owner.SelectedIndex) {
+						if (owner.textbox_ctrl == null)
+							owner.Refresh ();
+						else
+							owner.textbox_ctrl.SelectedText = value.ToString ();
+					}
 				}
 			}
 
@@ -1289,16 +1308,6 @@ namespace System.Windows.Forms
 
 			#endregion Public Properties
 			
-			#region Private Properties			
-			internal ArrayList ObjectItems {
-				get { return object_items;}
-				set {
-					object_items = value;
-				}
-			}
-			
-			#endregion Private Properties
-
 			#region Public Methods
 			public int Add (object item)
 			{
@@ -1322,7 +1331,7 @@ namespace System.Windows.Forms
 
 			public void Clear ()
 			{
-				owner.selected_item = null;
+				owner.selected_index = -1;
 				object_items.Clear ();
 				owner.UpdatedItems ();
 				owner.Refresh ();
@@ -1373,7 +1382,10 @@ namespace System.Windows.Forms
 				
 				owner.BeginUpdate ();
 				
-				object_items.Insert (index, item);
+				if (owner.Sorted)
+					AddItem (item);
+				else
+					object_items.Insert (index, item);
 												
 				owner.EndUpdate ();	// Calls UpdatedItems
 			}
@@ -1384,10 +1396,9 @@ namespace System.Windows.Forms
 					return;
 
 				if (IndexOf (value) == owner.SelectedIndex)
-					owner.SelectedItem = null;
+					owner.SelectedIndex = -1;
 				
 				RemoveAt (IndexOf (value));				
-				
 			}
 
 			public void RemoveAt (int index)
@@ -1396,7 +1407,7 @@ namespace System.Windows.Forms
 					throw new ArgumentOutOfRangeException ("Index of out range");
 					
 				if (index == owner.SelectedIndex)
-					owner.SelectedItem = null;
+					owner.SelectedIndex = -1;
 
 				object_items.RemoveAt (index);
 				owner.UpdatedItems ();
@@ -1409,9 +1420,18 @@ namespace System.Windows.Forms
 				if (item == null)
 					throw new ArgumentNullException ("item");
 
-				int cnt = object_items.Count;
+				if (owner.Sorted) {
+					int index = 0;
+					foreach (object o in object_items) {
+						if (String.Compare (item.ToString (), o.ToString ()) < 0) {
+							object_items.Insert (index, item);
+							return index;
+						}
+						index++;
+					}
+				}
 				object_items.Add (item);
-				return cnt;
+				return object_items.Count - 1;
 			}
 			
 			internal void AddRange (IList items)
@@ -1422,6 +1442,11 @@ namespace System.Windows.Forms
 				owner.UpdatedItems ();
 			}
 
+			internal void Sort ()
+			{
+				object_items.Sort ();
+			}
+
 			#endregion Private Methods
 		}
 
@@ -1489,34 +1514,31 @@ namespace System.Windows.Forms
 					set { }
 				}
 
-				public bool FireMouseDown (MouseEventArgs e) 
+				public void FireMouseDown (MouseEventArgs e) 
 				{
-					if (Visible) {
-						e = TranslateEvent (e);
-						if (ClientRectangle.Contains (e.X, e.Y)) {
-							OnMouseDown (e);
-							return true;
-						}
-					}
-					return false;
+					if (!Visible) 
+						return;
+
+					e = TranslateEvent (e);
+					OnMouseDown (e);
 				}	
 				
 				public void FireMouseUp (MouseEventArgs e) 
 				{
-					if (Visible) {
-						e = TranslateEvent (e);
-						if (ClientRectangle.Contains (e.X, e.Y))
-							OnMouseUp (e);
-					}
+					if (!Visible)
+						return;
+
+					e = TranslateEvent (e);
+					OnMouseUp (e);
 				}
 				
 				public void FireMouseMove (MouseEventArgs e) 
 				{
-					if (Visible) {
-						e = TranslateEvent (e);
-						if (ClientRectangle.Contains (e.X, e.Y))
-							OnMouseMove (e);
-					}
+					if (!Visible)
+						return;
+
+					e = TranslateEvent (e);
+					OnMouseMove (e);
 				}			
 				
 				MouseEventArgs TranslateEvent (MouseEventArgs e)
@@ -1538,6 +1560,9 @@ namespace System.Windows.Forms
 				SetStyle (ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
 				SetStyle (ControlStyles.ResizeRedraw | ControlStyles.Opaque, true);
 
+				this.is_visible = false;
+				Hwnd.ObjectFromHandle (this.Handle).no_activate = true;
+
 				if (owner.DropDownStyle == ComboBoxStyle.Simple)
 					InternalBorderStyle = BorderStyle.Fixed3D;
 				else
@@ -1611,6 +1636,8 @@ namespace System.Windows.Forms
 					}
 				}
 				
+				page_size = height / owner.ItemHeight;
+
 				if (owner.Items.Count <= owner.MaxDropDownItems) {					
 					if (vscrollbar_ctrl != null)
 						vscrollbar_ctrl.Visible = false;
@@ -1630,7 +1657,11 @@ namespace System.Windows.Forms
 							
 					vscrollbar_ctrl.Location = new Point (width - vscrollbar_ctrl.Width - BorderWidth - 1, 0);
 
-					vscrollbar_ctrl.Maximum = owner.Items.Count - (owner.DropDownStyle == ComboBoxStyle.Simple ? page_size : owner.maxdrop_items);
+					vscrollbar_ctrl.Maximum = owner.Items.Count - 2;
+					int large = (owner.DropDownStyle == ComboBoxStyle.Simple ? page_size : owner.maxdrop_items) - 1;
+					if (large < 0)
+						large = 0;
+					vscrollbar_ctrl.LargeChange = large;
 					show_scrollbar = vscrollbar_ctrl.Visible = true;
 
 					int hli = HighlightedIndex;
@@ -1649,7 +1680,6 @@ namespace System.Windows.Forms
 					textarea_drawable.Width -= vscrollbar_ctrl.Width;
 
 				last_item = LastVisibleItem ();
-				page_size = textarea_drawable.Height / owner.ItemHeight;
 			}			
 
 			private void Draw (Rectangle clip, Graphics dc)
@@ -1680,34 +1710,22 @@ namespace System.Windows.Forms
 				}
 			}
 
+			int highlighted_index = -1;
+
 			public int HighlightedIndex {
-				get { return highlighted_item == null ? -1 : owner.Items.IndexOf (highlighted_item); }
+				get { return highlighted_index; }
 				set { 
-					object item = null;
-					if (value != -1)
-						item = owner.Items [value];
-					HighlightedItem = item; 
-				}
-			}
-
-			object highlighted_item = null;
-
-			public object HighlightedItem {
-				get { return highlighted_item; }
-				set {
-					if (highlighted_item == value)
+					if (highlighted_index == value)
 						return;
-				
-					int index = highlighted_item == null ? -1 :  owner.Items.IndexOf (highlighted_item);
-    					if (index != -1)
-						Invalidate (GetItemDisplayRectangle (index, top_item));
-    					highlighted_item = value;
-				       	index = highlighted_item == null ? -1 : owner.Items.IndexOf (highlighted_item);
-    					if (index != -1)
-						Invalidate (GetItemDisplayRectangle (index, top_item));
+
+					if (highlighted_index != -1 && highlighted_index < this.owner.Items.Count)
+						Invalidate (GetItemDisplayRectangle (highlighted_index, top_item));
+					highlighted_index = value;
+    					if (highlighted_index != -1)
+						Invalidate (GetItemDisplayRectangle (highlighted_index, top_item));
 				}
 			}
-			
+
 			private Rectangle GetItemDisplayRectangle (int index, int top_index)
 			{
 				if (index < 0 || index >= owner.Items.Count)
@@ -1749,6 +1767,12 @@ namespace System.Windows.Forms
 				return -1;
 			}
 
+			public void InvalidateItem (int index)
+			{
+				if (Visible)
+					Invalidate (GetItemDisplayRectangle (index, top_item));
+			}
+
 			private int LastVisibleItem ()
 			{
 				Rectangle item_rect;
@@ -1770,13 +1794,26 @@ namespace System.Windows.Forms
 					return;
 				top_item = item;
 				UpdateLastVisibleItem ();
-				Refresh ();
+				Invalidate ();
 			}			
 
+			bool scrollbar_grabbed = false;
+
+			bool InScrollBar {
+				get {
+					if (vscrollbar_ctrl == null || !vscrollbar_ctrl.is_visible)
+						return false;
+
+					return vscrollbar_ctrl.Bounds.Contains (PointToClient (Control.MousePosition));
+				}
+			}
+
 			protected override void OnMouseDown (MouseEventArgs e)
 			{
-				if (vscrollbar_ctrl != null)
+				if (InScrollBar) {
 					vscrollbar_ctrl.FireMouseDown (e);
+					scrollbar_grabbed = true;
+				}
 			}
 
 			protected override void OnMouseMove (MouseEventArgs e)
@@ -1784,34 +1821,38 @@ namespace System.Windows.Forms
 				if (owner.DropDownStyle == ComboBoxStyle.Simple)
 					return;
 
+				if (scrollbar_grabbed || (!Capture && InScrollBar)) {
+					vscrollbar_ctrl.FireMouseMove (e);
+					return;
+				}
+
 				Point pt = PointToClient (Control.MousePosition);
 				int index = IndexFromPointDisplayRectangle (pt.X, pt.Y);
 
-    				if (index != -1) {
+    				if (index != -1)
 					HighlightedIndex = index;
-					return;
-				}
-				
-				if (vscrollbar_ctrl != null)
-					vscrollbar_ctrl.FireMouseMove (e);
 			}
 			
 			protected override void OnMouseUp (MouseEventArgs e)
 			{
 	    			int index = IndexFromPointDisplayRectangle (e.X, e.Y);
 
-    				if (index == -1) {    					
-					if (vscrollbar_ctrl == null || !vscrollbar_ctrl.FireMouseDown (e))
-		    				HideWindow ();
-				} else {
-					owner.OnSelectionChangeCommitted (new EventArgs ());
-					owner.SelectedIndex = index;
-					HighlightedIndex = index;
-					HideWindow ();
+				if (scrollbar_grabbed) {
+					vscrollbar_ctrl.FireMouseUp (e);
+					scrollbar_grabbed = false;
+					if (index != -1)
+						HighlightedIndex = index;
+					return;
 				}
 
-	    			if (vscrollbar_ctrl != null)
-		    			vscrollbar_ctrl.FireMouseUp (e);
+    				if (index == -1) {    					
+		    			HideWindow ();
+					return;
+				}
+
+				owner.SelectedIndex = index;
+				owner.OnSelectionChangeCommitted (new EventArgs ());
+				HideWindow ();
 			}
 
 			internal override void OnPaintInternal (PaintEventArgs pevent)
@@ -1844,7 +1885,7 @@ namespace System.Windows.Forms
 				if (delta == 0 || vscrollbar_ctrl == null || !vscrollbar_ctrl.Visible)
 					return;
 
-				int max = vscrollbar_ctrl.Maximum;//- (page_size) + 1;
+				int max = owner.Items.Count - page_size;
 
 				int val = vscrollbar_ctrl.Value + delta;
 				if (val > max)
@@ -1871,7 +1912,7 @@ namespace System.Windows.Forms
 
 				top_item =  vscrollbar_ctrl.Value;
 				UpdateLastVisibleItem ();
-				Refresh ();
+				Invalidate ();
 			}			
 			
 			protected override void WndProc(ref Message m) {
@@ -1888,3 +1929,4 @@ namespace System.Windows.Forms
 	}
 }
 
+
