@@ -175,6 +175,10 @@ namespace System.Windows.Forms {
                 [MonoTODO ("this needs re-addressing once DataViewManager.AllowNew is implemented")]
                 internal bool CanAddRows {
                 	get {
+				/* if we're readonly, don't even bother checking if we can add new rows */
+				if (list.IsReadOnly)
+					return false;
+
 				if (list is IBindingList) {
 					return true;
 					//return ((IBindingList)list).AllowNew;
@@ -344,7 +348,8 @@ namespace System.Windows.Forms {
 				break;
 			case ListChangedType.ItemAdded:
 				if (listposition == -1) {
-					listposition = e.NewIndex - 1;
+					/* do we need this logic up above in ItemDeleted as well? */
+					listposition = e.NewIndex == 0 ? 0 : e.NewIndex - 1;
 					OnCurrentChanged (EventArgs.Empty);
 					OnPositionChanged (EventArgs.Empty);
 				}
