@@ -38,11 +38,37 @@ using System.Text;
 
 namespace System.Windows.Forms
 {
-	internal struct DataGridRow {
+	internal class DataGridRelationshipRow {
+		DataGrid owner;
+
+		public DataGridRelationshipRow (DataGrid owner)
+		{
+			this.owner = owner;
+			IsSelected = false;
+			IsExpanded = false;
+			height = 0;
+			VerticalOffset = 0;
+			RelationHeight = 0;
+			relation_area = Rectangle.Empty;
+		}
+
+		public int height;
+
+		/* this needs to be a property so that the Autosize
+		 * example from the Windows.Forms FAQ will work */
+		public int Height {
+			get { return height; }
+			set {
+				if (height != value) {
+					height = value;
+					owner.UpdateRowsFrom (this);
+				}
+			}
+		}
+
 		public bool IsSelected;
 		public bool IsExpanded;
 		public int VerticalOffset;
-		public int Height;
 		public int RelationHeight;
 		public Rectangle relation_area; /* the Y coordinate of this rectangle is updated as needed */
 	}
@@ -51,23 +77,23 @@ namespace System.Windows.Forms
 	{
 		public DataGrid owner;
 		public CurrencyManager list_manager;
-		public DataRowView view;
+		public object view;
 		public string data_member;
 		public object data_source;
 		public DataGridCell current;
 
-		public DataGridDataSource (DataGrid owner, CurrencyManager list_manager, object data_source, string data_member, DataRowView view, DataGridCell current)
+		public DataGridDataSource (DataGrid owner, CurrencyManager list_manager, object data_source, string data_member, object view_data, DataGridCell current)
 		{
 			this.owner = owner;
 			this.list_manager = list_manager;
-			this.view = view;
+			this.view = view_data;
 			this.data_source = data_source;
 			this.data_member = data_member;
 			this.current = current;
 		}
 
-		DataGridRow[] rows;
-		public DataGridRow[] Rows {
+		DataGridRelationshipRow[] rows;
+		public DataGridRelationshipRow[] Rows {
 			get { return rows; }
 			set { rows = value; }
 		}
@@ -216,14 +242,9 @@ namespace System.Windows.Forms
 		object datasource;
 		string datamember;
 		CurrencyManager list_manager;
+		bool refetch_list_manager = true;
 		bool _readonly;
-		DataGridRow[] rows;
-
-		/* these fields make BeginInit/EndInit work */
-		bool initializing;
-		bool need_databinding;
-		object new_datasource;
-		string new_datamember;
+		DataGridRelationshipRow[] rows;
 
 		/* column resize fields */
 		bool column_resize_active;
@@ -269,7 +290,7 @@ namespace System.Windows.Forms
 			parentrowslabel_style = DataGridParentRowsLabelStyle.Both;
 			selected_rows = new Hashtable ();
 			selection_start = -1;
-			rows = new DataGridRow [0];
+			rows = new DataGridRelationshipRow [0];
 
 			default_style = new DataGridTableStyle (true);
 			grid_style = new DataGridTableStyle ();
@@ -288,8 +309,11 @@ namespace System.Windows.Forms
 
 			dataSourceStack = new Stack ();
 
+			// back these out until we figure out what's going on with ResourceImageLoader and the call to MakeTransparent
 			back_button_image = ResourceImageLoader.Get ("go-previous.png");
+			//back_button_image.MakeTransparent (Color.Transparent);
 			parent_rows_button_image = ResourceImageLoader.Get ("go-top.png");
+			//parent_rows_button_image.MakeTransparent (Color.Transparent);
 		}
 
 		#endregion	// Public Constructor
@@ -463,10 +487,7 @@ namespace System.Windows.Forms
 				if (value.RowNumber != current_cell.RowNumber) {
 					if (!from_positionchanged_handler) {
 						try {
-							if (cursor_in_add_row && !add_row_changed)
-								ListManager.CancelCurrentEdit ();
-							else
-								ListManager.EndCurrentEdit ();
+							ListManager.EndCurrentEdit ();
 						}
 						catch (Exception e) {
 							DialogResult r = MessageBox.Show (String.Format ("{0} Do you wish to correct the value?", e.Message),
@@ -481,6 +502,9 @@ namespace System.Windows.Forms
 						}
 					}
 
+					if (value.RowNumber == RowsCount && !ListManager.CanAddRows)
+						value.RowNumber --;
+
 					if (value.RowNumber == RowsCount) {
 						cursor_in_add_row = true;
 						add_row_changed = false;
@@ -545,15 +569,14 @@ namespace System.Windows.Forms
 		public string DataMember {
 			get { return datamember; }
 			set {
-				if (initializing) {
-					need_databinding = true;
-					new_datamember = value;
+				if (BindingContext != null) {
+					SetDataSource (datasource, value);
 				}
 				else {
-					if (SetDataMember (value)) {					
-						SetDataSource (datasource);
-						SetNewDataSource ();
-					}
+					if (list_manager != null)
+						list_manager = null;
+					datamember = value;
+					refetch_list_manager = true;
 				}
 			}
 		}
@@ -564,14 +587,17 @@ namespace System.Windows.Forms
 		public object DataSource {
 			get { return datasource; }
 			set {
-				if (initializing) {
-					need_databinding = true;
-					new_datasource = value;
+				if (BindingContext != null) {
+					SetDataSource (value, ListManager == null ? datamember : string.Empty);
 				}
 				else {
-					SetDataMember ("");
-					SetDataSource (value);
-					SetNewDataSource ();
+					datasource = value;
+					if (list_manager != null)
+						datamember = string.Empty;
+
+					if (list_manager != null)
+						list_manager = null;
+					refetch_list_manager = true;
 				}
 			}
 		}
@@ -686,18 +712,10 @@ namespace System.Windows.Forms
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected internal CurrencyManager ListManager {
 			get {
-				if (BindingContext == null || datasource == null)
-					return null;
-
-				if (list_manager != null)
-					return list_manager;
-
-				list_manager = (CurrencyManager) BindingContext [datasource, datamember];
-
-				if (list_manager != null)
-					ConnectListManagerEvents ();
-
-				RecreateDataGridRows ();
+				if (list_manager == null && refetch_list_manager) {
+					SetDataSource (datasource, datamember);
+					refetch_list_manager = false;
+				}
 
 				return list_manager;
 			}
@@ -793,7 +811,7 @@ namespace System.Windows.Forms
 			set { grid_style.RowHeaderWidth = value; }
 		}
 
-		internal DataGridRow[] Rows {
+		internal DataGridRelationshipRow[] DataGridRows {
 			get { return rows; }
 		}
 
@@ -922,11 +940,13 @@ namespace System.Windows.Forms
 
 		public void BeginInit ()
 		{
-			initializing = true;
 		}
 
 		protected virtual void CancelEditing ()
 		{
+			if (CurrentTableStyle.GridColumnStyles.Count == 0)
+				return;
+
 			CurrentTableStyle.GridColumnStyles[current_cell.ColumnNumber].ConcedeFocus ();
 
 			if (is_changing) {
@@ -955,7 +975,7 @@ namespace System.Windows.Forms
 			for (int i = 1; i < rows.Length - row; i ++)
 				rows[row + i].VerticalOffset -= rows[row].RelationHeight;
 
-			rows[row].Height -= rows[row].RelationHeight;
+			rows[row].height -= rows[row].RelationHeight;
 			rows[row].RelationHeight = 0;
 			ResumeLayout (false);
 
@@ -977,7 +997,7 @@ namespace System.Windows.Forms
 			is_changing = true;
 
 			if (cursor_in_add_row && need_invalidate)
-				RecreateDataGridRows ();
+				RecreateDataGridRows (true);
 
 			if (need_invalidate)
 				InvalidateRowHeader (CurrentRow);
@@ -1023,12 +1043,6 @@ namespace System.Windows.Forms
 
 		public void EndInit ()
 		{
-			initializing = false;
-			if (need_databinding) {
-				SetDataMember (new_datamember);
-				SetDataSource (new_datasource);
-				SetNewDataSource ();
-			}
 		}
 
 		public void Expand (int row)
@@ -1060,7 +1074,7 @@ namespace System.Windows.Forms
 
 			for (i = 1; i < rows.Length - row; i ++)
 				rows[row + i].VerticalOffset += rows[row].relation_area.Height;
-			rows[row].Height += rows[row].relation_area.Height;
+			rows[row].height += rows[row].relation_area.Height;
 			rows[row].RelationHeight = rows[row].relation_area.Height;
 
 			/* XX need to redraw from @row down */
@@ -1179,16 +1193,24 @@ namespace System.Windows.Forms
 				if (y <= pos_y + rows[row].Height) {
 					int col_pixel;
 					int column_cnt = first_visiblecolumn + visiblecolumn_count;
-					for (int column = first_visiblecolumn; column < column_cnt; column++) {
+					if (column_cnt > 0) {
+						for (int column = first_visiblecolumn; column < column_cnt; column++) {
+							if (CurrentTableStyle.GridColumnStyles[column].bound == false)
+								continue;
+							col_pixel = GetColumnStartingPixel (column);
+							pos_x = cells_area.X + col_pixel - horiz_pixeloffset;
+							width = CurrentTableStyle.GridColumnStyles[column].Width;
 
-						col_pixel = GetColumnStartingPixel (column);
-						pos_x = cells_area.X + col_pixel - horiz_pixeloffset;
-						width = CurrentTableStyle.GridColumnStyles[column].Width;
-
-						if (x <= pos_x + width) { // Column found
-							return new HitTestInfo (row, column, HitTestType.Cell);
+							if (x <= pos_x + width) { // Column found
+								return new HitTestInfo (row, column, HitTestType.Cell);
+							}
 						}
 					}
+					else if (CurrentTableStyle.HasRelations) {
+						/* XXX this needs checking against MS somehow... */
+						if (x < rows[row].relation_area.X + rows[row].relation_area.Width)
+							return new HitTestInfo (row, 0/*XXX?*/, HitTestType.Cell);
+					}
 
 					break;
 				}
@@ -1218,14 +1240,9 @@ namespace System.Windows.Forms
 			rows = source.Rows;
 			selected_rows = source.SelectedRows;
 			selection_start = source.SelectionStart;
-			datamember = source.data_member;
-			datasource = source.data_source;
-
-			SetNewDataSource ();
-			OnDataSourceChanged (EventArgs.Empty);
+			SetDataSource (source.data_source, source.data_member);
 
 			CurrentCell = source.current;
-			CalcAreasAndInvalidate ();
 		}
 
 		[MonoTODO]
@@ -1234,7 +1251,7 @@ namespace System.Windows.Forms
 			if (allow_navigation == false)
 				return;
 
-			DataGridDataSource previous_source = new DataGridDataSource (this, list_manager, datasource, datamember, (DataRowView)list_manager.Current, CurrentCell);
+			DataGridDataSource previous_source = new DataGridDataSource (this, list_manager, datasource, datamember, list_manager.Current, CurrentCell);
 			previous_source.Rows = rows;
 			previous_source.SelectedRows = selected_rows;
 			previous_source.SelectionStart = selection_start;
@@ -1277,8 +1294,7 @@ namespace System.Windows.Forms
 		{
 			base.OnBindingContextChanged (e);
 
-			current_style.CreateColumnsForTable (false);
-			CalcAreasAndInvalidate ();
+			SetDataSource (datasource, datamember);
 		}
 
 		protected virtual void OnBorderStyleChanged (EventArgs e)
@@ -1487,21 +1503,31 @@ namespace System.Windows.Forms
 				break;
 
 			case HitTestType.ColumnResize:
-				resize_column = testinfo.Column;
-				column_resize_active = true;
-				resize_column_x = e.X;
-				resize_column_width_delta = 0;
-				EndEdit ();
-				DrawResizeLineVert (resize_column_x);
+				if (e.Clicks == 2) {
+					// double click column resize goes here
+				}
+				else {
+					resize_column = testinfo.Column;
+					column_resize_active = true;
+					resize_column_x = e.X;
+					resize_column_width_delta = 0;
+					EndEdit ();
+					DrawResizeLineVert (resize_column_x);
+				}
 				break;
 
 			case HitTestType.RowResize:
-				resize_row = testinfo.Row;
-				row_resize_active = true;
-				resize_row_y = e.Y;
-				resize_row_height_delta = 0;
-				EndEdit ();
-				DrawResizeLineHoriz (resize_row_y);
+				if (e.Clicks == 2) {
+					// double click row resize goes here
+				}
+				else {
+					resize_row = testinfo.Row;
+					row_resize_active = true;
+					resize_row_y = e.Y;
+					resize_row_height_delta = 0;
+					EndEdit ();
+					DrawResizeLineHoriz (resize_row_y);
+				}
 				break;
 
 			case HitTestType.Caption:
@@ -1621,7 +1647,7 @@ namespace System.Windows.Forms
 				if (resize_row_height_delta + rows[resize_row].Height < 0)
 					resize_row_height_delta = -rows[resize_row].Height;
 
-				rows[resize_row].Height = rows[resize_row].Height + resize_row_height_delta;
+				rows[resize_row].height = rows[resize_row].Height + resize_row_height_delta;
 				for (int i = resize_row + 1; i < rows.Length; i ++)
 					rows[i].VerticalOffset += resize_row_height_delta;
 
@@ -1780,8 +1806,9 @@ namespace System.Windows.Forms
 				if (alt_pressed) {
 					if (is_editing)
 						CurrentTableStyle.GridColumnStyles[CurrentColumn].EnterNullValue ();
+					return true;
 				}
-				return true;
+				return false;
 
 			case Keys.Enter:
 				if (is_changing)
@@ -1789,10 +1816,18 @@ namespace System.Windows.Forms
 				return true;
 
 			case Keys.Tab:
-				if (CurrentColumn < CurrentTableStyle.GridColumnStyles.Count - 1)
-					CurrentColumn ++;
-				else if ((CurrentRow <= RowsCount - 1) && (CurrentColumn == CurrentTableStyle.GridColumnStyles.Count - 1))
-					CurrentCell = new DataGridCell (CurrentRow + 1, 0);
+				if (shift_pressed) {
+					if (CurrentColumn > 0)
+						CurrentColumn --;
+					else if ((CurrentRow > 0) && (CurrentColumn == 0))
+						CurrentCell = new DataGridCell (CurrentRow - 1, CurrentTableStyle.GridColumnStyles.Count - 1);
+				}
+				else {
+					if (CurrentColumn < CurrentTableStyle.GridColumnStyles.Count - 1)
+						CurrentColumn ++;
+					else if ((CurrentRow <= RowsCount) && (CurrentColumn == CurrentTableStyle.GridColumnStyles.Count - 1))
+						CurrentCell = new DataGridCell (CurrentRow + 1, 0);
+				}
 
 				UpdateSelectionAfterCursorMove (false);
 
@@ -1938,7 +1973,7 @@ namespace System.Windows.Forms
 
 		public override void ResetForeColor ()
 		{
-			base.ResetForeColor ();
+			grid_style.ForeColor = default_style.ForeColor;
 		}
 
 		public void ResetGridLineColor ()
@@ -2009,10 +2044,7 @@ namespace System.Windows.Forms
 
 		public void SetDataBinding (object dataSource, string dataMember)
 		{
-			this.datamember = string.Empty;
-			SetDataSource (dataSource);
-			SetDataMember (dataMember);		
-			SetNewDataSource ();
+			SetDataSource (dataSource, dataMember);
 		}
 
 		protected virtual bool ShouldSerializeAlternatingBackColor ()
@@ -2147,45 +2179,55 @@ namespace System.Windows.Forms
 				vert_scrollbar.Value = first_visiblerow;
 			}
 		}
-		
-		private bool SetDataMember (string member)
+
+		private void SetDataSource (object source, string member)
 		{
-			if (member == datamember)
-				return false;
-
-			datamember = member;
-
-			if (list_manager != null) {
-				DisconnectListManagerEvents ();
-				list_manager = null;
-			}
-
-			return true;
+			SetDataSource (source, member, true);
 		}
 
-		private void SetDataSource (object source)
-		{			
-			if (source != null && source as IListSource != null && source as IList != null) {
+		bool in_setdatasource;
+		private void SetDataSource (object source, string member, bool recreate_rows)
+		{
+			CurrencyManager old_lm = list_manager;
+
+			/* we need this bool flag to work around a
+			 * problem with OnBindingContextChanged.  once
+			 * that stuff works properly, remove this
+			 * hack */
+			if (in_setdatasource)
+				return;
+			in_setdatasource = true;
+
+#if false
+			if (datasource == source && member == datamember)
+				return;
+#endif
+
+			if (source != null && source as IListSource != null && source as IList != null)
 				throw new Exception ("Wrong complex data binding source");
-			}
+
+			datasource = source;
+			datamember = member;
 
 			if (is_editing)
 				CancelEditing ();
 
 			current_cell = new DataGridCell ();
-			datasource = source;
-			if (list_manager != null) {
+
+			if (list_manager != null)
 				DisconnectListManagerEvents ();
-				list_manager = null;
-			}
 
-			OnDataSourceChanged (EventArgs.Empty);
-		}
+			list_manager = null;
 
-		private void SetNewDataSource ()
-		{
-			if (ListManager != null) {
-				string list_name = ListManager.GetListName (null);
+			/* create the new list manager */
+			if (BindingContext != null && datasource != null)
+				list_manager = (CurrencyManager) BindingContext [datasource, datamember];
+
+			if (list_manager != null)
+				ConnectListManagerEvents ();
+
+			if (list_manager != null) {
+				string list_name = list_manager.GetListName (null);
 				if (TableStyles[list_name] == null) {
 					current_style.GridColumnStyles.Clear ();			
 					current_style.CreateColumnsForTable (false);
@@ -2195,13 +2237,30 @@ namespace System.Windows.Forms
 					CurrentTableStyle = styles_collection[list_name];
 					current_style.CreateColumnsForTable (true);
 				}
-				else
-					current_style.CreateColumnsForTable (false);
+				else {
+					current_style.CreateColumnsForTable (true);
+				}
 			}
 			else
 				current_style.CreateColumnsForTable (false);
-			
-			CalcAreasAndInvalidate ();			
+
+			if (old_lm != list_manager) {
+				/* reset first_visiblerow to 0 here before
+				 * doing anything that'll requires us to
+				 * figure out if we need a scrollbar. */
+				vert_scrollbar.Value = 0;
+				horiz_scrollbar.Value = 0;
+				first_visiblerow = 0;
+
+				if (recreate_rows)
+					RecreateDataGridRows (false);
+			}
+
+			CalcAreasAndInvalidate ();
+
+			in_setdatasource = false;
+
+			OnDataSourceChanged (EventArgs.Empty);
 		}
 
 		private void OnListManagerPositionChanged (object sender, EventArgs e)
@@ -2211,9 +2270,9 @@ namespace System.Windows.Forms
 			from_positionchanged_handler = false;
 		}
 
-		void RecreateDataGridRows ()
+		void RecreateDataGridRows (bool recalc)
 		{
-			DataGridRow[] new_rows = new DataGridRow[RowsCount + (ShowEditRow ? 1 : 0)];
+			DataGridRelationshipRow[] new_rows = new DataGridRelationshipRow[RowsCount + (ShowEditRow ? 1 : 0)];
 			int start_index = 0;
 			if (rows != null) {
 				start_index = rows.Length;
@@ -2221,14 +2280,28 @@ namespace System.Windows.Forms
 			}
 
 			for (int i = start_index; i < new_rows.Length; i ++) {
-				new_rows[i] = new DataGridRow ();
-				new_rows[i].Height = RowHeight;
+				new_rows[i] = new DataGridRelationshipRow (this);
+				new_rows[i].height = RowHeight;
 				if (i > 0)
 					new_rows[i].VerticalOffset = new_rows[i-1].VerticalOffset + new_rows[i-1].Height;
 			}
 
 			rows = new_rows;
 
+			if (recalc)
+				CalcAreasAndInvalidate ();
+		}
+
+		internal void UpdateRowsFrom (DataGridRelationshipRow row)
+		{
+			int start_index = Array.IndexOf (rows, row);
+			if (start_index == -1)
+				return;
+
+			for (int i = start_index + 1; i < rows.Length; i ++) {
+				rows[i].VerticalOffset = rows[i-1].VerticalOffset + rows[i-1].Height;
+			}
+
 			CalcAreasAndInvalidate ();
 		}
 
@@ -2236,8 +2309,8 @@ namespace System.Windows.Forms
 		{
 			if (e.Index == -1) {
 				ResetSelection ();
-				if (rows == null || RowsCount != rows.Length)
-					RecreateDataGridRows ();
+				if (rows == null || RowsCount != rows.Length - (ShowEditRow ? 1 : 0))
+					RecreateDataGridRows (true);
 			}
 			else {
 				InvalidateRow (e.Index);
@@ -2299,9 +2372,12 @@ namespace System.Windows.Forms
 
 		private void Edit ()
 		{
+			if (CurrentTableStyle.GridColumnStyles[current_cell.ColumnNumber].bound == false)
+				return;
+
 			is_editing = true;
 			is_changing = false;
-
+			
 			CurrentTableStyle.GridColumnStyles[current_cell.ColumnNumber].Edit (ListManager,
 				current_cell.RowNumber, GetCellBounds (current_cell.RowNumber, current_cell.ColumnNumber),
 				_readonly, "", true);
@@ -2309,6 +2385,12 @@ namespace System.Windows.Forms
 
 		private void EndEdit ()
 		{
+			if (CurrentTableStyle.GridColumnStyles.Count == 0)
+				return;
+
+			if (CurrentTableStyle.GridColumnStyles[current_cell.ColumnNumber].bound == false)
+				return;
+
 			EndEdit (CurrentTableStyle.GridColumnStyles[current_cell.ColumnNumber],
 				 current_cell.RowNumber,
 				 false);
@@ -2474,9 +2556,12 @@ namespace System.Windows.Forms
 			int width = 0;
 			int cnt = CurrentTableStyle.GridColumnStyles.Count;
 
-			for (int col = 0; col < cnt; col++)
+			for (int col = 0; col < cnt; col++) {
+				if (CurrentTableStyle.GridColumnStyles[col].bound == false) {
+					continue;
+				}
 				width += CurrentTableStyle.GridColumnStyles[col].Width;
-
+			}
 			return width;
 		}
 
@@ -2496,6 +2581,9 @@ namespace System.Windows.Forms
 			}
 
 			for (int col = 0; col < cnt; col++) {
+				if (CurrentTableStyle.GridColumnStyles[col].bound == false)
+					continue;
+
 				width += CurrentTableStyle.GridColumnStyles[col].Width;
 
 				if (pixel < width)
@@ -2513,6 +2601,9 @@ namespace System.Windows.Forms
 			int cnt = CurrentTableStyle.GridColumnStyles.Count;
 
 			for (int col = 0; col < cnt; col++) {
+				if (CurrentTableStyle.GridColumnStyles[col].bound == false) {
+					continue;
+				}
 
 				if (my_col == col)
 					return width;
@@ -2531,6 +2622,8 @@ namespace System.Windows.Forms
 			
 			if (column > current_first_visiblecolumn) { // Going forward								
 				for (new_col = column; new_col >= 0; new_col--){
+					if (CurrentTableStyle.GridColumnStyles[new_col].bound == false)
+						continue;
 					width += CurrentTableStyle.GridColumnStyles[new_col].Width;
 					
 					if (width >= cells_area.Width)
@@ -2588,7 +2681,7 @@ namespace System.Windows.Forms
 				UpdateVisibleRowCount ();
 
 				needHoriz = (width_of_all_columns > visible_cells_width);
-				needVert = (visiblerow_count != allrows);
+				needVert = (allrows > visiblerow_count);
 			}
 
 			int horiz_scrollbar_width = ClientRectangle.Width;
@@ -2778,7 +2871,13 @@ namespace System.Windows.Forms
 			col = FromPixelToColumn (max_pixel, out unused);
 			
 			visiblecolumn_count = 1 + col - first_visiblecolumn;
-			
+
+			visiblecolumn_count = 0;
+			for (int i = first_visiblecolumn; i <= col; i ++) {
+				if (CurrentTableStyle.GridColumnStyles[i].bound)
+					visiblecolumn_count++;
+			}
+
 			if (first_visiblecolumn + visiblecolumn_count < CurrentTableStyle.GridColumnStyles.Count) { 
 				visiblecolumn_count++; // Partially visible column
 			}
