@@ -171,7 +171,8 @@ namespace System.Windows.Forms
 
 				if (backcolor != value) {
 					backcolor = value;
-					OnBackColorChanged (EventArgs.Empty);
+					// XXX This should be OnBackColorChanged, MS made a c&p error, I think
+					OnForeColorChanged (EventArgs.Empty);
 
 					if (datagrid != null) {
 						datagrid.Refresh ();
@@ -221,7 +222,8 @@ namespace System.Windows.Forms
 
 				if (forecolor != value) {
 					forecolor = value;
-					OnForeColorChanged (EventArgs.Empty);
+					// XXX This should be OnForeColorChanged, MS made a c&p error, I think
+					OnBackColorChanged (EventArgs.Empty);
 
 					if (datagrid != null) {
 						datagrid.Refresh ();
@@ -358,7 +360,8 @@ namespace System.Windows.Forms
 			set {
 				if (link_hovercolor != value) {
 					link_hovercolor = value;
-					OnLinkHoverColorChanged (EventArgs.Empty);
+					// XXX MS doesn't emit this event (even though they should...)
+					// OnLinkHoverColorChanged (EventArgs.Empty);
 				}
 			}
 		}
@@ -867,6 +870,9 @@ namespace System.Windows.Forms
 					return;
 			}
 
+			for (int i = 0; i < column_styles.Count; i ++)
+				column_styles[i].bound = false;
+
 			table_relations.Clear ();
 			PropertyDescriptorCollection propcol = mgr.GetItemProperties ();
 
@@ -879,7 +885,7 @@ namespace System.Windows.Forms
 						st.Width = CurrentPreferredColumnWidth;
 
 					st.PropertyDescriptor = propcol[i];
-
+					st.bound = true;
 					continue;
 				}
 
@@ -891,6 +897,7 @@ namespace System.Windows.Forms
 					table_relations.Add (propcol[i].Name);
 				} else {
 					st = CreateGridColumn (propcol[i],  true);
+					st.bound = true;
 					st.grid = datagrid;
 					st.MappingName = propcol[i].Name;
 					st.HeaderText = propcol[i].Name;
