@@ -26,10 +26,6 @@
 
 // NOT COMPLETE - work in progress
 
-// TODO:
-// - select values for font/style/size via the TextBoxes
-// - etc
-
 using System.ComponentModel;
 using System.Drawing;
 using System.Text.RegularExpressions;
@@ -72,9 +68,9 @@ namespace System.Windows.Forms
 		private TextBox fontstyleTextBox;
 		private TextBox fontsizeTextBox;
 		
-		private ListBox fontListBox;
-		private ListBox fontstyleListBox;
-		private ListBox fontsizeListBox;
+		private MouseWheelListBox fontListBox;
+		private MouseWheelListBox fontstyleListBox;
+		private MouseWheelListBox fontsizeListBox;
 		
 		private GroupBox effectsGroupBox;
 		private CheckBox strikethroughCheckBox;
@@ -106,14 +102,75 @@ namespace System.Windows.Forms
 		private Hashtable fontHash = new Hashtable();
 		
 		private int[] a_sizes = {
-			8, 9, 10, 11, 12, 14, 16, 18, 20, 22, 24, 26, 28, 36, 48, 72
+			6, 7, 8, 9, 10, 11, 12, 14, 16, 18, 20, 22, 24, 26, 28, 36, 48, 72
 		};
 		
+		// char set stuff is only here to make me happy :-)
+		private string [] char_sets_names = {
+			"Western",
+			"Symbol",
+			"Shift Jis",
+			"Hangul",
+			"GB2312",
+			"BIG5",
+			"Greek",
+			"Turkish",
+			"Hebrew",
+			"Arabic",
+			"Baltic",
+			"Vietname",
+			"Cyrillic",
+			"East European",
+			"Thai",
+			"Johab",
+			"Mac",
+			"OEM",
+			"VISCII",
+			"TCVN",
+			"KOI-8",
+			"ISO-8859-3",
+			"ISO-8859-4",
+			"ISO-8859-10",
+			"Celtic"
+		};
+		
+		private string [] char_sets = {
+			"AaBbYyZz",
+			"Symbol",
+			"Aa" + (char)0x3042 + (char)0x3041 + (char)0x30a2  + (char)0x30a1 + (char)0x4e9c + (char)0x5b87,
+			(char)0xac00 + (char)0xb098 + (char)0xb2e4 + "AaBYyZz",
+			new String(new Char [] {(char)0x5fae, (char)0x8f6f, (char)0x4e2d, (char)0x6587, (char)0x8f6f, (char)0x4ef6}),
+			new String(new Char [] {(char)0x4e2d, (char)0x6587, (char)0x5b57, (char)0x578b, (char)0x7bc4, (char)0x4f8b}),
+			"AaBb" + (char)0x0391 + (char)0x03b1 + (char)0x0392 + (char)0x03b2,
+			"AaBb" + (char)0x011e + (char)0x011f + (char)0x015e + (char)0x015f,
+			"AaBb" + (char)0x05e0 + (char)0x05e1 + (char)0x05e9 + (char)0x05ea,
+			"AaBb" + (char)0x0627 + (char)0x0628 + (char)0x062c + (char)0x062f + (char)0x0647 + (char)0x0648 + (char)0x0632,
+			"AaBbYyZz",
+			"AaBb" + (char)0x01a0 + (char)0x01a1 + (char)0x01af + (char)0x01b0,
+			"AaBb" + (char)0x0411 + (char)0x0431 + (char)0x0424 + (char)0x0444,
+			"AaBb" + (char)0xc1 + (char)0xe1 + (char)0xd4 + (char)0xf4,
+			"AaBb" + (char)0x0e2d + (char)0x0e31 + (char)0x0e01 + (char)0x0e29 + (char)0x0e23 + (char)0x0e44 + (char)0x0e17 +(char)0x0e22,
+			(char)0xac00 + (char)0xb098 + (char)0xb2e4 + "AaBYyZz",
+			"AaBbYyZz",
+			"AaBb" + (char)0xf8 + (char)0xf1 + (char)0xfd,
+			"",
+			"",
+			"",
+			"",
+			"",
+			"",
+			""
+		};
+		
+		private string example_panel_text;
+		
 		private bool internal_change = false;
 		
 		#region Public Constructors
 		public FontDialog( )
 		{
+			example_panel_text = char_sets [0];
+			
 			okButton = new Button( );
 			cancelButton = new Button( );
 			applyButton = new Button( );
@@ -123,8 +180,9 @@ namespace System.Windows.Forms
 			fontstyleTextBox = new TextBox( );
 			fontsizeTextBox = new TextBox( );
 			
-			fontListBox = new ListBox( );
-			fontsizeListBox = new ListBox( );
+			fontListBox = new MouseWheelListBox ();
+			fontsizeListBox = new MouseWheelListBox ();
+			fontstyleListBox = new MouseWheelListBox ();
 			
 			fontLabel = new Label( );
 			fontstyleLabel = new Label( );
@@ -132,7 +190,6 @@ namespace System.Windows.Forms
 			scriptLabel = new Label( );
 			
 			exampleGroupBox = new GroupBox( );
-			fontstyleListBox = new ListBox( );
 			
 			effectsGroupBox = new GroupBox( );
 			underlinedCheckBox = new CheckBox( );
@@ -170,6 +227,7 @@ namespace System.Windows.Forms
 			fontsizeTextBox.Size = new Size( 52, 21 );
 			fontsizeTextBox.TabIndex = 7;
 			fontsizeTextBox.Text = "";
+			fontsizeTextBox.MaxLength = 2;
 			// schriftartListBox
 			fontListBox.Location = new Point( 16, 47 );
 			fontListBox.Size = new Size( 140, 95 );
@@ -219,7 +277,7 @@ namespace System.Windows.Forms
 			scriptComboBox.Location = new Point( 164, 253 );
 			scriptComboBox.Size = new Size( 172, 21 );
 			scriptComboBox.TabIndex = 14;
-			scriptComboBox.Text = "-/-";
+			scriptComboBox.DropDownStyle = ComboBoxStyle.DropDownList;
 			// okButton
 			okButton.FlatStyle = FlatStyle.System;
 			okButton.Location = new Point( 352, 26 );
@@ -311,6 +369,11 @@ namespace System.Windows.Forms
 			
 			CreateFontSizeListBoxItems ();
 			
+			scriptComboBox.BeginUpdate ();
+			scriptComboBox.Items.AddRange (char_sets_names);
+			scriptComboBox.SelectedIndex = 0;
+			scriptComboBox.EndUpdate ();
+			
 			applyButton.Hide( );
 			helpButton.Hide( );
 			colorComboBox.Hide( );
@@ -324,10 +387,23 @@ namespace System.Windows.Forms
 			fontstyleListBox.SelectedIndexChanged += new EventHandler( OnSelectedIndexChangedFontStyleListBox );
 			underlinedCheckBox.CheckedChanged += new EventHandler( OnCheckedChangedUnderlinedCheckBox );
 			strikethroughCheckBox.CheckedChanged += new EventHandler( OnCheckedChangedStrikethroughCheckBox );
+			scriptComboBox.SelectedIndexChanged += new EventHandler (OnSelectedIndexChangedScriptComboBox);
 			
-			fontTextBox.KeyUp += new KeyEventHandler (OnFontTextBoxKeyUp);
-			fontstyleTextBox.KeyUp += new KeyEventHandler (OnFontStyleTextBoxKeyUp);
-			fontsizeTextBox.KeyUp += new KeyEventHandler (OnFontSizeTextBoxKeyUp);
+			fontTextBox.KeyPress += new KeyPressEventHandler (OnFontTextBoxKeyPress);
+			fontstyleTextBox.KeyPress += new KeyPressEventHandler (OnFontStyleTextBoxKeyPress);
+			fontsizeTextBox.KeyPress += new KeyPressEventHandler (OnFontSizeTextBoxKeyPress);
+			
+			fontTextBox.TextChanged += new EventHandler (OnFontTextBoxTextChanged);
+			fontstyleTextBox.TextChanged += new EventHandler (OnFontStyleTextTextChanged);
+			fontsizeTextBox.TextChanged += new EventHandler (OnFontSizeTextBoxTextChanged);
+			
+			fontTextBox.KeyDown += new KeyEventHandler (OnFontTextBoxKeyDown);
+			fontstyleTextBox.KeyDown += new KeyEventHandler (OnFontStyleTextBoxKeyDown);
+			fontsizeTextBox.KeyDown += new KeyEventHandler (OnFontSizeTextBoxKeyDown);
+			
+			fontTextBox.MouseWheel += new MouseEventHandler (OnFontTextBoxMouseWheel);
+			fontstyleTextBox.MouseWheel += new MouseEventHandler (OnFontStyleTextBoxMouseWheel);
+			fontsizeTextBox.MouseWheel += new MouseEventHandler (OnFontSizeTextBoxMouseWheel);
 			
 			Font = form.Font;
 		}
@@ -684,9 +760,7 @@ namespace System.Windows.Forms
 			
 			e.Graphics.FillRectangle( ThemeEngine.Current.ResPool.GetSolidBrush( SystemColors.Control ), 0, 0, 156, 40 );
 			
-			string text = "AaBbYyZz";
-			
-			SizeF fontSizeF = e.Graphics.MeasureString( text, font );
+			SizeF fontSizeF = e.Graphics.MeasureString( example_panel_text, font );
 			
 			int text_width = (int)fontSizeF.Width;
 			int text_height = (int)fontSizeF.Height;
@@ -696,7 +770,7 @@ namespace System.Windows.Forms
 			
 			int y = ( examplePanel.Height / 2 ) - ( text_height / 2 );
 			
-			e.Graphics.DrawString( text, font, brush, new Point( x, y ) );
+			e.Graphics.DrawString( example_panel_text, font, brush, new Point( x, y ) );
 		}
 		
 		void OnSelectedIndexChangedFontListBox( object sender, EventArgs e )
@@ -803,8 +877,125 @@ namespace System.Windows.Forms
 			UpdateExamplePanel( );
 		}
 		
-		void OnFontTextBoxKeyUp (object sender, KeyEventArgs e)
+		bool internal_textbox_change = false;
+		
+		void OnFontTextBoxMouseWheel (object sender, MouseEventArgs e)
 		{
+			fontListBox.SendMouseWheelEvent (e);
+		}
+		
+		void OnFontStyleTextBoxMouseWheel (object sender, MouseEventArgs e)
+		{
+			fontstyleListBox.SendMouseWheelEvent (e);
+		}
+		
+		void OnFontSizeTextBoxMouseWheel (object sender, MouseEventArgs e)
+		{
+			fontsizeListBox.SendMouseWheelEvent (e);
+		}
+		
+		void OnFontTextBoxKeyDown (object sender, KeyEventArgs e)
+		{
+			if (e.KeyCode == Keys.Up) {
+				int sel_index = fontListBox.SelectedIndex;
+				
+				sel_index--;
+				
+				if (sel_index < 0)
+					sel_index = 0;
+				
+				fontListBox.SelectedIndex = sel_index;
+			} else if (e.KeyCode == Keys.Down) {
+				int sel_index = fontListBox.SelectedIndex;
+				
+				sel_index++;
+				
+				if (sel_index > fontListBox.Items.Count - 1)
+					sel_index = fontListBox.Items.Count - 1;
+				
+				fontListBox.SelectedIndex = sel_index;
+			}
+		}
+		
+		void OnFontStyleTextBoxKeyDown (object sender, KeyEventArgs e)
+		{
+			if (e.KeyCode == Keys.Up) {
+				int sel_index = fontstyleListBox.SelectedIndex;
+				
+				sel_index--;
+				
+				if (sel_index < 0)
+					sel_index = 0;
+				
+				fontstyleListBox.SelectedIndex = sel_index;
+			} else if (e.KeyCode == Keys.Down) {
+				int sel_index = fontstyleListBox.SelectedIndex;
+				
+				sel_index++;
+				
+				if (sel_index > fontstyleListBox.Items.Count - 1)
+					sel_index = fontstyleListBox.Items.Count - 1;
+				
+				fontstyleListBox.SelectedIndex = sel_index;
+			}
+		}
+		
+		void OnFontSizeTextBoxKeyDown (object sender, KeyEventArgs e)
+		{
+			if (e.KeyCode == Keys.Up) {
+				int sel_index = fontsizeListBox.SelectedIndex;
+				
+				sel_index--;
+				
+				if (sel_index < 0)
+					sel_index = 0;
+				
+				fontsizeListBox.SelectedIndex = sel_index;
+			} else if (e.KeyCode == Keys.Down) {
+				int sel_index = fontsizeListBox.SelectedIndex;
+				
+				sel_index++;
+				
+				if (sel_index > fontsizeListBox.Items.Count - 1)
+					sel_index = fontsizeListBox.Items.Count - 1;
+				
+				fontsizeListBox.SelectedIndex = sel_index;
+			}
+		}
+		
+		void OnFontTextBoxKeyPress (object sender, KeyPressEventArgs e)
+		{
+			internal_textbox_change = true;
+			
+			if (fontListBox.SelectedIndex > -1)
+				fontListBox.SelectedIndex = -1;
+		}
+		
+		void OnFontStyleTextBoxKeyPress (object sender, KeyPressEventArgs e)
+		{
+			internal_textbox_change = true;
+			
+			if (fontstyleListBox.SelectedIndex > -1)
+				fontstyleListBox.SelectedIndex = -1;
+		}
+		
+		void OnFontSizeTextBoxKeyPress (object sender, KeyPressEventArgs e)
+		{
+			if (Char.IsLetter (e.KeyChar) || Char.IsWhiteSpace (e.KeyChar) || Char.IsPunctuation (e.KeyChar) || e.KeyChar == ',') {
+				e.Handled = true;
+				return; 
+			}
+			
+			internal_textbox_change = true;
+		}
+		
+		void OnFontTextBoxTextChanged (object sender, EventArgs e)
+		{
+			if (!internal_textbox_change)
+				return;
+			
+			internal_textbox_change = false;
+			
 			for (int i = 0; i < fontListBox.Items.Count; i++) {
 				string name = fontListBox.Items [i] as string;
 				
@@ -819,24 +1010,35 @@ namespace System.Windows.Forms
 			}
 		}
 		
-		void OnFontStyleTextBoxKeyUp (object sender, KeyEventArgs e)
+		void OnFontStyleTextTextChanged (object sender, EventArgs e)
 		{
+			if (!internal_textbox_change)
+				return;
+			
+			internal_textbox_change = false;
+			
 			for (int i = 0; i < fontstyleListBox.Items.Count; i++) {
 				string name = fontstyleListBox.Items [i] as string;
 				
 				if (name.StartsWith(fontstyleTextBox.Text)) {
 					if (name == fontstyleTextBox.Text)
 						fontstyleListBox.SelectedIndex = i;
-					else
-						fontstyleListBox.TopIndex = i;
 					
 					break;
 				}
 			}
 		}
 		
-		void OnFontSizeTextBoxKeyUp (object sender, KeyEventArgs e)
+		void OnFontSizeTextBoxTextChanged (object sender, EventArgs e)
 		{
+			if (!internal_textbox_change)
+				return;
+			
+			internal_textbox_change = false;
+			
+			if (fontsizeTextBox.Text.Length == 0)
+				return;
+			
 			for (int i = 0; i < fontsizeListBox.Items.Count; i++) {
 				string name = fontsizeListBox.Items [i] as string;
 				
@@ -851,6 +1053,17 @@ namespace System.Windows.Forms
 			}
 		}
 		
+		void OnSelectedIndexChangedScriptComboBox (object sender, EventArgs e)
+		{
+			string tmp_str = char_sets [scriptComboBox.SelectedIndex];
+			
+			if (tmp_str.Length > 0) {
+				example_panel_text = tmp_str;
+				
+				UpdateExamplePanel ();
+			}
+		}
+		
 		void UpdateExamplePanel( )
 		{
 			if (font != null)
@@ -1054,5 +1267,13 @@ namespace System.Windows.Forms
 			remove { Events.RemoveHandler (EventApply, value); }
 		}
 	}
+		
+	internal class MouseWheelListBox : ListBox
+	{
+		public void SendMouseWheelEvent(MouseEventArgs e)
+		{
+			OnMouseWheel (e);
+		}
+	}
 }
 
