@@ -42,7 +42,6 @@ namespace System.Windows.Forms {
 	[ToolboxItem(false)]
 	public class Form : ContainerControl {
 		#region Local Variables
-		internal static Form		active_form;
 		internal bool			closing;
 		FormBorderStyle			form_border_style;
 		private bool		        autoscale;
@@ -79,23 +78,46 @@ namespace System.Windows.Forms {
 		internal ApplicationContext	context;
 		Color				transparency_key;
 		internal MenuTracker		active_tracker;
+		private bool			is_loaded;
 
 		#endregion	// Local Variables
 
 		#region Private & Internal Methods
 		static Form ()
 		{
-			default_icon = (Icon)Locale.GetResource("mono.ico");
+			default_icon = Locale.GetResource("mono.ico") as Icon;
 		}
 
 		// warning: this is only hooked up when an mdi container is created.
 		private void ControlAddedHandler (object sender, ControlEventArgs e)
 		{
 			if (mdi_container != null) {
-				Console.WriteLine ("SENDING TO BACK");
 				mdi_container.SendToBack ();
 			}
 		}
+
+		private void SelectActiveControl ()
+		{
+			if (this.IsMdiContainer)
+				return;
+				
+			if (this.ActiveControl == null) {
+				bool visible;
+
+				// This visible hack is to work around CanSelect always being false if one of the parents
+				// is not visible; and we by default create Form invisible...
+				visible = this.is_visible;
+				this.is_visible = true;
+
+				if (SelectNextControl (this, true, true, true, true) == false) {
+					Select (this);
+				}
+
+				this.is_visible = visible;
+			} else {
+				Select (ActiveControl);
+			}
+		}
 		#endregion	// Private & Internal Methods
 
 		#region Public Classes
@@ -154,6 +176,10 @@ namespace System.Windows.Forms {
 			default_maximized_bounds = Rectangle.Empty;
 			owned_forms = new Form.ControlCollection(this);
 			transparency_key = Color.Empty;
+
+			// FIXME: this should disappear just as soon as the handle creation is done in the right place (here is too soon()
+			UpdateBounds();
+
 		}
 		#endregion	// Public Constructor & Destructor
 
@@ -211,11 +237,11 @@ namespace System.Windows.Forms {
 					return;
 				}
 
-				if (XplatUI.SupportsTransparency()) {
-					allow_transparency = value;
+				allow_transparency = value;
 
-					if (value) {
-						if (IsHandleCreated) {
+				if (value) {
+					if (IsHandleCreated) {
+						if ((XplatUI.SupportsTransparency() & TransparencySupport.Set) != 0) {
 							XplatUI.SetWindowTransparency(Handle, Opacity, TransparencyKey);
 						}
 					} else {
@@ -262,7 +288,14 @@ namespace System.Windows.Forms {
 
 		public override Color BackColor {
 			get {
-				return base.BackColor;
+				/* we don't let parents override our
+				 default background color for forms.
+				 this fixes the default color for mdi
+				 children. */
+				if (background_color.IsEmpty)
+					return DefaultBackColor;
+				else
+					return background_color;
 			}
 			set {
 				base.BackColor = value;
@@ -344,9 +377,7 @@ namespace System.Windows.Forms {
 							typeof (DialogResult));
 
 				dialog_result = value;
-				if (is_modal) {
-					closing = true;
-				}
+				closing = (dialog_result != DialogResult.None && is_modal);
 			}
 		}
 
@@ -523,6 +554,7 @@ namespace System.Windows.Forms {
 					throw new ArgumentException ();
 
 				if (mdi_parent != null) {
+					mdi_parent.MdiContainer.original_order.Remove (this);
 					mdi_parent.MdiContainer.Controls.Remove (this);
 				}
 
@@ -530,6 +562,7 @@ namespace System.Windows.Forms {
 					mdi_parent = value;
 					window_manager = new MdiWindowManager (this,
 							mdi_parent.MdiContainer);
+					mdi_parent.MdiContainer.original_order.Add (this);
 					mdi_parent.MdiContainer.Controls.Add (this);
 
 					RecreateHandle ();
@@ -693,6 +726,11 @@ namespace System.Windows.Forms {
 		[MWFCategory("Window Style")]
 		public double Opacity {
 			get {
+				if (IsHandleCreated) {
+					if ((XplatUI.SupportsTransparency () & TransparencySupport.Get) != 0)
+						return XplatUI.GetWindowTransparency (Handle);
+				}
+
 				return opacity;
 			}
 
@@ -703,7 +741,8 @@ namespace System.Windows.Forms {
 
 				if (IsHandleCreated) {
 					UpdateStyles();
-					XplatUI.SetWindowTransparency(Handle, opacity, TransparencyKey);
+					if ((XplatUI.SupportsTransparency () & TransparencySupport.Set) != 0)
+						XplatUI.SetWindowTransparency(Handle, opacity, TransparencyKey);
 				}
 			}
 		}
@@ -738,7 +777,8 @@ namespace System.Windows.Forms {
 						owner.RemoveOwnedForm(this);
 					}
 					owner = value;
-					owner.AddOwnedForm(this);
+					if (owner != null)
+						owner.AddOwnedForm(this);
 					if (IsHandleCreated) {
 						if (owner != null && owner.IsHandleCreated) {
 							XplatUI.SetTopmost(this.window.Handle, owner.window.Handle, true);
@@ -885,7 +925,8 @@ namespace System.Windows.Forms {
 
 				AllowTransparency = true;
 				UpdateStyles();
-				XplatUI.SetWindowTransparency(Handle, Opacity, transparency_key);
+				if ((XplatUI.SupportsTransparency () & TransparencySupport.Set) != 0)
+					XplatUI.SetWindowTransparency(Handle, Opacity, transparency_key);
 			}
 		}
 
@@ -898,12 +939,9 @@ namespace System.Windows.Forms {
 					if (window_manager != null)
 						return window_manager.GetWindowState ();
 
-					try {
-						window_state = XplatUI.GetWindowState(Handle);
-					}
-
-					catch(NotSupportedException) {
-					}
+					FormWindowState new_state = XplatUI.GetWindowState(Handle);
+					if (new_state != (FormWindowState)(-1))
+						window_state = new_state;
 				}
 
 				return window_state;
@@ -919,12 +957,7 @@ namespace System.Windows.Forms {
 						return;
 					}
 
-					try {
-						XplatUI.SetWindowState(Handle, value);
-					}
-
-					catch(NotSupportedException) {
-					}
+					XplatUI.SetWindowState(Handle, value);
 				}
 			}
 		}
@@ -987,7 +1020,7 @@ namespace System.Windows.Forms {
 
 						case FormBorderStyle.FixedDialog: {
 							cp.Style |= (int)(WindowStyles.WS_CAPTION | WindowStyles.WS_BORDER);
-							cp.ExStyle |= (int)(WindowExStyles.WS_EX_DLGMODALFRAME);
+							cp.ExStyle |= (int)(WindowExStyles.WS_EX_DLGMODALFRAME | WindowExStyles.WS_EX_CONTROLPARENT);
 							break;
 						}
 
@@ -1132,9 +1165,15 @@ namespace System.Windows.Forms {
 			Form	active;
 
 			// The docs say activate only activates if our app is already active
-			active = ActiveForm;
-			if ((active != null) && (this != active)) {
-				XplatUI.Activate(window.Handle);
+			if (IsHandleCreated) {
+				if (IsMdiChild) {
+					MdiParent.ActivateMdiChild (this);
+				} else {
+					active = ActiveForm;
+					if ((active != null) && (this != active)) {
+						XplatUI.Activate(window.Handle);
+					}
+				}
 			}
 		}
 
@@ -1146,9 +1185,19 @@ namespace System.Windows.Forms {
 		}
 
 		public void Close () {
-			if (!IsDisposed) {
-				XplatUI.SendMessage(this.Handle, Msg.WM_CLOSE, IntPtr.Zero, IntPtr.Zero);
-			}
+			if (IsDisposed)
+				return;
+
+			if (!is_visible)
+				return;
+
+#if NET_2_0
+			FormClosingEventArgs ce = new FormClosingEventArgs (CloseReason.FormOwnerClosing, false);
+		        OnFormClosing (ce);
+			if (ce.Cancel)
+				return;
+#endif
+			XplatUI.SendMessage(this.Handle, Msg.WM_CLOSE, IntPtr.Zero, IntPtr.Zero);
 		}
 
 		public void LayoutMdi(MdiLayout value) {
@@ -1170,11 +1219,10 @@ namespace System.Windows.Forms {
 		}
 
 		public DialogResult ShowDialog() {
-			return ShowDialog(null);
+			return ShowDialog(this.owner);
 		}
 
 		public DialogResult ShowDialog(IWin32Window ownerWin32) {
-			Form		previous;
 			Rectangle	area;
 			bool		confined;
 			IntPtr		capture_window;
@@ -1218,8 +1266,6 @@ namespace System.Windows.Forms {
 				XplatUI.UngrabWindow(capture_window);
 			}
 
-			previous = Form.ActiveForm;
-
 #if not
 			// Commented out; we instead let the Visible=true inside the runloop create the control
 			// otherwise setting DialogResult inside any of the events that are triggered by the
@@ -1233,9 +1279,9 @@ namespace System.Windows.Forms {
 
 			Application.RunLoop(true, new ApplicationContext(this));
 
-			if (previous != null) {
+			if (owner != null) {
 				// Cannot use Activate(), it has a check for the current active window...
-				XplatUI.Activate(previous.window.Handle);
+				XplatUI.Activate(owner.window.Handle);
 			}
 
 			if (DialogResult != DialogResult.None) {
@@ -1275,7 +1321,6 @@ namespace System.Windows.Forms {
 				return;
 
 			if (Environment.GetEnvironmentVariable ("MONO_MWF_SCALING") == "disable"){
-				Console.WriteLine ("Not scaling");
 				return;
 			}
 			
@@ -1360,14 +1405,16 @@ namespace System.Windows.Forms {
 		protected override void CreateHandle() {
 			base.CreateHandle ();
 
-			if (XplatUI.SupportsTransparency()) {
+			UpdateBounds();
+
+			if ((XplatUI.SupportsTransparency() & TransparencySupport.Set) != 0) {
 				if (allow_transparency) {
 					XplatUI.SetWindowTransparency(Handle, Opacity, TransparencyKey);
 				}
 			}
 
 			XplatUI.SetWindowMinMax(window.Handle, maximized_bounds, minimum_size, maximum_size);
-			if (icon != null) {
+			if ((FormBorderStyle != FormBorderStyle.FixedDialog) && (icon != null)) {
 				XplatUI.SetIcon(window.Handle, icon);
 			}
 
@@ -1375,12 +1422,13 @@ namespace System.Windows.Forms {
 				XplatUI.SetTopmost(window.Handle, owner.window.Handle, true);
 			}
 
-			if (owned_forms.Count > 0) {
-				for (int i = 0; i < owned_forms.Count; i++) {
-					if (owned_forms[i].IsHandleCreated) {
-						XplatUI.SetTopmost(owned_forms[i].window.Handle, window.Handle, true);
-					}
-				}
+			for (int i = 0; i < owned_forms.Count; i++) {
+				if (owned_forms[i].IsHandleCreated)
+					XplatUI.SetTopmost(owned_forms[i].window.Handle, window.Handle, true);
+			}
+			
+			if (window_manager != null && window_state != FormWindowState.Normal) {
+				window_manager.SetWindowState (FormWindowState.Normal, window_state);
 			}
 
 		}
@@ -1390,12 +1438,22 @@ namespace System.Windows.Forms {
 			base.DefWndProc (ref m);
 		}
 
-		protected override void Dispose(bool disposing) {
+		protected override void Dispose(bool disposing)
+		{
+			for (int i = 0; i < owned_forms.Count; i++)
+				((Form)owned_forms[i]).Owner = null;
+
+			owned_forms.Clear ();
+			
 			base.Dispose (disposing);
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
-		protected virtual void OnActivated(EventArgs e) {
+		protected virtual void OnActivated(EventArgs e)
+		{
+			if (is_loaded)
+				SelectActiveControl ();
+
 			if (Activated != null) {
 				Activated(this, e);
 			}
@@ -1418,28 +1476,14 @@ namespace System.Windows.Forms {
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void OnCreateControl() {
 			base.OnCreateControl ();
-			if (this.ActiveControl == null) {
-				bool visible;
-
-				// This visible hack is to work around CanSelect always being false if one of the parents
-				// is not visible; and we by default create Form invisible...
-				visible = this.is_visible;
-				this.is_visible = true;
-
-				if (SelectNextControl(this, true, true, true, true) == false) {
-					Select(this);
-				}
-
-				this.is_visible = visible;
-			} else {
-				Select(ActiveControl);
-			}
 
 			if (menu != null) {
 				XplatUI.SetMenu(window.Handle, menu);
 			}
 
 			OnLoad(EventArgs.Empty);
+			
+			SelectActiveControl ();
 
 			// Send initial location
 			OnLocationChanged(EventArgs.Empty);
@@ -1511,6 +1555,7 @@ namespace System.Windows.Forms {
 						break;
 				}
 			}
+			is_loaded = true;
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
@@ -1563,6 +1608,11 @@ namespace System.Windows.Forms {
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void OnResize(EventArgs e) {
 			base.OnResize(e);
+
+			if (this.IsMdiChild && ParentForm != null) {
+				ParentForm.PerformLayout();
+				ParentForm.Size = ParentForm.Size;
+			}
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
@@ -1573,11 +1623,19 @@ namespace System.Windows.Forms {
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void OnTextChanged(EventArgs e) {
 			base.OnTextChanged (e);
+
+            if (mdi_container != null)
+                mdi_container.SetParentText(true);
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void OnVisibleChanged(EventArgs e) {
 			base.OnVisibleChanged (e);
+			
+			if (Visible) {
+				if (window_manager != null)
+					window_manager.SetWindowState (WindowState, WindowState);
+			}
 		}
 
 		protected override bool ProcessCmdKey(ref Message msg, Keys keyData) {
@@ -1601,9 +1659,17 @@ namespace System.Windows.Forms {
 
 		protected override bool ProcessDialogKey(Keys keyData) {
 			if ((keyData & Keys.Modifiers) == 0) {
-				if (keyData == Keys.Enter && accept_button != null) {
-					accept_button.PerformClick();
-					return true;
+				if (keyData == Keys.Enter) {
+					IntPtr window = XplatUI.GetFocus ();
+					Control c = Control.FromHandle (window);
+					if (c is Button && c.FindForm () == this) {
+						((Button)c).PerformClick ();
+						return true;
+					}
+					else if (accept_button != null) {
+						accept_button.PerformClick();
+						return true;
+					}
 				} else if (keyData == Keys.Escape && cancel_button != null) {
 					cancel_button.PerformClick();
 					return true;
@@ -1700,14 +1766,8 @@ namespace System.Windows.Forms {
 
 			clientsize_set = new Size(x, y);
 
-			if (!IsMdiChild) {
-				if (XplatUI.CalculateWindowRect(ref ClientRect, cp.Style, cp.ExStyle, cp.menu, out WindowRect)) {
-					SetBoundsCore(bounds.X, bounds.Y, WindowRect.Width, WindowRect.Height, BoundsSpecified.Size);
-				}
-			} else {
-				if (XplatUI.CalculateWindowRect(ref ClientRect, cp.Style, cp.ExStyle, cp.menu, out WindowRect)) {
-					SetBoundsCore(bounds.X, bounds.Y, WindowRect.Width, WindowRect.Height, BoundsSpecified.Size);
-				}
+			if (XplatUI.CalculateWindowRect(ref ClientRect, cp.Style, cp.ExStyle, cp.menu, out WindowRect)) {
+				SetBounds(bounds.X, bounds.Y, WindowRect.Width, WindowRect.Height, BoundsSpecified.Size);
 			}
 		}
 
@@ -1727,8 +1787,6 @@ namespace System.Windows.Forms {
 				return;
 			}
 
-			bool native_enabled = XplatUI.IsEnabled (Handle);
-
 			switch((Msg)m.Msg) {
 				case Msg.WM_DESTROY: {
 					base.WndProc(ref m);
@@ -1761,19 +1819,21 @@ namespace System.Windows.Forms {
 						OnClosing (args);
 						if (!args.Cancel) {
 							OnClosed (EventArgs.Empty);
-							base.Dispose();
+							closing = true;
 						}
-						return;
+						Dispose ();
 					} else {
 						OnClosing (args);
-						if (!args.Cancel) {
-							OnClosed (EventArgs.Empty);
-							closing = true;
-						} else {
+						if (args.Cancel) {
 							DialogResult = DialogResult.None;
 							closing = false;
+						} else {
+							OnClosed (EventArgs.Empty);
+							closing = true;
+							Hide ();
 						}
 					}
+
 					return;
 				}
 
@@ -1784,6 +1844,17 @@ namespace System.Windows.Forms {
 					return;
 				}
 
+#if NET_2_0
+				case Msg.WM_SYSCOMMAND: {
+					// Let *Strips know the app's title bar was clicked
+					if (XplatUI.IsEnabled (Handle))
+						ToolStripManager.FireAppClicked ();
+						
+					base.WndProc(ref m);
+					break;
+				}
+#endif
+	
 				case Msg.WM_ACTIVATE: {
 					if (m.WParam != (IntPtr)WindowActiveFlags.WA_INACTIVE) {
 						OnActivated(EventArgs.Empty);
@@ -1809,23 +1880,52 @@ namespace System.Windows.Forms {
 
 				// Menu drawing
 				case Msg.WM_NCLBUTTONDOWN: {
-					if (native_enabled && ActiveMenu != null) {
+					if (XplatUI.IsEnabled (Handle) && ActiveMenu != null) {
 						ActiveMenu.OnMouseDown(this, new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), mouse_clicks, Control.MousePosition.X, Control.MousePosition.Y, 0));
 					}
 
 					if (ActiveMaximizedMdiChild != null) {
-						ActiveMaximizedMdiChild.HandleMenuMouseDown (ActiveMenu,
+						if (ActiveMaximizedMdiChild.HandleMenuMouseDown (ActiveMenu,
 								LowOrder ((int) m.LParam.ToInt32 ()),
-								HighOrder ((int) m.LParam.ToInt32 ()));
+								HighOrder ((int) m.LParam.ToInt32 ()))) {
+							// Don't let base process this message, otherwise we won't
+							// get a WM_NCLBUTTONUP.
+							return;
+						}
 					}
 					base.WndProc(ref m);
 					return;
 				}
+				case Msg.WM_NCLBUTTONUP: {
+					if (ActiveMaximizedMdiChild != null) {
+						ActiveMaximizedMdiChild.HandleMenuMouseUp (ActiveMenu,
+								LowOrder ((int)m.LParam.ToInt32 ()),
+								HighOrder ((int)m.LParam.ToInt32 ()));
+					}
+					base.WndProc (ref m);
+					return;
+				}
 
+				case Msg.WM_NCMOUSELEAVE: {
+					if (ActiveMaximizedMdiChild != null) {
+						ActiveMaximizedMdiChild.HandleMenuMouseLeave(ActiveMenu,
+								LowOrder((int)m.LParam.ToInt32()),
+								HighOrder((int)m.LParam.ToInt32()));
+					}
+					base.WndProc(ref m);
+					return;
+				}
+				
 				case Msg.WM_NCMOUSEMOVE: {
-					if (native_enabled && ActiveMenu != null) {
+					if (XplatUI.IsEnabled (Handle) && ActiveMenu != null) {
 						ActiveMenu.OnMouseMove(this, new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), mouse_clicks, LowOrder ((int) m.LParam.ToInt32 ()), HighOrder ((int) m.LParam.ToInt32 ()), 0));
 					}
+					
+					if (ActiveMaximizedMdiChild != null) {
+						ActiveMaximizedMdiChild.HandleMenuMouseMove (ActiveMenu,
+								LowOrder ((int)m.LParam.ToInt32 ()),
+								HighOrder ((int)m.LParam.ToInt32 ()));
+					}
 					base.WndProc(ref m);
 					return;
 				}
@@ -1838,6 +1938,14 @@ namespace System.Windows.Forms {
 						pe = XplatUI.PaintEventStart(Handle, false);
 						pnt = XplatUI.GetMenuOrigin(window.Handle);
 
+						// The entire menu has to be in the clip rectangle because the 
+						// control buttons are right-aligned and otherwise they would
+						// stay painted when the window gets resized.
+						Rectangle clip = new Rectangle (pnt.X, pnt.Y, ClientSize.Width, 0);
+						clip = Rectangle.Union(clip, pe.ClipRectangle);
+						pe.SetClip(clip);
+						pe.Graphics.SetClip(clip);
+						
 						ActiveMenu.Draw (pe, new Rectangle (pnt.X, pnt.Y, ClientSize.Width, 0));
 
 						if (ActiveMaximizedMdiChild != null) {
@@ -1866,7 +1974,7 @@ namespace System.Windows.Forms {
 				}
 
 				case Msg.WM_MOUSEMOVE: {
-					if (native_enabled && active_tracker != null) {
+					if (XplatUI.IsEnabled (Handle) && active_tracker != null) {
 						MouseEventArgs args;
 
 						args = new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), 
@@ -1881,7 +1989,7 @@ namespace System.Windows.Forms {
 				case Msg.WM_LBUTTONDOWN:
 				case Msg.WM_MBUTTONDOWN:
 				case Msg.WM_RBUTTONDOWN: {					
-					if (native_enabled && active_tracker != null) {
+					if (XplatUI.IsEnabled (Handle) && active_tracker != null) {
 						MouseEventArgs args;
 
 						args = new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), 
@@ -1896,11 +2004,24 @@ namespace System.Windows.Forms {
 				case Msg.WM_LBUTTONUP:
 				case Msg.WM_MBUTTONUP:
 				case Msg.WM_RBUTTONUP: {
-					if (native_enabled && active_tracker != null) {
+					if (XplatUI.IsEnabled (Handle) && active_tracker != null) {
 						MouseEventArgs args;
-
-						args = new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), 
-							mouse_clicks, LowOrder ((int) m.LParam.ToInt32 ()), HighOrder ((int) m.LParam.ToInt32 ()), 0);
+						MouseButtons mb = FromParamToMouseButtons ((int) m.WParam.ToInt32());
+						
+						// We add in the button that was released (not sent in WParam)
+						switch((Msg)m.Msg) {
+							case Msg.WM_LBUTTONUP:
+								mb |= MouseButtons.Left;
+								break;
+							case Msg.WM_MBUTTONUP:
+								mb |= MouseButtons.Middle;
+								break;
+							case Msg.WM_RBUTTONUP:
+								mb |= MouseButtons.Right;
+								break;
+						}
+						
+						args = new MouseEventArgs (mb, mouse_clicks, LowOrder ((int) m.LParam.ToInt32 ()), HighOrder ((int) m.LParam.ToInt32 ()), 0);
 						active_tracker.OnMouseUp(new MouseEventArgs (args.Button, args.Clicks, Control.MousePosition.X, Control.MousePosition.Y, args.Delta));
 						mouse_clicks = 1;
 						return;
@@ -1936,6 +2057,27 @@ namespace System.Windows.Forms {
 					}
 					break;
 				}
+				
+#if NET_2_0
+				case Msg.WM_MOUSEACTIVATE: {
+					// Let *Strips know the form or another control has been clicked
+					if (XplatUI.IsEnabled (Handle))
+						ToolStripManager.FireAppClicked ();
+						
+					base.WndProc (ref m);
+					break;				
+				}
+				
+				case Msg.WM_ACTIVATEAPP: {
+					// Let *Strips know the app lost focus
+					if (m.WParam == (IntPtr)0) 
+						if (XplatUI.IsEnabled (Handle))
+							ToolStripManager.FireAppFocusChanged (this);
+							
+					base.WndProc (ref m);
+					break;				
+				}
+#endif
 
 				default: {
 					base.WndProc (ref m);
@@ -1991,5 +2133,36 @@ namespace System.Windows.Forms {
 			remove { base.TabIndexChanged -= value; }
 		}
 		#endregion	// Events
+
+#if NET_2_0
+		public override string Text {
+			get {
+				return base.Text;
+			}
+
+			set {
+				base.Text = value;
+			}
+		}
+
+		public new Point Location {
+			get {
+				return base.Location;
+			}
+
+			set {
+				base.Location = value;
+			}
+		}
+
+		public event FormClosingEventHandler FormClosing;
+		public event FormClosedEventHandler FormClosed;
+		
+		protected virtual void OnFormClosing (FormClosingEventArgs e)
+		{
+			if (FormClosing != null)
+				FormClosing (this, e);
+		}
+#endif
 	}
 }
