@@ -72,7 +72,7 @@ namespace System.Windows.Forms {
 
 			BinaryReader reader = new BinaryReader (decoded);
 			ushort nimages = reader.ReadUInt16 ();
-			ushort max_image = reader.ReadUInt16 ();	// cMaxImage
+			reader.ReadUInt16 ();	// cMaxImage
 			ushort grow = reader.ReadUInt16 (); // cGrow
 			ushort cx = reader.ReadUInt16 ();
 			ushort cy = reader.ReadUInt16 ();
@@ -85,14 +85,23 @@ namespace System.Windows.Forms {
 				ovls[i] = reader.ReadInt16 ();
 			}
 
+			byte [] decoded_buffer = decoded.GetBuffer ();
 			int bmp_offset = 28;
-			MemoryStream bmpms = new MemoryStream (decoded.GetBuffer (), bmp_offset, (int) decoded.Length - bmp_offset + 1);
+			// FileSize field from the bitmap file header
+			int filesize = decoded_buffer [bmp_offset + 2] + (decoded_buffer [bmp_offset + 3] << 8) +
+					(decoded_buffer [bmp_offset + 4] << 16) + (decoded_buffer [bmp_offset + 5] << 24);
+			// ImageSize field from the info header (can be 0)
+			int imagesize = decoded_buffer [bmp_offset + 34] + (decoded_buffer [bmp_offset + 35] << 8) +
+					(decoded_buffer [bmp_offset + 36] << 16) + (decoded_buffer [bmp_offset + 37] << 24);
+
+			int bmp_length = imagesize + filesize;
+			MemoryStream bmpms = new MemoryStream (decoded_buffer, bmp_offset, bmp_length);
 			Bitmap bmp = null;
 			Bitmap mask = null;
 			bmp = new Bitmap (bmpms);
-			MemoryStream mask_stream = new MemoryStream (decoded.GetBuffer (),
-							((int) bmpms.Position) + bmp_offset,
-							((int) decoded.Length) - (((int) bmpms.Position) + bmp_offset));
+			MemoryStream mask_stream = new MemoryStream (decoded_buffer,
+							bmp_offset + bmp_length,
+							(int) (decoded.Length - bmp_offset - bmp_length));
 
 			if (mask_stream.Length > 0)
 				mask = new Bitmap (mask_stream);
@@ -120,6 +129,9 @@ namespace System.Windows.Forms {
 			images = new Image [nimages];
 			image_size = new Size (cx, cy);
 			Rectangle dest_rect = new Rectangle (0, 0, cx, cy);
+			if (grow * bmp.Width > cx) // Some images store a wrong 'grow' factor
+				grow = (ushort) (bmp.Width / cx);
+
 			for (int r = 0 ; r < nimages ; r++) {
 				int col = r % grow;
 				int row = r / grow;
@@ -170,7 +182,8 @@ namespace System.Windows.Forms {
 
 			Bitmap main = new Bitmap (cols * ImageSize.Width, rows * ImageSize.Height);
 			using (Graphics g = Graphics.FromImage (main)) {
-				g.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (BackColor), 0, 0, cols * ImageSize.Width, rows * ImageSize.Height);
+				g.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (BackColor), 0, 0,
+						main.Width, main.Height);
 				for (int i = 0; i < images.Length; i++) {
 					g.DrawImage (images [i], (i % cols) * ImageSize.Width,
 							(i / cols) * ImageSize.Height);
@@ -180,34 +193,11 @@ namespace System.Windows.Forms {
 			MemoryStream tmp = new MemoryStream ();
 			main.Save (tmp, ImageFormat.Bmp);
 			tmp.WriteTo (stream);
+
+			Bitmap mask = Get1bppMask (main);
 			main.Dispose ();
 			main = null;
 
-			Bitmap mask = new Bitmap (cols * ImageSize.Width, rows * ImageSize.Height);
-			int w = images [0].Width;
-			int h = images [0].Height;
-			for (int idx = 0; idx < images.Length; idx++) {
-				Bitmap current = (Bitmap) images [idx];
-				// Hack for newly added images.
-				// Probably has to be done somewhere else.
-				Color c1 = current.GetPixel (0, 0);
-				if (c1.A != 0 && c1 == back_color)
-					current.MakeTransparent (back_color);
-				//
-				int basey = (idx / 4) * h;
-				int basex = (idx % 4) * w;
-				for (int y = 0; y < h; y++) {
-					for (int x = 0; x < w; x++) {
-						Color color = current.GetPixel (x, y);
-						if (color.A == 0) {
-							mask.SetPixel (x+basex, y+basey, Color.FromArgb (255,255,255));
-						} else {
-							mask.SetPixel (x+basex, y+basey, Color.FromArgb (0));
-						}
-					}
-				}
-			}
-
 			tmp = new MemoryStream ();
 			mask.Save (tmp, ImageFormat.Bmp);
 			tmp.WriteTo (stream);
@@ -217,6 +207,66 @@ namespace System.Windows.Forms {
 			info.AddValue ("Data", stream.ToArray (), typeof (byte []));
 		}
 
+		unsafe Bitmap Get1bppMask (Bitmap main)
+		{
+			Rectangle rect = new Rectangle (0, 0, main.Width, main.Height);
+			Bitmap result = new Bitmap (main.Width, main.Height, PixelFormat.Format1bppIndexed);
+			BitmapData dresult = result.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format1bppIndexed);
+
+			int w = images [0].Width;
+			int h = images [0].Height;
+			byte *scan = (byte *) dresult.Scan0.ToPointer ();
+			int stride = dresult.Stride;
+			Bitmap current = null;
+			for (int idx = 0; idx < images.Length; idx++) {
+				current = (Bitmap) images [idx];
+				// Hack for newly added images.
+				// Probably has to be done somewhere else.
+				Color c1 = current.GetPixel (0, 0);
+				if (c1.A != 0 && c1 == back_color)
+					current.MakeTransparent (back_color);
+				//
+			}
+
+			int yidx = 0;
+			int imgidx = 0;
+			int localy = 0;
+			int localx = 0;
+			int factor_y = 0;
+			int factor_x = 0;
+			for (int y = 0; y < main.Height; y++) {
+				if (localy == h) {
+					localy = 0;
+					factor_y += 4;
+				}
+				factor_x = 0;
+				localx = 0;
+				for (int x = 0; x < main.Width; x++) {
+					if (localx == w) {
+						localx = 0;
+						factor_x++;
+					}
+					imgidx = factor_y + factor_x;
+					if (imgidx >= images.Length)
+						break;
+					current = (Bitmap) images [imgidx];
+					Color color = current.GetPixel (localx, localy);
+					if (color.A == 0) {
+						int ptridx = yidx + (x >> 3);
+						scan [ptridx] |= (byte) (0x80 >> (x & 7));
+					}
+					localx++;
+				}
+				if (imgidx >= images.Length)
+					break;
+				yidx += stride;
+				localy++;
+			}
+			result.UnlockBits (dresult);
+
+			return result;
+		}
+
 		static MemoryStream GetDecodedStream (byte [] bytes, int offset, int size)
 		{
 			byte [] buffer = new byte [512];
