@@ -53,6 +53,8 @@ namespace System.Windows.Forms {
 		private FormPos sizing_edge;
 		internal Rectangle virtual_position;
 
+		private bool is_mouse_down_menu;
+		
 		public class TitleButton {
 			public Rectangle Rectangle;
 			public ButtonState State;
@@ -130,7 +132,7 @@ namespace System.Windows.Forms {
 			get { return TitleBarHeight - 5; }
 		}
 
-		public bool HandleMessage (ref Message m)
+		public virtual bool HandleMessage (ref Message m)
 		{
 			switch ((Msg)m.Msg) {
 
@@ -149,12 +151,16 @@ namespace System.Windows.Forms {
 			case Msg.WM_RBUTTONDOWN:
 			case Msg.WM_LBUTTONDOWN:
 				return HandleButtonDown (ref m);
-
+			case Msg.WM_PARENTNOTIFY:
+				if (Control.LowOrder(m.WParam.ToInt32()) == (int) Msg.WM_LBUTTONDOWN) 
+					Activate ();
+				break;
+				
 			case Msg.WM_NCHITTEST:
 				int x = Control.LowOrder ((int) m.LParam.ToInt32 ());
 				int y = Control.HighOrder ((int) m.LParam.ToInt32 ());
-				
-				form.PointToClient (ref x, ref y);
+
+				NCPointToClient (ref x, ref y);
 
 				FormPos pos = FormPosForCoords (x, y);
 				
@@ -197,16 +203,27 @@ namespace System.Windows.Forms {
 				}
 				return true;
 
+				// Return true from these guys, otherwise win32 will mess up z-order
 			case Msg.WM_NCLBUTTONUP:
-				return HandleNCLButtonUp (ref m);
+				HandleNCLButtonUp (ref m);
+				return true;
 
 			case Msg.WM_NCLBUTTONDOWN:
-				return HandleNCLButtonDown (ref m);
+				HandleNCLButtonDown (ref m);
+				return true;
 
+			case Msg.WM_NCMOUSEMOVE:
+				HandleNCMouseMove (ref m);
+				return true;
+				
 			case Msg.WM_NCLBUTTONDBLCLK:
 				HandleNCLButtonDblClick (ref m);
 				break;
 
+			case Msg.WM_NCMOUSELEAVE:
+				HandleNCMouseLeave (ref m);
+				break;
+			
 			case Msg.WM_MOUSE_LEAVE:
 				FormMouseLeave (ref m);
 				break;
@@ -233,9 +250,23 @@ namespace System.Windows.Forms {
 				break;
 
 			case Msg.WM_NCPAINT:
-				PaintDecorations ();
+				PaintEventArgs pe = XplatUI.PaintEventStart (form.Handle, false);
+
+				Rectangle clip;
+ 				// clip region is not correct on win32.
+				// if (m.WParam.ToInt32 () > 1) {
+				//	Region r = Region.FromHrgn (m.WParam);
+				//	RectangleF rf = r.GetBounds (pe.Graphics);
+				//	clip = new Rectangle ((int) rf.X, (int) rf.Y, (int) rf.Width, (int) rf.Height);
+				//} else {	
+				clip = new Rectangle (0, 0, form.Width, form.Height);
+				//}
+
+				ThemeEngine.Current.DrawManagedWindowDecorations (pe.Graphics, clip, this);
+				XplatUI.PaintEventEnd (form.Handle, false);
 				return true;
 			}
+
 			return false;
 		}
 
@@ -251,19 +282,79 @@ namespace System.Windows.Forms {
 			CreateButtons ();
 		}
 
-		public void HandleMenuMouseDown (MainMenu menu, int x, int y)
+		public bool HandleMenuMouseDown (MainMenu menu, int x, int y)
 		{
 			Point pt = MenuTracker.ScreenToMenu (menu, new Point (x, y));
 
+			is_mouse_down_menu = false;
 			foreach (TitleButton button in title_buttons) {
-				if (button != null && button.Rectangle.Contains (pt)) {
-					button.Clicked (this, EventArgs.Empty);
-					button.State = ButtonState.Pushed;
-					return;
+				if (button != null) {
+					if (button.Rectangle.Contains (pt)) {
+						button.State = ButtonState.Pushed;
+						is_mouse_down_menu = true;
+					} else {
+						button.State = ButtonState.Normal;
+					}
 				}
 			}
+			XplatUI.InvalidateNC (menu.GetForm().Handle);
+			return is_mouse_down_menu;
 		}
 
+		public void HandleMenuMouseUp (MainMenu menu, int x, int y)
+		{
+			Point pt = MenuTracker.ScreenToMenu (menu, new Point(x, y));
+
+			foreach (TitleButton button in title_buttons) {
+				if (button != null) {
+					if (button.Rectangle.Contains (pt)) {
+						button.Clicked (this, EventArgs.Empty);
+						button.State = ButtonState.Pushed;
+					} else {
+						button.State = ButtonState.Normal;
+					}
+				}
+			}
+			XplatUI.InvalidateNC (menu.GetForm().Handle);
+			is_mouse_down_menu = false;
+			return;
+		}
+		
+		public void HandleMenuMouseLeave(MainMenu menu, int x, int y)
+		{
+			foreach (TitleButton button in title_buttons) {
+				if (button != null) {
+					button.State = ButtonState.Normal;
+				}
+			}
+			XplatUI.InvalidateNC (menu.GetForm().Handle);
+			return;
+		}
+		
+		public void HandleMenuMouseMove (MainMenu menu, int x, int y)
+		{
+			Point pt = MenuTracker.ScreenToMenu (menu, new Point (x, y));
+			
+			if (!is_mouse_down_menu)
+				return;
+				
+			bool any_change = false;
+			foreach (TitleButton button in title_buttons) {
+				if (button == null) 
+					continue;
+				
+				if (button.Rectangle.Contains(pt)) {
+					any_change |= button.State != ButtonState.Pushed;
+					button.State = ButtonState.Pushed;
+				} else {
+					any_change |= button.State != ButtonState.Normal;
+					button.State = ButtonState.Normal;
+				}
+			}
+			if (any_change)
+				XplatUI.InvalidateNC (menu.GetForm().Handle);
+		}
+		
 		public virtual void SetWindowState (FormWindowState old_state, FormWindowState window_state)
 		{
 		}
@@ -303,7 +394,7 @@ namespace System.Windows.Forms {
 		protected virtual void Activate ()
 		{
 			// Hack to get a paint
-			NativeWindow.WndProc (form.Handle, Msg.WM_NCPAINT, IntPtr.Zero, IntPtr.Zero);
+			//NativeWindow.WndProc (form.Handle, Msg.WM_NCPAINT, IntPtr.Zero, IntPtr.Zero);
 			form.Refresh ();
 		}
 
@@ -312,26 +403,16 @@ namespace System.Windows.Forms {
 			return true;
 		}
 
+
 		private void FormSizeChangedHandler (object sender, EventArgs e)
 		{
 			ThemeEngine.Current.ManagedWindowSetButtonLocations (this);
-
-			PaintDecorations ();
-		}
-
-		public void PaintDecorations ()
-		{
-			PaintEventArgs pe = XplatUI.PaintEventStart (form.Handle, false);
-			ThemeEngine.Current.DrawManagedWindowDecorations (pe.Graphics, pe.ClipRectangle, this);
-
-			if (IsMaximized) {
-				MainMenu menu = form.ActiveMenu;
-				DrawMaximizedButtons (menu, pe);
-				if (menu != null)
-					menu.Draw (pe);
-			}
-
-			XplatUI.PaintEventEnd (form.Handle, false);
+			Message m = new Message ();
+			m.Msg = (int) Msg.WM_NCPAINT;
+			m.HWnd = form.Handle;
+			m.LParam = IntPtr.Zero;
+			m.WParam = new IntPtr (1);
+			XplatUI.SendMessage (ref m);
 		}
 
 		protected void CreateButtons ()
@@ -373,6 +454,40 @@ namespace System.Windows.Forms {
 			return false;
 		}
 
+		protected virtual bool HandleNCMouseLeave (ref Message m)
+		{
+			int x = Control.LowOrder ((int)m.LParam.ToInt32 ());
+			int y = Control.HighOrder ((int)m.LParam.ToInt32 ());
+
+			NCPointToClient (ref x, ref y);
+			FormPos pos = FormPosForCoords (x, y);
+
+			if (pos != FormPos.TitleBar) {
+				HandleTitleBarLeave (x, y);
+				return true;
+			}
+
+			return true;
+		}
+		
+		protected virtual bool HandleNCMouseMove (ref Message m)
+		{
+
+			int x = Control.LowOrder((int)m.LParam.ToInt32( ));
+			int y = Control.HighOrder((int)m.LParam.ToInt32( ));
+
+			NCPointToClient (ref x, ref y);
+			FormPos pos = FormPosForCoords (x, y);
+
+			if (pos == FormPos.TitleBar) {
+				HandleTitleBarMove (x, y);
+				return true;
+			}
+
+			return true;
+			
+		}
+		
 		protected virtual bool HandleNCLButtonDown (ref Message m)
 		{
 			Activate ();
@@ -380,14 +495,14 @@ namespace System.Windows.Forms {
 			start = Cursor.Position;
 			virtual_position = form.Bounds;
 
+			is_mouse_down_menu = false;
+			
 			int x = Control.LowOrder ((int) m.LParam.ToInt32 ());
 			int y = Control.HighOrder ((int) m.LParam.ToInt32 ());
-			Console.WriteLine ("MOUSE DOWN:   ({0}, {1})", x, y);
 			
-			form.PointToClient (ref x, ref y);
 			// Need to adjust because we are in NC land
+			NCPointToClient (ref x, ref y);
 			FormPos pos = FormPosForCoords (x, y);
-
 			
 			if (pos == FormPos.TitleBar) {
 				HandleTitleBarDown (x, y);
@@ -411,12 +526,56 @@ namespace System.Windows.Forms {
 		protected virtual void HandleNCLButtonDblClick (ref Message m)
 		{
 		}
-
+		
+		protected virtual void HandleTitleBarLeave (int x, int y)
+		{
+			is_mouse_down_menu = false;
+		}
+		
+		protected virtual void HandleTitleBarMove (int x, int y)
+		{
+			if (!is_mouse_down_menu)
+				return;
+			
+			bool any_change = false;
+			foreach (TitleButton button in title_buttons) {
+				if (button == null)
+					continue;
+				
+				if (button.Rectangle.Contains (x, y)) {
+					any_change |= button.State != ButtonState.Pushed;
+					button.State = ButtonState.Pushed;
+				} else {
+					any_change |= button.State != ButtonState.Normal;
+					button.State = ButtonState.Normal;
+				}
+			}
+			if (any_change)
+				XplatUI.InvalidateNC (form.Handle);
+		}
+		
+		protected virtual void HandleTitleBarUp (int x, int y)
+		{
+			is_mouse_down_menu = false;
+			
+			foreach (TitleButton button in title_buttons) {
+				if (button == null)
+					continue;
+					
+				button.State = ButtonState.Normal;
+				if (button.Rectangle.Contains (x, y)) {
+					button.Clicked (this, EventArgs.Empty);
+				} 
+			}
+		}
+		
 		protected virtual void HandleTitleBarDown (int x, int y)
 		{
 			foreach (TitleButton button in title_buttons) {
 				if (button != null && button.Rectangle.Contains (x, y)) {
 					button.State = ButtonState.Pushed;
+					XplatUI.InvalidateNC (form.Handle);
+					is_mouse_down_menu = true;
 					return;
 				}
 			}
@@ -519,7 +678,7 @@ namespace System.Windows.Forms {
 				switch (form.FormBorderStyle) {
 				case FormBorderStyle.Sizable:
 				case FormBorderStyle.SizableToolWindow:
-					return true;
+					return (form.window_state != FormWindowState.Minimized);
 				default:
 					return false;
 				}
@@ -588,20 +747,21 @@ namespace System.Windows.Forms {
 
 				form.Capture = false;
 				state = State.Idle;
+				if (form.MdiContainer != null)
+					form.MdiContainer.SizeScrollBars();
 			}
 				
 			int x = Control.LowOrder ((int) m.LParam.ToInt32 ());
 			int y = Control.HighOrder ((int) m.LParam.ToInt32 ());
 
-			form.PointToClient (ref x, ref y);
+			NCPointToClient (ref x, ref y);
+			FormPos pos = FormPosForCoords (x, y);
 
-			foreach (TitleButton button in title_buttons) {
-				if (button != null && button.Rectangle.Contains (x, y)) {
-					button.Clicked (this, EventArgs.Empty);
-					return true;
-				}
+			if (pos == FormPos.TitleBar) {
+				HandleTitleBarUp (x, y);
+				return true;
 			}
-
+			
 			return true;
 		}
 		
@@ -611,9 +771,8 @@ namespace System.Windows.Forms {
 				return;
 
 			dc.FillRectangle (SystemBrushes.Control, button.Rectangle);
-
 			ControlPaint.DrawCaptionButton (dc, button.Rectangle,
-					button.Caption, ButtonState.Normal);
+					button.Caption, button.State);
 		}
 
 		public virtual void DrawMaximizedButtons (object sender, PaintEventArgs pe)
@@ -664,6 +823,12 @@ namespace System.Windows.Forms {
 		{
 		}
 
+		protected virtual void NCPointToClient(ref int x, ref int y) {
+			form.PointToClient(ref x, ref y);
+			y += TitleBarHeight;
+			y += ThemeEngine.Current.ManagedWindowBorderWidth (this);
+		}
+
 		protected FormPos FormPosForCoords (int x, int y)
 		{
 			int bw = ThemeEngine.Current.ManagedWindowBorderWidth (this);
