@@ -26,7 +26,9 @@
 
 namespace System.Windows.Forms
 {
+#if !NET_2_0
 	[Serializable]
+#endif
 	public enum ItemBoundsPortion
 	{
 		Entire = 0,
