@@ -29,33 +29,11 @@ using System.Resources;
 
 namespace System.Windows.Forms {
 
-	internal class _KeyboardLayout {
-		public string Comment;
-		public int CodePage;
-		public uint [][] Key;
-		public short [] Scan;
-		public VirtualKeys [] VKey;
-
-		public _KeyboardLayout (string comment, int code_page, uint [][] key, short [] scan, VirtualKeys [] vkey)
-		{
-			Comment = comment;
-			CodePage = code_page;
-			Key = key;
-			Scan = scan;
-			VKey = vkey;
-		}
-	}
-
 	internal class KeyboardLayouts {
 
 		private KeyboardLayout [] keyboard_layouts;
 		public int [][] vkey_table;
 		public short [][] scan_table;
-		
-		public KeyboardLayouts ()
-		{
-			LoadLayouts ();
-		}
 
 		public void LoadLayouts ()
 		{
@@ -68,7 +46,11 @@ namespace System.Windows.Forms {
 		}
 
 		public KeyboardLayout [] Layouts {
-			get { return keyboard_layouts; }
+			get {
+				if (keyboard_layouts == null)
+					LoadLayouts ();
+				return keyboard_layouts;
+			}
 		}
 	}
 
