@@ -30,6 +30,8 @@
 
 namespace System.Windows.Forms
 {
+#if !NET_2_0
 	[Serializable]
+#endif
 	public delegate void LabelEditEventHandler (object sender, LabelEditEventArgs e);
 }
