@@ -158,11 +158,15 @@ namespace System.Windows.Forms
 				if (value.Start <0 || value.Length < -1)
 					throw new ArgumentException ();
 
-				if (!value.IsEmpty)
+				if (Links.IsDefault)
+					Links.Clear ();
+
+				if (!value.IsEmpty) {
 					Links.Add (value.Start, value.Length);
 
-				link_area = value;
-				Invalidate ();
+					link_area = value;
+					Invalidate ();
+				}
 			}
 		}
 				
@@ -507,7 +511,7 @@ namespace System.Windows.Forms
 			for (int i = 0; i < t.Length; i ++) {
 				if (t[i] == '\n') {
 					if (i != 0) {
-						Piece p = new Piece (start + ps, i + 1, t.Substring (ps, i+1-ps), link);
+						Piece p = new Piece (start + ps, i + 1 - ps, t.Substring (ps, i+1-ps), link);
 						rv.Add (p);
 					}
 					ps = i+1;
@@ -523,7 +527,7 @@ namespace System.Windows.Forms
 
 		private void CreateLinkPieces ()
 		{
-			if (Links.Count == 0 || Text.Length == 0) {
+			if (Text.Length == 0) {
 				SetStyle (ControlStyles.Selectable, false);
 				return;
 			}
@@ -578,13 +582,9 @@ namespace System.Windows.Forms
 			string_format.FormatFlags = StringFormatFlags.NoClip;
 			string_format.SetMeasurableCharacterRanges (ranges);
 
-			SizeF size = DeviceContext.MeasureString (Text, link_font, ClientRectangle.Location, string_format);
-
-			RectangleF layout_rect = new RectangleF (0.0f, 0.0f, size.Width, size.Height);			
-
 			Region[] regions = DeviceContext.MeasureCharacterRanges (Text,
 										 link_font,
-										 layout_rect,
+										 ClientRectangle,
 										 string_format);
 
 			for (int i = 0; i < pieces.Length; i ++)
@@ -860,16 +860,24 @@ namespace System.Windows.Forms
 			{
 				return Add (start, length, null);
 			}
-
+			
+			internal bool IsDefault {
+				get {
+					return (Count == 1
+						&& this[0].Start == 0
+						&& this[0].length == -1);
+				}
+			}
 
 			public Link Add (int start, int length, object o)
 			{
 				Link link = new Link (owner);
 				int idx;
 
-				if (Count == 1 && this[0].Start == 0
-					&& this[0].length == -1) {
-					Clear ();
+				/* remove the default 0,-1 link */
+				if (IsDefault) {
+					/* don't call Clear() here to save the additional CreateLinkPieces */
+					collection.Clear ();
 				}
 
 				link.Length = length;
