@@ -416,6 +416,18 @@ namespace System.Windows.Forms
 				else if (SelectionMode == SelectionMode.One)
 					UnSelectItem (selected_index, true);
 
+    				if (value < top_index)
+    				{
+    					top_index = value;
+    					UpdateTopItem ();
+    				} else {
+    					int rows = items_area.Height / ItemHeight;
+    					if (value >= (top_index + rows))
+    					{
+    						top_index = value - rows + 1;
+    						UpdateTopItem ();
+    					}
+    				}
     				SelectItem (value);
     				selected_index = value;
     				FocusedItem = value;
@@ -1475,9 +1487,9 @@ namespace System.Windows.Forms
 		private void OnMouseDownLB (object sender, MouseEventArgs e)
     		{
     			int index = IndexAtClientPoint (e.X, e.Y);
-    			
-    			if (index == -1)
-				return;
+    			    				
+			if (index == -1)
+				return;			
 
 			switch (SelectionMode) {
 			case SelectionMode.One:
@@ -1554,6 +1566,11 @@ namespace System.Windows.Forms
 
 		private void OnMouseUpLB (object sender, MouseEventArgs e)
     		{
+			if (e.Clicks > 1)
+				OnDoubleClick (EventArgs.Empty);
+			else if (e.Clicks == 1)
+				OnClick (EventArgs.Empty);
+			
 			if (!button_pressed)
 				return;
 
@@ -1766,18 +1783,19 @@ namespace System.Windows.Forms
 		internal virtual void CollectionChanged ()
 		{
 			if (sorted) 
-				Sort ();				
-
+				Sort ();
+			
+			if (Items.Count == 0) {
+				selected_index = -1;
+				focused_item = -1;
+				top_index = 0;
+			}
+			
 			if (!IsHandleCreated || suspend_layout)
 				return;
 
 			Layout ();
 
-			if (Items.Count == 0) {
-				selected_index = -1;
-				focused_item = -1;
-			}
-
 			base.Refresh ();
 		}
 
@@ -1786,7 +1804,7 @@ namespace System.Windows.Forms
 			if (requested_height == -1)
 				return;
 
-			SetBoundsCore (0, 0, 0, requested_height, BoundsSpecified.Height);
+			SetBounds(0, 0, 0, requested_height, BoundsSpecified.Height);
 		}
 
 		private void UpdateScrollBars ()
@@ -1840,7 +1858,7 @@ namespace System.Windows.Forms
 		/* Determines if the vertical scrollbar has to be displyed */
 		private bool UpdateVerticalScrollBar ()
 		{
-			if (MultiColumn) {
+			if (MultiColumn || Items.Count == 0) {
 				vscrollbar.Visible = false;
 				return false;
 			}
