@@ -48,7 +48,11 @@ namespace System.Windows.Forms
 			value_member = new BindingMemberInfo (string.Empty);
 			display_member = string.Empty;
 			data_manager = null;
-			SetStyle (ControlStyles.StandardClick | ControlStyles.UserPaint, false);
+			SetStyle (ControlStyles.StandardClick | ControlStyles.UserPaint
+#if NET_2_0
+				| ControlStyles.UseTextForAccessibility
+#endif
+				, false);
 		}
 
 		#region Events
@@ -81,8 +85,8 @@ namespace System.Windows.Forms
 				if (value == null)
 					display_member = String.Empty;
 				else if (!(value is IList || value is IListSource))
-					throw new ArgumentException ("Complex DataBinding accepts as a data source " +
-							"either an IList or an IListSource");
+					throw new Exception ("Complex DataBinding accepts as a data source " +
+							     "either an IList or an IListSource");
 
 				data_source = value;
 				ConnectToDataSource ();
@@ -131,15 +135,13 @@ namespace System.Windows.Forms
 				return fil;
 			}
 			set {
-				if (value == null) {
-					throw new InvalidOperationException("ListControl.SelectedValue cannot be set to null");
-				}
+				if (value == null)
+					return;
 
 				if (value is string) {
 					string valueString = value as string;
-					if (valueString == String.Empty) {
-						throw new InvalidOperationException("ListControl.SelectedValue cannot be set to empty string");
-					}
+					if (valueString == String.Empty)
+						return;
 				}
 
 				if (data_manager != null) {
