@@ -147,17 +147,8 @@ namespace System.Windows.Forms
 				is_checked = value;
 
 				if (owner != null) {
-					if (is_checked) {
-						if (owner.CheckedItems.Contains (this) == false) {
-							owner.CheckedItems.list.Add (this);
-							owner.CheckedIndices.list.Add (this.Index);
-						}
-					}
-					else {
-						owner.CheckedItems.list.Remove (this);
-						owner.CheckedIndices.list.Remove (this.Index);
-					}
-					
+					// force re-population of list
+					owner.CheckedItems.Reset ();
 					Layout ();
 				}			
 				Invalidate ();
@@ -174,6 +165,7 @@ namespace System.Windows.Forms
 
 				is_focused = value; 
 
+				Invalidate ();
 				if (owner != null)
 					Layout ();
 				Invalidate ();
@@ -272,11 +264,16 @@ namespace System.Windows.Forms
 				if (selected == value)
 					return;
 
-				selected = value;
-
 				if (owner != null) {
-					owner.UpdateSelection (this);
+					if (value && !owner.MultiSelect)
+						owner.SelectedItems.Clear ();
+					selected = value;
+					// force re-population of list
+					owner.SelectedItems.Reset ();
 					Layout ();
+					owner.OnSelectedIndexChanged ();
+				} else {
+					selected = value;
 				}
 				Invalidate ();
 			}
@@ -342,6 +339,9 @@ namespace System.Windows.Forms
 		#region Public Instance Methods
 		public void BeginEdit ()
 		{
+			if (owner != null && owner.LabelEdit) {
+				owner.item_control.BeginEdit (this);
+			}
 			// FIXME: TODO
 			// if (owner != null && owner.LabelEdit 
 			//    && owner.Activation == ItemActivation.Standard)
@@ -357,6 +357,7 @@ namespace System.Windows.Forms
 			clone.is_checked = this.is_checked;
 			clone.is_focused = this.is_focused;
 			clone.selected = this.selected;
+			clone.font = this.font;
 			clone.state_image_index = this.state_image_index;
 			clone.sub_items = new ListViewSubItemCollection (this);
 			
@@ -552,6 +553,18 @@ namespace System.Windows.Forms
 			case View.LargeIcon:
 				label_rect = icon_rect = Rectangle.Empty;
 
+				SizeF sz = owner.DeviceContext.MeasureString (Text, Font);
+				if ((int) sz.Width > text_size.Width) {
+					if (Focused) {
+						int text_width = text_size.Width;
+						StringFormat format = new StringFormat ();
+						format.Alignment = StringAlignment.Center;
+						sz = owner.DeviceContext.MeasureString (Text, Font, text_width, format);
+						text_size.Height = (int) sz.Height;
+					} else
+						text_size.Height = 2 * (int) sz.Height;
+				}
+
 				if (owner.LargeImageList != null) {
 					icon_rect.Width = owner.LargeImageList.ImageSize.Width;
 					icon_rect.Height = owner.LargeImageList.ImageSize.Height;
@@ -562,7 +575,6 @@ namespace System.Windows.Forms
 				else
 					checkbox_rect.Y = icon_rect.Height - checkbox_rect.Height;
 
-
 				if (text_size.Width <= icon_rect.Width) {
 			 		icon_rect.X = checkbox_rect.Width + 1;
 					label_rect.X = icon_rect.X + (icon_rect.Width - text_size.Width) / 2;
