@@ -28,6 +28,7 @@
 using System;
 using System.Drawing;
 using System.Drawing.Drawing2D;
+using System.Runtime.InteropServices;
 
 namespace System.Windows.Forms {
 
@@ -40,12 +41,13 @@ namespace System.Windows.Forms {
 		private MenuItem icon_menu;
 		private ContextMenu icon_popup_menu;
 		private FormWindowState prev_window_state;
+		internal bool was_minimized;
+		
 		private PaintEventHandler draw_maximized_buttons;
 		internal EventHandler form_closed_handler;
 		
 		private MdiClient mdi_container;
 		private Rectangle prev_virtual_position;
-
 		private Rectangle prev_bounds;
 
 		internal Rectangle IconicBounds;
@@ -59,10 +61,33 @@ namespace System.Windows.Forms {
 			form.GotFocus += new EventHandler (FormGotFocus);
 			form_closed_handler = new EventHandler (FormClosed);
 			form.Closed += form_closed_handler;
+			form.TextChanged += new EventHandler (FormTextChangedHandler);
+			form.SizeChanged += new EventHandler (FormSizeChangedHandler);
+			form.LocationChanged += new EventHandler (FormLocationChangedHandler);
 			draw_maximized_buttons = new PaintEventHandler (DrawMaximizedButtons);
 			CreateIconMenus ();
 		}
 
+		private void FormTextChangedHandler (object sender, EventArgs e)
+		{
+			mdi_container.SetParentText (false);
+		}
+
+		private void FormLocationChangedHandler (object sender, EventArgs e)
+		{
+			if (form.window_state == FormWindowState.Minimized)
+				IconicBounds = form.Bounds;
+			XplatUI.RequestNCRecalc (mdi_container.Handle);
+			form.MdiParent.MdiContainer.SizeScrollBars ();
+		}
+
+		private void FormSizeChangedHandler (object sender, EventArgs e)
+		{
+			XplatUI.RequestNCRecalc (form.MdiParent.MdiContainer.Handle);
+			XplatUI.RequestNCRecalc (form.Handle);
+			form.MdiParent.MdiContainer.SizeScrollBars ();
+		}
+	
 		public MainMenu MergedMenu {
 			get {
 				if (merged_menu == null)
@@ -103,8 +128,14 @@ namespace System.Windows.Forms {
 				MainMenu clone = (MainMenu) parent.Menu.CloneMenu ();
 				res.MergeMenu (clone);
 			}
-
-			res.MenuItems.Add (new MenuItem ()); // Dummy item to get the menu height correct
+			
+			if (form.Menu != null) {
+				MainMenu clone = (MainMenu) form.Menu.CloneMenu ();
+				res.MergeMenu (clone);
+			}
+			
+			if (res.MenuItems.Count == 0)
+				res.MenuItems.Add (new MenuItem ()); // Dummy item to get the menu height correct
 			
 			res.SetForm (parent);
 			return res;
@@ -118,6 +149,7 @@ namespace System.Windows.Forms {
 			icon_menu.OwnerDraw = true;
 			icon_menu.MeasureItem += new MeasureItemEventHandler (MeasureIconMenuItem);
 			icon_menu.DrawItem += new DrawItemEventHandler (DrawIconMenuItem);
+			icon_menu.Click += new EventHandler (ClickIconMenuItem);
 
 			MenuItem restore = new MenuItem ("Restore", new EventHandler (RestoreItemHandler));
 			MenuItem move = new MenuItem ("Move", new EventHandler (MoveItemHandler));
@@ -133,6 +165,24 @@ namespace System.Windows.Forms {
 									maximize, close, next });
 		}
 
+		private void ClickIconMenuItem(object sender, EventArgs e)
+		{
+			ShowPopup ();
+		}
+		
+		private void ShowPopup ()
+		{
+			icon_popup_menu.MenuItems[0].Enabled = form.window_state != FormWindowState.Normal;    // restore
+			icon_popup_menu.MenuItems[1].Enabled = form.window_state != FormWindowState.Maximized; // move
+			icon_popup_menu.MenuItems[2].Enabled = form.window_state != FormWindowState.Maximized; // size
+			icon_popup_menu.MenuItems[3].Enabled = form.window_state != FormWindowState.Minimized; // minimize
+			icon_popup_menu.MenuItems[4].Enabled = form.window_state != FormWindowState.Maximized; // maximize
+			icon_popup_menu.MenuItems[5].Enabled = true;  // close
+			icon_popup_menu.MenuItems[6].Enabled = true;  // next
+			
+			icon_popup_menu.Show(form, Point.Empty);
+		}
+		
 		private void RestoreItemHandler (object sender, EventArgs e)
 		{
 			form.WindowState = FormWindowState.Normal;
@@ -213,40 +263,38 @@ namespace System.Windows.Forms {
 
 		public override void SetWindowState (FormWindowState old_state, FormWindowState window_state)
 		{
+			if (this.mdi_container.SetWindowStates (this))
+				return;
+			
+			if (prev_window_state == window_state)
+				return;
+			
+			if (prev_window_state == FormWindowState.Normal)
+				prev_bounds = form.Bounds;
+			
 			switch (window_state) {
 			case FormWindowState.Minimized:
 				CreateButtons ();
 				maximize_button.Caption = CaptionButton.Maximize;
 				minimize_button.Caption = CaptionButton.Restore;
-				prev_window_state = old_state;
-				prev_bounds = form.Bounds;
+				prev_window_state = FormWindowState.Minimized;
 				mdi_container.ArrangeIconicWindows ();
-
 				MaximizedMenu.Paint -= draw_maximized_buttons;
 				break;
 			case FormWindowState.Maximized:
 				CreateButtons ();
 				maximize_button.Caption = CaptionButton.Restore;
 				minimize_button.Caption = CaptionButton.Minimize;
-				prev_window_state = old_state;
-				prev_bounds = form.Bounds;
+				prev_window_state = FormWindowState.Maximized;
 				SizeMaximized ();
-
 				MaximizedMenu.Paint += draw_maximized_buttons;
 				break;
 			case FormWindowState.Normal:
 				CreateButtons ();
-				if (prev_window_state == FormWindowState.Maximized) {
-					form.WindowState = FormWindowState.Maximized;
-					break;
-				} else if (prev_window_state == FormWindowState.Minimized) {
-					form.WindowState = FormWindowState.Minimized;
-					break;
-				}
 				maximize_button.Caption = CaptionButton.Maximize;
 				minimize_button.Caption = CaptionButton.Minimize;
-				prev_window_state = form.WindowState;
 				form.Bounds = prev_bounds;
+				prev_window_state =FormWindowState.Normal;
 
 				MaximizedMenu.Paint -= draw_maximized_buttons;
 				break;
@@ -254,11 +302,13 @@ namespace System.Windows.Forms {
 
 			form.ResetCursor ();
 			XplatUI.RequestNCRecalc (mdi_container.Parent.Handle);
+			XplatUI.RequestNCRecalc (form.Handle);
+			mdi_container.SizeScrollBars ();
 		}
 
 		internal void SizeMaximized ()
 		{
-			Rectangle pb = mdi_container.Bounds;
+			Rectangle pb = mdi_container.ClientRectangle;
 			int bw = ThemeEngine.Current.ManagedWindowBorderWidth (this);
 			form.Bounds = new Rectangle (pb.Left - bw,
 					pb.Top - TitleBarHeight - bw,
@@ -276,8 +326,7 @@ namespace System.Windows.Forms {
 			y += TitleBarHeight;
 			FormPos pos = FormPosForCoords (x, y);
 
-			Console.WriteLine ("POS:  {0}", pos);
-			if (pos != FormPos.TitleBar)
+			if (pos != FormPos.TitleBar && pos != FormPos.Top)
 				return;
 
 			form.WindowState = FormWindowState.Maximized;
@@ -286,8 +335,9 @@ namespace System.Windows.Forms {
 		private void FormClosed (object sender, EventArgs e)
 		{
 			mdi_container.CloseChildForm (form);
-			if (form.WindowState == FormWindowState.Maximized)
-				XplatUI.RequestNCRecalc (mdi_container.Parent.Handle);
+			XplatUI.RequestNCRecalc (mdi_container.Parent.Handle);
+			mdi_container.SizeScrollBars ();
+			mdi_container.SetParentText (false);
 		}
 
 		/*
@@ -309,7 +359,7 @@ namespace System.Windows.Forms {
 			Point pnt =  XplatUI.GetMenuOrigin (mdi_container.ParentForm.Handle);
 			int bw = ThemeEngine.Current.ManagedWindowBorderWidth (this);
 
-			close_button.Rectangle = new Rectangle (form.ClientRectangle.Right - bw - bs.Width - 2,
+			close_button.Rectangle = new Rectangle (mdi_container.ParentForm.ClientRectangle.Right - bw - bs.Width - 2,
 					pnt.Y + 2, bs.Width, bs.Height);
 
 			maximize_button.Rectangle = new Rectangle (close_button.Rectangle.Left - 2 - bs.Width,
@@ -327,6 +377,21 @@ namespace System.Windows.Forms {
 			close_button.Rectangle.Y -= pnt.Y;
 		}
 
+		protected override void HandleTitleBarUp (int x, int y)
+		{
+			if (form.Icon != null) {
+				int bw = ThemeEngine.Current.ManagedWindowBorderWidth (this);
+				Rectangle icon = new Rectangle (bw + 3,
+						bw + 2, IconWidth, IconWidth);
+				if (icon.Contains (x, y)) {
+					ShowPopup ();
+					return;
+				}
+			}
+
+			base.HandleTitleBarUp (x, y);
+		}
+
 		protected override void HandleTitleBarDown (int x, int y)
 		{
 			if (form.Icon != null) {
@@ -334,14 +399,12 @@ namespace System.Windows.Forms {
 				Rectangle icon = new Rectangle (bw + 3,
 						bw + 2, IconWidth, IconWidth);
 				if (icon.Contains (x, y)) {
-					icon_popup_menu.Show (form, Point.Empty);
 					return;
 				}
 			}
 
 			base.HandleTitleBarDown (x, y);
 		}
-
 		protected override bool ShouldRemoveWindowManager (FormBorderStyle style)
 		{
 			return false;
@@ -358,6 +421,59 @@ namespace System.Windows.Forms {
 			start = Cursor.Position;
 		}
 
+		public override bool HandleMessage (ref Message m)
+		{
+			switch ((Msg)m.Msg) {
+			case Msg.WM_NCMOUSEMOVE:
+				XplatUI.RequestAdditionalWM_NCMessages (form.Handle, true, true);
+				break;
+				
+			case Msg.WM_NCCALCSIZE:
+				XplatUIWin32.NCCALCSIZE_PARAMS ncp;
+
+				if (m.WParam == (IntPtr)1) {
+					ncp = (XplatUIWin32.NCCALCSIZE_PARAMS)Marshal.PtrToStructure (m.LParam,
+							typeof(XplatUIWin32.NCCALCSIZE_PARAMS));
+
+					int bw = ThemeEngine.Current.ManagedWindowBorderWidth (this);
+					if (!IsMaximized)
+						bw++;
+
+					if (HasBorders) {
+						ncp.rgrc1.top += TitleBarHeight + bw;
+						if (!IsMaximized) {
+							ncp.rgrc1.left += bw;
+							ncp.rgrc1.bottom -= bw;
+							ncp.rgrc1.right -= bw;
+						} else {
+							ncp.rgrc1.left += 1;
+							ncp.rgrc1.bottom -= 2; 
+							ncp.rgrc1.right -= 3;
+						}
+					}
+					
+					if (ncp.rgrc1.bottom < ncp.rgrc1.top)
+						ncp.rgrc1.bottom = ncp.rgrc1.top + 1;
+						
+					Marshal.StructureToPtr (ncp, m.LParam, true);
+				}
+
+				return false;
+
+			//case Msg.WM_NCPAINT:
+			//        PaintEventArgs pe = XplatUI.PaintEventStart (form.Handle, false);
+
+			//        Rectangle clip;
+			//        clip = new Rectangle (0, 0, form.Width, form.Height);
+
+			//        ThemeEngine.Current.DrawManagedWindowDecorations (pe.Graphics, clip, this);
+			//        XplatUI.PaintEventEnd (form.Handle, false);
+			//        return true;
+			}
+		
+			return base.HandleMessage (ref m);
+		}
+	
 		protected override void DrawVirtualPosition (Rectangle virtual_position)
 		{
 			ClearVirtualPosition ();
@@ -389,7 +505,10 @@ namespace System.Windows.Forms {
 
 		protected override void Activate ()
 		{
-			mdi_container.ActivateChild (form);
+			if (mdi_container.ActiveMdiChild != form) {
+				mdi_container.ActivateChild (form);
+				mdi_container.SetWindowStates (this);
+			}
 			base.Activate ();
 		}
 
