@@ -37,6 +37,7 @@ namespace System.Windows.Forms {
 	internal class MenuTracker {
 
 		internal bool active;
+		internal bool popdown_menu;
 		public Menu CurrentMenu;
 		public Menu TopMenu;
 		Form grab_control;
@@ -64,7 +65,7 @@ namespace System.Windows.Forms {
 		KeyNavState keynav_state = KeyNavState.Idle;
 
 		public bool Navigating {
-			get { return keynav_state != KeyNavState.Idle; }
+			get { return keynav_state != KeyNavState.Idle || active; }
 		}
 
 		internal static Point ScreenToMenu (Menu menu, Point pnt)		
@@ -82,6 +83,7 @@ namespace System.Windows.Forms {
 			keynav_state = KeyNavState.Idle;
 			if (TopMenu is ContextMenu) {
 				PopUpWindow puw = TopMenu.Wnd as PopUpWindow;
+				DeselectItem (TopMenu.SelectedItem);
 				puw.HideWindow ();
 			} else {
 #if false
@@ -131,24 +133,25 @@ namespace System.Windows.Forms {
 
 			MenuItem item = GetItemAtXY (args.X, args.Y);
 
-			if (item != null && !item.Enabled)
-				return;
-
-			if (item == null || (active && item.IsPopup && (item.Parent is MainMenu))) {
+			if (item == null) {
 				Deactivate ();
 				return;
 			}
 
+			if (item != null && !item.Enabled)
+				return;
+
+			popdown_menu = active;
+			
 			SelectItem (item.Parent, item, item.IsPopup);
 			if (item.IsPopup) {
 				active = true;
-				grab_control.ActiveTracker = this;
 				item.Parent.InvalidateItem (item);
-			} else if (item.Parent is MainMenu)
-				active = false;
-			else
-				Deactivate ();
-			item.PerformClick ();			
+			} else if (item.Parent is MainMenu){
+				active = true;
+				item.Parent.InvalidateItem (item);
+			}
+			grab_control.ActiveTracker = this;
 		}
 
 		public void OnMotion (MouseEventArgs args)
@@ -169,13 +172,30 @@ namespace System.Windows.Forms {
 
 		public void OnMouseUp (MouseEventArgs args)
 		{
-			// Doing nothing (yet)
+			if ((args.Button & MouseButtons.Left) == 0)
+				return;
+			
+			MenuItem item = GetItemAtXY (args.X, args.Y);
+
+			if (item == null) {
+				/* the user released the mouse button outside the menu */
+				Deactivate ();
+			}
+			else {
+				/* hide the menu when popdown */
+				if (popdown_menu || !item.IsPopup)
+					Deactivate ();
+				
+				/* Perform click when is not a popup */
+				if (!item.IsPopup)
+					item.PerformClick ();
+			}
 		}
 
 		void MoveSelection (MenuItem item)
 		{
 			if (item == null) {
-				if (CurrentMenu.SelectedItem.IsPopup)
+				if (CurrentMenu.SelectedItem.IsPopup || (keynav_state == KeyNavState.Navigating))
 					return;
 				MenuItem old_item = CurrentMenu.SelectedItem;
 				if (CurrentMenu != TopMenu)
@@ -186,6 +206,7 @@ namespace System.Windows.Forms {
 					DeselectItem (CurrentMenu.SelectedItem);
 				CurrentMenu = item.Parent;
 				SelectItem (CurrentMenu, item, active && item.IsPopup);
+				keynav_state = KeyNavState.Idle;
 			}
 		}
 
@@ -263,6 +284,9 @@ namespace System.Windows.Forms {
 		//	or a sub-popup menu has to be shown
 		void ExecFocusedItem (Menu menu, MenuItem item)
 		{
+			if (item == null)
+				return;
+
 			if (!item.Enabled)
 			 	return;			 
 			 	
@@ -498,10 +522,13 @@ namespace System.Windows.Forms {
 			if (item == null)
 				return false;
 
+			active = true;
+			grab_control.ActiveTracker = this;
+			
 			SelectItem (CurrentMenu, item, true);
 			if (item.IsPopup) {
-				SelectItem (item, item.MenuItems [0], false);
 				CurrentMenu = item;
+				SelectItem (item, item.MenuItems [0], false);
 			}
 			return true;
 		}
@@ -636,9 +663,9 @@ namespace System.Windows.Forms {
 				return true;
 			} else if ((keyData & Keys.Alt) == Keys.Alt)
 				return ProcessMnemonic (msg, keyData);
-			else if ((Msg)msg.Msg == Msg.WM_SYSKEYUP || keynav_state == KeyNavState.Idle)
+			else if ((Msg)msg.Msg == Msg.WM_SYSKEYUP)
 				return false;
-			else if (!active && !Navigating)
+			else if (!Navigating)
 				return false;
 
 			switch (keyData) {
@@ -676,6 +703,7 @@ namespace System.Windows.Forms {
 			SetStyle (ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
 			SetStyle (ControlStyles.ResizeRedraw | ControlStyles.Opaque, true);
 			is_visible = false;
+			Hwnd.ObjectFromHandle (this.Handle).no_activate = true;
 		}
 
 		protected override CreateParams CreateParams
@@ -737,3 +765,4 @@ namespace System.Windows.Forms {
 	}
 }
 
+
