@@ -125,7 +125,7 @@ namespace System.Windows.Forms
 
 					cp = base.CreateParams;
 
-					cp.Style = (int)(WindowStyles.WS_DLGFRAME | WindowStyles.WS_POPUP | WindowStyles.WS_CLIPCHILDREN | WindowStyles.WS_CLIPSIBLINGS | WindowStyles.WS_CAPTION);
+					cp.Style |= (int)(WindowStyles.WS_DLGFRAME | WindowStyles.WS_POPUP | WindowStyles.WS_CLIPCHILDREN | WindowStyles.WS_CLIPSIBLINGS | WindowStyles.WS_CAPTION);
 					cp.ExStyle = (int)(WindowExStyles.WS_EX_TOPMOST);
 					if (!is_enabled) {
 						cp.Style |= (int)(WindowStyles.WS_DISABLED);
@@ -253,6 +253,16 @@ namespace System.Windows.Forms
 				}
 
 			}
+
+			protected override bool ProcessDialogKey(Keys keyData) {
+				if (keyData == Keys.Escape) {
+					this.CancelClick(this, null);
+					return true;
+				}
+
+				return base.ProcessDialogKey (keyData);
+			}
+
 			#endregion	// MessageBoxForm Methods
 
 			#region Functions for Adding buttons
