@@ -61,7 +61,10 @@ namespace System.Windows.Forms {
 		ContextMenu		menu;
 		NumericUpDown	year_updown;
 		Timer		 	timer;
-		
+		Timer			updown_timer;
+		bool			showing_context_menu;
+		bool			updown_has_focus;
+
 		// internal variables used
 		internal DateTime 		current_month;			// the month that is being displayed in top left corner of the grid		
 		internal DateTimePicker	owner;					// used if this control is popped up
@@ -80,6 +83,8 @@ namespace System.Windows.Forms {
 		internal DateTime		first_select_start_date;
 		internal int			last_clicked_calendar_index;
 		internal Rectangle		last_clicked_calendar_rect;
+		internal Font 			bold_font;			// Cache the font in FontStyle.Bold
+		internal StringFormat		centered_format;		// Cache centered string format
 		private Point			month_title_click_location;
 		// this is used to see which item was actually clicked on in the beginning
 		// so that we know which item to fire on timer
@@ -135,6 +140,10 @@ namespace System.Windows.Forms {
 			title_fore_color = ThemeEngine.Current.ColorActiveCaptionText;
 			today_date_set = false;
 			trailing_fore_color = Color.Gray;
+			bold_font = new Font (Font, Font.Style | FontStyle.Bold);
+			centered_format = new StringFormat ();
+			centered_format.LineAlignment = StringAlignment.Center;
+			centered_format.Alignment = StringAlignment.Center;
 			
 			// initialise the arraylest for bolded dates
 			added_bolded_dates = new ArrayList ();
@@ -168,7 +177,7 @@ namespace System.Windows.Forms {
 
 			
 			// event handlers
-//			LostFocus += new EventHandler (LostFocusHandler);
+			LostFocus += new EventHandler (LostFocusHandler);
 			timer.Tick += new EventHandler (TimerHandler);
 			MouseMove += new MouseEventHandler (MouseMoveHandler);
 			MouseDown += new MouseEventHandler (MouseDownHandler);
@@ -610,8 +619,8 @@ namespace System.Windows.Forms {
 				int row_count = 7;		// not including the today date
 
 				// set the date_cell_size and the title_size
-				date_cell_size = new Size ((int) Math.Ceiling (2.5 * multiplier), (int) Math.Ceiling (1.5 * multiplier));
-				title_size = new Size ((date_cell_size.Width * column_count), 3 * multiplier);
+				date_cell_size = new Size ((int) Math.Ceiling (1.8 * multiplier), multiplier);
+				title_size = new Size ((date_cell_size.Width * column_count), 2 * multiplier);
 
 				return new Size (column_count * date_cell_size.Width, row_count * date_cell_size.Height + title_size.Height);
 			}
@@ -890,6 +899,8 @@ namespace System.Windows.Forms {
 			year_updown.Visible = false;
 			this.Controls.AddImplicit (year_updown);
 			year_updown.ValueChanged += new EventHandler(UpDownYearChangedHandler);
+			year_updown.GotFocus += new EventHandler(UpDownYearGotFocusHandler);
+			year_updown.LostFocus += new EventHandler(UpDownYearLostFocusHandler);
 		}
 
 		// not sure why this needs to be overriden
@@ -897,8 +908,18 @@ namespace System.Windows.Forms {
 			base.Dispose (disposing);
 		}
 
-		// not sure why this needs to be overriden
+		// Handle arrow keys
 		protected override bool IsInputKey (Keys keyData) {
+			switch (keyData) {
+				case Keys.Up:
+				case Keys.Down:
+				case Keys.Right:
+				case Keys.Left:
+					return true;
+				default:
+					break;
+			}
+
 			return base.IsInputKey (keyData);
 		}
 
@@ -923,6 +944,10 @@ namespace System.Windows.Forms {
 		}
 
 		protected override void OnFontChanged (EventArgs e) {
+			// Update size based on new font's space requirements
+                        Size = new Size (CalendarDimensions.Width * SingleMonthSize.Width,
+                                        CalendarDimensions.Height * SingleMonthSize.Height);
+			bold_font = new Font (Font, Font.Style | FontStyle.Bold);
 			base.OnFontChanged (e);
 		}
 
@@ -1364,7 +1389,7 @@ namespace System.Windows.Forms {
 		// convert a Day enum into a DayOfWeek enum
 		internal DayOfWeek GetDayOfWeek (Day day) {
 			if (day == Day.Default) {
-				return DayOfWeek.Sunday;
+				return Threading.Thread.CurrentThread.CurrentUICulture.DateTimeFormat.FirstDayOfWeek;
 			} else {
 				return (DayOfWeek) DayOfWeek.Parse (typeof (DayOfWeek), day.ToString ());
 			}
@@ -1598,17 +1623,62 @@ namespace System.Windows.Forms {
 			this.is_next_clicked = false;
 			this.is_date_clicked = false;
 		}
-		
-//		// need when in windowed mode
-//		private void LostFocusHandler (object sender, EventArgs e) 
-//		{
-//			if (this.owner != null) {
-//				if (this.Visible) {
-//					this.owner.HideMonthCalendar ();
-//				}
-//			}
-//		}
-		
+
+		// need when in windowed mode
+		private void LostFocusHandler(object sender, EventArgs e)
+		{
+			StartHideTimer ();
+		}
+
+		// needed when in windowed mode to close the calendar if no 
+		// part of it has focus.
+		private void UpDownTimerTick(object sender, EventArgs e)
+		{
+			HideMonthCalendarIfWindowed ();
+			if (updown_timer != null)
+			{
+				updown_timer.Dispose();
+				updown_timer = null;
+			}
+		}
+
+		// Needed when in windowed mode.
+		private void UpDownYearLostFocusHandler(object sender, EventArgs e)
+		{
+			updown_has_focus = false;
+			StartHideTimer ();
+		}
+
+		// Needed when in windowed mode.
+		private void UpDownYearGotFocusHandler(object sender, EventArgs e)
+		{
+			updown_has_focus = true;
+		}
+
+		// Needed when in windowed mode.
+		private void StartHideTimer()
+		{
+			if (updown_timer == null) {
+				updown_timer = new Timer ();
+				updown_timer.Interval = 50;
+				updown_timer.Tick += new EventHandler (UpDownTimerTick);
+				updown_timer.Enabled = true;
+			}
+		}
+
+		// needed when in windowed mode.
+		private void HideMonthCalendarIfWindowed()
+		{
+			if (this.owner != null && this.Visible) {
+				if (updown_has_focus)  
+					return; 
+				if (showing_context_menu)  
+					return; 
+
+				this.owner.HideMonthCalendar ();
+			}
+		}
+
 		// occurs when mouse moves around control, used for selection
 		private void MouseMoveHandler (object sender, MouseEventArgs e) {
 			HitTestInfo hti = this.HitTest (e.X, e.Y);
@@ -1693,7 +1763,9 @@ namespace System.Windows.Forms {
 					break;
 				case HitArea.TitleMonth:
 					month_title_click_location = hti.Point;
+					showing_context_menu = true;
 					menu.Show (this, hti.Point);		
+					showing_context_menu = false;
 					break;
 				case HitArea.TitleYear:
 					// place the numeric up down
@@ -1744,7 +1816,7 @@ namespace System.Windows.Forms {
 							DateTime date = GetFirstDateInMonth (this.SelectionStart);
 							this.SetSelectionRange (date, date);
 						}
-						this.OnDateSelected (new DateRangeEventArgs (SelectionStart, SelectionEnd));
+						this.OnDateChanged (new DateRangeEventArgs (SelectionStart, SelectionEnd));
 						break;
 					case Keys.End:
 						// set the date to the last of the month
@@ -1758,7 +1830,7 @@ namespace System.Windows.Forms {
 							DateTime date = GetLastDateInMonth (this.SelectionStart);
 							this.SetSelectionRange (date, date);
 						}
-						this.OnDateSelected (new DateRangeEventArgs (SelectionStart, SelectionEnd));
+						this.OnDateChanged (new DateRangeEventArgs (SelectionStart, SelectionEnd));
 						break;
 					case Keys.PageUp:
 						// set the date to the last of the month
@@ -1768,7 +1840,7 @@ namespace System.Windows.Forms {
 							DateTime date = this.SelectionStart.AddMonths (-1);
 							this.SetSelectionRange (date, date);
 						}
-						this.OnDateSelected (new DateRangeEventArgs (SelectionStart, SelectionEnd));
+						this.OnDateChanged (new DateRangeEventArgs (SelectionStart, SelectionEnd));
 						break;
 					case Keys.PageDown:
 						// set the date to the last of the month
@@ -1778,7 +1850,7 @@ namespace System.Windows.Forms {
 							DateTime date = this.SelectionStart.AddMonths (1);
 							this.SetSelectionRange (date, date);
 						}
-						this.OnDateSelected (new DateRangeEventArgs (SelectionStart, SelectionEnd));
+						this.OnDateChanged (new DateRangeEventArgs (SelectionStart, SelectionEnd));
 						break;
 					case Keys.Up:
 						// set the back 1 week
@@ -1788,7 +1860,7 @@ namespace System.Windows.Forms {
 							DateTime date = this.SelectionStart.AddDays (-7);
 							this.SetSelectionRange (date, date);
 						}
-						this.OnDateSelected (new DateRangeEventArgs (SelectionStart, SelectionEnd));
+						this.OnDateChanged (new DateRangeEventArgs (SelectionStart, SelectionEnd));
 						break;
 					case Keys.Down:
 						// set the date forward 1 week
@@ -1798,7 +1870,7 @@ namespace System.Windows.Forms {
 							DateTime date = this.SelectionStart.AddDays (7);
 							this.SetSelectionRange (date, date);
 						}
-						this.OnDateSelected (new DateRangeEventArgs (SelectionStart, SelectionEnd));					
+						this.OnDateChanged (new DateRangeEventArgs (SelectionStart, SelectionEnd));					
 						break;
 					case Keys.Left:
 						// move one left
@@ -1808,7 +1880,7 @@ namespace System.Windows.Forms {
 							DateTime date = this.SelectionStart.AddDays (-1);
 							this.SetSelectionRange (date, date);
 						}
-						this.OnDateSelected (new DateRangeEventArgs (SelectionStart, SelectionEnd));
+						this.OnDateChanged (new DateRangeEventArgs (SelectionStart, SelectionEnd));
 						break;
 					case Keys.Right:
 						// move one left
@@ -1818,7 +1890,7 @@ namespace System.Windows.Forms {
 							DateTime date = this.SelectionStart.AddDays (1);
 							this.SetSelectionRange (date, date);
 						}
-						this.OnDateSelected (new DateRangeEventArgs (SelectionStart, SelectionEnd));
+						this.OnDateChanged (new DateRangeEventArgs (SelectionStart, SelectionEnd));
 						break;
 					default:
 						break;
