@@ -58,7 +58,11 @@ namespace System.Windows.Forms {
 			redraw = true;
 			recalc = true;
 			allow_drop = false;
+#if NET_2_0
+			SetStyle (ControlStyles.OptimizedDoubleBuffer, true);
+#else
 			SetStyle (ControlStyles.DoubleBuffer, true);
+#endif
 			SetStyle (ControlStyles.Opaque, false);
 			SetStyle (ControlStyles.Selectable, false);
 			SetStyle (ControlStyles.SupportsTransparentBackColor, true);
@@ -339,7 +343,7 @@ namespace System.Windows.Forms {
 
 		#region Public Instance Methods
 		public override string ToString() {
-			return base.ToString ();
+			return String.Format("{0}, SizeMode: {1}", base.ToString (), SizeMode);
 		}
 		#endregion
 
