@@ -82,7 +82,7 @@ namespace System.Windows.Forms
 
 		public override void Reset ()
 		{
-			current_settings = new PrinterSettings ();
+			current_settings = null;
 			AllowPrintToFile = true;
 			AllowSelection = false;
 			AllowSomePages = false;
@@ -430,6 +430,11 @@ namespace System.Windows.Forms
 			label.AutoSize = true;
 			label.Location = new Point (20, 100);
 			group_box_prn.Controls.Add (label);
+			
+			label_where = new Label ();
+			label_where.AutoSize = true;
+			label_where.Location = new Point (80, 100);
+			group_box_prn.Controls.Add (label_where);
 
 			label = new Label ();
 			label.Text = "Comment:";
@@ -440,6 +445,7 @@ namespace System.Windows.Forms
 			label_comment = new Label ();
 			label_comment.AutoSize = true;
 			label_comment.Location = new Point (80, 120);
+			group_box_prn.Controls.Add (label_comment);
 
 			radio_all = new RadioButton ();
 			radio_all.TabIndex = 21;
@@ -591,8 +597,10 @@ namespace System.Windows.Forms
 				label_where.Text = port;
 				label_comment.Text = comment;
 
+				accept_button.Enabled = true;
 			}
 			catch  {
+				accept_button.Enabled = false;
 			}
 		}
 	}
