@@ -36,7 +36,7 @@ using System.Reflection;
 
 namespace System.Windows.Forms {
 	[DefaultProperty("Document")]
-	[Designer("System.ComponentModel.Design.ComponentDesigner, System.Design, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+	[Designer("System.ComponentModel.Design.ComponentDesigner, " + Consts.AssemblySystem_Design)]
 	[DesignTimeVisible(true)]
 	[ToolboxItem(true)]
 	public class PrintPreviewDialog : Form {
@@ -530,9 +530,6 @@ namespace System.Windows.Forms {
 			get { return print_preview.Document; }
 			set {
 				print_preview.Document = value;
-				print_preview.GeneratePreview ();
-				pageUpDown.Minimum = print_preview.page_infos.Length > 0 ? 1 : 0;
-				pageUpDown.Maximum = print_preview.page_infos.Length;
 			}
 		}
 
@@ -882,7 +879,12 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		[MonoTODO("Throw InvalidPrinterException")]
 		protected override void CreateHandle() {
+
+//			if (this.Document != null && !this.Document.PrinterSettings.IsValid) {
+//				throw new InvalidPrinterException(this.Document.PrinterSettings);
+//			}
 			base.CreateHandle ();
 		}
 
