@@ -174,7 +174,11 @@ namespace System.Windows.Forms
 			SetStyle (ControlStyles.UserPaint | 
 				ControlStyles.Selectable | 
 				ControlStyles.ResizeRedraw | 
-				ControlStyles.Opaque, false);
+				ControlStyles.Opaque
+#if NET_2_0
+				| ControlStyles.UseTextForAccessibility
+#endif
+				, false);
 		}
 		#endregion	// Public Constructors
 
@@ -403,8 +407,8 @@ namespace System.Windows.Forms
 		{
 			return string.Format ("{0}, Minimum: {1}, Maximum: {2}, Value: {3}",
 				GetType().FullName.ToString (),
-				Maximum.ToString (),
 				Minimum.ToString (),
+				Maximum.ToString (),
 				Value.ToString () );
 		}
 
