@@ -29,7 +29,9 @@ using System.Runtime.InteropServices;
 
 namespace System.Windows.Forms
 {
+#if !NET_2_0
 	[Serializable]
+#endif
 	[ComVisible(true)]
 	public enum PropertySort
 	{
