@@ -83,6 +83,10 @@ namespace System.Windows.Forms {
 			ForeColor = ThemeEngine.Current.ColorWindowText;
 			base.HScrolled += new EventHandler(RichTextBox_HScrolled);
 			base.VScrolled += new EventHandler(RichTextBox_VScrolled);
+
+#if NET_2_0
+			SetStyle (ControlStyles.StandardDoubleClick, false);
+#endif
 		}
 		#endregion	// Public Constructors
 
@@ -312,7 +316,7 @@ namespace System.Windows.Forms {
 				LineTag		tag;
 
 				if (document.selection_visible) {
-					document.ReplaceSelection("");
+					document.ReplaceSelection("", false);
 				}
 
 				sel_start = document.LineTagToCharIndex(document.selection_start.line, document.selection_start.pos);
@@ -528,7 +532,7 @@ namespace System.Windows.Forms {
 
 				document.UpdateView(document.selection_start.line, 0);
 				document.AlignCaret();
-				
+
 			}
 		}
 
@@ -909,7 +913,7 @@ namespace System.Windows.Forms {
 		}
 
 		public void Paste(DataFormats.Format clipFormat) {
-			base.Paste(clipFormat, false);
+			base.Paste(Clipboard.GetDataObject(), clipFormat, false);
 		}
 
 		[MonoTODO()]
@@ -1163,7 +1167,14 @@ namespace System.Windows.Forms {
 		#endregion	// Events
 
 		#region Private Methods
+
+		internal override void SelectWord ()
+		{
+			document.ExpandSelection(CaretSelection.Word, false);
+		}
+
 		private void HandleControl(RTF.RTF rtf) {
+//			Console.WriteLine ("HANDLING MAJOR:  {0}      MINOR:  {1}", rtf.Major, rtf.Minor);
 			switch(rtf.Major) {
 				case RTF.Major.Unicode: {
 					switch(rtf.Minor) {
@@ -1182,7 +1193,7 @@ namespace System.Windows.Forms {
 				}
 
 				case RTF.Major.Destination: {
-					Console.Write("[Got Destination control {0}]", rtf.Minor);
+//					Console.Write("[Got Destination control {0}]", rtf.Minor);
 					rtf.SkipGroup();
 					break;
 				}
@@ -1306,30 +1317,36 @@ namespace System.Windows.Forms {
 				}
 
 				case Minor.Tab: {
-					Console.Write("\t");
+					rtf_line.Append ("\t");
+//					FlushText (rtf, false);
 					break;
 				}
 
+				case Minor.NoReqHyphen:
 				case Minor.NoBrkHyphen: {
-					Console.Write("-");
+					rtf_line.Append ("-");
+//					FlushText (rtf, false);
 					break;
 				}
 
 				case Minor.Bullet: {
-					Console.Write("*");
+					Console.WriteLine("*");
 					break;
 				}
 
+			case Minor.WidowCtrl:
+				break;
+
 				case Minor.EmDash: {
-					Console.Write("\u2014");
+				rtf_line.Append ("\u2014");
 					break;
 				}
 
 				case Minor.EnDash: {
-					Console.Write("\u2013");
+					rtf_line.Append ("\u2013");
 					break;
 				}
-
+/*
 				case Minor.LQuote: {
 					Console.Write("\u2018");
 					break;
@@ -1349,9 +1366,10 @@ namespace System.Windows.Forms {
 					Console.Write("\u201D");
 					break;
 				}
-
+*/
 				default: {
-					rtf.SkipGroup();
+//					Console.WriteLine ("skipped special char:   {0}", rtf.Minor);
+//					rtf.SkipGroup();
 					break;
 				}
 			}
