@@ -328,6 +328,7 @@ namespace System.Windows.Forms {
 
 			set {
 				auto_scroll_min_size = value;
+				AutoScroll = true;
 			}
 		}
 
@@ -369,35 +370,41 @@ namespace System.Windows.Forms {
 
 		public override Rectangle DisplayRectangle {
 			get {
-				int		width;
-				int		height;
+				if (auto_scroll) {
+					int		width;
+					int		height;
 
-				if (!auto_scroll) {
-					return base.DisplayRectangle;
-				}
-
-				if (canvas_size.Width <= base.DisplayRectangle.Width) {
-					width = base.DisplayRectangle.Width;
-					if (vscroll_visible) {
-						width -= vscrollbar.Width;
+					if (canvas_size.Width <= base.DisplayRectangle.Width) {
+						width = base.DisplayRectangle.Width;
+						if (vscroll_visible) {
+							width -= vscrollbar.Width;
+						}
+					} else {
+						width = canvas_size.Width;
 					}
-				} else {
-					width = canvas_size.Width;
-				}
 
-				if (canvas_size.Height <= base.DisplayRectangle.Height) {
-					height = base.DisplayRectangle.Height;
-					if (hscroll_visible) {
-						height -= hscrollbar.Height;
+					if (canvas_size.Height <= base.DisplayRectangle.Height) {
+						height = base.DisplayRectangle.Height;
+						if (hscroll_visible) {
+							height -= hscrollbar.Height;
+						}
+					} else {
+						height = canvas_size.Height;
 					}
-				} else {
-					height = canvas_size.Height;
+
+					display_rectangle.X = -scroll_position.X;
+					display_rectangle.Y = -scroll_position.Y;
+					display_rectangle.Width = Math.Max(auto_scroll_min_size.Width, width);
+					display_rectangle.Height = Math.Max(auto_scroll_min_size.Height, height);
+				}
+				else {
+					display_rectangle = base.DisplayRectangle;
 				}
 
-				display_rectangle.X = -scroll_position.X + dock_padding.Left;
-				display_rectangle.Y = -scroll_position.Y + dock_padding.Top;
-				display_rectangle.Width = Math.Max(auto_scroll_min_size.Width, width) - dock_padding.Left - dock_padding.Right;
-				display_rectangle.Height = Math.Max(auto_scroll_min_size.Height, height) - dock_padding.Top - dock_padding.Bottom;
+				display_rectangle.X += dock_padding.Left;
+				display_rectangle.Y += dock_padding.Top;
+				display_rectangle.Width -= dock_padding.Left + dock_padding.Right;
+				display_rectangle.Height -= dock_padding.Top + dock_padding.Bottom;
 
 				return display_rectangle;
 			}
@@ -474,7 +481,11 @@ namespace System.Windows.Forms {
 			x += scroll_position.X;
 			y += scroll_position.Y;
 
-			// FIXME - Bail if the control is already visible?
+			// Don't scroll if already visible
+			if ((activeControl.Left >= scroll_position.X) && (activeControl.Left < (scroll_position.X + client_size.Width)) &&
+			    (activeControl.Top >= scroll_position.Y) && (activeControl.Top < (scroll_position.Y + client_size.Height))) {
+				return;
+			}
 
 			// try to center
 			corner_x = Math.Max(0, x + activeControl.Width / 2 - client_size.Width / 2);
@@ -589,16 +600,6 @@ namespace System.Windows.Forms {
 		#endregion	// Protected Instance Methods
 
 		#region Internal & Private Methods
-		private Size Canvas {
-			get {
-				if (!canvas_size.IsEmpty) {
-					return canvas_size;
-				}
-				CalculateCanvasSize();
-				return canvas_size;
-			}
-		}
-
 		private void CalculateCanvasSize() {
 			Control		child;
 			int		num_of_children;
@@ -632,8 +633,8 @@ namespace System.Windows.Forms {
 
 				switch(child.Dock) {
 					case DockStyle.Left: {
-						if ((child_controls[i].Right + extra_width) > width) {
-							width = child_controls[i].Right + extra_width;
+						if ((child.Right + extra_width) > width) {
+							width = child.Right + extra_width;
 						}
 						continue;
 					}
@@ -657,8 +658,8 @@ namespace System.Windows.Forms {
 						anchor = child.Anchor;
 
 						if (((anchor & AnchorStyles.Left) != 0) && ((anchor & AnchorStyles.Right) == 0)) {
-							if ((child_controls[i].Right + extra_width) > width) {
-								width = child_controls[i].Right + extra_width;
+							if ((child.Right + extra_width) > width) {
+								width = child.Right + extra_width;
 							}
 						}
 
@@ -716,6 +717,9 @@ namespace System.Windows.Forms {
 
 			} while (right_edge != prev_right_edge || bottom_edge != prev_bottom_edge);
 
+			if (right_edge < 0) right_edge = 0;
+			if (bottom_edge < 0) bottom_edge = 0;
+
 			Rectangle hscroll_bounds;
 			Rectangle vscroll_bounds;
 
@@ -806,7 +810,7 @@ namespace System.Windows.Forms {
 			// Should we call XplatUI.ScrollWindow??? If so, we need to position our windows by other means above
 			// Since we're already causing a redraw above
 			Invalidate(false);
-			ResumeLayout();
+			ResumeLayout(false);
 		}
 		#endregion	// Internal & Private Methods
 
