@@ -448,7 +448,7 @@ namespace System.Windows.Forms {
 
 			base.OnMouseMove (e);
 
-			if (!Capture  || e.Button != MouseButtons.Left) {
+			if (!Capture  || e.Button != MouseButtons.Left || affected == null) {
 				return;
 			}
 
@@ -457,11 +457,25 @@ namespace System.Windows.Forms {
 
 			// Grab our new coordinates
 			prev_split_position = split_position;
-			if (horizontal) {
-				split_position = pt.Y;
-			} else {
-				split_position = pt.X;
-			}
+
+			int candidate = horizontal ? pt.Y : pt.X;
+
+			// Enforce limit on what we send to the event
+			if (candidate < limit_min)
+				candidate = limit_min;
+			else if (candidate > limit_max)
+				candidate = limit_max;
+
+			sevent.x = pt.X;
+			sevent.y = pt.Y;
+			sevent.split_x = horizontal ? 0 : candidate;
+			sevent.split_y = horizontal ? candidate : 0;
+
+			// Fire the event
+			OnSplitterMoving(sevent);
+
+			split_position = horizontal ? sevent.split_y : sevent.split_x;
+
 			// Enforce limits
 			if (split_position < limit_min) {
 				#if Debug
@@ -480,25 +494,10 @@ namespace System.Windows.Forms {
 				// Update our handle location
 				DrawDragHandle(DrawType.Redraw);
 			}
-
-			// Prepare the event
-			if (horizontal) {
-				sevent.split_x = 0;
-				sevent.split_y = split_position;
-			} else {
-				sevent.split_x = split_position;
-				sevent.split_y = 0;
-			}
-
-			sevent.x = pt.X;
-			sevent.y = pt.Y;
-
-			// Fire the event
-			OnSplitterMoving(sevent);
 		}
 
 		protected override void OnMouseUp(MouseEventArgs e) {
-			if (!Capture || e.Button != MouseButtons.Left) {
+			if (!Capture || e.Button != MouseButtons.Left || affected == null) {
 				base.OnMouseUp (e);
 				return;
 			}
@@ -557,8 +556,16 @@ namespace System.Windows.Forms {
 		protected override void SetBoundsCore(int x, int y, int width, int height, BoundsSpecified specified) {
 			// enforce our width / height
 			if (horizontal) {
+				splitter_size = height;
+				if (splitter_size < 1) {
+					splitter_size = 3;
+				}
 				base.SetBoundsCore (x, y, width, splitter_size, specified);
 			} else {
+				splitter_size = width;
+				if (splitter_size < 1) {
+					splitter_size = 3;
+				}
 				base.SetBoundsCore (x, y, splitter_size, height, specified);
 			}
 		}
@@ -662,11 +669,11 @@ namespace System.Windows.Forms {
 			Rectangle	current;
 
 			if (horizontal) {
-				prev = new Rectangle(0, prev_split_position - click_offset + 1, Width, 0);
-				current = new Rectangle(0, split_position - click_offset + 1, Width, 0);
+				prev = new Rectangle(Location.X, prev_split_position - click_offset + 1, Width, 0);
+				current = new Rectangle(Location.X, split_position - click_offset + 1, Width, 0);
 			} else {
-				prev = new Rectangle(prev_split_position - click_offset + 1, 0, 0, Height);
-				current = new Rectangle(split_position - click_offset + 1, 0, 0, Height);
+				prev = new Rectangle(prev_split_position - click_offset + 1, Location.Y, 0, Height);
+				current = new Rectangle(split_position - click_offset + 1, Location.Y, 0, Height);
 			}
 
 			switch(type) {
@@ -686,7 +693,7 @@ namespace System.Windows.Forms {
 				}
 
 				case DrawType.Finish: {
-					XplatUI.DrawReversibleRectangle(Parent.window.Handle, prev, 3);
+					XplatUI.DrawReversibleRectangle(Parent.window.Handle, current, 3);
 					return;
 				}
 			}
