@@ -276,11 +276,11 @@ namespace System.Windows.Forms {
 		internal void UpdatePanelContents (StatusBarPanel panel)
 		{
 			if (panel.AutoSize == StatusBarPanelAutoSize.Contents) {
-				Update ();
+				Invalidate ();
 				return;
 			}
 
-			Update ();
+			Invalidate (new Rectangle (panel.X + 2, 2, panel.Width - 4, bounds.Height - 4));
 		}
 
 		internal void Update ()
@@ -353,7 +353,8 @@ namespace System.Windows.Forms {
 
 		private void Draw (Graphics dc, Rectangle clip)
 		{
-			ThemeEngine.Current.DrawStatusBar (dc, this.ClientRectangle, this);
+			ThemeEngine.Current.DrawStatusBar (dc, clip, this);
+			
 		}
 		#endregion	// Internal Methods
 
