@@ -44,7 +44,7 @@ namespace System.Windows.Forms {
 		private ImageList image_list;
 		private Size item_size = Size.Empty;
 		private Point padding;
-		private int row_count = 1;
+		private int row_count = 0;
 		private bool hottrack;
 		private TabPageCollection tab_pages;
 		private bool show_tool_tips;
@@ -205,8 +205,10 @@ namespace System.Windows.Forms {
 		public int SelectedIndex {
 			get { return selected_index; }
 			set {
-				if (selected_index == value)
+				if (selected_index == value) {
+					Invalidate(GetTabRect (selected_index));
 					return;
+				}
 				if (selected_index < -1) {
 					throw new ArgumentException ("'" + value + "' is not a valid value for 'value'. " +
 							"'value' must be greater than or equal to -1.");
@@ -225,15 +227,16 @@ namespace System.Windows.Forms {
 				}
 
 				if (-1 != value) {
-        				int le = TabPages [value].TabBounds.Right;
-                                	int re = ThemeEngine.Current.GetTabControlLeftScrollRect (this).Left;
+					int le = TabPages [value].TabBounds.Right;
+					int re = ThemeEngine.Current.GetTabControlLeftScrollRect (this).Left;
 					if (show_slider && le > re) {
-						int diff = le - re;
-						int ind = value - 1;
-						while (ind > 0 && diff > 0) {
-							diff -= TabPages [ind++].Width;
+						int i = 0;
+
+						for (i = value; i < TabPages.Count; i++) {
+							if (TabPages [i].TabBounds.Right > re)
+								break;
 						}
-						slider_pos = ind - 1;
+						slider_pos = i;
 						refresh = true;
 					}
 				}
@@ -241,7 +244,7 @@ namespace System.Windows.Forms {
 				if (selected_index != -1) {
 					if (!refresh)
 						invalid = GetTabRect (selected_index);
-					Controls [selected_index].Visible = false;
+					((TabPage) Controls [selected_index]).SetVisible (false);
 				}
 				selected_index = value;
 
@@ -252,7 +255,7 @@ namespace System.Windows.Forms {
 				if (selected_index != -1) {
 					selected = (TabPage) Controls [selected_index];
 					invalid = Rectangle.Union (invalid, GetTabRect (selected_index));
-					selected.Visible = true;
+					selected.SetVisible (true);
 				}
 
 				ResumeLayout ();
@@ -499,7 +502,6 @@ namespace System.Windows.Forms {
 			case Keys.End:
 			case Keys.Left:
 			case Keys.Right:
-			case Keys.Tab:
 				return true;
 			}
 			return base.IsInputKey (key);
@@ -551,6 +553,23 @@ namespace System.Windows.Forms {
 			Draw (pe.Graphics, pe.ClipRectangle);
 			pe.Handled = true;
 		}
+
+		internal override void OnGotFocusInternal(EventArgs e) {
+			has_focus = true;
+			if (selected_index != -1)
+				Invalidate(GetTabRect(selected_index));
+			base.OnGotFocusInternal (e);
+		}
+
+
+		internal override void OnLostFocusInternal(EventArgs e) {
+			has_focus = false;
+			if (selected_index != -1)
+				Invalidate(GetTabRect(selected_index));
+			base.OnLostFocusInternal (e);
+		}
+
+
 		#endregion	// Protected Instance Methods
 
 		#region Internal & Private Methods
@@ -677,7 +696,8 @@ namespace System.Windows.Forms {
 			int xpos = 4;
 			Size spacing = TabSpacing;
 
-			row_count = 1;
+			if (TabPages.Count > 0)
+				row_count = 1;
 			show_slider = false;
 			
 			for (int i = 0; i < TabPages.Count; i++) {
@@ -690,6 +710,10 @@ namespace System.Windows.Forms {
 					width = item_size.Width;
 				} else {
 					width = (int) DeviceContext.MeasureString (page.Text, Font).Width + (Padding.X * 2);
+					if (ImageList != null && page.ImageIndex >= 0 && page.ImageIndex < ImageList.Images.Count) {
+						width += ImageList.ImageSize.Width + 2;
+						item_size.Height = ImageList.ImageSize.Height + 3;
+					}
 				}
 
 				if (i == SelectedIndex)
@@ -808,6 +832,10 @@ namespace System.Windows.Forms {
 					width = item_size.Width;
 				} else {
 					width = (int) DeviceContext.MeasureString (page.Text, Font).Width + (Padding.X * 2);
+					if (ImageList != null && page.ImageIndex >= 0 && page.ImageIndex < ImageList.Images.Count) {
+						width += ImageList.ImageSize.Width + 2;
+						item_size.Height = ImageList.ImageSize.Height + 3;
+					}
 				}
 
 				if (width < MinimumTabWidth)
@@ -865,6 +893,10 @@ namespace System.Windows.Forms {
 					width = item_size.Width;
 				} else {					
 					width = (int) DeviceContext.MeasureString (page.Text, Font).Width + (Padding.X * 2);
+					if (ImageList != null && page.ImageIndex >= 0 && page.ImageIndex < ImageList.Images.Count) {
+						width += ImageList.ImageSize.Width + 2;
+						item_size.Height = ImageList.ImageSize.Height + 3;
+					}
 				}
 
 				if (width < MinimumTabWidth)
@@ -1039,19 +1071,20 @@ namespace System.Windows.Forms {
 
 			public override void Add (Control value)
 			{
-				if (!(value is TabPage))
+				TabPage page = value as TabPage;
+				if (page == null)
 					throw new ArgumentException ("Cannot add " +
 						value.GetType ().Name + " to TabControl. " +
 						"Only TabPages can be directly added to TabControls.");
 
-				value.Visible = false;
+				page.SetVisible (false);
 				base.Add (value);
 				if (Count == 1) {
 					owner.SelectedIndex = 0;
 				} else {
 					// Setting the selected index will calc the tab rows so
 					// we don't need to do it again
-					owner.ResizeTabPages ();
+					owner.Redraw ();
 				}
 			}
 
