@@ -60,6 +60,7 @@ namespace System.Windows.Forms {
 		internal ImplicitHScrollBar	hscroll;
 		internal ImplicitVScrollBar	vscroll;
 		internal RichTextBoxScrollBars	scrollbars;
+		internal Timer			scroll_timer;
 		internal bool			richtext;
 		internal bool			show_selection;		// set to true to always show selection, even if no focus is set
 		internal int			selection_length;	// set to the user-specified selection length, or -1 if none
@@ -121,20 +122,24 @@ namespace System.Windows.Forms {
 			hscroll.control_style &= ~ControlStyles.Selectable;
 			hscroll.Enabled = false;
 			hscroll.Visible = false;
+			hscroll.Maximum = Int32.MaxValue;
 
 			vscroll = new ImplicitVScrollBar();
 			vscroll.ValueChanged += new EventHandler(vscroll_ValueChanged);
 			vscroll.control_style &= ~ControlStyles.Selectable;
 			vscroll.Enabled = false;
 			vscroll.Visible = false;
+			vscroll.Maximum = Int32.MaxValue;
 
 			SuspendLayout ();
 			this.Controls.AddImplicit (hscroll);
 			this.Controls.AddImplicit (vscroll);
 			ResumeLayout ();
 			
-			//SetStyle(ControlStyles.ResizeRedraw, true);
 			SetStyle(ControlStyles.UserPaint | ControlStyles.StandardClick, false);
+#if NET_2_0
+			SetStyle(ControlStyles.UseTextForAccessibility, false);
+#endif
 
 			canvas_width = ClientSize.Width;
 			canvas_height = ClientSize.Height;
@@ -286,7 +291,7 @@ namespace System.Windows.Forms {
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 		[Editor("System.Windows.Forms.Design.StringArrayEditor, " + Consts.AssemblySystem_Design, typeof(System.Drawing.Design.UITypeEditor))]
 		[Localizable(true)]
-		[MWFCategory("Layout")]
+		[MWFCategory("Appearance")]
 		public string[] Lines {
 			get {
 				string[]	lines;
@@ -430,11 +435,10 @@ namespace System.Windows.Forms {
 			}
 
 			set {
-				if (!read_only) {
-					document.ReplaceSelection(CaseAdjust(value));
-					ScrollToCaret();
-					OnTextChanged(EventArgs.Empty);
-				}
+				document.ReplaceSelection(CaseAdjust(value), false);
+					
+				ScrollToCaret();
+				OnTextChanged(EventArgs.Empty);
 			}
 		}
 
@@ -490,6 +494,7 @@ namespace System.Windows.Forms {
 				} else {
 					document.SetSelectionEnd(value);
 				}
+				document.PositionCaret(document.selection_start.line, document.selection_start.pos);
 				ScrollToCaret();
 			}
 		}
@@ -509,10 +514,12 @@ namespace System.Windows.Forms {
 
 					sb = new StringBuilder();
 
+					Line line = null;
 					for (i = 1; i < document.Lines; i++) {
-						if (i > 1)
+						if (line != null && !line.soft_break)
 							sb.Append (Environment.NewLine);
-						sb.Append(document.GetLine(i).text.ToString());
+						line = document.GetLine (i);
+						sb.Append(line.text.ToString());
 					}
 					sb.Append(document.GetLine(document.Lines).text.ToString());
 					return sb.ToString();
@@ -520,6 +527,9 @@ namespace System.Windows.Forms {
 			}
 
 			set {
+				if (value == Text)
+					return;
+
 				if ((value != null) && (value != "")) {
 					Line	line;
 
@@ -535,38 +545,19 @@ namespace System.Windows.Forms {
 						}
 						this.Lines = lines;
 
-						line = document.GetLine(1);
+						document.PositionCaret (document.GetLine (1), 0);
+						document.SetSelectionToCaret (true);
 
-						if (!Focused) {
-							document.SetSelectionStart(line, 0);
-
-							line = document.GetLine(document.Lines);
-							document.SetSelectionEnd(line, line.text.Length);
-							selection_length = -1;
-							document.PositionCaret(line, line.text.Length);
-						} else {
-							document.SetSelectionStart(line, 0);
-							document.SetSelectionEnd(line, 0);
-							selection_length = -1;
-							document.PositionCaret(line, 0);
-						}
+						ScrollToCaret ();
 					} else {
 						document.Clear();
 						document.Add(1, CaseAdjust(value), alignment, Font, ThemeEngine.Current.ResPool.GetSolidBrush(ForeColor));
 						CalculateDocument();
-						line = document.GetLine(1);
 
-						if (!Focused) {
-							document.SetSelectionStart(line, 0);
-							document.SetSelectionEnd(line, value.Length);
-							selection_length = -1;
-							document.PositionCaret(line, value.Length);
-						} else {
-							document.SetSelectionStart(line, 0);
-							document.SetSelectionEnd(line, 0);
-							selection_length = -1;
-							document.PositionCaret(line, 0);
-						}
+						document.PositionCaret (document.GetLine (1), 0);
+						document.SetSelectionToCaret (true);
+
+						ScrollToCaret ();
 					}
 				} else {
 					document.Empty();
@@ -687,11 +678,11 @@ namespace System.Windows.Forms {
 				o.SetData(DataFormats.Rtf, ((RichTextBox)this).SelectedRtf);
 			}
 			Clipboard.SetDataObject(o);
-			document.ReplaceSelection("");
+			document.ReplaceSelection("", false);
 		}
 
 		public void Paste() {
-			Paste(null, false);
+			Paste(Clipboard.GetDataObject(), null, false);
 		}
 
 		public void ScrollToCaret() {
@@ -858,7 +849,7 @@ namespace System.Windows.Forms {
 
 				case Keys.V: {	// Paste (Ctrl-V)
 					if (control) {
-						return Paste(null, true);
+						return Paste(Clipboard.GetDataObject(), null, true);
 					}
 					return false;
 				}
@@ -1002,9 +993,8 @@ namespace System.Windows.Forms {
 						Line	line;
 
 						if (document.selection_visible) {
-							document.ReplaceSelection("");
+							document.ReplaceSelection("", false);
 						}
-						document.SetSelectionToCaret(true);
 
 						line = document.CaretLine;
 
@@ -1012,6 +1002,7 @@ namespace System.Windows.Forms {
 						OnTextChanged(EventArgs.Empty);
 						document.UpdateView(line, 2, 0);
 						document.MoveCaret(CaretDirection.CharForward);
+						document.SetSelectionToCaret(true);
 						CaretMoved(this, null);
 						return true;
 					}
@@ -1022,7 +1013,7 @@ namespace System.Windows.Forms {
 					if (!read_only && accepts_tab && multiline) {
 						document.InsertChar(document.CaretLine, document.CaretPosition, '\t');
 						if (document.selection_visible) {
-							document.ReplaceSelection("");
+							document.ReplaceSelection("", false);
 						}
 						document.SetSelectionToCaret(true);
 
@@ -1035,7 +1026,7 @@ namespace System.Windows.Forms {
 
 				case Keys.Insert: {
 					if (shift) {
-						Paste(null, true);
+						Paste(Clipboard.GetDataObject(), null, true);
 						return true;
 					}
 
@@ -1077,7 +1068,7 @@ namespace System.Windows.Forms {
 					}
 
 					if (document.selection_visible) {
-						document.ReplaceSelection("");
+						document.ReplaceSelection("", false);
 					} else {
 						// DeleteChar only deletes on the line, doesn't do the combine
 						if (document.CaretPosition == document.CaretLine.Text.Length) {
@@ -1137,7 +1128,7 @@ namespace System.Windows.Forms {
 
 			// delete only deletes on the line, doesn't do the combine
 			if (document.selection_visible) {
-				document.ReplaceSelection("");
+				document.ReplaceSelection("", false);
 				fire_changed = true;
 			}
 			document.SetSelectionToCaret(true);
@@ -1194,9 +1185,6 @@ namespace System.Windows.Forms {
 			}
 
 			base.SetBoundsCore (x, y, width, height, specified);
-
-			TextBoxBase_SizeChanged(this, EventArgs.Empty);
-			CalculateDocument();
 		}
 
 		protected override void WndProc(ref Message m) {
@@ -1230,7 +1218,7 @@ namespace System.Windows.Forms {
 						HandleBackspace(true);
 					} else if (ch >= 32) {
 						if (document.selection_visible) {
-							document.ReplaceSelection("");
+							document.ReplaceSelection("", false);
 						}
 
 						char c = (char)m.WParam;
@@ -1404,7 +1392,7 @@ namespace System.Windows.Forms {
 
 				interval = DateTime.Now - click_last;
 				document.PositionCaret(e.X + document.ViewPortX, e.Y + document.ViewPortY);
-				
+
 				// Handle place caret/select word/select line behaviour
 				if (e.Clicks == 1) {
 					if (SystemInformation.DoubleClickTime < interval.TotalMilliseconds) {
@@ -1413,12 +1401,14 @@ namespace System.Windows.Forms {
 						#endif
 						document.SetSelectionToCaret(true);
 						click_mode = CaretSelection.Position;
-					} else {
+					} else if (this is RichTextBox) {
 						#if DebugClick
 							Console.WriteLine("Tripple Click Selecting line");
 						#endif
 						document.ExpandSelection(CaretSelection.Line, false);
 						click_mode = CaretSelection.Line;
+					} else {
+						document.SetSelectionToCaret(true);
 					}
 				} else {
 					// We select the line if the word is already selected, and vice versa
@@ -1427,8 +1417,9 @@ namespace System.Windows.Forms {
 							document.Invalidate(document.selection_start.line, 0, document.selection_start.line, document.selection_start.line.text.Length);
 						}
 						click_mode = CaretSelection.Word;
-						document.ExpandSelection(CaretSelection.Word, false);	// Setting initial selection
-					} else {
+
+						SelectWord ();
+					} else if (this is RichTextBox) {
 						click_mode = CaretSelection.Line;
 						document.ExpandSelection(CaretSelection.Line, false);	// Setting initial selection
 					}
@@ -1447,7 +1438,7 @@ namespace System.Windows.Forms {
 				marker.height = marker.tag.height;
 
 				document.SetSelection(marker.line, marker.pos, marker.line, marker.pos);
-				Paste();
+				Paste (Clipboard.GetDataObject (true), null, true);
 
 			}
 
@@ -1495,18 +1486,20 @@ namespace System.Windows.Forms {
 
 		private void TextBoxBase_MouseUp(object sender, MouseEventArgs e) {
 			if (e.Button == MouseButtons.Left) {
-				document.PositionCaret(e.X + document.ViewPortX, e.Y + document.ViewPortY);
 				if (click_mode == CaretSelection.Position) {
 					document.SetSelectionToCaret(false);
 					document.DisplayCaret();
-				} else {
-					document.ExpandSelection(click_mode, true);
+				}
+
+				if (scroll_timer != null) {
+					scroll_timer.Enabled = false;
 				}
 				return;
 			}
 		}
 
-		private void TextBoxBase_SizeChanged(object sender, EventArgs e) {
+		private void PositionControls ()
+		{
 			if (hscroll.Visible) {
 				//vscroll.Maximum += hscroll.Height;
 				canvas_height = ClientSize.Height - hscroll.Height;
@@ -1531,6 +1524,10 @@ namespace System.Windows.Forms {
 			
 		}
 
+		private void TextBoxBase_SizeChanged(object sender, EventArgs e) {
+			CalculateDocument();
+		}
+
 		private void TextBoxBase_MouseWheel(object sender, MouseEventArgs e) {
 			Line	line;
 			int	line_no;
@@ -1539,24 +1536,52 @@ namespace System.Windows.Forms {
 				return;
 			}
 
-			if (e.Delta < 0) {
-				line_no = document.GetLineByPixel(document.ViewPortY, false).line_no + SystemInformation.MouseWheelScrollLines;
-				if (line_no > document.Lines) {
-					line_no = document.Lines;
-				}
-			} else {
-				line_no = document.GetLineByPixel(document.ViewPortY, false).line_no - SystemInformation.MouseWheelScrollLines;
-				if (line_no < 1) {
-					line_no = 1;
-				}
+			if (e.Delta < 0)
+				vscroll.Value = Math.Min (vscroll.Value + SystemInformation.MouseWheelScrollLines, vscroll.Maximum - document.ViewPortHeight + 1);
+			else
+				vscroll.Value = Math.Max (0, vscroll.Value - SystemInformation.MouseWheelScrollLines);
+		}
+
+		internal virtual void SelectWord ()
+		{
+			StringBuilder s = document.caret.line.text;
+			int start = document.caret.pos;
+			int end = document.caret.pos;
+
+			if (s.Length < 1) {
+				if (document.caret.line.line_no >= document.Lines)
+					return;
+				Line line = document.GetLine (document.caret.line.line_no + 1);
+				document.PositionCaret (line, 0);
+				return;
 			}
 
-			line = document.GetLine(line_no);
-			if (line.Y < vscroll.Maximum) {
-				vscroll.Value = line.Y;
-			} else {
-				vscroll.Value = vscroll.Maximum;
+			if (start > 0) {
+				start--;
+				end--;
 			}
+
+			// skip whitespace until we hit a word
+			while (start > 0 && s [start] == ' ')
+				start--;
+			if (start > 0) {
+				while (start > 0 && (s [start] != ' '))
+					start--;
+				if (s [start] == ' ')
+					start++;
+			}
+
+			if (s [end] == ' ') {
+				while (end < s.Length && s [end] == ' ')
+					end++;
+			} else {
+				while (end < s.Length && s [end] != ' ')
+					end++;
+				while (end < s.Length && s [end] == ' ')
+					end++;
+			}
+
+			document.SetSelection (document.caret.line, start, document.caret.line, end);
 		}
 
 		internal void CalculateDocument() {
@@ -1570,12 +1595,16 @@ namespace System.Windows.Forms {
 
 		internal void CalculateScrollBars() {
 			// FIXME - need separate calculations for center and right alignment
-			// No scrollbars for a single line
+
+			if (!multiline) {
+				PositionControls ();
+				return;
+			}
 
 			if (document.Width >= document.ViewPortWidth) {
 				hscroll.Enabled = true;
 				hscroll.Minimum = 0;
-				hscroll.LargeChange = document.ViewPortWidth;
+				hscroll.LargeChange = document.ViewPortWidth < 0 ? 0 : document.ViewPortWidth;
 				hscroll.Maximum = document.Width;
 			} else {
 				hscroll.Maximum = document.ViewPortWidth;
@@ -1585,7 +1614,7 @@ namespace System.Windows.Forms {
 			if (document.Height >= document.ViewPortHeight) {
 				vscroll.Enabled = true;
 				vscroll.Minimum = 0;
-				vscroll.LargeChange = document.ViewPortHeight;
+				vscroll.LargeChange = document.ViewPortHeight < 0 ? 0 : document.ViewPortHeight;
 				vscroll.Maximum = document.Height;
 			} else {
 				vscroll.Maximum = document.ViewPortHeight;
@@ -1593,10 +1622,6 @@ namespace System.Windows.Forms {
 			}
 
 
-			if (!multiline) {
-				return;
-			}
-
 			if (!WordWrap) {
 				if ((scrollbars & RichTextBoxScrollBars.Horizontal) != 0) {
 					if (((scrollbars & RichTextBoxScrollBars.ForcedHorizontal) != 0) || hscroll.Enabled) {
@@ -1619,7 +1644,7 @@ namespace System.Windows.Forms {
 				vscroll.Visible = false;
 			}
 
-			TextBoxBase_SizeChanged(this, EventArgs.Empty);
+			PositionControls ();
 		}
 
 		private void document_WidthChanged(object sender, EventArgs e) {
@@ -1669,12 +1694,27 @@ namespace System.Windows.Forms {
 		private void TextBoxBase_MouseMove(object sender, MouseEventArgs e) {
 			// FIXME - handle auto-scrolling if mouse is to the right/left of the window
 			if (Capture) {
+				if (!ClientRectangle.Contains (e.X, e.Y)) {
+					if (scroll_timer == null) {
+						scroll_timer = new Timer ();
+						scroll_timer.Interval = 100;
+						scroll_timer.Tick += new EventHandler (ScrollTimerTickHandler);
+					}
+
+					if (!scroll_timer.Enabled) {
+						scroll_timer.Start ();
+
+						// Force the first tick
+						ScrollTimerTickHandler (null, EventArgs.Empty);
+					}
+
+					return;
+				}
+
 				document.PositionCaret(e.X + document.ViewPortX, e.Y + document.ViewPortY);
 				if (click_mode == CaretSelection.Position) {
 					document.SetSelectionToCaret(false);
 					document.DisplayCaret();
-				} else {
-					document.ExpandSelection(click_mode, true);
 				}
 			}
 		}
@@ -1694,6 +1734,35 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		private void ScrollTimerTickHandler (object sender, EventArgs e)
+		{
+			Point pt = Cursor.Position;
+
+			pt = PointToClient (pt);
+
+			if (pt.X < ClientRectangle.Left) {
+				document.MoveCaret(CaretDirection.CharBackNoWrap);
+				document.SetSelectionToCaret(false);
+				
+				CaretMoved(this, null);
+			} else if (pt.X > ClientRectangle.Right) {
+				document.MoveCaret(CaretDirection.CharForwardNoWrap);
+				document.SetSelectionToCaret(false);
+				
+				CaretMoved(this, null);
+			} else if (pt.Y > ClientRectangle.Bottom) {
+				document.MoveCaret(CaretDirection.LineDown);
+				document.SetSelectionToCaret(false);
+				
+				CaretMoved(this, null);
+			} else if (pt.Y < ClientRectangle.Top) {
+				document.MoveCaret(CaretDirection.LineUp);
+				document.SetSelectionToCaret(false);
+				
+				CaretMoved(this, null);
+			}
+		}
+
 		/// <summary>Ensure the caret is always visible</summary>
 		internal void CaretMoved(object sender, EventArgs e) {
 			Point	pos;
@@ -1722,7 +1791,7 @@ namespace System.Windows.Forms {
 				}
 
 				// Check if we moved out of view to the right
-				if ((pos.X >= (document.ViewPortWidth + document.ViewPortX)) && (hscroll.Enabled && (hscroll.Value != hscroll.Maximum))) {
+				if ((pos.X >= (document.ViewPortWidth + document.ViewPortX)) && (hscroll.Value != hscroll.Maximum)) {
 					if ((pos.X - document.ViewPortWidth + 1) <= hscroll.Maximum) {
 						if (pos.X - document.ViewPortWidth >= 0) {
 							hscroll.Value = pos.X - document.ViewPortWidth + 1;
@@ -1759,11 +1828,9 @@ namespace System.Windows.Forms {
 			}
 		}
 
-		internal bool Paste(DataFormats.Format format, bool obey_length) {
-			IDataObject	clip;
+		internal bool Paste(IDataObject clip, DataFormats.Format format, bool obey_length) {
 			string		s;
 
-			clip = Clipboard.GetDataObject();
 			if (clip == null)
 				return false;
 			
