@@ -25,13 +25,13 @@
 //
 // Authors:
 //	Jordi Mas i Hernandez, jordi@ximian.com
-//
+//	Rolf Bjarne Kvinge, RKvinge@novell.com
+// 
 // TODO:
 //		- The AutoSize functionality seems quite broken for vertical controls in .Net 1.1. Not
 //		sure if we are implementing it the right way.
 //
 
-
 // NOT COMPLETE
 
 using System.ComponentModel;
@@ -63,7 +63,11 @@ namespace System.Windows.Forms
 		private System.Timers.Timer holdclick_timer = new System.Timers.Timer ();
 		internal int thumb_mouseclick;		
 		private bool mouse_clickmove;
-
+		private bool is_moving_right; // which way the thumb should move when mouse is down (right=up, left=down) 
+		internal int mouse_down_x_offset; // how far from left side of thumb was the mouse clicked.
+		internal bool mouse_moved; // has the mouse moved since it was clicked?
+		private const int size_of_autosize = 45;
+	
 		#region events
 		[Browsable (false)]
 		[EditorBrowsable (EditorBrowsableState.Never)]
@@ -142,9 +146,15 @@ namespace System.Windows.Forms
 			MouseUp += new MouseEventHandler (OnMouseUpTB); 
 			MouseMove += new MouseEventHandler (OnMouseMoveTB);
 			KeyDown += new KeyEventHandler (OnKeyDownTB);
+			LostFocus += new EventHandler (OnLostFocusTB);
+			GotFocus += new EventHandler (OnGotFocusTB);
 			holdclick_timer.Elapsed += new ElapsedEventHandler (OnFirstClickTimer);
 
-			SetStyle (ControlStyles.UserPaint | ControlStyles.Opaque, false);
+			SetStyle (ControlStyles.UserPaint | ControlStyles.Opaque
+#if NET_2_0
+				| ControlStyles.UseTextForAccessibility
+#endif
+				, false);
 		}
 
 		#region Private & Internal Properties
@@ -225,8 +235,8 @@ namespace System.Windows.Forms
 			get { return largeChange; }
 			set {
 				if (value < 0)
-					throw new Exception( string.Format("Value '{0}' must be greater than or equal to 0.", value));
-
+					throw new ArgumentException( string.Format("Value '{0}' must be greater than or equal to 0.", value));
+				
 				largeChange = value;				
 			}
 		}
@@ -275,11 +285,11 @@ namespace System.Windows.Forms
 				/* Orientation can be changed once the control has been created */
 				if (orientation != value) {
 					orientation = value;
-				
-					int old_witdh = Width;
-					Width = Height;
-					Height = old_witdh;
-					Refresh (); 
+					
+					if (this.IsHandleCreated) {
+						Size = new Size (Height, Width);
+						Refresh (); 
+					}
 				}
 			}
 		}
@@ -289,7 +299,7 @@ namespace System.Windows.Forms
 			get { return smallChange;}
 			set {
 				if ( value < 0 )
-					throw new Exception( string.Format("Value '{0}' must be greater than or equal to 0.", value));
+					throw new ArgumentException( string.Format("Value '{0}' must be greater than or equal to 0.", value));
 
 				if (smallChange != value) {
 					smallChange = value;					
@@ -364,6 +374,17 @@ namespace System.Windows.Forms
 			base.CreateHandle ();
 		}
 
+		protected override void SetBoundsCore (int x, int y,int width, int height, BoundsSpecified specified)
+		{
+			if (AutoSize) {
+				if (orientation == Orientation.Vertical) {
+					width = size_of_autosize;
+				} else {
+					height = size_of_autosize;
+				}
+			}
+			base.SetBoundsCore (x, y, width, height, specified);
+		}
 
 		public void EndInit ()		
 		{
@@ -439,7 +460,7 @@ namespace System.Windows.Forms
 
 		public override string ToString()
 		{
-			return string.Format("System.Windows.Forms.Trackbar, Minimum: {0}, Maximum: {1}, Value: {2}",
+			return string.Format("System.Windows.Forms.TrackBar, Minimum: {0}, Maximum: {1}, Value: {2}",
 						Minimum, Maximum, Value);
 		}
 							
@@ -512,6 +533,8 @@ namespace System.Windows.Forms
     		{
     			if (!Enabled) return;			    			
 
+			mouse_moved = false;
+
 			bool fire_timer = false;
     			
     			Point point = new Point (e.X, e.Y);
@@ -522,11 +545,13 @@ namespace System.Windows.Forms
 					this.Capture = true;
 					thumb_pressed = true;
 					thumb_mouseclick = e.X;
+					mouse_down_x_offset = e.X - thumb_pos.X;
 					Invalidate (thumb_area);
 				}
 				else {
 					if (ClientRectangle.Contains (point)) {
-						if (e.X > thumb_pos.X + thumb_pos.Width)
+						is_moving_right = e.X > thumb_pos.X + thumb_pos.Width; 
+						if (is_moving_right)
 							LargeIncrement ();
 						else
 							LargeDecrement ();
@@ -538,16 +563,20 @@ namespace System.Windows.Forms
 				}
 			}
 			else {
-				if (thumb_pos.Contains (point)) {
+				Rectangle vertical_thumb_pos = thumb_pos;
+				vertical_thumb_pos.Width = thumb_pos.Height;
+				vertical_thumb_pos.Height = thumb_pos.Width;
+				if (vertical_thumb_pos.Contains (point)) {
 					this.Capture = true;
 					thumb_pressed = true;
 					thumb_mouseclick = e.Y;
+					mouse_down_x_offset = e.Y - thumb_pos.Y;
 					Invalidate (thumb_area);
-					
 				}
 				else {
 					if (ClientRectangle.Contains (point)) {
-						if (e.Y > thumb_pos.Y + thumb_pos.Height)
+						is_moving_right = e.Y > thumb_pos.Y + thumb_pos.Width;
+						if (is_moving_right)
 							LargeDecrement ();
 						else
 							LargeIncrement ();
@@ -569,6 +598,8 @@ namespace System.Windows.Forms
     		{    			
     			if (!Enabled) return;
     		
+    			mouse_moved = true;
+
     			/* Moving the thumb */
     			if (thumb_pressed) {
 								 				
@@ -591,6 +622,15 @@ namespace System.Windows.Forms
 			ThemeEngine.Current.DrawTrackBar (pevent.Graphics, pevent.ClipRectangle, this);
 		}
 
+		private void OnLostFocusTB (object sender, EventArgs e)
+		{
+			Invalidate();
+		}
+
+		private void OnGotFocusTB (object sender, EventArgs e)
+		{
+			Invalidate();
+		}
 		private void OnKeyDownTB (object sender, KeyEventArgs e) 
 		{
 			switch (e.KeyCode) {
@@ -630,34 +670,38 @@ namespace System.Windows.Forms
 		{						
 			Point pnt;
 			pnt = PointToClient (MousePosition);			
-
+			/*
+				On Win32 the thumb only moves in one direction after a click, 
+				if the thumb passes the clicked point it will never go in the 
+				other way unless the mouse is released and clicked again. This
+				is also true if the mouse moves while beeing hold down.
+			*/
+		
 			if (thumb_area.Contains (pnt)) 	{
+				bool invalidate = false;
 				if (orientation == Orientation.Horizontal) {
-					if (pnt.X > thumb_pos.X + thumb_pos.Width)
+					if (pnt.X > thumb_pos.X + thumb_pos.Width && is_moving_right) {
 						LargeIncrement ();
-
-					if (pnt.X < thumb_pos.X)
-						LargeDecrement ();						
-				}
-				else 				{
-					if (pnt.Y > thumb_pos.Y + thumb_pos.Height)
-						LargeIncrement ();
-
-					if (pnt.Y < thumb_pos.Y)
-						LargeDecrement ();
-				}
-
-				Invalidate (thumb_area);
-
+						invalidate = true;
+					} else if (pnt.X < thumb_pos.X && !is_moving_right) {
+						LargeDecrement ();			
+						invalidate = true;
+					}					
+				} else {
+					if (pnt.Y > thumb_pos.Y + thumb_pos.Width && is_moving_right) {
+						LargeDecrement ();		
+						invalidate = true;
+					} else if (pnt.Y < thumb_pos.Y && !is_moving_right) {
+						LargeIncrement ();		
+						invalidate = true;
+					}
+ 				}
+ 				if (invalidate)
+ 					// A Refresh is necessary because the mouse is down and if we just invalidate
+ 					// we'll only get paint events once in a while.
+ 					Refresh();
 			}			
 		}					
-
-		protected override void SetBoundsCore (int x, int y,int width, int height, BoundsSpecified specified)
-		{
-			base.SetBoundsCore (x, y,width,	height, specified);
-		}
-
-		
     		#endregion // Private Methods
 	}
 }
