@@ -17,12 +17,11 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-// Copyright (c) 2004-2005 Novell, Inc.
+// Copyright (c) 2004-2006 Novell, Inc.
 //
 // Authors:
 //	Jackson Harper (jackson@ximian.com)
 
-// TODO: Sorting
 
 using System;
 using System.Collections;
@@ -73,16 +72,12 @@ namespace System.Windows.Forms {
 
 		object IList.this [int index] {
 			get {
-				if (index < 0 || index >= Count)
-					throw new ArgumentOutOfRangeException ("index");
-				return nodes [index];
+				return this [index];
 			}
 			set {
-				if (index < 0 || index >= Count)
-					throw new ArgumentOutOfRangeException ("index");
-				TreeNode node = (TreeNode) value;
-				SetupNode (node);
-				nodes [index] = node;
+				if (!(value is TreeNode))
+					throw new ArgumentException ("value");
+				this [index] = (TreeNode) value;
 			}
 		}
 
@@ -156,6 +151,7 @@ namespace System.Windows.Forms {
 				if (tree_view != null) {
 					tree_view.UpdateBelow (owner);
 					tree_view.RecalculateVisibleOrder (owner);
+					tree_view.UpdateScrollBars ();
 				}
 			}
 		}
@@ -194,9 +190,16 @@ namespace System.Windows.Forms {
 
 		public void Remove (TreeNode node)
 		{
+			if (node == null)
+				throw new NullReferenceException ();
+
 			int index = IndexOf (node);
-			if (index > 0)
+			if (index != -1)
 				RemoveAt (index);
+#if ONLY_1_1
+			else
+				throw new NullReferenceException ();
+#endif
 		}
 
 		public virtual void RemoveAt (int index)
@@ -211,7 +214,7 @@ namespace System.Windows.Forms {
 			TreeNode new_selected = null;
 			bool visible = removed.IsVisible;
 
-                        TreeView tree_view = null;
+			TreeView tree_view = null;
 			if (owner != null)
 				tree_view = owner.TreeView;
 
@@ -295,7 +298,8 @@ namespace System.Windows.Forms {
 
 				if (node.IsVisible)
 					tree_view.RecalculateVisibleOrder (prev);
-				tree_view.UpdateScrollBars ();
+				if (owner == tree_view.root_node || node.Parent.IsVisible && node.Parent.IsExpanded)
+					tree_view.UpdateScrollBars ();
 			}
 
 			if (owner != null && tree_view != null && (owner.IsExpanded || owner.IsRoot)) {
@@ -365,16 +369,11 @@ namespace System.Windows.Forms {
 
 		// Would be nice to do this without running through the collection twice
 		internal void Sort () {
-
 			Array.Sort (nodes, 0, count, new TreeNodeComparer (Application.CurrentCulture.CompareInfo));
 
 			for (int i = 0; i < count; i++) {
 				nodes [i].Nodes.Sort ();
 			}
-
-			// No null checks since sort can only be called from the treeviews root node collection
-			owner.TreeView.RecalculateVisibleOrder (owner);
-			owner.TreeView.UpdateScrollBars ();
 		}
 
 		private void Grow ()
@@ -404,7 +403,11 @@ namespace System.Windows.Forms {
 			}
 
 			public object Current {
-				get { return collection [index]; }
+				get {
+					if (index == -1)
+						return null;
+					return collection [index];
+				}
 			}
 
 			public bool MoveNext ()
@@ -417,7 +420,7 @@ namespace System.Windows.Forms {
 
 			public void Reset ()
 			{
-				index = 0;
+				index = -1;
 			}
 		}
 
