@@ -48,6 +48,8 @@ namespace System.Windows.Forms {
 		private TreeViewAction selection_action = TreeViewAction.Unknown;
 		internal TreeNode selected_node = null;
 		private TreeNode focused_node = null;
+		private TreeNode highlighted_node;
+		private Rectangle mouse_rect = Rectangle.Empty;
 		private bool select_mmove = false;
 
 		private ImageList image_list;
@@ -122,7 +124,11 @@ namespace System.Windows.Forms {
 			GotFocus += new EventHandler (GotFocusHandler);
 			MouseWheel += new MouseEventHandler(MouseWheelHandler);
 
-			SetStyle (ControlStyles.UserPaint | ControlStyles.StandardClick, false);
+			SetStyle (ControlStyles.UserPaint | ControlStyles.StandardClick
+#if NET_2_0
+				| ControlStyles.UseTextForAccessibility
+#endif
+				, false);
 
 			string_format = new StringFormat ();
 			string_format.LineAlignment = StringAlignment.Center;
@@ -275,6 +281,15 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		internal int ActualItemHeight {
+			get {
+				int res = ItemHeight;
+				if (ImageList != null && ImageList.ImageSize.Height > res)
+					res = ImageList.ImageSize.Height;
+				return res;
+			}
+		}
+
 		[DefaultValue(false)]
 		public bool LabelEdit {
 			get { return label_edit; }
@@ -349,6 +364,7 @@ namespace System.Windows.Forms {
 				if (value != null)
 					invalid = Rectangle.Union (invalid, Bloat (value.Bounds));
 
+				highlighted_node = value;
 				selected_node = value;
 				focused_node = value;
 
@@ -420,7 +436,9 @@ namespace System.Windows.Forms {
 					sorted = value;
 				if (sorted) {
 					Nodes.Sort ();
-					top_node = null;
+					top_node = root_node;
+					RecalculateVisibleOrder (root_node);
+					UpdateScrollBars ();
 					Invalidate ();
 				}
 			}
@@ -444,7 +462,7 @@ namespace System.Windows.Forms {
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 		public int VisibleCount {
 			get {
-				return ViewportRectangle.Height / ItemHeight;
+				return ViewportRectangle.Height / ActualItemHeight;
 			}
 		}
 
@@ -491,15 +509,34 @@ namespace System.Windows.Forms {
 		{
 			BeginUpdate ();
 			root_node.ExpandAll ();
+
 			EndUpdate ();
+
+			// Walk all the way to the end, then walk back visible count
+			//to find the new top node
+			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (root_node);
+			while (walk.MoveNext ())
+			{ }
+
+			for (int i = 0; i < VisibleCount - 1; i++)
+				walk.MovePrevious ();
+
+			SetTop (walk.CurrentNode);
 		}
 
 		
 		public void CollapseAll ()
 		{
+			TreeNode walk = top_node;
+			
+			while (walk.parent != root_node)
+				walk = walk.parent;
+
 			BeginUpdate ();
 			root_node.CollapseAll ();
 			EndUpdate ();
+
+			SetTop (walk);
 		}
 
 		public TreeNode GetNodeAt (Point pt) {
@@ -821,7 +858,7 @@ namespace System.Windows.Forms {
 				top_node = nodes [0];
 
 			OpenTreeNodeEnumerator o = new OpenTreeNodeEnumerator (TopNode);
-			int move = y / ItemHeight;
+			int move = y / ActualItemHeight;
 			for (int i = -1; i < move; i++) {
 				if (!o.MoveNext ())
 					return null;
@@ -863,6 +900,12 @@ namespace System.Windows.Forms {
 		}
 
 		private bool IsCheckboxArea (TreeNode node, int x)
+		{
+			int l = CheckBoxLeft (node);
+			return (x > l && x < l + 10);
+		}
+
+		private int CheckBoxLeft (TreeNode node)
 		{
 			int l = node.Bounds.Left + 5;
 
@@ -870,7 +913,8 @@ namespace System.Windows.Forms {
 				l -= indent;
 			if (ImageList != null)
 				l -= ImageList.ImageSize.Width + 3;
-			return (x > l && x < l + 10);
+
+			return l;
 		}
 
 		internal void RecalculateVisibleOrder (TreeNode start)
@@ -906,10 +950,11 @@ namespace System.Windows.Forms {
 
 			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (first);
 			int offset = -1;
-			while (walk.CurrentNode != node && walk.MoveNext ())
+			int vc = VisibleCount;
+			while (walk.CurrentNode != node && walk.MoveNext () && offset < vbar.Maximum - vc)
 				offset++;
 
-			vbar.Value = offset;
+			SetVScrollTop (walk.CurrentNode);
 		}
 
 		internal void SetBottom (TreeNode node)
@@ -1022,7 +1067,7 @@ namespace System.Windows.Forms {
 				TreeNode current = walk.CurrentNode;
 
 				// Haven't gotten to visible nodes yet
-				if (current.GetY () + ItemHeight < clip.Top)
+				if (current.GetY () + ActualItemHeight < clip.Top)
 					continue;
 
 				// Past the visible nodes
@@ -1054,16 +1099,20 @@ namespace System.Windows.Forms {
 
 		private void DrawNodeCheckBox (TreeNode node, Graphics dc, int x, int middle)
 		{
-			using(Pen pen = new Pen (Color.Black, 2) ) 
-				dc.DrawRectangle (pen, x+ 3, middle - 4, 11, 11);
+			Pen pen = ThemeEngine.Current.ResPool.GetSizedPen(Color.Black, 2);
+				dc.DrawRectangle (pen, x + 3, middle - 4, 11, 11);
 
 			if (node.Checked) {
-				using(Pen check_pen = new Pen (Color.Black, 1)) {
-					dc.DrawLine (check_pen, x + 6, middle + 0, x + 8, middle + 3);
-					dc.DrawLine (check_pen, x + 6, middle + 1, x + 8, middle + 4);
-	
-					dc.DrawLine (check_pen, x + 7, middle + 3, x + 13, middle - 2);
-					dc.DrawLine (check_pen, x + 7, middle + 4, x + 13, middle - 1);
+				Pen check_pen = ThemeEngine.Current.ResPool.GetPen(Color.Black);
+				
+				int check_size = 5;
+				int lineWidth = 3;
+				
+				Rectangle rect = new Rectangle (x + 5, middle - 2, check_size, check_size);
+				
+				for (int i = 0; i < lineWidth; i++) {
+					dc.DrawLine (check_pen, rect.Left + 1, rect.Top + lineWidth + i, rect.Left + 3, rect.Top + 5 + i);
+					dc.DrawLine (check_pen, rect.Left + 3, rect.Top + 5 + i, rect.Left + 7, rect.Top + 1 + i);
 				}
 			}
 		}
@@ -1108,7 +1157,7 @@ namespace System.Windows.Forms {
 
 		private void DrawLinesToNext (TreeNode node, Graphics dc, Rectangle clip, Pen dash, int x, int y)
 		{
-			int middle = y + (ItemHeight / 2);
+			int middle = y + (ActualItemHeight / 2);
 
 			if (node.NextNode != null) {
 				int top = (node.Nodes.Count > 0 && show_plus_minus ? middle + 4 : middle);
@@ -1220,16 +1269,25 @@ namespace System.Windows.Forms {
 			return (int) DeviceContext.MeasureString (node.Text, font, 0, string_format).Width + 3;
 		}
 
+		private bool ShowSelection {
+			get {
+				return Focused || !hide_selection;
+			}
+		}
+
 		private void DrawSelectionAndFocus(TreeNode node, Graphics dc, Rectangle r)
 		{
 			if (Focused && focused_node == node) {
 				ControlPaint.DrawFocusRectangle (dc, r, ForeColor, BackColor);
 			}
 			r.Inflate(-1, -1);
-			if ((!HideSelection || Focused) && SelectedNode == node)
+			if (Focused && node == highlighted_node) {
 				dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (ThemeEngine.Current.ColorHighlight), r);
-			else
+			} else if (!hide_selection && node == highlighted_node) {
+				dc.FillRectangle (SystemBrushes.Control, r);
+			} else {
 				dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (node.BackColor), r);
+			}
 		}
 		 
 		private void DrawStaticNode (TreeNode node, Graphics dc)
@@ -1240,7 +1298,7 @@ namespace System.Windows.Forms {
 			Font font = node.NodeFont;
 			if (node.NodeFont == null)
 				font = Font;
-			Color text_color = ((Focused || !HideSelection) && SelectedNode == node ?
+			Color text_color = (Focused && node == highlighted_node ?
 					ThemeEngine.Current.ColorHighlightText : node.ForeColor);
 			dc.DrawString (node.Text, font,
 					ThemeEngine.Current.ResPool.GetSolidBrush (text_color),
@@ -1251,10 +1309,10 @@ namespace System.Windows.Forms {
 		{
 			int child_count = node.nodes.Count;
 			int y = node.GetY ();
-			int middle = y + (ItemHeight / 2);
+			int middle = y + (ActualItemHeight / 2);
 
 			if (full_row_select) {
-				Rectangle r = new Rectangle (1, y + 2, ViewportRectangle.Width - 2, ItemHeight);
+				Rectangle r = new Rectangle (1, y + 2, ViewportRectangle.Width - 2, ActualItemHeight);
 				DrawSelectionAndFocus (node, dc, r);
 			}
 
@@ -1262,7 +1320,7 @@ namespace System.Windows.Forms {
 				DrawNodePlusMinus (node, dc, node.GetLinesX () - Indent + 5, middle);
 
 			if (checkboxes)
-				DrawNodeCheckBox (node, dc, node.GetX () - 19, middle);
+				DrawNodeCheckBox (node, dc, CheckBoxLeft (node) - 3, middle);
 
 			if (show_lines)
 				DrawNodeLines (node, dc, clip, dash, node.GetLinesX (), y, middle);
@@ -1276,7 +1334,7 @@ namespace System.Windows.Forms {
 
 		internal void UpdateScrollBars ()
 		{
-			if (update_stack > 0)
+			if (IsDisposed || update_stack > 0 || !IsHandleCreated || !Visible)
 				return;
 
 			bool vert = false;
@@ -1316,7 +1374,8 @@ namespace System.Windows.Forms {
 			}
 
 			if (vert) {
-				vbar.SetValues (Math.Max (0, max_visible_order - 2), VisibleCount);
+				int visible_height = horz ? ClientRectangle.Height - hbar.Height : ClientRectangle.Height;
+				vbar.SetValues (Math.Max (0, max_visible_order - 2), visible_height / ActualItemHeight);
 				/*
 				vbar.Maximum = max_visible_order;
 				vbar.LargeChange = ClientRectangle.Height / ItemHeight;
@@ -1418,10 +1477,22 @@ namespace System.Windows.Forms {
 			}
 
 			top_node = new_top;
-			int y_move = diff * ItemHeight;
+			int y_move = diff * ActualItemHeight;
 			XplatUI.ScrollWindow (Handle, ViewportRectangle, 0, y_move, false);
 		}
 
+		private void SetVScrollTop (TreeNode new_top)
+		{
+			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (root_node);
+
+			skipped_nodes = -1;
+			while (walk.MoveNext () && walk.CurrentNode != new_top)
+				skipped_nodes++;
+
+			top_node = new_top;
+			vbar.Value = skipped_nodes;
+		}
+
 		private void HScrollBarValueChanged(object sender, EventArgs e)
 		{
 			EndEdit (edit_node);
@@ -1443,17 +1514,17 @@ namespace System.Windows.Forms {
 				return;
 			}
 
-			Rectangle below = new Rectangle (0, node.Bounds.Bottom, ViewportRectangle.Width,
+			Rectangle below = new Rectangle (0, node.Bounds.Bottom + 2, ViewportRectangle.Width,
 					ViewportRectangle.Height - node.Bounds.Bottom);
-				
-			int amount = count_to_next * ItemHeight;
+
+			int amount = count_to_next * ActualItemHeight;
 
 			if (amount > 0)
 				XplatUI.ScrollWindow (Handle, below, 0, amount, false);
 
 			if (show_plus_minus) {
 				//int linesx = node.GetLinesX ();
-				Invalidate (new Rectangle (0, node.GetY (), Width, ItemHeight));
+				Invalidate (new Rectangle (0, node.GetY (), Width, ActualItemHeight));
 			}
 		}
 
@@ -1464,16 +1535,17 @@ namespace System.Windows.Forms {
 				return;
 			}
 
-			Rectangle below = new Rectangle (0, node.Bounds.Bottom, ViewportRectangle.Width,
-					ViewportRectangle.Height - node.Bounds.Bottom);
-			int amount = count_to_next * ItemHeight;
+			Rectangle below = new Rectangle (0, node.Bounds.Bottom + 2, ViewportRectangle.Width,
+					ViewportRectangle.Height - node.Bounds.Bottom + 2);
+
+			int amount = count_to_next * ActualItemHeight;
 
 			if (amount > 0)
 				XplatUI.ScrollWindow (Handle, below, 0, -amount, false);
 
 			if (show_plus_minus) {
 				//int linesx = node.GetLinesX ();
-				Invalidate (new Rectangle (0, node.GetY (), Width, ItemHeight));
+				Invalidate (new Rectangle (0, node.GetY (), Width, ActualItemHeight));
 			}
 		}
 
@@ -1518,6 +1590,9 @@ namespace System.Windows.Forms {
 
 		private void MouseDownHandler (object sender, MouseEventArgs e)
 		{
+			if (e.Button == MouseButtons.Right)
+				Focus ();
+
 			TreeNode node = GetNodeAt (e.Y);
 			if (node == null)
 				return;
@@ -1531,14 +1606,23 @@ namespace System.Windows.Forms {
 				UpdateNode(node);
 				return;
 			} else if (IsSelectableArea (node, e.X) || full_row_select) {
-				TreeNode old_selected = selected_node;
-				selection_action = TreeViewAction.ByMouse;
-				SelectedNode = node;
-				if (label_edit && e.Clicks == 1 && selected_node == old_selected) {
+				TreeNode old_highlighted = highlighted_node;
+				highlighted_node = node;
+				if (label_edit && e.Clicks == 1 && highlighted_node == old_highlighted) {
 					BeginEdit (node);
-				} else if (selected_node != focused_node) {
+				} else if (highlighted_node != focused_node) {
+					Size ds = SystemInformation.DragSize;
+					mouse_rect.X = e.X - ds.Width;
+					mouse_rect.Y = e.Y - ds.Height;
+					mouse_rect.Width = ds.Width * 2;
+					mouse_rect.Height = ds.Height * 2;
+
 					select_mmove = true;
 				}
+
+				Invalidate (highlighted_node.Bounds);
+				if (old_highlighted != null)
+					Invalidate (Bloat (old_highlighted.Bounds));
 			} 
 		}
 
@@ -1547,33 +1631,36 @@ namespace System.Windows.Forms {
 			drag_begin_x = -1;
 			drag_begin_y = -1;
 
-			OnClick (EventArgs.Empty);
-
 			if (!select_mmove)
 				return;
 
+			if (e.Button == MouseButtons.Right) {
+				Invalidate (highlighted_node.Bounds);
+				highlighted_node = selected_node;
+				Invalidate (selected_node.Bounds);
+			}
 			select_mmove = false;
 
-			TreeViewCancelEventArgs ce = new TreeViewCancelEventArgs (selected_node, false, TreeViewAction.ByMouse);
+			TreeViewCancelEventArgs ce = new TreeViewCancelEventArgs (highlighted_node, false, TreeViewAction.ByMouse);
 			OnBeforeSelect (ce);
 
 			Rectangle invalid;
 			if (!ce.Cancel) {
 				if (focused_node != null) {
 					invalid = Rectangle.Union (Bloat (focused_node.Bounds),
-							Bloat (selected_node.Bounds));
+							Bloat (highlighted_node.Bounds));
 				} else {
-					invalid = Bloat (selected_node.Bounds);
+					invalid = Bloat (highlighted_node.Bounds);
 				}
-				focused_node = selected_node;
+				selected_node = highlighted_node;
+				focused_node = highlighted_node;
 				OnAfterSelect (new TreeViewEventArgs (selected_node, TreeViewAction.ByMouse));
 
 				Invalidate (invalid);
 			} else {
+				highlighted_node = focused_node;
 				selected_node = focused_node;
 			}
-
-			
 		}
 
 		private void MouseMoveHandler (object sender, MouseEventArgs e) {
@@ -1598,16 +1685,20 @@ namespace System.Windows.Forms {
 				}
 				
 			}
-			if(!select_mmove)
-				return;
-			TreeNode node = GetNodeAtUseX (e.X,e.Y);
-			if(node == selected_node)
+
+			// If there is enough movement before the mouse comes up,
+			// selection is reverted back to the originally selected node
+			if (!select_mmove || mouse_rect.Contains (e.X, e.Y))
 				return;
 
-			selected_node = focused_node;
-			select_mmove = false;
-			Invalidate (focused_node.Bounds);
+			Invalidate (highlighted_node.Bounds);
 			Invalidate (selected_node.Bounds);
+			Invalidate (focused_node.Bounds);
+
+			highlighted_node = selected_node;
+			focused_node = selected_node;
+
+			select_mmove = false;
 		}
 
 		private void DoubleClickHandler (object sender, MouseEventArgs e) {
