@@ -226,16 +226,6 @@ namespace System.Windows.Forms
 			{
 				SetStyle (ControlStyles.Selectable, selectable);
 			}
-
-			internal override void OnGotFocusInternal (EventArgs e)
-			{
-				owner.Select (false, true);
-			}
-
-			internal override void OnLostFocusInternal (EventArgs e)
-			{
-				owner.Select (false, true);
-			}
 		}
 		#endregion	// UpDownSpinner Sub-class
 
@@ -262,17 +252,13 @@ namespace System.Windows.Forms
 				else
 					document.CaretLostFocus ();
 			}
+            protected override void OnGotFocus(EventArgs e) {
+                this.parent.OnGotFocusInternal(e);
+            }
 
-			
-			internal override void OnGotFocusInternal (EventArgs e)
-			{
-				owner.Select (false, true);
-			}
-
-			internal override void OnLostFocusInternal (EventArgs e)
-			{
-				owner.Select (false, true);
-			}			
+            protected override void OnLostFocus(EventArgs e) {
+                this.parent.OnLostFocusInternal(e);
+            }
 		}
 
 		#region Local Variables
@@ -323,8 +309,12 @@ namespace System.Windows.Forms
 			// So the child controls don't get auto selected when the updown is selected
 			auto_select_child = false;
 			SetStyle(ControlStyles.FixedHeight, true);
+#if NET_2_0
+			SetStyle (ControlStyles.Opaque | ControlStyles.ResizeRedraw, true);
+			SetStyle (ControlStyles.StandardClick | ControlStyles.UseTextForAccessibility, false);
+#endif
 
-			UpdateEditText();
+			UpdateEditText ();
 		}
 		#endregion
 
@@ -381,6 +371,7 @@ namespace System.Windows.Forms
 			txtView.SetSelectable (true);
 			spnSpinner.SetSelectable (true);
 		}
+
 		#endregion	// Private Methods
 
 		#region Public Instance Properties
@@ -725,8 +716,10 @@ namespace System.Windows.Forms
 			case Msg.WM_CHAR:
 				XplatUI.SendMessage (txtView.Handle, (Msg) m.Msg, m.WParam, m.LParam);
 				break;
+			default:
+				base.WndProc (ref m);
+				break;
 			}
-			base.WndProc (ref m);
 		}
 		#endregion	// Protected Instance Methods
 
