@@ -81,11 +81,10 @@ namespace System.Windows.Forms {
 			}
 
 			IntPtr xim = XOpenIM (display, IntPtr.Zero, IntPtr.Zero, IntPtr.Zero);
-			if (xim == IntPtr.Zero) {
+			if (xim == IntPtr.Zero) 
 				Console.Error.WriteLine ("Could not get XIM");
-			}
-
-			xic = CreateXic (window, xim);
+			else 
+				xic = CreateXic (window, xim);
 			
 			initialized = true;
 		}
@@ -402,13 +401,13 @@ namespace System.Windows.Forms {
 
 			switch (xevent.type) {
 			case XEventName.KeyRelease:
-				key_state_table [(int) vkey] &= unchecked ((byte) ~0x80);
+				key_state_table [vkey & 0xff] &= unchecked ((byte) ~0x80);
 				break;
 			case XEventName.KeyPress:
-				if ((key_state_table [(int) vkey] & 0x80) == 0) {
-					key_state_table [(int) vkey] ^= 0x01;
+				if ((key_state_table [vkey & 0xff] & 0x80) == 0) {
+					key_state_table [vkey & 0xff] ^= 0x01;
 				}
-				key_state_table [(int) vkey] |= 0x80;
+				key_state_table [vkey & 0xff] |= 0x80;
 				break;
 			}
 		}
