@@ -484,12 +484,6 @@ namespace System.Windows.Forms {
 		internal byte		minor_code;
 	}
 
-	[StructLayout(LayoutKind.Sequential)]
-	internal struct XTimerNotifyEvent {
-		internal XEventName	type;
-		internal EventHandler	handler;
-	}
-
 	[StructLayout(LayoutKind.Sequential)]
 	internal struct XEventPad {
 		internal IntPtr pad0;
@@ -552,7 +546,6 @@ namespace System.Windows.Forms {
 		[ FieldOffset(0) ] internal XMappingEvent MappingEvent;
 		[ FieldOffset(0) ] internal XErrorEvent ErrorEvent;
 		[ FieldOffset(0) ] internal XKeymapEvent KeymapEvent;
-		[ FieldOffset(0) ] internal XTimerNotifyEvent TimerNotifyEvent;
 
 		//[MarshalAs(System.Runtime.InteropServices.UnmanagedType.ByValArray, SizeConst=24)]
 		//[ FieldOffset(0) ] internal int[] pad;
@@ -655,7 +648,6 @@ namespace System.Windows.Forms {
 		ColormapNotify          = 32,
 		ClientMessage		= 33,
 		MappingNotify		= 34,
-		TimerNotify		= 100,
 
 		LASTEvent
 	}
@@ -1340,89 +1332,6 @@ namespace System.Windows.Forms {
 		internal int			height_inc;
 	}
 
-	internal enum NA {
-		WM_PROTOCOLS,
-		WM_DELETE_WINDOW,
-		WM_TAKE_FOCUS,
-
-		_NET_SUPPORTED,
-		_NET_CLIENT_LIST,
-		_NET_NUMBER_OF_DESKTOPS,
-		_NET_DESKTOP_GEOMETRY,
-		_NET_DESKTOP_VIEWPORT,
-		_NET_CURRENT_DESKTOP,
-		_NET_DESKTOP_NAMES,
-		_NET_ACTIVE_WINDOW,
-		_NET_WORKAREA,
-		_NET_SUPPORTING_WM_CHECK,
-		_NET_VIRTUAL_ROOTS,
-		_NET_DESKTOP_LAYOUT,
-		_NET_SHOWING_DESKTOP,
-
-		_NET_CLOSE_WINDOW,
-		_NET_MOVERESIZE_WINDOW,
-		_NET_WM_MOVERESIZE,
-		_NET_RESTACK_WINDOW,
-		_NET_REQUEST_FRAME_EXTENTS,
-
-		_NET_WM_NAME,
-		_NET_WM_VISIBLE_NAME,
-		_NET_WM_ICON_NAME,
-		_NET_WM_VISIBLE_ICON_NAME,
-		_NET_WM_DESKTOP,
-		_NET_WM_WINDOW_TYPE,
-		_NET_WM_STATE,
-		_NET_WM_ALLOWED_ACTIONS,
-		_NET_WM_STRUT,
-		_NET_WM_STRUT_PARTIAL,
-		_NET_WM_ICON_GEOMETRY,
-		_NET_WM_ICON,
-		_NET_WM_PID,
-		_NET_WM_HANDLED_ICONS,
-		_NET_WM_USER_TIME,
-		_NET_FRAME_EXTENTS,
-
-		_NET_WM_PING,
-		_NET_WM_SYNC_REQUEST,
-
-		_NET_SYSTEM_TRAY_S,
-		_NET_SYSTEM_TRAY_ORIENTATION,
-		_NET_SYSTEM_TRAY_OPCODE,
-
-		_NET_WM_STATE_MAXIMIZED_HORZ,
-		_NET_WM_STATE_MAXIMIZED_VERT,
-
-		_XEMBED,
-		_XEMBED_INFO,
-
-		_MOTIF_WM_HINTS,
-
-		_NET_WM_STATE_NO_TASKBAR,
-		_NET_WM_STATE_ABOVE,
-		_NET_WM_STATE_MODAL,
-		_NET_WM_STATE_HIDDEN,
-		_NET_WM_CONTEXT_HELP,
-
-		_NET_WM_WINDOW_OPACITY,
-
-		_NET_WM_WINDOW_TYPE_DESKTOP,
-		_NET_WM_WINDOW_TYPE_DOCK,
-		_NET_WM_WINDOW_TYPE_TOOLBAR,
-		_NET_WM_WINDOW_TYPE_MENU,
-		_NET_WM_WINDOW_TYPE_UTILITY,
-		_NET_WM_WINDOW_TYPE_SPLASH,
-		_NET_WM_WINDOW_TYPE_DIALOG,
-		_NET_WM_WINDOW_TYPE_NORMAL,
-
-		CLIPBOARD,
-		DIB,
-		OEMTEXT,
-		UNICODETEXT,
-		TARGETS,
-
-		LAST_NET_ATOM
-	}
-
 	internal struct CaretStruct {
 		internal Timer	Timer;				// Blink interval
 		internal IntPtr	Hwnd;				// Window owning the caret
@@ -1616,4 +1525,20 @@ namespace System.Windows.Forms {
 		Both			= 3
 	}
 
+	internal enum XEmbedMessage {
+		EmbeddedNotify = 0,
+		WindowActivate = 1,
+		WindowDeactivate = 2,
+		RequestFocus = 3,
+		FocusIn = 4,
+		FocusOut = 5,
+		FocusNext = 6,
+		FocusPrev = 7,
+		/* 8-9 were used for XEMBED_GRAB_KEY/XEMBED_UNGRAB_KEY */
+		ModalityOn = 10,
+		ModalityOff = 11,
+		RegisterAccelerator = 12,
+		UnregisterAccelerator = 13,
+		ActivateAccelerator = 14
+	}
 }
