@@ -38,6 +38,7 @@ using System.Collections;
 using System.Diagnostics;
 using System.Runtime.InteropServices;
 using System.Threading;
+using System.Windows.Forms.X11Internal;
 
 namespace System.Windows.Forms {
 	internal class XplatUI {
@@ -77,18 +78,21 @@ namespace System.Windows.Forms {
 
 		#region Constructor & Destructor
 		static XplatUI() {
-			Console.WriteLine("Mono System.Windows.Forms Assembly [$auto_build_revision$]");
-
-			default_class_name="SWFClass";
+			// Compose name with current domain id because on Win32 we register class name
+			// and name must be unique to process. If we load MWF into multiple appdomains
+			// and try to register same class name we fail.
+			default_class_name = "SWFClass" + System.Threading.Thread.GetDomainID().ToString();
 
 			// check for Unix platforms - see FAQ for more details
 			// http://www.mono-project.com/FAQ:_Technical#How_to_detect_the_execution_platform_.3F
 			int platform = (int) Environment.OSVersion.Platform;
 			if ((platform == 4) || (platform == 128)) {
 				if (Environment.GetEnvironmentVariable ("not_supported_MONO_MWF_USE_QUARTZ_BACKEND") != null)
-					driver=XplatUIOSX.GetInstance();
+					driver=XplatUIOSX.GetInstance ();
+				else if (Environment.GetEnvironmentVariable ("not_supported_MONO_MWF_USE_NEW_X11_BACKEND") != null)
+					driver=XplatUIX11_new.GetInstance ();
 				else
-					driver=XplatUIX11.GetInstance();
+					driver=XplatUIX11.GetInstance ();
 			} else {
 				driver=XplatUIWin32.GetInstance();
 			}
@@ -366,11 +370,11 @@ namespace System.Windows.Forms {
 			return driver.ClipboardGetID(handle, format);
 		}
 
-		internal static IntPtr ClipboardOpen() {
+		internal static IntPtr ClipboardOpen(bool primary_selection) {
 			#if DriverDebug
 				Console.WriteLine("ClipboardOpen(): Called");
 			#endif
-			return driver.ClipboardOpen();
+			return driver.ClipboardOpen (primary_selection);
 		}
 
 		internal static void ClipboardStore(IntPtr handle, object obj, int type, XplatUI.ObjectToClipboard converter) {
@@ -544,6 +548,12 @@ namespace System.Windows.Forms {
 			driver.GetWindowPos(handle, is_toplevel, out x, out y, out width, out height, out client_width, out client_height);
 		}
 
+		/* this method can (and does, on X11) return
+		 * (FormWindowState)(-1), when the state of the window
+		 * cannot be determined (in the X11 case, when the
+		 * window isn't mapped.)  Checking for the additional
+		 * return value is less expensive than
+		 * throwing/catching an exception. */
 		internal static FormWindowState GetWindowState(IntPtr handle) {
 			#if DriverDebug
 				Console.WriteLine("GetWindowState({0}): Called", Window(handle));
@@ -576,6 +586,15 @@ namespace System.Windows.Forms {
 			driver.Invalidate(handle, rc, clear);
 		}
 
+		internal static void InvalidateNC (IntPtr handle)
+		{
+			#if DriverDebug
+				Console.WriteLine("InvalidateNC({0}): Called", Window(handle));
+			#endif
+			driver.InvalidateNC(handle);
+		}
+
+
 		internal static bool IsEnabled(IntPtr handle) {
 			#if DriverDebug || DriverDebugState
 				Console.WriteLine("IsEnabled({0}): Called, Result={1}", Window(handle), driver.IsEnabled(handle));
@@ -651,6 +670,13 @@ namespace System.Windows.Forms {
 			driver.PostQuitMessage(exitCode);
 		}
 
+		internal static void RequestAdditionalWM_NCMessages(IntPtr handle, bool hover, bool leave) {
+			#if DriverDebug
+				Console.WriteLine("RequestAdditionalWM_NCMessages({0}, {1}, {2}): Called", Window(handle), hover, leave);
+			#endif
+			driver.RequestAdditionalWM_NCMessages (handle, hover, leave);
+		}
+
 		internal static void RequestNCRecalc(IntPtr handle) {
 			#if DriverDebug
 				Console.WriteLine("RequestNCRecalc({0}): Called", Window(handle));
@@ -807,11 +833,12 @@ namespace System.Windows.Forms {
 			return driver.SetTopmost(handle, hWndOwner, Enabled);
 		}
 
-		internal static bool SetVisible(IntPtr handle, bool visible) {
+		internal static bool SetVisible (IntPtr handle, bool visible, bool activate)
+		{
 			#if DriverDebug || DriverDebugState
-				Console.WriteLine("SetVisible({0}, {1}): Called", Window(handle), visible);
+				Console.WriteLine("SetVisible({0}, {1}, {2}): Called", Window(handle), visible, activate);
 			#endif
-			return driver.SetVisible(handle, visible);
+			return driver.SetVisible (handle, visible, activate);
 		}
 
 		internal static void SetWindowMinMax(IntPtr handle, Rectangle maximized, Size min, Size max) {
@@ -842,6 +869,14 @@ namespace System.Windows.Forms {
 			driver.SetWindowStyle(handle, cp);
 		}
 
+		internal static double GetWindowTransparency (IntPtr handle)
+		{
+			#if DriverDebug
+				Console.WriteLine("SetWindowTransparency({0}): Called", Window(handle));
+			#endif
+			return driver.GetWindowTransparency(handle);
+		}
+
 		internal static void SetWindowTransparency(IntPtr handle, double transparency, Color key) 
 		{
 			#if DriverDebug
@@ -878,7 +913,7 @@ namespace System.Windows.Forms {
 			return driver.StartLoop(thread);
 		}
 
-		internal static bool SupportsTransparency() {
+		internal static TransparencySupport SupportsTransparency() {
 			#if DriverDebug
 				Console.WriteLine("SupportsTransparency(): Called, result={0}", driver.SupportsTransparency());
 			#endif
