@@ -859,7 +859,7 @@ namespace System.Windows.Forms {
 			return 0;
 		}
 
-		internal override IntPtr ClipboardOpen() {
+		internal override IntPtr ClipboardOpen(bool primary_selection) {
 			throw new NotImplementedException();
 		}
 
@@ -1303,6 +1303,11 @@ namespace System.Windows.Forms {
 				hwnd.expose_pending = true;
 			}
 		}
+
+		internal override void InvalidateNC (IntPtr handle)
+		{
+			// XXX FIXME
+		}
 		
 		internal override bool IsEnabled(IntPtr handle) {
 			return Hwnd.ObjectFromHandle(handle).Enabled;
@@ -1351,9 +1356,14 @@ namespace System.Windows.Forms {
 				HideCaret();
 			}
 
-			hwnd.client_dc  = Graphics.FromHwnd (hwnd.client_window);
-			paint_event = new PaintEventArgs(hwnd.client_dc, hwnd.Invalid);
-			
+			Graphics dc = Graphics.FromHwnd (hwnd.client_window);
+			paint_event = new PaintEventArgs(dc, hwnd.Invalid);
+
+			hwnd.expose_pending = false;
+			hwnd.ClearInvalidArea();
+
+			hwnd.drawing_stack.Push (dc);
+
 			return paint_event;
 		}
 		
@@ -1362,12 +1372,9 @@ namespace System.Windows.Forms {
 
 			hwnd = Hwnd.ObjectFromHandle(handle);
 
-			hwnd.ClearInvalidArea();
-
-			hwnd.client_dc.Flush();
-			hwnd.client_dc.Dispose();
-			hwnd.client_dc = null;
-			hwnd.expose_pending = false;
+			Graphics dc = (Graphics)hwnd.drawing_stack.Pop();
+			dc.Flush ();
+			dc.Dispose ();
 			
 			if (Caret.Visible == 1) {
 				ShowCaret();
@@ -1395,6 +1402,11 @@ namespace System.Windows.Forms {
 			throw new NotImplementedException();
 		}
 
+		[MonoTODO]
+		internal override void RequestAdditionalWM_NCMessages(IntPtr hwnd, bool hover, bool leave) {
+			throw new NotImplementedException();
+		}
+
 		[MonoTODO]		
 		internal override void RequestNCRecalc(IntPtr handle) {
 			throw new NotImplementedException();
@@ -1582,7 +1594,7 @@ namespace System.Windows.Forms {
 		}
 		
 		
-		internal override bool SetVisible(IntPtr handle, bool visible) {
+		internal override bool SetVisible(IntPtr handle, bool visible, bool activate) {
 			Hwnd hwnd = Hwnd.ObjectFromHandle (handle);
 			object window = WindowMapping [hwnd.Handle];
 			if (window != null)
@@ -1697,8 +1709,13 @@ namespace System.Windows.Forms {
 		internal override void SetWindowTransparency(IntPtr handle, double transparency, Color key) {
 		}
 
-		internal override bool SupportsTransparency() {
-			return false;
+		internal override double GetWindowTransparency(IntPtr handle)
+		{
+			return 1.0;
+		}
+
+		internal override TransparencySupport SupportsTransparency() {
+			return TransparencySupport.None;
 		}
 		
 		internal override bool SetZOrder(IntPtr handle, IntPtr after_handle, bool Top, bool Bottom) {
