@@ -302,7 +302,9 @@ namespace System.Windows.Forms {
 		WM_IME_REQUEST            = 0x0288,
 		WM_IME_KEYDOWN            = 0x0290,
 		WM_IME_KEYUP              = 0x0291,
+		WM_NCMOUSEHOVER           = 0x02A0,
 		WM_MOUSEHOVER             = 0x02A1,
+		WM_NCMOUSELEAVE           = 0x02A2,
 		WM_MOUSELEAVE             = 0x02A3,
 		WM_CUT                    = 0x0300,
 		WM_COPY                   = 0x0301,
@@ -422,6 +424,15 @@ namespace System.Windows.Forms {
 		internal IntPtr	lParam;
 		internal uint	time;  
 		internal POINT	pt;
+		internal object refobject;
+	}
+
+	[Flags]
+	internal enum TransparencySupport {
+		None = 0x00,
+		Get = 0x01,
+		Set = 0x02,
+		GetSet = 0x03
 	}
 
 	internal enum WindowActiveFlags {
@@ -826,5 +837,16 @@ namespace System.Windows.Forms {
 		SB_RIGHT		= 7,
 		SB_ENDSCROLL		= 8
 	}
+
+	internal enum ClipCombineMode {
+		RGN_AND = 1,
+		RGN_OR,
+		RGN_XOR,
+		RGN_DIFF,
+		RGN_COPY,
+
+		RGN_MIN = RGN_AND,
+		RGN_MAX = RGN_COPY
+	}
 }
 
