@@ -82,13 +82,11 @@ namespace MonoTests.System.Windows.Forms {
 		[SetUp]
 		protected virtual void SetUp ()
 		{
-			Console.WriteLine ("SETTING UP");
 		}
 
 		[TearDown]
 		protected virtual void TearDown ()
 		{
-			Console.WriteLine ("TEARING DOWN");
 		}
 
 		[Test]
@@ -176,7 +174,6 @@ namespace MonoTests.System.Windows.Forms {
 			BindingManagerBase a = bc [data_source, "Items"];
 		}
 
-		[Category ("NotWorking")]
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
 		public void CantCreateChildList2 ()
