@@ -14,17 +14,76 @@ using System.Runtime.Remoting;
 
 using NUnit.Framework;
 
-namespace MWF.MonoTest
+namespace MonoTests.System.Windows.Forms
 {
 	[TestFixture]
 	public class ControlTest
 	{
-		internal static void TestAccessibility(Control c, string Default, string Description, string Name, AccessibleRole Role) {
-			Assert.AreEqual(false, c.AccessibilityObject == null, "Acc1");
-			Assert.AreEqual(Default, c.AccessibleDefaultActionDescription, "Acc2");
-			Assert.AreEqual(Description, c.AccessibleDescription, "Acc3");
-			Assert.AreEqual(Name, c.AccessibleName, "Acc4");
-			Assert.AreEqual(Role, c.AccessibleRole, "Acc5");
+		class Helper {
+			public static void TestAccessibility(Control c, string Default, string Description, string Name, AccessibleRole Role)
+			{
+				Assert.IsNotNull (c.AccessibilityObject, "Acc1");
+				Assert.AreEqual (Default, c.AccessibleDefaultActionDescription, "Acc2");
+				Assert.AreEqual (Description, c.AccessibleDescription, "Acc3");
+				Assert.AreEqual (Name, c.AccessibleName, "Acc4");
+				Assert.AreEqual (Role, c.AccessibleRole, "Acc5");
+			}
+
+			public static string TestControl(Control container, Control start, bool forward) {
+				Control	ctl;
+
+				ctl = container.GetNextControl(start, forward);
+
+				if (ctl == null) {
+					return null;
+				}
+
+				return ctl.Text;
+			}
+		}
+
+		[Test]
+		public void CreatedTest ()
+		{
+			Control c = new Control ();
+			Assert.IsFalse (c.Created, "A1");
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void CreatedAccessibilityTest ()
+		{
+			Control c = new Control ();
+			Assert.IsFalse (c.Created, "A1");
+
+			Helper.TestAccessibility(c, null, null, null, AccessibleRole.Default);
+
+			Assert.IsTrue (c.Created, "A2");
+
+			c.Dispose ();
+
+			Assert.IsFalse (c.Created, "A3");
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void BoundsTest ()
+		{
+			Control c = new Control ();
+			Assert.IsTrue (c.Bounds.IsEmpty, "A1");
+			Assert.IsTrue (c.Size.IsEmpty, "A2");
+			Assert.IsTrue (c.ClientSize.IsEmpty, "A3");
+			Assert.IsTrue (c.ClientRectangle.IsEmpty, "A4");
+
+			Assert.AreEqual (((IWin32Window)c).Handle, c.Handle, "A5");
+
+			/* this part fails on linux because we can't allocate X windows which are 0x0,
+			   and the Control bounds directly reflect the size of the X window */
+
+			Assert.IsTrue (c.Bounds.IsEmpty, "A6");
+			Assert.IsTrue (c.Size.IsEmpty, "A7");
+			Assert.IsTrue (c.ClientSize.IsEmpty, "A8");
+			Assert.IsTrue (c.ClientRectangle.IsEmpty, "A9");
 		}
 
 		[Test]
@@ -32,129 +91,140 @@ namespace MWF.MonoTest
     		{
 			Control c = new Control();
 
-			TestAccessibility(c, null, null, null, AccessibleRole.Default);
-
-			// A
-			Assert.AreEqual(false, c.AllowDrop , "A1");
+			Assert.IsFalse (c.AllowDrop , "A1");
 			Assert.AreEqual(AnchorStyles.Top | AnchorStyles.Left, c.Anchor, "A2");
 
-			// B
-			Assert.AreEqual("Control", c.BackColor.Name , "B1");
-			Assert.AreEqual(null, c.BackgroundImage, "B2");
-			Assert.AreEqual(null, c.BindingContext, "B3");
-			Assert.AreEqual(0, c.Bottom, "B4");
-			Assert.AreEqual (new Rectangle (0,0, 0, 0) , c.Bounds , "B5");
+			Assert.AreEqual ("Control", c.BackColor.Name , "B1");
+			Assert.IsNull (c.BackgroundImage, "B2");
+			Assert.IsNull (c.BindingContext, "B3");
 
-			// C
-			Assert.AreEqual(false, c.CanFocus, "C1");
-			Assert.AreEqual(true, c.CanSelect, "C2");
-			Assert.AreEqual(false, c.Capture, "C3");
-			Assert.AreEqual(true, c.CausesValidation, "C4");
-			Assert.AreEqual (new Rectangle(0, 0, 0, 0) , c.ClientRectangle , "C5");
-			Assert.AreEqual (new Size(0, 0), c.ClientSize , "C6");
+			Assert.IsFalse (c.CanFocus, "C1");
+			Assert.IsTrue (c.CanSelect, "C2");
+			Assert.IsFalse (c.Capture, "C3");
+			Assert.IsTrue (c.CausesValidation, "C4");
 
-			string name = c.CompanyName;
-			if (!name.Equals("Mono Project, Novell, Inc.") && !name.Equals("Microsoft Corporation")) {
-				Assert.Fail("CompanyName property does not match any accepted value - C7");
-			}
-			Assert.AreEqual(null, c.Container, "C8");
-			Assert.AreEqual(false, c.ContainsFocus, "C9");
-			Assert.AreEqual(null, c.ContextMenu, "C10");
-			Assert.AreEqual(0, c.Controls.Count, "C11");
-			Assert.AreEqual(true, c.Created, "C12");
-			Assert.AreEqual(Cursors.Default, c.Cursor, "C13");
+			Assert.IsNotNull (c.CompanyName, "C7");
+			Assert.IsNull (c.Container, "C8");
+			Assert.IsFalse (c.ContainsFocus, "C9");
+			Assert.IsNull (c.ContextMenu, "C10");
+			Assert.AreEqual (0, c.Controls.Count, "C11");
+			Assert.IsFalse (c.Created, "C12");
+			Assert.AreEqual (Cursors.Default, c.Cursor, "C13");
 
-			// D
-			Assert.AreEqual(false, c.DataBindings == null, "D1");
+			Assert.IsNotNull(c.DataBindings, "D1");
 			Assert.AreEqual("Control", Control.DefaultBackColor.Name, "D2");
 			Assert.AreEqual("ControlText", Control.DefaultForeColor.Name, "D3");
 			Assert.AreEqual(FontStyle.Regular, Control.DefaultFont.Style, "D4");
 			Assert.AreEqual (new Rectangle(0, 0, 0, 0), c.DisplayRectangle , "D5");
-			Assert.AreEqual(false, c.Disposing, "D6");
+			Assert.IsFalse (c.Disposing, "D6");
 			Assert.AreEqual(DockStyle.None, c.Dock, "D7");
 
-			// E
-			Assert.AreEqual(true, c.Enabled, "E1");
+			Assert.IsTrue (c.Enabled, "E1");
 
-			// F
-			Assert.AreEqual(false, c.Focused, "F1");
-			Assert.AreEqual(FontStyle.Regular, c.Font.Style, "F2");
-			Assert.AreEqual(SystemColors.ControlText, c.ForeColor, "F3");
+			Assert.IsFalse  (c.Focused, "F1");
+			Assert.AreEqual (FontStyle.Regular, c.Font.Style, "F2");
+			Assert.AreEqual (SystemColors.ControlText, c.ForeColor, "F3");
 
-			// G
+			Assert.IsFalse  (c.HasChildren, "H2");
 
-			// H
-			Assert.AreEqual (((IWin32Window)c).Handle, c.Handle, "H1");
-			Assert.AreEqual(false, c.HasChildren, "H2");
-			Assert.AreEqual(0, c.Height, "H3");
-
-			// I
 			Assert.AreEqual (ImeMode.NoControl, c.ImeMode, "I1");
-			Assert.AreEqual(false, c.InvokeRequired, "I2");
-			Assert.AreEqual(false, c.IsAccessible, "I3");
-			Assert.AreEqual(false, c.IsDisposed, "I4");
-			Assert.AreEqual(true, c.IsHandleCreated, "I5");
+			Assert.IsFalse (c.InvokeRequired, "I2");
+			Assert.IsFalse (c.IsAccessible, "I3");
+			Assert.IsFalse (c.IsDisposed, "I4");
+			Assert.IsFalse (c.IsHandleCreated, "I5");
 
-			// J
-
-			// K
-
-			// L
-			Assert.AreEqual(0, c.Left, "L1");
 			Assert.AreEqual(Point.Empty, c.Location, "L2");
 
-			// M
 #if NET_2_0
 			Assert.IsTrue(c.MaximumSize.IsEmpty);
 			Assert.IsTrue(c.MinimumSize.IsEmpty);
 #endif
-			Assert.AreEqual(Keys.None, Control.ModifierKeys, "M1");
-			Assert.AreEqual(false, Control.MousePosition.IsEmpty, "M2");
-			Assert.AreEqual(MouseButtons.None, Control.MouseButtons, "M3");
+			Assert.AreEqual (Keys.None, Control.ModifierKeys, "M1");
+			Assert.IsFalse (Control.MousePosition.IsEmpty, "M2");
+			Assert.AreEqual (MouseButtons.None, Control.MouseButtons, "M3");
 
-			// N
 			Assert.AreEqual("", c.Name, "N1");
+			c.Name = "Control Name";
+			Assert.AreEqual("Control Name", c.Name, "N2");
 
-			// O
+			Assert.IsNull (c.Parent, "P1");
+			Assert.IsNotNull (c.ProductName, "P2");
+			Assert.IsTrue (c.ProductName != "", "P3");
+			Assert.IsNotNull (c.ProductVersion, "P4");
+			Assert.IsTrue (c.ProductVersion != "", "P5");
 
-			// P
-			Assert.AreEqual(null, c.Parent, "P1");
-			name = c.ProductName;
-			if (!name.Equals("Novell Mono MWF") && !name.Equals("Microsoft (R) .NET Framework"))
-				Assert.Fail("ProductName property does not match any accepted value - P2");
+			Assert.IsFalse (c.RecreatingHandle, "R1");
+			Assert.IsNull (c.Region, "R2");
+			Assert.AreEqual (RightToLeft.No, c.RightToLeft, "R4");
 
-			name = c.ProductVersion;
-			if (!name.Equals("1.1.4322.2032")) {
-				Assert.Fail("This test is being run against the wrong framework version.\nExpected is Net 1.1sp1. - P3");
-			}
+			Assert.IsNull (c.Site, "S1");
 
-			// R
-			Assert.AreEqual(false, c.RecreatingHandle, "R1");
-			Assert.AreEqual(null, c.Region, "R2");
-			Assert.AreEqual(0, c.Right, "R3");
-			Assert.AreEqual(RightToLeft.No, c.RightToLeft, "R4");
+			Assert.AreEqual (0, c.TabIndex , "T1");
+			Assert.IsTrue (c.TabStop, "T2");
+			Assert.IsNull (c.Tag, "T3");
+			Assert.AreEqual ("", c.Text, "T4");
 
-			// S
-			Assert.AreEqual(null, c.Site, "S1");
-			Assert.AreEqual (new Size(0, 0), c.Size, "S2");
+			Assert.IsTrue (c.Visible, "V1");
+		}
 
-			// T
-			Assert.AreEqual(0, c.TabIndex , "T1");
-			Assert.AreEqual(true, c.TabStop, "T2");
-			Assert.AreEqual(null, c.Tag, "T3");
-			Assert.AreEqual("", c.Text, "T4");
-			Assert.AreEqual(0, c.Top, "T5");
-			Assert.AreEqual(null, c.TopLevelControl, "T6");
+		[Test]
+		public void SizeChangeTest ()
+		{
+			Form f = new Form ();
+			Control c = new Control ();
+			f.Controls.Add(c);
+			f.Show();
+			c.Resize += new EventHandler(SizeChangedTest_ResizeHandler);
+			c.Tag = true;
+			c.Size = c.Size;
+			Assert.AreEqual (true, (bool) c.Tag, "#1");
+			f.Close ();
+		}
 
-			// U
+		private void SizeChangedTest_ResizeHandler (object sender, EventArgs e)
+		{
+			Console.WriteLine("Resized");
+			((Control) sender).Tag = false;
+		}
 
-			// V
-			Assert.AreEqual(true, c.Visible, "V1");
+		[Test]
+		public void NegativeHeightTest ()
+		{
+			Control c = new Control ();
+			IntPtr handle = c.Handle;
+			c.Resize += new EventHandler(NegativeHeightTest_ResizeHandler);
+			c.Tag = -2;
+			c.Height = 2;
+			c.Height = -2;
+			Assert.AreEqual (0, (int) c.Tag, "#1");
+			c.Dispose ();
+		}
+		
+		private void NegativeHeightTest_ResizeHandler (object sender, EventArgs e)
+		{
+			Control c = (Control) sender;
+			c.Tag = c.Height;
+		}
+		
+		[Test]
+		public void TopLevelControlTest () {
+			Control c = new Control ();
 
-			// W
-			Assert.AreEqual(0, c.Width, "W1");
+			Assert.AreEqual(null, c.TopLevelControl, "T1");
 
-			// XYZ
+			Panel p = new Panel ();
+
+			p.Controls.Add (c);
+
+			Assert.AreEqual(null, c.TopLevelControl, "T2");
+
+			Form f = new Form ();
+			f.ShowInTaskbar = false;
+
+			f.Controls.Add (p);
+
+			Assert.AreEqual (f, c.TopLevelControl, "T3");
+			Assert.AreEqual (f, f.TopLevelControl, "T4");
 		}
 
 		[Test]
@@ -185,19 +255,8 @@ namespace MWF.MonoTest
 			Assert.AreEqual(c1, c2.Parent, "Rel8");
 		}
 
-		private string TestControl(Control container, Control start, bool forward) {
-			Control	ctl;
-
-			ctl = container.GetNextControl(start, forward);
-
-			if (ctl == null) {
-				return null;
-			}
-
-			return ctl.Text;
-		}
-
 		[Test]
+		[Category ("NotWorking")]
 		public void TabOrder() {
 			Form		form;
 			Control		active;
@@ -225,8 +284,9 @@ namespace MWF.MonoTest
 			RadioButton	radio34 = new RadioButton();
 
 			form = new Form();
+			form.ShowInTaskbar = false;
 
-			form.ClientSize = new System.Drawing.Size (520, 520);
+			form.ClientSize = new Size (520, 520);
 			Assert.AreEqual(new Size(520, 520), form.ClientSize, "Tab1");
 
 			form.Text = "SWF Taborder Test App Form";
@@ -358,7 +418,7 @@ namespace MWF.MonoTest
 			group2.Controls.Add(group3);
 
 			// Perform some tests, the TabIndex stuff below will alter the outcome
-			Assert.AreEqual(null, TestControl(group2, radio34, true), "Tab8");
+			Assert.AreEqual(null, Helper.TestControl(group2, radio34, true), "Tab8");
 			Assert.AreEqual(31, group2.TabIndex, "Tab9");
 
 			// Does the taborder of containers and non-selectable things change behaviour?
@@ -368,23 +428,23 @@ namespace MWF.MonoTest
 			group2.TabIndex = 1;
 
 			// Start verification
-			Assert.AreEqual(null, TestControl(group2, radio34, true), "Tab10");
-			Assert.AreEqual(radio24.Text, TestControl(group2, group2, true), "Tab11");
-			Assert.AreEqual(radio31.Text, TestControl(group2, group3, true), "Tab12");
-			Assert.AreEqual(null, TestControl(group1, radio14, true), "Tab13");
-			Assert.AreEqual(radio23.Text, TestControl(group2, radio24, true), "Tab14");
-			Assert.AreEqual(group3.Text, TestControl(group2, radio21, true), "Tab15");
-			Assert.AreEqual(radio13.Text, TestControl(form, radio12, true), "Tab16");
-			Assert.AreEqual(label2.Text, TestControl(form, radio14, true), "Tab17");
-			Assert.AreEqual(group1.Text, TestControl(form, radio34, true), "Tab18");
-			Assert.AreEqual(radio23.Text, TestControl(group2, radio24, true), "Tab19");
+			Assert.AreEqual(null, Helper.TestControl(group2, radio34, true), "Tab10");
+			Assert.AreEqual(radio24.Text, Helper.TestControl(group2, group2, true), "Tab11");
+			Assert.AreEqual(radio31.Text, Helper.TestControl(group2, group3, true), "Tab12");
+			Assert.AreEqual(null, Helper.TestControl(group1, radio14, true), "Tab13");
+			Assert.AreEqual(radio23.Text, Helper.TestControl(group2, radio24, true), "Tab14");
+			Assert.AreEqual(group3.Text, Helper.TestControl(group2, radio21, true), "Tab15");
+			Assert.AreEqual(radio13.Text, Helper.TestControl(form, radio12, true), "Tab16");
+			Assert.AreEqual(label2.Text, Helper.TestControl(form, radio14, true), "Tab17");
+			Assert.AreEqual(group1.Text, Helper.TestControl(form, radio34, true), "Tab18");
+			Assert.AreEqual(radio23.Text, Helper.TestControl(group2, radio24, true), "Tab19");
 
 			// Sanity checks
-			Assert.AreEqual(null, TestControl(radio11, radio21, true), "Tab20");
-			Assert.AreEqual(text1.Text, TestControl(group1, radio21, true), "Tab21");
+			Assert.AreEqual(null, Helper.TestControl(radio11, radio21, true), "Tab20");
+			Assert.AreEqual(text1.Text, Helper.TestControl(group1, radio21, true), "Tab21");
 
-			Assert.AreEqual(radio14.Text, TestControl(form, label2, false), "Tab22");
-			Assert.AreEqual(radio21.Text, TestControl(group2, group3, false), "Tab23");
+			Assert.AreEqual(radio14.Text, Helper.TestControl(form, label2, false), "Tab22");
+			Assert.AreEqual(radio21.Text, Helper.TestControl(group2, group3, false), "Tab23");
 
 			Assert.AreEqual(4, radio21.TabIndex, "Tab24");
 			Assert.AreEqual(1, radio11.TabIndex, "Tab25");
@@ -392,12 +452,13 @@ namespace MWF.MonoTest
 			Assert.AreEqual(35, group3.TabIndex, "Tab27");
 			Assert.AreEqual(1, group2.TabIndex, "Tab28");
 
-			Assert.AreEqual(label1.Text, TestControl(form, form, false), "Tab29");
-			Assert.AreEqual(radio14.Text, TestControl(group1, group1, false), "Tab30");
-			Assert.AreEqual(radio34.Text, TestControl(group3, group3, false), "Tab31");
+			Assert.AreEqual(label1.Text, Helper.TestControl(form, form, false), "Tab29");
+			Assert.AreEqual(radio14.Text, Helper.TestControl(group1, group1, false), "Tab30");
+			Assert.AreEqual(radio34.Text, Helper.TestControl(group3, group3, false), "Tab31");
 
-			Assert.AreEqual(null, TestControl(label1, label1, false), "Tab31");
-			Assert.AreEqual(null, TestControl(radio11, radio21, false), "Tab32");
+			Assert.AreEqual(null, Helper.TestControl(label1, label1, false), "Tab31");
+			Assert.AreEqual(null, Helper.TestControl(radio11, radio21, false), "Tab32");
+			form.Dispose ();
 		}
 
 		[Test]
@@ -445,10 +506,10 @@ namespace MWF.MonoTest
 			//r1.DataBindings = System.Windows.Forms.Binding ;
 			//r1.ResetBindings() ;
 			//Assert.AreEqual(ControlBindingsCollection , r1.DataBindings  , "#83");
-			r1.BackColor = System.Drawing.Color.Black ;
+			r1.BackColor = Color.Black ;
 			r1.ResetBackColor() ;
 			Assert.AreEqual( SystemColors.Control , r1.BackColor  , "#84");
-			r1.BackColor = System.Drawing.Color.Black ;
+			r1.BackColor = Color.Black ;
 			r1.Refresh() ;
 			Assert.AreEqual( null , r1.Region , "#85");
 			Rectangle M = new Rectangle(10, 20, 30 ,40);
@@ -540,6 +601,48 @@ namespace MWF.MonoTest
 			}
 		}
 
+		[Test]
+		[Category ("NotWorking")]
+		public void CreateHandleTest2 ()
+		{
+			// This should eventually test all operations
+			// that can be performed on a control (within
+			// reason)
+			Control c = new Control ();
+
+			Assert.IsFalse (c.IsHandleCreated, "0");
+
+			c.Width = 100;
+			Assert.IsFalse (c.IsHandleCreated, "1");
+
+			c.Height = 100;
+			Assert.IsFalse (c.IsHandleCreated, "2");
+
+			c.Name = "hi";
+			Assert.IsFalse (c.IsHandleCreated, "3");
+
+			c.Left = 5;
+			Assert.IsFalse (c.IsHandleCreated, "5");
+
+			c.Top = 5;
+			Assert.IsFalse (c.IsHandleCreated, "6");
+
+			c.Location = new Point (1,1);
+			Assert.IsFalse (c.IsHandleCreated, "7");
+
+			c.Region = new Region ();
+			Assert.IsFalse (c.IsHandleCreated, "8");
+
+			c.Size = new Size (100, 100);
+			Assert.IsFalse (c.IsHandleCreated, "9");
+
+			c.Text = "bye";
+			Assert.IsFalse (c.IsHandleCreated, "10");
+
+			c.Visible = !c.Visible;
+			Assert.IsFalse (c.IsHandleCreated, "11");
+		}
+
 		[Test]
 		public void CreateGraphicsTest ()
 		{
@@ -562,8 +665,12 @@ namespace MWF.MonoTest
 		bool delegateCalled = false;
 		public delegate void TestDelegate ();
 
+		public void delegate_call () {
+			delegateCalled = true;
+		}
+
 		[Test]
-		[ExpectedException(typeof(System.InvalidOperationException))]
+		[ExpectedException(typeof(InvalidOperationException))]
 		public void InvokeException1 () {
 			Control c = new Control ();
 			IAsyncResult result;
@@ -572,33 +679,11 @@ namespace MWF.MonoTest
 			c.EndInvoke (result);
 		}
 
-		[Test]
-		[Ignore("Seems to hang")]
-		public void InvokeTest () {
-			Control c = null;
-
-			try {
-				c = new Control ();
-				IAsyncResult result;
-
-				c.CreateControl ();
-				result = c.BeginInvoke (new TestDelegate (delegate_call));
-				c.EndInvoke (result);
-				Assert.AreEqual (true, delegateCalled, "Invoke1");
-			} finally {
-				if (c != null)
-					c.Dispose ();
-			}
-		}
-
-		public void delegate_call () {
-			delegateCalled = true;
-		}
-
 		[Test]
 		public void FindFormTest () {
 			Form f = new Form ();
 
+			f.ShowInTaskbar = false;
 			f.Name = "form";
 			Control c = null;
 
@@ -634,6 +719,7 @@ namespace MWF.MonoTest
 
 			try {
 				f = new Form ();
+				f.ShowInTaskbar = false;
 				f.Visible = true;
 				c = new Button ();
 				c.Visible = true;
@@ -793,7 +879,7 @@ namespace MWF.MonoTest
 		}
 
 		[Test]
-		[ExpectedException(typeof(System.ArgumentException))]
+		[ExpectedException(typeof(ArgumentException))]
 		public void TransparentBackgroundTest1() {
 			Control	c;
 
@@ -867,4 +953,71 @@ namespace MWF.MonoTest
 			Assert.AreEqual(3, EnabledCalledCount, "Child Enabled Event not properly fired");
 		}
 	}
+
+	[TestFixture]
+	public class ControlInvokeTest {
+		public delegate void TestDelegate ();
+
+		Form f;
+		Control c;
+		Thread control_t;
+		ApplicationContext control_context;
+		bool delegateCalled = false;
+
+		object m;
+
+		void CreateControl ()
+		{
+			f = new Form ();
+			f.ShowInTaskbar = false;
+			
+			c = new Control ();
+
+			f.Controls.Add (c);
+
+			Console.WriteLine ("f.Handle = {0}", f.Handle);
+			Console.WriteLine ("c.Handle = {0}", c.Handle);
+
+			control_context = new ApplicationContext (f);
+
+			Monitor.Enter (m);
+			Console.WriteLine ("pulsing");
+			Monitor.Pulse (m);
+			Monitor.Exit (m);
+			Console.WriteLine ("control thread running");
+			Application.Run (control_context);
+			c.Dispose ();
+		}
+
+		[Test]
+		public void InvokeTest ()
+		{
+			m = new object ();
+
+			control_t = new Thread(new ThreadStart(CreateControl));
+
+			Monitor.Enter (m);
+
+			control_t.Start ();
+
+			Console.WriteLine ("waiting on monitor");
+			Monitor.Wait (m);
+
+			Console.WriteLine ("making async call");
+
+			IAsyncResult result;
+			result = c.BeginInvoke (new TestDelegate (delegate_call));
+			c.EndInvoke (result);
+
+			Assert.AreEqual (true, delegateCalled, "Invoke1");
+		}
+
+		public void delegate_call () {
+			/* invoked on control_context's thread */
+			delegateCalled = true;
+			f.Dispose ();
+			Application.Exit ();
+		}
+		
+	}
 }
