@@ -96,6 +96,7 @@ namespace MonoTests.System.Windows.Forms {
 		public void ControlSelectNextFlatTest ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 
 			form.Controls.AddRange (flat_controls);
 			form.Show ();
@@ -127,12 +128,14 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.IsFalse (flat_controls [1].Focused, "A14");
 			Assert.IsFalse (flat_controls [2].Focused, "A15");
 			Assert.AreEqual (flat_controls [0], form.ActiveControl, "A16");
+			form.Dispose ();
 		}
 
 		[Test]
 		public void SelectNextControlNullTest ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 
 			form.Show ();
 			form.Controls.AddRange (flat_controls);
@@ -148,12 +151,14 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.IsFalse (flat_controls [1].Focused, "A6");
 			Assert.IsFalse (flat_controls [2].Focused, "A7");
 			Assert.AreEqual (flat_controls [0], form.ActiveControl, "A8");
+			form.Dispose ();
 		}
 
 		[Test]
 		public void SelectControlTest ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 
 			form.Show ();
 			form.Controls.AddRange (flat_controls);
@@ -166,24 +171,28 @@ namespace MonoTests.System.Windows.Forms {
 
 			flat_controls [0]._Select (true, true);
 			Assert.AreEqual (flat_controls [0], form.ActiveControl, "A3");
+			form.Dispose ();
 		}
 
 		[Test]
 		public void EnsureDirectedSelectUsed ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 
 			form.Show ();
 			form.Controls.AddRange (flat_controls);
 
 			form.SelectNextControl (null, true, false, false, false);
 			Assert.IsTrue (flat_controls [0].directed_select_called, "A1");
+			form.Dispose ();
 		}
 
 		[Test]
 		public void ContainerSelectDirectedForward ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			ContainerPoker cp = new ContainerPoker ("container-a");
 			
 			form.Show ();
@@ -205,12 +214,14 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.IsFalse (flat_controls [2].Focused, "A8");
 			Assert.AreEqual (flat_controls [0], cp.ActiveControl, "A9");
 			Assert.AreEqual (cp, form.ActiveControl, "A10");
+			form.Dispose ();
 		}
 
 		[Test]
 		public void ContainerSelectDirectedBackward ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			ContainerPoker cp = new ContainerPoker ("container-a");
 			
 			form.Show ();
@@ -232,12 +243,15 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.IsTrue (flat_controls [2].Focused, "A8");
 			Assert.AreEqual (flat_controls [2], cp.ActiveControl, "A9");
 			Assert.AreEqual (cp, form.ActiveControl, "A10");
+			form.Dispose ();
 		}
 
 		[Test]
+		[Category ("NotWorking")]
 		public void ContainerSelectUndirectedForward ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			ContainerPoker cp = new ContainerPoker ("container-a");
 			
 			form.Show ();
@@ -251,12 +265,14 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.IsFalse (flat_controls [2].Focused, "A3");
 			Assert.AreEqual (null, cp.ActiveControl, "A4");
 			Assert.AreEqual (cp, form.ActiveControl, "A5");
+			form.Dispose ();
 		}
 
 		[Test]
 		public void GetNextControlFromForm ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			ContainerPoker con_a = new ContainerPoker ("container-a");
 			ContainerPoker con_b = new ContainerPoker ("container-b");
 			ContainerPoker con_c = new ContainerPoker ("container-c");
@@ -281,6 +297,9 @@ namespace MonoTests.System.Windows.Forms {
 			form.Show ();
 
 			// top level movement, 
+			Assert.AreEqual (form.GetNextControl (null, true), con_a, "null-1");
+			Assert.AreEqual (form.GetNextControl (null, false), con_c, "null-2");
+
 			Assert.AreEqual (form.GetNextControl (form, true), con_a, "form-1");
 			Assert.AreEqual (form.GetNextControl (form, false), con_c, "form-2");
 			
@@ -311,12 +330,14 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.AreEqual (form.GetNextControl (ctrls_c [1], false), ctrls_c [0], "ctrls-c-4");
 			Assert.AreEqual (form.GetNextControl (ctrls_c [2], true), null, "ctrls-c-5");
 			Assert.AreEqual (form.GetNextControl (ctrls_c [2], false), ctrls_c [1], "ctrls-c-6");
+			form.Dispose ();
 		}
 
 		[Test]
 		public void GetNextControlFromContainerA ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			ContainerPoker con_a = new ContainerPoker ("container-a");
 			ContainerPoker con_b = new ContainerPoker ("container-b");
 			ContainerPoker con_c = new ContainerPoker ("container-c");
@@ -341,6 +362,9 @@ namespace MonoTests.System.Windows.Forms {
 			form.Show ();
 
 			// top level movement, 
+			Assert.AreEqual (con_a.GetNextControl (null, true), ctrls_a [0], "null-1");
+			Assert.AreEqual (con_a.GetNextControl (null, false), ctrls_a [2], "null-2");
+
 			Assert.AreEqual (con_a.GetNextControl (form, true), ctrls_a [0], "form-1");
 			Assert.AreEqual (con_a.GetNextControl (form, false), ctrls_a [2], "form-2");
 			
@@ -371,12 +395,14 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.AreEqual (con_a.GetNextControl (ctrls_c [1], false), ctrls_a [2], "ctrls-c-4");
 			Assert.AreEqual (con_a.GetNextControl (ctrls_c [2], true), ctrls_a [0], "ctrls-c-5");
 			Assert.AreEqual (con_a.GetNextControl (ctrls_c [2], false), ctrls_a [2], "ctrls-c-6");
+			form.Dispose ();
 		}
 
 		[Test]
 		public void GetNextControlFromContainerB ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			ContainerPoker con_a = new ContainerPoker ("container-a");
 			ContainerPoker con_b = new ContainerPoker ("container-b");
 			ContainerPoker con_c = new ContainerPoker ("container-c");
@@ -401,6 +427,9 @@ namespace MonoTests.System.Windows.Forms {
 			form.Show ();
 
 			// top level movement
+			Assert.AreEqual (con_b.GetNextControl (null, true), ctrls_b [0], "null-1");
+			Assert.AreEqual (con_b.GetNextControl (null, false), ctrls_b [2], "null-2");
+
 			Assert.AreEqual (con_b.GetNextControl (form, true), ctrls_b [0], "form-1");
 			Assert.AreEqual (con_b.GetNextControl (form, false), ctrls_b [2], "form-2");
 			
@@ -431,12 +460,14 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.AreEqual (con_b.GetNextControl (ctrls_c [1], false), ctrls_b [2], "ctrls-c-4");
 			Assert.AreEqual (con_b.GetNextControl (ctrls_c [2], true), ctrls_b [0], "ctrls-c-5");
 			Assert.AreEqual (con_b.GetNextControl (ctrls_c [2], false), ctrls_b [2], "ctrls-c-6");
+			form.Dispose ();
 		}
 
 		[Test]
 		public void GetNextControlFromContainerC ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			ContainerPoker con_a = new ContainerPoker ("container-a");
 			ContainerPoker con_b = new ContainerPoker ("container-b");
 			ContainerPoker con_c = new ContainerPoker ("container-c");
@@ -461,6 +492,9 @@ namespace MonoTests.System.Windows.Forms {
 			form.Show ();
 
 			// top level movement, 
+			Assert.AreEqual (con_c.GetNextControl (null, true), ctrls_c [0], "null-1");
+			Assert.AreEqual (con_c.GetNextControl (null, false), ctrls_c [2], "null-2");
+
 			Assert.AreEqual (con_c.GetNextControl (form, true), ctrls_c [0], "form-1");
 			Assert.AreEqual (con_c.GetNextControl (form, false), ctrls_c [2], "form-2");
 			
@@ -491,12 +525,14 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.AreEqual (con_c.GetNextControl (ctrls_c [1], false), ctrls_c [0], "ctrls-c-4");
 			Assert.AreEqual (con_c.GetNextControl (ctrls_c [2], true), null, "ctrls-c-5");
 			Assert.AreEqual (con_c.GetNextControl (ctrls_c [2], false), ctrls_c [1], "ctrls-c-6");
+			form.Dispose ();
 		}
 
 		[Test]
 		public void GetNextControl2FromForm ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			ContainerPoker con_a = new ContainerPoker ("container-a");
 			ContainerPoker con_b = new ContainerPoker ("container-b");
 			ContainerPoker con_c = new ContainerPoker ("container-c");
@@ -513,6 +549,9 @@ namespace MonoTests.System.Windows.Forms {
 			form.Show ();
 
 			// top level movement, 
+			Assert.AreEqual (form.GetNextControl (null, true), con_a, "null-1");
+			Assert.AreEqual (form.GetNextControl (null, false), ctrl_b, "null-2");
+
 			Assert.AreEqual (form.GetNextControl (form, true), con_a, "form-1");
 			Assert.AreEqual (form.GetNextControl (form, false), ctrl_b, "form-2");
 
@@ -528,13 +567,14 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.AreEqual (form.GetNextControl (ctrls_a [1], false), ctrls_a [0], "ctrl-a-2");
 			Assert.AreEqual (form.GetNextControl (ctrls_a [2], true), ctrl_b, "ctrl-a-1");
 			Assert.AreEqual (form.GetNextControl (ctrls_a [2], false), ctrls_a [1], "ctrl-a-2");
-
+			form.Dispose();
 		}
 
 		[Test]
 		public void GetNextControlFlat ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 
 			form.Controls.AddRange (flat_controls);
 			form.Show ();
@@ -557,12 +597,14 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.AreEqual (flat_controls [0].GetNextControl (flat_controls [1], false), null, "ctrls-0-6");
 			Assert.AreEqual (flat_controls [0].GetNextControl (flat_controls [2], true), null, "ctrls-0-7");
 			Assert.AreEqual (flat_controls [0].GetNextControl (flat_controls [2], false), null,"ctrls-0-8");
+			form.Dispose ();
 		}
 
 		[Test]
 		public void GetNextGroupBoxControlFlat ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			GroupBoxPoker gbp = new GroupBoxPoker ("group-box");
 
 			gbp.Controls.AddRange (flat_controls);
@@ -579,12 +621,28 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.AreEqual (gbp.GetNextControl (null, false), flat_controls [2], "gb-4");
 			Assert.AreEqual (gbp.GetNextControl (gbp, true), flat_controls [0], "gb-5");
 			Assert.AreEqual (gbp.GetNextControl (gbp, false), flat_controls [2], "gb-6");
+
+			Assert.AreEqual (form.GetNextControl (flat_controls [0], true), flat_controls [1], "form-ctrls-0-forward");
+			Assert.AreEqual (form.GetNextControl (flat_controls [0], false), gbp, "form-ctrls-0-backward");
+			Assert.AreEqual (form.GetNextControl (flat_controls [1], true), flat_controls [2], "form-ctrls-1-forward");
+			Assert.AreEqual (form.GetNextControl (flat_controls [1], false), flat_controls [0], "form-ctrls-1-backward");
+			Assert.AreEqual (form.GetNextControl (flat_controls [2], true), null, "form-ctrls-2-forward");
+			Assert.AreEqual (form.GetNextControl (flat_controls [2], false), flat_controls [1],"form-ctrls-2-backward");
+
+			Assert.AreEqual (gbp.GetNextControl (flat_controls [0], true), flat_controls [1], "gbp-ctrls-0-forward");
+			Assert.AreEqual (gbp.GetNextControl (flat_controls [0], false), null, "gbp-ctrls-0-backward");
+			Assert.AreEqual (gbp.GetNextControl (flat_controls [1], true), flat_controls [2], "gbp-ctrls-1-forward");
+			Assert.AreEqual (gbp.GetNextControl (flat_controls [1], false), flat_controls [0], "gbp-ctrls-1-backward");
+			Assert.AreEqual (gbp.GetNextControl (flat_controls [2], true), null, "gbp-ctrls-2-forward");
+			Assert.AreEqual (gbp.GetNextControl (flat_controls [2], false), flat_controls [1],"gbp-ctrls-2-backward");		
+			form.Dispose ();
 		}
 
 		[Test]
 		public void GetNextControlFromTabControl ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			TabControl tab = new TabControl ();
 			TabPage page1 = new TabPage ("page one");
 			TabPage page2 = new TabPage ("page two");
@@ -606,12 +664,62 @@ namespace MonoTests.System.Windows.Forms {
 
 			Assert.AreEqual (form.GetNextControl (page2, true), null, "page-two-1");
 			Assert.AreEqual (form.GetNextControl (page2, false), page1, "page-two-2");
+			form.Dispose ();
+		}
+
+		[Test]
+		public void GetNextControlFromTabControl2 () {
+			Form form = new Form ();
+			form.ShowInTaskbar = false;
+			TabControl tab = new TabControl ();
+			
+			TabPage page1 = new TabPage ("page one");
+			page1.Controls.AddRange (flat_controls);
+
+			TabPage page2 = new TabPage ("page two");
+
+			tab.TabPages.Add (page1);
+
+			tab.TabPages.Add (page2);
+
+			form.Controls.Add (tab);
+			form.Show ();
+
+			Assert.AreEqual (form.GetNextControl (null, true), tab, "form-1");
+			Assert.AreEqual (form.GetNextControl (null, false), page2, "form-2");
+
+			Assert.AreEqual (form.GetNextControl (tab, true), page1, "tab-1");
+			Assert.AreEqual (form.GetNextControl (tab, false), null, "tab-2");
+
+			Assert.AreEqual (form.GetNextControl (page1, true), flat_controls [0], "page-one-1");
+			Assert.AreEqual (form.GetNextControl (page1, false), tab, "page-one-2");
+
+			Assert.AreEqual (form.GetNextControl (page2, true), null, "page-two-1");
+			Assert.AreEqual (form.GetNextControl (page2, false), flat_controls [2], "page-two-2");
+
+			Assert.AreEqual (form.GetNextControl (flat_controls [0], false), page1, "form-ctrls-0-backward");
+			Assert.AreEqual (form.GetNextControl (flat_controls [2], true), page2, "form-ctrls-2-forward");
+
+			Assert.AreEqual (tab.GetNextControl (null, true), page1, "tab-null-forward");
+			Assert.AreEqual (tab.GetNextControl (page1, false), null, "tab-page1-backward");
+
+			Assert.AreEqual (tab.GetNextControl (flat_controls [0], false), page1, "tab-ctrls-0-backward");
+			Assert.AreEqual (tab.GetNextControl (flat_controls [2], true), page2, "tab-ctrls-2-forward");
+
+			Assert.AreEqual (page1.GetNextControl (flat_controls [0], true), flat_controls [1], "page1-ctrls-0-forward");
+			Assert.AreEqual (page1.GetNextControl (flat_controls [0], false), null, "page1-ctrls-0-backward");
+			Assert.AreEqual (page1.GetNextControl (flat_controls [1], true), flat_controls [2], "page1-ctrls-1-forward");
+			Assert.AreEqual (page1.GetNextControl (flat_controls [1], false), flat_controls [0], "page1-ctrls-1-backward");
+			Assert.AreEqual (page1.GetNextControl (flat_controls [2], true), null, "page1-ctrls-2-forward");
+			Assert.AreEqual (page1.GetNextControl (flat_controls [2], false), flat_controls [1],"page1-ctrls-2-backward");
+			form.Dispose ();
 		}
 
 		[Test]
 		public void GetNextControlTabIndex ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			ControlPoker [] ctrls = new ControlPoker [5];
 
 			for (int i = 0; i < 5; i++) {
@@ -641,12 +749,14 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.AreEqual (form.GetNextControl (ctrls [4], true), null, "A11");
 			Assert.AreEqual (form.GetNextControl (ctrls [4], false), ctrls [3], "A12");
 
+			form.Dispose ();
 		}
 
 		[Test]
 		public void GetNextControlDuplicateTabIndex ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			ControlPoker [] ctrls = new ControlPoker [5];
 
 			for (int i = 0; i < 5; i++) {
@@ -678,12 +788,14 @@ namespace MonoTests.System.Windows.Forms {
 			Assert.AreEqual (form.GetNextControl (ctrls [4], true), null, "A11");
 			Assert.AreEqual (form.GetNextControl (ctrls [4], false), ctrls [3], "A12");
 
+			form.Dispose ();
 		}
 
 		[Test]
 		public void GetNextControlComposite ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 			ControlPoker a = new ControlPoker ("a");
 			ControlPoker b = new ControlPoker ("b");
 			ControlPoker c = new ControlPoker ("c");
@@ -696,12 +808,15 @@ namespace MonoTests.System.Windows.Forms {
 
 			Assert.AreEqual (form.GetNextControl (a, true), b, "form-1");
 			Assert.AreEqual (form.GetNextControl (a, false), null, "form-2");
+
+			form.Dispose ();
 		}
 
 		[Test]
 		public void FocusSetsActive ()
 		{
 			Form form = new Form ();
+			form.ShowInTaskbar = false;
 
 			form.Controls.AddRange (flat_controls);
 			form.Show ();
@@ -711,6 +826,8 @@ namespace MonoTests.System.Windows.Forms {
 			flat_controls [1].Focus ();
 
 			Assert.AreEqual (form.ActiveControl, flat_controls [1], "A2");
+
+			form.Dispose ();
 		}
 	}
 
