@@ -22,6 +22,7 @@ namespace MonoTests.System.Windows.Forms
 		public void GroupBoxPropertyTest ()
 		{
 			Form myform = new Form ();
+			myform.ShowInTaskbar = false;
 			GroupBox mygrpbox = new GroupBox ();
 			RadioButton myradiobutton1 = new RadioButton ();
 			RadioButton myradiobutton2 = new RadioButton ();
@@ -35,6 +36,7 @@ namespace MonoTests.System.Windows.Forms
 			Assert.AreEqual (FlatStyle.Flat, mygrpbox.FlatStyle, "#3");
 			mygrpbox.FlatStyle = FlatStyle.System;
 			Assert.AreEqual (FlatStyle.System, mygrpbox.FlatStyle, "#4");
+			myform.Dispose ();
 		}
 	}
 }
