@@ -23,6 +23,7 @@ namespace MonoTests.System.Windows.Forms
 		public void MonthCalendarPropertyTest ()
 		{
 			Form myfrm = new Form ();
+			myfrm.ShowInTaskbar = false;
 			MonthCalendar myMonthCal1 = new MonthCalendar ();
 			MonthCalendar myMonthCal2 = new MonthCalendar ();
 			myMonthCal1.Name = "MonthCendar";
@@ -75,6 +76,8 @@ namespace MonoTests.System.Windows.Forms
 			Assert.AreEqual (DateTime.Today, myMonthCal1.TodayDate, "#T3");
 			Assert.AreEqual (false, myMonthCal1.TodayDateSet, "#T4");
 			Assert.AreEqual ("GrayText", myMonthCal1.TrailingForeColor.Name, "#T5");
+
+			myfrm.Dispose ();
 		}
 	
 		[Test]		
@@ -115,36 +118,43 @@ namespace MonoTests.System.Windows.Forms
 		public void AddAnnuallyBoldedDateTest ()
 		{
 			Form myForm = new Form ();
+			myForm.ShowInTaskbar = false;
 			MonthCalendar myMonthCal = new MonthCalendar ();
 			myMonthCal.AddAnnuallyBoldedDate (new DateTime (2005, 09, 01));
 			myForm.Controls.Add (myMonthCal);
 			Assert.AreEqual (new DateTime (2005, 09, 01), myMonthCal.AnnuallyBoldedDates.GetValue (0), "#add1");
+			myForm.Dispose ();
 		}
 
 		[Test]
 		public void AddBoldedDateTest ()
 		{
 			Form myForm = new Form ();
+			myForm.ShowInTaskbar = false;
 			MonthCalendar myMonthCal = new MonthCalendar ();
 			myMonthCal.AddBoldedDate (new DateTime (2005, 09, 02));
 			myForm.Controls.Add (myMonthCal);
 			Assert.AreEqual (new DateTime (2005, 09, 02), myMonthCal.BoldedDates.GetValue (0), "#add2");
+			myForm.Dispose ();
 		}
 
 		[Test]
 		public void AddMonthlyBoldedDateTest ()
 		{
 			Form myForm = new Form ();
+			myForm.ShowInTaskbar = false;
 			MonthCalendar myMonthCal = new MonthCalendar ();
 			myMonthCal.AddMonthlyBoldedDate (new DateTime (2005, 09, 03));
 			myForm.Controls.Add (myMonthCal);
 			Assert.AreEqual (new DateTime (2005, 09, 03), myMonthCal.MonthlyBoldedDates.GetValue (0), "#add2");
+			myForm.Dispose ();
 		}
 		
 		[Test]
 		public void GetDispalyRangeTest ()
 		{
 			Form myForm = new Form ();
+			myForm.ShowInTaskbar = false;
 			MonthCalendar myMonthCal = new MonthCalendar ();
 			myForm.Controls.Add (myMonthCal);
 			SelectionRange mySelRange = new SelectionRange ();
@@ -152,12 +162,14 @@ namespace MonoTests.System.Windows.Forms
 			mySelRange.End = new DateTime (2005, 09, 30);
 			Assert.AreEqual (mySelRange.Start, myMonthCal.GetDisplayRange (true).Start, "#Get1");
 			Assert.AreEqual (mySelRange.End, myMonthCal.GetDisplayRange (true).End, "#Get22");
+			myForm.Dispose ();
 		}
 		
 		[Test]
 		public void HitTest ()
 		{
 			Form myForm = new Form ();
+			myForm.ShowInTaskbar = false;
 			MonthCalendar myMonthCal = new MonthCalendar ();
 			myForm.Controls.Add (myMonthCal);
 			Assert.AreEqual (new DateTime (01, 01, 01), myMonthCal.HitTest(10, 10).Time, "#Hit1");
