@@ -60,6 +60,7 @@ namespace MonoTests.System.Windows.Forms
 		public void PanelClickTest ()
 		{
 			Form myForm = new Form ();
+			myForm.ShowInTaskbar = false;
 			RadioButton rButton1 = new RadioButton ();
 			rButton1.Select ();
 			rButton1.Visible = true;
@@ -69,12 +70,14 @@ namespace MonoTests.System.Windows.Forms
 			myForm.Show ();
 			rButton1.PerformClick ();
 			Assert.AreEqual (true, eventhandled, "#2");
+			myForm.Dispose ();
 		}
 
 		[Test]
 		public void ApperanceChangedTest ()
 		{
 			Form myForm = new Form ();
+			myForm.ShowInTaskbar = false;
 			RadioButton rButton1 = new RadioButton ();
 			rButton1.Select ();
 			rButton1.Visible = true;
@@ -84,12 +87,14 @@ namespace MonoTests.System.Windows.Forms
 			rButton1.AppearanceChanged += new EventHandler (RadioButton_EventHandler);
 			rButton1.Appearance = Appearance.Button;
 			Assert.AreEqual (true, eventhandled, "#2");
+			myForm.Dispose ();
 		}
 	
 		[Test]
 		public void CheckedChangedTest ()
 		{
 			Form myForm = new Form ();
+			myForm.ShowInTaskbar = false;
 			RadioButton rButton1 = new RadioButton ();
 			rButton1.Select ();
 			rButton1.Visible = true;
@@ -99,6 +104,7 @@ namespace MonoTests.System.Windows.Forms
 			rButton1.CheckedChanged += new EventHandler (RadioButton_EventHandler);
 			rButton1.Checked = true;
 			Assert.AreEqual (true, eventhandled, "#3");
+			myForm.Dispose ();
 		}
 	}
 }
