@@ -274,7 +274,7 @@ public class MyScrollBar : HScrollBar
 	     }
     }
    [TestFixture]
-   public class ScrollbarTest
+   public class ScrollBarTest
     {
 	    [Test]
 	    public void PubPropTest ()
@@ -351,12 +351,12 @@ public class MyScrollBar : HScrollBar
 		       myscrlbar.LargeChange = -1;
 	       }
 
-	    [Test, Ignore ("Look into this")]
+	    [Test]
 	    public void PubMethodTest ()
 	       {
 		       MyScrollBar myscrlbar = new MyScrollBar ();
 		       myscrlbar.Text = "New HScrollBar";
-		       Assert.AreEqual ("System.Windows.Forms.HScrollBar, Minimum: 0, Maximum: 100, Value: 0",
+		       Assert.AreEqual ("MonoTests.System.Windows.Forms.MyScrollBar, Minimum: 0, Maximum: 100, Value: 0",
 					myscrlbar.ToString (), "T5");
 	       }
     }
@@ -389,6 +389,7 @@ public class MyScrollBar : HScrollBar
 	    public void BackColorChangedTest ()
 	       {
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       ScrollBar myHscrlbar = new HScrollBar ();
 		       myform.Controls.Add (myHscrlbar);
@@ -396,12 +397,14 @@ public class MyScrollBar : HScrollBar
 		       myHscrlbar.BackColor = Color.Red;
 		       Assert.AreEqual (true, eventhandled, "B4");
 		       eventhandled = false;
+		       myform.Dispose ();
 	       }
 
 	    [Test]
 	    public void BackgroundImageChangedTest ()
 	       {
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       ScrollBar myHscrlbar = new HScrollBar ();
 		       myform.Controls.Add (myHscrlbar);
@@ -409,38 +412,14 @@ public class MyScrollBar : HScrollBar
 		       myHscrlbar.BackgroundImage = Image.FromFile ("Test/System.Windows.Forms/bitmaps/a.png");
 		       Assert.AreEqual (true, eventhandled, "B5");
 		       eventhandled = false;
-	       }
-
-	    [Test, Ignore ("Need to send proper Click / DoubleClick")]
-	    public void ClickTest ()
-	       {
-		       Form myform = new Form ();
-		       myform.Visible = true;
-		       MyScrollBar myHscrlbar = new MyScrollBar ();
-		       myform.Controls.Add (myHscrlbar);
-		       myHscrlbar.Click += new EventHandler (ScrollBar_EventHandler);
-		       myHscrlbar.MouseClick ();
-
-		       Assert.AreEqual (false, eventhandled, "C1");
-	       }
-
-	    [Test, Ignore ("Need to send proper Click / DoubleClick")]
-	    public void DoubleClickTest ()
-	       {
-		       Form myform = new Form ();
-		       myform.Visible = true;
-		       MyScrollBar myHscrlbar = new MyScrollBar ();
-		       myform.Controls.Add (myHscrlbar);
-		       myHscrlbar.DoubleClick += new EventHandler (ScrollBar_EventHandler);
-		       myHscrlbar.MouseDoubleClick ();
-
-		       Assert.AreEqual (false, eventhandled, "D1");
+		       myform.Dispose ();
 	       }
 
 	    [Test]
 	    public void FontChangedTest ()
 	       {
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       ScrollBar myHscrlbar = new HScrollBar ();
 		       myform.Controls.Add (myHscrlbar);
@@ -450,12 +429,14 @@ public class MyScrollBar : HScrollBar
 		       myHscrlbar.Font = myFontDialog.Font;
 		       Assert.AreEqual (true, eventhandled, "F2");
 		       eventhandled = false;
+		       myform.Dispose ();
 	       }
 
 	    [Test]
 	    public void ForeColorChangedTest ()
 	       {
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       ScrollBar myHscrlbar = new HScrollBar ();
 		       myform.Controls.Add (myHscrlbar);
@@ -463,12 +444,14 @@ public class MyScrollBar : HScrollBar
 		       myHscrlbar.ForeColor = Color.Azure;
 		       Assert.AreEqual (true, eventhandled, "F3");
 		       eventhandled = false;
+		       myform.Dispose ();
 	       }
 
 	    [Test]
 	    public void ImeModehangedTest ()
 	       {
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       ScrollBar myHscrlbar = new HScrollBar ();
 		       myform.Controls.Add (myHscrlbar);
@@ -476,12 +459,14 @@ public class MyScrollBar : HScrollBar
 		       myHscrlbar.ImeMode = ImeMode.Katakana;
 		       Assert.AreEqual (true, eventhandled, "I2");
 		       eventhandled = false;
+		       myform.Dispose ();
 	       }
 
 	    [Test]
 	    public void MouseDownTest ()
 	       {
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar myHscrlbar = new MyScrollBar ();
 		       myform.Controls.Add (myHscrlbar);
@@ -490,12 +475,14 @@ public class MyScrollBar : HScrollBar
 
 		       Assert.AreEqual (true, eventhandled, "M5");
 		       eventhandled = false;
+		       myform.Dispose ();
 	       }
 
 	    [Test]
 	    public void MouseMoveTest ()
 	       {
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar myHscrlbar = new MyScrollBar ();
 		       myform.Controls.Add (myHscrlbar);
@@ -504,12 +491,14 @@ public class MyScrollBar : HScrollBar
 
 		       Assert.AreEqual (true, eventhandled, "M6");
 		       eventhandled = false;
+		       myform.Dispose ();
 	       }
 
 	    [Test]
 	    public void MouseUpTest ()
 	       {
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar myHscrlbar = new MyScrollBar ();
 		       myform.Controls.Add (myHscrlbar);
@@ -518,27 +507,15 @@ public class MyScrollBar : HScrollBar
 
 		       Assert.AreEqual (true, eventhandled, "M7");
 		       eventhandled = false;
-	       }
-
-	    [Test, Ignore ("How can we force it to pain?")]
-	    public void PaintTest ()
-	       {
-		       Form myform = new Form ();
-		       myform.Visible = true;
-		       MyScrollBar myHscrlbar = new MyScrollBar ();
-		       myform.Controls.Add (myHscrlbar);
-		       myHscrlbar.Paint += new PaintEventHandler (ScrollBarPaint_EventHandler);
-
-		       myHscrlbar.Refresh ();
-
-		       Assert.AreEqual (true, eventhandled, "P1");
-		       eventhandled = false;
+		       myform.Dispose ();
 	       }
 
 	    [Test]
+	    [Category ("NotWorking")]
 	    public void ScrollTest ()
 	       {
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar myHscrlbar = new MyScrollBar ();
 		       myform.Controls.Add (myHscrlbar);
@@ -547,12 +524,14 @@ public class MyScrollBar : HScrollBar
 
 		       Assert.AreEqual (true, eventhandled, "S4");
 		       eventhandled = false;
+		       myform.Dispose ();
 	       }
 
 	    [Test]
 	    public void TextChangedTest ()
 	       {
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar myHscrlbar = new MyScrollBar ();
 		       myform.Controls.Add (myHscrlbar);
@@ -561,12 +540,14 @@ public class MyScrollBar : HScrollBar
 
 		       Assert.AreEqual (true, eventhandled, "T6");
 		       eventhandled = false;
+		       myform.Dispose ();
 	       }
 
 	    [Test]
 	    public void ValueChangeTest ()
 	       {
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar myHscrlbar = new MyScrollBar ();
 		       myform.Controls.Add (myHscrlbar);
@@ -575,6 +556,7 @@ public class MyScrollBar : HScrollBar
 		       myHscrlbar.Value = 50 ;
 		       Assert.AreEqual (true, eventhandled, "V3");
 		       eventhandled = false;
+		       myform.Dispose ();
 	       }
     }
 
@@ -666,11 +648,13 @@ public class MyVScrollBar : VScrollBar
 				 "OnBindingContextChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -684,12 +668,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnInvalidated"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.BackColor = Color.Aqua;
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -703,12 +689,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnInvalidated"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.BackgroundImage = Image.FromFile ("Test/System.Windows.Forms/bitmaps/a.png");
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test, Ignore ("Need to send proper Click / DoubleClick")]
@@ -720,12 +708,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnBindingContextChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.MouseClick ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test, Ignore ("Need to send proper Click / DoubleClick")]
@@ -737,12 +727,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnBindingContextChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.MouseDoubleClick ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -754,6 +746,7 @@ public class MyVScrollBar : VScrollBar
 				 "OnBindingContextChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
@@ -761,6 +754,7 @@ public class MyVScrollBar : VScrollBar
 		       s.Font = myFontDialog.Font;
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -774,12 +768,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnInvalidated"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.ForeColor = Color.Aqua;
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -792,12 +788,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnImeModeChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.ImeMode = ImeMode.Katakana;
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -810,12 +808,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnMouseDown"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.MouseRightDown ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -829,12 +829,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnMouseHover"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.MoveMouse ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -847,12 +849,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnMouseUp"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.MouseRightUp ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -865,12 +869,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnInvalidated"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.Visible = true;
 		       s.Refresh ();
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -884,12 +890,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnValueChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.ScrollNow ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -902,12 +910,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnTextChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.Text = "foobar";
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -920,12 +930,14 @@ public class MyVScrollBar : VScrollBar
 				 "OnValueChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar s = new MyScrollBar ();
 		       myform.Controls.Add (s);
 		       s.Value = 10;
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
     }
 
@@ -1191,11 +1203,13 @@ public class MyScrollBar2 : HScrollBar
 				 "BindingContextChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -1209,12 +1223,14 @@ public class MyScrollBar2 : HScrollBar
 				 "BackColorChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 		       s.BackColor = Color.Aqua;
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -1229,12 +1245,14 @@ public class MyScrollBar2 : HScrollBar
 
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 		       s.BackgroundImage = Image.FromFile ("Test/System.Windows.Forms/bitmaps/a.png");
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test, Ignore ("Need to send proper Click / DoubleClick")]
@@ -1246,12 +1264,14 @@ public class MyScrollBar2 : HScrollBar
 				 "BindingContextChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 		       s.MouseClick ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test, Ignore ("Need to send proper Click / DoubleClick")]
@@ -1263,12 +1283,14 @@ public class MyScrollBar2 : HScrollBar
 				 "BindingContextChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 		       s.MouseDoubleClick ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -1280,6 +1302,7 @@ public class MyScrollBar2 : HScrollBar
 				 "BindingContextChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
@@ -1287,6 +1310,7 @@ public class MyScrollBar2 : HScrollBar
 		       s.Font = myFontDialog.Font;
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -1300,12 +1324,14 @@ public class MyScrollBar2 : HScrollBar
 				 "ForeColorChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 		       s.ForeColor = Color.Aqua;
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -1318,12 +1344,14 @@ public class MyScrollBar2 : HScrollBar
 				 "ImeModeChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 		       s.ImeMode = ImeMode.Katakana;
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -1336,12 +1364,14 @@ public class MyScrollBar2 : HScrollBar
 				 "MouseDown"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 		       s.MouseRightDown ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -1354,12 +1384,14 @@ public class MyScrollBar2 : HScrollBar
 				 "MouseMove"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 		       s.MoveMouse ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -1372,12 +1404,14 @@ public class MyScrollBar2 : HScrollBar
 				 "MouseUp"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 		       s.MouseRightUp ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -1390,6 +1424,7 @@ public class MyScrollBar2 : HScrollBar
 				 "Invalidated"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
@@ -1397,6 +1432,7 @@ public class MyScrollBar2 : HScrollBar
 		       s.Refresh ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -1410,12 +1446,14 @@ public class MyScrollBar2 : HScrollBar
 				 "ValueChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 		       s.ScrollNow ();
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -1428,12 +1466,14 @@ public class MyScrollBar2 : HScrollBar
 				 "TextChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 		       s.Text = "foobar";
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
 
 	    [Test]
@@ -1446,12 +1486,14 @@ public class MyScrollBar2 : HScrollBar
 				 "ValueChanged"
 		       };
 		       Form myform = new Form ();
+		       myform.ShowInTaskbar = false;
 		       myform.Visible = true;
 		       MyScrollBar2 s = new MyScrollBar2 ();
 		       myform.Controls.Add (s);
 		       s.Value = 10;
 
 		       Assert.AreEqual (EventsWanted, ArrayListToString (s.Results));
+		       myform.Dispose ();
 	       }
     }
 
