@@ -55,7 +55,7 @@ namespace Microsoft.Build.BuildEngine {
 		{
 			// FIXME: change this to ITaskItem instead of string
 		
-			Expression inputExpr, outputExpr;
+			OldExpression inputExpr, outputExpr;
 			string[] inputFiles, outputFiles;
 			DateTime oldestInput, youngestOutput;
 		
@@ -66,11 +66,13 @@ namespace Microsoft.Build.BuildEngine {
 				return true;
 			}
 			
-			inputExpr = new Expression (project, inputs);
-			outputExpr = new Expression (project, outputs);
+			inputExpr = new OldExpression (project);
+			inputExpr.ParseSource (inputs);
+			outputExpr = new OldExpression (project);
+			outputExpr.ParseSource (outputs);
 			
-			inputFiles = (string[]) inputExpr.ToArray (typeof (string[]));
-			outputFiles = (string[]) outputExpr.ToArray (typeof (string[]));
+			inputFiles = (string[]) inputExpr.ConvertTo (typeof (string[]));
+			outputFiles = (string[]) outputExpr.ConvertTo (typeof (string[]));
 			
 			if (inputFiles == null) {
 				return true;
@@ -127,11 +129,22 @@ namespace Microsoft.Build.BuildEngine {
 			}
 		}
 		
+		// FIXME: should do everything from task batching specification, not just run task once
 		public bool BatchBuildTask (BuildTask buildTask)
 		{
-			return buildTask.Execute ();
+			if (buildTask.Condition == String.Empty)
+				return buildTask.Execute ();
+			
+			ConditionExpression ce = ConditionParser.ParseCondition (buildTask.Condition);
+			
+			if (ce.BoolEvaluate (project))
+				return buildTask.Execute ();
+			else
+			// FIXME: skipped, it should be logged
+				return true;
+			
 		}
 	}
 }
 
-#endif
\ No newline at end of file
+#endif
