@@ -47,7 +47,7 @@ namespace Microsoft.Build.BuildEngine {
 		BuildPropertyGroup	globalProperties;
 		//IDictionary		importedProjects;
 		List <ILogger>		loggers;
-		bool			onlyLogCriticalEvents;
+		//bool			onlyLogCriticalEvents;
 		Dictionary <string, Project>	projects;
 
 		static Engine		globalEngine;
@@ -172,9 +172,12 @@ namespace Microsoft.Build.BuildEngine {
 				project = (Project) projects [projectFile];
 				LogProjectStarted (project, targetNames);
 				result = project.Build (targetNames, targetOutputs);
+			} else {
+				project = CreateNewProject ();
+				project.Load (projectFile);
+				LogProjectStarted (project, targetNames);
+				result = project.Build (targetNames, targetOutputs);
 			}
-			else
-				return false;
 			
 			LogProjectFinished (project, result);
 			
@@ -204,7 +207,7 @@ namespace Microsoft.Build.BuildEngine {
 			if (projectFullFileName == null)
 				throw new ArgumentNullException ("projectFullFileName");
 			
-			return (Project) projects [projectFullFileName];
+			return projects [projectFullFileName];
 		}
 
 		internal void RemoveLoadedProject (Project p)
@@ -235,7 +238,7 @@ namespace Microsoft.Build.BuildEngine {
 		public void UnloadAllProjects ()
 		{
 			foreach (KeyValuePair <string, Project> e in projects)
-				UnloadProject ((Project) e.Value);
+				UnloadProject (e.Value);
 		}
 
 		[MonoTODO]
