@@ -35,29 +35,36 @@ using Microsoft.Build.Utilities;
 namespace Microsoft.Build.BuildEngine {
 	internal class ItemReference {
 	
-		string		itemName;
-		string		separator;
-		Expression	parent;
-		Expression	transform;
-	
-		public ItemReference (Expression parent)
-		{
-			if (parent == null)
-				throw new Exception ("Parent Expression needed to find project.");
-			this.parent = parent;
-			this.itemName = null;
-			this.separator = ";";
-		}
+		string			itemName;
+		OldExpression		separator;
+		OldExpression		transform;
+		Project			parentProject;
 		
-		public ItemReference (Expression parent, string source)
-			: this (parent)
+		/*
+		public ItemReference (Project parentProject, string itemName,
+				      LiteralExpression transform,
+				      LiteralExpression separator)
 		{
-			ParseSource (source);
+			this.parentProject = parentProject;
+			this.itemName = itemName;
+			this.transform = transform;
+			this.separator = separator;
+		}
+		*/
+	
+		public ItemReference (Project parentProject)
+		{
+			if (parentProject == null)
+				throw new ArgumentNullException ("Parent Project needed.");
+			this.parentProject = parentProject;
+			this.itemName = null;
+			this.separator = null;
 		}
 		
 		public void ParseSource (string source)
 		{
 			string sourceWithoutParens;
+			string separatorString = null;
 			ApostropheState aState = ApostropheState.Out;
 			ItemParsingState iState = ItemParsingState.Name;
 			int c = -1;
@@ -103,44 +110,50 @@ namespace Microsoft.Build.BuildEngine {
 			itemName = sourceWithoutParens.Substring (0, itemNameEnd + 1);
 			if (transformEnd != -1) {
 				if (separatorStart != -1) {
-					separator = sourceWithoutParens.Substring (separatorStart + 1, sourceWithoutParens.Length
+					separatorString = sourceWithoutParens.Substring (separatorStart + 1, sourceWithoutParens.Length
 						- separatorStart - 2);
-					transform = new Expression (parent.Project, sourceWithoutParens.Substring (itemNameEnd + 4,
-					transformEnd - itemNameEnd - 4));
+					transform = new OldExpression (parentProject);
+					transform.ParseSource (sourceWithoutParens.Substring (itemNameEnd + 4,
+						transformEnd - itemNameEnd - 4));
 				} else {
-					transform = new Expression (parent.Project, sourceWithoutParens.Substring (itemNameEnd + 4,
-					sourceWithoutParens.Length - itemNameEnd - 5));
+					transform = new OldExpression (parentProject);
+					transform.ParseSource (sourceWithoutParens.Substring (itemNameEnd + 4,
+						sourceWithoutParens.Length - itemNameEnd - 5));
 				}
 			} else {
 				if (separatorStart != -1) {
-					separator = sourceWithoutParens.Substring (separatorStart + 1, sourceWithoutParens.Length
+					separatorString = sourceWithoutParens.Substring (separatorStart + 1, sourceWithoutParens.Length
 						- separatorStart - 2);
 				}
 			}
+			
+			if (separatorString != null) {
+				separator = new OldExpression (parentProject);
+				separator.ParseSource (separatorString);
+			}
 		}
 		
-		public ITaskItem[] ToITaskItemArray ()
+		public ITaskItem[] ConvertToITaskItemArray ()
 		{
 			if (itemName != String.Empty) {
-				Project p = parent.Project;
 				BuildItemGroup big;
-				if (p.EvaluatedItemsByName.Contains (itemName)) {
-					big = (BuildItemGroup)p.EvaluatedItemsByName [itemName];
-					return big.ToITaskItemArray (transform);
+				if (parentProject.EvaluatedItemsByName.Contains (itemName)) {
+					big = (BuildItemGroup)parentProject.EvaluatedItemsByName [itemName];
+					return big.ConvertToITaskItemArray (transform);
 				} else
 					return null;
 			} else
 				return null;
 		}
 		
-		public new string ToString ()
+		public string ConvertToString ()
 		{
 			if (itemName != String.Empty) {
-				Project p = parent.Project;
 				BuildItemGroup big;
-				if (p.EvaluatedItemsByName.Contains (itemName)) {
-					big = (BuildItemGroup)p.EvaluatedItemsByName [itemName];
-					return big.ToString (transform, separator);
+				if (parentProject.EvaluatedItemsByName.Contains (itemName)) {
+					big = (BuildItemGroup)parentProject.EvaluatedItemsByName [itemName];
+					
+					return big.ConvertToString (transform, separator);
 				} else
 					return String.Empty;
 			} else
@@ -150,10 +163,6 @@ namespace Microsoft.Build.BuildEngine {
 		public string ItemName {
 			get { return itemName; }
 		}
-		
-		public string Separator {
-			get { return separator; }
-		}
 	}
 }
 
