@@ -159,7 +159,7 @@ namespace Microsoft.Build.BuildEngine {
 		public bool Build (string targetName)
 		{
 			if (targets.Exists (targetName) == false)
-				throw new Exception ("Target specified to build does not exist.");
+				throw new Exception (String.Format ("Target {0} does not exist.", targetName));
 			
 			return this.targets [targetName].Build ();
 		}
@@ -268,7 +268,11 @@ namespace Microsoft.Build.BuildEngine {
 		public void Load (string projectFileName)
 		{
 			this.fullFileName = Path.GetFullPath (projectFileName);
-			DoLoad (new StreamReader (projectFileName));
+			try {
+				DoLoad (new StreamReader (projectFileName));
+			} catch {
+				Console.WriteLine ("Failure to load: {0}", projectFileName);
+			}
 		}
 		
 		[MonoTODO]
@@ -291,9 +295,8 @@ namespace Microsoft.Build.BuildEngine {
 
 		internal void CheckUnloaded ()
 		{
-			if (unloaded) {
-				throw new InvalidOperationException("This project object is no longer valid.");
-			}
+			if (unloaded)
+				throw new InvalidOperationException ("This project object is no longer valid.");
 		}
 
 		private void ProcessXml ()
@@ -340,6 +343,11 @@ namespace Microsoft.Build.BuildEngine {
 
 			InitializeProperties ();
 
+			// FIXME: questionable order of evaluation
+			
+			foreach (Import import in Imports)
+				import.Evaluate ();
+			
 			foreach (BuildPropertyGroup bpg in PropertyGroups) {
 				if (bpg.Condition == String.Empty)
 					bpg.Evaluate ();
@@ -350,9 +358,6 @@ namespace Microsoft.Build.BuildEngine {
 				}
 			}
 			
-			foreach (Import import in Imports)
-				import.Evaluate ();
-			
 			foreach (BuildItemGroup big in ItemGroups) {
 				if (big.Condition == String.Empty)
 					big.Evaluate ();
