@@ -88,7 +88,7 @@ namespace Microsoft.Build.BuildEngine {
 			buildState = BuildState.Started;
 			if (dependsOnTargets == null) {
 				;
-			} else if (dependsOnTargets.Value == "") {
+			} else if (dependsOnTargets.Value == String.Empty) {
 				;
 			} else {
 				OldExpression dependencies = new OldExpression (Project);
@@ -123,9 +123,10 @@ namespace Microsoft.Build.BuildEngine {
 			
 			if (this.batchingImpl.BuildNeeded ()) {
 				foreach (BuildTask bt in buildTasks) {
-					if (this.batchingImpl.BatchBuildTask (bt) == false && bt.ContinueOnError == false) {
+					result = batchingImpl.BatchBuildTask (bt);
+				
+					if (!result && !bt.ContinueOnError) {
 						executeOnErrors = true;
-						result = false;
 						break;
 					}
 				}
@@ -192,7 +193,6 @@ namespace Microsoft.Build.BuildEngine {
 
 		public void RemoveTask (BuildTask buildTask)
 		{
-			//taskElements.Remove (taskElement);
 			buildTasks.Remove (buildTask);
 		}
 
