@@ -46,6 +46,7 @@ namespace Microsoft.Build.Utilities
 		int			timeout;
 		string			toolPath;
 		Process			process;
+		Encoding		responseFileEncoding;
 		MessageImportance	standardErrorLoggingImportance;
 		MessageImportance	standardOutputLoggingImportance;
 		
@@ -69,6 +70,7 @@ namespace Microsoft.Build.Utilities
 			this.TaskResources = taskResources;
 			this.HelpKeywordPrefix = helpKeywordPrefix;
 			this.toolPath = MonoLocationHelper.GetBinDir ();
+			this.responseFileEncoding = Encoding.UTF8;
 		}
 
 		static ToolTask ()
@@ -84,11 +86,14 @@ namespace Microsoft.Build.Utilities
 				RegexOptions.IgnoreCase);
 		}
 
+		[MonoTODO]
 		protected virtual bool CallHostObjectToExecute ()
 		{
 			return true;
 		}
 
+		[MonoTODO]
+		// FIXME: it should write responseFileCommands to temporary response file
 		protected virtual int ExecuteTool (string pathToTool,
 						   string responseFileCommands,
 						   string commandLineCommands)
@@ -119,6 +124,7 @@ namespace Microsoft.Build.Utilities
 				return false;
 		}
 		
+		[MonoTODO]
 		protected virtual string GetWorkingDirectory ()
 		{
 			return null;
@@ -147,6 +153,8 @@ namespace Microsoft.Build.Utilities
 			process.Start ();
 			process.WaitForExit ();
 			
+			exitCode = process.ExitCode;
+			
 			while ((line = process.StandardError.ReadLine ()) != null) {
 				LogEventsFromTextOutput (line, MessageImportance.Normal);
 			}
@@ -237,6 +245,7 @@ namespace Microsoft.Build.Utilities
 			}
 		}
 
+		[MonoTODO]
 		protected virtual string GenerateCommandLineCommands ()
 		{
 			return null;
@@ -244,11 +253,13 @@ namespace Microsoft.Build.Utilities
 
 		protected abstract string GenerateFullPathToTool ();
 
+		[MonoTODO]
 		protected virtual string GenerateResponseFileCommands ()
 		{
 			return null;
 		}
 
+		[MonoTODO]
 		protected virtual string GetResponseFileSwitch (string responseFilePath)
 		{
 			return String.Format ("@{0}", responseFilePath);
@@ -299,7 +310,7 @@ namespace Microsoft.Build.Utilities
 
 		protected virtual Encoding ResponseFileEncoding
 		{
-			get { return Encoding.UTF8; }
+			get { return responseFileEncoding; }
 		}
 
 		protected virtual Encoding StandardErrorEncoding
