@@ -104,6 +104,11 @@ namespace Microsoft.JScript {
 				
 			}
 		}
+
+		internal virtual void PropagateParent (AST parent)
+		{
+			this.parent = parent;
+		}
 	}
 
 	public abstract class Function : AST 
@@ -163,15 +168,15 @@ namespace Microsoft.JScript {
 
 		internal void set_prefix ()
 		{
-			if (parent != null && parent is Function) {
+			if (parent != null && InFunction) {
 				Function tmp;
-				tmp = (Function) parent;
+				tmp = GetContainerFunction;
 				if (tmp.prefix != String.Empty)
 					prefix = tmp.prefix + "." + tmp.func_obj.name;
 				else
 					prefix = tmp.func_obj.name;
 			} else
-				prefix = String.Empty;			
+				prefix = String.Empty;
 		}
 
 		internal void set_function_type ()
