@@ -32,6 +32,8 @@ using System.Data;
 using System.Security;
 using System.Security.Cryptography;
 using System.Security.Cryptography.X509Certificates;
+using System.Threading;
+using System.Net.Sockets;
 
 using Mono.Security.Protocol.Tls;
 
@@ -80,6 +82,8 @@ namespace Npgsql
         // The physical network connection to the backend.
         private Stream                           _stream;
 
+        private Socket                           _socket;
+         
         // Mediator which will hold data generated from backend.
         private NpgsqlMediator                   _mediator;
 
@@ -113,6 +117,15 @@ namespace Npgsql
         private const String                     _planNamePrefix = "npgsqlplan";
         private const String                     _portalNamePrefix = "npgsqlportal";
 
+                
+        private Thread                           _notificationThread;
+        
+        // The AutoResetEvent to synchronize processing threads.
+        internal AutoResetEvent                   _notificationAutoResetEvent;
+        
+        // Counter of notification thread start/stop requests in order to 
+        internal Int16                            _notificationThreadStopCount;
+
 
 
         /// <summary>
@@ -131,6 +144,8 @@ namespace Npgsql
             _oidToNameMapping = new NpgsqlBackendTypeMapping();
             _planIndex = 0;
             _portalIndex = 0;
+            _notificationThreadStopCount = 1;
+            _notificationAutoResetEvent = new AutoResetEvent(true);
 
         }
 
@@ -191,6 +206,21 @@ namespace Npgsql
             }
         }
         
+        internal Int32 ConnectionTimeout
+        {
+            get
+            {
+                return ConnectionString.ToInt32(ConnectionStringKeys.Timeout, ConnectionStringDefaults.Timeout);
+            }
+        }
+
+        internal Int32 CommandTimeout
+        {
+            get
+            {
+                return ConnectionString.ToInt32(ConnectionStringKeys.CommandTimeout, ConnectionStringDefaults.CommandTimeout);
+            }
+        }
         
 
         /// <summary>
@@ -234,6 +264,11 @@ namespace Npgsql
         {
             CurrentState.Bind(this, bind);
         }
+        
+        internal void Describe (NpgsqlDescribe describe)
+        {
+            CurrentState.Describe(this, describe);
+        }
 
         internal void Execute (NpgsqlExecute execute)
         {
@@ -270,6 +305,25 @@ namespace Npgsql
         }
 
 
+
+        /// <summary>
+        /// This method is responsible for releasing all resources associated with this Connector.
+        /// </summary>
+
+        internal void ReleaseResources()
+        {
+            ReleasePlansPortals();
+            ReleaseRegisteredListen();
+
+
+        }
+
+        internal void ReleaseRegisteredListen()
+        {
+            Query(new NpgsqlCommand("unlisten *", this));
+
+        }
+
         /// <summary>
         /// This method is responsible to release all portals used by this Connector.
         /// </summary>
@@ -289,6 +343,8 @@ namespace Npgsql
 
         }
 
+        
+
 
 
         /// <summary>
@@ -329,11 +385,19 @@ namespace Npgsql
         {
             if (Notification != null)
             {
-                foreach (NpgsqlNotificationEventArgs E in _mediator.Notifications)
-                {
-                    Notification(this, E);
+
+                    foreach (NpgsqlNotificationEventArgs E in _mediator.Notifications)
+                    {
+                        // Wrap our notification thread call while running on user land code.
+                        // This prevents our thread of possibly dying there if there is no exception handling.
+                        try
+                        {
+                            Notification(this, E);
+                        }
+                        catch(Exception){}
+                    }
                 }
-            }
+
         }
 
         /// <summary>
@@ -454,6 +518,21 @@ namespace Npgsql
                 _stream = value;
             }
         }
+        
+        /// <summary>
+        /// The physical connection socket to the backend.
+        /// </summary>
+
+        internal Socket Socket {
+            get
+            {
+                return _socket;
+            }
+            set
+            {
+                _socket = value;
+            }
+        }
 
         /// <summary>
         /// Reports if this connector is fully connected.
@@ -689,12 +768,29 @@ namespace Npgsql
             }
             catch {}
         }
+        
+        internal void CancelRequest()
+        {
+            
+            NpgsqlConnector CancelConnector = new NpgsqlConnector(ConnectionString, false, false);
+            
+            CancelConnector._backend_keydata = BackEndKeyData;
+            
+            
+            // Get a raw connection, possibly SSL...
+            CancelConnector.CurrentState.Open(CancelConnector);
+            
+            // Cancel current request.
+            CancelConnector.CurrentState.CancelRequest(CancelConnector);
+            
+            
+        }
 
 
-    ///<summary>
-    /// Returns next portal index.
-    ///</summary>
-    internal String NextPortalName()
+        ///<summary>
+        /// Returns next portal index.
+        ///</summary>
+        internal String NextPortalName()    
         {
             
             return _portalNamePrefix + System.Threading.Interlocked.Increment(ref _portalIndex);
@@ -708,6 +804,110 @@ namespace Npgsql
         {
             return _planNamePrefix + System.Threading.Interlocked.Increment(ref _planIndex);
         }
+        
+        
+        internal void RemoveNotificationThread()
+        {
+            // Wait notification thread finish its work.
+            _notificationAutoResetEvent.WaitOne();
+            
+            // Kill notification thread.
+            _notificationThread.Abort();
+            _notificationThread = null;
+            
+            // Special case in order to not get problems with thread synchronization.
+            // It will be turned to 0 when synch thread is created.
+            _notificationThreadStopCount = 1;  
+            
+        }
+        
+        internal void AddNotificationThread()
+        {
+            
+            _notificationThreadStopCount = 0;
+            _notificationAutoResetEvent.Set();
+            
+            NpgsqlContextHolder contextHolder = new NpgsqlContextHolder(this, CurrentState);
+            
+            _notificationThread = new Thread(new ThreadStart(contextHolder.ProcessServerMessages));
+            
+            _notificationThread.Start();
+            
+            
+            
+        }
+        
+        internal void StopNotificationThread()
+        {
+            
+            _notificationThreadStopCount++;
+            
+            if (_notificationThreadStopCount == 1) // If this call was the first to increment.
+            {
+                
+                _notificationAutoResetEvent.WaitOne();
+                
+            }
+        }
+        
+        internal void ResumeNotificationThread()
+        {
+            _notificationThreadStopCount--;
+            if (_notificationThreadStopCount == 0)
+            {
+                // Release the synchronization handle.
+                
+                _notificationAutoResetEvent.Set();
+            }
+            
+        }
+        
+        internal Boolean IsNotificationThreadRunning
+        {
+            get
+            {
+                return _notificationThreadStopCount <= 0;
+                
+            }
+        }
+        
+        
+        internal class NpgsqlContextHolder
+        {
+        
+            private NpgsqlConnector connector;
+            private NpgsqlState     state;
+        
+            internal NpgsqlContextHolder(NpgsqlConnector connector, NpgsqlState state)
+            {
+                this.connector = connector;
+                this.state = state;
+            
+            }
+        
+            internal void ProcessServerMessages()
+            {
+                
+                while(true)
+                {
+                    this.connector._notificationAutoResetEvent.WaitOne();
+                    
+                    if (this.connector.Socket.Poll(1000, SelectMode.SelectRead))
+                    {
+                        // reset any responses just before getting new ones
+                        this.connector.Mediator.ResetResponses();
+                        this.state.ProcessBackendResponses(this.connector);
+                        this.connector.CheckErrorsAndNotifications();
+                    }
+               
+                    this.connector._notificationAutoResetEvent.Set();
+                }
+    
+                
+                
+            }
+        
+        }
 
 
 
