@@ -476,8 +476,9 @@ namespace System.Configuration
 				}
 				else if (elementName == removeElementName) {
 					ConfigurationElement elem = CreateNewElementInternal (null);
-					elem.DeserializeElement (reader, true);
-					BaseRemove (GetElementKey (elem));
+					ConfigurationRemoveElement removeElem = new ConfigurationRemoveElement (elem);
+					removeElem.DeserializeElement (reader, true);
+					BaseRemove (removeElem.KeyValue);
 					modified = false;
 					return true;
 				}
