@@ -31,18 +31,26 @@
 using System.Collections;
 using System.Xml;
 using System.IO;
+#if !TARGET_JVM
 using System.Security.Cryptography.Xml;
+#endif
 
 namespace System.Configuration
 {
 	public abstract class ConfigurationSection : ConfigurationElement
 	{
 		SectionInformation sectionInformation;
-		
+		IConfigurationSectionHandler section_handler;
+
 		protected ConfigurationSection ()
 		{
 		}
-		
+
+		internal IConfigurationSectionHandler SectionHandler {
+			get { return section_handler; }
+			set { section_handler = value; }
+		}
+
 		[MonoTODO]
 		public SectionInformation SectionInformation {
 			get {
@@ -51,9 +59,19 @@ namespace System.Configuration
 				return sectionInformation;
 			}
 		}
-		
+
+		[MonoTODO]
 		protected internal virtual object GetRuntimeObject ()
 		{
+			// FIXME: this hack is nasty. We should make some
+			// refactory on the entire assembly.
+			if (SectionHandler != null) {
+				if (RawXml == null)
+					return null;
+				XmlDocument doc = new XmlDocument ();
+				doc.LoadXml (RawXml);
+				return SectionHandler.Create (null, null, doc.DocumentElement);
+			}
 			return this;
 		}
 
