@@ -263,9 +263,8 @@ namespace System.Data.OracleClient.Oci
 		{
 			fieldType = typeof (System.String);
 
-			// The buffer is able to contain twice the defined size
-			// to allow usage of multibyte characters
-			value = Marshal.AllocHGlobal (definedSize * 2);
+			int maxByteCount = Encoding.UTF8.GetMaxByteCount (definedSize);
+			value = Marshal.AllocHGlobal (maxByteCount);
 
 			ociType = OciDataType.Char;
 
@@ -276,7 +275,7 @@ namespace System.Data.OracleClient.Oci
 						ErrorHandle,
 						position + 1,
 						value,
-						definedSize * 2,
+						maxByteCount,
 						ociType,
 						ref indicator,
 						ref rlenp,
@@ -412,7 +411,7 @@ namespace System.Data.OracleClient.Oci
 			return new OracleLob (lobLocator, ociType);
 		}
 
-		public object GetValue ()
+                public object GetValue (IFormatProvider formatProvider)
 		{
 			object tmp;
 
@@ -458,7 +457,7 @@ namespace System.Data.OracleClient.Oci
 			case OciDataType.Float:
 				tmp = Marshal.PtrToStringAnsi (Value, Size);
 				if (tmp != null)
-					return Decimal.Parse (String.Copy ((string) tmp));
+                                        return Decimal.Parse (String.Copy ((string) tmp), formatProvider);
 				break;
 			case OciDataType.TimeStamp:
 				return dateTimeDesc.GetDateTime (Parent.Parent, dateTimeDesc.ErrorHandle);
@@ -476,9 +475,9 @@ namespace System.Data.OracleClient.Oci
 			return DBNull.Value;
 		}
 
-		internal object GetOracleValue () 
+                internal object GetOracleValue (IFormatProvider formatProvider)
 		{
-			object ovalue = GetValue ();
+                        object ovalue = GetValue (formatProvider);
 
 			switch (DataType) {
 			case OciDataType.Raw:
