@@ -130,9 +130,8 @@ namespace System.Data.OracleClient {
 		[DefaultValue (null)]
 		IDbConnection IDbCommand.Connection {
 			get { return Connection; }
-			set { 
-				if (!(value is OracleConnection))
-					throw new InvalidCastException ("The value was not a valid OracleConnection.");
+			set {
+                                // InvalidCastException is expected when types do not match
 				Connection = (OracleConnection) value;
 			}
 		}
@@ -143,9 +142,8 @@ namespace System.Data.OracleClient {
 
 		IDbTransaction IDbCommand.Transaction {
 			get { return Transaction; }
-			set { 
-				if (!(value is OracleTransaction))
-					throw new ArgumentException ();
+			set {
+                                // InvalidCastException is expected when types do not match
 				Transaction = (OracleTransaction) value; 
 			}
 		}
@@ -382,7 +380,7 @@ namespace System.Data.OracleClient {
 					if (statement.Fetch ()) {
 						OciDefineHandle defineHandle = (OciDefineHandle) statement.Values [0];
 						if (!defineHandle.IsNull)
-							output = defineHandle.GetOracleValue ();
+                                                        output = defineHandle.GetOracleValue (Connection.SessionFormatProvider);
 						switch (defineHandle.DataType) {
 						case OciDataType.Blob:
 						case OciDataType.Clob:
@@ -463,7 +461,7 @@ namespace System.Data.OracleClient {
 
 		public object ExecuteScalar ()
 		{
-			object output = DBNull.Value;
+			object output = null;//if we find nothing we return this
 
 			AssertConnectionIsOpen ();
 			AssertTransactionMatch ();
@@ -488,25 +486,24 @@ namespace System.Data.OracleClient {
 
 					if (statement.Fetch ()) {
 						OciDefineHandle defineHandle = (OciDefineHandle) statement.Values [0];
-						if (defineHandle.IsNull)
-							output = DBNull.Value;
-						else {
+						if (!defineHandle.IsNull)
+						{
 							switch (defineHandle.DataType) {
 							case OciDataType.Blob:
 							case OciDataType.Clob:
-								OracleLob lob = (OracleLob) defineHandle.GetValue ();
+                                                                OracleLob lob = (OracleLob) defineHandle.GetValue (
+                                                                        Connection.SessionFormatProvider);
 								lob.connection = Connection;
 								output = lob.Value;
 								lob.Close ();
 								break;
 							default:
-								output = defineHandle.GetValue ();
+								output = defineHandle.GetValue (
+                                                                        Connection.SessionFormatProvider);
 								break;
 							}
 						}
 					}
-					else
-						output = DBNull.Value;
 					UpdateParameterValues ();
 				}
 			}
@@ -577,6 +574,15 @@ namespace System.Data.OracleClient {
 			preparedStatement = statement;
 		}
 
+                protected override void Dispose (bool disposing)
+                {
+                        if (disposing)
+                                if (Parameters.Count > 0)
+                                        foreach (OracleParameter parm in Parameters)
+                                                parm.FreeHandle ();
+                        base.Dispose (disposing);
+                }
+
 		#endregion // Methods
 	}
 }
