@@ -121,8 +121,13 @@ namespace System.Data.OracleClient {
 		public OracleParameter (string name, OracleType dataType, int size, ParameterDirection direction, bool isNullable, byte precision, byte scale, string srcColumn, DataRowVersion srcVersion, object value)
 		{
 			this.name = name;
+			if(size < 0)
+				throw new ArgumentException("Size must be not be negative.");
 			this.size = size;
 			this.value = value;
+			this.isNullable = isNullable;
+			this.precision = precision;
+			this.scale = scale;
 
 			OracleType = dataType;
 			Direction = direction;
@@ -219,9 +224,13 @@ namespace System.Data.OracleClient {
 		[DefaultValue (null)]
 		[RefreshProperties (RefreshProperties.All)]
 		[TypeConverter (typeof(StringConverter))]
+		[MonoTODO("InferOracleType is not always needed")]
 		public object Value {
 			get { return this.value; }
-			set { this.value = value; }
+			set {
+				this.value = value;
+				InferOracleType(value);
+			}
 		}
 
 		#endregion // Properties
@@ -245,7 +254,7 @@ namespace System.Data.OracleClient {
 			}
 		}
 
-		internal void Bind (OciStatementHandle statement, OracleConnection con, uint pos) 
+		internal void Bind (OciStatementHandle statement, OracleConnection con, uint pos)
 		{
 			connection = con;
 
@@ -474,9 +483,17 @@ namespace System.Data.OracleClient {
 					Marshal.Copy (val, 0, bindValue, val.Length);
 					bindSize = val.Length;
 				}
-				else if (oracleType == OracleType.Number) {
-					// TODO: move number type to a non-locale specific way
-					string svalue = v.ToString ();
+                                else if (oracleType == OracleType.Number
+                                        || oracleType == OracleType.Double
+                                        || oracleType == OracleType.Int32
+                                        || oracleType == OracleType.Int16
+                                        || oracleType == OracleType.Byte
+                                        || oracleType == OracleType.Float) {
+                                        string svalue = null;
+                                        if(v is IFormattable)
+                                                svalue = ((IFormattable)v).ToString (null, con.SessionFormatProvider);
+                                        else
+                                                svalue = v.ToString();
 					rsize = 0;
 			
 					// Get size of buffer
@@ -487,10 +504,10 @@ namespace System.Data.OracleClient {
 					OciCalls.OCIUnicodeToCharSet (statement.Parent, bytes, svalue, out rsize);
 
 					bindType = OciDataType.VarChar2;
-					bindSize = v.ToString ().Length;
+                                        bindSize = svalue.Length;
 				}
 				else {
-					string svalue = v.ToString ();
+					string svalue = v.ToString () + '\0';
 					rsize = 0;
 			
 					// Get size of buffer
@@ -500,8 +517,8 @@ namespace System.Data.OracleClient {
 					bytes = new byte[rsize];
 					OciCalls.OCIUnicodeToCharSet (statement.Parent, bytes, svalue, out rsize);
 
-					bindType = OciDataType.VarChar2;
-					bindSize = v.ToString ().Length;
+					bindType = OciDataType.String;
+                                        bindSize = svalue.Length;
 				}
 			}
 
@@ -607,7 +624,7 @@ namespace System.Data.OracleClient {
 		private void InferOracleType (object value)
 		{
 			Type type = value.GetType ();
-			string exception = String.Format ("The parameter data type of {0} is invalid.", type.Name);
+			string exception = String.Format ("The parameter data type of {0} is invalid.", type.FullName);
 			switch (type.FullName) {
 			case "System.Int64":
 				SetOracleType (OracleType.Number);
@@ -642,6 +659,8 @@ namespace System.Data.OracleClient {
 			case "System.Int16":
 				SetOracleType (OracleType.Int16);
 				break;
+			case "System.DBNull":
+				break; //unable to guess type
 			default:
 				throw new ArgumentException (exception);
 			}
@@ -768,6 +787,7 @@ namespace System.Data.OracleClient {
 
 		private void SetOracleType (OracleType type)
 		{
+                        FreeHandle ();
 			string exception = String.Format ("No mapping exists from OracleType {0} to a known DbType.", type);
 			switch (type) {
 			case OracleType.BFile:
@@ -908,7 +928,7 @@ namespace System.Data.OracleClient {
 			case OciDataType.Float:
 				tmp = Marshal.PtrToStringAnsi (bindOutValue, bindSize);
 				if (tmp != null)
-					value = Decimal.Parse (String.Copy ((string) tmp));
+                                        value = Decimal.Parse (String.Copy ((string) tmp), cmd.Connection.SessionFormatProvider);
 				break;
 			case OciDataType.TimeStamp:
 				value = dateTimeDesc.GetDateTime (connection.Environment, dateTimeDesc.ErrorHandle);
@@ -946,7 +966,7 @@ namespace System.Data.OracleClient {
 			FreeHandle ();
 		}
 
-		private void FreeHandle () 
+                internal void FreeHandle ()
 		{
 			switch (ociType) {
 			case OciDataType.Clob:
@@ -955,7 +975,12 @@ namespace System.Data.OracleClient {
 				break;
 			case OciDataType.Raw:
 				Marshal.FreeHGlobal (bindValue);
-				break;				
+				break;
+                        case OciDataType.TimeStamp:
+                                break;
+                        default:
+                                Marshal.FreeHGlobal (bindOutValue);
+                                break;
 			} 
 
 			bindOutValue = IntPtr.Zero;
