@@ -144,9 +144,8 @@ namespace System.Data {
 						DataContainer[index] = value;
 					}
 					catch(Exception e) {
-						throw new ArgumentException(e.Message +
-							String.Format("Couldn't store <{0}> in {1} Column.  Expected type is {2}.",
-							value, ColumnName, DataType.Name), e);
+						throw new ArgumentException(String.Format("{0}. Couldn't store <{1}> in Column named '{2}'. Expected type is {3}.",
+							e.Message, value, ColumnName, DataType.Name), e);
 					}
 				}
 
@@ -218,7 +217,9 @@ namespace System.Data {
 						bool nullsFound = false;
 						for(int r = 0; r < _table.Rows.Count; r++) {
 							DataRow row = _table.Rows[r];
-							if(row.IsNull(this)) {
+							DataRowVersion version = row.HasVersion (DataRowVersion.Default) ?
+								DataRowVersion.Default : DataRowVersion.Original;
+							if(row.IsNull(this, version)) {
 								nullsFound = true;
 								break;
 							}
@@ -600,6 +601,9 @@ namespace System.Data {
 			get {
 				return _extendedProperties;
 			}
+#if NET_2_0
+			internal set { _extendedProperties = value; }
+#endif
 		}
 
 		[DataCategory ("Data")]
@@ -782,15 +786,6 @@ namespace System.Data {
 
 		#region Methods
 		
-/* ??
-		[MonoTODO]
-		protected internal void CheckNotAllowNull() {
-		}
-
-		[MonoTODO]
-		protected void CheckUnique() {
-		}
-*/
 		[MonoTODO]
 		internal DataColumn Clone() {
 			DataColumn copy = new DataColumn ();
