@@ -158,6 +158,11 @@ namespace System.Data {
 			return column;
 		}
 
+		public void CopyTo (DataColumn [] array, int index)
+		{
+			CopyTo ((Array) array, index);
+		}
+
 		internal void RegisterName(string name, DataColumn column)
 		{
 			if (columnFromName.Contains(name))
@@ -237,7 +242,11 @@ namespace System.Data {
 			column.SetTable (parentTable);
 			RegisterName(column.ColumnName, column);
 			int ordinal = base.List.Add(column);
+#if NET_2_0
+			column.Ordinal = ordinal;
+#else
 			column.SetOrdinal (ordinal);
+#endif
 		
 			// Check if the Column Expression is ok	
 			if (column.CompiledExpression != null)
@@ -488,31 +497,30 @@ namespace System.Data {
 		/// Raises the OnCollectionChanged event.
 		/// </summary>
 		/// <param name="ccevent">A CollectionChangeEventArgs that contains the event data.</param>
-		protected
 #if !NET_2_0
-		virtual
+		protected virtual
+#else
+		internal
 #endif
 		void OnCollectionChanged(CollectionChangeEventArgs ccevent)
 		{
 			parentTable.ResetPropertyDescriptorsCache();
 			if (CollectionChanged != null) 
-			{
 				CollectionChanged(this, ccevent);
-			}
 		}
 
 		/// <summary>
 		/// Raises the OnCollectionChanging event.
 		/// </summary>
 		/// <param name="ccevent">A CollectionChangeEventArgs that contains the event data.</param>
-		protected internal
 #if !NET_2_0
-		virtual
+		protected internal virtual
+#else
+		internal
 #endif
 		void OnCollectionChanging(CollectionChangeEventArgs ccevent)
 		{
-			if (CollectionChanged != null) 
-			{
+			if (CollectionChanged != null) {
 				//FIXME: this is not right
 				//CollectionChanged(this, ccevent);
 				throw new NotImplementedException();
@@ -546,7 +554,11 @@ namespace System.Data {
 	
 			//Update the ordinals
 			for( int i = ordinal ; i < this.Count ; i ++ )
-				this[i].SetOrdinal( i );
+#if NET_2_0
+				this[i].Ordinal = i;
+#else
+				this[i].SetOrdinal(i);
+#endif
 
 			if (parentTable != null)
 				parentTable.OnRemoveColumn(column);
@@ -655,5 +667,27 @@ namespace System.Data {
 		public event CollectionChangeEventHandler CollectionChanged;
 
 		#endregion 
+		
+#if NET_2_0
+		internal void MoveColumn (int oldOrdinal, int newOrdinal)
+		{
+			if (newOrdinal == -1 || newOrdinal > this.Count)
+				throw new ArgumentOutOfRangeException ("ordinal", "Ordinal '" + newOrdinal + "' exceeds the maximum number.");
+			if (oldOrdinal == newOrdinal)
+				return;
+			
+			int start = newOrdinal > oldOrdinal ? oldOrdinal : newOrdinal;
+			int end = newOrdinal > oldOrdinal ?  newOrdinal : oldOrdinal;
+			int direction = newOrdinal > oldOrdinal ? 1 : (-1);
+
+			DataColumn currColumn = this [start];
+			for (int i=start; i < end; i+=direction) {
+				List [i] = List [i+direction];
+				((DataColumn)List [i]).Ordinal = i;
+			}
+			List [end] = currColumn;
+			currColumn.Ordinal = end;
+		}
+#endif
 	}
 }
