@@ -137,6 +137,25 @@ namespace System.Data {
 					return base.List;
 				}
 			}
+
+			public override DataRelation this [string name] {
+				get {
+					int index = IndexOf (name, true);
+					return index < 0 ? null : (DataRelation) List[index];
+				}
+			}
+
+			/// <summary>
+			/// Gets the DataRelation object at the specified index.
+			/// </summary>
+			public override DataRelation this [int index] {
+				get {
+					if (index < 0 || index >= List.Count)
+						throw new IndexOutOfRangeException (String.Format ("Cannot find relation {0}.", index));
+
+					return (DataRelation)List [index];
+				}
+			}
 		}
 
 		/// <summary>
@@ -159,25 +178,43 @@ namespace System.Data {
 				return dataTable.DataSet;
 			}
 
+			public override DataRelation this [string name] {
+				get {
+					int index = IndexOf (name, true);
+					return index < 0 ? null : (DataRelation) List[index];
+				}
+			}
+
+			/// <summary>
+			/// Gets the DataRelation object at the specified index.
+			/// </summary>
+			public override DataRelation this [int index] {
+				get {
+					if (index < 0 || index >= List.Count)
+						throw new IndexOutOfRangeException (String.Format ("Cannot find relation {0}.", index));
+
+					return (DataRelation)List [index];
+				}
+			}
+
 			/// <summary>
 			/// Performs verification on the table.
 			/// </summary>
 			/// <param name="relation">The relation to check.</param>
 			protected override void AddCore (DataRelation relation)
 			{
-                if (dataTable.ParentRelations == this && relation.ChildTable != dataTable)
-                    throw new ArgumentException ("Cannot add a relation to this table's " +
-													"ParentRelations where this table is not" +
-													" the Child table.");
+				if (dataTable.ParentRelations == this && relation.ChildTable != dataTable)
+					throw new ArgumentException ("Cannot add a relation to this table's " +
+								     "ParentRelations where this table is not" +
+								     " the Child table.");
 
-                if (dataTable.ChildRelations == this && relation.ParentTable != dataTable)   
-                    throw new ArgumentException("Cannot add a relation to this table's " +
-                                                "ChildRelations where this table is not" +
-                                                " the Parent table.");
+				if (dataTable.ChildRelations == this && relation.ParentTable != dataTable)   
+					throw new ArgumentException("Cannot add a relation to this table's " +
+								    "ChildRelations where this table is not" +
+								    " the Parent table.");
                 
 				dataTable.DataSet.Relations.Add(relation);
-                base.AddCore (relation);
-
+				base.AddCore (relation);
 			}
                         
 			protected override void RemoveCore (DataRelation relation)
@@ -211,23 +248,15 @@ namespace System.Data {
 		/// <summary>
 		/// Gets the DataRelation object specified by name.
 		/// </summary>
-		public DataRelation this [string name] {
-			get {
-				int index = IndexOf (name, true);
-				return index < 0 ? null : (DataRelation) List[index];
-			}
+		public abstract DataRelation this [string name] {
+			get;
 		}
 
 		/// <summary>
 		/// Gets the DataRelation object at the specified index.
 		/// </summary>
-		public DataRelation this [int index] {
-			get {
-				if (index < 0 || index >= List.Count)
-					throw new IndexOutOfRangeException (String.Format ("Cannot find relation {0}.", index));
-
-				return (DataRelation)List [index];
-			}
+		public abstract DataRelation this [int index] {
+			get;
 		}
 
 		
@@ -246,7 +275,6 @@ namespace System.Data {
 		/// Adds a DataRelation to the DataRelationCollection.
 		/// </summary>
 		/// <param name="relation">The DataRelation to add to the collection.</param>
-		[MonoTODO]
 		public void Add(DataRelation relation)
 		{
             // To prevent endless recursion
@@ -259,6 +287,9 @@ namespace System.Data {
 
 			try
 			{
+				CollectionChangeEventArgs e = new CollectionChangeEventArgs (CollectionChangeAction.Add, this);
+				OnCollectionChanging (e);
+
 				this.AddCore (relation);
 				if(relation.RelationName == string.Empty)
 					relation.RelationName = GenerateRelationName();
@@ -266,8 +297,8 @@ namespace System.Data {
 				relation.ParentTable.ResetPropertyDescriptorsCache();
 				relation.ChildTable.ResetPropertyDescriptorsCache();
 			
-				CollectionChangeEventArgs e = new CollectionChangeEventArgs(CollectionChangeAction.Add, this);
-				OnCollectionChanged(e);
+				e = new CollectionChangeEventArgs (CollectionChangeAction.Add, this);
+				OnCollectionChanged (e);
 			}
 			finally
 			{
@@ -397,7 +428,6 @@ namespace System.Data {
 		/// Adds to the list
 		/// </summary>
 		/// <param name="relation">The relation to check.</param>
-		[MonoTODO]
 		protected virtual void AddCore(DataRelation relation)
 		{
 			if (relation == null) {
@@ -515,6 +545,13 @@ namespace System.Data {
 
 		public virtual bool Contains(string name)
 		{
+			DataSet tmpDataSet = GetDataSet();
+			if (tmpDataSet != null) {
+				DataRelation tmpRelation = tmpDataSet.Relations [name];
+				if (tmpRelation != null) {
+					return true;
+				}
+			}
 			return (-1 != IndexOf (name, false));
 		}
 
@@ -563,9 +600,9 @@ namespace System.Data {
 		}
 
 		[MonoTODO]
-		protected internal virtual void OnCollectionChanging (CollectionChangeEventArgs ccevent)
+		protected virtual void OnCollectionChanging (CollectionChangeEventArgs ccevent)
 		{
-			throw new NotImplementedException ();
+			//throw new NotImplementedException ();
 		}
 
 		public void Remove (DataRelation relation)
@@ -587,6 +624,8 @@ namespace System.Data {
 				if (!(List.Contains(relation)))
 					throw new ArgumentException("Relation doesnot belong to this Collection.");
 
+				OnCollectionChanging (CreateCollectionChangeEvent (CollectionChangeAction.Remove));
+
 				RemoveCore (relation);
 				string name = "Relation" + index;
 				if (relation.RelationName == name)
@@ -616,7 +655,6 @@ namespace System.Data {
 			Remove(relation);
 		}
 
-		[MonoTODO]
 		protected virtual void RemoveCore(DataRelation relation)
 		{
 			// Remove from collection
