@@ -71,6 +71,7 @@ namespace System.Data {
 		private bool _hasParentCollection;
 		private bool _inChangingEvent;
 		private int _rowId;
+		internal bool _rowChanged = false;
 
 		private XmlDataDocument.XmlDataElement mappedElement;
 		internal bool _inExpressionEvaluation = false;
@@ -140,8 +141,10 @@ namespace System.Data {
 			get { return this[columnName, DataRowVersion.Default]; }
 			set {
 				int columnIndex = _table.Columns.IndexOf (columnName);
-				if (columnIndex == -1)
-					throw new IndexOutOfRangeException ();
+				if (columnIndex == -1) {
+					throw new ArgumentException ("The column " + columnName +
+								    " does not belong to the table : " + _table.TableName);
+				}
 				this[columnIndex] = value;
 			}
 		}
@@ -178,6 +181,7 @@ namespace System.Data {
 				if (value == null && column.DataType != typeof(string)) {
 					throw new ArgumentException("Cannot set column " + column.ColumnName + " to be null, Please use DBNull instead");
 				}
+				_rowChanged = true;
 				
 				CheckValue (value, column);
 				bool orginalEditing = Proposed >= 0;
@@ -199,8 +203,10 @@ namespace System.Data {
 		public object this[string columnName, DataRowVersion version] {
 			get {
 				int columnIndex = _table.Columns.IndexOf (columnName);
-				if (columnIndex == -1)
-					throw new IndexOutOfRangeException ();
+				if (columnIndex == -1) {
+					throw new ArgumentException ("The column " + columnName +
+								     " does not belong to the table : " + _table.TableName);
+				}
 				return this[columnIndex, version];
 			}
 		}
@@ -334,6 +340,20 @@ namespace System.Data {
 					return DataRowState.Deleted;
 				return DataRowState.Modified;
 			}
+#if NET_2_0
+			internal set {
+				if (DataRowState.Detached == value) {
+					Original = -1;
+					Current = -1;
+				}
+				if (DataRowState.Unchanged == value)
+					Original = Current;
+				if (DataRowState.Added == value)
+					Original = -1;
+				if (DataRowState.Deleted == value)
+					Current = -1;
+			}
+#endif
 		}
 
 #if NET_2_0
@@ -919,7 +939,10 @@ namespace System.Data {
 				// Note : row state must not be changed before all the job on indexes finished,
 				// since the indexes works with recods rather than with rows and the decision
 				// which of row records to choose depends on row state.
-				_table.ChangedDataRow(this, DataRowAction.Change);
+				if (_rowChanged == true) {
+					_table.ChangedDataRow(this, DataRowAction.Change);
+					_rowChanged = false;
+				}
 			}
 		}
 
