@@ -144,7 +144,6 @@ namespace System.Data
 		// "Detached", and when this.EndEdit() is invoked, the row
 		// will be added to the table.
 		public bool IsNew {
-			[MonoTODO]
 			get {
 				return Row == DataView._lastAdded;
 			}
@@ -215,14 +214,12 @@ namespace System.Data
 		}
 
 		public DataRow Row {
-			[MonoTODO]
 			get {
 				return _dataRow;
 			}
 		}
 
 		public DataRowVersion RowVersion {
-			[MonoTODO]
 			get {
 				DataRowVersion version = DataView.GetRowVersion(_index);
 				if (version != DataRowVersion.Original)
@@ -232,7 +229,6 @@ namespace System.Data
 			}
 		}
 
-		[MonoTODO]
 		public override int GetHashCode() {
 			return _dataRow.GetHashCode();
 		}	
@@ -245,7 +241,6 @@ namespace System.Data
 		
 		#region ICustomTypeDescriptor implementations
 		
-		[MonoTODO]
 		AttributeCollection ICustomTypeDescriptor.GetAttributes  ()
 		{
 			System.ComponentModel.AttributeCollection attributes;
@@ -253,55 +248,54 @@ namespace System.Data
 			return attributes;
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented.   Always returns String.Empty")]
 		string ICustomTypeDescriptor.GetClassName ()
 		{
 			return "";
 		}
 		
-		[MonoTODO]
+		[MonoTODO ("Not implemented.   Always returns null")]
 		string ICustomTypeDescriptor.GetComponentName ()
 		{
 			return null;
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented.   Always returns null")]
 		TypeConverter ICustomTypeDescriptor.GetConverter ()
 		{
 			return null;
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented.   Always returns null")]
 		EventDescriptor ICustomTypeDescriptor.GetDefaultEvent ()
 		{
 			return null;
 		}
 		
-		[MonoTODO]
+		[MonoTODO ("Not implemented.   Always returns null")]
 		PropertyDescriptor ICustomTypeDescriptor.GetDefaultProperty ()
 		{
 			return null;
 		}
 		
-		[MonoTODO]
+		[MonoTODO ("Not implemented.   Always returns null")]
 		object ICustomTypeDescriptor.GetEditor (Type editorBaseType)
 		{
 			return null;
 		}
 		
-		[MonoTODO]
+		[MonoTODO ("Not implemented.   Always returns an empty collection")]
 		EventDescriptorCollection ICustomTypeDescriptor.GetEvents ()
 		{
 			return new EventDescriptorCollection(null);
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented.   Always returns an empty collection")]
 		EventDescriptorCollection ICustomTypeDescriptor.GetEvents (Attribute [] attributes)
 		{
 			return new EventDescriptorCollection(null);
 		}
 
-		[MonoTODO]
 		PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties ()
 		{
 			if (DataView == null) {
@@ -313,7 +307,7 @@ namespace System.Data
 			}
 		}
 
-		[MonoTODO]
+		[MonoTODO ("It currently reports more descriptors than necessary")]
 		PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties (Attribute [] attributes)
 		{
 			PropertyDescriptorCollection descriptors;
@@ -336,14 +330,14 @@ namespace System.Data
 		#region IDataErrorInfo implementation
 
 		string IDataErrorInfo.Error {
-			[MonoTODO]
+			[MonoTODO("Not implemented, always returns String.Empty")]
 			get {
 				return ""; // FIXME
 			}
 		}
 
 		string IDataErrorInfo.this[string columnName] {
-			[MonoTODO]
+			[MonoTODO("Not implemented, always returns String.Empty")]
 			get {
 				return ""; // FIXME
 			}
@@ -356,7 +350,6 @@ namespace System.Data
 
 		public event PropertyChangedEventHandler PropertyChanged;
 
-		[MonoTODO ("to be tested..")]
 		void OnPropertyChanged (string propertyName)
 		{
 			if (PropertyChanged != null) {
