@@ -36,7 +36,7 @@ namespace System.Data
 	[DefaultProperty ("Table")]
 	[DesignerAttribute ("Microsoft.VSDesigner.Data.VS.DataViewDesigner, "+ Consts.AssemblyMicrosoft_VSDesigner, "System.ComponentModel.Design.IDesigner")]
 #if NET_2_0
-	public class DataView : MarshalByValueComponent, IBindingList, IList, ICollection, IEnumerable, ITypedList, ISupportInitialize, IBindingListView
+	public class DataView : MarshalByValueComponent, IBindingList, IList, ICollection, IEnumerable, ITypedList, ISupportInitialize, IBindingListView, ISupportInitializeNotification
 #else
 	public class DataView : MarshalByValueComponent, IBindingList, IList, ICollection, IEnumerable, ITypedList, ISupportInitialize
 #endif
@@ -50,6 +50,9 @@ namespace System.Data
 		DataColumn [] sortColumns = null;
 		internal DataViewRowState rowState;
 		internal DataRowView[] rowCache = new DataRowView [0];
+#if NET_2_0
+		private bool dataViewInitialized = true;
+#endif
 
 		// BeginInit() support
 		bool isInitPhase = false;
@@ -65,7 +68,7 @@ namespace System.Data
 		bool allowEdit = true;
 		bool allowDelete = true;
 		bool applyDefaultSort = false;
-		bool isSorted = false;
+		//bool isSorted = false;
 
 		bool isOpen = false;
 
@@ -195,7 +198,6 @@ namespace System.Data
 		[DataSysDescription ("Returns the number of items currently in this view.")]
 #endif
 		public int Count {
-			[MonoTODO]
 			get {
 				return rowCache.Length;
 			}
@@ -206,7 +208,6 @@ namespace System.Data
 		[DataSysDescription ("This returns a pointer to back to the DataViewManager that owns this DataSet (if any).")]
 #endif
 		public DataViewManager DataViewManager {
-			[MonoTODO]
 			get {
 				return dataViewManager;
 			}
@@ -217,7 +218,6 @@ namespace System.Data
 		// this IndexerNameAttribute
 		[System.Runtime.CompilerServices.IndexerName("Item")]
 		public DataRowView this[int recordIndex] {
-			[MonoTODO]
 			get {
 				if (recordIndex > rowCache.Length)
 					throw new IndexOutOfRangeException ("There is no row at " +
@@ -233,7 +233,6 @@ namespace System.Data
 		[DefaultValue ("")]
 		public virtual string RowFilter {
 			get { return rowFilter; }
-			[MonoTODO]
 			set {
 				if (value == null)
 					value = String.Empty;
@@ -366,11 +365,16 @@ namespace System.Data
 			}
 		}
 
+#if NET_2_0
+		[Browsable (false)]
+		public bool IsInitialized {
+			get { return dataViewInitialized;}
+		}
+#endif
 		#endregion // PublicProperties
 		
 		#region	PublicMethods
 
-		[MonoTODO]
 		public virtual DataRowView AddNew() 
 		{
 			if (!IsOpen)
@@ -386,7 +390,7 @@ namespace System.Data
 			_lastAdded = dataTable.NewRow ();
 			UpdateIndex(true);
 			OnListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, Count - 1, -1));
-			 
+
 			return this[Count - 1];
 		}
 
@@ -414,7 +418,6 @@ namespace System.Data
 			}
 		}
 
-		[MonoTODO]
 		public void BeginInit() 
 		{
 			initTable = Table;
@@ -424,9 +427,11 @@ namespace System.Data
 			initRowState = RowStateFilter;
 
 			isInitPhase = true;
+#if NET_2_0
+			dataViewInitialized = false;
+#endif
 		}
 
-		[MonoTODO]
 		public void CopyTo (Array array, int index) 
 		{
 			if (index + rowCache.Length > array.Length) {
@@ -480,7 +485,6 @@ namespace System.Data
 		}
 #endif
 
-		[MonoTODO]
 		public void EndInit() 
 		{
 			isInitPhase = false;
@@ -496,16 +500,19 @@ namespace System.Data
 			inEndInit = false;
 
 			UpdateIndex (true);
+
+#if NET_2_0
+			dataViewInitialized = true;
+			DataViewInitialized ();
+#endif
 		}
 
-		[MonoTODO]
 		public int Find(object key) 
 		{
 			object [] keys = new object[] { key };
 			return Find(keys);
 		}
 		
-		[MonoTODO]
 		public int Find(object[] keys) 
 		{
 			if (sort == null || sort == string.Empty) {
@@ -529,13 +536,11 @@ namespace System.Data
 			return index;
 		}
 		
-		[MonoTODO]
 		public DataRowView[] FindRows(object key) 
 		{
 			return FindRows(new object[] {key});
 		}
 
-		[MonoTODO]
 		public DataRowView[] FindRows(object[] keys) 
 		{
 			if (sort == null || sort == string.Empty) {
@@ -570,6 +575,10 @@ namespace System.Data
 #endif
 		public event ListChangedEventHandler ListChanged;
 
+#if NET_2_0
+		public event EventHandler Initialized;
+#endif
+
 		[Browsable (false)]
 #if !NET_2_0
 		[DataSysDescription ("Indicates whether the view is open.  ")]
@@ -620,7 +629,6 @@ namespace System.Data
 		{
 		}
 
-		[MonoTODO]
 		protected virtual void OnListChanged(ListChangedEventArgs e) 
 		{
 			// Yes, under MS.NET, when it is overriden, the 
@@ -639,7 +647,6 @@ namespace System.Data
 			OnListChanged(e);
 		}
 
-		[MonoTODO]
 		protected void Open() 
 		{
 			// I wonder if this comment is still valid, but keep
@@ -748,6 +755,19 @@ namespace System.Data
 			OnListChanged (new ListChangedEventArgs (ListChangedType.ItemDeleted, newIndex, -1));
 		}
 		
+#if NET_2_0
+		private void DataViewInitialized ()
+		{
+			EventArgs e = new EventArgs ();
+			OnDataViewInitialized (e);
+		}
+
+		private void OnDataViewInitialized (EventArgs e) {
+			if (null != Initialized) {
+				Initialized (this, e);
+			}
+		}
+#endif
 		protected virtual void ColumnCollectionChanged (object sender, CollectionChangeEventArgs args)
 		{
 			// UpdateIndex() is not invoked here (even if the sort
@@ -934,7 +954,6 @@ namespace System.Data
 			}
 		}
 
-		[MonoTODO]
 		PropertyDescriptorCollection ITypedList.GetItemProperties (PropertyDescriptor[] listAccessors) 
 		{
 			// FIXME: use listAccessors somehow
@@ -974,36 +993,30 @@ namespace System.Data
 		}
 
 		bool ICollection.IsSynchronized { 
-			[MonoTODO]
 			get {
 				return false;
 			} 
 		}
 
 		object ICollection.SyncRoot { 
-			[MonoTODO]
 			get {
-				// FIXME:
 				return this;
 			}
 		}
 
 		bool IList.IsFixedSize {
-			[MonoTODO]
 			get {
 				return false;
 			}
 		}
 		
 		bool IList.IsReadOnly {
-			[MonoTODO]
 			get {
 				return false;
 			}
 		}
 
 		object IList.this[int recordIndex] {
-			[MonoTODO]
 			get {
 				return this[recordIndex];
 			}
