@@ -570,6 +570,9 @@ namespace MonoTests.System.Data
 		}	
 
 		[Test]
+#if TARGET_JVM
+		[Ignore ("Does not work with TARGET_JVM")]
+#endif
 		public void ToStringTest ()
 		{
 			DataTable Table = new DataTable ("test_table");
