@@ -422,6 +422,59 @@ namespace MonoTests_System.Data
 			}
 		}
 
+		[Test]
+		[ExpectedException (typeof (ConstraintException))]
+		public void LoadDataRow_DuplicateValues ()
+		{
+			DataTable table = new DataTable ();
+			table.Columns.Add ("col1", typeof (int));
+			table.Columns.Add ("col2", typeof (int));
+
+			table.PrimaryKey = new DataColumn[] {table.Columns [0]};
+
+			table.BeginLoadData ();
+			table.LoadDataRow (new object[] {1 , 1}, false);
+			table.LoadDataRow (new object[] {1 , 10}, false);
+			table.EndLoadData ();
+		}
+
+		[Test]
+		public void LoadDataRow_WithoutBeginLoadData ()
+		{
+			DataTable table = new DataTable ();
+			table.Columns.Add ("col1", typeof (int));
+			table.Columns.Add ("col2", typeof (int));
+
+			table.PrimaryKey = new DataColumn[] {table.Columns [0]};
+			table.Rows.Add (new object[] {1,1});
+			table.AcceptChanges ();
+
+			table.LoadDataRow (new object[] {10,1}, false);
+			DataRow row = table.Rows.Find (10);
+			Assert.IsNotNull (row, "#1");
+			Assert.AreEqual (1, row [1], "#2");
+			Assert.AreEqual (DataRowState.Added, row.RowState, "#3");
+			table.AcceptChanges ();
+
+			table.LoadDataRow (new object[] {10,2}, true);
+			row = table.Rows.Find (10);
+			Assert.IsNotNull (row, "#4");
+			Assert.AreEqual (2, row [1], "#5");
+			Assert.AreEqual (DataRowState.Unchanged, row.RowState, "#6");
+
+			table.LoadDataRow (new object[] {1,2}, false);
+			row = table.Rows.Find (1);
+			Assert.IsNotNull (row, "#7");
+			Assert.AreEqual (2, row [1], "#8");
+			Assert.AreEqual (DataRowState.Modified, table.Rows.Find (1).RowState, "#9");
+
+			table.LoadDataRow (new object[] {1,3}, true);
+			row = table.Rows.Find (1);
+			Assert.IsNotNull (row, "#10");
+			Assert.AreEqual (3, row [1], "#11");
+			Assert.AreEqual (DataRowState.Unchanged, table.Rows.Find (1).RowState, "#12");
+		}
+
 		[Test]
 		public void EndLoadData_MergeDuplcateValues ()
 		{
@@ -430,6 +483,8 @@ namespace MonoTests_System.Data
 			table.Columns.Add ("col2", typeof (int));
 
 			table.PrimaryKey = new DataColumn[] {table.Columns [0]};
+			table.Rows.Add (new object[] {1, 500});
+			table.AcceptChanges ();
 
 			table.BeginLoadData ();
 			table.LoadDataRow (new object[] {1 , 1}, false);
@@ -2198,6 +2253,182 @@ namespace MonoTests_System.Data
 			dt.Select(dt.Columns[0].ColumnName,dt.Columns[0].ColumnName + "1");
 		}
 
+		[Test]
+		public void Select_NonBooleanFilter ()
+		{
+			DataTable table = new DataTable ();
+			table.Columns.Add ("col1", typeof (int));
+			
+			DataRow[] row = table.Select ("col1*10");
+			Assert.AreEqual (0, row.Length, "#1");
+
+			// No exception shud be thrown 
+			// The filter created earlier (if cached), will raise an EvaluateException
+			// and so shouldnt be cached
+			for (int i=0; i < 5; ++i)
+				table.Rows.Add (new object[] {i});
+
+			try {
+				table.Select ("col1*10");
+				Assert.Fail ("#2 Invalid Filter.. Should return a Boolean result");
+			} catch (EvaluateException) {
+			}
+		}
+
+		[Test]
+		public void Select_BoolColumn ()
+		{
+			DataTable table = new DataTable ();
+			table.Columns.Add ("col1", typeof (int));
+			table.Columns.Add ("col2", typeof (bool));
+
+			for (int i=0; i < 5; i++)
+				table.Rows.Add (new object[] {i});
+
+			DataRow[] result;
+			try {
+				result = table.Select ("col1");
+				Assert.Fail ("#1 col1 is not a boolean expression");
+			} catch (EvaluateException) {
+			}
+
+			//col2 is a boolean expression, and a null value translates to
+			//false.
+			result = table.Select ("col2");
+			Assert.AreEqual (0, result.Length);
+		}
+
+		[Test]
+		public void Select_OrderOfRows ()
+		{
+			DataTable table = new DataTable ();
+			table.Columns.Add ("col1", typeof (int));
+			table.Columns.Add ("col2", typeof (int));
+		
+			for (int i=0; i < 10; i++)
+				table.Rows.Add (new object[] {10-i, i});
+			DataRow[] result = table.Select ("col1 > 5");
+
+			Assert.AreEqual (6, result [0][0], "# incorrect sorting order");
+		}
+#if NET_2_0
+
+		[Test]
+		public void DataTable_Clone ()
+		{
+			DataTable table = new DataTable ();
+			table.Columns.Add ("col1", typeof (DateTime));
+			table.Columns [0].DateTimeMode = DataSetDateTime.Local;
+			DataTable table1 = table.Clone ();
+			Assert.AreEqual (DataSetDateTime.Local, table1.Columns [0].DateTimeMode, "#1");
+
+			//Test any other new prop in 2.0
+		}
+
+		[Test]
+		public void Merge_SchemaTest ()
+		{
+			DataTable table1 = new DataTable ("t1");
+			table1.Columns.Add ("col1", typeof (int));
+
+			DataTable table2 = new DataTable ("t2");
+			table2.Columns.Add ("col2", typeof (int));
+
+			DataTable table3;
+
+			table3 = table1.Clone ();
+			table3.Merge (table2);
+			Assert.AreEqual (2, table3.Columns.Count, "#1");
+			table3 = table1.Clone ();
+			table3.Merge (table2, false, MissingSchemaAction.Ignore);
+			Assert.AreEqual (1, table3.Columns.Count, "#2");
+
+			// source constraints are ignored
+			table2.Constraints.Add ("uc", table2.Columns [0], false);
+			table3 = table1.Clone ();
+			table3.Merge (table2);
+			Assert.AreEqual (0, table3.Constraints.Count, "#3");
+
+			// source PK is merged depending on MissingSchemaAction
+			table2.PrimaryKey = new DataColumn[] {table2.Columns[0]};
+			table3 = table1.Clone ();
+			table3.Merge (table2);
+			Assert.AreEqual (1, table3.Constraints.Count, "#4");
+			table3 = table1.Clone ();
+			table3.Merge (table2, false, MissingSchemaAction.Ignore);
+			Assert.AreEqual (0, table3.Constraints.Count, "#5");
+
+			//FIXME : If both source and target have PK, then 
+			// shud be the exception raised when schema is merged? 
+			// ms.net throws a nullreference exception.
+			// If any data is merged, exception is anyways raised.
+			/*
+			table1.PrimaryKey = new DataColumn[] {table1.Columns[0]};
+			table3 = table1.Clone ();
+			try {
+				table3.Merge(table2);
+				Assert.Fail("#6");
+			} catch (DataException e) {}
+			*/
+
+			table3.Merge (table2,false,MissingSchemaAction.Ignore);
+			table1.PrimaryKey = null;
+
+			// DateTime columns, DataType match only if DateTimeMode matches
+			table1.Columns.Add ("col_datetime", typeof (DateTime));
+			table2.Columns.Add ("col_datetime", typeof (DateTime));
+			table1.Columns ["col_datetime"].DateTimeMode = DataSetDateTime.Local;
+			table2.Columns ["col_datetime"].DateTimeMode = DataSetDateTime.Unspecified;
+
+			table3 = table1.Clone ();
+			try {
+				table3.Merge (table2);
+				Assert.Fail ("#7");
+			} catch (DataException) { }
+
+			table1.Columns ["col_datetime"].DateTimeMode = DataSetDateTime.Unspecified;
+			table2.Columns ["col_datetime"].DateTimeMode = DataSetDateTime.UnspecifiedLocal;
+			table3 = table1.Clone ();
+			table3.Merge (table2);
+			Assert.AreEqual (DataSetDateTime.Unspecified, table3.Columns ["col_datetime"].DateTimeMode, "#9");
+		}
+
+
+		[Test]
+		public void Merge_TestData ()
+		{
+			DataTable t1 = new DataTable ("t1");
+			DataTable t2 = new DataTable ("t2");
+
+			t1.Columns.Add ("c1", typeof (int));
+			t1.Columns.Add ("c2", typeof (int));
+			t2.Columns.Add ("c1", typeof (int));
+			t2.Columns.Add ("c2", typeof (int));
+
+			t1.Rows.Add (new object[] {1, 1});
+			t1.Rows.Add (new object[] {2, 2});
+
+			t2.Rows.Add (new object[] {1, 5});
+			t2.Rows.Add (new object[] {1, 10});
+
+			DataTable t3 = t1.Copy ();
+			// When primary key is not defined, rows are not merged.
+			t3.Merge (t2);
+			Assert.AreEqual (4, t3.Rows.Count, "#1");
+
+			t1.PrimaryKey = new DataColumn[] {t1.Columns[0]};
+			t3 = t1.Copy ();
+			t3.Merge (t2);
+			Assert.AreEqual (2, t3.Rows.Count, "#2");
+			Assert.AreEqual (10, t3.Rows [0][1], "#3");
+
+			t3 = t1.Copy ();
+			t3.Merge (t2, true);
+			Assert.AreEqual (2, t3.Rows.Count, "#4");
+			Assert.AreEqual (1, t3.Rows [0][1], "#5");
+		}
+#endif
+
 		internal class DataRowsComparer : System.Collections.IComparer
 		{
 			#region Memebers
