@@ -378,6 +378,43 @@ namespace MonoTests.System.Data
 			AssertEquals ("#4", 2, table.Columns.Count);
 		}
 
+#if NET_2_0
+		private bool dvInitialized;
+                private void OnDataViewInitialized (object src, EventArgs args)
+                {
+			dvInitialized = true;
+		}
+		[Test]
+		public void BeginInit2 ()
+		{
+			DataTable table = new DataTable ("table");
+			DataView dv = new DataView ();
+			DataColumn col1 = new DataColumn ("col1");
+			DataColumn col2 = new DataColumn ("col2");
+
+			dvInitialized = false;
+
+			dv.Initialized += new EventHandler (OnDataViewInitialized);
+			
+			dv.BeginInit ();
+			table.BeginInit ();
+			table.Columns.AddRange (new DataColumn[] {col1,col2});
+
+			dv.Table = table;
+			AssertNull ("#1", dv.Table);
+			dv.EndInit ();
+
+			dv.Initialized -= new EventHandler (OnDataViewInitialized);
+			
+			AssertEquals ("#2", table, dv.Table);
+			AssertEquals ("#3", 0, table.Columns.Count);
+
+			table.EndInit ();
+			AssertEquals ("#4", 2, table.Columns.Count);
+			AssertEquals("DataViewInitialized #5", dvInitialized, true);
+		}
+#endif
+
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]
 		public void Find_1 ()
@@ -414,6 +451,9 @@ namespace MonoTests.System.Data
 
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
+#if TARGET_JVM
+		[NUnit.Framework.Category ("NotWorking")] // defect 5446
+#endif
 		public void Find_3 ()
 		{
 			dataView.Sort = "itemID, itemName";
