@@ -6,10 +6,7 @@
 //   Dennis Hayes (dennish@Raytek.com)
 //
 // (C) 2002/3 Ximian, Inc
-//
-
-//
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004,2006 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -30,13 +27,12 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-using System;
 
 namespace System.Drawing.Drawing2D {
-	/// <summary>
-	/// Summary description for CompostingMode.
-	/// </summary>
+
+#if ONLY_1_1
 	[Serializable]
+#endif
 	public enum CompositingMode {
 		SourceCopy = 1,
 		SourceOver = 0
