@@ -60,6 +60,9 @@ namespace System.Drawing.Printing
 		[MonoTODO]
 		public override void OnStartPrint(PrintDocument document, PrintEventArgs e)
 		{
+			if (!document.PrinterSettings.IsValid)
+				throw new InvalidPrinterException(document.PrinterSettings);
+		
 			/* maybe we should reuse the images, and clear them? */
 			foreach (PreviewPageInfo pi in pageInfoList)
 				pi.Image.Dispose ();
