@@ -42,7 +42,7 @@ namespace System.Drawing.Printing
 
 		// Methods
 		internal abstract bool StartDoc (GraphicsPrinter gr, string doc_name, string output_file);
-		internal abstract IntPtr CreateGraphicsContext (PrinterSettings settings);
+		internal abstract IntPtr CreateGraphicsContext (PrinterSettings settings, PageSettings page_settings);
 		internal abstract bool StartPage (GraphicsPrinter gr);
 		internal abstract bool EndPage (GraphicsPrinter gr);
 		internal abstract bool EndDoc (GraphicsPrinter gr);
@@ -50,6 +50,8 @@ namespace System.Drawing.Printing
 		internal abstract void LoadPrinterSettings (string printer, PrinterSettings settings);
 		internal abstract void LoadPrinterResolutions (string printer, PrinterSettings settings);
 		internal abstract void LoadPrinterPaperSizes (string printer, PrinterSettings settings);
+		internal abstract void LoadPrinterPaperSources (string printer, PrinterSettings settings);
+		internal abstract bool IsPrinterValid(string printer, bool force);
 
 		//Used from SWF
 		internal abstract void GetPrintDialogInfo (string printer, ref string port, ref string type, ref string status, ref string comment);
