@@ -6,11 +6,7 @@
 //   Ravindra (rkumar@novell.com)
 //
 // (C) Ximian, Inc.  http://www.ximian.com
-// (C) Novell, Inc.  Http://www.novell.com
-//
-
-//
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004,2006 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -32,11 +28,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System;
-using System.Drawing;
-using System.Drawing.Drawing2D;
-using System.Collections;
-
 namespace System.Drawing
 {
 	public abstract class Brush : MarshalByRefObject, ICloneable, IDisposable
@@ -74,23 +65,6 @@ namespace System.Drawing
 
 		}
 #endif
-		
-                internal Brush CreateBrush (IntPtr brush, System.Drawing.BrushType type)
-                {
-                        switch (type) {
-                        case BrushType.BrushTypeSolidColor:
-                                return new SolidBrush (brush);
-
-                        case BrushType.BrushTypeHatchFill:
-                                return new HatchBrush (brush);
-
-                        case BrushType.BrushTypeTextureFill:
-                                return new TextureBrush (brush);
-
-                        default:
-                                throw new NotImplementedException ();
-                        }
-                }
 
 		public void Dispose ()
 		{
@@ -100,7 +74,9 @@ namespace System.Drawing
 
 		protected virtual void Dispose (bool disposing)
 		{
-			if ((GDIPlus.Display != IntPtr.Zero) && (nativeObject != IntPtr.Zero)) {
+			// NOTE: this has been known to fail in the past (cairo)
+			// but it's the only way to reclaim brush related memory
+			if (nativeObject != IntPtr.Zero) {
 				GDIPlus.GdipDeleteBrush (nativeObject);
 				nativeObject = IntPtr.Zero;
 			}			
@@ -112,4 +88,3 @@ namespace System.Drawing
 		}
 	}
 }
-
