@@ -5,10 +5,10 @@
 //	Gonzalo Paniagua Javier (gonzalo@ximian.com) (stubbed out)
 //      Alexandre Pigolkine(pigolkine@gmx.de)
 //	Jordi Mas i Hernandez (jordi@ximian.com)
+//	Sebastien Pouliot  <sebastien@ximian.com>
 //
 // Copyright (C) 2003 Ximian, Inc. (http://www.ximian.com)
-//
-// Copyright (C) 2004-2005 Novell, Inc. (http://www.novell.com)
+// Copyright (C) 2004-2006 Novell, Inc. (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -114,7 +114,7 @@ namespace System.Drawing
 			}
 		}
 		
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void AddMetafileComment (byte [] data)
 		{
 			throw new NotImplementedException ();
@@ -130,7 +130,7 @@ namespace System.Drawing
                         return new GraphicsContainer(state);
 		}
 
-		[MonoTODO ("rectangles and unit aren't supported in libgdiplus")]		
+		[MonoTODO ("The rectangles and unit parameters aren't supported in libgdiplus")]		
 		public GraphicsContainer BeginContainer (Rectangle dstrect, Rectangle srcrect, GraphicsUnit unit)
 		{
 			uint state;
@@ -141,7 +141,7 @@ namespace System.Drawing
 			return new GraphicsContainer (state);
 		}
 
-		[MonoTODO ("rectangles and unit aren't supported in libgdiplus")]		
+		[MonoTODO ("The rectangles and unit parameters aren't supported in libgdiplus")]		
 		public GraphicsContainer BeginContainer (RectangleF dstrect, RectangleF srcrect, GraphicsUnit unit)
 		{
 			uint state;
@@ -1185,217 +1185,217 @@ namespace System.Drawing
 			GDIPlus.CheckStatus (status);
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Point [] destPoints, EnumerateMetafileProc callback)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, RectangleF destRect, EnumerateMetafileProc callback)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, PointF [] destPoints, EnumerateMetafileProc callback)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Rectangle destRect, EnumerateMetafileProc callback)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Point destPoint, EnumerateMetafileProc callback)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, PointF destPoint, EnumerateMetafileProc callback)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, PointF destPoint, EnumerateMetafileProc callback, IntPtr callbackData)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Rectangle destRect, EnumerateMetafileProc callback, IntPtr callbackData)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, PointF [] destPoints, EnumerateMetafileProc callback, IntPtr callbackData)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Point destPoint, EnumerateMetafileProc callback, IntPtr callbackData)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Point [] destPoints, EnumerateMetafileProc callback, IntPtr callbackData)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, RectangleF destRect, EnumerateMetafileProc callback, IntPtr callbackData)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, PointF destPoint, RectangleF srcRect, GraphicsUnit srcUnit, EnumerateMetafileProc callback)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Point destPoint, Rectangle srcRect, GraphicsUnit srcUnit, EnumerateMetafileProc callback)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, PointF [] destPoints, RectangleF srcRect, GraphicsUnit srcUnit, EnumerateMetafileProc callback)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Point [] destPoints, Rectangle srcRect, GraphicsUnit srcUnit, EnumerateMetafileProc callback)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, RectangleF destRect, RectangleF srcRect, GraphicsUnit srcUnit, EnumerateMetafileProc callback)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Rectangle destRect, Rectangle srcRect, GraphicsUnit srcUnit, EnumerateMetafileProc callback)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, RectangleF destRect, EnumerateMetafileProc callback, IntPtr callbackData, ImageAttributes imageAttr)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Point destPoint, EnumerateMetafileProc callback, IntPtr callbackData, ImageAttributes imageAttr)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, PointF destPoint, EnumerateMetafileProc callback, IntPtr callbackData, ImageAttributes imageAttr)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Point [] destPoints, EnumerateMetafileProc callback, IntPtr callbackData, ImageAttributes imageAttr)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, PointF [] destPoints, EnumerateMetafileProc callback, IntPtr callbackData, ImageAttributes imageAttr)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Rectangle destRect, EnumerateMetafileProc callback, IntPtr callbackData, ImageAttributes imageAttr)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Rectangle destRect, Rectangle srcRect, GraphicsUnit srcUnit, EnumerateMetafileProc callback, IntPtr callbackData)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, PointF [] destPoints, RectangleF srcRect, GraphicsUnit srcUnit, EnumerateMetafileProc callback, IntPtr callbackData)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, RectangleF destRect, RectangleF srcRect, GraphicsUnit srcUnit, EnumerateMetafileProc callback, IntPtr callbackData)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, PointF destPoint, RectangleF srcRect, GraphicsUnit srcUnit, EnumerateMetafileProc callback, IntPtr callbackData)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Point destPoint, Rectangle srcRect, GraphicsUnit srcUnit, EnumerateMetafileProc callback, IntPtr callbackData)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Point [] destPoints, Rectangle srcRect, GraphicsUnit srcUnit, EnumerateMetafileProc callback, IntPtr callbackData)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Point [] destPoints, Rectangle srcRect, GraphicsUnit unit, EnumerateMetafileProc callback, IntPtr callbackData, ImageAttributes imageAttr)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Rectangle destRect, Rectangle srcRect, GraphicsUnit unit, EnumerateMetafileProc callback, IntPtr callbackData, ImageAttributes imageAttr)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, Point destPoint, Rectangle srcRect, GraphicsUnit unit, EnumerateMetafileProc callback, IntPtr callbackData, ImageAttributes imageAttr)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, RectangleF destRect, RectangleF srcRect, GraphicsUnit unit, EnumerateMetafileProc callback, IntPtr callbackData, ImageAttributes imageAttr)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, PointF [] destPoints, RectangleF srcRect, GraphicsUnit unit, EnumerateMetafileProc callback, IntPtr callbackData, ImageAttributes imageAttr)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Metafiles, both WMF and EMF formats, aren't supported.")]
 		public void EnumerateMetafile (Metafile metafile, PointF destPoint, RectangleF srcRect, GraphicsUnit unit, EnumerateMetafileProc callback, IntPtr callbackData, ImageAttributes imageAttr)
 		{
 			throw new NotImplementedException ();
@@ -1875,10 +1875,21 @@ namespace System.Drawing
 		}
 
 		
-		public Region [] MeasureCharacterRanges (string text, Font font, RectangleF layoutRect, StringFormat stringFormat)
-		{	
-			Status status;			
+		public Region[] MeasureCharacterRanges (string text, Font font, RectangleF layoutRect, StringFormat stringFormat)
+		{
+			if ((text == null) || (text.Length == 0))
+				return new Region [0];
+
+			if (font == null)
+				throw new ArgumentNullException ("font");
+
+			if (stringFormat == null)
+				throw new ArgumentException ("stringFormat");
+
 			int regcount = stringFormat.GetMeasurableCharacterRangeCount ();
+			if (regcount == 0)
+				return new Region[0];
+
 			IntPtr[] native_regions = new IntPtr [regcount];
 			Region[] regions = new Region [regcount];
 			
@@ -1887,35 +1898,32 @@ namespace System.Drawing
 				native_regions[i] = regions[i].NativeObject;
 			}
 			
-			status =  GDIPlus.GdipMeasureCharacterRanges (nativeObject, text, text.Length,
-				font.NativeObject, ref layoutRect, stringFormat.NativeObject, 
-				regcount, out native_regions[0]); 
-			
+			Status status = GDIPlus.GdipMeasureCharacterRanges (nativeObject, text, text.Length,
+				font.NativeObject, ref layoutRect, stringFormat.NativeObject, regcount, out native_regions[0]); 
 			GDIPlus.CheckStatus (status);				
-							
+
 			return regions;							
 		}
 
 		
 		public SizeF MeasureString (string text, Font font)
 		{
-			return MeasureString (text, font, new Size (0, 0));
+			return MeasureString (text, font, SizeF.Empty);
 		}
 
 		
 		public SizeF MeasureString (string text, Font font, SizeF layoutArea)
 		{
-			int charactersFitted, linesFilled;
-			RectangleF boundingBox = new RectangleF ();
-			RectangleF rect = new RectangleF (0, 0, layoutArea.Width,
-							  layoutArea.Height);
-
 			if (text == null || text.Length == 0)
 				return SizeF.Empty;
 
 			if (font == null)
 				throw new ArgumentNullException ("font");
 
+			int charactersFitted, linesFilled;
+			RectangleF boundingBox = new RectangleF ();
+			RectangleF rect = new RectangleF (0, 0, layoutArea.Width, layoutArea.Height);
+
 			Status status = GDIPlus.GdipMeasureString (nativeObject, text, text.Length,
 								   font.NativeObject, ref rect,
 								   IntPtr.Zero, out boundingBox,
@@ -1928,16 +1936,16 @@ namespace System.Drawing
 		
 		public SizeF MeasureString (string text, Font font, int width)
 		{				
-			RectangleF boundingBox = new RectangleF ();
-			RectangleF rect = new RectangleF (0, 0, width, 999999);
-			int charactersFitted, linesFilled;
-
 			if (text == null || text.Length == 0)
 				return SizeF.Empty;
 
 			if (font == null)
 				throw new ArgumentNullException ("font");
 
+			RectangleF boundingBox = new RectangleF ();
+			RectangleF rect = new RectangleF (0, 0, width, Int32.MaxValue);
+			int charactersFitted, linesFilled;
+
 			Status status = GDIPlus.GdipMeasureString (nativeObject, text, text.Length, 
 								   font.NativeObject, ref rect,
 								   IntPtr.Zero, out boundingBox,
@@ -1960,7 +1968,7 @@ namespace System.Drawing
 		public SizeF MeasureString (string text, Font font, int width, StringFormat format)
 		{
 			int charactersFitted, linesFilled;			
-			return MeasureString (text, font, new SizeF (width, 999999), 
+			return MeasureString (text, font, new SizeF (width, Int32.MaxValue), 
 					      format, out charactersFitted, out linesFilled);
 		}
 
@@ -1968,22 +1976,20 @@ namespace System.Drawing
 		public SizeF MeasureString (string text, Font font, PointF origin,
 					    StringFormat stringFormat)
 		{
-			RectangleF boundingBox = new RectangleF ();
-			RectangleF rect = new RectangleF (origin.X, origin.Y, 0, 0);
-			int charactersFitted, linesFilled;
-
 			if (text == null || text.Length == 0)
 				return SizeF.Empty;
 
 			if (font == null)
 				throw new ArgumentNullException ("font");
 
-			if (stringFormat == null)
-				stringFormat = new StringFormat ();
+			RectangleF boundingBox = new RectangleF ();
+			RectangleF rect = new RectangleF (origin.X, origin.Y, 0, 0);
+			int charactersFitted, linesFilled;
+
+			IntPtr format = (stringFormat == null) ? IntPtr.Zero : stringFormat.NativeObject;
 
 			Status status = GDIPlus.GdipMeasureString (nativeObject, text, text.Length, 
-								   font.NativeObject, ref rect,
-								   stringFormat.NativeObject, 
+								   font.NativeObject, ref rect, format, 
 								   out boundingBox,
 								   out charactersFitted,
 								   out linesFilled);
@@ -1997,8 +2003,6 @@ namespace System.Drawing
 					    StringFormat stringFormat, out int charactersFitted,
 					    out int linesFilled)
 		{	
-			RectangleF boundingBox = new RectangleF ();
-			RectangleF rect = new RectangleF (0, 0, layoutArea.Width, layoutArea.Height);
 			charactersFitted = 0;
 			linesFilled = 0;
 
@@ -2008,12 +2012,13 @@ namespace System.Drawing
 			if (font == null)
 				throw new ArgumentNullException ("font");
 
-			if (stringFormat == null)
-				stringFormat = new StringFormat ();
+			RectangleF boundingBox = new RectangleF ();
+			RectangleF rect = new RectangleF (0, 0, layoutArea.Width, layoutArea.Height);
+
+			IntPtr format = (stringFormat == null) ? IntPtr.Zero : stringFormat.NativeObject;
 
 			Status status = GDIPlus.GdipMeasureString (nativeObject, text, text.Length, 
-								   font.NativeObject, ref rect,
-								   stringFormat.NativeObject,
+								   font.NativeObject, ref rect, format,
 								   out boundingBox,
 								   out charactersFitted,
 								   out linesFilled);
@@ -2056,7 +2061,7 @@ namespace System.Drawing
 		}
 #endif
 
-		[MonoTODO]
+		[MonoTODO ("Parameter hdc doesn't map well into libgdiplus")]
 #if NET_2_0
 		[EditorBrowsable (EditorBrowsableState.Never)]
 #else
@@ -2372,7 +2377,7 @@ namespace System.Drawing
 			}
 		}
 
-		[MonoTODO ("not supported by libgdiplus")]
+		[MonoTODO ("This property does not do anything when used with libgdiplus.")]
 		public PixelOffsetMode PixelOffsetMode {
 			get {
 			        PixelOffsetMode pixelOffset = PixelOffsetMode.Invalid;
@@ -2416,7 +2421,7 @@ namespace System.Drawing
 			}
 		}
 
-		[MonoTODO ("not supported by libgdiplus")]
+		[MonoTODO ("This property does not do anything when used with libgdiplus.")]
 		public int TextContrast {
 			get {	
                                 int contrast;
