@@ -5,11 +5,7 @@
 //   Mike Kestner (mkestner@speakeasy.net)
 //
 // Copyright (C) 2001 Mike Kestner
-// Copyright (C) 2004 Novell, Inc. http://www.novell.com
-//
-
-//
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004,2006 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -72,6 +68,12 @@ namespace System.Drawing
 		{
 			return new PointF (pt.X + sz.Width, pt.Y + sz.Height);
 		}
+#if NET_2_0
+		public static PointF operator + (PointF pt, SizeF sz)
+		{
+			return new PointF (pt.X + sz.Width, pt.Y + sz.Height);
+		}
+#endif
 		
 		/// <summary>
 		///	Equality Operator
@@ -116,6 +118,12 @@ namespace System.Drawing
 		{
 			return new PointF (pt.X - sz.Width, pt.Y - sz.Height);
 		}
+#if NET_2_0
+		public static PointF operator - (PointF pt, SizeF sz)
+		{
+			return new PointF (pt.X - sz.Width, pt.Y - sz.Height);
+		}
+#endif
 		
 		// -----------------------
 		// Public Constructor
