@@ -3,7 +3,7 @@
 //
 // Author:
 //
-// 	 Jordi Mas i Hernndez (jmas@softcatala.org>
+// 	 Jordi Mas i Hernàndez (jmas@softcatala.org>
 //	 Jonathan Gilbert <logic@deltaq.org>
 //
 // (C) 2004 Ximian, Inc.  http://www.ximian.com
@@ -42,6 +42,9 @@ namespace MonoTests.System.Drawing{
 
 	[TestFixture]
 	[SecurityPermission (SecurityAction.Deny, UnmanagedCode = true)]
+#if TARGET_JVM
+	[Category ("NotWorking")]
+#endif
 	public class TestBitmap {
 		
 		[Test]
@@ -60,24 +63,200 @@ namespace MonoTests.System.Drawing{
 		}
 
 		[Test]
-		public void LockBits_NonIndexedWrite ()
+		public void LockBits_32_32_NonIndexedWrite ()
 		{
-			Bitmap bmp= new Bitmap(100,100, PixelFormat.Format32bppRgb);
-			Rectangle rect = new Rectangle(0, 0, bmp.Width, bmp.Height);
-                        BitmapData mybitmapdata = bmp.LockBits(rect, ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
-			bmp.UnlockBits (mybitmapdata);
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format32bppRgb)) {
+				Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+				BitmapData data = bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format32bppRgb);
+				Assert.AreEqual (100, data.Height, "Height");
+				Assert.AreEqual (PixelFormat.Format32bppRgb, data.PixelFormat, "PixelFormat");
+				Assert.AreEqual (400, data.Stride, "Stride");
+				Assert.AreEqual (100, data.Width, "Width");
+				bmp.UnlockBits (data);
+			}
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void LockBits_32_24_NonIndexedWrite ()
+		{
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format32bppRgb)) {
+				Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+				BitmapData data = bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
+				Assert.AreEqual (100, data.Height, "Height");
+				Assert.AreEqual (PixelFormat.Format24bppRgb, data.PixelFormat, "PixelFormat");
+				Assert.AreEqual (300, data.Stride, "Stride");
+				Assert.AreEqual (100, data.Width, "Width");
+				bmp.UnlockBits (data);
+			}
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void LockBits_24_24_NonIndexedWrite ()
+		{
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format24bppRgb)) {
+				Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+				BitmapData data = bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
+				Assert.AreEqual (100, data.Height, "Height");
+				Assert.AreEqual (PixelFormat.Format24bppRgb, data.PixelFormat, "PixelFormat");
+				Assert.AreEqual (300, data.Stride, "Stride");
+				Assert.AreEqual (100, data.Width, "Width");
+				bmp.UnlockBits (data);
+			}
+		}
+
+		[Test]
+		public void LockBits_24_32_NonIndexedWrite ()
+		{
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format24bppRgb)) {
+				Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+				BitmapData data = bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format32bppRgb);
+				Assert.AreEqual (100, data.Height, "Height");
+				Assert.AreEqual (PixelFormat.Format32bppRgb, data.PixelFormat, "PixelFormat");
+				Assert.AreEqual (400, data.Stride, "Stride");
+				Assert.AreEqual (100, data.Width, "Width");
+				bmp.UnlockBits (data);
+			}
 		}
 
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
-		public void LockBits_IndexedWrite ()
+		public void LockBits_IndexedWrite_NonIndexed ()
 		{
-			Bitmap bmp= new Bitmap(100,100, PixelFormat.Format8bppIndexed);
-			Rectangle rect = new Rectangle(0, 0, bmp.Width, bmp.Height);
-                        BitmapData mybitmapdata = bmp.LockBits(rect, ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
-			bmp.UnlockBits (mybitmapdata);
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format8bppIndexed)) {
+				Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+				bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
+			}
 		}
 
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void LockBits_NonIndexedWrite_ToIndexed ()
+		{
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format32bppRgb)) {
+				Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+				bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format8bppIndexed);
+			}
+		}
+
+		[Test]
+		public void LockBits_IndexedWrite_SameIndexedFormat ()
+		{
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format8bppIndexed)) {
+				Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+				BitmapData data = bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format8bppIndexed);
+				Assert.AreEqual (100, data.Height, "Height");
+				Assert.AreEqual (PixelFormat.Format8bppIndexed, data.PixelFormat, "PixelFormat");
+				Assert.AreEqual (100, data.Stride, "Stride");
+				Assert.AreEqual (100, data.Width, "Width");
+				bmp.UnlockBits (data);
+			}
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void LockBits_Disposed ()
+		{
+			Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format32bppRgb);
+			Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+			bmp.Dispose ();
+			bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void UnlockBits_Disposed ()
+		{
+			Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format32bppRgb);
+			Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+			BitmapData data = bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format32bppRgb);
+			bmp.Dispose ();
+			bmp.UnlockBits (data);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void UnlockBits_Null ()
+		{
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format32bppRgb)) {
+				bmp.UnlockBits (null);
+			}
+		}
+#if NET_2_0
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void LockBits_BitmapData_Null ()
+		{
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format32bppRgb)) {
+				Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+				bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb, null);
+			}
+		}
+
+		[Test]
+		public void LockBits_32_32_BitmapData ()
+		{
+			BitmapData data = new BitmapData ();
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format32bppRgb)) {
+				Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+				bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format32bppRgb, data);
+				Assert.AreEqual (100, data.Height, "Height");
+				Assert.AreEqual (PixelFormat.Format32bppRgb, data.PixelFormat, "PixelFormat");
+				Assert.AreEqual (400, data.Stride, "Stride");
+				Assert.AreEqual (100, data.Width, "Width");
+				bmp.UnlockBits (data);
+			}
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void LockBits_32_24_BitmapData ()
+		{
+			BitmapData data = new BitmapData ();
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format32bppRgb)) {
+				Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+				bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb, data);
+				Assert.AreEqual (100, data.Height, "Height");
+				Assert.AreEqual (PixelFormat.Format24bppRgb, data.PixelFormat, "PixelFormat");
+				Assert.AreEqual (300, data.Stride, "Stride");
+				Assert.AreEqual (100, data.Width, "Width");
+				bmp.UnlockBits (data);
+			}
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void LockBits_24_24_BitmapData ()
+		{
+			BitmapData data = new BitmapData ();
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format24bppRgb)) {
+				Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+				bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb, data);
+				Assert.AreEqual (100, data.Height, "Height");
+				Assert.AreEqual (PixelFormat.Format24bppRgb, data.PixelFormat, "PixelFormat");
+				Assert.AreEqual (300, data.Stride, "Stride");
+				Assert.AreEqual (100, data.Width, "Width");
+				bmp.UnlockBits (data);
+			}
+		}
+
+		[Test]
+		public void LockBits_24_32_BitmapData ()
+		{
+			BitmapData data = new BitmapData ();
+			using (Bitmap bmp = new Bitmap (100, 100, PixelFormat.Format24bppRgb)) {
+				Rectangle rect = new Rectangle (0, 0, bmp.Width, bmp.Height);
+				bmp.LockBits (rect, ImageLockMode.ReadWrite, PixelFormat.Format32bppRgb, data);
+				Assert.AreEqual (100, data.Height, "Height");
+				Assert.AreEqual (PixelFormat.Format32bppRgb, data.PixelFormat, "PixelFormat");
+				Assert.AreEqual (400, data.Stride, "Stride");
+				Assert.AreEqual (100, data.Width, "Width");
+				bmp.UnlockBits (data);
+			}
+		}
+#endif
+
 		/* Get the output directory depending on the runtime and location*/
 		public static string getOutSubDir()
 		{				
@@ -103,13 +282,9 @@ namespace MonoTests.System.Drawing{
 		/* Get the input directory depending on the runtime*/
 		public static string getInFile(string file)
 		{				
-			string sRslt;						
-			
-			sRslt = Path.GetFullPath (file);
-				
-			if (File.Exists(file)==false) 
-				sRslt = "Test/System.Drawing/" + file;							
-			
+			string sRslt = Path.GetFullPath ("../System.Drawing/" + file);
+			if (!File.Exists (sRslt))
+				sRslt = "Test/System.Drawing/" + file;
 			return sRslt;
 		}
 
@@ -297,9 +472,12 @@ namespace MonoTests.System.Drawing{
 			resulting pixels using MD5
 		*/
 		[Test]
-		[Ignore ("this fails when using MS GDI+ - with or without Mono")]
 		public void Rotate1bit4bit()
 		{
+			if ((Environment.OSVersion.Platform != (PlatformID)4)
+			 && (Environment.OSVersion.Platform != (PlatformID)128))
+				Assert.Ignore("This fails with Microsoft's GDIPLUS.DLL due to off-by-1 errors in their GdipBitmapRotateFlip function.");
+
 			string[] files = {
 			                   getInFile ("bitmaps/1bit.png"),
 			                   getInFile ("bitmaps/4bit.png")
@@ -476,14 +654,23 @@ namespace MonoTests.System.Drawing{
 			Color red  = Color.FromArgb (Color.Red.A,  Color.Red.R,  Color.Red.G,  Color.Red.B);
 			Color blue = Color.FromArgb (Color.Blue.A, Color.Blue.R, Color.Blue.G, Color.Blue.B);
 
-			using (Bitmap bmp = new Bitmap (1, 1, PixelFormat.Format24bppRgb))
+			using (Bitmap bmp = new Bitmap (1, 1, PixelFormat.Format32bppRgb))
 			{
 				bmp.SetPixel (0, 0, red);
 				pixel_colour = bmp.GetPixel (0, 0);
 				Assert.AreEqual (red, pixel_colour, "Set/Get-Red");
 
 				data = bmp.LockBits (new Rectangle (0, 0, 1, 1), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
-				pixel_value = Marshal.ReadInt32 (data.Scan0);
+
+				// Marshal follows CPU endianess
+				if (BitConverter.IsLittleEndian) {
+					pixel_value = Marshal.ReadInt32 (data.Scan0);
+				} else {
+					pixel_value = Marshal.ReadByte (data.Scan0, 0);
+					pixel_value |= Marshal.ReadByte (data.Scan0, 1) << 8;
+					pixel_value |= Marshal.ReadByte (data.Scan0, 2) << 16;
+					pixel_value |= Marshal.ReadByte (data.Scan0, 3) << 24;
+				}
 				pixel_colour = Color.FromArgb (pixel_value);
 
 				// Disregard alpha information in the test
