@@ -31,10 +31,12 @@
 using NUnit.Framework;
 using System;
 using System.Drawing;
+using System.Security.Permissions;
 
 namespace MonoTests.System.Drawing
 {
 	[TestFixture]
+	[SecurityPermission (SecurityAction.Deny, UnmanagedCode = true)]
 	public class BrushesTest : Assertion
 	{
 		[SetUp]
@@ -42,6 +44,15 @@ namespace MonoTests.System.Drawing
 
 		[TearDown]
 		public void TearDown () { }
+		
+		[Test]
+		public void TestEquals ()
+		{
+			Brush brush1 = Brushes.Blue;
+			Brush brush2 = Brushes.Blue;
+			
+			AssertEquals ("Equals", true, brush1.Equals (brush2));			
+		}
 
 		[Test]
 		public void TestProperties ()
