@@ -38,9 +38,28 @@ using System.Reflection;
 namespace MonoTests.System.Drawing
 {
 	[TestFixture]
-	public class GraphicsTest : Assertion
-	{
+	public class GraphicsTest {
+
 		private RectangleF[] rects;
+		private Font font;
+
+		[TestFixtureSetUp]
+		public void FixtureSetUp ()
+		{
+			try {
+				font = new Font ("Arial", 12);
+			}
+			catch {
+			}
+		}
+
+		[TestFixtureTearDown]
+		public void FixtureTearDown ()
+		{
+			if (font != null)
+				font.Dispose ();
+		}
+		
 
 		private bool IsEmptyBitmap (Bitmap bitmap, out int x, out int y)
 		{
@@ -74,14 +93,24 @@ namespace MonoTests.System.Drawing
 		{
 			int x, y;
 			if (!IsEmptyBitmap (bitmap, out x, out y))
-				Fail (String.Format ("Position {0},{1}", x, y));
+				Assert.Fail (String.Format ("Position {0},{1}", x, y));
 		}
 
 		private void CheckForNonEmptyBitmap (Bitmap bitmap)
 		{
 			int x, y;
 			if (IsEmptyBitmap (bitmap, out x, out y))
-				Fail ("Bitmap was empty");
+				Assert.Fail ("Bitmap was empty");
+		}
+
+		private void AssertEquals (string msg, object expected, object actual)
+		{
+			Assert.AreEqual (expected, actual, msg);
+		}
+
+		private void AssertEquals (string msg, double expected, double actual, double delta)
+		{
+			Assert.AreEqual (expected, actual, delta, msg);
 		}
 
 		[Test]
@@ -155,10 +184,10 @@ namespace MonoTests.System.Drawing
 		{
 			Bitmap bmp = new Bitmap (200, 200);
 			Graphics g = Graphics.FromImage (bmp);
-			Assert ("IsInfinite", g.Clip.IsInfinite (g));
+			Assert.IsTrue (g.Clip.IsInfinite (g), "IsInfinite");
 			g.Clip.IsEmpty (g);
-			Assert ("!IsEmpty", !g.Clip.IsEmpty (g));
-			Assert ("IsInfinite-2", g.Clip.IsInfinite (g));
+			Assert.IsFalse (g.Clip.IsEmpty (g), "!IsEmpty");
+			Assert.IsTrue (g.Clip.IsInfinite (g), "IsInfinite-2");
 		}
 
 		[Test]
@@ -373,7 +402,7 @@ namespace MonoTests.System.Drawing
 			AssertEquals ("Y", 0, bounds.Y);
 			AssertEquals ("Width", 16, bounds.Width);
 			AssertEquals ("Height", 16, bounds.Height);
-			Assert ("Identity", g.Transform.IsIdentity);
+			Assert.IsTrue (g.Transform.IsIdentity, "Identity");
 			g.Dispose ();
 		}
 
@@ -403,7 +432,7 @@ namespace MonoTests.System.Drawing
 			AssertEquals ("reset.Y", 0, bounds.Y);
 			AssertEquals ("reset.Width", 16, bounds.Width);
 			AssertEquals ("reset.Height", 16, bounds.Height);
-			Assert ("Identity", g.Transform.IsIdentity);
+			Assert.IsTrue (g.Transform.IsIdentity, "Identity");
 			g.Dispose ();
 		}
 
@@ -412,7 +441,7 @@ namespace MonoTests.System.Drawing
 		public void Transform_NonInvertibleMatrix ()
 		{
 			Matrix matrix = new Matrix (123, 24, 82, 16, 47, 30);
-			Assert ("IsInvertible", !matrix.IsInvertible);
+			Assert.IsFalse (matrix.IsInvertible, "IsInvertible");
 			Graphics g = Get (16, 16);
 			g.Transform = matrix;
 		}
@@ -423,7 +452,7 @@ namespace MonoTests.System.Drawing
 		public void Multiply_NonInvertibleMatrix ()
 		{
 			Matrix matrix = new Matrix (123, 24, 82, 16, 47, 30);
-			Assert ("IsInvertible", !matrix.IsInvertible);
+			Assert.IsFalse (matrix.IsInvertible, "IsInvertible");
 			Graphics g = Get (16, 16);
 			g.MultiplyTransform (matrix);
 		}
@@ -518,7 +547,7 @@ namespace MonoTests.System.Drawing
 			g.ResetClip ();
 			// see next test for ClipBounds
 			CheckBounds ("resetclip.Clip.GetBounds", g.Clip.GetBounds (g), -4194304, -4194304, 8388608, 8388608);
-			Assert ("IsInfinite", g.Clip.IsInfinite (g));
+			Assert.IsTrue (g.Clip.IsInfinite (g), "IsInfinite");
 		}
 
 		[Test]
@@ -988,7 +1017,7 @@ namespace MonoTests.System.Drawing
 
 		private void CheckDefaultProperties (string message, Graphics g)
 		{
-			Assert (message + ".Clip.IsInfinite", g.Clip.IsInfinite (g));
+			Assert.IsTrue (g.Clip.IsInfinite (g), message + ".Clip.IsInfinite");
 			AssertEquals (message + ".CompositingMode", CompositingMode.SourceOver, g.CompositingMode);
 			AssertEquals (message + ".CompositingQuality", CompositingQuality.Default, g.CompositingQuality);
 			AssertEquals (message + ".InterpolationMode", InterpolationMode.Bilinear, g.InterpolationMode);
@@ -998,12 +1027,12 @@ namespace MonoTests.System.Drawing
 			AssertEquals (message + ".SmoothingMode", SmoothingMode.None, g.SmoothingMode);
 			AssertEquals (message + ".TextContrast", 4, g.TextContrast);
 			AssertEquals (message + ".TextRenderingHint", TextRenderingHint.SystemDefault, g.TextRenderingHint);
-			Assert (message + ".Transform.IsIdentity", g.Transform.IsIdentity);
+			Assert.IsTrue (g.Transform.IsIdentity, message + ".Transform.IsIdentity");
 		}
 
 		private void CheckCustomProperties (string message, Graphics g)
 		{
-			Assert (message + ".Clip.IsInfinite", !g.Clip.IsInfinite (g));
+			Assert.IsFalse (g.Clip.IsInfinite (g), message + ".Clip.IsInfinite");
 			AssertEquals (message + ".CompositingMode", CompositingMode.SourceCopy, g.CompositingMode);
 			AssertEquals (message + ".CompositingQuality", CompositingQuality.HighQuality, g.CompositingQuality);
 			AssertEquals (message + ".InterpolationMode", InterpolationMode.HighQualityBicubic, g.InterpolationMode);
@@ -1014,7 +1043,7 @@ namespace MonoTests.System.Drawing
 			AssertEquals (message + ".SmoothingMode", SmoothingMode.AntiAlias, g.SmoothingMode);
 			AssertEquals (message + ".TextContrast", 0, g.TextContrast);
 			AssertEquals (message + ".TextRenderingHint", TextRenderingHint.AntiAlias, g.TextRenderingHint);
-			Assert (message + ".Transform.IsIdentity", !g.Transform.IsIdentity);
+			Assert.IsFalse (g.Transform.IsIdentity, message + ".Transform.IsIdentity");
 		}
 
 		private void CheckMatrix (string message, Matrix m, float xx, float yx, float xy, float yy, float x0, float y0)
@@ -1298,6 +1327,616 @@ namespace MonoTests.System.Drawing
 			}
 		}
 
+		[Test]
+		public void MeasureString_StringFont ()
+		{
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					SizeF size = g.MeasureString (null, font);
+					Assert.IsTrue (size.IsEmpty, "MeasureString(null,font)");
+					size = g.MeasureString (String.Empty, font);
+					Assert.IsTrue (size.IsEmpty, "MeasureString(empty,font)");
+					// null font
+					size = g.MeasureString (null, null);
+					Assert.IsTrue (size.IsEmpty, "MeasureString(null,null)");
+					size = g.MeasureString (String.Empty, null);
+					Assert.IsTrue (size.IsEmpty, "MeasureString(empty,null)");
+				}
+			}
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void MeasureString_StringFont_Null ()
+		{
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					g.MeasureString ("a", null);
+				}
+			}
+		}
+
+		[Test]
+		public void MeasureString_StringFontSizeF ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					SizeF size = g.MeasureString ("a", font, SizeF.Empty);
+					Assert.IsFalse (size.IsEmpty, "MeasureString(a,font,empty)");
+
+					size = g.MeasureString (String.Empty, font, SizeF.Empty);
+					Assert.IsTrue (size.IsEmpty, "MeasureString(empty,font,empty)");
+				}
+			}
+		}
+
+		private void MeasureString_StringFontInt (string s)
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					SizeF size0 = g.MeasureString (s, font, 0);
+					SizeF sizeN = g.MeasureString (s, font, Int32.MinValue);
+					SizeF sizeP = g.MeasureString (s, font, Int32.MaxValue);
+					Assert.AreEqual (size0, sizeN, "0-Min");
+					Assert.AreEqual (size0, sizeP, "0-Max");
+				}
+			}
+		}
+
+		[Test]
+		public void MeasureString_StringFontInt_ShortString ()
+		{
+			MeasureString_StringFontInt ("a");
+		}
+
+		[Test]
+		public void MeasureString_StringFontInt_LongString ()
+		{
+			MeasureString_StringFontInt ("A very long string..."); // see bug #79643
+		}
+
+		[Test]
+		public void MeasureString_StringFormat_Alignment ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			string text = "Hello Mono::";
+			StringFormat string_format = new StringFormat ();
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					string_format.Alignment = StringAlignment.Near;
+					SizeF near = g.MeasureString (text, font, Int32.MaxValue, string_format);
+
+					string_format.Alignment = StringAlignment.Center;
+					SizeF center = g.MeasureString (text, font, Int32.MaxValue, string_format);
+
+					string_format.Alignment = StringAlignment.Far;
+					SizeF far = g.MeasureString (text, font, Int32.MaxValue, string_format);
+
+					Assert.AreEqual (near.Width, center.Width, 0.1, "near-center/Width");
+					Assert.AreEqual (near.Height, center.Height, 0.1, "near-center/Height");
+
+					Assert.AreEqual (center.Width, far.Width, 0.1, "center-far/Width");
+					Assert.AreEqual (center.Height, far.Height, 0.1, "center-far/Height");
+				}
+			}
+		}
+
+		[Test]
+		public void MeasureString_StringFormat_Alignment_DirectionVertical ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			string text = "Hello Mono::";
+			StringFormat string_format = new StringFormat ();
+			string_format.FormatFlags = StringFormatFlags.DirectionVertical;
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					string_format.Alignment = StringAlignment.Near;
+					SizeF near = g.MeasureString (text, font, Int32.MaxValue, string_format);
+
+					string_format.Alignment = StringAlignment.Center;
+					SizeF center = g.MeasureString (text, font, Int32.MaxValue, string_format);
+
+					string_format.Alignment = StringAlignment.Far;
+					SizeF far = g.MeasureString (text, font, Int32.MaxValue, string_format);
+
+					Assert.AreEqual (near.Width, center.Width, 0.1, "near-center/Width");
+					Assert.AreEqual (near.Height, center.Height, 0.1, "near-center/Height");
+
+					Assert.AreEqual (center.Width, far.Width, 0.1, "center-far/Width");
+					Assert.AreEqual (center.Height, far.Height, 0.1, "center-far/Height");
+				}
+			}
+		}
+
+		[Test]
+		public void MeasureString_StringFormat_LineAlignment ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			string text = "Hello Mono::";
+			StringFormat string_format = new StringFormat ();
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					string_format.LineAlignment = StringAlignment.Near;
+					SizeF near = g.MeasureString (text, font, Int32.MaxValue, string_format);
+
+					string_format.LineAlignment = StringAlignment.Center;
+					SizeF center = g.MeasureString (text, font, Int32.MaxValue, string_format);
+
+					string_format.LineAlignment = StringAlignment.Far;
+					SizeF far = g.MeasureString (text, font, Int32.MaxValue, string_format);
+
+					Assert.AreEqual (near.Width, center.Width, 0.1, "near-center/Width");
+					Assert.AreEqual (near.Height, center.Height, 0.1, "near-center/Height");
+
+					Assert.AreEqual (center.Width, far.Width, 0.1, "center-far/Width");
+					Assert.AreEqual (center.Height, far.Height, 0.1, "center-far/Height");
+				}
+			}
+		}
+
+		[Test]
+		public void MeasureString_StringFormat_LineAlignment_DirectionVertical ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			string text = "Hello Mono::";
+			StringFormat string_format = new StringFormat ();
+			string_format.FormatFlags = StringFormatFlags.DirectionVertical;
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					string_format.LineAlignment = StringAlignment.Near;
+					SizeF near = g.MeasureString (text, font, Int32.MaxValue, string_format);
+
+					string_format.LineAlignment = StringAlignment.Center;
+					SizeF center = g.MeasureString (text, font, Int32.MaxValue, string_format);
+
+					string_format.LineAlignment = StringAlignment.Far;
+					SizeF far = g.MeasureString (text, font, Int32.MaxValue, string_format);
+
+					Assert.AreEqual (near.Width, center.Width, 0.1, "near-center/Width");
+					Assert.AreEqual (near.Height, center.Height, 0.1, "near-center/Height");
+
+					Assert.AreEqual (center.Width, far.Width, 0.1, "center-far/Width");
+					Assert.AreEqual (center.Height, far.Height, 0.1, "center-far/Height");
+				}
+			}
+		}
+
+		[Test]
+		public void MeasureString_Bug76664 ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					string s = "aaa aa aaaa a aaa";
+					SizeF size = g.MeasureString (s, font);
+
+					int chars, lines;
+					SizeF size2 = g.MeasureString (s, font, new SizeF (80, size.Height), null, out chars, out lines);
+
+					// in pixels
+					Assert.IsTrue (size2.Width < size.Width, "Width/pixel");
+					Assert.AreEqual (size2.Height, size.Height, "Height/pixel");
+
+					Assert.AreEqual (1, lines, "lines fitted");
+					// LAMESPEC: documentation seems to suggest chars is total length
+					Assert.IsTrue (chars < s.Length, "characters fitted");
+				}
+			}
+		}
+
+		[Test]
+		public void MeasureCharacterRanges_NullOrEmptyText ()
+		{
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					Region[] regions = g.MeasureCharacterRanges (null, font, new RectangleF (), null);
+					AssertEquals ("text null", 0, regions.Length);
+					regions = g.MeasureCharacterRanges (String.Empty, font, new RectangleF (), null);
+					AssertEquals ("text empty", 0, regions.Length);
+					// null font is ok with null or empty string
+					regions = g.MeasureCharacterRanges (null, null, new RectangleF (), null);
+					AssertEquals ("text null/null font", 0, regions.Length);
+					regions = g.MeasureCharacterRanges (String.Empty, null, new RectangleF (), null);
+					AssertEquals ("text empty/null font", 0, regions.Length);
+				}
+			}
+		}
+
+		[Test]
+		public void MeasureCharacterRanges_EmptyStringFormat ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					// string format without character ranges
+					Region[] regions = g.MeasureCharacterRanges ("Mono", font, new RectangleF (), new StringFormat ());
+					AssertEquals ("empty stringformat", 0, regions.Length);
+				}
+			}
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void MeasureCharacterRanges_FontNull ()
+		{
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					g.MeasureCharacterRanges ("a", null, new RectangleF (), null);
+				}
+			}
+		}
+
+		[Test] // adapted from bug #78777
+		public void MeasureCharacterRanges_TwoLines ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			string text = "this\nis a test";
+			CharacterRange[] ranges = new CharacterRange [2];
+			ranges[0] = new CharacterRange (0,5);
+			ranges[1] = new CharacterRange (5,9);
+
+			StringFormat string_format = new StringFormat ();
+			string_format.FormatFlags = StringFormatFlags.NoClip;
+			string_format.SetMeasurableCharacterRanges (ranges);
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					SizeF size = g.MeasureString (text, font, new Point (0,0), string_format);
+					RectangleF layout_rect = new RectangleF (0.0f, 0.0f, size.Width, size.Height);			
+					Region[] regions = g.MeasureCharacterRanges (text, font, layout_rect, string_format);
+
+					AssertEquals ("Length", 2, regions.Length);
+					AssertEquals ("Height", regions[0].GetBounds (g).Height, regions[1].GetBounds (g).Height);
+				}
+			}
+		}
+
+		private void MeasureCharacterRanges (string text, int first, int length)
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			CharacterRange[] ranges = new CharacterRange[1];
+			ranges[0] = new CharacterRange (first, length);
+
+			StringFormat string_format = new StringFormat ();
+			string_format.FormatFlags = StringFormatFlags.NoClip;
+			string_format.SetMeasurableCharacterRanges (ranges);
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					SizeF size = g.MeasureString (text, font, new Point (0, 0), string_format);
+					RectangleF layout_rect = new RectangleF (0.0f, 0.0f, size.Width, size.Height);
+					g.MeasureCharacterRanges (text, font, layout_rect, string_format);
+				}
+			}
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void MeasureCharacterRanges_FirstTooFar ()
+		{
+			string text = "this\nis a test";
+			MeasureCharacterRanges (text, text.Length, 1);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void MeasureCharacterRanges_LengthTooLong ()
+		{
+			string text = "this\nis a test";
+			MeasureCharacterRanges (text, 0, text.Length + 1);
+		}
+
+		[Test]
+		public void MeasureCharacterRanges_Prefix ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			string text = "Hello &Mono::";
+			CharacterRange[] ranges = new CharacterRange[1];
+			ranges[0] = new CharacterRange (5, 4);
+
+			StringFormat string_format = new StringFormat ();
+			string_format.SetMeasurableCharacterRanges (ranges);
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					SizeF size = g.MeasureString (text, font, new Point (0, 0), string_format);
+					RectangleF layout_rect = new RectangleF (0.0f, 0.0f, size.Width, size.Height);
+
+					// here & is part of the measure and visible
+					string_format.HotkeyPrefix = HotkeyPrefix.None;
+					Region[] regions = g.MeasureCharacterRanges (text, font, layout_rect, string_format);
+					RectangleF bounds_none = regions[0].GetBounds (g);
+
+					// here & is part of the measure (range) but visible as an underline
+					string_format.HotkeyPrefix = HotkeyPrefix.Show;
+					regions = g.MeasureCharacterRanges (text, font, layout_rect, string_format);
+					RectangleF bounds_show = regions[0].GetBounds (g);
+					Assert.IsTrue (bounds_show.Width < bounds_none.Width, "Show<None");
+
+					// here & is part of the measure (range) but invisible
+					string_format.HotkeyPrefix = HotkeyPrefix.Hide;
+					regions = g.MeasureCharacterRanges (text, font, layout_rect, string_format);
+					RectangleF bounds_hide = regions[0].GetBounds (g);
+					AssertEquals ("Hide==None", bounds_hide.Width, bounds_show.Width);
+				}
+			}
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void MeasureCharacterRanges_NullStringFormat ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					g.MeasureCharacterRanges ("Mono", font, new RectangleF (), null);
+				}
+			}
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void MeasureCharacterRanges_StringFormat_Alignment ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			string text = "Hello Mono::";
+			CharacterRange[] ranges = new CharacterRange[1];
+			ranges[0] = new CharacterRange (5, 4);
+			StringFormat string_format = new StringFormat ();
+			string_format.SetMeasurableCharacterRanges (ranges);
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					string_format.Alignment = StringAlignment.Near;
+					Region[] regions = g.MeasureCharacterRanges (text, font, new RectangleF (0, 0, 320, 32), string_format);
+					Assert.AreEqual (1, regions.Length, "Near.Region");
+					RectangleF near = regions[0].GetBounds (g);
+
+					string_format.Alignment = StringAlignment.Center;
+					regions = g.MeasureCharacterRanges (text, font, new RectangleF (0, 0, 320, 32), string_format);
+					Assert.AreEqual (1, regions.Length, "Center.Region");
+					RectangleF center = regions[0].GetBounds (g);
+
+					string_format.Alignment = StringAlignment.Far;
+					regions = g.MeasureCharacterRanges (text, font, new RectangleF (0, 0, 320, 32), string_format);
+					Assert.AreEqual (1, regions.Length, "Far.Region");
+					RectangleF far = regions[0].GetBounds (g);
+
+					Assert.IsTrue (near.X < center.X, "near-center/X");
+					Assert.AreEqual (near.Y, center.Y, 0.1, "near-center/Y");
+					Assert.AreEqual (near.Width, center.Width, 0.1, "near-center/Width");
+					Assert.AreEqual (near.Height, center.Height, 0.1, "near-center/Height");
+
+					Assert.IsTrue (center.X < far.X, "center-far/X");
+					Assert.AreEqual (center.Y, far.Y, "center-far/Y");
+					Assert.AreEqual (center.Width, far.Width, 0.1, "center-far/Width");
+					Assert.AreEqual (center.Height, far.Height, 0.1, "center-far/Height");
+				}
+			}
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void MeasureCharacterRanges_StringFormat_LineAlignment ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			string text = "Hello Mono::";
+			CharacterRange[] ranges = new CharacterRange[1];
+			ranges[0] = new CharacterRange (5, 4);
+			StringFormat string_format = new StringFormat ();
+			string_format.SetMeasurableCharacterRanges (ranges);
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					string_format.LineAlignment = StringAlignment.Near;
+					Region[] regions = g.MeasureCharacterRanges (text, font, new RectangleF (0, 0, 320, 32), string_format);
+					Assert.AreEqual (1, regions.Length, "Near.Region");
+					RectangleF near = regions[0].GetBounds (g);
+
+					string_format.LineAlignment = StringAlignment.Center;
+					regions = g.MeasureCharacterRanges (text, font, new RectangleF (0, 0, 320, 32), string_format);
+					Assert.AreEqual (1, regions.Length, "Center.Region");
+					RectangleF center = regions[0].GetBounds (g);
+
+					string_format.LineAlignment = StringAlignment.Far;
+					regions = g.MeasureCharacterRanges (text, font, new RectangleF (0, 0, 320, 32), string_format);
+					Assert.AreEqual (1, regions.Length, "Far.Region");
+					RectangleF far = regions[0].GetBounds (g);
+
+					Assert.AreEqual (near.X, center.X, 0.1, "near-center/X");
+					Assert.IsTrue (near.Y < center.Y, "near-center/Y");
+					Assert.AreEqual (near.Width, center.Width, 0.1, "near-center/Width");
+					Assert.AreEqual (near.Height, center.Height, 0.1, "near-center/Height");
+
+					Assert.AreEqual (center.X, far.X, 0.1, "center-far/X");
+					Assert.IsTrue (center.Y < far.Y, "center-far/Y");
+					Assert.AreEqual (center.Width, far.Width, 0.1, "center-far/Width");
+					Assert.AreEqual (center.Height, far.Height, 0.1, "center-far/Height");
+				}
+			}
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void MeasureCharacterRanges_StringFormat_Alignment_DirectionVertical ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			string text = "Hello Mono::";
+			CharacterRange[] ranges = new CharacterRange[1];
+			ranges[0] = new CharacterRange (5, 4);
+			StringFormat string_format = new StringFormat ();
+			string_format.FormatFlags = StringFormatFlags.DirectionVertical;
+			string_format.SetMeasurableCharacterRanges (ranges);
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					string_format.Alignment = StringAlignment.Near;
+					Region[] regions = g.MeasureCharacterRanges (text, font, new RectangleF (0, 0, 320, 32), string_format);
+					Assert.AreEqual (1, regions.Length, "Near.Region");
+					RectangleF near = regions[0].GetBounds (g);
+
+					string_format.Alignment = StringAlignment.Center;
+					regions = g.MeasureCharacterRanges (text, font, new RectangleF (0, 0, 320, 32), string_format);
+					Assert.AreEqual (1, regions.Length, "Center.Region");
+					RectangleF center = regions[0].GetBounds (g);
+
+					string_format.Alignment = StringAlignment.Far;
+					regions = g.MeasureCharacterRanges (text, font, new RectangleF (0, 0, 320, 32), string_format);
+					Assert.AreEqual (1, regions.Length, "Far.Region");
+					RectangleF far = regions[0].GetBounds (g);
+
+					Assert.IsTrue (near.X < center.X, "near-center/X"); // ???
+					Assert.IsTrue (near.Y < center.Y, "near-center/Y");
+					Assert.IsTrue (near.Width > center.Width, "near-center/Width"); // ???
+					Assert.AreEqual (near.Height, center.Height, 0.1, "near-center/Height");
+
+					Assert.AreEqual (center.X, far.X, 0.1, "center-far/X");
+					Assert.IsTrue (center.Y < far.Y, "center-far/Y");
+					Assert.AreEqual (center.Width, far.Width, 0.1, "center-far/Width");
+					Assert.AreEqual (center.Height, far.Height, 0.1, "center-far/Height");
+				}
+			}
+		}
+
+		[Test]
+		[Category ("NotWorking")]
+		public void MeasureCharacterRanges_StringFormat_LineAlignment_DirectionVertical ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			string text = "Hello Mono::";
+			CharacterRange[] ranges = new CharacterRange[1];
+			ranges[0] = new CharacterRange (5, 4);
+			StringFormat string_format = new StringFormat ();
+			string_format.FormatFlags = StringFormatFlags.DirectionVertical;
+			string_format.SetMeasurableCharacterRanges (ranges);
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					string_format.LineAlignment = StringAlignment.Near;
+					Region[] regions = g.MeasureCharacterRanges (text, font, new RectangleF (0, 0, 320, 32), string_format);
+					Assert.AreEqual (1, regions.Length, "Near.Region");
+					RectangleF near = regions[0].GetBounds (g);
+
+					string_format.LineAlignment = StringAlignment.Center;
+					regions = g.MeasureCharacterRanges (text, font, new RectangleF (0, 0, 320, 32), string_format);
+					Assert.AreEqual (1, regions.Length, "Center.Region");
+					RectangleF center = regions[0].GetBounds (g);
+
+					string_format.LineAlignment = StringAlignment.Far;
+					regions = g.MeasureCharacterRanges (text, font, new RectangleF (0, 0, 320, 32), string_format);
+					Assert.AreEqual (1, regions.Length, "Far.Region");
+					RectangleF far = regions[0].GetBounds (g);
+
+					Assert.IsTrue (near.X < center.X, "near-center/X");
+					Assert.AreEqual (near.Y, center.Y, 0.1, "near-center/Y");
+					Assert.AreEqual (near.Width, center.Width, 0.1, "near-center/Width");
+					Assert.AreEqual (near.Height, center.Height, 0.1, "near-center/Height");
+
+					Assert.IsTrue (center.X < far.X, "center-far/X");
+					Assert.AreEqual (center.Y, far.Y, 0.1, "center-far/Y");
+					Assert.AreEqual (center.Width, far.Width, 0.1, "center-far/Width");
+					Assert.AreEqual (center.Height, far.Height, 0.1, "center-far/Height");
+				}
+			}
+		}
+
+		[Test]
+		public void DrawString_EndlessLoop_Bug77699 ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					Rectangle rect = Rectangle.Empty;
+					rect.Location = new Point (10, 10);
+					rect.Size = new Size (1, 20);
+					StringFormat fmt = new StringFormat ();
+					fmt.Alignment = StringAlignment.Center;
+					fmt.LineAlignment = StringAlignment.Center;
+					fmt.FormatFlags = StringFormatFlags.NoWrap;
+					fmt.Trimming = StringTrimming.EllipsisWord;
+					g.DrawString ("Test String", font, Brushes.Black, rect, fmt);
+				}
+			}
+		}
+
+		[Test]
+		public void DrawString_EndlessLoop_Wrapping ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					Rectangle rect = Rectangle.Empty;
+					rect.Location = new Point (10, 10);
+					rect.Size = new Size (1, 20);
+					StringFormat fmt = new StringFormat ();
+					fmt.Alignment = StringAlignment.Center;
+					fmt.LineAlignment = StringAlignment.Center;
+					fmt.Trimming = StringTrimming.EllipsisWord;
+					g.DrawString ("Test String", font, Brushes.Black, rect, fmt);
+				}
+			}
+		}
+
+		[Test]
+		public void MeasureString_Wrapping_Dots ()
+		{
+			if (font == null)
+				Assert.Ignore ("Couldn't create required font");
+
+			string text = "this is really long text........................................... with a lot o periods.";
+			using (Bitmap bitmap = new Bitmap (20, 20)) {
+				using (Graphics g = Graphics.FromImage (bitmap)) {
+					using (StringFormat format = new StringFormat ()) {
+						format.Alignment = StringAlignment.Center;
+						SizeF sz = g.MeasureString (text, font, 80, format);
+						Assert.IsTrue (sz.Width < 80, "Width");
+						Assert.IsTrue (sz.Height > font.Height * 2, "Height");
+					}
+				}
+			}
+		}
 #if NET_2_0
 		[Test]
 		public void TestReleaseHdc ()
