@@ -85,7 +85,10 @@ namespace System.Security.Cryptography.Xml {
 		private XmlResolver xmlResolver = new XmlUrlResolver ();
 #endif
 		private ArrayList manifests;
-		
+#if NET_2_0
+		private IEnumerator _x509Enumerator;
+#endif
+
 		private static readonly char [] whitespaceChars = new char [] {' ', '\r', '\n', '\t'};
 
 		public SignedXml () 
@@ -620,8 +623,11 @@ namespace System.Security.Cryptography.Xml {
 		public void ComputeSignature () 
 		{
 			if (key != null) {
-				// required before hashing
-				m_signature.SignedInfo.SignatureMethod = key.SignatureAlgorithm;
+				if (m_signature.SignedInfo.SignatureMethod == null)
+					// required before hashing
+					m_signature.SignedInfo.SignatureMethod = key.SignatureAlgorithm;
+				else if (m_signature.SignedInfo.SignatureMethod != key.SignatureAlgorithm)
+					throw new CryptographicException ("Specified SignatureAlgorithm is not supported by the signing key.");
 				DigestReferences ();
 
 				AsymmetricSignatureFormatter signer = null;
@@ -680,8 +686,18 @@ namespace System.Security.Cryptography.Xml {
 			if (pkEnumerator == null) {
 				pkEnumerator = m_signature.KeyInfo.GetEnumerator ();
 			}
-
-			if (pkEnumerator.MoveNext ()) {
+			
+#if NET_2_0
+			if (_x509Enumerator != null) {
+				if (_x509Enumerator.MoveNext ()) {
+					X509Certificate cert = (X509Certificate) _x509Enumerator.Current;
+					return new X509Certificate2 (cert.GetRawCertData ()).PublicKey.Key;
+				} else {
+					_x509Enumerator = null;
+				}
+			}
+#endif
+			while (pkEnumerator.MoveNext ()) {
 				AsymmetricAlgorithm key = null;
 				KeyInfoClause kic = (KeyInfoClause) pkEnumerator.Current;
 
@@ -694,6 +710,16 @@ namespace System.Security.Cryptography.Xml {
 					key.FromXmlString (kic.GetXml ().InnerXml);
 					return key;
 				}
+
+#if NET_2_0
+				if (kic is KeyInfoX509Data) {
+					_x509Enumerator = ((KeyInfoX509Data) kic).Certificates.GetEnumerator ();
+					if (_x509Enumerator.MoveNext ()) {
+						X509Certificate cert = (X509Certificate) _x509Enumerator.Current;
+						return new X509Certificate2 (cert.GetRawCertData ()).PublicKey.Key;
+					}
+				}
+#endif
 			}
 			return null;
 		}
