@@ -30,6 +30,9 @@
 using System.Collections;
 using System.IO;
 using System.Security.Permissions;
+#if NET_2_0
+using System.Text;
+#endif
 
 namespace System.Web.Caching
 {
@@ -51,8 +54,17 @@ namespace System.Web.Caching
 		FileSystemWatcher[] watchers;
 #endif
 		bool hasChanged;
+#if NET_2_0
+		bool used;
+#endif
 		object locker = new object ();
 		
+#if NET_2_0
+		public CacheDependency (): this (null, null, null, DateTime.Now)
+		{
+		}
+#endif
+		
 		public CacheDependency (string filename): this (new string[] { filename }, null, null, DateTime.Now)
 		{
 		}
@@ -114,7 +126,28 @@ namespace System.Web.Caching
 				dependency.DependencyChanged += new EventHandler (OnChildDependencyChanged);
 			this.start = start;
 		}
+
+#if NET_2_0
+		public virtual string GetUniqueID ()
+		{
+			StringBuilder sb = new StringBuilder ();
+#if !TARGET_JVM
+			lock (locker) {
+				if (watchers != null)
+					foreach (FileSystemWatcher fsw in watchers)
+						if (fsw != null && fsw.Path != null && fsw.Path.Length != 0)
+							sb.AppendFormat ("_{0}", fsw.Path);
+			}
+#endif
+
+			if (cachekeys != null)
+				foreach (string key in cachekeys)
+					sb.AppendFormat ("_{0}", key);
+			return sb.ToString ();
+		}
+#endif
 		
+#if !TARGET_JVM
 		void OnChanged (object sender, FileSystemEventArgs args)
 		{
 			if (DateTime.Now < start)
@@ -125,12 +158,7 @@ namespace System.Web.Caching
 			if (cache != null)
 				cache.CheckExpiration ();
 		}
-		
-#if TARGET_JVM
-		void DisposeWatchers ()
-		{
-		}
-#else
+	
 		void DisposeWatchers ()
 		{
 			lock (locker) {
@@ -142,6 +170,10 @@ namespace System.Web.Caching
 				watchers = null;
 			}
 		}
+#else
+		void DisposeWatchers ()
+		{
+		}
 #endif
 		public void Dispose ()
 		{
@@ -154,8 +186,22 @@ namespace System.Web.Caching
 		internal void SetCache (Cache c)
 		{
 			cache = c;
+#if NET_2_0
+			used = c != null;
+#endif
 		}
 		
+#if NET_2_0
+		internal bool IsUsed {
+			get { return used; }
+		}
+
+		internal DateTime Start {
+			get { return start; }
+			set { start = value; }
+		}
+#endif
+		
 		public bool HasChanged {
 			get {
 				if (hasChanged)
@@ -191,6 +237,12 @@ namespace System.Web.Caching
 				DependencyChanged (this, null);
 		}
 		
+#if NET_2_0
+		internal void SignalDependencyChanged ()
+		{
+			OnDependencyChanged ();
+		}
+#endif
 		internal event EventHandler DependencyChanged;
 	}
 }
