@@ -31,6 +31,7 @@
 using System;
 using System.Collections;
 using System.CodeDom.Compiler;
+using System.Globalization;
 using System.IO;
 using System.Text;
 using System.Web.Caching;
@@ -121,6 +122,10 @@ namespace System.Web.Compilation
 
 			return result;
 		}
+
+		public int Count {
+			get { return parsers.Count; }
+		}
 		
 		public AspParser Parser {
 			get { return current; }
@@ -158,7 +163,7 @@ namespace System.Web.Compilation
 			if (tags.Count == 0)
 				return false;
 
-			return 0 == String.Compare (tagid, (string) tags.Peek (), true);
+			return 0 == String.Compare (tagid, (string) tags.Peek (), true, CultureInfo.InvariantCulture);
 		}
 		
 		public int Count {
@@ -221,7 +226,7 @@ namespace System.Web.Compilation
 				return new UserControlCompiler ((UserControlParser) tparser);
 #if NET_2_0
 			if (type == typeof(MasterPageParser))
-				return new UserControlCompiler ((UserControlParser) tparser);
+				return new MasterPageCompiler ((MasterPageParser) tparser);
 #endif
 
 			throw new Exception ("Got type: " + type);
@@ -254,7 +259,7 @@ namespace System.Web.Compilation
 			PrintTree (rootBuilder, 0);
 #endif
 
-			if (stack.Count > 1)
+			if (stack.Count > 1 && pstack.Count == 0)
 				throw new ParseException (stack.Builder.location,
 						"Expecting </" + stack.Builder.TagName + "> " + stack.Builder);
 		}
@@ -272,7 +277,9 @@ namespace System.Web.Compilation
 			}
 
 			isApplication = tparser.DefaultDirectiveName == "application";
-
+#if NET_2_0
+			tparser.RegisterConfigControls ();
+#endif
 			Parse ();
 
 			BaseCompiler compiler = GetCompilerFromType ();
@@ -328,15 +335,15 @@ namespace System.Web.Compilation
 
 			// MS ignores tbody/thead
 			if ((attributes == null || !attributes.IsRunAtServer ())) {
-				if (String.Compare (tagid, "tbody", true) == 0 ||
-				    String.Compare (tagid, "thead", true) == 0)
+				if (String.Compare (tagid, "tbody", true, CultureInfo.InvariantCulture) == 0 ||
+				    String.Compare (tagid, "thead", true, CultureInfo.InvariantCulture) == 0)
 				return;
 			}
 
 			if (text.Length != 0)
 				FlushText ();
 
-			if (0 == String.Compare (tagid, "script", true)) {
+			if (0 == String.Compare (tagid, "script", true, CultureInfo.InvariantCulture)) {
 				bool in_script = (inScript || ignore_text);
 				if (in_script || (tagtype != TagType.Close && attributes != null)) {
 					if ((in_script || attributes.IsRunAtServer ()) && ProcessScript (tagtype, attributes))
@@ -417,13 +424,16 @@ namespace System.Web.Compilation
 			if (otags == null || otags.Count == 0)
 				return false;
 
-			int idx = otags.Count - 1;
-			string otagid = (string) otags [idx];
-			if (0 != String.Compare (tagid, otagid, true))
-				return false;
-
-			otags.RemoveAt (idx);
-			return true;
+			for (int idx = otags.Count - 1; idx >= 0; idx--) {
+				string otagid = (string) otags [idx];
+				if (0 == String.Compare (tagid, otagid, true, CultureInfo.InvariantCulture)) {
+					do {
+						otags.RemoveAt (idx);
+					} while (otags.Count - 1 >= idx);
+					return true;
+				}
+			}
+			return false;
 		}
 
 		static string GetIncludeFilePath (string basedir, string filename)
@@ -474,7 +484,7 @@ namespace System.Web.Compilation
 		bool ProcessTag (string tagid, TagAttributes atts, TagType tagtype)
 		{
 			if (isApplication) {
-				if (String.Compare (tagid, "object", true) != 0)
+				if (String.Compare (tagid, "object", true, CultureInfo.InvariantCulture) != 0)
 					throw new ParseException (location, "Invalid tag for application file.");
 			}
 
@@ -606,7 +616,8 @@ namespace System.Web.Compilation
 		{
 			ControlBuilder current = stack.Builder;
 			string btag = current.TagName;
-			if (String.Compare (btag, "tbody", true) != 0 && String.Compare (tagid, "tbody", true) == 0) {
+			if (String.Compare (btag, "tbody", true, CultureInfo.InvariantCulture) != 0 &&
+			    String.Compare (tagid, "tbody", true, CultureInfo.InvariantCulture) == 0) {
 				if (!current.ChildrenAsProperties) {
 					try {
 						TextParsed (location, location.PlainText);
@@ -616,7 +627,7 @@ namespace System.Web.Compilation
 				return true;
 			}
 			
-			if (0 != String.Compare (tagid, btag, true))
+			if (0 != String.Compare (tagid, btag, true, CultureInfo.InvariantCulture))
 				return false;
 
 			// if (current is TemplateBuilder)
@@ -666,7 +677,7 @@ namespace System.Web.Compilation
 			if (lang == null || lang == "")
 				return;
 
-			if (String.Compare (lang, tparser.Language, true) == 0)
+			if (String.Compare (lang, tparser.Language, true, CultureInfo.InvariantCulture) == 0)
 				return;
 
 #if NET_2_0
