@@ -34,6 +34,7 @@ using System.IO;
 using System.Runtime.InteropServices;
 using System.Security;
 using System.Security.Permissions;
+using System.Web.Configuration;
 using System.Web.UI;
 using System.Web.Util;
 
@@ -126,6 +127,11 @@ namespace System.Web.Hosting {
 				return null;
 			}
 		}
+#if NET_2_0
+		public override string RootWebConfigPath {
+			get { return WebConfigurationManager.OpenWebConfiguration ("~").FilePath; }
+		}
+#endif
 
 		public override void EndOfRequest ()
 		{
