@@ -44,17 +44,16 @@ namespace System.Web.Security {
 		private bool listChanged;
 		string[] cachedArray;
 		private HybridDictionary cachedRoles;
-		private RoleProvider provider;
+		readonly string providerName;
 
 		public RolePrincipal (IIdentity identity)
 		{
 			if (identity == null)
 				throw new ArgumentNullException ("identity");
 			this.identity = identity;
-			this.provider = Roles.Provider;
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public RolePrincipal (IIdentity identity, string encryptedTicket)
 			: this (identity)
 		{
@@ -67,17 +66,17 @@ namespace System.Web.Security {
 			if (providerName == null)
 				throw new ArgumentNullException ("providerName");
 
-			this.provider = Roles.Providers[providerName];
+			this.providerName = providerName;
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public RolePrincipal (string providerName, IIdentity identity, string encryptedTicket)
 			: this (identity)
 		{
 			if (providerName == null)
 				throw new ArgumentNullException ("providerName");
 
-			this.provider = Roles.Providers[providerName];
+			this.providerName = providerName;
 
 			throw new NotImplementedException ();
 		}
@@ -88,7 +87,7 @@ namespace System.Web.Security {
 				return new string[0];
 
 			if (cachedRoles == null) {
-				cachedArray = provider.GetRolesForUser (identity.Name);
+				cachedArray = Provider.GetRolesForUser (identity.Name);
 				cachedRoles = new HybridDictionary (true);
 
 				foreach (string r in cachedArray)
@@ -108,7 +107,7 @@ namespace System.Web.Security {
 			return cachedRoles[role] != null;
 		}
 		
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public string ToEncryptedTicket ()
 		{
 			throw new NotImplementedException ();
@@ -118,17 +117,17 @@ namespace System.Web.Security {
 			get { return listChanged; }
 		}
 		
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public string CookiePath {
 			get { throw new NotImplementedException (); }
 		}
 		
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public bool Expired {
 			get { throw new NotImplementedException (); }
 		}
 		
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public DateTime ExpireDate {
 			get { throw new NotImplementedException (); }
 		}
@@ -141,19 +140,28 @@ namespace System.Web.Security {
 			get { return cachedRoles != null; }
 		}
 		
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public DateTime IssueDate {
 			get { throw new NotImplementedException (); }
 		}
 		
 		public string ProviderName {
-			get { return provider.Name; }
+			get { return String.IsNullOrEmpty(providerName) ? Provider.Name : providerName; }
 		}
 		
 		public int Version {
 			get { return 1; }
 		}
 
+		RoleProvider Provider {
+			get {
+				if (String.IsNullOrEmpty(providerName))
+					return Roles.Provider;
+
+				return Roles.Providers [providerName];
+			}
+		}
+
 		public void SetDirty ()
 		{
 			listChanged = true;
