@@ -530,20 +530,17 @@ namespace System.Web.UI.HtmlControls
 		}
 
 #if NET_2_0
-		[MonoTODO]
 		protected virtual void OnDataPropertyChanged ()
 		{
 			RequiresDataBinding = true;
 		}
 
-		[MonoTODO]
 		protected virtual void OnDataSourceViewChanged (object sender,
 								EventArgs e)
 		{
 			RequiresDataBinding = true;
 		}
 
-		[MonoTODO]
 		protected internal override void OnInit (EventArgs e)
 		{
 			base.OnInit (e);
@@ -611,6 +608,10 @@ namespace System.Web.UI.HtmlControls
 		
 		protected override void RenderAttributes (HtmlTextWriter w)
 		{
+#if NET_2_0
+			if (Page != null)
+				Page.ClientScript.RegisterForEventValidation (this.UniqueID);
+#endif
 			/* If there is no "name" attribute,
 			 * LoadPostData doesn't work...
 			 */
@@ -645,7 +646,6 @@ namespace System.Web.UI.HtmlControls
 			int count = items.Count;
 			for (int i = 0; i < count; i++) {
 				ListItem item = items[i];
-
 				w.Indent++;
 				
 				/* Write the <option> elements this
