@@ -1085,6 +1085,9 @@ namespace System.Web.UI.WebControls {
 			hasLink = (Page != null && hasLink == true) ? true : false;
 
 			if (hasLink) {
+#if NET_2_0
+				Page.ClientScript.RegisterForEventValidation(this.UniqueID, arg);
+#endif			
 				str = "<a href=\"";
 				str += Page.ClientScript.GetPostBackClientHyperlink (this, arg);
 				str += "\"";
