@@ -317,7 +317,14 @@ namespace System.Web.UI.WebControls {
 		DataControlButton CreateButton (string text, string image, string command, string arg)
 		{
 			DataControlButton c = new DataControlButton (Control, text, image, command, arg, false);
-			c.CausesValidation = CausesValidation;
+			c.ButtonType = ButtonType;
+			if (CausesValidation) {
+				if (command == DataControlCommands.UpdateCommandName || command == DataControlCommands.InsertCommandName) {
+					c.Container = null;
+					c.CausesValidation = true;
+					c.ValidationGroup = ValidationGroup;
+				}
+			}
 			return c;
 		}
 		
@@ -363,7 +370,7 @@ namespace System.Web.UI.WebControls {
 		public override void ValidateSupportsCallback ()
 		{
 			if (ShowSelectButton)
-				throw new NotSupportedException ("ShowSelectButton = true");
+				throw new NotSupportedException ("Callbacks are not supported on CommandField when the select button is enabled because other controls on your page that are dependent on the selected value of '" + Control.ID + "' for their rendering will not update in a callback.  Turn callbacks off on '" + Control.ID + "'.");
 		}
 	}
 }
