@@ -39,7 +39,7 @@ namespace System.Web.UI.WebControls
 	[BindableAttribute (false)]
 	public class CreateUserWizard : Wizard
 	{
-		public static readonly string ContinueButtonCommandName;
+		public static readonly string ContinueButtonCommandName = "Continue";
 		private string _password = "";
 		private string _confirmPassword = "";
 		private MembershipProvider _provider = null;
@@ -1215,13 +1215,29 @@ namespace System.Web.UI.WebControls
 
 			if (CompleteStep == null)
 				WizardSteps.AddAt (WizardSteps.Count, new CompleteWizardStep ());
+
+			if (ActiveStepIndex < 0)
+				ActiveStepIndex = 0;
 		}
 
 		protected override bool OnBubbleEvent (object source, EventArgs e)
 		{
+			CommandEventArgs args = e as CommandEventArgs;
+			if (e != null && args.CommandName == ContinueButtonCommandName) {
+				ProcessContinueEvent ();
+				return true;
+			}
 			return base.OnBubbleEvent (source, e);
 		}
 
+		private void ProcessContinueEvent () {
+			OnContinueButtonClick (EventArgs.Empty);
+
+			if (ContinueDestinationPageUrl.Length > 0) {
+				Context.Response.Redirect (ContinueDestinationPageUrl);
+			}
+		}
+
 		protected virtual void OnContinueButtonClick (EventArgs e)
 		{
 			if (Events != null) {
@@ -1420,7 +1436,7 @@ namespace System.Web.UI.WebControls
 					break;
 
 				case MembershipCreateStatus.InvalidPassword:
-					ShowErrorMessage (InvalidPasswordErrorMessage);
+					ShowErrorMessage (String.Format (InvalidPasswordErrorMessage, MembershipProviderInternal.MinRequiredPasswordLength, MembershipProviderInternal.MinRequiredNonAlphanumericCharacters));
 					break;
 
 				case MembershipCreateStatus.DuplicateEmail:
@@ -1467,10 +1483,9 @@ namespace System.Web.UI.WebControls
 				_errorMessageLabel.Text = errorMessage;
 		}
 
-		[MonoTODO]
 		private string GeneratePassword ()
 		{
-			return "password";
+			return Membership.GeneratePassword (8, 3);
 		}
 
 		#endregion
@@ -1512,4 +1527,4 @@ namespace System.Web.UI.WebControls
 	}
 }
 
-#endif
\ No newline at end of file
+#endif
