@@ -84,15 +84,11 @@ namespace System.Web.UI.WebControls {
 				return (IDataSource) ctrl;
 			}
 			
-			if (DataSource == null) return null;
-			
 			IDataSource ds = DataSource as IDataSource;
 			if (ds != null) return ds;
 			
-			IEnumerable ie = DataSourceHelper.GetResolvedDataSource (DataSource, DataMember);
-			if (ie != null) return new CollectionDataSource (ie);
-			
-			throw new HttpException (string.Format ("Unexpected data source type: {0}", DataSource.GetType()));
+			IEnumerable ie = DataSourceResolver.ResolveDataSource (DataSource, DataMember);
+			return new CollectionDataSource (ie);
 		}
 		
 		protected virtual DataSourceView GetData ()
@@ -117,7 +113,7 @@ namespace System.Web.UI.WebControls {
 		protected override void OnDataPropertyChanged ()
 		{
 			base.OnDataPropertyChanged ();
-			UpdateViewData ();
+			currentView = null;
 		}
 		
 		protected virtual void OnDataSourceViewChanged (object sender, EventArgs e)
@@ -150,6 +146,10 @@ namespace System.Web.UI.WebControls {
 			if (!Page.IsPostBack || (IsViewStateEnabled && !IsDataBound))
 				RequiresDataBinding = true;
 
+			// MSDN: The ConfirmInitState method sets the initialized state of the data-bound 
+			// control. The method is called by the DataBoundControl class in its OnLoad method.
+			ConfirmInitState ();
+
 			base.OnLoad(e);
 		}
 		
@@ -159,7 +159,7 @@ namespace System.Web.UI.WebControls {
 
 		protected override void ValidateDataSource (object dataSource)
 		{
-			if (dataSource is IListSource || dataSource is IEnumerable || dataSource is IDataSource)
+			if (dataSource == null || dataSource is IListSource || dataSource is IEnumerable || dataSource is IDataSource)
 				return;
 			throw new ArgumentException ("Invalid data source source type. The data source must be of type IListSource, IEnumerable or IDataSource.");
 		}
@@ -192,17 +192,24 @@ namespace System.Web.UI.WebControls {
 			if (!IsBoundUsingDataSourceID)
 				OnDataBinding (EventArgs.Empty);
 
-			DataSourceView view = GetData ();
-			if (view != null) {
-				view.Select (SelectArguments, new DataSourceViewSelectCallback (OnSelect));
-				MarkAsDataBound ();
-			}
+			SelectArguments = CreateDataSourceSelectArguments ();
+			GetData ().Select (SelectArguments, new DataSourceViewSelectCallback (OnSelect));
+			MarkAsDataBound ();
+			
+			// Raise the DataBound event.
+			OnDataBound (EventArgs.Empty);
 		}
 		
 		void OnSelect (IEnumerable data)
 		{
+			// Call OnDataBinding only if it has not already been 
+			// called in the PerformSelect method.
+			if (IsBoundUsingDataSourceID) {
+				OnDataBinding (EventArgs.Empty);
+			}
+			// The PerformDataBinding method binds the data in the  
+			// retrievedData collection to elements of the data-bound control.
 			PerformDataBinding (data);
-			OnDataBound (EventArgs.Empty);
 		}
 		
 		protected virtual DataSourceSelectArguments CreateDataSourceSelectArguments ()
@@ -216,6 +223,9 @@ namespace System.Web.UI.WebControls {
 					selectArguments = CreateDataSourceSelectArguments ();
 				return selectArguments;
 			}
+			private set {
+				selectArguments = value;
+			}
 		}
 
 		bool IsDataBound {
@@ -236,3 +246,7 @@ namespace System.Web.UI.WebControls {
 }
 #endif
 
+
+
+
+
