@@ -33,6 +33,7 @@ using System.Web;
 using System.Web.UI;
 using System.ComponentModel;
 using System.ComponentModel.Design;
+using System.Drawing;
 
 namespace System.Web.UI.WebControls
 {
@@ -43,11 +44,12 @@ namespace System.Web.UI.WebControls
 		public DataControlButton (Control container)
 		{
 			this.container = container;
+			CausesValidation = false;
 		}
 		
 		public DataControlButton (Control container, string text, string image, string command, string commandArg, bool allowCallback)
+			: this (container)
 		{
-			this.container = container;
 			Text = text;
 			ImageUrl = image;
 			CommandName = command;
@@ -55,6 +57,11 @@ namespace System.Web.UI.WebControls
 			AllowCallback = allowCallback;
 		}
 		
+		public Control Container {
+			get { return container; }
+			set { container = value; }
+		}
+
 		public string ImageUrl {
 			get {
 				object o = ViewState["iu"];
@@ -90,23 +97,25 @@ namespace System.Web.UI.WebControls
 
 		protected internal override void Render (HtmlTextWriter writer)
 		{
+			if (CommandName.Length > 0 && ButtonType == ButtonType.Link) {
+				EnsureForeColor ();
+			}
+
 			if (CommandName.Length > 0 || ButtonType == ButtonType.Button)
 			{
 				string postScript = null;
 				string callScript = null;
-				
+				PostBackOptions ops = null;
+
 				IPostBackContainer pcner = container as IPostBackContainer;
 				if (pcner != null) {
-					PostBackOptions ops = pcner.GetPostBackOptions (this);
-					postScript = container.Page.ClientScript.GetPostBackEventReference (ops);
-				} else
-					postScript = Page.ClientScript.GetPostBackEventReference (this, "");
-
-				if (CausesValidation && Page.Validators.Count > 0) {
-					// TOSHOK: review if this is the correct usage of the "fresh" client side stuff
-					ClientScriptManager csm = new ClientScriptManager (Page);
-					postScript = csm.GetClientValidationEvent () + postScript;
+					ops = pcner.GetPostBackOptions (this);
+					ops.RequiresJavaScriptProtocol = ButtonType == ButtonType.Link;
 				}
+				else
+					ops = GetPostBackOptions ();
+
+				postScript = Page.ClientScript.GetPostBackEventReference (ops, true);
 				
 				if (AllowCallback) {
 					ICallbackContainer ccner = container as ICallbackContainer;
@@ -132,10 +141,8 @@ namespace System.Web.UI.WebControls
 				if (ButtonType == ButtonType.Link) {
 					if (callScript != null) {
 						writer.AddAttribute (HtmlTextWriterAttribute.Onclick, callScript);
-						writer.AddAttribute (HtmlTextWriterAttribute.Href, "javascript:");
 					}
-					else
-						writer.AddAttribute (HtmlTextWriterAttribute.Href, "javascript:" + postScript);
+					writer.AddAttribute (HtmlTextWriterAttribute.Href, postScript);
 					writer.RenderBeginTag (HtmlTextWriterTag.A);
 					writer.Write (Text);
 					writer.RenderEndTag ();
@@ -146,7 +153,7 @@ namespace System.Web.UI.WebControls
 					else
 						writer.AddAttribute (HtmlTextWriterAttribute.Onclick, postScript);
 
-					writer.AddAttribute (HtmlTextWriterAttribute.Type, "submit");
+					writer.AddAttribute (HtmlTextWriterAttribute.Type, "button");
 					writer.AddAttribute (HtmlTextWriterAttribute.Name, ClientID);
 					writer.AddAttribute (HtmlTextWriterAttribute.Value, Text);
 					writer.RenderBeginTag (HtmlTextWriterTag.Input);
@@ -173,6 +180,33 @@ namespace System.Web.UI.WebControls
 				}
 			}
 		}
+		
+		protected override PostBackOptions GetPostBackOptions () {
+			PostBackOptions options = new PostBackOptions (this);
+			options.Argument = "";
+			options.RequiresJavaScriptProtocol = ButtonType == ButtonType.Link;
+			options.ClientSubmit = true;
+			options.PerformValidation = CausesValidation && Page != null && Page.AreValidatorsUplevel (ValidationGroup);
+			if (options.PerformValidation)
+				options.ValidationGroup = ValidationGroup;
+
+			return options;
+		}
+
+		private void EnsureForeColor () {
+			if (ForeColor != Color.Empty)
+				return;
+
+			for (Control parent = Parent; parent != null; parent = parent.Parent) {
+				WebControl wc = parent as WebControl;
+				if (wc != null && wc.ForeColor != Color.Empty) {
+					ForeColor = wc.ForeColor;
+					break;
+				}
+				if (parent == container)
+					break;
+			}
+		}
 	}
 }
 
