@@ -70,9 +70,6 @@ namespace System.Web.UI.WebControls
 		PropertyDescriptor[] cachedKeyProperties;
 		readonly string[] emptyKeys = new string[0];
 		
-		CommandField commandField;
-		DetailsViewRow commandRow;
-			
 		// View state
 		DataControlFieldCollection columns;
 		PagerSettings pagerSettings;
@@ -88,6 +85,7 @@ namespace System.Web.UI.WebControls
 		TableItemStyle commandRowStyle;
 		TableItemStyle fieldHeaderStyle;
 		
+		IOrderedDictionary _keyTable;
 		DataKey key;
 		DataKey oldEditValues;
 		AutoGeneratedFieldProperties[] autoFieldProperties;
@@ -108,12 +106,12 @@ namespace System.Web.UI.WebControls
 		// Control state
 		int pageIndex;
 		DetailsViewMode currentMode = DetailsViewMode.ReadOnly; 
-		int pageCount = 0;
+		bool hasCurrentMode;
+		int pageCount;
 		
 		public DetailsView ()
 		{
 			rows = new DetailsViewRowCollection (new ArrayList ());
-			key = new DataKey (new OrderedDictionary ());
 		}
 		
 		public event EventHandler PageIndexChanged {
@@ -365,13 +363,12 @@ namespace System.Web.UI.WebControls
 		[EditorAttribute ("System.Web.UI.Design.ImageUrlEditor, " + Consts.AssemblySystem_Design, "System.Drawing.Design.UITypeEditor, " + Consts.AssemblySystem_Drawing)]
 		public virtual string BackImageUrl {
 			get {
-				object ob = ViewState ["BackImageUrl"];
-				if (ob != null) return (string) ob;
-				return string.Empty;
+				if (ControlStyleCreated)
+					return ((TableStyle) ControlStyle).BackImageUrl;
+				return String.Empty;
 			}
 			set {
-				ViewState ["BackImageUrl"] = value;
-				RequireBinding ();
+				((TableStyle) ControlStyle).BackImageUrl = value;
 			}
 		}
 
@@ -419,15 +416,12 @@ namespace System.Web.UI.WebControls
 		public virtual int CellPadding
 		{
 			get {
-				object o = ViewState ["CellPadding"];
-				if (o != null) return (int) o;
+				if (ControlStyleCreated)
+					return ((TableStyle) ControlStyle).CellPadding;
 				return -1;
 			}
 			set {
-				if (value < -1)
-					throw new ArgumentOutOfRangeException ("< -1");
-				ViewState["CellPadding"] = value;
-				RequireBinding ();
+				((TableStyle) ControlStyle).CellPadding = value;
 			}
 		}
 
@@ -436,15 +430,12 @@ namespace System.Web.UI.WebControls
 		public virtual int CellSpacing
 		{
 			get {
-				object o = ViewState ["CellSpacing"];
-				if (o != null) return (int) o;
+				if (ControlStyleCreated)
+					return ((TableStyle) ControlStyle).CellSpacing;
 				return 0;
 			}
 			set {
-				if (value < -1)
-					throw new ArgumentOutOfRangeException ("< -1");
-				ViewState ["CellSpacing"] = value;
-				RequireBinding ();
+				((TableStyle) ControlStyle).CellSpacing = value;
 			}
 		}
 		
@@ -468,7 +459,11 @@ namespace System.Web.UI.WebControls
 		[BrowsableAttribute (false)]
 		public DetailsViewMode CurrentMode {
 			get {
-				return currentMode;
+				return hasCurrentMode ? currentMode : DefaultMode;
+			}
+			private set {
+				hasCurrentMode = true;
+				currentMode = value;
 			}
 		}
 
@@ -522,15 +517,35 @@ namespace System.Web.UI.WebControls
 			}
 		}
 		
+		IOrderedDictionary KeyTable {
+			get {
+				if (_keyTable == null) {
+					_keyTable = new OrderedDictionary (DataKeyNames.Length);
+				}
+				return _keyTable;
+			}
+		}
+
 		[BrowsableAttribute (false)]
 		[DesignerSerializationVisibilityAttribute (DesignerSerializationVisibility.Hidden)]
 		public virtual DataKey DataKey {
 			get {
-				EnsureDataBound ();
+				if (key == null) {
+					key = new DataKey (KeyTable);
+				}
 				return key;
 			}
 		}
 
+		DataKey OldEditValues {
+			get {
+				if (oldEditValues == null) {
+					oldEditValues = new DataKey (new OrderedDictionary ());
+				}
+				return oldEditValues;
+			}
+		}
+
 		[WebCategoryAttribute ("Styles")]
 		[PersistenceMode (PersistenceMode.InnerProperty)]
 		[NotifyParentProperty (true)]
@@ -670,10 +685,12 @@ namespace System.Web.UI.WebControls
 		[DefaultValueAttribute (GridLines.Both)]
 		public virtual GridLines GridLines {
 			get {
-				return ((TableStyle)ControlStyle).GridLines;
+				if (ControlStyleCreated)
+					return ((TableStyle) ControlStyle).GridLines;
+				return GridLines.Both;
 			}
 			set {
-				((TableStyle)ControlStyle).GridLines = value;
+				((TableStyle) ControlStyle).GridLines = value;
 			}
 		}
 
@@ -730,13 +747,12 @@ namespace System.Web.UI.WebControls
 		[DefaultValueAttribute (HorizontalAlign.NotSet)]
 		public virtual HorizontalAlign HorizontalAlign {
 			get {
-				object ob = ViewState ["HorizontalAlign"];
-				if (ob != null) return (HorizontalAlign) ob;
+				if (ControlStyleCreated)
+					return ((TableStyle) ControlStyle).HorizontalAlign;
 				return HorizontalAlign.NotSet;
 			}
 			set {
-				ViewState ["HorizontalAlign"] = value;
-				RequireBinding ();
+				((TableStyle) ControlStyle).HorizontalAlign = value;
 			}
 		}
 
@@ -760,8 +776,6 @@ namespace System.Web.UI.WebControls
 		[DesignerSerializationVisibilityAttribute (DesignerSerializationVisibility.Hidden)]
 		public virtual int PageCount {
 			get {
-				if (pageCount != 0) return pageCount;
-				EnsureDataBound ();
 				return pageCount;
 			}
 		}
@@ -774,6 +788,10 @@ namespace System.Web.UI.WebControls
 				return pageIndex;
 			}
 			set {
+				if (value < 0)
+					throw new ArgumentOutOfRangeException ("PageIndex must be non-negative");
+				if (value == pageIndex)
+					return;
 				pageIndex = value;
 				RequireBinding ();
 			}
@@ -811,8 +829,7 @@ namespace System.Web.UI.WebControls
 		
 		
 		[DefaultValue (null)]
-		/* DataControlPagerCell isnt specified in the docs */
-		//[TemplateContainer (typeof(DataControlPagerCell), BindingDirection.OneWay)]
+		[TemplateContainer (typeof (DetailsView), BindingDirection.OneWay)]
 		[PersistenceMode (PersistenceMode.InnerProperty)]
 		[Browsable (false)]
 		public virtual ITemplate PagerTemplate {
@@ -892,7 +909,19 @@ namespace System.Web.UI.WebControls
 		
 		protected override DataSourceSelectArguments CreateDataSourceSelectArguments ()
 		{
-			return base.CreateDataSourceSelectArguments ();
+			DataSourceSelectArguments arg = new DataSourceSelectArguments ();
+			DataSourceView view = GetData ();
+			if (AllowPaging && view.CanPage) {
+				arg.StartRowIndex = PageIndex;
+				if (view.CanRetrieveTotalRowCount) {
+					arg.RetrieveTotalRowCount = true;
+					arg.MaximumRows = 1;
+				}
+				else {
+					arg.MaximumRows = -1;
+				}
+			}
+			return arg;
 		}
 		
 		protected virtual ICollection CreateFieldSet (object dataItem, bool useDataSource)
@@ -919,7 +948,6 @@ namespace System.Web.UI.WebControls
 				field.ShowDeleteButton = AutoGenerateDeleteButton;
 				field.ShowInsertButton = AutoGenerateInsertButton;
 				fields.Add (field);
-				commandField = field;
 			}
 			
 			return fields;
@@ -980,22 +1008,12 @@ namespace System.Web.UI.WebControls
 		{
 			if (Initialized) {
 				RequiresDataBinding = true;
-				pageCount = 0;
 			}
 		}
 		
 		protected virtual Table CreateTable ()
 		{
-			Table table = new Table ();
-			table.Caption = Caption;
-			table.CaptionAlign = CaptionAlign;
-			table.CellPadding = CellPadding;
-			//CellSpacing defaults to 0 and Table.CellSpacing to -1
-			if (CellSpacing != 0)
-				table.CellSpacing = CellSpacing;
-			table.HorizontalAlign = HorizontalAlign;
-			table.BackImageUrl = BackImageUrl;
-			return table;
+			return new ContainedTable (this);
 		}
 
 		protected override Style CreateControlStyle ()
@@ -1021,7 +1039,7 @@ namespace System.Web.UI.WebControls
 					dataSource.CurrentPageIndex = PageIndex;
 					if (view.CanPage) {
 						dataSource.AllowServerPaging = true;
-						if (view.CanRetrieveTotalRowCount)
+						if (SelectArguments.RetrieveTotalRowCount)
 							dataSource.VirtualCount = SelectArguments.TotalRowCount;
 					}
 				}
@@ -1046,10 +1064,6 @@ namespace System.Web.UI.WebControls
 			Controls.Add (table);
 				
 			ArrayList list = new ArrayList ();
-
-			if (Page == null || !Page.IsPostBack)
-				currentMode = DefaultMode;
-
 			
 			// Gets the current data item
 			
@@ -1092,30 +1106,16 @@ namespace System.Web.UI.WebControls
 			if (dataSource.Count > 0) {
 				foreach (DataControlField field in fields) {
 					DataControlRowState rstate = GetRowState (list.Count);
-					DetailsViewRow row = CreateRow (list.Count, DataControlRowType.DataRow, rstate);
+					DetailsViewRow row = CreateRow (PageIndex, DataControlRowType.DataRow, rstate);
 					InitializeRow (row, field);
 					table.Rows.Add (row);
 					list.Add (row);
-
-					if (commandField == field)
-						commandRow = row;
-				}
-				if (!dataBinding) {
-					if (CurrentMode == DetailsViewMode.Edit)
-						oldEditValues = new DataKey (new OrderedDictionary ());
-					key = new DataKey (new OrderedDictionary (), DataKeyNames);
 				}
 			} else {
 				table.Rows.Add (CreateEmptyrRow ());
 			}
 
 			rows = new DetailsViewRowCollection (list);
-			
-			if (showPager && PagerSettings.Position == PagerPosition.Bottom || PagerSettings.Position == PagerPosition.TopAndBottom) {
-				bottomPagerRow = CreateRow (-1, DataControlRowType.Pager, DataControlRowState.Normal);
-				InitializePager (bottomPagerRow, dataSource);
-				table.Rows.Add (bottomPagerRow);
-			}
 
 			footerRow = CreateRow (-1, DataControlRowType.Footer, DataControlRowState.Normal);
 			DataControlFieldCell footerCell = new DataControlFieldCell (null);
@@ -1127,13 +1127,18 @@ namespace System.Web.UI.WebControls
 			footerRow.Cells.Add (footerCell);
 			table.Rows.Add (footerRow);
 			
+			if (showPager && PagerSettings.Position == PagerPosition.Bottom || PagerSettings.Position == PagerPosition.TopAndBottom) {
+				bottomPagerRow = CreateRow (-1, DataControlRowType.Pager, DataControlRowState.Normal);
+				InitializePager (bottomPagerRow, dataSource);
+				table.Rows.Add (bottomPagerRow);
+			}
+			
 			if (dataBinding)
 				DataBind (false);
 			
 			return dataSource.DataSourceCount;
 		}
 
-		[MonoTODO]
 		protected override void EnsureDataBound ()
 		{
 			base.EnsureDataBound ();
@@ -1177,6 +1182,7 @@ namespace System.Web.UI.WebControls
 		
 		protected virtual void InitializeRow (DetailsViewRow row, DataControlField field)
 		{
+			row.ContainingField = field;
 			DataControlFieldCell cell;
 			
 			if (field.ShowHeader) {
@@ -1190,10 +1196,15 @@ namespace System.Web.UI.WebControls
 				cell.ColumnSpan = 2;
 			row.Cells.Add (cell);
 			field.InitializeCell (cell, DataControlCellType.DataCell, row.RowState, row.RowIndex);
+
+			if (CurrentMode == DetailsViewMode.Insert && !field.InsertVisible)
+				row.Visible = false;
 		}
 		
-		IOrderedDictionary CreateRowDataKey (object dataItem)
+		void FillRowDataKey (object dataItem)
 		{
+			KeyTable.Clear ();
+
 			if (cachedKeyProperties == null) {
 				PropertyDescriptorCollection props = TypeDescriptor.GetProperties (dataItem);
 				cachedKeyProperties = new PropertyDescriptor [DataKeyNames.Length];
@@ -1205,10 +1216,8 @@ namespace System.Web.UI.WebControls
 				}
 			}
 			
-			OrderedDictionary dic = new OrderedDictionary ();
 			foreach (PropertyDescriptor p in cachedKeyProperties)
-				dic [p.Name] = p.GetValue (dataItem);
-			return dic;
+				KeyTable [p.Name] = p.GetValue (dataItem);
 		}
 		
 		IOrderedDictionary GetRowValues (bool includeReadOnlyFields, bool includePrimaryKey)
@@ -1242,21 +1251,20 @@ namespace System.Web.UI.WebControls
 		
 		public sealed override void DataBind ()
 		{
-			DataSourceView view = GetData ();
-			if (AllowPaging && view.CanPage) {
-				SelectArguments.StartRowIndex = PageIndex;
-				SelectArguments.MaximumRows = 1;
-				if (view.CanRetrieveTotalRowCount)
-					SelectArguments.RetrieveTotalRowCount = true;
+			if (CurrentMode == DetailsViewMode.Insert) {
+				RequiresDataBinding = false;
+				PerformDataBinding (new object [] { null });
+				return;
 			}
-
+			
 			cachedKeyProperties = null;
 			base.DataBind ();
 			
 			if (dataItem != null) {
 				if (CurrentMode == DetailsViewMode.Edit)
 					oldEditValues = new DataKey (GetRowValues (false, true));
-				key = new DataKey (CreateRowDataKey (dataItem), DataKeyNames);
+				FillRowDataKey (dataItem);
+				key = new DataKey (KeyTable);
 			}
 		}
 		
@@ -1265,10 +1273,72 @@ namespace System.Web.UI.WebControls
 			base.PerformDataBinding (data);
 		}
 
-		[MonoTODO]
 		protected internal virtual void PrepareControlHierarchy ()
 		{
-			throw new NotImplementedException ();
+			if (table == null)
+				return;
+
+			table.Caption = Caption;
+			table.CaptionAlign = CaptionAlign;
+
+			foreach (DetailsViewRow row in table.Rows) {
+				switch (row.RowType) {
+				case DataControlRowType.Header:
+					if (headerStyle != null && !headerStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (headerStyle);
+					break;
+				case DataControlRowType.Footer:
+					if (footerStyle != null && !footerStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (footerStyle);
+					break;
+				case DataControlRowType.Pager:
+					if (pagerStyle != null && !pagerStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (pagerStyle);
+					break;
+				case DataControlRowType.EmptyDataRow:
+					if (emptyDataRowStyle != null && !emptyDataRowStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (emptyDataRowStyle);
+					break;
+				case DataControlRowType.DataRow:
+					if (rowStyle != null && !rowStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (rowStyle);
+					if ((row.RowState & DataControlRowState.Alternate) != 0 && alternatingRowStyle != null && !alternatingRowStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (alternatingRowStyle);
+					if ((row.RowState & DataControlRowState.Edit) != 0 && editRowStyle != null && !editRowStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (editRowStyle);
+					if ((row.RowState & DataControlRowState.Insert) != 0 && insertRowStyle != null && !insertRowStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (insertRowStyle);
+					break;
+				default:
+					break;
+				}
+
+				if ((row.ContainingField is CommandField) && commandRowStyle != null && !commandRowStyle.IsEmpty)
+					row.ControlStyle.CopyFrom (commandRowStyle);
+
+				for (int n = 0; n < row.Cells.Count; n++) {
+					DataControlFieldCell fcell = row.Cells [n] as DataControlFieldCell;
+					if (fcell != null && fcell.ContainingField != null) {
+						DataControlField field = fcell.ContainingField;
+						if (n == 0 && field.ShowHeader) {
+							if (fieldHeaderStyle != null && !fieldHeaderStyle.IsEmpty)
+								fcell.ControlStyle.CopyFrom (fieldHeaderStyle);
+							if (field.HeaderStyleCreated && !field.HeaderStyle.IsEmpty)
+								fcell.ControlStyle.CopyFrom (field.HeaderStyle);
+						}
+						else {
+							if (field.ControlStyleCreated && !field.ControlStyle.IsEmpty)
+								foreach (Control c in fcell.Controls) {
+									WebControl wc = c as WebControl;
+									if (wc != null)
+										wc.ControlStyle.MergeWith (field.ControlStyle);
+								}
+							if (field.ItemStyleCreated && !field.ItemStyle.IsEmpty)
+								fcell.ControlStyle.CopyFrom (field.ItemStyle);
+						}
+					}
+				}
+			}
 		}
 		
 		protected internal override void OnInit (EventArgs e)
@@ -1294,6 +1364,7 @@ namespace System.Web.UI.WebControls
 			if (args != null) {
 				OnItemCommand (args);
 				ProcessEvent (args.CommandName, args.CommandArgument as string);
+				return true;
 			}
 			return base.OnBubbleEvent (source, e);
 		}
@@ -1327,13 +1398,15 @@ namespace System.Web.UI.WebControls
 					newIndex = PageCount - 1;
 					break;
 				case DataControlCommands.NextPageCommandArgument:
-					if (PageIndex < PageCount - 1) newIndex = PageIndex + 1;
+					newIndex = PageIndex + 1;
 					break;
 				case DataControlCommands.PreviousPageCommandArgument:
-					if (PageIndex > 0) newIndex = PageIndex - 1;
+					newIndex = PageIndex - 1;
 					break;
 				default:
-					newIndex = int.Parse (param) - 1;
+					int paramIndex = 0;
+					int.TryParse (param, out paramIndex);
+					newIndex = paramIndex - 1;
 					break;
 				}
 				ShowPage (newIndex);
@@ -1388,7 +1461,9 @@ namespace System.Web.UI.WebControls
 			DetailsViewPageEventArgs args = new DetailsViewPageEventArgs (newIndex);
 			OnPageIndexChanging (args);
 			if (!args.Cancel) {
-				EndRowEdit ();
+				if (args.NewPageIndex < 0 || args.NewPageIndex >= PageCount)
+					return;
+				EndRowEdit (false);
 				PageIndex = args.NewPageIndex;
 				OnPageIndexChanged (EventArgs.Empty);
 			}
@@ -1399,7 +1474,7 @@ namespace System.Web.UI.WebControls
 			DetailsViewModeEventArgs args = new DetailsViewModeEventArgs (newMode, false);
 			OnModeChanging (args);
 			if (!args.Cancel) {
-				currentMode = args.NewMode;
+				CurrentMode = args.NewMode;
 				OnModeChanged (EventArgs.Empty);
 				RequireBinding ();
 			}
@@ -1421,12 +1496,12 @@ namespace System.Web.UI.WebControls
 		
 		void UpdateItem (string param, bool causesValidation)
 		{
-			if (causesValidation)
+			if (causesValidation && Page != null)
 				Page.Validate ();
 			
-			if (currentMode != DetailsViewMode.Edit) throw new NotSupportedException ();
-			
-			currentEditOldValues = oldEditValues.Values;
+			if (CurrentMode != DetailsViewMode.Edit) throw new HttpException ();
+
+			currentEditOldValues = OldEditValues.Values;
 
 			currentEditRowKeys = DataKey.Values;
 			currentEditNewValues = GetRowValues (false, false);
@@ -1435,9 +1510,12 @@ namespace System.Web.UI.WebControls
 			OnItemUpdating (args);
 			if (!args.Cancel) {
 				DataSourceView view = GetData ();
-				if (view == null) throw new HttpException ("The DataSourceView associated to data bound control was null");
-				view.Update (currentEditRowKeys, currentEditNewValues, currentEditOldValues, new DataSourceViewOperationCallback (UpdateCallback));
-			} else
+				if (view == null)
+					throw new HttpException ("The DataSourceView associated to data bound control was null");
+				if (view.CanUpdate)
+					view.Update (currentEditRowKeys, currentEditNewValues, currentEditOldValues, new DataSourceViewOperationCallback (UpdateCallback));
+			}
+			else
 				EndRowEdit ();
 		}
 
@@ -1459,10 +1537,10 @@ namespace System.Web.UI.WebControls
 		
 		void InsertItem (string param, bool causesValidation)
 		{
-			if (causesValidation)
+			if (causesValidation && Page != null)
 				Page.Validate ();
 			
-			if (currentMode != DetailsViewMode.Insert) throw new NotSupportedException ();
+			if (CurrentMode != DetailsViewMode.Insert) throw new HttpException ();
 			
 			currentEditNewValues = GetRowValues (false, true);
 			DetailsViewInsertEventArgs args = new DetailsViewInsertEventArgs (param, currentEditNewValues);
@@ -1470,9 +1548,11 @@ namespace System.Web.UI.WebControls
 			if (!args.Cancel) {
 				DataSourceView view = GetData ();
 				if (view == null)
-					return;
-				view.Insert (currentEditNewValues, new DataSourceViewOperationCallback (InsertCallback));
-			} else
+					throw new HttpException ("The DataSourceView associated to data bound control was null");
+				if (view.CanInsert)
+					view.Insert (currentEditNewValues, new DataSourceViewOperationCallback (InsertCallback));
+			}
+			else
 				EndRowEdit ();
 		}
 		
@@ -1489,7 +1569,7 @@ namespace System.Web.UI.WebControls
 
 		public virtual void DeleteItem ()
 		{
-			currentEditRowKeys = DataKey == null ? null : DataKey.Values;
+			currentEditRowKeys = DataKey.Values;
 			currentEditNewValues = GetRowValues (true, true);
 			
 			DetailsViewDeleteEventArgs args = new DetailsViewDeleteEventArgs (PageIndex, currentEditRowKeys, currentEditNewValues);
@@ -1500,7 +1580,7 @@ namespace System.Web.UI.WebControls
 					PageIndex --;
 				RequireBinding ();
 				DataSourceView view = GetData ();
-				if (view != null)
+				if (view != null && view.CanDelete)
 					view.Delete (currentEditRowKeys, currentEditNewValues, new DataSourceViewOperationCallback (DeleteCallback));
 				else {
 					DetailsViewDeletedEventArgs dargs = new DetailsViewDeletedEventArgs (0, null, currentEditRowKeys, currentEditNewValues);
@@ -1518,7 +1598,12 @@ namespace System.Web.UI.WebControls
 		
 		void EndRowEdit ()
 		{
-			ChangeMode (DefaultMode);
+			EndRowEdit (true);
+		}
+
+		void EndRowEdit (bool switchToDefaultMode) {
+			if (switchToDefaultMode)
+				ChangeMode (DefaultMode);
 			oldEditValues = new DataKey (new OrderedDictionary ());
 			currentEditRowKeys = null;
 			currentEditOldValues = null;
@@ -1533,16 +1618,27 @@ namespace System.Web.UI.WebControls
 			base.LoadControlState (state[0]);
 			pageIndex = (int) state[1];
 			pageCount = (int) state[2];
-			currentMode = (DetailsViewMode) state[3];
+			CurrentMode = (DetailsViewMode) state[3];
 			dataKeyNames = (string[]) state[4];
 			defaultMode = (DetailsViewMode) state[5];
+			if (state [6] != null)
+				((IStateManager) DataKey).LoadViewState (state [6]);
+			if (state [7] != null)
+				((IStateManager) OldEditValues).LoadViewState (state [7]);
 		}
 		
 		protected internal override object SaveControlState ()
 		{
 			object bstate = base.SaveControlState ();
 			return new object[] {
-				bstate, pageIndex, pageCount, currentMode, dataKeyNames, defaultMode
+				bstate, 
+				pageIndex, 
+				pageCount, 
+				CurrentMode, 
+				dataKeyNames, 
+				defaultMode,
+				(key == null ? null : ((IStateManager)key).SaveViewState()),
+				(oldEditValues == null ? null : ((IStateManager) oldEditValues).SaveViewState ())
 					};
 		}
 		
@@ -1568,7 +1664,7 @@ namespace System.Web.UI.WebControls
 
 		protected override object SaveViewState()
 		{
-			object[] states = new object [14];
+			object[] states = new object [12];
 			states[0] = base.SaveViewState();
 			states[1] = (columns == null ? null : ((IStateManager)columns).SaveViewState());
 			states[2] = (pagerSettings == null ? null : ((IStateManager)pagerSettings).SaveViewState());
@@ -1580,8 +1676,6 @@ namespace System.Web.UI.WebControls
 			states[8] = (insertRowStyle == null ? null : ((IStateManager)insertRowStyle).SaveViewState());
 			states[9] = (editRowStyle == null ? null : ((IStateManager)editRowStyle).SaveViewState());
 			states[10] = (emptyDataRowStyle == null ? null : ((IStateManager)emptyDataRowStyle).SaveViewState());
-			states[11] = (key == null ? null : ((IStateManager)key).SaveViewState());
-			states[12] = (oldEditValues == null ? null : ((IStateManager)oldEditValues).SaveViewState());
 			
 			if (autoFieldProperties != null) {
 				object[] data = new object [autoFieldProperties.Length];
@@ -1590,7 +1684,7 @@ namespace System.Web.UI.WebControls
 					data [n] = ((IStateManager)autoFieldProperties [n]).SaveViewState ();
 					if (data [n] != null) allNull = false;
 				}
-				if (!allNull) states [13] = data;
+				if (!allNull) states [11] = data;
 			}
 
 			for (int i = states.Length - 1; i >= 0; i--) {
@@ -1610,8 +1704,8 @@ namespace System.Web.UI.WebControls
 
 			object [] states = (object []) savedState;
 			
-			if (states[13] != null) {
-				object[] data = (object[]) states [13];
+			if (states[11] != null) {
+				object[] data = (object[]) states [11];
 				autoFieldProperties = new AutoGeneratedFieldProperties [data.Length];
 				for (int n=0; n<data.Length; n++) {
 					IStateManager p = new AutoGeneratedFieldProperties ();
@@ -1622,7 +1716,6 @@ namespace System.Web.UI.WebControls
 			}
 
 			base.LoadViewState (states[0]);
-			EnsureChildControls ();
 			
 			if (states[1] != null) ((IStateManager)Fields).LoadViewState (states[1]);
 			if (states[2] != null) ((IStateManager)PagerSettings).LoadViewState (states[2]);
@@ -1634,8 +1727,6 @@ namespace System.Web.UI.WebControls
 			if (states[8] != null) ((IStateManager)InsertRowStyle).LoadViewState (states[8]);
 			if (states[9] != null) ((IStateManager)EditRowStyle).LoadViewState (states[9]);
 			if (states[10] != null) ((IStateManager)EmptyDataRowStyle).LoadViewState (states[10]);
-			if (states[11] != null) ((IStateManager)DataKey).LoadViewState (states[11]);
-			if (states[12] != null && oldEditValues != null) ((IStateManager)oldEditValues).LoadViewState (states[12]);
 		}
 		
 		void ICallbackEventHandler.RaiseCallbackEvent (string eventArgs)
@@ -1643,22 +1734,13 @@ namespace System.Web.UI.WebControls
 			RaiseCallbackEvent (eventArgs);
 		}
 		
-		string callbackResult;
 		protected virtual void RaiseCallbackEvent (string eventArgs)
 		{
 			string[] clientData = eventArgs.Split ('|');
-			pageIndex = int.Parse (clientData[0]);
-			RequireBinding ();
+			PageIndex = int.Parse (clientData[0]);
 			
-			RaisePostBackEvent (clientData[2]);
-			EnsureDataBound ();
-			
-			StringWriter sw = new StringWriter ();
-			sw.Write (PageIndex.ToString());
-
-			HtmlTextWriter writer = new HtmlTextWriter (sw);
-			RenderGrid (writer);
-			callbackResult = sw.ToString ();
+			RaisePostBackEvent (clientData[1]);
+			DataBind ();
 		}
 
 		string ICallbackEventHandler.GetCallbackResult ()
@@ -1668,27 +1750,38 @@ namespace System.Web.UI.WebControls
 
 		protected virtual string GetCallbackResult ()
 		{
-			return callbackResult;
+			PrepareControlHierarchy ();
+
+			StringWriter sw = new StringWriter ();
+			sw.Write (PageIndex.ToString () + '|');
+
+			HtmlTextWriter writer = new HtmlTextWriter (sw);
+			RenderGrid (writer);
+			return sw.ToString ();
 		}
 
-		[MonoTODO]
 		protected virtual string GetCallbackScript (IButtonControl buttonControl, string argument)
 		{
-			throw new NotImplementedException ();
+			if (EnablePagingCallbacks)
+				return "javascript:DetailsView_ClientEvent (\"" + ClientID + "\",\"" + buttonControl.CommandName + "$" + buttonControl.CommandArgument + "\"); return false;";
+			else
+				return null;
 		}
 		
 		string ICallbackContainer.GetCallbackScript (IButtonControl control, string argument)
 		{
-			if (EnablePagingCallbacks)
-				return "javascript:GridView_ClientEvent (\"" + ClientID + "\",\"" + control.CommandName + "$" + control.CommandArgument + "\"); return false;";
-			else
-				return null;
+			return GetCallbackScript (control, argument);
 		}
 
-		[MonoTODO]
 		protected override void OnPagePreLoad (object sender, EventArgs e)
 		{
 			base.OnPagePreLoad (sender, e);
+
+			if (Page.IsPostBack && EnablePagingCallbacks) {
+				int page;
+				if (int.TryParse (Page.Request.Form [ClientID + "_Page"], out page))
+					PageIndex = page;
+			}
 		}
 		
 		protected internal override void OnPreRender (EventArgs e)
@@ -1697,10 +1790,11 @@ namespace System.Web.UI.WebControls
 			
 			if (EnablePagingCallbacks)
 			{
-				if (!Page.ClientScript.IsClientScriptIncludeRegistered (typeof(GridView), "GridView.js")) {
-					string url = Page.ClientScript.GetWebResourceUrl (typeof(GridView), "GridView.js");
-					Page.ClientScript.RegisterClientScriptInclude (typeof(GridView), "GridView.js", url);
+				if (!Page.ClientScript.IsClientScriptIncludeRegistered (typeof(DetailsView), "DetailsView.js")) {
+					string url = Page.ClientScript.GetWebResourceUrl (typeof(DetailsView), "DetailsView.js");
+					Page.ClientScript.RegisterClientScriptInclude (typeof(DetailsView), "DetailsView.js", url);
 				}
+				Page.ClientScript.RegisterHiddenField (ClientID + "_Page", PageIndex.ToString ());
 				
 				string cgrid = ClientID + "_data";
 				string script = string.Format ("var {0} = new Object ();\n", cgrid);
@@ -1716,6 +1810,8 @@ namespace System.Web.UI.WebControls
 		
 		protected internal override void Render (HtmlTextWriter writer)
 		{
+			PrepareControlHierarchy ();
+			
 			if (EnablePagingCallbacks)
 				base.RenderBeginTag (writer);
 			else
@@ -1731,66 +1827,25 @@ namespace System.Web.UI.WebControls
 		
 		void RenderGrid (HtmlTextWriter writer)
 		{
-			table.GridLines = GridLines;
-			table.ControlStyle.MergeWith (ControlStyle);
-			table.RenderBeginTag (writer);
-			
-			foreach (DetailsViewRow row in table.Rows)
-			{
-				switch (row.RowType) {
-				case DataControlRowType.Header:
-					if (headerStyle != null)headerStyle.AddAttributesToRender (writer, row);
-					break;
-				case DataControlRowType.Footer:
-					if (footerStyle != null) footerStyle.AddAttributesToRender (writer, row);
-					break;
-				case DataControlRowType.Pager:
-					if (pagerStyle != null) pagerStyle.AddAttributesToRender (writer, row);
-					break;
-				case DataControlRowType.EmptyDataRow:
-					if (emptyDataRowStyle != null) emptyDataRowStyle.AddAttributesToRender (writer, row);
-					break;
-				default:
-					if (rowStyle != null) rowStyle.AddAttributesToRender (writer, row);
-					break;
-				}
+			if (table == null)
+				return;
 
-				if ((row.RowState & DataControlRowState.Alternate) != 0 && alternatingRowStyle != null)
-					alternatingRowStyle.AddAttributesToRender (writer, row);
-				if ((row.RowState & DataControlRowState.Edit) != 0 && editRowStyle != null)
-					editRowStyle.AddAttributesToRender (writer, row);
-				if ((row.RowState & DataControlRowState.Insert) != 0 && insertRowStyle != null)
-					insertRowStyle.AddAttributesToRender (writer, row);
-					
-				if (row == commandRow && commandRowStyle != null)
-					commandRowStyle.AddAttributesToRender (writer, row);
-				
-				row.RenderBeginTag (writer);
-				
-				for (int n=0; n<row.Cells.Count; n++) {
-					DataControlFieldCell fcell = row.Cells[n] as DataControlFieldCell;
-					if (fcell != null && fcell.ContainingField != null) {
-						if (n == 0 && fcell.ContainingField.ShowHeader) {
-							if (fieldHeaderStyle != null)
-								fieldHeaderStyle.AddAttributesToRender (writer, fcell);
-							fcell.ContainingField.HeaderStyle.AddAttributesToRender (writer, fcell);
-						}
-						else
-							fcell.ContainingField.ItemStyle.AddAttributesToRender (writer, fcell);
-					}
-					row.Cells[n].Render (writer);
-				}
-				row.RenderEndTag (writer);
-			}
-			table.RenderEndTag (writer);
+			table.Render (writer);
 		}
 
-
-		[MonoTODO]
 		PostBackOptions IPostBackContainer.GetPostBackOptions (IButtonControl control)
 		{
-			PostBackOptions pbo = new PostBackOptions (this, control.CommandName + "$" + control.CommandArgument);
-			return pbo;
+			if (control == null)
+				throw new ArgumentNullException ("control");
+
+			if (control.CausesValidation)
+				throw new InvalidOperationException ("A button that causes validation in DetailsView '" + ID + "' is attempting to use the container GridView as the post back target.  The button should either turn off validation or use itself as the post back container.");
+
+			PostBackOptions options = new PostBackOptions (this);
+			options.Argument = control.CommandName + "$" + control.CommandArgument;
+			options.RequiresJavaScriptProtocol = true;
+
+			return options;
 		}
 	}
 }
