@@ -83,6 +83,7 @@ namespace System.Web.UI.WebControls
 		TableItemStyle pagerStyle;
 		TableItemStyle rowStyle;
 		
+		IOrderedDictionary _keyTable;
 		DataKey key;
 		DataKey oldEditValues;
 		
@@ -102,12 +103,8 @@ namespace System.Web.UI.WebControls
 		// Control state
 		int pageIndex;
 		FormViewMode currentMode = FormViewMode.ReadOnly; 
-		int pageCount = 0;
-		
-		public FormView ()
-		{
-			key = new DataKey (new OrderedDictionary ());
-		}
+		bool hasCurrentMode;
+		int pageCount;
 		
 		public event EventHandler PageIndexChanged {
 			add { Events.AddHandler (PageIndexChangedEvent, value); }
@@ -286,13 +283,12 @@ namespace System.Web.UI.WebControls
 		[EditorAttribute ("System.Web.UI.Design.ImageUrlEditor, " + Consts.AssemblySystem_Design, "System.Drawing.Design.UITypeEditor, " + Consts.AssemblySystem_Drawing)]
 		public virtual string BackImageUrl {
 			get {
-				object ob = ViewState ["BackImageUrl"];
-				if (ob != null) return (string) ob;
-				return string.Empty;
+				if (ControlStyleCreated)
+					return ((TableStyle) ControlStyle).BackImageUrl;
+				return String.Empty;
 			}
 			set {
-				ViewState ["BackImageUrl"] = value;
-				RequireBinding ();
+				((TableStyle) ControlStyle).BackImageUrl = value;
 			}
 		}
 
@@ -300,7 +296,7 @@ namespace System.Web.UI.WebControls
 		[BrowsableAttribute (false)]
 		public virtual FormViewRow BottomPagerRow {
 			get {
-				EnsureDataBound ();
+				EnsureChildControls ();
 				return bottomPagerRow;
 			}
 		}
@@ -340,15 +336,12 @@ namespace System.Web.UI.WebControls
 		public virtual int CellPadding
 		{
 			get {
-				object o = ViewState ["CellPadding"];
-				if (o != null) return (int) o;
+				if (ControlStyleCreated)
+					return ((TableStyle) ControlStyle).CellPadding;
 				return -1;
 			}
 			set {
-				if (value < -1)
-					throw new ArgumentOutOfRangeException ("< -1");
-				ViewState ["CellPadding"] = value;
-				RequireBinding ();
+				((TableStyle) ControlStyle).CellPadding = value;
 			}
 		}
 
@@ -357,15 +350,12 @@ namespace System.Web.UI.WebControls
 		public virtual int CellSpacing
 		{
 			get {
-				object o = ViewState ["CellSpacing"];
-				if (o != null) return (int) o;
+				if (ControlStyleCreated)
+					return ((TableStyle) ControlStyle).CellSpacing;
 				return 0;
 			}
 			set {
-				if (value < -1)
-					throw new ArgumentOutOfRangeException ("< -1");
-				ViewState["CellSpacing"] = value;
-				RequireBinding ();
+				((TableStyle) ControlStyle).CellSpacing = value;
 			}
 		}
 		
@@ -373,7 +363,11 @@ namespace System.Web.UI.WebControls
 		[BrowsableAttribute (false)]
 		public FormViewMode CurrentMode {
 			get {
-				return currentMode;
+				return hasCurrentMode ? currentMode : DefaultMode;
+			}
+			private set {
+				hasCurrentMode = true;
+				currentMode = value;
 			}
 		}
 
@@ -409,15 +403,35 @@ namespace System.Web.UI.WebControls
 			}
 		}
 		
+		IOrderedDictionary KeyTable {
+			get {
+				if (_keyTable == null) {
+					_keyTable = new OrderedDictionary (DataKeyNames.Length);
+				}
+				return _keyTable;
+			}
+		}
+
 		[BrowsableAttribute (false)]
 		[DesignerSerializationVisibilityAttribute (DesignerSerializationVisibility.Hidden)]
 		public virtual DataKey DataKey {
 			get {
-				EnsureDataBound ();
+				if (key == null) {
+					key= new DataKey (KeyTable);
+				}
 				return key;
 			}
 		}
 
+		DataKey OldEditValues {
+			get {
+				if (oldEditValues == null) {
+					oldEditValues = new DataKey (new OrderedDictionary ());
+				}
+				return oldEditValues;
+			}
+		}
+
 		[DefaultValue (null)]
 		[TemplateContainer (typeof(FormView), BindingDirection.TwoWay)]
 		[PersistenceMode (PersistenceMode.InnerProperty)]
@@ -536,7 +550,9 @@ namespace System.Web.UI.WebControls
 		[DefaultValueAttribute (GridLines.None)]
 		public virtual GridLines GridLines {
 			get {
-				return ((TableStyle) ControlStyle).GridLines;
+				if (ControlStyleCreated)
+					return ((TableStyle) ControlStyle).GridLines;
+				return GridLines.None;
 			}
 			set {
 				((TableStyle) ControlStyle).GridLines = value;
@@ -596,13 +612,12 @@ namespace System.Web.UI.WebControls
 		[DefaultValueAttribute (HorizontalAlign.NotSet)]
 		public virtual HorizontalAlign HorizontalAlign {
 			get {
-				object ob = ViewState ["HorizontalAlign"];
-				if (ob != null) return (HorizontalAlign) ob;
+				if (ControlStyleCreated)
+					return ((TableStyle) ControlStyle).HorizontalAlign;
 				return HorizontalAlign.NotSet;
 			}
 			set {
-				ViewState ["HorizontalAlign"] = value;
-				RequireBinding ();
+				((TableStyle) ControlStyle).HorizontalAlign = value;
 			}
 		}
 
@@ -644,8 +659,6 @@ namespace System.Web.UI.WebControls
 		[DesignerSerializationVisibilityAttribute (DesignerSerializationVisibility.Hidden)]
 		public virtual int PageCount {
 			get {
-				if (pageCount != 0) return pageCount;
-				EnsureDataBound ();
 				return pageCount;
 			}
 		}
@@ -658,6 +671,8 @@ namespace System.Web.UI.WebControls
 				return pageIndex;
 			}
 			set {
+				if (value < 0)
+					throw new ArgumentOutOfRangeException ("PageIndex must be non-negative");
 				if (pageIndex == value)
 					return;
 				pageIndex = value;
@@ -710,7 +725,7 @@ namespace System.Web.UI.WebControls
 		[BrowsableAttribute (false)]
 		public virtual FormViewRow Row {
 			get {
-				EnsureDataBound ();
+				EnsureChildControls ();
 				return itemRow;
 			}
 		}
@@ -741,7 +756,7 @@ namespace System.Web.UI.WebControls
 		[BrowsableAttribute (false)]
 		public virtual FormViewRow TopPagerRow {
 			get {
-				EnsureDataBound ();
+				EnsureChildControls ();
 				return topPagerRow;
 			}
 		}
@@ -782,7 +797,19 @@ namespace System.Web.UI.WebControls
 		
 		protected override DataSourceSelectArguments CreateDataSourceSelectArguments ()
 		{
-			return base.CreateDataSourceSelectArguments ();
+			DataSourceSelectArguments arg = new DataSourceSelectArguments ();
+			DataSourceView view = GetData ();
+			if (AllowPaging && view.CanPage) {
+				arg.StartRowIndex = PageIndex;
+				if (view.CanRetrieveTotalRowCount) {
+					arg.RetrieveTotalRowCount = true;
+					arg.MaximumRows = 1;
+				}
+				else {
+					arg.MaximumRows = -1;
+				}
+			}
+			return arg;
 		}
 		
 		protected virtual FormViewRow CreateRow (int rowIndex, DataControlRowType rowType, DataControlRowState rowState)
@@ -795,24 +822,15 @@ namespace System.Web.UI.WebControls
 		void RequireBinding ()
 		{
 			if (Initialized) {
-				pageCount = -1;
 				RequiresDataBinding = true;
 			}
 		}
 		
 		protected virtual Table CreateTable ()
 		{
-			Table table = new Table ();
-			table.Caption = Caption;
-			table.CaptionAlign = CaptionAlign;
-			table.CellPadding = CellPadding;
-			table.CellSpacing = CellSpacing;
-			table.HorizontalAlign = HorizontalAlign;
-			table.BackImageUrl = BackImageUrl;
-			return table;
+			return new ContainedTable (this);
 		}
 
-		[MonoTODO]
 		protected override void EnsureDataBound ()
 		{
 			base.EnsureDataBound ();
@@ -833,24 +851,21 @@ namespace System.Web.UI.WebControls
 			dataSource.PageSize = 1;
 			dataSource.CurrentPageIndex = PageIndex;
 
-			if (dataBinding) {
+			if (dataBinding && CurrentMode != FormViewMode.Insert) {
 				DataSourceView view = GetData ();
 				if (view != null && view.CanPage) {
 					dataSource.AllowServerPaging = true;
-					if (view.CanRetrieveTotalRowCount)
+					if (SelectArguments.RetrieveTotalRowCount)
 						dataSource.VirtualCount = SelectArguments.TotalRowCount;
 				}
 			}
 
 			pageCount = dataSource.DataSourceCount;
-			bool showPager = AllowPaging && (dataSource.Count > 0);
+			bool showPager = AllowPaging && (pageCount > 1);
 			
 			Controls.Clear ();
 			table = CreateTable ();
 			Controls.Add (table);
-				
-			if (!Page.IsPostBack)
-				currentMode = DefaultMode;
 
 			// Gets the current data item
 			
@@ -885,16 +900,10 @@ namespace System.Web.UI.WebControls
 				itemRow = CreateRow (0, DataControlRowType.DataRow, rstate);
 				InitializeRow (itemRow);
 				table.Rows.Add (itemRow);
-				
-				if (!dataBinding) {
-					if (CurrentMode == FormViewMode.Edit)
-						oldEditValues = new DataKey (new OrderedDictionary ());
-					key = new DataKey (new OrderedDictionary (), DataKeyNames);
-				}
 			} else {
 				switch (CurrentMode) {
 				case FormViewMode.Edit:
-					itemRow = CreateRow (-1, DataControlRowType.DataRow, DataControlRowState.Edit);
+					itemRow = CreateRow (-1, DataControlRowType.EmptyDataRow, DataControlRowState.Edit);
 					break;
 				case FormViewMode.Insert:
 					itemRow = CreateRow (-1, DataControlRowType.DataRow, DataControlRowState.Insert);
@@ -906,12 +915,6 @@ namespace System.Web.UI.WebControls
 				table.Rows.Add (itemRow);
 				InitializeRow (itemRow);
 			}
-				
-			if (showPager && PagerSettings.Position == PagerPosition.Bottom || PagerSettings.Position == PagerPosition.TopAndBottom) {
-				bottomPagerRow = CreateRow (0, DataControlRowType.Pager, DataControlRowState.Normal);
-				InitializePager (bottomPagerRow, dataSource);
-				table.Rows.Add (bottomPagerRow);
-			}
 
 			if (FooterText.Length != 0 || footerTemplate != null) {
 				footerRow = CreateRow (-1, DataControlRowType.Footer, DataControlRowState.Normal);
@@ -919,6 +922,12 @@ namespace System.Web.UI.WebControls
 				table.Rows.Add (footerRow);
 			}
 			
+			if (showPager && PagerSettings.Position == PagerPosition.Bottom || PagerSettings.Position == PagerPosition.TopAndBottom) {
+				bottomPagerRow = CreateRow (0, DataControlRowType.Pager, DataControlRowState.Normal);
+				InitializePager (bottomPagerRow, dataSource);
+				table.Rows.Add (bottomPagerRow);
+			}
+
 			if (dataBinding)
 				DataBind (false);
 			
@@ -986,8 +995,10 @@ namespace System.Web.UI.WebControls
 			row.Cells.Add (cell);
 		}
 		
-		IOrderedDictionary CreateRowDataKey (object dataItem)
+		void FillRowDataKey (object dataItem)
 		{
+			KeyTable.Clear ();
+
 			if (cachedKeyProperties == null) {
 				PropertyDescriptorCollection props = TypeDescriptor.GetProperties (dataItem);
 				cachedKeyProperties = new PropertyDescriptor [DataKeyNames.Length];
@@ -998,11 +1009,8 @@ namespace System.Web.UI.WebControls
 					cachedKeyProperties [n] = p;
 				}
 			}
-			
-			OrderedDictionary dic = new OrderedDictionary ();
 			foreach (PropertyDescriptor p in cachedKeyProperties)
-				dic [p.Name] = p.GetValue (dataItem);
-			return dic;
+				KeyTable [p.Name] = p.GetValue (dataItem);
 		}
 		
 		IOrderedDictionary GetRowValues (bool includePrimaryKey)
@@ -1044,12 +1052,10 @@ namespace System.Web.UI.WebControls
 		
 		public sealed override void DataBind ()
 		{
-			DataSourceView view = GetData ();
-			if (AllowPaging && view.CanPage) {
-				SelectArguments.StartRowIndex = PageIndex;
-				SelectArguments.MaximumRows = 1;
-				if (view.CanRetrieveTotalRowCount)
-					SelectArguments.RetrieveTotalRowCount = true;
+			if (CurrentMode == FormViewMode.Insert) {
+				RequiresDataBinding = false;
+				PerformDataBinding (new object [] { null });
+				return;
 			}
 
 			cachedKeyProperties = null;
@@ -1058,7 +1064,8 @@ namespace System.Web.UI.WebControls
 			if (pageCount > 0) {
 				if (CurrentMode == FormViewMode.Edit)
 					oldEditValues = new DataKey (GetRowValues (true));
-				key = new DataKey (CreateRowDataKey (dataItem), DataKeyNames);
+				FillRowDataKey (dataItem);
+				key = new DataKey (KeyTable);
 			}
 		}
 		
@@ -1067,10 +1074,44 @@ namespace System.Web.UI.WebControls
 			base.PerformDataBinding (data);
 		}
 
-		[MonoTODO]
 		protected internal virtual void PrepareControlHierarchy ()
 		{
-			throw new NotImplementedException ();
+			if (table == null)
+				return;
+
+			table.Caption = Caption;
+			table.CaptionAlign = CaptionAlign;
+
+			foreach (FormViewRow row in table.Rows) {
+				switch (row.RowType) {
+				case DataControlRowType.Header:
+					if (headerStyle != null && !headerStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (headerStyle);
+					break;
+				case DataControlRowType.Footer:
+					if (footerStyle != null && !footerStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (footerStyle);
+					break;
+				case DataControlRowType.Pager:
+					if (pagerStyle != null && !pagerStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (pagerStyle);
+					break;
+				case DataControlRowType.EmptyDataRow:
+					if (emptyDataRowStyle != null && !emptyDataRowStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (emptyDataRowStyle);
+					break;
+				case DataControlRowType.DataRow:
+					if (rowStyle != null && !rowStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (rowStyle);
+					if ((row.RowState & (DataControlRowState.Edit | DataControlRowState.Insert)) != 0 && editRowStyle != null && !editRowStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (editRowStyle);
+					if ((row.RowState & DataControlRowState.Insert) != 0 && insertRowStyle != null && !insertRowStyle.IsEmpty)
+						row.ControlStyle.CopyFrom (insertRowStyle);
+					break;
+				default:
+					break;
+				}
+			}
 		}
 		
 		protected internal override void OnInit (EventArgs e)
@@ -1085,6 +1126,7 @@ namespace System.Web.UI.WebControls
 			if (args != null) {
 				OnItemCommand (args);
 				ProcessEvent (args.CommandName, args.CommandArgument as string);
+				return true;
 			}
 			return base.OnBubbleEvent (source, e);
 		}
@@ -1117,13 +1159,15 @@ namespace System.Web.UI.WebControls
 					newIndex = PageCount - 1;
 					break;
 				case DataControlCommands.NextPageCommandArgument:
-					if (PageIndex < PageCount - 1) newIndex = PageIndex + 1;
+					newIndex = PageIndex + 1;
 					break;
 				case DataControlCommands.PreviousPageCommandArgument:
-					if (PageIndex > 0) newIndex = PageIndex - 1;
+					newIndex = PageIndex - 1;
 					break;
 				default:
-					newIndex = int.Parse (param) - 1;
+					int paramIndex = 0;
+					int.TryParse (param, out paramIndex);
+					newIndex = paramIndex - 1;
 					break;
 				}
 				ShowPage (newIndex);
@@ -1173,15 +1217,15 @@ namespace System.Web.UI.WebControls
 			}
 		}
 		
-		void ShowPage (int newIndex)
+		void ShowPage (int index)
 		{
-			FormViewPageEventArgs args = new FormViewPageEventArgs (newIndex);
+			FormViewPageEventArgs args = new FormViewPageEventArgs (index);
 			OnPageIndexChanging (args);
 			if (!args.Cancel) {
-				newIndex = args.NewPageIndex;
+				int newIndex = args.NewPageIndex;
 				if (newIndex < 0 || newIndex >= PageCount)
 					return;
-				EndRowEdit ();
+				EndRowEdit (false);
 				PageIndex = newIndex;
 				OnPageIndexChanged (EventArgs.Empty);
 			}
@@ -1192,7 +1236,7 @@ namespace System.Web.UI.WebControls
 			FormViewModeEventArgs args = new FormViewModeEventArgs (newMode, false);
 			OnModeChanging (args);
 			if (!args.Cancel) {
-				currentMode = args.NewMode;
+				CurrentMode = args.NewMode;
 				OnModeChanged (EventArgs.Empty);
 				RequireBinding ();
 			}
@@ -1214,12 +1258,12 @@ namespace System.Web.UI.WebControls
 		
 		void UpdateItem (string param, bool causesValidation)
 		{
-			if (causesValidation)
+			if (causesValidation && Page != null)
 				Page.Validate ();
 			
-			if (currentMode != FormViewMode.Edit) throw new NotSupportedException ();
-			
-			currentEditOldValues = oldEditValues.Values;
+			if (currentMode != FormViewMode.Edit) throw new HttpException ("Must be in Edit mode");
+
+			currentEditOldValues = OldEditValues.Values;
 			currentEditRowKeys = DataKey.Values;
 			currentEditNewValues = GetRowValues (false);
 			
@@ -1227,9 +1271,12 @@ namespace System.Web.UI.WebControls
 			OnItemUpdating (args);
 			if (!args.Cancel) {
 				DataSourceView view = GetData ();
-				if (view == null) throw new HttpException ("The DataSourceView associated to data bound control was null");
-				view.Update (currentEditRowKeys, currentEditNewValues, currentEditOldValues, new DataSourceViewOperationCallback (UpdateCallback));
-			} else
+				if (view == null)
+					throw new HttpException ("The DataSourceView associated to data bound control was null");
+				if (view.CanUpdate)
+					view.Update (currentEditRowKeys, currentEditNewValues, currentEditOldValues, new DataSourceViewOperationCallback (UpdateCallback));
+			}
+			else
 				EndRowEdit ();
 		}
 
@@ -1251,20 +1298,22 @@ namespace System.Web.UI.WebControls
 		
 		void InsertItem (string param, bool causesValidation)
 		{
-			if (causesValidation)
+			if (causesValidation && Page != null)
 				Page.Validate ();
 			
-			if (currentMode != FormViewMode.Insert) throw new NotSupportedException ();
+			if (currentMode != FormViewMode.Insert) throw new HttpException ("Must be in Insert mode");
 			
 			currentEditNewValues = GetRowValues (true);
 			FormViewInsertEventArgs args = new FormViewInsertEventArgs (param, currentEditNewValues);
 			OnItemInserting (args);
 			if (!args.Cancel) {
 				DataSourceView view = GetData ();
-				if (view == null) 
-					return;
-				view.Insert (currentEditNewValues, new DataSourceViewOperationCallback (InsertCallback));
-			} else
+				if (view == null)
+					throw new HttpException ("The DataSourceView associated to data bound control was null");
+				if (view.CanInsert)
+					view.Insert (currentEditNewValues, new DataSourceViewOperationCallback (InsertCallback));
+			}
+			else
 				EndRowEdit ();
 		}
 		
@@ -1288,13 +1337,13 @@ namespace System.Web.UI.WebControls
 			OnItemDeleting (args);
 
 			if (!args.Cancel) {
-				if (PageIndex == PageCount - 1)
-					PageIndex --;
+				if (PageIndex > 0 && PageIndex == PageCount - 1)
+					PageIndex--;
 					
 				RequireBinding ();
 					
 				DataSourceView view = GetData ();
-				if (view != null)
+				if (view != null && view.CanDelete)
 					view.Delete (currentEditRowKeys, currentEditNewValues, new DataSourceViewOperationCallback (DeleteCallback));
 				else {
 					FormViewDeletedEventArgs dargs = new FormViewDeletedEventArgs (0, null, currentEditRowKeys, currentEditNewValues);
@@ -1312,7 +1361,13 @@ namespace System.Web.UI.WebControls
 		
 		void EndRowEdit ()
 		{
-			ChangeMode (DefaultMode);
+			EndRowEdit (true);
+		}
+
+		void EndRowEdit (bool switchToDefaultMode) 
+		{
+			if (switchToDefaultMode)
+				ChangeMode (DefaultMode);
 			oldEditValues = new DataKey (new OrderedDictionary ());
 			currentEditRowKeys = null;
 			currentEditOldValues = null;
@@ -1327,16 +1382,27 @@ namespace System.Web.UI.WebControls
 			base.LoadControlState (state[0]);
 			pageIndex = (int) state[1];
 			pageCount = (int) state[2];
-			currentMode = (FormViewMode) state[3];
+			CurrentMode = (FormViewMode) state[3];
 			defaultMode = (FormViewMode) state[4];
 			dataKeyNames = (string[]) state[5];
+			if (state [6] != null)
+				((IStateManager) DataKey).LoadViewState (state [6]);
+			if (state [7] != null)
+				((IStateManager) OldEditValues).LoadViewState (state [7]);
 		}
 		
 		protected internal override object SaveControlState ()
 		{
 			object bstate = base.SaveControlState ();
-			return new object[] {
-				bstate, pageIndex, pageCount, currentMode, defaultMode, dataKeyNames
+			return new object [] {
+				bstate, 
+				pageIndex, 
+				pageCount, 
+				CurrentMode, 
+				defaultMode, 
+				dataKeyNames,
+				(key == null ? null : ((IStateManager)key).SaveViewState()),
+				(oldEditValues == null ? null : ((IStateManager) oldEditValues).SaveViewState ())
 			};
 		}
 		
@@ -1351,23 +1417,20 @@ namespace System.Web.UI.WebControls
 			if (editRowStyle != null) ((IStateManager)editRowStyle).TrackViewState();
 			if (insertRowStyle != null) ((IStateManager)insertRowStyle).TrackViewState();
 			if (emptyDataRowStyle != null) ((IStateManager)emptyDataRowStyle).TrackViewState();
-			if (key != null) ((IStateManager)key).TrackViewState();
 		}
 
 		protected override object SaveViewState()
 		{
-			object[] states = new object [14];
+			object[] states = new object [10];
 			states[0] = base.SaveViewState();
-			states[2] = (pagerSettings == null ? null : ((IStateManager)pagerSettings).SaveViewState());
-			states[4] = (footerStyle == null ? null : ((IStateManager)footerStyle).SaveViewState());
-			states[5] = (headerStyle == null ? null : ((IStateManager)headerStyle).SaveViewState());
-			states[6] = (pagerStyle == null ? null : ((IStateManager)pagerStyle).SaveViewState());
-			states[7] = (rowStyle == null ? null : ((IStateManager)rowStyle).SaveViewState());
-			states[8] = (insertRowStyle == null ? null : ((IStateManager)insertRowStyle).SaveViewState());
-			states[9] = (editRowStyle == null ? null : ((IStateManager)editRowStyle).SaveViewState());
-			states[10] = (emptyDataRowStyle == null ? null : ((IStateManager)emptyDataRowStyle).SaveViewState());
-			states[11] = (key == null ? null : ((IStateManager)key).SaveViewState());
-			states[12] = (oldEditValues == null ? null : ((IStateManager)oldEditValues).SaveViewState());
+			states[1] = (pagerSettings == null ? null : ((IStateManager)pagerSettings).SaveViewState());
+			states[2] = (footerStyle == null ? null : ((IStateManager)footerStyle).SaveViewState());
+			states[3] = (headerStyle == null ? null : ((IStateManager)headerStyle).SaveViewState());
+			states[4] = (pagerStyle == null ? null : ((IStateManager)pagerStyle).SaveViewState());
+			states[5] = (rowStyle == null ? null : ((IStateManager)rowStyle).SaveViewState());
+			states[6] = (insertRowStyle == null ? null : ((IStateManager)insertRowStyle).SaveViewState());
+			states[7] = (editRowStyle == null ? null : ((IStateManager)editRowStyle).SaveViewState());
+			states[8] = (emptyDataRowStyle == null ? null : ((IStateManager)emptyDataRowStyle).SaveViewState());
 			
 			for (int i = states.Length - 1; i >= 0; i--) {
 				if (states [i] != null)
@@ -1387,86 +1450,40 @@ namespace System.Web.UI.WebControls
 			object [] states = (object []) savedState;
 			
 			base.LoadViewState (states[0]);
-			EnsureChildControls ();
 			
-			if (states[2] != null) ((IStateManager)PagerSettings).LoadViewState (states[2]);
-			if (states[4] != null) ((IStateManager)FooterStyle).LoadViewState (states[4]);
-			if (states[5] != null) ((IStateManager)HeaderStyle).LoadViewState (states[5]);
-			if (states[6] != null) ((IStateManager)PagerStyle).LoadViewState (states[6]);
-			if (states[7] != null) ((IStateManager)RowStyle).LoadViewState (states[7]);
-			if (states[8] != null) ((IStateManager)InsertRowStyle).LoadViewState (states[8]);
-			if (states[9] != null) ((IStateManager)EditRowStyle).LoadViewState (states[9]);
-			if (states[10] != null) ((IStateManager)EmptyDataRowStyle).LoadViewState (states[10]);
-			if (states[11] != null && DataKey != null) ((IStateManager)DataKey).LoadViewState (states[11]);
-			if (states[12] != null && oldEditValues != null) ((IStateManager)oldEditValues).LoadViewState (states[12]);
+			if (states[1] != null) ((IStateManager)PagerSettings).LoadViewState (states[1]);
+			if (states[2] != null) ((IStateManager)FooterStyle).LoadViewState (states[2]);
+			if (states[3] != null) ((IStateManager)HeaderStyle).LoadViewState (states[3]);
+			if (states[4] != null) ((IStateManager)PagerStyle).LoadViewState (states[4]);
+			if (states[5] != null) ((IStateManager)RowStyle).LoadViewState (states[5]);
+			if (states[6] != null) ((IStateManager)InsertRowStyle).LoadViewState (states[6]);
+			if (states[7] != null) ((IStateManager)EditRowStyle).LoadViewState (states[7]);
+			if (states[8] != null) ((IStateManager)EmptyDataRowStyle).LoadViewState (states[8]);
 		}
 		
 		protected internal override void Render (HtmlTextWriter writer)
 		{
-			switch (GridLines) {
-			case GridLines.Horizontal:
-				writer.AddAttribute (HtmlTextWriterAttribute.Rules, "rows");
-				writer.AddAttribute (HtmlTextWriterAttribute.Border, "1");
-				break;
-			case GridLines.Vertical:
-				writer.AddAttribute (HtmlTextWriterAttribute.Rules, "cols");
-				writer.AddAttribute (HtmlTextWriterAttribute.Border, "1");
-				break;
-			case GridLines.Both:
-				writer.AddAttribute (HtmlTextWriterAttribute.Rules, "all");
-				writer.AddAttribute (HtmlTextWriterAttribute.Border, "1");
-				break;
-			default:
-				writer.AddAttribute (HtmlTextWriterAttribute.Border, "0");
-				break;
-			}
+			PrepareControlHierarchy ();
 			
-			writer.AddAttribute (HtmlTextWriterAttribute.Cellspacing, "0");
-			if (!string.IsNullOrEmpty (ControlStyle.CssClass))
-				writer.AddAttribute (HtmlTextWriterAttribute.Class, ControlStyle.CssClass);
+			if (table == null)
+				return;
 
-			table.ControlStyle.MergeWith (ControlStyle);
-			table.RenderBeginTag (writer);
-			
-			foreach (FormViewRow row in table.Rows)
-			{
-				switch (row.RowType) {
-				case DataControlRowType.Header:
-					if (headerStyle != null)headerStyle.AddAttributesToRender (writer, row);
-					break;
-				case DataControlRowType.Footer:
-					if (footerStyle != null) footerStyle.AddAttributesToRender (writer, row);
-					break;
-				case DataControlRowType.Pager:
-					if (pagerStyle != null) pagerStyle.AddAttributesToRender (writer, row);
-					break;
-				case DataControlRowType.EmptyDataRow:
-					if (emptyDataRowStyle != null) emptyDataRowStyle.AddAttributesToRender (writer, row);
-					break;
-				default:
-					if (rowStyle != null) rowStyle.AddAttributesToRender (writer, row);
-					break;
-				}
-
-				if ((row.RowState & DataControlRowState.Edit) != 0 && editRowStyle != null)
-					editRowStyle.AddAttributesToRender (writer, row);
-				if ((row.RowState & DataControlRowState.Insert) != 0 && insertRowStyle != null)
-					insertRowStyle.AddAttributesToRender (writer, row);
-					
-				row.RenderBeginTag (writer);
-				
-				for (int n=0; n<row.Cells.Count; n++)
-					row.Cells[n].Render (writer);
-
-				row.RenderEndTag (writer);
-			}
-			table.RenderEndTag (writer);
+			table.Render (writer);
 		}
 
 		PostBackOptions IPostBackContainer.GetPostBackOptions (IButtonControl control)
 		{
-			Control ctrl = control as Control;
-			return new PostBackOptions(ctrl);
+			if (control == null)
+				throw new ArgumentNullException ("control");
+
+			if (control.CausesValidation)
+				throw new InvalidOperationException ("A button that causes validation in FormView '" + ID + "' is attempting to use the container GridView as the post back target.  The button should either turn off validation or use itself as the post back container.");
+
+			PostBackOptions options = new PostBackOptions (this);
+			options.Argument = control.CommandName + "$" + control.CommandArgument;
+			options.RequiresJavaScriptProtocol = true;
+
+			return options;
 		}
 
 	}
