@@ -70,7 +70,6 @@ namespace System.Web.UI.WebControls {
 #if NET_2_0
 		[DefaultValue (false)]
 		[Themeable (false)]
-		[MonoTODO]
 		[WebSysDescription ("")]
 		[WebCategory ("Behavior")]
 		public virtual bool AppendDataBoundItems
@@ -201,6 +200,13 @@ namespace System.Web.UI.WebControls {
 					return;
 				}
 
+#if NET_2_0
+				if (value >= Items.Count) {
+					ClearSelection ();
+					return;
+				}
+#endif
+
 				if (value < -1 || value >= Items.Count)
 					throw new ArgumentOutOfRangeException ("value");
 
@@ -266,10 +272,12 @@ namespace System.Web.UI.WebControls {
 					}
 				}
 
+#if !NET_2_0
 				if (thr) {
 					string msg = String.Format ("Argument value is out of range: {0}", value);
 					throw new ArgumentOutOfRangeException (msg);
 				}
+#endif
 			}
 		}
 
@@ -278,7 +286,6 @@ namespace System.Web.UI.WebControls {
 		[DefaultValue ("")]
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 		[Browsable (false)]
-		[MonoTODO]
 		[WebSysDescription ("")]
 		[WebCategoryAttribute ("Behavior")]
 		public virtual string Text {
@@ -321,64 +328,19 @@ namespace System.Web.UI.WebControls {
 				items [i].Selected = false;
 		}
 
-#if NET_2_0
-		protected internal override void LoadControlState (object savedState)
-		{
-			object first = null;
-			ArrayList indices = null;
-			Pair pair = savedState as Pair;
-
-			if (pair != null) {
-				first = pair.First;
-				indices = pair.Second as ArrayList;
-			}
-
-			base.LoadControlState (first);
-
-			if (indices != null) {
-				int count = Items.Count;
-				foreach (int index in indices) {
-					if (index >= 0 && index < count)
-						Items [index].Selected = true;
-				}
-			}
-		}
-#endif		
-
 		protected override void OnDataBinding (EventArgs e)
 		{
 			base.OnDataBinding (e);
 
+#if !NET_2_0
 			IEnumerable list = DataSourceResolver.ResolveDataSource (DataSource, DataMember);
 			if (list == null)
 				return;
 
-#if NET_2_0
-			if (!AppendDataBoundItems)
-#endif
-				Items.Clear();
-
-#if !NET_2_0
 			DoDataBinding (list);
 #endif
-			if (saved_selected_value != null) {
-				SelectedValue = saved_selected_value;
-				if (saved_selected_index != -2 && saved_selected_index != SelectedIndex)
-					throw new ArgumentException ("SelectedIndex and SelectedValue are mutually exclusive.");
-			} else if (saved_selected_index != -2) {
-				SelectedIndex = saved_selected_index;
-				// No need to check saved_selected_value here, as it's done before.
-			}
 		}
 
-#if NET_2_0
-		protected internal override void OnInit (EventArgs e)
-		{
-			Page.RegisterRequiresControlState (this);
-			base.OnInit (e);
-		}
-#endif		
-
 #if NET_2_0
 		protected internal
 #else		
@@ -392,6 +354,11 @@ namespace System.Web.UI.WebControls {
 		void DoDataBinding (IEnumerable dataSource)
 		{
 			if (dataSource != null) {
+#if NET_2_0
+				if (!AppendDataBoundItems)
+#endif
+					Items.Clear ();
+
 				string format = DataTextFormatString;
 				if (format == "")
 					format = null;
@@ -424,6 +391,16 @@ namespace System.Web.UI.WebControls {
 					coll.Add (new ListItem (text, val));
 				}
 			}
+			
+			if (saved_selected_value != null) {
+				SelectedValue = saved_selected_value;
+				if (saved_selected_index != -2 && saved_selected_index != SelectedIndex)
+					throw new ArgumentException ("SelectedIndex and SelectedValue are mutually exclusive.");
+			}
+			else if (saved_selected_index != -2) {
+				SelectedIndex = saved_selected_index;
+				// No need to check saved_selected_value here, as it's done before.
+			}
 		}
 
 #if NET_2_0
@@ -438,6 +415,8 @@ namespace System.Web.UI.WebControls {
 		{
 			base.PerformDataBinding (dataSource);
 
+			if (dataSource == null)
+				return;
 			DoDataBinding (dataSource);
 		}
 
@@ -447,24 +426,11 @@ namespace System.Web.UI.WebControls {
 			base.PerformSelect ();
 		}
 
-		[MonoTODO]
 		protected internal override void RenderContents (HtmlTextWriter w)
 		{
 			base.RenderContents (w);
 		}
 
-		protected internal override object SaveControlState ()
-		{
-			object first;
-			ArrayList second;
-
-			first = base.SaveControlState ();
-			second = GetSelectedIndicesInternal ();
-			if (second == null)
-				second = new ArrayList();
-
-			return new Pair (first, second);
-		}
 #endif		
 
 		internal ArrayList GetSelectedIndicesInternal ()
@@ -492,46 +458,26 @@ namespace System.Web.UI.WebControls {
 			if (manager != null)
 				second = manager.SaveViewState ();
 
-#if !NET_2_0
 			ArrayList selected = GetSelectedIndicesInternal ();
-#endif
 
-			if (first == null && second == null
-#if !NET_2_0
-			    && selected == null
-#endif
-			    )
+			if (first == null && second == null && selected == null)
 				return null;
 
-#if NET_2_0
-			return new Pair (first, second);
-#else
 			return new Triplet (first, second, selected);
-#endif
 		}
 
 		protected override void LoadViewState (object savedState)
 		{
 			object first = null;
 			object second = null;
-#if !NET_2_0
 			ArrayList indices = null;
-#endif
 
-#if NET_2_0
-			Pair pair = savedState as Pair;
-			if (pair != null) {
-				first = pair.First;
-				second = pair.Second;
-			}
-#else
 			Triplet triplet = savedState as Triplet;
 			if (triplet != null) {
 				first = triplet.First;
 				second = triplet.Second;
 				indices = triplet.Third as ArrayList;
 			}
-#endif
 
 			base.LoadViewState (first);
 
@@ -540,16 +486,14 @@ namespace System.Web.UI.WebControls {
 				manager.LoadViewState (second);
 			}
 
-#if !NET_2_0
 			if (indices != null) {
 				foreach (int index in indices)
 					Items [index].Selected = true;
 			}
-#endif
 		}
 
 #if NET_2_0
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		protected void SetPostDataSelection (int selectedIndex)
 		{
 			throw new NotImplementedException ();
@@ -596,31 +540,30 @@ namespace System.Web.UI.WebControls {
 		}
 		
 		
-		[MonoTODO]
 		[Themeable (false)]
 		[DefaultValue (false)]
 		[WebSysDescription ("")]
 		[WebCategory ("Behavior")]
-	        public virtual bool CausesValidation {
+		public virtual bool CausesValidation {
 			get {
-				throw new NotImplementedException ();
+				return ViewState.GetBool ("CausesValidation", false);
 			}
+
 			set {
-				throw new NotImplementedException ();	
+				ViewState ["CausesValidation"] = value;
 			}
 		}
 
-		[MonoTODO]
 		[Themeable (false)]
 		[DefaultValue ("")]
 		[WebSysDescription ("")]
 		[WebCategoryAttribute ("Behavior")]
 		public virtual string ValidationGroup {
 			get {
-				throw new NotImplementedException ();
+				return ViewState.GetString ("ValidationGroup", "");
 			}
 			set {
-				throw new NotImplementedException ();
+				ViewState ["ValidationGroup"] = value;
 			}
 		}
 
@@ -629,3 +572,5 @@ namespace System.Web.UI.WebControls {
 	}
 }
 
+
+
