@@ -306,7 +306,7 @@ namespace System.Web.UI.WebControls {
 					writer.RenderBeginTag (HtmlTextWriterTag.Td);
 
 					if (userImg) {
-						writer.AddAttribute (HtmlTextWriterAttribute.Src, _login.CreateUserIconUrl);
+						writer.AddAttribute (HtmlTextWriterAttribute.Src, ResolveClientUrl (_login.CreateUserIconUrl));
 						writer.AddStyleAttribute (HtmlTextWriterStyle.BorderWidth, "0px");
 						if (userText)
 							writer.AddAttribute (HtmlTextWriterAttribute.Alt, _login.CreateUserText);
@@ -335,7 +335,7 @@ namespace System.Web.UI.WebControls {
 						}
 
 						if (passImg) {
-							writer.AddAttribute (HtmlTextWriterAttribute.Src, _login.PasswordRecoveryIconUrl);
+							writer.AddAttribute (HtmlTextWriterAttribute.Src, ResolveClientUrl (_login.PasswordRecoveryIconUrl));
 							writer.AddStyleAttribute (HtmlTextWriterStyle.BorderWidth, "0px");
 							if (passText)
 								writer.AddAttribute (HtmlTextWriterAttribute.Alt, _login.PasswordRecoveryText);
@@ -365,7 +365,7 @@ namespace System.Web.UI.WebControls {
 						}
 
 						if (helpImg) {
-							writer.AddAttribute (HtmlTextWriterAttribute.Src, _login.HelpPageIconUrl);
+							writer.AddAttribute (HtmlTextWriterAttribute.Src, ResolveClientUrl (_login.HelpPageIconUrl));
 							writer.AddStyleAttribute (HtmlTextWriterStyle.BorderWidth, "0px");
 							if (helpText)
 								writer.AddAttribute (HtmlTextWriterAttribute.Alt, _login.HelpPageText);
@@ -440,6 +440,11 @@ namespace System.Web.UI.WebControls {
 		{
 		}
 
+		ITextControl FailureTextLabel {
+			get {
+				return FindControl ("FailureText") as ITextControl;
+			}
+		}
 
 		[DefaultValue (1)]
 		public virtual int BorderPadding {
@@ -1071,19 +1076,24 @@ namespace System.Web.UI.WebControls {
 
 			editable = container.UserNameTextBox as IEditableTextControl;
 
-			if (editable != null)
+			if (editable != null) {
+				editable.Text = UserName;
 				editable.TextChanged += new EventHandler (UserName_TextChanged);
+			}
+			else
+				throw new HttpException ("LayoutTemplate does not contain an IEditableTextControl with ID UserName for the username.");
 
 			editable = container.PasswordTextBox as IEditableTextControl;
 
 			if (editable != null)
 				editable.TextChanged += new EventHandler (Password_TextChanged);
+			else
+				throw new HttpException ("LayoutTemplate does not contain an IEditableTextControl with ID Password for the password.");
 
 			ICheckBoxControl checkBox = container.RememberMeCheckBox as ICheckBoxControl;
 
 			if (checkBox != null)
 				checkBox.CheckedChanged += new EventHandler (RememberMe_CheckedChanged);
-			
 		}
 
 		protected override void LoadViewState (object savedState)
@@ -1128,7 +1138,11 @@ namespace System.Web.UI.WebControls {
 			// check for submit button
 			CommandEventArgs cea = (e as CommandEventArgs);
 			if ((cea != null) && (cea.CommandName == LoginButtonCommandName)) {
-				AuthenticateUser ();
+				if (!AuthenticateUser ()) {
+					ITextControl failureText = FailureTextLabel;
+					if (failureText != null)
+						failureText.Text = FailureText;
+				}
 				return true;
 			}
 			return false;
@@ -1285,8 +1299,13 @@ namespace System.Web.UI.WebControls {
 			OnAuthenticate (aea);
 
 			if (aea.Authenticated) {
+				FormsAuthentication.SetAuthCookie (UserName, RememberMeSet);				
+
 				string url = DestinationPageUrl;
-				FormsAuthentication.SetAuthCookie (UserName, RememberMeSet);
+				if (url.Length == 0 && Page.Request.UrlReferrer != null)
+					url = Page.Request.UrlReferrer.ToString();
+				if (url.Length == 0 && FormsAuthentication.DefaultUrl != null)
+					url = FormsAuthentication.DefaultUrl;
 				if (url.Length == 0) {
 					Redirect (FormsAuthentication.LoginUrl);
 				} else {
@@ -1304,6 +1323,9 @@ namespace System.Web.UI.WebControls {
 			}
 		}
 
+		// TODO: its called from default template only, not usefully, OnBubbleEvent 
+		// do handle command, need be removed
+		[MonoTODO()]
 		private void LoginClick (object sender, CommandEventArgs e)
 		{
 			RaiseBubbleEvent (sender, (EventArgs)e);
@@ -1320,8 +1342,10 @@ namespace System.Web.UI.WebControls {
 		{
 			if ((Page == null) || (Page.Request == null))
 				return false;
-			string url = Page.Request.Url.AbsolutePath;
 			string defaultLogin = FormsAuthentication.LoginUrl;
+			if (defaultLogin == null)
+				return false;
+			string url = Page.Request.Url.AbsolutePath;
 			return (String.Compare (defaultLogin, 0, url, url.Length - defaultLogin.Length, defaultLogin.Length,
 				true, CultureInfo.InvariantCulture) == 0);
 		}
