@@ -656,7 +656,6 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
-		[MonoTODO]
 		public string DynamicPopOutImageTextFormatString 
 		{
 			get
@@ -686,7 +685,6 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
-		[MonoTODO]
 		public string StaticPopOutImageTextFormatString
 		{
 			get
@@ -765,7 +763,6 @@ namespace System.Web.UI.WebControls
 			get { return selectedItem != null ? selectedItem.Value : ""; }
 		}
 
-		[MonoTODO]
 		[Localizable (true)]
 		public string SkipLinkText 
 		{
@@ -944,11 +941,11 @@ namespace System.Web.UI.WebControls
 				dynamicMenuItemStyle.TrackViewState ();
 			if (dynamicMenuStyle != null)
 				dynamicMenuStyle.TrackViewState ();
-			if (levelMenuItemStyles != null)
+			if (levelMenuItemStyles != null && levelMenuItemStyles.Count > 0)
 				((IStateManager)levelMenuItemStyles).TrackViewState();
-			if (levelSelectedStyles != null)
+			if (levelSelectedStyles != null && levelMenuItemStyles.Count > 0)
 				((IStateManager)levelSelectedStyles).TrackViewState();
-			if (levelSubMenuStyles != null)
+			if (levelSubMenuStyles != null && levelSubMenuStyles.Count > 0)
 				((IStateManager)levelSubMenuStyles).TrackViewState();
 			if (dynamicSelectedStyle != null)
 				dynamicSelectedStyle.TrackViewState();
@@ -1105,13 +1102,13 @@ namespace System.Web.UI.WebControls
 			_requiresChildControlsDataBinding = false;
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		protected override IDictionary GetDesignModeState ()
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		protected override void SetDesignModeState (IDictionary data)
 		{
 			throw new NotImplementedException ();
@@ -1178,8 +1175,8 @@ namespace System.Web.UI.WebControls
 			if (dynamicMenuStyle != null)
 				RegisterStyle (DynamicMenuStyle);
 
-			if (levelMenuItemStyles != null) {
-				levelMenuItemLinkStyles = new List<Style> ();
+			if (levelMenuItemStyles != null && levelMenuItemStyles.Count > 0) {
+				levelMenuItemLinkStyles = new List<Style> (levelMenuItemStyles.Count);
 				foreach (Style style in levelMenuItemStyles) {
 					Style linkStyle = new Style ();
 					levelMenuItemLinkStyles.Add (linkStyle);
@@ -1197,8 +1194,8 @@ namespace System.Web.UI.WebControls
 			if (dynamicSelectedStyle != null)
 				RegisterStyle (dynamicSelectedStyle, DynamicSelectedLinkStyle);
 
-			if (levelSelectedStyles != null) {
-				levelSelectedLinkStyles = new List<Style> ();
+			if (levelSelectedStyles != null && levelSelectedStyles.Count > 0) {
+				levelSelectedLinkStyles = new List<Style> (levelSelectedStyles.Count);
 				foreach (Style style in levelSelectedStyles) {
 					Style linkStyle = new Style ();
 					levelSelectedLinkStyles.Add (linkStyle);
@@ -1244,11 +1241,7 @@ namespace System.Web.UI.WebControls
 		}
 
 		void RegisterStyle (Style baseStyle, Style linkStyle) {
-			if (baseStyle.ForeColor != Color.Empty)
-				linkStyle.ForeColor = baseStyle.ForeColor;
-			if (!baseStyle.Font.IsEmpty) {
-				linkStyle.Font.CopyFrom (baseStyle.Font);
-			}
+			linkStyle.CopyTextStylesFrom (baseStyle);
 			linkStyle.BorderStyle = BorderStyle.None;
 			RegisterStyle (linkStyle);
 			RegisterStyle (baseStyle);
@@ -1298,12 +1291,7 @@ namespace System.Web.UI.WebControls
 		}
 
 		void AddCssClass (Style style, string cssClass) {
-			if (String.IsNullOrEmpty (cssClass))
-				return;
-
-			if(style.CssClass.Length>0)
-				style.CssClass += " ";
-			style.CssClass += cssClass;
+			style.AddCssClass (cssClass);
 		}
 		
 		public override void RenderBeginTag (HtmlTextWriter writer)
@@ -1329,9 +1317,12 @@ namespace System.Web.UI.WebControls
 		{
 			base.RenderEndTag (writer);
 
+			if (StaticDisplayLevels == 1 && MaximumDynamicDisplayLevels > 0)
+				RenderDynamicMenu (writer, Items);
+
 			if (SkipLinkText != "") {
 				System.Web.UI.HtmlControls.HtmlAnchor anchor = new System.Web.UI.HtmlControls.HtmlAnchor ();
-				anchor.ID = "SkipLink";
+				anchor.ID = ClientID + "_SkipLink";
 				anchor.Render (writer);
 			}
 		}
@@ -1642,7 +1633,6 @@ namespace System.Web.UI.WebControls
 
 			RenderItemHref (writer, item);
 			
-			//ControlLinkStyle.AddAttributesToRender (writer);
 			Style linkStyle = new Style ();
 			if (Page.Header != null) {
 				// styles are registered
