@@ -46,7 +46,9 @@ namespace System.Web.UI.WebControls {
 #endif
 	public class RadioButtonList : ListControl, IRepeatInfoUser,
 		INamingContainer, IPostBackDataHandler {
+#if !NET_2_0
 		bool need_raise;
+#endif
 		short tabIndex = 0;
 
 		public RadioButtonList ()
@@ -171,7 +173,7 @@ namespace System.Web.UI.WebControls {
 		// Interface properties
 
 #if NET_2_0
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		protected virtual bool HasFooter
 		{
 			get {
@@ -179,7 +181,7 @@ namespace System.Web.UI.WebControls {
 			}
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		protected virtual bool HasHeader 
 		{
 			get {
@@ -195,7 +197,7 @@ namespace System.Web.UI.WebControls {
 			}
 		}
 		
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		protected virtual int RepeatedItemCount
 		{
 			get {
@@ -226,7 +228,7 @@ namespace System.Web.UI.WebControls {
 		}
 
 #if NET_2_0
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		protected override Control FindControl (string id, int pathOffset)
 		{
 			throw new NotImplementedException ();
@@ -257,6 +259,10 @@ namespace System.Web.UI.WebControls {
 			radio.AutoPostBack = AutoPostBack;
 			radio.Enabled = Enabled;
 			radio.TabIndex = tabIndex;
+#if NET_2_0
+			radio.ValidationGroup = ValidationGroup;
+			radio.CausesValidation = CausesValidation;
+#endif
 			radio.RenderControl (writer);
 		}
 #if NET_2_0
@@ -275,9 +281,15 @@ namespace System.Web.UI.WebControls {
 
 				if (i != selected) {
 					SelectedIndex = i;
+#if NET_2_0
+					return true;
+#else
 					need_raise = true;
+#endif
 				}
+#if !NET_2_0
 				return true;
+#endif
 			}
 
 			return false;
@@ -288,7 +300,14 @@ namespace System.Web.UI.WebControls {
 #endif
 		void RaisePostDataChangedEvent ()
 		{
+#if NET_2_0
+			if (CausesValidation)
+				Page.Validate (ValidationGroup);
+#endif
+
+#if !NET_2_0
 			if (need_raise)
+#endif
 				OnSelectedIndexChanged (EventArgs.Empty);
 		}
 
@@ -319,6 +338,13 @@ namespace System.Web.UI.WebControls {
 #endif		
 		override void Render (HtmlTextWriter writer)
 		{
+#if NET_2_0
+			if (Page != null)
+				Page.ClientScript.RegisterForEventValidation (this.UniqueID);
+
+			if (Items.Count == 0)
+				return;
+#endif
 			RepeatInfo repeat = new RepeatInfo ();
 			repeat.RepeatColumns = RepeatColumns;
 			repeat.RepeatDirection = RepeatDirection;
@@ -335,3 +361,4 @@ namespace System.Web.UI.WebControls {
 
 }
 
+
