@@ -37,6 +37,7 @@ using System.Web.Handlers;
 using System.Collections.Specialized;
 using System.IO;
 using System.Security.Permissions;
+using System.Collections.Generic;
 
 namespace System.Web.UI.WebControls
 {
@@ -50,7 +51,7 @@ namespace System.Web.UI.WebControls
 	[Designer ("System.Web.UI.Design.WebControls.TreeViewDesigner, " + Consts.AssemblySystem_Design, "System.ComponentModel.Design.IDesigner")]
 	public class TreeView: HierarchicalDataBoundControl, IPostBackEventHandler, IPostBackDataHandler, ICallbackEventHandler
 	{
-		bool cssStylesRegistered;
+		bool stylesPrepared;
 		Style hoverNodeStyle;
 		TreeNodeStyle leafNodeStyle;
 		TreeNodeStyle nodeStyle;
@@ -64,6 +65,16 @@ namespace System.Web.UI.WebControls
 		
 		TreeNode selectedNode;
 		Hashtable bindings;
+
+		int registeredStylesCounter = -1;
+		List<Style> levelLinkStyles;
+		Style controlLinkStyle;
+		Style nodeLinkStyle;
+		Style rootNodeLinkStyle;
+		Style parentNodeLinkStyle;
+		Style leafNodeLinkStyle;
+		Style selectedNodeLinkStyle;
+		Style hoverNodeLinkStyle;
 		
 		private static readonly object TreeNodeCheckChangedEvent = new object();
 		private static readonly object SelectedNodeChangedEvent = new object();
@@ -319,6 +330,8 @@ namespace System.Web.UI.WebControls
 				return (TreeViewImageSet)ViewState.GetInt ("ImageSet", (int)TreeViewImageSet.Custom);
 			}
 			set {
+				if (!Enum.IsDefined (typeof (TreeViewImageSet), value))
+					throw new ArgumentOutOfRangeException ();
 				ViewState["ImageSet"] = value;
 			}
 		}
@@ -506,6 +519,70 @@ namespace System.Web.UI.WebControls
 				return selectedNodeStyle;
 			}
 		}
+
+		private Style ControlLinkStyle {
+			get {
+				if (controlLinkStyle == null) {
+					controlLinkStyle = new Style ();
+					controlLinkStyle.AlwaysRenderTextDecoration = true;
+				}
+				return controlLinkStyle;
+			}
+		}
+
+		private Style NodeLinkStyle {
+			get {
+				if (nodeLinkStyle == null) {
+					nodeLinkStyle = new Style ();
+				}
+				return nodeLinkStyle;
+			}
+		}
+
+		private Style RootNodeLinkStyle {
+			get {
+				if (rootNodeLinkStyle == null) {
+					rootNodeLinkStyle = new Style ();
+				}
+				return rootNodeLinkStyle;
+			}
+		}
+
+		private Style ParentNodeLinkStyle {
+			get {
+				if (parentNodeLinkStyle == null) {
+					parentNodeLinkStyle = new Style ();
+				}
+				return parentNodeLinkStyle;
+			}
+		}
+
+		private Style SelectedNodeLinkStyle {
+			get {
+				if (selectedNodeLinkStyle == null) {
+					selectedNodeLinkStyle = new Style ();
+				}
+				return selectedNodeLinkStyle;
+			}
+		}
+
+		private Style LeafNodeLinkStyle {
+			get {
+				if (leafNodeLinkStyle == null) {
+					leafNodeLinkStyle = new Style ();
+				}
+				return leafNodeLinkStyle;
+			}
+		}
+
+		private Style HoverNodeLinkStyle {
+			get {
+				if (hoverNodeLinkStyle == null) {
+					hoverNodeLinkStyle = new Style ();
+				}
+				return hoverNodeLinkStyle;
+			}
+		}
 		
 		[DefaultValue (TreeNodeTypes.None)]
 		public TreeNodeTypes ShowCheckBoxes {
@@ -513,6 +590,8 @@ namespace System.Web.UI.WebControls
 				return (TreeNodeTypes)ViewState.GetInt ("ShowCheckBoxes", (int)TreeNodeTypes.None);
 			}
 			set {
+				if ((int) value > 7)
+					throw new ArgumentOutOfRangeException ();
 				ViewState ["ShowCheckBoxes"] = value;
 			}
 		}
@@ -674,10 +753,6 @@ namespace System.Web.UI.WebControls
 		protected internal override void OnInit (EventArgs e)
 		{
 			base.OnInit (e);
-			if (!Page.IsPostBack && ExpandDepth != 0) {
-				foreach (TreeNode node in Nodes)
-					node.Expand (ExpandDepth - 1);
-			}
 		}
 		
 		internal void SetSelectedNode (TreeNode node, bool loading)
@@ -720,7 +795,7 @@ namespace System.Web.UI.WebControls
 			if (leafNodeStyle != null) {
 				leafNodeStyle.TrackViewState();
 			}
-			if (levelStyles != null) {
+			if (levelStyles != null && levelStyles.Count > 0) {
 				((IStateManager)levelStyles).TrackViewState();
 			}
 			if (nodeStyle != null) {
@@ -847,7 +922,9 @@ namespace System.Web.UI.WebControls
 			
 			StringWriter sw = new StringWriter ();
 			HtmlTextWriter writer = new HtmlTextWriter (sw);
+			EnsureStylesPrepared ();
 			
+			node.Expanded = true;
 			int num = node.ChildNodes.Count;
 			for (int n=0; n<num; n++)
 				RenderNode (writer, node.ChildNodes [n], node.Depth + 1, levelLines, true, n<num-1);
@@ -894,13 +971,31 @@ namespace System.Web.UI.WebControls
 		protected internal override void PerformDataBinding ()
 		{
 			base.PerformDataBinding ();
+			InitializeDataBindings ();
 			HierarchicalDataSourceView data = GetData ("");
+			if (data == null)
+				return;
+			Nodes.Clear ();
 			IHierarchicalEnumerable e = data.Select ();
-			foreach (object obj in e) {
-				IHierarchyData hdata = e.GetHierarchyData (obj);
-				TreeNode node = CreateNode ();
-				Nodes.Add (node);
-				node.Bind (hdata);
+			FillBoundChildrenRecursive (e, Nodes);
+		}
+		
+		private void FillBoundChildrenRecursive (IHierarchicalEnumerable hEnumerable, TreeNodeCollection nodeCollection) {
+			foreach (object obj in hEnumerable) {
+				IHierarchyData hdata = hEnumerable.GetHierarchyData (obj);
+				TreeNode child = new TreeNode ();
+				nodeCollection.Add (child);
+				child.Bind (hdata);
+				OnTreeNodeDataBound (new TreeNodeEventArgs (child));
+
+				if (MaxDataBindDepth >= 0 && child.Depth == MaxDataBindDepth)
+					continue;
+
+				if (hdata == null || !hdata.HasChildren)
+					continue;
+
+				IHierarchicalEnumerable e = hdata.GetChildren ();
+				FillBoundChildrenRecursive (e, child.ChildNodes);
 			}
 		}
 		
@@ -908,6 +1003,20 @@ namespace System.Web.UI.WebControls
 		{
 			bool res = false;
 
+			if (EnableClientScript && PopulateNodesFromClient) {
+				string states = postCollection [ClientID + "_PopulatedStates"];
+				if (states != null) {
+					foreach (string id in states.Split ('|')) {
+						if (String.IsNullOrEmpty(id))
+							continue;
+						TreeNode node = FindNodeByPos (id);
+						if (node != null && node.PopulateOnDemand && !node.Populated)
+							node.Populate();
+					}
+				}
+				res = true;
+			}
+
 			if (ShowCheckBoxes != TreeNodeTypes.None) {
 				UnsetCheckStates (Nodes, postCollection);
 				SetCheckStates (postCollection);
@@ -932,8 +1041,6 @@ namespace System.Web.UI.WebControls
 		{
 			base.OnPreRender (e);
 
-			cssStylesRegistered = false;
-			ControlStyle.AlwaysRenderTextDecoration = true;
 			if (Page != null) {
 				if (Enabled)
 					Page.RegisterRequiresPostBack (this);
@@ -942,15 +1049,6 @@ namespace System.Web.UI.WebControls
 					string url = Page.ClientScript.GetWebResourceUrl (typeof(TreeView), "TreeView.js");
 					Page.ClientScript.RegisterClientScriptInclude (typeof(TreeView), "TreeView.js", url);
 				}
-
-				if (Page.Header != null) {
-					cssStylesRegistered = true;
-					ControlStyle.Font.Underline = false;
-					Page.Header.StyleSheet.RegisterStyle(ControlStyle, this);
-					if (levelStyles != null)
-						for (int i = 0; i < levelStyles.Count; i++)
-							Page.Header.StyleSheet.RegisterStyle(levelStyles[i], this);
-				}				
 			}
 			
 			string ctree = ClientID + "_data";
@@ -973,7 +1071,10 @@ namespace System.Web.UI.WebControls
 				script += string.Format ("{0}.populateFromClient = {1};\n", ctree, ClientScriptManager.GetScriptLiteral (PopulateNodesFromClient));
 				script += string.Format ("{0}.expandAlt = {1};\n", ctree, ClientScriptManager.GetScriptLiteral (GetNodeImageToolTip (true, null)));
 				script += string.Format ("{0}.collapseAlt = {1};\n", ctree, ClientScriptManager.GetScriptLiteral (GetNodeImageToolTip (false, null)));
-				Page.ClientScript.RegisterStartupScript (typeof(TreeView), this.UniqueID, script, true);
+
+				if (!Page.IsPostBack) {
+					SetNodesExpandedToDepthRecursive (Nodes);
+				}
 
 				if (EnableClientScript) {
 					Page.ClientScript.RegisterHiddenField (ClientID + "_ExpandStates", GetExpandStates ());
@@ -982,27 +1083,116 @@ namespace System.Web.UI.WebControls
 					Page.ClientScript.GetCallbackEventReference (this, "null", "", "null");
 					Page.ClientScript.GetPostBackClientHyperlink (this, "");
 				}
+
+				if (EnableClientScript && PopulateNodesFromClient) {
+					Page.ClientScript.RegisterHiddenField (ClientID + "_PopulatedStates", "|");
+				}
+
+				EnsureStylesPrepared ();
+
+				if (hoverNodeStyle != null) {
+					if (Page.Header == null)
+						throw new InvalidOperationException ("Using TreeView.HoverNodeStyle requires Page.Header to be non-null (e.g. <head runat=\"server\" />).");
+					RegisterStyle (HoverNodeStyle, HoverNodeLinkStyle);
+					script += string.Format ("{0}.hoverClass = {1};\n", ctree, ClientScriptManager.GetScriptLiteral (HoverNodeStyle.RegisteredCssClass));
+					script += string.Format ("{0}.hoverLinkClass = {1};\n", ctree, ClientScriptManager.GetScriptLiteral (HoverNodeLinkStyle.RegisteredCssClass));
+				}
+				
+				Page.ClientScript.RegisterStartupScript (typeof(TreeView), this.UniqueID, script, true);
 			}
 		}
+
+		void EnsureStylesPrepared () {
+			if (stylesPrepared)
+				return;
+			stylesPrepared = true;
+			PrepareStyles ();
+		}
+
+		private void PrepareStyles () {
+			// The order in which styles are defined matters when more than one class
+			// is assigned to an element
+			ControlLinkStyle.CopyTextStylesFrom (ControlStyle);
+			RegisterStyle (ControlLinkStyle);
+
+			if (nodeStyle != null)
+				RegisterStyle (NodeStyle, NodeLinkStyle);
+
+			if (rootNodeStyle != null)
+				RegisterStyle (RootNodeStyle, RootNodeLinkStyle);
+
+			if (parentNodeStyle != null)
+				RegisterStyle (ParentNodeStyle, ParentNodeLinkStyle);
+
+			if (leafNodeStyle != null)
+				RegisterStyle (LeafNodeStyle, LeafNodeLinkStyle);
+
+
+			if (levelStyles != null && levelStyles.Count > 0) {
+				levelLinkStyles = new List<Style> (levelStyles.Count);
+				foreach (Style style in levelStyles) {
+					Style linkStyle = new Style ();
+					levelLinkStyles.Add (linkStyle);
+					RegisterStyle (style, linkStyle);
+				}
+			}
+
+			if (selectedNodeStyle != null)
+				RegisterStyle (SelectedNodeStyle, SelectedNodeLinkStyle);
+		}
+
+		void SetNodesExpandedToDepthRecursive (TreeNodeCollection nodes) {
+			foreach (TreeNode node in nodes) {
+				if (!node.Expanded.HasValue) {
+					if (ExpandDepth < 0 || node.Depth < ExpandDepth)
+						node.Expanded = true;
+				}
+				SetNodesExpandedToDepthRecursive (node.ChildNodes);
+			}
+		}
+
+		string IncrementStyleClassName () {
+			registeredStylesCounter++;
+			return ClientID + "_" + registeredStylesCounter;
+		}
+
+		void RegisterStyle (Style baseStyle, Style linkStyle) {
+			linkStyle.CopyTextStylesFrom (baseStyle);
+			linkStyle.BorderStyle = BorderStyle.None;
+			baseStyle.Font.Reset ();
+			RegisterStyle (linkStyle);
+			RegisterStyle (baseStyle);
+		}
+
+		void RegisterStyle (Style baseStyle) {
+			if (Page.Header == null)
+				return;
+			string className = IncrementStyleClassName ();
+			baseStyle.SetRegisteredCssClass (className);
+			Page.Header.StyleSheet.CreateStyleRule (baseStyle, this, "." + className);
+		}
 		
 		string GetBindingKey (string dataMember, int depth)
 		{
 			return dataMember + " " + depth;
 		}
 		
+		void InitializeDataBindings () {
+			if (dataBindings != null && dataBindings.Count > 0) {
+				bindings = new Hashtable ();
+				foreach (TreeNodeBinding bin in dataBindings) {
+					string key = GetBindingKey (bin.DataMember, bin.Depth);
+					bindings [key] = bin;
+				}
+			}
+			else
+				bindings = null;
+		}
+		
 		internal TreeNodeBinding FindBindingForNode (string type, int depth)
 		{
-			if (bindings == null) {
-				if (dataBindings != null && dataBindings.Count > 0) {
-					bindings = new Hashtable ();
-					foreach (TreeNodeBinding bind in dataBindings) {
-						string key = GetBindingKey (bind.DataMember, bind.Depth);
-						bindings [key] = bind;
-					}
-				}
-				else
-					return null;
-			}
+			if (bindings == null)
+				return null;
 				
 			TreeNodeBinding bin = (TreeNodeBinding) bindings [GetBindingKey (type, depth)];
 			if (bin != null) return bin;
@@ -1051,12 +1241,32 @@ namespace System.Web.UI.WebControls
 		
 		public override void RenderBeginTag (HtmlTextWriter writer)
 		{
+			if (SkipLinkText != "") {
+				writer.AddAttribute (HtmlTextWriterAttribute.Href, "#" + ClientID + "_SkipLink");
+				writer.RenderBeginTag (HtmlTextWriterTag.A);
+
+				Image img = new Image ();
+				ClientScriptManager csm = new ClientScriptManager (null);
+				img.ImageUrl = csm.GetWebResourceUrl (typeof (SiteMapPath), "transparent.gif");
+				img.Attributes.Add ("height", "0");
+				img.Attributes.Add ("width", "0");
+				img.AlternateText = SkipLinkText;
+				img.Render (writer);
+
+				writer.RenderEndTag ();
+			}
 			base.RenderBeginTag (writer);
 		}
 		
 		public override void RenderEndTag (HtmlTextWriter writer)
 		{
 			base.RenderEndTag (writer);
+
+			if (SkipLinkText != "") {
+				writer.AddAttribute (HtmlTextWriterAttribute.Id, ClientID + "_SkipLink");
+				writer.RenderBeginTag (HtmlTextWriterTag.A);
+				writer.RenderEndTag ();
+			}
 		}
 		
  		void RenderNode (HtmlTextWriter writer, TreeNode node, int level, ArrayList levelLines, bool hasPrevious, bool hasNext)
@@ -1069,7 +1279,7 @@ namespace System.Web.UI.WebControls
 			bool renderChildNodes = node.Expanded.HasValue && node.Expanded.Value;
 			
 			if (clientExpand && !renderChildNodes)
-				renderChildNodes = (!PopulateNodesFromClient || HasChildInputData (node));
+				renderChildNodes = (!node.PopulateOnDemand || node.Populated);
 				
 			bool hasChildNodes;
 			
@@ -1082,28 +1292,28 @@ namespace System.Web.UI.WebControls
 			writer.AddAttribute ("cellspacing", "0");
 			writer.AddStyleAttribute ("border-width", "0");
 			writer.RenderBeginTag (HtmlTextWriterTag.Table);
+
+			Unit nodeSpacing = GetNodeSpacing (node);
+
+			if (nodeSpacing != Unit.Empty && (node.Depth > 0 || node.Index > 0))
+				RenderMenuItemSpacing (writer, nodeSpacing);
+			
 			writer.RenderBeginTag (HtmlTextWriterTag.Tr);
 			
 			// Vertical lines from previous levels
-			
+
+			nodeImage = GetNodeImageUrl ("i", imageStyle);
 			for (int n=0; n<level; n++) {
 				writer.RenderBeginTag (HtmlTextWriterTag.Td);
-				if (ShowLines) {
-					if (levelLines [n] == null)
-						nodeImage = GetNodeImageUrl ("noexpand", imageStyle);
-					else
-						nodeImage = GetNodeImageUrl ("i", imageStyle);
-
+				writer.AddStyleAttribute ("width", NodeIndent + "px");
+				writer.AddStyleAttribute ("height", "1px");
+				writer.RenderBeginTag (HtmlTextWriterTag.Div);
+				if (ShowLines && levelLines [n] != null) {
 					writer.AddAttribute ("src", nodeImage);
-					writer.AddAttribute ("border", "0");
+					writer.AddAttribute (HtmlTextWriterAttribute.Alt, "");
 					writer.RenderBeginTag (HtmlTextWriterTag.Img);
 					writer.RenderEndTag ();
-					
-					writer.RenderEndTag ();	// TD
-					writer.RenderBeginTag (HtmlTextWriterTag.Td);
 				}
-				writer.AddStyleAttribute ("width", NodeIndent + "px");
-				writer.RenderBeginTag (HtmlTextWriterTag.Div);
 				writer.RenderEndTag ();
 				writer.RenderEndTag ();	// TD
 			}
@@ -1112,16 +1322,15 @@ namespace System.Web.UI.WebControls
 			
 			if (ShowExpandCollapse || ShowLines) {
 				bool buttonImage = false;
-				string tooltip = null;
-				string shape;
+				string tooltip = "";
+				string shape = "";
 				
 				if (ShowLines) {
 					if (hasPrevious && hasNext) shape = "t";
 					else if (hasPrevious && !hasNext) shape = "l";
 					else if (!hasPrevious && hasNext) shape = "r";
 					else shape = "dash";
-				} else
-					shape = "";
+				}
 				
 				if (ShowExpandCollapse) {
 					if (hasChildNodes) {
@@ -1138,19 +1347,20 @@ namespace System.Web.UI.WebControls
 					writer.RenderBeginTag (HtmlTextWriterTag.Td);	// TD
 					
 					if (buttonImage) {
-						if (!clientExpand)
+						if (!clientExpand || (!PopulateNodesFromClient && node.PopulateOnDemand && !node.Populated))
 							writer.AddAttribute ("href", GetClientEvent (node, "ec"));
 						else
 							writer.AddAttribute ("href", GetClientExpandEvent(node));
 						writer.RenderBeginTag (HtmlTextWriterTag.A);	// Anchor
 					}
-					
-					if (tooltip != null)
-						writer.AddAttribute ("alt", tooltip);
+
+					writer.AddAttribute ("alt", tooltip);
+
 					if (buttonImage && clientExpand)
 						writer.AddAttribute ("id", GetNodeClientId (node, "img"));
 					writer.AddAttribute ("src", nodeImage);
-					writer.AddAttribute ("border", "0");
+					if (buttonImage)
+						writer.AddStyleAttribute (HtmlTextWriterStyle.BorderWidth, "0");
 					writer.RenderBeginTag (HtmlTextWriterTag.Img);
 					writer.RenderEndTag ();
 					
@@ -1177,13 +1387,22 @@ namespace System.Web.UI.WebControls
 				writer.RenderBeginTag (HtmlTextWriterTag.Td);	// TD
 				BeginNodeTag (writer, node, clientExpand);
 				writer.AddAttribute ("src", imageUrl);
-				writer.AddAttribute ("border", "0");
-				if (node.ImageToolTip != "") writer.AddAttribute ("alt", node.ImageToolTip);
+				writer.AddStyleAttribute (HtmlTextWriterStyle.BorderWidth, "0");
+				writer.AddAttribute ("alt", node.ImageToolTip);
 				writer.RenderBeginTag (HtmlTextWriterTag.Img);
 				writer.RenderEndTag ();	// IMG
 				writer.RenderEndTag ();	// style tag
 				writer.RenderEndTag ();	// TD
 			}
+
+			if (!NodeWrap)
+				writer.AddStyleAttribute ("white-space", "nowrap");
+			AddNodeStyle (writer, node, level);
+			if (EnableClientScript) {
+				writer.AddAttribute ("onmouseout", "TreeView_UnhoverNode(this)");
+				writer.AddAttribute ("onmouseover", "TreeView_HoverNode('" + ClientID + "', this)");
+			}
+			writer.RenderBeginTag (HtmlTextWriterTag.Td);	// TD
 			
 			// Checkbox
 			
@@ -1192,32 +1411,26 @@ namespace System.Web.UI.WebControls
 				showChecks = node.ShowCheckBox.Value;
 			else
 				showChecks = (ShowCheckBoxes == TreeNodeTypes.All) ||
-							 (ShowCheckBoxes == TreeNodeTypes.Leaf && node.ChildNodes.Count == 0) ||
-							 (ShowCheckBoxes == TreeNodeTypes.Parent && node.ChildNodes.Count > 0 && node.Parent != null) ||
-							 (ShowCheckBoxes == TreeNodeTypes.Root && node.Parent == null && node.ChildNodes.Count > 0);
+							 ((ShowCheckBoxes & TreeNodeTypes.Leaf) > 0 && node.IsLeafNode) ||
+							 ((ShowCheckBoxes & TreeNodeTypes.Parent) > 0 && node.IsParentNode && node.Parent != null) ||
+							 ((ShowCheckBoxes & TreeNodeTypes.Root) > 0 && node.Parent == null && node.ChildNodes.Count > 0);
 
 			if (showChecks) {
-				AddNodeStyle (writer, node, level);
-				writer.RenderBeginTag (HtmlTextWriterTag.Td);	// TD
 				writer.AddAttribute ("name", ClientID + "_cs_" + node.Path);
 				writer.AddAttribute ("type", "checkbox");
+				writer.AddAttribute ("title", node.Text);
 				if (node.Checked) writer.AddAttribute ("checked", "checked");
 				writer.RenderBeginTag (HtmlTextWriterTag.Input);	// INPUT
 				writer.RenderEndTag ();	// INPUT
-				writer.RenderEndTag ();	// TD
 			}
 			
 			// Text
 			
-			if (!NodeWrap)
-				writer.AddAttribute ("nowrap", "nowrap");
-			writer.RenderBeginTag (HtmlTextWriterTag.Td);	// TD
-			
 			node.BeginRenderText (writer);
 			
-			AddNodeStyle (writer, node, level);
 			if (clientExpand)
 				writer.AddAttribute ("id", GetNodeClientId (node, "txt"));
+			AddNodeLinkStyle (writer, node, level);
 			BeginNodeTag (writer, node, clientExpand);
 			writer.Write (node.Text);
 			writer.RenderEndTag ();	// style tag
@@ -1227,6 +1440,10 @@ namespace System.Web.UI.WebControls
 			writer.RenderEndTag ();	// TD
 			
 			writer.RenderEndTag ();	// TR
+
+			if (nodeSpacing != Unit.Empty)
+				RenderMenuItemSpacing (writer, nodeSpacing);
+			
 			writer.RenderEndTag ();	// TABLE
 			
 			// Children
@@ -1261,44 +1478,189 @@ namespace System.Web.UI.WebControls
 				}
 			}
 		}
+
+		private void RenderMenuItemSpacing (HtmlTextWriter writer, Unit itemSpacing) {
+			writer.AddStyleAttribute ("height", itemSpacing.ToString ());
+			writer.RenderBeginTag (HtmlTextWriterTag.Tr);
+			writer.RenderBeginTag (HtmlTextWriterTag.Td);
+			writer.RenderEndTag ();
+			writer.RenderEndTag ();
+		}
+
+		private Unit GetNodeSpacing (TreeNode node) {
+			if (node.Selected && selectedNodeStyle != null && selectedNodeStyle.NodeSpacing != Unit.Empty) {
+				return selectedNodeStyle.NodeSpacing;
+			}
+
+			if (levelStyles != null && node.Depth < levelStyles.Count && levelStyles [node.Depth].NodeSpacing != Unit.Empty) {
+				return levelStyles [node.Depth].NodeSpacing;
+			}
+
+			if (node.IsLeafNode) {
+				if (leafNodeStyle != null && leafNodeStyle.NodeSpacing != Unit.Empty)
+					return leafNodeStyle.NodeSpacing;
+			}
+			else if (node.IsRootNode) {
+				if (rootNodeStyle != null && rootNodeStyle.NodeSpacing != Unit.Empty)
+					return rootNodeStyle.NodeSpacing;
+			}
+			else if (node.IsParentNode) {
+				if (parentNodeStyle != null && parentNodeStyle.NodeSpacing != Unit.Empty)
+					return parentNodeStyle.NodeSpacing;
+			}
+
+			if (nodeStyle != null)
+				return nodeStyle.NodeSpacing;
+			else
+				return Unit.Empty;
+		}
 		
 		void AddNodeStyle (HtmlTextWriter writer, TreeNode node, int level)
 		{
-			if (nodeStyle != null)
-				nodeStyle.AddAttributesToRender (writer);
-
-			if (levelStyles != null && level < levelStyles.Count) {
-				if (cssStylesRegistered)
-					writer.AddAttribute("class", String.Format("{0} {1}", ControlStyle.RegisteredCssClass, levelStyles[level].RegisteredCssClass));
-				else
-					levelStyles [level].AddAttributesToRender (writer);
-			} else {
-				if (rootNodeStyle != null && node.IsRootNode)
-					rootNodeStyle.AddAttributesToRender (writer);
-	
-				if (leafNodeStyle != null && node.IsLeafNode)
-					leafNodeStyle.AddAttributesToRender (writer);
-	
-				if (parentNodeStyle != null && node.IsParentNode)
-					parentNodeStyle.AddAttributesToRender (writer);
+			TreeNodeStyle style = new TreeNodeStyle ();
+			if (Page.Header != null) {
+				// styles are registered
+				if (nodeStyle != null) {
+					style.AddCssClass (nodeStyle.CssClass);
+					style.AddCssClass (nodeStyle.RegisteredCssClass);
+				}
+				if (node.IsLeafNode) {
+					if (leafNodeStyle != null) {
+						style.AddCssClass (leafNodeStyle.CssClass);
+						style.AddCssClass (leafNodeStyle.RegisteredCssClass);
+					}
+				}
+				else if (node.IsRootNode) {
+					if (rootNodeStyle != null) {
+						style.AddCssClass (rootNodeStyle.CssClass);
+						style.AddCssClass (rootNodeStyle.RegisteredCssClass);
+					}
+				}
+				else if (node.IsParentNode) {
+					if (parentNodeStyle != null) {
+						style.AddCssClass (parentNodeStyle.CssClass);
+						style.AddCssClass (parentNodeStyle.RegisteredCssClass);
+					}
+				}
+				if (levelStyles != null && levelStyles.Count > level) {
+					style.AddCssClass (levelStyles [level].CssClass);
+					style.AddCssClass (levelStyles [level].RegisteredCssClass);
+				}
+				if (node == SelectedNode && selectedNodeStyle != null) {
+					style.AddCssClass (selectedNodeStyle.CssClass);
+					style.AddCssClass (selectedNodeStyle.RegisteredCssClass);
+				}
 			}
-			
-			if (node.Selected && selectedNodeStyle != null)
-				selectedNodeStyle.AddAttributesToRender (writer);
+			else {
+				// styles are not registered
+				if (nodeStyle != null) {
+					style.CopyFrom (nodeStyle);
+				}
+				if (node.IsLeafNode) {
+					if (leafNodeStyle != null) {
+						style.CopyFrom (leafNodeStyle);
+					}
+				}
+				else if (node.IsRootNode) {
+					if (rootNodeStyle != null) {
+						style.CopyFrom (rootNodeStyle);
+					}
+				}
+				else if (node.IsParentNode) {
+					if (parentNodeStyle != null) {
+						style.CopyFrom (parentNodeStyle);
+					}
+				}
+				if (levelStyles != null && levelStyles.Count > level) {
+					style.CopyFrom (levelStyles [level]);
+				}
+				if (node == SelectedNode && selectedNodeStyle != null) {
+					style.CopyFrom (selectedNodeStyle);
+				}
+			}
+			style.AddAttributesToRender (writer);
 		}
-		
+
+		void AddNodeLinkStyle (HtmlTextWriter writer, TreeNode node, int level) {
+			Style style = new Style ();
+			if (Page.Header != null) {
+				// styles are registered
+				style.AddCssClass (ControlLinkStyle.RegisteredCssClass);
+
+				if (nodeStyle != null) {
+					style.AddCssClass (nodeLinkStyle.CssClass);
+					style.AddCssClass (nodeLinkStyle.RegisteredCssClass);
+				}
+				if (node.IsLeafNode) {
+					if (leafNodeStyle != null) {
+						style.AddCssClass (leafNodeLinkStyle.CssClass);
+						style.AddCssClass (leafNodeLinkStyle.RegisteredCssClass);
+					}
+				}
+				else if (node.IsRootNode) {
+					if (rootNodeStyle != null) {
+						style.AddCssClass (rootNodeLinkStyle.CssClass);
+						style.AddCssClass (rootNodeLinkStyle.RegisteredCssClass);
+					}
+				}
+				else if (node.IsParentNode) {
+					if (parentNodeStyle != null) {
+						style.AddCssClass (parentNodeLinkStyle.CssClass);
+						style.AddCssClass (parentNodeLinkStyle.RegisteredCssClass);
+					}
+				}
+				if (levelStyles != null && levelStyles.Count > level) {
+					style.AddCssClass (levelLinkStyles [level].CssClass);
+					style.AddCssClass (levelLinkStyles [level].RegisteredCssClass);
+				}
+				if (node == SelectedNode && selectedNodeStyle != null) {
+					style.AddCssClass (selectedNodeLinkStyle.CssClass);
+					style.AddCssClass (selectedNodeLinkStyle.RegisteredCssClass);
+				}
+			}
+			else {
+				// styles are not registered
+				style.CopyFrom (ControlLinkStyle);
+				if (nodeStyle != null) {
+					style.CopyFrom (nodeLinkStyle);
+				}
+				if (node.IsLeafNode) {
+					if (node.IsLeafNode && leafNodeStyle != null) {
+						style.CopyFrom (leafNodeLinkStyle);
+					}
+				}
+				else if (node.IsRootNode) {
+					if (node.IsRootNode && rootNodeStyle != null) {
+						style.CopyFrom (rootNodeLinkStyle);
+					}
+				}
+				else if (node.IsParentNode) {
+					if (node.IsParentNode && parentNodeStyle != null) {
+						style.CopyFrom (parentNodeLinkStyle);
+					}
+				}
+				if (levelStyles != null && levelStyles.Count > level) {
+					style.CopyFrom (levelLinkStyles [level]);
+				}
+				if (node == SelectedNode && selectedNodeStyle != null) {
+					style.CopyFrom (selectedNodeLinkStyle);
+				}
+				style.AlwaysRenderTextDecoration = true;
+			}
+			style.AddAttributesToRender (writer);
+		}
+
 		void BeginNodeTag (HtmlTextWriter writer, TreeNode node, bool clientExpand)
 		{
+			if(node.ToolTip.Length>0)
+				writer.AddAttribute ("title", node.ToolTip);
+
 			if (node.NavigateUrl != "") {
 				writer.AddAttribute ("href", node.NavigateUrl);
-				if (node.Target != null)
+				if (node.Target.Length > 0)
 					writer.AddAttribute ("target", node.Target);
-				if (cssStylesRegistered)
-					writer.AddStyleAttribute ("white-space", "nowrap");
-				else {
-					writer.AddStyleAttribute ("text-decoration", "none");
-					writer.AddStyleAttribute ("white-space", "nowrap");
-				}
+				else if (Target.Length > 0)
+					writer.AddAttribute ("target", Target);
 				writer.RenderBeginTag (HtmlTextWriterTag.A);
 			}
 			else if (node.SelectAction != TreeNodeSelectAction.None) {
@@ -1306,42 +1668,23 @@ namespace System.Web.UI.WebControls
 					writer.AddAttribute ("href", GetClientExpandEvent (node));
 				else
 					writer.AddAttribute ("href", GetClientEvent (node, "sel"));
-				if (cssStylesRegistered)
-					writer.AddStyleAttribute ("white-space", "nowrap");
-				else {
-					writer.AddStyleAttribute ("text-decoration", "none");
-					writer.AddStyleAttribute ("white-space", "nowrap");
-				}
 				writer.RenderBeginTag (HtmlTextWriterTag.A);
 			}
 			else
 				writer.RenderBeginTag (HtmlTextWriterTag.Span);
 		}
 		
-		bool HasChildInputData (TreeNode node)
-		{
-			// Returns true if this node contain childs whose state is hold in
-			// input elements that are rendered together with the node.
-			
-			if (node.Checked) return true;
-			if (!node.HasChildData) return false;
-
-			foreach (TreeNode n in node.ChildNodes)
-				if (HasChildInputData (n)) return true;
-			return false;
-		}
-		
 		string GetNodeImageToolTip (bool expand, string txt) {
 			if (expand)  {
 				if (ExpandImageToolTip != "")
-					return ExpandImageToolTip;
+					return String.Format (ExpandImageToolTip, txt);
 				else if (txt != null)
 					return "Expand " + txt;
 				else
 					return "Expand {0}";
 			} else {
 				if (CollapseImageToolTip != "")
-					return CollapseImageToolTip;
+					return String.Format (CollapseImageToolTip, txt);
 				else if (txt != null)
 					return "Collapse " + txt;
 				else
@@ -1457,7 +1800,8 @@ namespace System.Web.UI.WebControls
 			foreach (string id in states) {
 				if (id == null || id == "") continue;
 				TreeNode node = FindNodeByPos (id);
-				node.Expanded = true;
+				if (node != null)
+					node.Expanded = true;
 			}
 		}
 		
