@@ -86,7 +86,9 @@ namespace System.Web.UI.WebControls
 		
 		// Control state
 		
-		int activeStepIndex;
+		int activeStepIndex = -1;
+		bool inited = false;
+		bool dontRender = false;
 		ArrayList history;
 
 		Table wizardTable;
@@ -200,21 +202,28 @@ namespace System.Web.UI.WebControls
 	    [ThemeableAttribute (false)]
 		public virtual int ActiveStepIndex {
 			get {
-				if (WizardSteps.Count == 0)
-					return -1;
-
 				return activeStepIndex;
 			}
 			set {
-				if (!AllowNavigationToStep (value))
+				if (value < -1 || (value > WizardSteps.Count && (inited || WizardSteps.Count > 0)))
+					throw new ArgumentOutOfRangeException ("The ActiveStepIndex must be less than WizardSteps.Count and at least -1");
+
+				if (inited && value == -1) {
+					dontRender = true;
+					activeStepIndex = -1;
 					return;
-				if (activeStepIndex != value) {
-					if (history == null) history = new ArrayList ();
-					history.Insert (0, activeStepIndex);
 				}
+
+				if (inited && !AllowNavigationToStep (value))
+					return;
+
 				activeStepIndex = value;
+				dontRender = false;
+
 				UpdateControls ();
-				OnActiveStepChanged (this, EventArgs.Empty);
+				if (inited) {
+					OnActiveStepChanged (this, EventArgs.Empty);
+				}
 			}
 		}
 		
@@ -569,7 +578,6 @@ namespace System.Web.UI.WebControls
 		}
 
 		[Localizable (true)]
-		[MonoTODO]
 		public virtual string SkipLinkText 
 		{
 			get
@@ -787,7 +795,6 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
-		[MonoTODO]
 		protected virtual new HtmlTextWriterTag TagKey
 		{
 			get {
@@ -819,7 +826,10 @@ namespace System.Web.UI.WebControls
 		public WizardStepType GetStepType (WizardStepBase wizardStep, int index)
 		{
 			if (wizardStep.StepType == WizardStepType.Auto) {
-				if (index == WizardSteps.Count - 1)
+				if ((index == WizardSteps.Count - 1) || 
+						(WizardSteps.Count > 2 && 
+						WizardSteps[WizardSteps.Count - 1].StepType == WizardStepType.Complete && 
+						index == WizardSteps.Count - 2))
 					return WizardStepType.Finish;
 				else if (index == 0)
 					return WizardStepType.Start;
@@ -840,6 +850,8 @@ namespace System.Web.UI.WebControls
 		
 		protected internal override void OnInit (EventArgs e)
 		{
+			inited = true;
+			
 			Page.RegisterRequiresControlState (this);
 			base.OnInit (e);
 		}
@@ -879,43 +891,48 @@ namespace System.Web.UI.WebControls
 					multiView.Views.Add (v);
 				}
 			}
-			
-			multiView.ActiveViewIndex = activeStepIndex;
-			
-			RegisterApplyStyle (viewCell, StepStyle);
-			viewCell.Controls.Add (multiView);
-			viewRow.Cells.Add (viewCell);
-			viewRow.Height = new Unit ("100%");
-			wizardTable.Rows.Add (viewRow);
-			
-			TableRow buttonRow = new TableRow ();
-			TableCell buttonCell = new TableCell ();
-			buttonCell.HorizontalAlign = HorizontalAlign.Right;
-			CreateButtonBar (buttonCell);
-			buttonRow.Cells.Add (buttonCell);
-			wizardTable.Rows.Add (buttonRow);
-			
-			if (DisplaySideBar && ActiveStep.StepType != WizardStepType.Complete) {
-				Table contentTable = wizardTable;
-				contentTable.Height = new Unit ("100%");
-				contentTable.Width = new Unit ("100%");
-				
-				wizardTable = new Table ();
-				wizardTable.CellPadding = CellPadding; 
-				wizardTable.CellSpacing = CellSpacing;
-				TableRow row = new TableRow ();
 
-				TableCellNamingContainer sideBarCell = new TableCellNamingContainer ();
-				sideBarCell.ControlStyle.Height = Unit.Percentage (100);
-				CreateSideBar (sideBarCell);
-				row.Cells.Add (sideBarCell);
-				
-				TableCell contentCell = new TableCell ();
-				contentCell.Controls.Add (contentTable);
-				contentCell.Height = new Unit ("100%");
-				row.Cells.Add (contentCell);
-				
-				wizardTable.Rows.Add (row);
+			if (ActiveStepIndex == -1 && !dontRender)
+				ActiveStepIndex = 0;
+
+			multiView.ActiveViewIndex = ActiveStepIndex;
+
+			if (ActiveStepIndex >= 0) {
+				RegisterApplyStyle (viewCell, StepStyle);
+				viewCell.Controls.Add (multiView);
+				viewRow.Cells.Add (viewCell);
+				viewRow.Height = new Unit ("100%");
+				wizardTable.Rows.Add (viewRow);
+
+				TableRow buttonRow = new TableRow ();
+				TableCell buttonCell = new TableCell ();
+				buttonCell.HorizontalAlign = HorizontalAlign.Right;
+				CreateButtonBar (buttonCell);
+				buttonRow.Cells.Add (buttonCell);
+				wizardTable.Rows.Add (buttonRow);
+
+				if (DisplaySideBar && ActiveStep.StepType != WizardStepType.Complete) {
+					Table contentTable = wizardTable;
+					contentTable.Height = new Unit ("100%");
+					contentTable.Width = new Unit ("100%");
+
+					wizardTable = new Table ();
+					wizardTable.CellPadding = CellPadding;
+					wizardTable.CellSpacing = CellSpacing;
+					TableRow row = new TableRow ();
+
+					TableCellNamingContainer sideBarCell = new TableCellNamingContainer ();
+					sideBarCell.ControlStyle.Height = Unit.Percentage (100);
+					CreateSideBar (sideBarCell);
+					row.Cells.Add (sideBarCell);
+
+					TableCell contentCell = new TableCell ();
+					contentCell.Controls.Add (contentTable);
+					contentCell.Height = new Unit ("100%");
+					row.Cells.Add (contentCell);
+
+					wizardTable.Rows.Add (row);
+				}
 			}
 			
 			Controls.SetReadonly (false);
@@ -996,16 +1013,50 @@ namespace System.Web.UI.WebControls
 			buttonBarCell.Controls.Add (t);
 		}
 		
-		Control CreateButton (string id, string command, ButtonType type, string text, string image, Style style)
+		internal Control CreateButton (string id, string command, ButtonType type, string text, string image, Style style)
 		{
-			DataControlButton b = new DataControlButton (this, text, image, command, "", false);
+			WebControl b;
+			switch (type) {
+			case ButtonType.Button:
+				b = CreateStandartButton ();
+				break;
+			case ButtonType.Image:
+				b = CreateImageButton (image);
+				break;
+			case ButtonType.Link:
+				b = CreateLinkButton ();
+				break;
+			default:
+				throw new ArgumentOutOfRangeException ("type");
+			}
+			
 			b.ID = id;
-			b.ButtonType = type;
+			b.EnableTheming = false;
+			((IButtonControl) b).CommandName = command;
+			((IButtonControl) b).Text = text;
+			((IButtonControl) b).ValidationGroup = ID;
+			
 			RegisterApplyStyle (b, NavigationButtonStyle);
 			RegisterApplyStyle (b, style);
 			return b;
 		}
-		
+
+		WebControl CreateStandartButton () {
+			Button btn = new Button ();
+			return btn;
+		}
+
+		WebControl CreateImageButton (string imageUrl) {
+			ImageButton img = new ImageButton ();
+			img.ImageUrl = imageUrl;
+			return img;
+		}
+
+		WebControl CreateLinkButton () {
+			LinkButton link = new LinkButton ();
+			return link;
+		}
+
 		void AddTemplateButtonBar (TableCell cell, ITemplate template, params string[] buttonIds)
 		{
 			template.InstantiateIn (cell);
@@ -1073,6 +1124,9 @@ namespace System.Web.UI.WebControls
 				IButtonControl button = (IButtonControl) e.Item.FindControl (SideBarButtonID);
 				WizardStep step = (WizardStep) e.Item.DataItem;
 
+				if (button is Button)
+					((Button) button).UseSubmitBehavior = false;
+
 				button.CommandName = Wizard.MoveToCommandName;
 				button.CommandArgument = WizardSteps.IndexOf (step).ToString ();
 				button.Text = step.Name;
@@ -1107,7 +1161,6 @@ namespace System.Web.UI.WebControls
 			return style;
 		}
 
-		[MonoTODO]
 		protected override IDictionary GetDesignModeState ()
 		{
 			throw new NotImplementedException ();
@@ -1124,6 +1177,9 @@ namespace System.Web.UI.WebControls
 		
 		protected internal override object SaveControlState ()
 		{
+			if (GetHistory ().Count == 0 || (int) history [0] != ActiveStepIndex)
+				history.Insert (0, ActiveStepIndex);
+
 			object bstate = base.SaveControlState ();
 			return new object[] {
 				bstate, activeStepIndex, history
@@ -1156,7 +1212,7 @@ namespace System.Web.UI.WebControls
 		
 		protected override object SaveViewState ()
 		{
-			object[] state = new object [13];
+			object [] state = new object [13];
 			state [0] = base.SaveViewState ();
 			
 			if (stepStyle != null) state [1] = ((IStateManager)stepStyle).SaveViewState ();
@@ -1244,11 +1300,6 @@ namespace System.Web.UI.WebControls
 					break;
 
 				case "MoveComplete":
-					if (FinishDestinationPageUrl.Length > 0) {
-						Context.Response.Redirect (FinishDestinationPageUrl);
-						return;
-					}
-				
 					int next = -1;
 					for (int n=0; n<WizardSteps.Count; n++) {
 						if (WizardSteps [n].StepType == WizardStepType.Complete) {
@@ -1256,12 +1307,21 @@ namespace System.Web.UI.WebControls
 							break;
 						}
 					}
-					if (next != -1) {
-						WizardNavigationEventArgs args = new WizardNavigationEventArgs (ActiveStepIndex, next);
-						OnFinishButtonClick (args);
-						if (!args.Cancel)
-							ActiveStepIndex = next;
+
+					if (next == -1 && ActiveStepIndex == WizardSteps.Count - 1)
+						next = ActiveStepIndex;
+
+					WizardNavigationEventArgs navArgs = new WizardNavigationEventArgs (ActiveStepIndex, next);
+					OnFinishButtonClick (navArgs);
+
+					if (FinishDestinationPageUrl.Length > 0) {
+						Context.Response.Redirect (FinishDestinationPageUrl);
+						return;
 					}
+
+					if (next != -1 && !navArgs.Cancel)
+						ActiveStepIndex = next;
+
 					break;
 					
 				case "MoveNext":
