@@ -174,8 +174,13 @@ namespace System.Web.UI.WebControls {
 					slidindExpiraion = new TimeSpan (CacheDuration * 10000);
 			}
 
-			DataCache.Add (GetDataKey (), xmlDocument,
-				new CacheDependency (new string [] { }, new string [] { CacheKeyDependency }),
+			CacheDependency dependency = null;
+			if (CacheKeyDependency.Length > 0)
+				dependency = new CacheDependency (new string [] { }, new string [] { CacheKeyDependency });
+			else
+				dependency = new CacheDependency (new string [] { }, new string [] { });
+
+			DataCache.Add (GetDataKey (), xmlDocument, dependency,
 				absoluteExpiration, slidindExpiraion, CacheItemPriority.Default, null);
 		}
 
@@ -235,7 +240,6 @@ namespace System.Web.UI.WebControls {
 		
 		[DefaultValue (0)]
 		//[TypeConverter (typeof(DataSourceCacheDurationConverter))]
-		[MonoTODO]
 		public virtual int CacheDuration {
 			get {
 				return _cacheDuration;
@@ -246,7 +250,6 @@ namespace System.Web.UI.WebControls {
 		}
 
 		[DefaultValue (DataSourceCacheExpiry.Absolute)]
-		[MonoTODO]
 		public virtual DataSourceCacheExpiry CacheExpirationPolicy {
 			get {
 				return _cacheExpirationPolicy;
@@ -257,7 +260,6 @@ namespace System.Web.UI.WebControls {
 		}
 
 		[DefaultValue ("")]
-		[MonoTODO]
 		public virtual string CacheKeyDependency {
 			get {
 				return _cacheKeyDependency;
@@ -268,7 +270,6 @@ namespace System.Web.UI.WebControls {
 		}
 
 		[DefaultValue (true)]
-		[MonoTODO]
 		public virtual bool EnableCaching {
 			get {
 				return _enableCaching;
