@@ -106,6 +106,9 @@ namespace System.Web.UI
 		Hashtable pendingVS; // may hold unused viewstate data from child controls
 		
 
+#if NET_2_0
+			bool _isChildControlStateCleared;
+#endif
 		/*************/
 		int stateMask;
 		const int ENABLE_VIEWSTATE 	= 1;
@@ -144,7 +147,7 @@ namespace System.Web.UI
                 }
 
 #if NET_2_0
-		[MonoTODO]
+		[MonoTODO("Not implemented, always returns null")]
 		protected ControlAdapter Adapter 
 		{
 			get {
@@ -192,20 +195,23 @@ namespace System.Web.UI
 				string client = UniqueID;
 
 				if (client != null)
-					client = client.Replace (':', '_');
-
+					client = client.Replace (':', ClientIDSeparator);
+				
 				return client;
 			}
 		}
 
 #if NET_2_0
-		protected char ClientIDSeparator 
+		protected char ClientIDSeparator
+#else
+		char ClientIDSeparator
+#endif		
 		{
 			get {
-				throw new NotImplementedException ();
+				return '_';
 			}
 		}
-#endif		
+
 
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 		[Browsable (false)]
@@ -262,15 +268,12 @@ namespace System.Web.UI
 		protected char IdSeparator 
 		{
 			get {
-				throw new NotImplementedException ();
+				return '$';
 			}
 		}
 
-		protected internal bool IsChildControlStateCleared 
-		{
-			get {
-				throw new NotImplementedException ();
-			}
+		protected internal bool IsChildControlStateCleared {
+			get { return _isChildControlStateCleared; }
 		}
 
 		protected internal bool IsViewStateEnabled 
@@ -290,7 +293,7 @@ namespace System.Web.UI
 		protected bool LoadViewStateByID 
 		{
 			get {
-				throw new NotImplementedException ();
+				return false;
 			}
 		}
 #endif		
@@ -595,7 +598,6 @@ namespace System.Web.UI
                 }
 
 #if NET_2_0
-		[MonoTODO]
 		[EditorBrowsable (EditorBrowsableState.Advanced)]
 		public virtual void ApplyStyleSheetSkin (Page page)
 		{
@@ -619,12 +621,13 @@ namespace System.Web.UI
 #if NET_2_0
 		protected void ClearChildControlState ()
 		{
-			throw new NotImplementedException ();
+			_isChildControlStateCleared = true;
 		}
 
 		protected void ClearChildState ()
 		{
-			throw new NotImplementedException ();
+			ClearChildViewState ();
+			ClearChildControlState ();
 		}
 #endif		
 
@@ -663,7 +666,9 @@ namespace System.Web.UI
 #if NET_2_0
 		protected void EnsureID ()
 		{
-			throw new NotImplementedException ();
+			if (Page == null)
+				return;
+			ID = NamingContainer.GetDefaultName ();
 		}
 
 		protected bool HasEvents ()
@@ -890,7 +895,13 @@ namespace System.Web.UI
 #if NET_2_0
 		protected internal Stream OpenFile (string path)
 		{
-			throw new NotImplementedException ();
+			try {
+				string filePath = Context.Server.MapPath (path);
+				return File.OpenRead (filePath);
+			}
+			catch (UnauthorizedAccessException) {
+				throw new HttpException ("Access to the specified file was denied.");
+			}
 		}
 #endif		
 
@@ -1537,10 +1548,9 @@ namespace System.Web.UI
 			}
 		}
 
-		[MonoTODO]
 		public virtual void Focus()
 		{
-			throw new NotImplementedException();
+			Page.SetFocus (this);
 		}
 		
 		protected internal virtual void LoadControlState (object state)
