@@ -70,7 +70,13 @@ namespace System.Web.UI
 #endif
 public class Page : TemplateControl, IHttpHandler
 {
+#if NET_2_0
+	private PageLifeCycle _lifeCycle = PageLifeCycle.Unknown;
 	private bool _eventValidation = true;
+	private object [] _savedControlState;
+	private bool _doLoadPreviousPage;
+	string _focusedControlID;
+#endif
 	private bool _viewState = true;
 	private bool _viewStateMac;
 	private string _errorPage;
@@ -128,6 +134,8 @@ public class Page : TemplateControl, IHttpHandler
 	
 	Page previousPage;
 	bool isCrossPagePostBack;
+	bool isPostBack;
+	bool isCallback;
 	ArrayList requireStateControls;
 	Hashtable _validatorsByGroup;
 	HtmlForm _form;
@@ -264,7 +272,7 @@ public class Page : TemplateControl, IHttpHandler
 	public string Culture
 	{
 		get { return Thread.CurrentThread.CurrentCulture.Name; }
-		set { Thread.CurrentThread.CurrentCulture = new CultureInfo (value); }
+		set { Thread.CurrentThread.CurrentCulture = GetPageCulture (value, Thread.CurrentThread.CurrentCulture); }
 	}
 #else
 	[EditorBrowsable (EditorBrowsableState.Never)]
@@ -277,7 +285,15 @@ public class Page : TemplateControl, IHttpHandler
 #if NET_2_0
 	public virtual bool EnableEventValidation {
 		get { return _eventValidation; }
-		set { _eventValidation = value;}
+		set {
+			if (_lifeCycle > PageLifeCycle.Init)
+				throw new InvalidOperationException ("The 'EnableEventValidation' property can be set only in the Page_init, the Page directive or in the <pages> configuration section.");
+			_eventValidation = value;
+		}
+	}
+
+	internal PageLifeCycle LifeCycle {
+		get { return _lifeCycle; }
 	}
 #endif
 
@@ -344,7 +360,11 @@ public class Page : TemplateControl, IHttpHandler
 	public bool IsPostBack
 	{
 		get {
+#if NET_2_0
+			return isPostBack;
+#else
 			return _requestValueCollection != null;
+#endif
 		}
 	}
 
@@ -360,7 +380,14 @@ public class Page : TemplateControl, IHttpHandler
 			if (!is_validated)
 				throw new HttpException (Locale.GetText ("Page hasn't been validated."));
 
+#if NET_2_0
+			foreach (IValidator val in Validators)
+				if (!val.IsValid)
+					return false;
+			return true;
+#else
 			return ValidateCollection (_validators);
+#endif
 		}
 	}
 #if NET_2_0
@@ -530,8 +557,17 @@ public class Page : TemplateControl, IHttpHandler
 	[Bindable (true)] 
 	[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 	public string Title {
-		get { return _title; }
-		set { _title = value; }
+		get {
+			if (_title == null)
+				return htmlHeader.Title;
+			return _title;
+		}
+		set {
+			if (htmlHeader != null)
+				htmlHeader.Title = value;
+			else
+				_title = value;
+		}
 	}
 #endif
 
@@ -590,7 +626,7 @@ public class Page : TemplateControl, IHttpHandler
 	public string UICulture
 	{
 		get { return Thread.CurrentThread.CurrentUICulture.Name; }
-		set { Thread.CurrentThread.CurrentUICulture = new CultureInfo (value); }
+		set { Thread.CurrentThread.CurrentUICulture = GetPageCulture (value, Thread.CurrentThread.CurrentUICulture); }
 	}
 #else
 	[EditorBrowsable (EditorBrowsableState.Never)]
@@ -636,6 +672,29 @@ public class Page : TemplateControl, IHttpHandler
 
 	#region Methods
 
+#if NET_2_0
+	CultureInfo GetPageCulture (string culture, CultureInfo deflt)
+	{
+		if (culture == null)
+			return deflt;
+		CultureInfo ret = null;
+		if (culture.StartsWith ("auto")) {
+			string[] languages = Request.UserLanguages;
+			try {
+				if (languages != null && languages.Length > 0)
+					ret = new CultureInfo (languages[0]);
+			} catch {
+			}
+			
+			if (ret == null)
+				ret = deflt;
+		} else
+			ret = new CultureInfo (culture);
+
+		return ret;
+	}
+#endif
+
 	[EditorBrowsable (EditorBrowsableState.Never)]
 	protected IAsyncResult AspCompatBeginProcessRequest (HttpContext context,
 							     AsyncCallback cb, 
@@ -748,6 +807,19 @@ public class Page : TemplateControl, IHttpHandler
 		return 0;
 	}
 
+#if NET_2_0
+    [MonoTODO("The following properties of OutputCacheParameters are silently ignored: CacheProfile, NoStore, SqlDependency")]
+    protected internal virtual void InitOutputCache(OutputCacheParameters cacheSettings)
+    {
+        if (cacheSettings.Enabled)
+            InitOutputCache(cacheSettings.Duration,
+                cacheSettings.VaryByHeader,
+                cacheSettings.VaryByCustom,
+                cacheSettings.Location,
+                cacheSettings.VaryByParam);
+    }
+#endif
+
 	[EditorBrowsable (EditorBrowsableState.Never)]
 	protected virtual void InitOutputCache (int duration,
 						string varyByHeader,
@@ -872,11 +944,8 @@ public class Page : TemplateControl, IHttpHandler
 		writer.WriteLine ("<script language=\"javascript\">");
 		writer.WriteLine ("<!--");
 
-		if (Request.Browser.Browser == ("Netscape") && Request.Browser.MajorVersion == 4)
-			writer.WriteLine ("\tvar theForm = document.{0};", formUniqueID);
-		else
-			writer.WriteLine ("\tvar theForm = document.getElementById ('{0}');", formUniqueID);
-
+		writer.WriteLine ("\tvar theForm;\n\tif (document.getElementById) {{ theForm = document.getElementById ('{0}'); }}", formUniqueID);
+		writer.WriteLine ("\telse {{ theForm = document.{0}; }}", formUniqueID);
 		writer.WriteLine ("\tfunction __doPostBack(eventTarget, eventArgument) {");
 		writer.WriteLine ("\t\ttheForm.{0}.value = eventTarget;", postEventSourceID);
 		writer.WriteLine ("\t\ttheForm.{0}.value = eventArgument;", postEventArgumentID);
@@ -908,7 +977,7 @@ public class Page : TemplateControl, IHttpHandler
 		scriptManager.WriteClientScriptBlocks (writer);
 	}
 
-	LosFormatter GetFormatter ()
+	internal LosFormatter GetFormatter ()
 	{
 #if NET_2_0
 		PagesSection config = (PagesSection) WebConfigurationManager.GetSection ("system.web/pages");
@@ -949,7 +1018,11 @@ public class Page : TemplateControl, IHttpHandler
 
 		if (!postBackScriptRendered && requiresPostBackScript)
 			RenderPostBackScript (writer, formUniqueID);
-
+		
+#if NET_2_0
+		scriptManager.SaveEventValidationState ();
+		scriptManager.WriteExpandoAttributes (writer);
+#endif
 		scriptManager.WriteHiddenFields (writer);
 		scriptManager.WriteClientScriptIncludes (writer);
 		scriptManager.WriteStartupScriptBlocks (writer);
@@ -1035,6 +1108,9 @@ public class Page : TemplateControl, IHttpHandler
 	public void ProcessRequest (HttpContext context)
 #endif
 	{
+#if NET_2_0
+		_lifeCycle = PageLifeCycle.Unknown;
+#endif
 		_context = context;
 		if (clientTarget != null)
 			Request.ClientTarget = clientTarget;
@@ -1058,10 +1134,18 @@ public class Page : TemplateControl, IHttpHandler
 			context.ClearError (e);
 			// We want to remove that error, as we're rethrowing to stop
 			// further processing.
+			Trace.Warn ("Unhandled Exception", e.ToString (), e);
 			throw;
 		} finally {
 			try {
+#if NET_2_0
+				_lifeCycle = PageLifeCycle.Unload;
+#endif
+				RenderTrace ();
 				UnloadRecursive (true);
+#if NET_2_0
+				_lifeCycle = PageLifeCycle.End;
+#endif
 			} catch {}
 			if (Thread.CurrentThread.CurrentCulture.Equals (culture) == false)
 				Thread.CurrentThread.CurrentCulture = culture;
@@ -1084,28 +1168,48 @@ public class Page : TemplateControl, IHttpHandler
 		_requestValueCollection = this.DeterminePostBackMode();
 
 #if NET_2_0
-		if (!IsCrossPagePostBack)
-			LoadPreviousPageReference ();
-			
+		_lifeCycle = PageLifeCycle.Start;
+		// http://msdn2.microsoft.com/en-us/library/ms178141.aspx
+		if (_requestValueCollection != null) {
+			if (!isCrossPagePostBack && _requestValueCollection [PreviousPageID] != null && _requestValueCollection [PreviousPageID] != Request.FilePath) {
+				_doLoadPreviousPage = true;
+			}
+			else {
+				isCallback = _requestValueCollection [CallbackArgumentID] != null;
+				isPostBack = !isCallback;
+			}
+		}
+		
+		// if request was transfered from other page - track Prev. Page
+		previousPage = _context.LastPage;
+		_context.LastPage = this;
+
+		InitializeCulture ();
+		_lifeCycle = PageLifeCycle.PreInit;
 		OnPreInit (EventArgs.Empty);
 
 		InitializeTheme ();
 		ApplyMasterPage ();
+		_lifeCycle = PageLifeCycle.Init;
 #endif
 		Trace.Write ("aspx.page", "Begin Init");
 		InitRecursive (null);
 		Trace.Write ("aspx.page", "End Init");
 
 #if NET_2_0
+		_lifeCycle = PageLifeCycle.InitComplete;
 		OnInitComplete (EventArgs.Empty);
-		
-
-		if (_title != null && htmlHeader != null)
-			htmlHeader.Title = _title;
 #endif
 			
 		renderingForm = false;	
+#if NET_2_0
+		if (IsPostBack || IsCallback) {
+			_lifeCycle = PageLifeCycle.PreLoad;
+			if (_requestValueCollection != null)
+				scriptManager.RestoreEventValidationState (_requestValueCollection [scriptManager.EventStateFieldName]);
+#else
 		if (IsPostBack) {
+#endif
 			Trace.Write ("aspx.page", "Begin LoadViewState");
 			LoadPageViewState ();
 			Trace.Write ("aspx.page", "End LoadViewState");
@@ -1115,14 +1219,17 @@ public class Page : TemplateControl, IHttpHandler
 		}
 
 #if NET_2_0
-		if (IsCrossPagePostBack)
-			return;
-
 		OnPreLoad (EventArgs.Empty);
+		_lifeCycle = PageLifeCycle.Load;
 #endif
 
 		LoadRecursive ();
+#if NET_2_0
+		if (IsPostBack || IsCallback) {
+			_lifeCycle = PageLifeCycle.ControlEvents;
+#else
 		if (IsPostBack) {
+#endif
 			Trace.Write ("aspx.page", "Begin ProcessPostData Second Try");
 			ProcessPostData (secondPostData, true);
 			Trace.Write ("aspx.page", "End ProcessPostData Second Try");
@@ -1135,8 +1242,12 @@ public class Page : TemplateControl, IHttpHandler
 		}
 		
 #if NET_2_0
+		_lifeCycle = PageLifeCycle.LoadComplete;
 		OnLoadComplete (EventArgs.Empty);
 
+		if (IsCrossPagePostBack)
+			return;
+
 		if (IsCallback) {
 			string result = ProcessCallbackData ();
 			HtmlTextWriter callbackOutput = new HtmlTextWriter (_context.Response.Output);
@@ -1144,6 +1255,8 @@ public class Page : TemplateControl, IHttpHandler
 			callbackOutput.Flush ();
 			return;
 		}
+
+		_lifeCycle = PageLifeCycle.PreRender;
 #endif
 		
 		Trace.Write ("aspx.page", "Begin PreRender");
@@ -1151,6 +1264,7 @@ public class Page : TemplateControl, IHttpHandler
 		Trace.Write ("aspx.page", "End PreRender");
 		
 #if NET_2_0
+		_lifeCycle = PageLifeCycle.PreRenderComplete;
 		OnPreRenderComplete (EventArgs.Empty);
 #endif
 
@@ -1159,7 +1273,9 @@ public class Page : TemplateControl, IHttpHandler
 		Trace.Write ("aspx.page", "End SaveViewState");
 		
 #if NET_2_0
+		_lifeCycle = PageLifeCycle.SaveStateComplete;
 		OnSaveStateComplete (EventArgs.Empty);
+		_lifeCycle = PageLifeCycle.Render;
 #endif
 		
 		//--
@@ -1167,11 +1283,9 @@ public class Page : TemplateControl, IHttpHandler
 		HtmlTextWriter output = new HtmlTextWriter (_context.Response.Output);
 		RenderControl (output);
 		Trace.Write ("aspx.page", "End Render");
-		
-		RenderTrace (output);
 	}
 
-	private void RenderTrace (HtmlTextWriter output)
+	private void RenderTrace ()
 	{
 		TraceManager traceManager = HttpRuntime.TraceManager;
 
@@ -1183,13 +1297,37 @@ public class Page : TemplateControl, IHttpHandler
 		if (!Trace.HaveTrace && traceManager.Enabled && !traceManager.PageOutput) 
 			return;
 
-		if (!traceManager.LocalOnly || Context.Request.IsLocal)
+		if (!traceManager.LocalOnly || Context.Request.IsLocal) {
+			HtmlTextWriter output = new HtmlTextWriter (_context.Response.Output);
 			Trace.Render (output);
+		}
 	}
 	
+#if NET_2_0
+	bool CheckForValidationSupport (Control targetControl)
+	{
+		if (targetControl == null)
+			return false;
+		Type type = targetControl.GetType ();
+		object[] attributes = type.GetCustomAttributes (true);
+		foreach (object attr in attributes)
+			if (attr is SupportsEventValidationAttribute)
+				return true;
+		return false;
+	}
+#endif
+	
 	void RaisePostBackEvents ()
 	{
+#if NET_2_0
+		Control targetControl;
+#endif
 		if (requiresRaiseEvent != null) {
+#if NET_2_0
+			targetControl = requiresRaiseEvent as Control;
+			if (targetControl != null && CheckForValidationSupport (targetControl))
+				scriptManager.ValidateEvent (targetControl.UniqueID, null);
+#endif
 			RaisePostBackEvent (requiresRaiseEvent, null);
 			return;
 		}
@@ -1204,11 +1342,21 @@ public class Page : TemplateControl, IHttpHandler
 			return;
                 }
 
+#if NET_2_0
+		targetControl = FindControl (eventTarget);
+		IPostBackEventHandler target = targetControl as IPostBackEventHandler;
+#else
 		IPostBackEventHandler target = FindControl (eventTarget) as IPostBackEventHandler;
+#endif
+			
 		if (target == null)
 			return;
 
 		string eventArgument = postdata [postEventArgumentID];
+#if NET_2_0
+		if (CheckForValidationSupport (targetControl))
+			scriptManager.ValidateEvent (targetControl.UniqueID, eventArgument);
+#endif
 		RaisePostBackEvent (target, eventArgument);
 	}
 
@@ -1256,7 +1404,7 @@ public class Page : TemplateControl, IHttpHandler
 		scriptManager.RegisterHiddenField (hiddenFieldName, hiddenFieldInitialValue);
 	}
 
-	[MonoTODO("Used in HtmlForm")]
+	[MonoTODO("Not implemented, Used in HtmlForm")]
 	internal void RegisterClientScriptFile (string a, string b, string c)
 	{
 		throw new NotImplementedException ();
@@ -1320,17 +1468,15 @@ public class Page : TemplateControl, IHttpHandler
 		if (postdata == null || (view_state = postdata ["__VIEWSTATE"]) == null)
 			return null;
 
-		_savedViewState = null;
 		if (view_state == "")
 			return null;
 
 		LosFormatter fmt = GetFormatter ();
 		try {
-			_savedViewState = fmt.Deserialize (view_state);
+			return fmt.Deserialize (view_state);
 		} catch (Exception e) {
 			throw new HttpException ("Error restoring page viewstate.", e);
 		}
-		return _savedViewState;
 	}
 
 	internal void LoadPageViewState()
@@ -1394,7 +1540,15 @@ public class Page : TemplateControl, IHttpHandler
 		ValidateCollection (_validators);
 	}
 
+#if NET_2_0
+	internal bool AreValidatorsUplevel () {
+		return AreValidatorsUplevel (String.Empty);
+	}
+
+	internal bool AreValidatorsUplevel (string valGroup)
+#else
 	internal virtual bool AreValidatorsUplevel ()
+#endif
 	{
 		bool uplevel = false;
 
@@ -1402,6 +1556,10 @@ public class Page : TemplateControl, IHttpHandler
 			BaseValidator bv = v as BaseValidator;
 			if (bv == null) continue;
 
+#if NET_2_0
+			if (valGroup != bv.ValidationGroup)
+				continue;
+#endif
 			if (bv.GetRenderUplevel()) {
 				uplevel = true;
 				break;
@@ -1571,6 +1729,45 @@ public class Page : TemplateControl, IHttpHandler
 			EventHandler eh = (EventHandler) (Events [PreRenderCompleteEvent]);
 			if (eh != null) eh (this, e);
 		}
+
+		if (Form == null)
+			return;
+		if (!Form.DetermineRenderUplevel ())
+			return;
+
+		/* figure out if we have some control we're going to focus */
+		if (String.IsNullOrEmpty (_focusedControlID)) {
+			_focusedControlID = Form.DefaultFocus;
+			if (String.IsNullOrEmpty (_focusedControlID))
+				_focusedControlID = Form.DefaultButton;
+		}
+
+		if (!String.IsNullOrEmpty (_focusedControlID) || Form.SubmitDisabledControls) {
+
+			RequiresPostBackScript ();
+			ClientScript.RegisterWebFormClientScript ();
+
+			if (!String.IsNullOrEmpty (_focusedControlID)) {
+				ClientScript.RegisterStartupScript ("HtmlForm-DefaultButton-StartupScript",
+									 String.Format ("<script type=\"text/javascript\">\n" +
+											"<!--\n" +
+											"WebForm_AutoFocus('{0}');// -->\n" +
+											"</script>\n", _focusedControlID));
+			}
+
+			if (Form.SubmitDisabledControls) {
+				ClientScript.RegisterOnSubmitStatement ("HtmlForm-SubmitDisabledControls-SubmitStatement",
+										 "javascript: return WebForm_OnSubmit();");
+				ClientScript.RegisterStartupScript ("HtmlForm-SubmitDisabledControls-StartupScript",
+@"<script language=""JavaScript"">
+<!--
+function WebForm_OnSubmit() {
+WebForm_ReEnableControls();
+return true;
+} // -->
+</script>");
+			}
+		}
 	}
 	
 	protected virtual void OnSaveStateComplete (EventArgs e)
@@ -1593,20 +1790,26 @@ public class Page : TemplateControl, IHttpHandler
 	[BrowsableAttribute (false)]
 	[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 	public Page PreviousPage {
-		get { return previousPage; }
+		get {
+			if (_doLoadPreviousPage) {
+				_doLoadPreviousPage = false;
+				LoadPreviousPageReference ();
+			}
+			return previousPage;
+		}
 	}
 
 	
 	[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 	[BrowsableAttribute (false)]
 	public bool IsCallback {
-		get { return _requestValueCollection != null && _requestValueCollection [CallbackArgumentID] != null; }
+		get { return isCallback; }
 	}
 	
 	[BrowsableAttribute (false)]
 	[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 	public bool IsCrossPagePostBack {
-		get { return _requestValueCollection != null && isCrossPagePostBack; }
+		get { return isCrossPagePostBack; }
 	}
 	
 	string ProcessCallbackData ()
@@ -1615,11 +1818,14 @@ public class Page : TemplateControl, IHttpHandler
 		if (callbackTarget == null || callbackTarget.Length == 0)
 			throw new HttpException ("Callback target not provided.");
 
-		ICallbackEventHandler target = FindControl (callbackTarget) as ICallbackEventHandler;
+		Control targetControl = FindControl (callbackTarget);
+		ICallbackEventHandler target = targetControl as ICallbackEventHandler;
 		if (target == null)
 			throw new HttpException (string.Format ("Invalid callback target '{0}'.", callbackTarget));
 
 		string callbackArgument = _requestValueCollection [CallbackArgumentID];
+		if (CheckForValidationSupport (targetControl))
+			scriptManager.ValidateEvent (targetControl.UniqueID, callbackArgument);
 		target.RaiseCallbackEvent (callbackArgument);
 		return target.GetCallbackResult ();
 	}
@@ -1633,8 +1839,96 @@ public class Page : TemplateControl, IHttpHandler
 	internal void SetHeader (HtmlHead header)
 	{
 		htmlHeader = header;
+		if (_title != null) {
+			htmlHeader.Title = _title;
+			_title = null;
+		}
+	}
+
+	[MonoTODO("Not Implemented")]
+	protected bool AsyncMode {
+		get {
+			throw new NotImplementedException ();
+		}
+		set {
+			throw new NotImplementedException ();
+		}
+	}
+
+	[MonoTODO ("Not Implemented")]
+	public TimeSpan AsyncTimeout {
+		get {
+			throw new NotImplementedException ();
+		}
+		set {
+			throw new NotImplementedException ();
+		}
+	}
+
+	[MonoTODO ("Not Implemented")]
+	public bool IsAsync {
+		get {
+			throw new NotImplementedException ();
+		}
 	}
 	
+	[MonoTODO ("Not Implemented")]
+	protected internal virtual string UniqueFilePathSuffix {
+		get {
+			throw new NotImplementedException ();
+		}
+	}
+
+	[MonoTODO ("Not Implemented")]
+	public int MaxPageStateFieldLength {
+		get {
+			throw new NotImplementedException ();
+		}
+		set {
+			throw new NotImplementedException ();
+		}
+	}
+
+	[MonoTODO ("Not Implemented")]
+	public ViewStateEncryptionMode ViewStateEncryptionMode {
+		get {
+			throw new NotImplementedException ();
+		}
+		set {
+			throw new NotImplementedException ();
+		}
+	}
+
+	[MonoTODO ("Not Implemented")]
+	public void AddOnPreRenderCompleteAsync (BeginEventHandler beginHandler, EndEventHandler endHandler)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoTODO ("Not Implemented")]
+	public void AddOnPreRenderCompleteAsync (BeginEventHandler beginHandler, EndEventHandler endHandler, Object state)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoTODO ("Not Implemented")]
+	public void ExecuteRegisteredAsyncTasks ()
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoTODO ("Not Implemented")]
+	public static HtmlTextWriter CreateHtmlTextWriterFromType (TextWriter tw, Type writerType)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoTODO ("Not Implemented")]
+	public void RegisterRequiresViewStateEncryption ()
+	{
+		throw new NotImplementedException ();
+	}
+
 	void ApplyMasterPage ()
 	{
 		if (masterPageFile != null && masterPageFile.Length > 0) {
@@ -1663,6 +1957,9 @@ public class Page : TemplateControl, IHttpHandler
 	[BrowsableAttribute (false)]
 	public MasterPage Master {
 		get {
+			if (Context == null || String.IsNullOrEmpty (masterPageFile))
+				return null;
+
 			if (masterPage == null)
 				masterPage = MasterPage.CreateMasterPage (this, Context, masterPageFile, contentTemplates);
 
@@ -1670,11 +1967,51 @@ public class Page : TemplateControl, IHttpHandler
 		}
 	}
 	
+	public void SetFocus (string clientID)
+	{
+		if (String.IsNullOrEmpty (clientID))
+			throw new ArgumentNullException ("control");
+
+		if (_lifeCycle > PageLifeCycle.PreRender)
+			throw new InvalidOperationException ("SetFocus can only be called before and during PreRender.");
+
+		if(Form==null)
+			throw new InvalidOperationException ("A form tag with runat=server must exist on the Page to use SetFocus() or the Focus property.");
+
+		_focusedControlID = clientID;
+	}
+
+	public void SetFocus (Control control)
+	{
+		if (control == null)
+			throw new ArgumentNullException ("control");
+
+		SetFocus (control.ClientID);
+	}
+	
 	[EditorBrowsable (EditorBrowsableState.Advanced)]
 	public void RegisterRequiresControlState (Control control)
 	{
-		if (requireStateControls == null) requireStateControls = new ArrayList ();
-		requireStateControls.Add (control);
+		if (control == null)
+			throw new ArgumentNullException ("control");
+
+		if (RequiresControlState (control))
+			return;
+
+		if (requireStateControls == null)
+			requireStateControls = new ArrayList ();
+		int n = requireStateControls.Add (control);
+
+		if (_savedControlState == null || n >= _savedControlState.Length) 
+			return;
+
+		for (Control parent = control.Parent; parent != null; parent = parent.Parent)
+			if (parent.IsChildControlStateCleared)
+				return;
+
+		object state = _savedControlState [n];
+		if (state != null)
+			control.LoadControlState (state);
 	}
 	
 	public bool RequiresControlState (Control control)
@@ -1692,14 +2029,15 @@ public class Page : TemplateControl, IHttpHandler
 	
 	public ValidatorCollection GetValidators (string validationGroup)
 	{
-		if (validationGroup == null || validationGroup == "")
-			return Validators;
+		string valgr = validationGroup;
+		if (valgr == null)
+			valgr = String.Empty;
 
 		if (_validatorsByGroup == null) _validatorsByGroup = new Hashtable ();
-		ValidatorCollection col = _validatorsByGroup [validationGroup] as ValidatorCollection;
+		ValidatorCollection col = _validatorsByGroup [valgr] as ValidatorCollection;
 		if (col == null) {
 			col = new ValidatorCollection ();
-			_validatorsByGroup [validationGroup] = col;
+			_validatorsByGroup [valgr] = col;
 		}
 		return col;
 	}
@@ -1707,8 +2045,8 @@ public class Page : TemplateControl, IHttpHandler
 	public virtual void Validate (string validationGroup)
 	{
 		is_validated = true;
-		if (validationGroup == null || validationGroup == "")
-			ValidateCollection (_validators);
+		if (validationGroup == null)
+			ValidateCollection (_validatorsByGroup [String.Empty] as ValidatorCollection);
 		else if (_validatorsByGroup != null) {
 			ValidateCollection (_validatorsByGroup [validationGroup] as ValidatorCollection);
 		}
@@ -1730,13 +2068,14 @@ public class Page : TemplateControl, IHttpHandler
 	
 	void LoadPageControlState (object data)
 	{
+		_savedControlState = (object []) data;
+		
 		if (requireStateControls == null) return;
 
-		object[] state = (object[]) data;
-		int max = Math.Min (requireStateControls.Count, state != null ? state.Length : requireStateControls.Count);
+		int max = Math.Min (requireStateControls.Count, _savedControlState != null ? _savedControlState.Length : requireStateControls.Count);
 		for (int n=0; n < max; n++) {
 			Control ctl = (Control) requireStateControls [n];
-			ctl.LoadControlState (state != null ? state [n] : null);
+			ctl.LoadControlState (_savedControlState != null ? _savedControlState [n] : null);
 		}
 	}
 
@@ -1747,11 +2086,8 @@ public class Page : TemplateControl, IHttpHandler
 			if (prevPage != null) {
 				previousPage = (Page) PageParser.GetCompiledPageInstance (prevPage, Server.MapPath (prevPage), Context);
 				previousPage.ProcessCrossPagePostBack (_context);
-			} else {
-				previousPage = _context.LastPage;
-			}
+			} 
 		}
-		_context.LastPage = this;
 	}
 
 
@@ -1822,18 +2158,18 @@ public class Page : TemplateControl, IHttpHandler
 	#endif
 
 #if NET_2_0
-	[MonoTODO ("Used by .net aspx parser")]
+	[MonoTODO ("Not implemented.  Only used by .net aspx parser")]
 	protected object GetWrappedFileDependencies (string [] list)
 	{
 		return null;
 	}
 
-	[MonoTODO ("Used by .net aspx parser")]
+	[MonoTODO ("Does nothing.  Used by .net aspx parser")]
 	protected virtual void InitializeCulture ()
 	{
 	}
 
-	[MonoTODO ("Used by .net aspx parser")]
+	[MonoTODO ("Does nothing. Used by .net aspx parser")]
 	protected internal void AddWrappedFileDependencies (object virtualFileDependencies)
 	{
 	}
