@@ -50,6 +50,14 @@ namespace System.Web.UI {
 		static object commitTransaction = new object ();
 		static object error = new object ();
 		static string [] methodNames = { "Page_Init",
+#if NET_2_0
+						 "Page_PreInit",
+						 "Page_PreLoad",
+						 "Page_LoadComplete",
+						 "Page_PreRenderComplete",
+						 "Page_SaveStateComplete",
+						 "Page_InitComplete",
+#endif
 						 "Page_Load",
 						 "Page_DataBind",
 						 "Page_PreRender",
@@ -57,7 +65,7 @@ namespace System.Web.UI {
 						 "Page_Error",
 						 "Page_Unload",
 						 "Page_AbortTransaction",
-						 "Page_CommitTransaction" };
+						 "Page_CommitTransaction"};
 
 		const BindingFlags bflags = BindingFlags.Public |
 					    BindingFlags.NonPublic |
@@ -94,7 +102,7 @@ namespace System.Web.UI {
 		{
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		protected LiteralControl CreateResourceBasedLiteralControl (int offset,
 										    int size,
 										    bool fAsciiOnly)
@@ -224,7 +232,7 @@ namespace System.Web.UI {
 				eh (this, e);
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented, always returns null")]
 		public Control ParseControl (string content)
 		{
 			if (content == null)
@@ -243,20 +251,45 @@ namespace System.Web.UI {
 			throw new NotSupportedException ();
 		}
 
+#if NET_2_0
+		protected object GetGlobalResourceObject (string className, string resourceKey)
+		{
+			return HttpContext.GetGlobalResourceObject (className, resourceKey);
+		}
+
+		[MonoTODO ("Not implemented")]
+		protected object GetGlobalResourceObject (string className, string resourceKey, Type objType, string propName)
+		{
+			// FIXME: not sure how to implement that one yet
+			throw new NotSupportedException();
+		}
+
+		protected object GetLocalResourceObject (string resourceKey)
+		{
+			return HttpContext.GetLocalResourceObject (Context.Request.CurrentExecutionFilePath, resourceKey);
+		}
+		
+		protected object GetLocalResourceObject (string resourceKey, Type objType, string propName)
+		{
+			// FIXME: not sure how to implement that one yet
+			throw new NotSupportedException();
+		}
+#endif
+		
 		[EditorBrowsable (EditorBrowsableState.Never)]
 		public static object ReadStringResource (Type t)
 		{
 			throw new NotSupportedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented, does nothing")]
 		[EditorBrowsable (EditorBrowsableState.Never)]
 		protected void SetStringResourcePointer (object stringResourcePointer,
 							 int maxResourceOffset)
 		{
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented, does nothing")]
 		[EditorBrowsable (EditorBrowsableState.Never)]
 		protected void WriteUTF8ResourceString (HtmlTextWriter output, int offset,
 							int size, bool fAsciiOnly)
@@ -332,13 +365,13 @@ namespace System.Web.UI {
 
 		// IFilterResolutionService
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		int IFilterResolutionService.CompareFilters (string filter1, string filter2)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		bool IFilterResolutionService.EvaluateFilter (string filterName)
 		{
 			throw new NotImplementedException ();
