@@ -95,6 +95,13 @@ namespace System.Web {
 		// The factory for the handler currently running.
 		//
 		IHttpHandlerFactory factory;
+
+		//
+		// Whether the thread culture is to be auto-set.
+		// Used only in the 2.0 profile, always false for 1.x
+		//
+		bool autoCulture;
+		bool autoUICulture;
 		
 		//
 		// Whether the pipeline should be stopped
@@ -140,7 +147,7 @@ namespace System.Web {
 		{
 			done = new ManualResetEvent (false);
 		}
-
+		
 		internal void InitOnce (bool full_init)
 		{
 			lock (this) {
@@ -165,7 +172,9 @@ namespace System.Web {
 				GlobalizationSection cfg;
 				cfg = (GlobalizationSection) WebConfigurationManager.GetSection ("system.web/globalization");
 				app_culture = cfg.GetCulture();
+				autoCulture = cfg.IsAutoCulture;
 				appui_culture = cfg.GetUICulture();
+				autoUICulture = cfg.IsAutoUICulture;
 #else
 				GlobalizationConfiguration cfg;
 				cfg = GlobalizationConfiguration.GetInstance (null);
@@ -596,8 +605,10 @@ namespace System.Web {
 		//
 		void ProcessError (Exception e)
 		{
+			if (context == null)
+				context = HttpContext.Current;
+
 			bool first = context.Error == null;
-			
 			context.AddError (e);
 			if (first){
 				if (Error != null){
@@ -620,10 +631,18 @@ namespace System.Web {
 		void Tick ()
 		{
 			try {
-				if (pipeline.MoveNext ()){
-					if ((bool)pipeline.Current)
+				if (pipeline != null && pipeline.MoveNext ()){
+					if (pipeline == null || (bool)pipeline.Current)
 						PipelineDone ();
 				}
+			} catch (ThreadAbortException taex) {
+				object obj = taex.ExceptionState;
+				Thread.ResetAbort ();
+				stop_processing = true;
+				if (obj is StepTimeout)
+					ProcessError (new HttpException ("The request timed out."));
+				else
+					PipelineDone ();
 			} catch (Exception e) {
 				Console.WriteLine ("Tick caught an exception that has not been propagated:\n" + e);
 			}
@@ -787,12 +806,21 @@ namespace System.Web {
 			}
 
 			try {
-
 				OutputPage ();
 			} catch (Exception e) {
 				Console.WriteLine ("Internal error: OutputPage threw an exception " + e);
 			} finally {
+				if (context == null)
+					context = HttpContext.Current;
 				context.WorkerRequest.EndOfRequest();
+				if (factory != null && context.Handler != null){
+					factory.ReleaseHandler (context.Handler);
+					context.Handler = null;
+					factory = null;
+				}
+#if NET_2_0
+				context.PopHandler ();
+#endif
 				if (begin_iar != null){
 					try {
 						begin_iar.Complete ();
@@ -802,17 +830,13 @@ namespace System.Web {
 						// Not really too bad, since the only failure might be
 						// `HttpRuntime.request_processed'
 						//
+					} finally {
+						done.Set ();
 					}
+				} else {
+					done.Set ();
 				}
 				
-				done.Set ();
-
-				if (factory != null && context.Handler != null){
-					factory.ReleaseHandler (context.Handler);
-					factory = null;
-				}
-				
-				context.Handler = null;
 				// context = null; -> moved to PostDone
 				pipeline = null;
 				current_ai = null;
@@ -859,17 +883,28 @@ namespace System.Web {
 				foreach (bool stop in RunHooks (ResolveRequestCache))
 					yield return stop;
 
+#if NET_2_0
+			if (PostResolveRequestCache != null)
+				foreach (bool stop in RunHooks (PostResolveRequestCache))
+					yield return stop;
+#endif
+
 			// Obtain the handler for the request.
 			IHttpHandler handler = null;
 			try {
 				handler = GetHandler (context);
 				context.Handler = handler;
+#if NET_2_0
+				context.PushHandler (handler);
+#endif
 			} catch (FileNotFoundException fnf){
 				if (context.Request.IsLocal)
 					ProcessError (new HttpException (404, String.Format ("File not found {0}", fnf.FileName), fnf));
 				else
 					ProcessError (new HttpException (404, "File not found: " + Path.GetFileName (fnf.FileName)));
 			} catch (DirectoryNotFoundException dnf){
+				if (!context.Request.IsLocal)
+					dnf = null; // Do not "leak" real path information
 				ProcessError (new HttpException (404, "Directory not found", dnf));
 			} catch (Exception e) {
 				ProcessError (e);
@@ -879,10 +914,6 @@ namespace System.Web {
 				yield return true;
 
 #if NET_2_0
-			if (PostResolveRequestCache != null)
-				foreach (bool stop in RunHooks (PostResolveRequestCache))
-					yield return stop;
-
 			if (PostMapRequestHandler != null)
 				foreach (bool stop in RunHooks (PostMapRequestHandler))
 					yield return stop;
@@ -984,6 +1015,30 @@ namespace System.Web {
 			PipelineDone ();
 		}
 
+
+		internal CultureInfo GetThreadCulture (HttpRequest request, CultureInfo culture, bool isAuto)
+		{
+#if NET_2_0
+			if (!isAuto)
+				return culture;
+			CultureInfo ret = null;
+			string[] languages = request.UserLanguages;
+			try {
+				if (languages != null && languages.Length > 0)
+					ret = new CultureInfo (languages[0]);
+			} catch {
+			}
+			
+			if (ret == null)
+				ret = culture;
+			
+			return ret;
+#else
+			return culture;
+#endif
+		}
+
+
 		void PreStart ()
 		{
 #if !TARGET_J2EE
@@ -992,12 +1047,12 @@ namespace System.Web {
 			Thread th = Thread.CurrentThread;
 			if (app_culture != null) {
 				prev_app_culture = th.CurrentCulture;
-				th.CurrentCulture = app_culture;
+				th.CurrentCulture = GetThreadCulture (Request, app_culture, autoCulture);
 			}
 
 			if (appui_culture != null) {
 				prev_appui_culture = th.CurrentUICulture;
-				th.CurrentUICulture = appui_culture;
+				th.CurrentUICulture = GetThreadCulture (Request, appui_culture, autoUICulture);
 			}
 
 #if !TARGET_JVM
@@ -1018,6 +1073,8 @@ namespace System.Web {
 				th.CurrentCulture = prev_app_culture;
 
 #if !TARGET_J2EE
+			if (context == null)
+				context = HttpContext.Current;
 			HttpRuntime.TimeoutManager.Remove (context);
 #endif
 			context = null;
