@@ -40,6 +40,10 @@ using System.Web.Compilation;
 using vmw.common;
 #endif
 
+#if NET_2_0 && !TARGET_J2EE
+using System.CodeDom.Compiler;
+#endif
+
 namespace System.Web {
 	class HttpApplicationFactory {
 		// Initialized in InitType
@@ -74,6 +78,7 @@ namespace System.Web {
 #if !TARGET_JVM
 		FileSystemWatcher app_file_watcher;
 		FileSystemWatcher bin_watcher;
+		FileSystemWatcher config_watcher;
 #endif
 		Stack available = new Stack ();
 		Stack available_for_end = new Stack ();
@@ -224,6 +229,8 @@ namespace System.Web {
 
 		void OnAppFileChanged (object sender, FileSystemEventArgs args)
 		{
+			if (config_watcher != null)
+				config_watcher.EnableRaisingEvents = false;
 			if (bin_watcher != null)
 				bin_watcher.EnableRaisingEvents = false;
 			if (app_file_watcher != null)
@@ -369,13 +376,27 @@ namespace System.Web {
 				app_file = Path.Combine (physical_app_path, "Global.asax");
 				if (!File.Exists (app_file))
 					app_file = Path.Combine (physical_app_path, "global.asax");
+
 			
 #if NET_2_0
 				WebConfigurationManager.Init ();
 #else
 				WebConfigurationSettings.Init (context);
 #endif
+		
+#if NET_2_0 && !TARGET_J2EE
+				AppGlobalResourcesCompiler agrc = new AppGlobalResourcesCompiler();
+				agrc.Compile();
 				
+				// Todo: Process App_WebResources here
+				
+				// Todo: Generate profile properties assembly from Web.config here
+				
+				// Todo: Compile code from App_Code here
+				AppCodeCompiler acc = new AppCodeCompiler ();
+				acc.Compile ();
+#endif
+
 				if (File.Exists (app_file)) {
 #if TARGET_J2EE
 					app_type = System.Web.J2EE.PageMapper.GetObjectType(app_file);
@@ -385,15 +406,25 @@ namespace System.Web {
 						string msg = String.Format ("Error compiling application file ({0}).", app_file);
 						throw new ApplicationException (msg);
 					}
-
-					FileSystemEventHandler fseh = new FileSystemEventHandler (OnAppFileChanged);
-					RenamedEventHandler reh = new RenamedEventHandler (OnAppFileRenamed);
-					app_file_watcher = CreateWatcher (app_file, fseh, reh);
 #endif
 				} else {
 					app_type = typeof (System.Web.HttpApplication);
 					app_state = new HttpApplicationState ();
 				}
+
+#if !TARGET_JVM
+				FileSystemEventHandler fseh = new FileSystemEventHandler (OnAppFileChanged);
+				RenamedEventHandler reh = new RenamedEventHandler (OnAppFileRenamed);
+				app_file_watcher = CreateWatcher (app_file, fseh, reh);
+
+				string config_file = Path.Combine (physical_app_path, "Web.config");
+				if (!File.Exists (config_file))
+					config_file = Path.Combine (physical_app_path, "web.config");
+				if (!File.Exists (config_file))
+					config_file = Path.Combine (physical_app_path, "Web.Config");
+
+				config_watcher = CreateWatcher (config_file, fseh, reh);
+#endif
 				needs_init = false;
 
 				//
