@@ -42,6 +42,9 @@ using System.Web.SessionState;
 using System.Web.UI;
 using System.Web.Util;
 #if NET_2_0
+using System.Collections.Generic;
+using System.Reflection;
+using System.Resources;
 using System.Web.Profile;
 using CustomErrorMode = System.Web.Configuration.CustomErrorsMode;
 #endif
@@ -67,6 +70,10 @@ namespace System.Web {
 		object config_timeout;
 		int timeout_possible;
 		DateTime time_stamp = DateTime.UtcNow;
+#if NET_2_0
+		ProfileBase profile = null;
+		LinkedList<IHttpHandler> handlers;
+#endif
 #if TARGET_JVM // No remoting support (CallContext) yet in Grasshopper
 		static LocalDataStoreSlot _ContextSlot = Thread.GetNamedDataSlot ("Context");
 #endif
@@ -274,19 +281,59 @@ namespace System.Web {
 		}
 
 #if NET_2_0
-		[MonoTODO]
+		internal void PushHandler (IHttpHandler handler)
+		{
+			if (handler == null)
+				return;
+			if (handlers == null)
+				handlers = new LinkedList <IHttpHandler> ();
+			handlers.AddLast (handler);
+		}
+
+		internal void PopHandler ()
+		{
+			if (handlers == null || handlers.Count == 0)
+				return;
+			handlers.RemoveLast ();
+		}
+		
+		IHttpHandler GetCurrentHandler ()
+		{
+			if (handlers == null || handlers.Count == 0)
+				return null;
+			
+			return handlers.Last.Value;
+		}
+
+		IHttpHandler GetPreviousHandler ()
+		{
+			if (handlers == null || handlers.Count <= 1)
+				return null;
+			LinkedListNode <IHttpHandler> previous = handlers.Last.Previous;
+			if (previous != null)
+				return previous.Value;
+			return null;
+		}
+		
 		public IHttpHandler CurrentHandler {
-			get { throw new NotImplementedException (); }
+			get { return GetCurrentHandler (); }
 		}
 
-		[MonoTODO]
 		public IHttpHandler PreviousHandler {
-			get { throw new NotImplementedException (); }
+			get { return GetPreviousHandler (); }
 		}
 
-		[MonoTODO]
 		public ProfileBase Profile {
-			get { throw new NotImplementedException (); }
+			get 
+			{
+				if (profile == null) {
+					if (Request.IsAuthenticated)
+						profile = ProfileBase.Create (User.Identity.Name);
+					else
+						profile = ProfileBase.Create (Request.AnonymousID, false);
+				}
+				return profile;
+			}
 		}
 #endif
 
@@ -340,38 +387,84 @@ namespace System.Web {
 		}
 
 #if NET_2_0
-		[MonoTODO]
+		internal static Type GetGLResourceType (string typeName)
+		{
+			Type type = null;
+			Assembly [] assemblies = AppDomain.CurrentDomain.GetAssemblies ();
+
+			foreach (Assembly ass in assemblies) {
+				type = ass.GetType (typeName);
+				if (type == null)
+					continue;
+
+				return type;
+			}
+
+			throw new MissingManifestResourceException (String.Format ("Missing resource class {0}", typeName));
+		}
+
+		internal static object GetGLResourceObject (Type type, string resourceKey)
+		{
+			object ret = null;
+			try {
+				PropertyInfo pi = type.GetProperty (resourceKey,
+								    BindingFlags.GetProperty |
+								    BindingFlags.Public |
+								    BindingFlags.Static);
+				if (pi == null)
+					return null;
+				ret = pi.GetValue (null, null);
+			} catch {
+			}
+			
+			return ret;
+		}
+
+		internal static void SetGLResourceObjectCulture (Type type, CultureInfo ci)
+		{
+			try {
+				PropertyInfo pi = type.GetProperty ("Culture",
+								    BindingFlags.SetProperty |
+								    BindingFlags.Public |
+								    BindingFlags.Static);
+				if (pi == null)
+					return; // internal error actually...
+				pi.SetValue (null, ci, null);
+			} catch {
+			}
+		}
+		
 		public static object GetGlobalResourceObject (string classKey, string resourceKey)
 		{
-			throw new NotImplementedException ();
+			string className = String.Format ("System.Resources.{0}", classKey);
+			Type type = GetGLResourceType (className);
+			SetGLResourceObjectCulture (type, null);
+			return GetGLResourceObject (type, resourceKey);
 		}
 
-		[MonoTODO]
 		public static object GetGlobalResourceObject (string classKey, string resourceKey, CultureInfo culture)
 		{
-			throw new NotImplementedException ();
+			string className = String.Format ("System.Resources.{0}", classKey);
+			Type type = GetGLResourceType (className);
+			SetGLResourceObjectCulture (type, culture);
+			return GetGLResourceObject (type, resourceKey);
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public static object GetLocalResourceObject (string virtualPath, string resourceKey)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public static object GetLocalResourceObject (string virtualPath, string resourceKey, CultureInfo culture)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
 		public object GetSection (string name)
 		{
-#if NET_2_0
 			return WebConfigurationManager.GetSection (name);
-#else
-			throw new NotImplementedException ();
-#endif
 		}
 #endif
 		object IServiceProvider.GetService (Type service)
@@ -421,14 +514,38 @@ namespace System.Web {
 
 		public void RewritePath (string path)
 		{
-			int qmark = path.IndexOf ('?');
-			if (qmark != -1)
-				RewritePath (path.Substring (0, qmark), "", path.Substring (qmark + 1));
-			else
-				RewritePath (path, null, null);
+#if NET_2_0
+			RewritePath (path, true);
+#else
+			RewritePath (path, false);
+#endif
 		}
 
 		public void RewritePath (string filePath, string pathInfo, string queryString)
+		{
+			RewritePath (filePath, pathInfo, queryString, false);
+		}
+
+#if NET_2_0
+		public
+#else
+		internal
+#endif
+		void RewritePath (string path, bool rebaseClientPath)
+		{
+			int qmark = path.IndexOf ('?');
+			if (qmark != -1)
+				RewritePath (path.Substring (0, qmark), "", path.Substring (qmark + 1), rebaseClientPath);
+			else
+				RewritePath (path, null, null, rebaseClientPath);
+		}
+
+#if NET_2_0
+		public
+#else
+		internal
+#endif
+		void RewritePath (string filePath, string pathInfo, string queryString, bool setClientFilePath)
 		{
 			filePath = UrlUtils.Combine (Request.BaseVirtualDir, filePath);
 			if (!filePath.StartsWith (HttpRuntime.AppDomainAppVirtualPath))
@@ -442,15 +559,11 @@ namespace System.Web {
 
 			if (queryString != null)
 				Request.QueryStringRaw = queryString;
-		}
-
 #if NET_2_0
-		[MonoTODO]
-		public void RewritePath (string path, bool rebaseClientPath)
-		{
-			throw new NotImplementedException ();
-		}
+			if (setClientFilePath)
+				Request.SetFilePath (filePath);
 #endif
+		}
 
 #region internals
 		
