@@ -97,6 +97,10 @@ namespace System.Web {
 		//
 		internal object FlagEnd = new object ();
 
+#if NET_2_0
+		bool is_request_being_redirected;
+#endif
+		
 		internal HttpResponse ()
 		{
 			output_stream = new HttpResponseStream (this);
@@ -118,10 +122,8 @@ namespace System.Web {
 		
 		internal TextWriter SetTextWriter (TextWriter writer)
 		{
-			TextWriter prev = writer;
-			
+			TextWriter prev = this.writer;
 			this.writer = writer;
-			
 			return prev;
 		}
 		
@@ -247,7 +249,7 @@ namespace System.Web {
 			}
 		}
 #if NET_2_0
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public Encoding HeaderEncoding {
 			get { throw new NotImplementedException (); }
 			set {
@@ -266,9 +268,8 @@ namespace System.Web {
 			}
 		}
 #if NET_2_0
-		[MonoTODO]
 		public bool IsRequestBeingRedirected {
-			get { throw new NotImplementedException (); }
+			get { return is_request_being_redirected; }
 		}
 #endif
 		public TextWriter Output {
@@ -364,43 +365,43 @@ namespace System.Web {
 			}
 		}
 #if NET_2_0
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public void AddCacheDependency (CacheDependency[] dependencies)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public void AddCacheItemDependencies (string[] cacheKeys)
 		{
 			throw new NotImplementedException ();
 		}
 #endif
-		[MonoTODO]
+		[MonoTODO("Currently does nothing")]
 		public void AddCacheItemDependencies (ArrayList cacheKeys)
 		{
 			// TODO: talk to jackson about the cache
 		}
 
-		[MonoTODO]
+		[MonoTODO("Currently does nothing")]
 		public void AddCacheItemDependency (string cacheKey)
 		{
 			// TODO: talk to jackson about the cache
 		}
 
-		[MonoTODO]
+		[MonoTODO("Currently does nothing")]
 		public void AddFileDependencies (ArrayList filenames)
 		{
 			// TODO: talk to jackson about the cache
 		}
 #if NET_2_0
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public void AddFileDependencies (string[] filenames)
 		{
 			throw new NotImplementedException ();
 		}
 #endif
-		[MonoTODO]
+		[MonoTODO ("Currently does nothing")]
 		public void AddFileDependency (string filename)
 		{
 			// TODO: talk to jackson about the cache
@@ -590,7 +591,9 @@ namespace System.Web {
 					// If we are not chunked, we need to set "Connection: close".
 					//
 					if (use_chunked){
+#if DEBUG
 						Console.WriteLine ("Setting to close2");
+#endif
 						write_headers.Add (new KnownResponseHeader (HttpWorkerRequest.HeaderConnection, "close"));
 					}
 				}
@@ -600,7 +603,9 @@ namespace System.Web {
 				// close at the end.
 				//
 				if (use_chunked){
+#if DEBUG
 					Console.WriteLine ("Setting to close");
+#endif
 					write_headers.Add (new KnownResponseHeader (HttpWorkerRequest.HeaderConnection, "close"));
 				}
 			}
@@ -735,7 +740,9 @@ namespace System.Web {
 		{
 			if (headers_sent)
 				throw new HttpException ("header have been already sent");
-
+#if NET_2_0
+			is_request_being_redirected = true;
+#endif
 			ClearHeaders ();
 			ClearContent ();
 			
@@ -750,6 +757,9 @@ namespace System.Web {
 			
 			if (endResponse)
 				End ();
+#if NET_2_0
+			is_request_being_redirected = false;
+#endif
 		}
 
 		public static void RemoveOutputCacheItem (string path)
@@ -882,7 +892,7 @@ namespace System.Web {
 			Flush ();
 		}
 #if NET_2_0
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public void WriteSubstitution (HttpResponseSubstitutionCallback callback)
 		{
 			throw new NotImplementedException ();
