@@ -41,6 +41,11 @@ using System.Web.Configuration;
 using System.Web.UI;
 using System.Threading;
 
+#if NET_2_0 && !TARGET_JVM
+using System.CodeDom.Compiler;
+using System.Web.Compilation;
+#endif
+
 namespace System.Web {
 	
 	// CAS - no InheritanceDemand here as the class is sealed
@@ -255,6 +260,14 @@ namespace System.Web {
 			} else {
 				context.ApplicationInstance = app;
 			
+#if NET_2_0 && !TARGET_JVM
+				//
+				// Compile the local resources, if any
+				//
+				AppLocalResourcesCompiler alrc = new AppLocalResourcesCompiler();
+				alrc.Compile();
+#endif
+				
 				//
 				// Ask application to service the request
 				//
@@ -320,6 +333,7 @@ namespace System.Web {
 		//
 		static void ShutdownAppDomain (object args)
 		{
+			queue_manager.Dispose ();
 			// This will call Session_End if needed.
 			Cache.InvokePrivateCallbacks ();
 			// Kill our application.
