@@ -59,7 +59,8 @@ namespace MonoTests.System.Web.Hosting {
 	[TestFixture]
 	public class ApplicationHostTest {
 
-		[SetUp] public void Setup ()
+		[SetUp] 
+		public void Setup ()
 		{
 			try {
 				//
@@ -92,7 +93,9 @@ namespace MonoTests.System.Web.Hosting {
 			ApplicationHost.CreateApplicationHost (null, null, null);
 		}
 
-		[Test][ExpectedException (typeof (UriFormatException))]
+		[Test]
+        [Category ("NotWorking")]
+        [ExpectedException (typeof (ArgumentException))]
 		public void ConstructorTestNull2 ()
 		{
 			//
@@ -101,7 +104,9 @@ namespace MonoTests.System.Web.Hosting {
 			ApplicationHost.CreateApplicationHost (null, "/app", ".");
 		}
 
-		[Test][ExpectedException (typeof (UriFormatException))]
+		[Test]
+        [Category("NotWorking")]
+        [ExpectedException (typeof (ArgumentNullException))]
 		public void ConstructorTestNull3 ()
 		{
 			//
@@ -119,7 +124,11 @@ namespace MonoTests.System.Web.Hosting {
 			ApplicationHost.CreateApplicationHost (typeof (MBR), "/app", null);
 		}
 
-		[Test][ExpectedException(typeof(SerializationException))]
+		[Test]
+		[ExpectedException(typeof(SerializationException))]
+#if TARGET_JVM //System.Security.Policy.Evidence not implemented
+		[Category ("NotWorking")]
+#endif
 		public void Constructor_PlainType ()
 		{
 			ApplicationHost.CreateApplicationHost (typeof (ApplicationHostTest), "/app", Environment.CurrentDirectory);
@@ -141,6 +150,7 @@ namespace MonoTests.System.Web.Hosting {
 		}
 		
 		[Test]
+		[Category ("NotDotNet")] // D2 and D3 asserts will fail in windows on because file system environment 
 		public void ConstructorTest ()
 		{
 			object o = ApplicationHost.CreateApplicationHost (typeof (MBR), "/app", Environment.CurrentDirectory);
