@@ -22,6 +22,7 @@ namespace MonoTests.System.Web.Security {
 	public class FormsAuthenticationTest {
 
 		[Test]
+		[Category ("NotDotNet")] // Dot.net url must include Namespace name
 		[Category("NunitWeb")]
 		public void DefaultValues ()
 		{
@@ -47,6 +48,7 @@ namespace MonoTests.System.Web.Security {
 		}
 
 		[Test]
+		[Category ("NotDotNet")] // Dot.net url must include Namespace name
 		[Category("NunitWeb")]
 		public void Initialize ()
 		{
@@ -127,5 +129,11 @@ namespace MonoTests.System.Web.Security {
 			FormsAuthentication.RedirectToLoginPage (String.Empty);
 		}
 #endif
+
+		[TestFixtureTearDown]
+		public void TestFixtureTearDown()
+		{
+			WebTest.Unload();
+		}
 	}
 }
