@@ -43,6 +43,7 @@ using System.Web.UI.HtmlControls;
 using System.Web.UI.WebControls;
 using MonoTests.SystemWeb.Framework;
 using MonoTests.stand_alone.WebHarness;
+using System.Collections.Generic;
 
 
 namespace MonoTests.System.Web.UI.WebControls
@@ -50,6 +51,36 @@ namespace MonoTests.System.Web.UI.WebControls
 	[TestFixture]	
 	public class DetailsViewTest {
 
+		public class DataSourceObject
+		{
+			public static List<string> GetList (string sortExpression, int startRowIndex, int maximumRows) {
+				return GetList ();
+			}
+
+			public static List<string> GetList (int startRowIndex, int maximumRows) {
+				return GetList ();
+			}
+
+			public static List<string> GetList (string sortExpression) {
+				return GetList ();
+			}
+
+			public static List<string> GetList () {
+				List<string> list = new List<string> ();
+				list.Add ("Norway");
+				list.Add ("Sweden");
+				list.Add ("France");
+				list.Add ("Italy");
+				list.Add ("Israel");
+				list.Add ("Russia");
+				return list;
+			}
+
+			public static int GetCount () {
+				return GetList ().Count;
+			}
+		}
+		
 		public class PokerDetailsView: DetailsView 
 		{
 			public bool ensureDataBound=false;
@@ -171,9 +202,9 @@ namespace MonoTests.System.Web.UI.WebControls
 			{
 				OnUnload (e);
 			}
-			public void DoOnBubbleEvent (Object sender, EventArgs e)
+			public bool DoOnBubbleEvent (Object sender, EventArgs e)
 			{
-				OnBubbleEvent (sender,e);
+				return OnBubbleEvent (sender, e);
 			}
 
 			public Object DoSaveControlState ()
@@ -291,11 +322,27 @@ namespace MonoTests.System.Web.UI.WebControls
 			{
 				base.EnsureChildControls ();
 			}
+
+			public DataSourceSelectArguments DoCreateDataSourceSelectArguments ()
+			{
+				return CreateDataSourceSelectArguments ();
+		}
+
+			public DataSourceView DoGetData ()
+			{
+				return GetData ();
+			}
+
+			public void SetRequiresDataBinding (bool value)
+			{
+				RequiresDataBinding = value;
+			}
 		}
 
 		ArrayList myds = new ArrayList ();
+
 		[TestFixtureSetUp]
-		public void setup ()
+		public void constract ()
 		{
 			myds.Add ("Item1");
 			myds.Add ("Item2");
@@ -303,6 +350,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			myds.Add ("Item4");
 			myds.Add ("Item5");
 			myds.Add ("Item6");
+
 #if VISUAL_STUDIO
 			WebTest.CopyResource (GetType (), "MonoTests.System.Web.UI.WebControls.Resources.FooterTemplateTest.aspx",
 				"FooterTemplateTest.aspx");
@@ -318,13 +366,14 @@ namespace MonoTests.System.Web.UI.WebControls
 			WebTest.CopyResource (GetType (), "DetailsViewDataActions.aspx", "DetailsViewDataActions.aspx");
 			WebTest.CopyResource (GetType (), "DetailsViewProperties1.aspx", "DetailsViewProperties1.aspx");
 #endif
-			
 		}
 
 
 
 		[Test]
-		[Category ("NotWorking")] 
+#if TARGET_JVM //probably dependant on BUG #6489
+		[Category ("NotWorking")]
+#endif
 		public void DetailsView_DefaultProperties ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
@@ -380,7 +429,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
+#if TARGET_JVM //probably dependant on BUG #6489
+		[Category ("NotWorking")]
+#endif
 		public void DetailsView_AssignToDefaultProperties ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
@@ -497,14 +548,13 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_AssignedPropertiesRender ()
 		{
-						string RenderedPageHtml = new WebTest ("DetailsViewProperties1.aspx").Run ();
+			string RenderedPageHtml = new WebTest ("DetailsViewProperties1.aspx").Run ();
 			string newHtmlValue = RenderedPageHtml.Substring (RenderedPageHtml.IndexOf ("starttest") + 9, RenderedPageHtml.IndexOf ("endtest") - RenderedPageHtml.IndexOf ("starttest") - 9);
 			string origHtmlValue = @" <div>
 			<div>
-			<table cellspacing=""20"" cellpadding=""30"" rules=""all"" border=""1"" id=""DetailsView1"" style=""height:50px;width:125px;background-image:url(Blue%20hills.jpg);"">
+			<table cellspacing=""20"" cellpadding=""30"" rules=""all"" border=""1"" id=""DetailsView1"" style=""height:50px;width:125px;background-image:url(Blue_hills.jpg);"">
 			<caption align=""Bottom"">
 			Caption Test
 			</caption><tr>
@@ -522,15 +572,13 @@ namespace MonoTests.System.Web.UI.WebControls
 			Assert.AreEqual (true, origHtmlValue.Contains (@"cellspacing=""20"""), "CellspacingRender");
 			Assert.AreEqual (true, origHtmlValue.Contains ("Caption Test"), "CaptionRender");
 			Assert.AreEqual (true, origHtmlValue.Contains (@"caption align=""Bottom"""), "CaptionalignRender");
-			Assert.AreEqual (true, origHtmlValue.Contains ("Blue%20hills.jpg"), "BackImageRender");
-			Assert.AreEqual (true, origHtmlValue.Contains ("Blue%20hills.jpg"), "BackImageRender");
+			Assert.AreEqual (true, origHtmlValue.Contains ("Blue_hills.jpg"), "BackImageRender");
 			//GridLines and HorizontalAlign were set but can not be shown in this rendering.
 		}
 
 		
 		[Test]
 		[Category ("NunitWeb")]
-		[Category ("NotWorking")] 
 		public void DetailsView_EmptyDataTextPropertyRender ()
 		{	
 			PageDelegate pd = new PageDelegate (DetailsView_EmptyDataTextProperty);
@@ -594,7 +642,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
+#if TARGET_JVM	//BUG #6484
+		[Category ("NotWorking")]
+#endif
 		public void FormView_DataBind ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
@@ -609,7 +659,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void FormView_IsBindableType ()
 		{
 			bool isBindable = false;
@@ -636,7 +685,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		private bool isDeleted = false;
 
 		[Test]
-		[Category ("NotWorking")] 
+#if TARGET_JVM	//BUG #6484
+		[Category ("NotWorking")]
+#endif
 		public void DetailsView_DeleteItemHandler ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
@@ -656,7 +707,6 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		[Test]
 		[Category ("NunitWeb")]
-		[Category ("NotWorking")] 
 		public void DetailsView_DeleteItemTest ()
 		{
 			string html = new WebTest (PageInvoker.CreateOnLoad (
@@ -716,11 +766,11 @@ namespace MonoTests.System.Web.UI.WebControls
 		//ToDo: InsertItem method should be checked using postback
 		private bool insertItem = false;
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_InsertItem ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
 			dv.Page = new Page ();
+			dv.Page.Validate();
 			dv.ChangeMode (DetailsViewMode.Insert);
 			dv.ItemInserting += new DetailsViewInsertEventHandler (insert_item);
 			Assert.AreEqual (false, insertItem, "BeforeInsertItem");
@@ -737,12 +787,12 @@ namespace MonoTests.System.Web.UI.WebControls
 		//ToDo: UpdateItem method should be checked using postback
 		private bool updateItem = false;
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_UpdateItem ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
 			dv.ChangeMode (DetailsViewMode.Edit);
 			dv.Page = new Page ();
+			dv.Page.Validate ();
 			dv.ItemUpdating += new DetailsViewUpdateEventHandler (update_item);
 			Assert.AreEqual (false, updateItem, "BeforeUpdateItem");
 			dv.UpdateItem (true);
@@ -758,7 +808,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		//protected methods
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_CreateAutoGeneratedRows ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();			
@@ -776,7 +825,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_CreateAutoGenerateRow ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();			
@@ -792,7 +840,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
+#if TARGET_JVM	//BUG #6484
+		[Category ("NotWorking")]
+#endif
 		public void DetailsView_CreateChildControls ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();			
@@ -801,7 +851,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_CreateControlStyle ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
@@ -812,7 +861,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_CreateRow ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
@@ -837,7 +885,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_CreateTable ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
@@ -853,7 +900,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_EnsureDataBound ()
 		{
 			ObjectDataSource ds = new ObjectDataSource ();
@@ -874,19 +920,17 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void DetailsView_EnsureChildControls ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
 			int i = dv.Rows.Count;
 			Assert.IsTrue (dv.ensureCreateChildControls);
 			Assert.IsFalse (dv.ensureDataBound);
-			Assert.IsFalse (dv.createChildControls1);
+			Assert.IsTrue (dv.createChildControls1);
 			Assert.IsFalse (dv.createChildControls2);
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_ExtractRowValues ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
@@ -956,28 +1000,24 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_GetCallBackResult ()
 		{
+
 			PokerDetailsView dv = new PokerDetailsView ();
 			Page p = new Page ();
+			p.EnableEventValidation = false;
 			p.Controls.Add (dv);
-			DataTable ds = TableObject.CreateDataTable ();			
+
+			DataTable ds = TableObject.CreateDataTable ();
+			dv.AllowPaging = true;
+			dv.EnablePagingCallbacks = true;
 			dv.DataSource = ds;
 			dv.DataBind ();
-			dv.DoRaiseCallbackEvent ("a|b$c");
-			string callbackResult=@"<table cellspacing=""0"" rules=""all"" border=""1"" style=""border-collapse:collapse;"">
-						<tr>
-						<td>ID</td><td>1001</td>
-						</tr><tr>
-						<td>FName</td><td>Mahesh</td>
-						</tr><tr>
-						<td>LName</td><td>Chand</td>
-						</tr>
-						</table>";
+			dv.DoRaiseCallbackEvent ("1||0|");
 			string cbres = dv.DoGetCallbackResult ();
 			Assert.IsNotNull (cbres);
-			HtmlDiff.AssertAreEqual (callbackResult, cbres.Substring(dv.DoGetCallbackResult().IndexOf ("<table")), "GetCallbackResult");
+			if (cbres.IndexOf ("1002") == -1)
+				Assert.Fail ("Wrong item rendered fail");
 				
 		}
 
@@ -988,7 +1028,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
+#if TARGET_JVM	//BUG #6484
+		[Category ("NotWorking")]
+#endif
 		public void DetailsView_InitializePager ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
@@ -1010,7 +1052,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_PerformDataBinding ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
@@ -1020,7 +1061,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_PrepareControlHierarchy ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
@@ -1036,8 +1076,10 @@ namespace MonoTests.System.Web.UI.WebControls
 			Assert.AreEqual (1, dv.Controls.Count, "ControlHierarchy3");
 			Assert.AreEqual (true, dv.controlHierarchy, "ControlHierarchy4");
 			Button bt = new Button ();
+			dv.Page.EnableEventValidation = false;
 			dv.Controls.Add (bt);
 			dv.controlHierarchy = false;
+
 			dv.Render ();
 			Assert.AreEqual (2, dv.Controls.Count, "ControlHierarchy3");
 			Assert.AreEqual (true, dv.controlHierarchy, "ControlHierarchy4");
@@ -1045,9 +1087,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		
 		//Render Methods
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_FooterTemplateRender ()
 		{
+
 			//Footer Template property is checked.
 			string RenderedPageHtml = new WebTest ("FooterTemplateTest.aspx").Run ();
 			string newHtmlValue = RenderedPageHtml.Substring (RenderedPageHtml.IndexOf ("starttest")+9, RenderedPageHtml.IndexOf ("endtest") - RenderedPageHtml.IndexOf ("starttest")-9); 
@@ -1080,7 +1122,7 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
+		[Category ("NunitWeb")] 
 		public void DetailsView_RenderHeaderTemplate ()
 		{
 			//Header Template property is checked
@@ -1116,7 +1158,7 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
+		[Category ("NunitWeb")] 
 		public void DetailsView_PagerTemplateRender ()
 		{
 			//Pager Template property is checked
@@ -1143,7 +1185,7 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
+		[Category ("NunitWeb")] 
 		public void DetailsView_EditFieldsRender ()
 		{
 			string RenderedPageHtml = new WebTest ("DetailsViewTemplates.aspx").Run ();
@@ -1183,7 +1225,6 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		[Test]
 		[Category ("NunitWeb")]
-		[Category ("NotWorking")] 
 		public void DetailsView_PagingPostback ()
 		{
 			WebTest t = new WebTest ("DetailsViewDataActions.aspx");
@@ -1248,7 +1289,6 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		[Test]
 		[Category ("NunitWeb")]
-		[Category ("NotWorking")] 
 		public void DetailsView_EditPostback ()
 		{			
 			WebTest t = new WebTest ("DetailsViewDataActions.aspx");
@@ -1285,13 +1325,10 @@ namespace MonoTests.System.Web.UI.WebControls
 						</div>     
 						</div>";
 					HtmlDiff.AssertAreEqual (origHtmlValue, pageHTML, "AfterEditPostback");
-			  
-
 		}
 
 		[Test]
 		[Category ("NunitWeb")]
-		[Category ("NotWorking")] 
 		public void DetailsView_DeletePostback ()
 		{			
 			WebTest t = new WebTest ("DetailsViewDataActions.aspx");			
@@ -1304,6 +1341,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			fr.Controls["__EVENTARGUMENT"].Value = "Delete$0";
 			t.Request = fr;
 			pageHTML = t.Run ();
+			Console.WriteLine ("XXXXFAIL {0}", pageHTML);
 			pageHTML = pageHTML.Substring (pageHTML.IndexOf ("starttest") + 9, pageHTML.IndexOf ("endtest") - pageHTML.IndexOf ("starttest") - 9);
 			string origHtmlValue = @"   <div>
 						   &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;
@@ -1330,17 +1368,12 @@ namespace MonoTests.System.Web.UI.WebControls
 
 			HtmlDiff.AssertAreEqual (origHtmlValue, pageHTML, "DeleteDataPostback");
 			Assert.AreEqual (false, pageHTML.Contains ("1001"), "AfterDeletePostback");
-			
-			
-
 		}
 
 		[Test]
 		[Category ("NunitWeb")]
-		[Category ("NotWorking")] 
 		public void DetailsView_InsertPostback ()
 		{
-
 			WebTest t = new WebTest ("DetailsViewDataActions.aspx");
 			string pageHTML = t.Run ();
 			Assert.AreEqual (true, pageHTML.Contains ("1001"), "BeforeDeletePostback");
@@ -1374,7 +1407,6 @@ namespace MonoTests.System.Web.UI.WebControls
 
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_ViewState ()
 		{
 			PokerDetailsView dv = new PokerDetailsView ();
@@ -1390,10 +1422,8 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_ControlState ()
 		{
-
 			PokerDetailsView dv = new PokerDetailsView ();
 			PokerDetailsView copy = new PokerDetailsView ();
 			string[] keys = new String[2];
@@ -1452,7 +1482,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_BubbleEvents ()
 		{
 			ResetEvents ();
@@ -1462,7 +1491,9 @@ namespace MonoTests.System.Web.UI.WebControls
 			Page page = new Page ();
 			Button bt = new Button ();
 			dv.AllowPaging = true;
+			dv.DataSource = myds;
 			page.Controls.Add (dv);		
+			dv.DataBind ();
 			dv.ItemCommand += new DetailsViewCommandEventHandler (dv_ItemCommand );
 			dv.ItemDeleted += new DetailsViewDeletedEventHandler (dv_ItemDeleted );
 			//Delete
@@ -1470,7 +1501,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new DetailsViewCommandEventArgs (bt, new CommandEventArgs ("Delete", null));
 			Assert.AreEqual (false, itemCommand, "BeforeDeleteCommandBubbleEvent");
 			Assert.AreEqual (false, itemDeleting, "BeforeDeleteBubbleEvent");			
-			dv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (dv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Delete");
 			Assert.AreEqual (true, itemDeleting, "AfterDeleteBubbleEvent");			
 			Assert.AreEqual (true, itemCommand, "AfterDeleteCommandBubbleEvent");
 
@@ -1481,7 +1512,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new DetailsViewCommandEventArgs (bt, new CommandEventArgs ("Insert", null));
 			Assert.AreEqual (false, itemCommand, "BeforeInsertCommandBubbleEvent");
 			Assert.AreEqual (false, itemInserting, "BeforeInsertBubbleEvent");
-			dv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (dv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Insert");
 			Assert.AreEqual (true, itemCommand, "AfterInsertCommandBubbleEvent");
 			Assert.AreEqual (true, itemInserting, "AfterInsertBubbleEvent");
 
@@ -1492,7 +1523,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new DetailsViewCommandEventArgs (bt, new CommandEventArgs ("Update", null));
 			Assert.AreEqual (false, itemUpdating, "BeforeUpdateEvent");
 			Assert.AreEqual (false, itemCommand, "BeforeUpdateCommandEvent");
-			dv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (dv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Update");
 			Assert.AreEqual (true, itemCommand, "AfterUpdateCommandBubbleEvent");
 			Assert.AreEqual (true, itemUpdating, "AfterUpdateBubbleEvent");
 
@@ -1502,7 +1533,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new DetailsViewCommandEventArgs (bt, new CommandEventArgs ("Cancel", null));
 			Assert.AreEqual (false, itemCommand, "BeforeCancelCommandBubbleEvent");
 			Assert.AreEqual (false, modeChanging, "BeforeCancelBubbleEvent");
-			dv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (dv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Cancel");
 			Assert.AreEqual (true, itemCommand, "AfterCancelCommandBubbleEvent");
 			Assert.AreEqual (true, modeChanging, "AfterCancelBubbleEvent");
 
@@ -1512,7 +1543,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new DetailsViewCommandEventArgs (bt, new CommandEventArgs ("Edit", null));
 			Assert.AreEqual (false, itemCommand, "BeforeEditCommandBubbleEvent");
 			Assert.AreEqual (false, modeChanging, "BeforeEditBubbleEvent");
-			dv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (dv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Edit");
 			Assert.AreEqual (true, itemCommand, "AfterEditCommandBubbleEvent");
 			Assert.AreEqual (true, modeChanging, "AfterEditBubbleEvent");
 
@@ -1522,7 +1553,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new DetailsViewCommandEventArgs (bt, new CommandEventArgs ("New", null));
 			Assert.AreEqual (false, itemCommand, "BeforeNewCommandBubbleEvent");
 			Assert.AreEqual (false, modeChanging, "BeforeNewBubbleEvent");
-			dv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (dv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - New");
 			Assert.AreEqual (true, itemCommand, "AfterNewCommandBubbleEvent");
 			Assert.AreEqual (true, modeChanging, "AfterNewBubbleEvent");
 
@@ -1532,9 +1563,10 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new DetailsViewCommandEventArgs (bt, new CommandEventArgs ("Page", null));
 			Assert.AreEqual (false, itemCommand, "BeforePageCommandBubbleEvent");
 			Assert.AreEqual (false, pageIndexChanging, "BeforePageBubbleEvent");
-			dv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (dv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Page Index default");
 			Assert.AreEqual (true, itemCommand, "AfterPageCommandBubbleEvent");
 			Assert.AreEqual (true, pageIndexChanging, "AfterPageBubbleEvent");
+			Assert.AreEqual (-1, newPageIndex, "PageIndex");
 
 			//Next Page
 			itemCommand = false;
@@ -1542,7 +1574,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new DetailsViewCommandEventArgs (bt, new CommandEventArgs ("Page", "Next"));
 			Assert.AreEqual (false, itemCommand, "BeforeNextPageCommandBubbleEvent");
 			Assert.AreEqual (false, pageIndexChanging, "BeforeNextPageBubbleEvent");
-			dv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (dv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Next Page");
 			Assert.AreEqual (true, itemCommand, "AfterNextPageCommandBubbleEvent");
 			Assert.AreEqual (true, pageIndexChanging, "AfterNextPageBubbleEvent");
 			Assert.AreEqual (1, newPageIndex, "NextPageIndex");
@@ -1553,7 +1585,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new DetailsViewCommandEventArgs (bt, new CommandEventArgs ("Page", "Prev"));
 			Assert.AreEqual (false, itemCommand, "BeforePrevPageCommandBubbleEvent");
 			Assert.AreEqual (false, pageIndexChanging, "BeforePrevPageBubbleEvent");
-			dv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (dv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Prev Page");
 			Assert.AreEqual (true, itemCommand, "AfterPrevPageCommandBubbleEvent");
 			Assert.AreEqual (true, pageIndexChanging, "AfterPrevPageBubbleEvent");
 			Assert.AreEqual (-1, newPageIndex, "PrevPageIndex");
@@ -1564,7 +1596,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new DetailsViewCommandEventArgs (bt, new CommandEventArgs ("Page", "First"));
 			Assert.AreEqual (false, itemCommand, "BeforeFirstPageCommandBubbleEvent");
 			Assert.AreEqual (false, pageIndexChanging, "BeforeFirstPageBubbleEvent");
-			dv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (dv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - First Page");
 			Assert.AreEqual (true, itemCommand, "AfterFirstPageCommandBubbleEvent");
 			Assert.AreEqual (true, pageIndexChanging, "AfterFirstPageBubbleEvent");
 			Assert.AreEqual (0, newPageIndex, "FirstPageIndex");
@@ -1575,15 +1607,14 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new DetailsViewCommandEventArgs (bt, new CommandEventArgs ("Page", "Last"));
 			Assert.AreEqual (false, itemCommand, "BeforeLastPageCommandBubbleEvent");
 			Assert.AreEqual (false, pageIndexChanging, "BeforeLastPageBubbleEvent");
-			dv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (dv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Last Page");
 			Assert.AreEqual (true, itemCommand, "AfterLastPageCommandBubbleEvent");
 			Assert.AreEqual (true, pageIndexChanging, "AfterLastPageBubbleEvent");
-			Assert.AreEqual (-1, newPageIndex, "FirstPageIndex");
+			Assert.AreEqual (5, newPageIndex, "FirstPageIndex");
 
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		public void DetailsView_Events ()
 		{
 			ResetEvents ();
@@ -1684,6 +1715,7 @@ namespace MonoTests.System.Web.UI.WebControls
 		{
 			pageIndexChanging = true;
 			newPageIndex = e.NewPageIndex;
+			e.NewPageIndex = -1;
 		}
 
 		void dv_PageIndexChanged (object sender, EventArgs e)
@@ -1752,7 +1784,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		//Exceptions
 
 		[Test]
-		[Category ("NotWorking")] 
 		[ExpectedException (typeof (ArgumentOutOfRangeException))]
 		public void CellPaddingException ()
 		{
@@ -1761,7 +1792,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		[ExpectedException (typeof (ArgumentOutOfRangeException))]
 		public void CellSpacingException ()
 		{
@@ -1770,7 +1800,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		[ExpectedException (typeof (ArgumentOutOfRangeException))]
 		public void PageIndexException ()
 		{
@@ -1779,7 +1808,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		[ExpectedException (typeof (HttpException))]
 		public void InsertItemException ()
 		{
@@ -1788,7 +1816,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")] 
 		[ExpectedException (typeof (HttpException))]
 		public void UpdateItemException ()
 		{
@@ -1796,8 +1823,216 @@ namespace MonoTests.System.Web.UI.WebControls
 			dv.UpdateItem (true);
 		}
 
+		[Test]
+		public void DetailsView_PageCount () {
+			Page p = new Page ();
 
+			PokerDetailsView gv = new PokerDetailsView ();
+			p.Controls.Add (gv);
 
+			ObjectDataSource data = new ObjectDataSource ();
+			data.ID = "ObjectDataSource1";
+			data.TypeName = typeof (TableObject).AssemblyQualifiedName;
+			data.SelectMethod = "GetMyData";
+			p.Controls.Add (data);
+
+			gv.DataSourceID = "ObjectDataSource1";
+			gv.DataKeyNames = new string [] { "ID", "FName" };
+			gv.SetRequiresDataBinding (true);
+
+			Assert.AreEqual (0, gv.PageCount, "PageCount before binding");
+
+			gv.DataBind ();
+
+			Assert.AreEqual (3, gv.PageCount, "PageCount after binding");
+
+			//PokerDetailsView copy = new PokerDetailsView ();
+			//object state = gv.SaveState ();
+			//copy.LoadState (state);
+
+			//Assert.AreEqual (3, copy.PageCount, "PageCount from ViewState");
+		}
+
+		[Test]
+		public void DetailsView_DataKey () {
+			Page p = new Page ();
+
+			PokerDetailsView fv = new PokerDetailsView ();
+			p.Controls.Add (fv);
+
+			ObjectDataSource data = new ObjectDataSource ();
+			data.TypeName = typeof (TableObject).AssemblyQualifiedName;
+			data.SelectMethod = "GetMyData";
+			p.Controls.Add (data);
+
+			fv.DataSource = data;
+			fv.DataKeyNames = new string [] { "ID", "FName" };
+
+			DataKey key1 = fv.DataKey;
+
+			Assert.AreEqual (null, key1.Value, "DataKey.Value before binding");
+			Assert.AreEqual (0, key1.Values.Count, "DataKey.Values count before binding");
+
+			fv.DataBind ();
+
+			DataKey key2 = fv.DataKey;
+			DataKey key3 = fv.DataKey;
+
+			Assert.IsFalse (Object.ReferenceEquals (key1, key2), "DataKey returns the same instans");
+			Assert.IsTrue (Object.ReferenceEquals (key2, key3), "DataKey returns the same instans");
+
+			Assert.AreEqual (1001, key1.Value, "DataKey.Value after binding");
+			Assert.AreEqual (2, key1.Values.Count, "DataKey.Values count after binding");
+			Assert.AreEqual (1001, key1.Values [0], "DataKey.Values[0] after binding");
+			Assert.AreEqual ("Mahesh", key1.Values [1], "DataKey.Values[1] after binding");
+
+			PokerDetailsView copy = new PokerDetailsView ();
+			object state = fv.DoSaveControlState ();
+			copy.DoLoadControlState (state);
+
+			DataKey key4 = copy.DataKey;
+
+			Assert.AreEqual (1001, key4.Value, "DataKey.Value from ControlState");
+			Assert.AreEqual (2, key4.Values.Count, "DataKey.Values count from ControlState");
+			Assert.AreEqual (1001, key4.Values [0], "DataKey.Values[0] from ControlState");
+			Assert.AreEqual ("Mahesh", key4.Values [1], "DataKey.Values[1] from ControlState");
+		}
+
+		[Test]
+		public void DetailsView_CreateDataSourceSelectArguments () {
+			DataSourceView view;
+			Page p = new Page ();
+
+			PokerDetailsView dv = new PokerDetailsView ();
+			p.Controls.Add (dv);
+
+			ObjectDataSource data = new ObjectDataSource ();
+			data.TypeName = typeof (DataSourceObject).AssemblyQualifiedName;
+			data.SelectMethod = "GetList";
+			data.SortParameterName = "sortExpression";
+			DataSourceSelectArguments arg;
+			p.Controls.Add (data);
+
+			dv.DataSource = data;
+			dv.DataBind ();
+
+			arg = dv.DoCreateDataSourceSelectArguments ();
+			Assert.IsTrue (arg.Equals (DataSourceSelectArguments.Empty), "Default");
+
+			dv.AllowPaging = true;
+			dv.PageIndex = 2;
+			arg = dv.DoCreateDataSourceSelectArguments ();
+			view = dv.DoGetData ();
+			Assert.IsFalse (view.CanPage);
+			Assert.IsTrue (view.CanRetrieveTotalRowCount);
+			Assert.IsTrue (arg.Equals (DataSourceSelectArguments.Empty), "AllowPaging = true, CanPage = false, CanRetrieveTotalRowCount = true");
+
+			// make DataSourceView.CanPage = true
+			data.EnablePaging = true;
+
+			arg = dv.DoCreateDataSourceSelectArguments ();
+			view = dv.DoGetData ();
+			Assert.IsTrue (view.CanPage);
+			Assert.IsFalse (view.CanRetrieveTotalRowCount);
+			Assert.IsTrue (arg.Equals (new DataSourceSelectArguments (2, -1)), "AllowPaging = true, CanPage = true, CanRetrieveTotalRowCount = false");
+
+			dv.AllowPaging = false;
+			arg = dv.DoCreateDataSourceSelectArguments ();
+			Assert.IsTrue (arg.Equals (DataSourceSelectArguments.Empty), "AllowPaging = false, CanPage = true, CanRetrieveTotalRowCount = false");
+
+			// make DataSourceView.CanRetrieveTotalRowCount = true
+			data.SelectCountMethod = "GetCount";
+
+			arg = dv.DoCreateDataSourceSelectArguments ();
+			Assert.IsTrue (arg.Equals (DataSourceSelectArguments.Empty), "AllowPaging = false, CanPage = true, CanRetrieveTotalRowCount = true");
+
+			dv.AllowPaging = true;
+			arg = dv.DoCreateDataSourceSelectArguments ();
+			DataSourceSelectArguments arg1 = new DataSourceSelectArguments (2, 1);
+			arg1.RetrieveTotalRowCount = true;
+			view = dv.DoGetData ();
+			Assert.IsTrue (view.CanPage);
+			Assert.IsTrue (view.CanRetrieveTotalRowCount);
+			Assert.IsTrue (arg.Equals (arg1), "AllowPaging = true, CanPage = true, CanRetrieveTotalRowCount = true");
+	}
+
+		[Test]
+		public void DetailsView_CurrentMode () {
+			DetailsView view = new DetailsView ();
+			view.DefaultMode = DetailsViewMode.Insert;
+			Assert.AreEqual (DetailsViewMode.Insert, view.CurrentMode, "DetailsView_CurrentMode#1");
+			view.ChangeMode (DetailsViewMode.Edit);
+			Assert.AreEqual (DetailsViewMode.Edit, view.CurrentMode, "DetailsView_CurrentMode#2");
+		}
+
+		[TestFixtureTearDown]
+		public void Tear ()
+		{
+			WebTest.Unload ();
+		}
+
+		[Test]
+		public void DetailsView_GetPostBackOptions () {
+			DetailsView dv = new DetailsView ();
+			dv.Page = new Page ();
+			IButtonControl btn = new Button ();
+			btn.CausesValidation = false;
+			Assert.IsFalse (btn.CausesValidation, "DetailsView_GetPostBackOptions #1");
+			Assert.AreEqual (String.Empty, btn.CommandName, "DetailsView_GetPostBackOptions #2");
+			Assert.AreEqual (String.Empty, btn.CommandArgument, "DetailsView_GetPostBackOptions #3");
+			Assert.AreEqual (String.Empty, btn.PostBackUrl, "DetailsView_GetPostBackOptions #4");
+			Assert.AreEqual (String.Empty, btn.ValidationGroup, "DetailsView_GetPostBackOptions #5");
+			PostBackOptions options = ((IPostBackContainer) dv).GetPostBackOptions (btn);
+			Assert.IsFalse (options.PerformValidation, "DetailsView_GetPostBackOptions #6");
+			Assert.IsFalse (options.AutoPostBack, "DetailsView_GetPostBackOptions #7");
+			Assert.IsFalse (options.TrackFocus, "DetailsView_GetPostBackOptions #8");
+			Assert.IsTrue (options.ClientSubmit, "DetailsView_GetPostBackOptions #9");
+			Assert.IsTrue (options.RequiresJavaScriptProtocol, "DetailsView_GetPostBackOptions #10");
+			Assert.AreEqual ("$", options.Argument, "DetailsView_GetPostBackOptions #11");
+			Assert.AreEqual (null, options.ActionUrl, "DetailsView_GetPostBackOptions #12");
+			Assert.AreEqual (null, options.ValidationGroup, "DetailsView_GetPostBackOptions #13");
+
+			btn.ValidationGroup = "VG";
+			btn.CommandName = "CMD";
+			btn.CommandArgument = "ARG";
+			btn.PostBackUrl = "Page.aspx";
+			Assert.IsFalse (btn.CausesValidation, "DetailsView_GetPostBackOptions #14");
+			Assert.AreEqual ("CMD", btn.CommandName, "DetailsView_GetPostBackOptions #15");
+			Assert.AreEqual ("ARG", btn.CommandArgument, "DetailsView_GetPostBackOptions #16");
+			Assert.AreEqual ("Page.aspx", btn.PostBackUrl, "DetailsView_GetPostBackOptions #17");
+			Assert.AreEqual ("VG", btn.ValidationGroup, "DetailsView_GetPostBackOptions #18");
+			options = ((IPostBackContainer) dv).GetPostBackOptions (btn);
+			Assert.IsFalse (options.PerformValidation, "DetailsView_GetPostBackOptions #19");
+			Assert.IsFalse (options.AutoPostBack, "DetailsView_GetPostBackOptions #20");
+			Assert.IsFalse (options.TrackFocus, "DetailsView_GetPostBackOptions #21");
+			Assert.IsTrue (options.ClientSubmit, "DetailsView_GetPostBackOptions #22");
+			Assert.IsTrue (options.RequiresJavaScriptProtocol, "DetailsView_GetPostBackOptions #23");
+			Assert.AreEqual ("CMD$ARG", options.Argument, "DetailsView_GetPostBackOptions #24");
+			Assert.AreEqual (null, options.ActionUrl, "DetailsView_GetPostBackOptions #25");
+			Assert.AreEqual (null, options.ValidationGroup, "DetailsView_GetPostBackOptions #26");
+		}
+
+		[Test]
+		[ExpectedException (typeof (InvalidOperationException))]
+		public void DetailsView_GetPostBackOptions_CausesValidation () {
+			DetailsView dv = new DetailsView ();
+			dv.Page = new Page ();
+			IButtonControl btn = new Button ();
+			Assert.IsTrue (btn.CausesValidation);
+			Assert.AreEqual (String.Empty, btn.CommandName);
+			Assert.AreEqual (String.Empty, btn.CommandArgument);
+			Assert.AreEqual (String.Empty, btn.PostBackUrl);
+			Assert.AreEqual (String.Empty, btn.ValidationGroup);
+			PostBackOptions options = ((IPostBackContainer) dv).GetPostBackOptions (btn);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void DetailsView_GetPostBackOptions_Null_Argument () {
+			DetailsView dv = new DetailsView ();
+			dv.Page = new Page ();
+			PostBackOptions options = ((IPostBackContainer) dv).GetPostBackOptions (null);
+		}
 	}
 
 	public class DTemplate : ITemplate
@@ -1939,3 +2174,7 @@ namespace MonoTests.System.Web.UI.WebControls
 }
 #endif
 
+
+
+
+
