@@ -54,6 +54,37 @@ namespace MonoTests.System.Web.UI.WebControls
 {
 	[TestFixture]	
 	public class FormViewTest {	
+
+		public class DataSourceObject
+		{
+			public static List<string> GetList (string sortExpression, int startRowIndex, int maximumRows) {
+				return GetList ();
+			}
+
+			public static List<string> GetList (int startRowIndex, int maximumRows) {
+				return GetList ();
+			}
+
+			public static List<string> GetList (string sortExpression) {
+				return GetList ();
+			}
+
+			public static List<string> GetList () {
+				List<string> list = new List<string> ();
+				list.Add ("Norway");
+				list.Add ("Sweden");
+				list.Add ("France");
+				list.Add ("Italy");
+				list.Add ("Israel");
+				list.Add ("Russia");
+				return list;
+			}
+
+			public static int GetCount () {
+				return GetList ().Count;
+			}
+		}
+		
 		public class Poker : FormView {
 			public bool isInitializePager=false;
 			public bool ensureDataBound=false;
@@ -95,6 +126,11 @@ namespace MonoTests.System.Web.UI.WebControls
 				return CreateDataSourceSelectArguments ();
 			}
 
+			public DataSourceView DoGetData ()
+			{
+				return GetData ();
+			}
+
 			public FormViewRow DoCreateRow (int itemIndex,DataControlRowType rowType,DataControlRowState rowState)
 			{
 				return CreateRow( itemIndex, rowType,rowState); 
@@ -327,7 +363,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void Defaults ()
 		{
 			Poker p = new Poker ();
@@ -465,7 +500,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
+#if TARGET_JVM //BUG #6484
 		[Category ("NotWorking")]
+#endif
 		public void FormView_ItemsProperties ()
 		{
 			Poker p = new Poker ();
@@ -501,7 +538,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		// Protected methods
 
 		[Test]
-		[Category ("NotWorking")]
 		public void FormView_CreateChildControls ()
 		{
 			Poker fv = new Poker ();
@@ -526,7 +562,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void FormView_CreateControlStyle ()
 		{
 			Poker fv = new Poker ();
@@ -546,7 +581,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
+#if TARGET_JVM //BUG #6484
 		[Category ("NotWorking")]
+#endif
 		public void FormView_InitializePager ()
 		{
 			Poker fv = new Poker ();
@@ -562,7 +599,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 		
 		[Test]
+#if TARGET_JVM //BUG #6484
 		[Category ("NotWorking")]
+#endif
 		public void FormView_CreateRow ()
 		{
 			Poker fv = new Poker ();
@@ -586,7 +625,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
+#if TARGET_JVM //BUG #6484
 		[Category ("NotWorking")]
+#endif
 		public void FormView_CreateTable ()
 		{
 			Poker fv = new Poker ();
@@ -613,7 +654,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void FormView_PerformDataBinding ()
 		{
 			Poker fv = new Poker ();
@@ -654,7 +694,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
+#if TARGET_JVM //BUG #6484
 		[Category ("NotWorking")]
+#endif
 		public void FormView_PrepareControlHierarcy ()
 		{
 			Poker fv = new Poker ();
@@ -690,7 +732,76 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
+		public void FormView_PageCount () {
+			Page p = new Page ();
+
+			Poker fv = new Poker ();
+			p.Controls.Add (fv);
+
+			ObjectDataSource data = new ObjectDataSource ();
+			data.TypeName = typeof (FormViewDataObject).AssemblyQualifiedName;
+			data.SelectMethod = "Select";
+			p.Controls.Add (data);
+
+			fv.DataSource = data;
+
+			Assert.AreEqual (0, fv.PageCount, "PageCount before binding");
+
+			fv.DataBind ();
+			
+			Assert.AreEqual (3, fv.PageCount, "PageCount after binding");
+		}
+
+		[Test]
+		public void FormView_DataKey ()
+		{
+			Page p = new Page ();
+
+			Poker fv = new Poker ();
+			p.Controls.Add (fv);
+
+			ObjectDataSource data = new ObjectDataSource ();
+			data.TypeName = typeof (FormViewDataObject).AssemblyQualifiedName;
+			data.SelectMethod = "Select";
+			p.Controls.Add (data);
+
+			fv.DataSource = data;
+			fv.DataKeyNames = new string [] { "ID", "FName" };
+
+			DataKey key1 = fv.DataKey;
+
+			Assert.AreEqual (null, key1.Value, "DataKey.Value before binding");
+			Assert.AreEqual (0, key1.Values.Count, "DataKey.Values count before binding");
+
+			fv.DataBind ();
+
+			DataKey key2 = fv.DataKey;
+			DataKey key3 = fv.DataKey;
+
+			Assert.IsFalse (Object.ReferenceEquals (key1, key2), "DataKey returns the same instans");
+			Assert.IsTrue (Object.ReferenceEquals (key2, key3), "DataKey returns the same instans");
+			
+			Assert.AreEqual (1001, key1.Value, "DataKey.Value after binding");
+			Assert.AreEqual (2, key1.Values.Count, "DataKey.Values count after binding");
+			Assert.AreEqual (1001, key1.Values [0], "DataKey.Values[0] after binding");
+			Assert.AreEqual ("Mahesh", key1.Values [1], "DataKey.Values[1] after binding");
+
+			Poker copy = new Poker ();
+			object state = fv.DoSaveControlState ();
+			copy.DoLoadControlState (state);
+
+			DataKey key4 = copy.DataKey;
+
+			Assert.AreEqual (1001, key4.Value, "DataKey.Value from ViewState");
+			Assert.AreEqual (2, key4.Values.Count, "DataKey.Values count from ViewState");
+			Assert.AreEqual (1001, key4.Values [0], "DataKey.Values[0] from ViewState");
+			Assert.AreEqual ("Mahesh", key4.Values [1], "DataKey.Values[1] from ViewState");
+		}
+
+		[Test]
+#if TARGET_JVM //BUG #6484
 		[Category ("NotWorking")]
+#endif
 		public void FormView_DataBind ()
 		{
 			Poker fv = new Poker ();
@@ -708,7 +819,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		private bool isDeleted = false;
 
 		[Test]
+#if TARGET_JVM //BUG #6484
 		[Category ("NotWorking")]
+#endif
 		public void FormView_DeleteItem ()
 		{
 			Poker fv = new Poker ();
@@ -730,7 +843,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		private bool insertItem = false;
 
 		[Test]
-		[Category ("NotWorking")]
 		public void FormView_InsertItem ()
 		{
 			Poker fv = new Poker ();
@@ -738,7 +850,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			fv.ChangeMode (FormViewMode.Insert);
 			fv.ItemInserting += new FormViewInsertEventHandler (insert_item);
 			Assert.AreEqual (false, insertItem, "BeforeInsertItem");
-			fv.InsertItem (true);
+			fv.InsertItem (false);
 			Assert.AreEqual (true, insertItem, "AfterInsertItem");
 
 		}
@@ -750,14 +862,19 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		private bool updateItem = false;
 		[Test]
+#if TARGET_JVM //BUG #6484
 		[Category ("NotWorking")]
+#endif
 		public void FormView_UpdateItem ()
 		{
 			Poker fv = new Poker ();
+			fv.Page = new Page ();
+			fv.DataSource = myds;
+			fv.DataBind ();
 			fv.ChangeMode (FormViewMode.Edit);
 			fv.ItemUpdating += new FormViewUpdateEventHandler (update_item);
 			Assert.AreEqual (false, updateItem, "BeforeUpdateItem");
-			fv.UpdateItem (true);
+			fv.UpdateItem (false);
 			Assert.AreEqual (true, updateItem, "AfterUpdateItem");
 
 		}
@@ -768,7 +885,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void FormView_IsBindableType ()
 		{
 			bool isBindable = false;
@@ -792,7 +908,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void FormView_ControlState ()		{
 
 			Poker fv = new Poker ();
@@ -803,12 +918,14 @@ namespace MonoTests.System.Web.UI.WebControls
 			fv.DataKeyNames = keys;
 			fv.BackImageUrl = "photo.jpg";			
 			fv.DefaultMode  = FormViewMode.Insert  ;
+			fv.ChangeMode (FormViewMode.Edit);
 			object state = fv.DoSaveControlState ();
 			copy.DoLoadControlState (state);
 			Assert.AreEqual (2, copy.DataKeyNames.Length, "DataKeyNames.Length");
 			Assert.AreEqual ("key1", copy.DataKeyNames[0], "ControlStateDataKeyValue");
 			Assert.AreEqual ("key2", copy.DataKeyNames[1], "ControlStateDataKeyValue2");			
 			Assert.AreEqual (FormViewMode.Insert, copy.DefaultMode, "ControlStateDefaultMode");
+			Assert.AreEqual (FormViewMode.Edit, copy.CurrentMode, "ControlStateCurrentMode");
 
 		}
 
@@ -862,7 +979,9 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
+#if TARGET_JVM //BUG #6484
 		[Category ("NotWorking")]
+#endif
 		public void FormView_BubbleEvent ()
 		{
 			FormViewCommandEventArgs com;
@@ -872,6 +991,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			fv.AllowPaging = true;
 			fv.DataSource = myds;
 			page.Controls.Add (fv);
+			fv.DataBind ();
 			ResetEvents ();
 			fv.ItemCommand += new FormViewCommandEventHandler (fv_ItemCommand);
 			fv.ItemDeleted += new FormViewDeletedEventHandler (fv_ItemDeleted);
@@ -880,7 +1000,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new FormViewCommandEventArgs (bt, new CommandEventArgs ("Delete", null));
 			Assert.AreEqual (false, itemDeleting, "BeforeDeleteCommandBubbleEvent");
 			Assert.AreEqual (false, itemCommand, "BeforeDeleteBubbleEvent");
-			fv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (fv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Delete");
 			Assert.AreEqual (true, itemDeleting, "AfterDeleteBubbleEvent");
 			Assert.AreEqual (true, itemCommand, "AfterDeleteCommandBubbleEvent");
 
@@ -892,7 +1012,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new FormViewCommandEventArgs (bt, new CommandEventArgs ("Insert", null));
 			Assert.AreEqual (false, itemCommand, "BeforeInsertCommandBubbleEvent");
 			Assert.AreEqual (false, itemInserting, "BeforeInsertBubbleEvent");
-			fv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (fv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Insert");
 			Assert.AreEqual (true, itemCommand, "AfterInsertCommandBubbleEvent");
 			Assert.AreEqual (true, itemInserting, "AfterInsertBubbleEvent");
 
@@ -904,7 +1024,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new FormViewCommandEventArgs (bt, new CommandEventArgs ("Update", null));
 			Assert.AreEqual (false, itemUpdating, "BeforeUpdateEvent");
 			Assert.AreEqual (false, itemCommand, "BeforeUpdateCommandEvent");
-			fv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (fv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Update");
 			Assert.AreEqual (true, itemCommand, "AfterUpdateCommandBubbleEvent");
 			Assert.AreEqual (true, itemUpdating, "AfterUpdateBubbleEvent");
 
@@ -915,7 +1035,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new FormViewCommandEventArgs (bt, new CommandEventArgs ("Cancel", null));
 			Assert.AreEqual (false, itemCommand, "BeforeCancelCommandBubbleEvent");
 			Assert.AreEqual (false, modeChanging, "BeforeCancelBubbleEvent");
-			fv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (fv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Cancel");
 			Assert.AreEqual (true, itemCommand, "AfterCancelCommandBubbleEvent");
 			Assert.AreEqual (true, modeChanging, "AfterCancelBubbleEvent");
 
@@ -925,7 +1045,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new FormViewCommandEventArgs (bt, new CommandEventArgs ("Edit", null));
 			Assert.AreEqual (false, itemCommand, "BeforeEditCommandBubbleEvent");
 			Assert.AreEqual (false, modeChanging, "BeforeEditBubbleEvent");
-			fv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (fv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Edit");
 			Assert.AreEqual (true, itemCommand, "AfterEditCommandBubbleEvent");
 			Assert.AreEqual (true, modeChanging, "AfterEditBubbleEvent");
 
@@ -935,7 +1055,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new FormViewCommandEventArgs (bt, new CommandEventArgs ("New", null));
 			Assert.AreEqual (false, itemCommand, "BeforeNewCommandBubbleEvent");
 			Assert.AreEqual (false, modeChanging, "BeforeNewBubbleEvent");
-			fv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (fv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - New");
 			Assert.AreEqual (true, itemCommand, "AfterNewCommandBubbleEvent");
 			Assert.AreEqual (true, modeChanging, "AfterNewBubbleEvent");
 
@@ -945,9 +1065,10 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new FormViewCommandEventArgs (bt, new CommandEventArgs ("Page", null));
 			Assert.AreEqual (false, itemCommand, "BeforePageCommandBubbleEvent");
 			Assert.AreEqual (false, pageIndexChanging, "BeforePageBubbleEvent");
-			fv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (fv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Page Index default");
 			Assert.AreEqual (true, itemCommand, "AfterPageCommandBubbleEvent");
 			Assert.AreEqual (true, pageIndexChanging, "AfterPageBubbleEvent");
+			Assert.AreEqual (-1, newPageIndex, "PageIndex");
 
 			//Next Page
 			itemCommand = false;
@@ -955,7 +1076,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new FormViewCommandEventArgs (bt, new CommandEventArgs ("Page", "Next"));
 			Assert.AreEqual (false, itemCommand, "BeforeNextPageCommandBubbleEvent");
 			Assert.AreEqual (false, pageIndexChanging, "BeforeNextPageBubbleEvent");
-			fv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (fv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Next Page");
 			Assert.AreEqual (true, itemCommand, "AfterNextPageCommandBubbleEvent");
 			Assert.AreEqual (true, pageIndexChanging, "AfterNextPageBubbleEvent");
 			Assert.AreEqual (1, newPageIndex, "NextPageIndex");
@@ -966,7 +1087,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new FormViewCommandEventArgs (bt, new CommandEventArgs ("Page", "Prev"));
 			Assert.AreEqual (false, itemCommand, "BeforePrevPageCommandBubbleEvent");
 			Assert.AreEqual (false, pageIndexChanging, "BeforePrevPageBubbleEvent");
-			fv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (fv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Prev Page");
 			Assert.AreEqual (true, itemCommand, "AfterPrevPageCommandBubbleEvent");
 			Assert.AreEqual (true, pageIndexChanging, "AfterPrevPageBubbleEvent");
 			Assert.AreEqual (-1, newPageIndex, "PrevPageIndex");
@@ -977,7 +1098,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new FormViewCommandEventArgs (bt, new CommandEventArgs ("Page", "First"));
 			Assert.AreEqual (false, itemCommand, "BeforeFirstPageCommandBubbleEvent");
 			Assert.AreEqual (false, pageIndexChanging, "BeforeFirstPageBubbleEvent");
-			fv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (fv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - First Page");
 			Assert.AreEqual (true, itemCommand, "AfterFirstPageCommandBubbleEvent");
 			Assert.AreEqual (true, pageIndexChanging, "AfterFirstPageBubbleEvent");
 			Assert.AreEqual (0, newPageIndex, "FirstPageIndex");
@@ -988,16 +1109,15 @@ namespace MonoTests.System.Web.UI.WebControls
 			com = new FormViewCommandEventArgs (bt, new CommandEventArgs ("Page", "Last"));
 			Assert.AreEqual (false, itemCommand, "BeforeLastPageCommandBubbleEvent");
 			Assert.AreEqual (false, pageIndexChanging, "BeforeLastPageBubbleEvent");
-			fv.DoOnBubbleEvent (bt, com);
+			Assert.IsTrue (fv.DoOnBubbleEvent (bt, com), "OnBubbleEvent - Last Page");
 			Assert.AreEqual (true, itemCommand, "AfterLastPageCommandBubbleEvent");
 			Assert.AreEqual (true, pageIndexChanging, "AfterLastPageBubbleEvent");
-			Assert.AreEqual (-1, newPageIndex, "FirstPageIndex");
+			Assert.AreEqual (5, newPageIndex, "FirstPageIndex");
 
 		}
 
 
 		[Test]
-		[Category ("NotWorking")]
 		public void FormView_Events ()
 		{
 			ResetEvents ();
@@ -1121,11 +1241,11 @@ namespace MonoTests.System.Web.UI.WebControls
 		{
 			pageIndexChanging = true;
 			newPageIndex = e.NewPageIndex;
+			e.NewPageIndex = -1;
 		}
 
 		//Exceptions		
 		[Test]
-		[Category ("NotWorking")]
 		[ExpectedException (typeof (ArgumentOutOfRangeException))]
 		public void CellPaddingException ()
 		{
@@ -1134,7 +1254,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		[ExpectedException (typeof (ArgumentOutOfRangeException))]
 		public void CellSpacingException ()
 		{
@@ -1143,7 +1262,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		[ExpectedException (typeof (ArgumentOutOfRangeException))]
 		public void PageIndexException ()
 		{
@@ -1152,7 +1270,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		[ExpectedException (typeof (HttpException))]
 		public void InsertItemException ()
 		{
@@ -1161,7 +1278,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		[ExpectedException (typeof (HttpException))]
 		public void UpdateItemException ()
 		{
@@ -1171,6 +1287,7 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		
 		[Test]
+		[Category ("NotDotNet")] //TODO
 		public void PageIndex ()
 		{
 			ObjectDataSource ds = new ObjectDataSource ();
@@ -1266,18 +1383,10 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 		[TestFixtureTearDown]
 		public void TearDown ()
 		{
-			Thread.Sleep (100);
-		        //WebTest.Unload ();
+			WebTest.Unload ();
 		}
-	
-		//[TestFixtureStartUp]
-		//public void StartUp ()
-		//{
-		//	WebTest.CopyResource (GetType (), "FormView.aspx", "FormView.aspx");
-		//}
 
 		[Test]
-		[Category ("NotWorking")]
 		[Category("NunitWeb")]
 		public void FormViewCssClass ()
 		{
@@ -1296,7 +1405,10 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 
 
 		[Test]
+		[Category ("NunitWeb")]
+#if TARGET_JVM //BUG #6518
 		[Category ("NotWorking")]
+#endif
 		public void FormView_RenderSimpleTemplate()
 		{
 			string RenderedPageHtml = new WebTest ("FormViewTest1.aspx").Run ();			
@@ -1320,7 +1432,10 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 		}
 
 		[Test]
+		[Category ("NunitWeb")]
+#if TARGET_JVM //BUG #6518
 		[Category ("NotWorking")]
+#endif
 		public void FormView_RenderFooterAndPager()
 		{
 			string RenderedPageHtml = new WebTest ("FormViewTest1.aspx").Run ();			
@@ -1348,7 +1463,10 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 		}
 
 		[Test]
+		[Category ("NunitWeb")]
+#if TARGET_JVM //BUG #6518
 		[Category ("NotWorking")]
+#endif
 		public void FormView_RenderWithHeader()
 		{
 			string RenderedPageHtml = new WebTest ("FormViewTest1.aspx").Run ();
@@ -1371,7 +1489,10 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 
 
 		[Test]
+		[Category ("NunitWeb")]
+#if TARGET_JVM //BUG #6518
 		[Category ("NotWorking")]
+#endif
 		public void FormView_Render ()
 		{
 			string RenderedPageHtml = new WebTest ("FormViewTest1.aspx").Run ();
@@ -1407,7 +1528,8 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 		
 
 		[Test]
-		[Category ("NotWorking")]
+		[Category ("NotDotNet")] // becaue Naming container: use "FormView1$....." for DotNet
+		[Category ("NunitWeb")]
 		public void FormView_EditPostback ()
 		{
 			WebTest t = new WebTest ("FormViewInsertEditDelete.aspx");
@@ -1441,7 +1563,7 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 			FormRequest fr = new FormRequest (t.Response, "form1");
 			fr.Controls.Add ("__EVENTTARGET");
 			fr.Controls.Add ("__EVENTARGUMENT");
-			fr.Controls["__EVENTTARGET"].Value = "FormView1$EditButton";
+			fr.Controls["__EVENTTARGET"].Value = "FormView1:EditButton";
 			fr.Controls["__EVENTARGUMENT"].Value = "";
 			t.Request = fr;
 			pageHTML = t.Run ();
@@ -1472,12 +1594,12 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 			fr = new FormRequest (t.Response, "form1");
 			fr.Controls.Add ("__EVENTTARGET");
 			fr.Controls.Add ("__EVENTARGUMENT");
-			fr.Controls.Add ("FormView1$FNameEdit");
-			fr.Controls.Add ("FormView1$LNameEdit");
-			fr.Controls["__EVENTTARGET"].Value = "FormView1$UpdateButton";
+			fr.Controls.Add ("FormView1:FNameEdit");
+			fr.Controls.Add ("FormView1:LNameEdit");
+			fr.Controls["__EVENTTARGET"].Value = "FormView1:UpdateButton";
 			fr.Controls["__EVENTARGUMENT"].Value = "";			
-			fr.Controls["FormView1$FNameEdit"].Value = "Merav";
-			fr.Controls["FormView1$LNameEdit"].Value = "Test";			
+			fr.Controls["FormView1:FNameEdit"].Value = "Merav";
+			fr.Controls["FormView1:LNameEdit"].Value = "Test";			
 			t.Request = fr;
 			pageHTML = t.Run ();
 			newHtml = pageHTML.Substring (pageHTML.IndexOf ("start") + 5, pageHTML.IndexOf ("end") - pageHTML.IndexOf ("start") - 5);
@@ -1508,7 +1630,7 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 			fr = new FormRequest (t.Response, "form1");
 			fr.Controls.Add ("__EVENTTARGET");
 			fr.Controls.Add ("__EVENTARGUMENT");
-			fr.Controls["__EVENTTARGET"].Value = "FormView1$EditButton";
+			fr.Controls["__EVENTTARGET"].Value = "FormView1:EditButton";
 			fr.Controls["__EVENTARGUMENT"].Value = "";			
 			t.Request = fr;
 			pageHTML = t.Run ();
@@ -1520,11 +1642,11 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 			fr = new FormRequest (t.Response, "form1");
 			fr.Controls.Add ("__EVENTTARGET");
 			fr.Controls.Add ("__EVENTARGUMENT");
-			fr.Controls.Add ("FormView1$FNameEdit");
-			fr.Controls.Add ("FormView1$LNameEdit");
-			fr.Controls["FormView1$FNameEdit"].Value = "EditFirstName";
-			fr.Controls["FormView1$LNameEdit"].Value = "EditLastName";
-			fr.Controls["__EVENTTARGET"].Value = "FormView1$CancelUpdateButton";
+			fr.Controls.Add ("FormView1:FNameEdit");
+			fr.Controls.Add ("FormView1:LNameEdit");
+			fr.Controls["FormView1:FNameEdit"].Value = "EditFirstName";
+			fr.Controls["FormView1:LNameEdit"].Value = "EditLastName";
+			fr.Controls["__EVENTTARGET"].Value = "FormView1:CancelUpdateButton";
 			fr.Controls["__EVENTARGUMENT"].Value = "";
 			t.Request = fr;
 			pageHTML = t.Run ();
@@ -1555,7 +1677,8 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 		}
 
 		[Test]
-		[Category ("NotWorking")]
+		[Category ("NotDotNet")] // becaue Naming container: use "FormView1$....." for DotNet
+		[Category ("NunitWeb")] 
 		public void FormView_InsertPostback ()
 		{
 			WebTest t = new WebTest ("FormViewInsertEditDelete.aspx");
@@ -1567,7 +1690,7 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 			FormRequest fr = new FormRequest (t.Response, "form1"); 
 			fr.Controls.Add ("__EVENTTARGET");
 			fr.Controls.Add ("__EVENTARGUMENT");			
-			fr.Controls["__EVENTTARGET"].Value = "FormView1$NewButton";
+			fr.Controls["__EVENTTARGET"].Value = "FormView1:NewButton";
 			fr.Controls["__EVENTARGUMENT"].Value = "";						
 			t.Request = fr;
 			pageHTML = t.Run ();
@@ -1597,13 +1720,13 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 			fr = new FormRequest (t.Response, "form1");
 			fr.Controls.Add ("__EVENTTARGET");
 			fr.Controls.Add ("__EVENTARGUMENT");
-			fr.Controls.Add ("FormView1$IDInsert");
-			fr.Controls.Add ("FormView1$FNameInsert");
-			fr.Controls.Add ("FormView1$LNameInsert");
-			fr.Controls["FormView1$IDInsert"].Value = "33";
-			fr.Controls["FormView1$FNameInsert"].Value = "InsertFirstName";
-			fr.Controls["FormView1$LNameInsert"].Value ="InsertLastName";
-			fr.Controls["__EVENTTARGET"].Value = "FormView1$InsertButton";
+			fr.Controls.Add ("FormView1:IDInsert");
+			fr.Controls.Add ("FormView1:FNameInsert");
+			fr.Controls.Add ("FormView1:LNameInsert");
+			fr.Controls["FormView1:IDInsert"].Value = "33";
+			fr.Controls["FormView1:FNameInsert"].Value = "InsertFirstName";
+			fr.Controls["FormView1:LNameInsert"].Value ="InsertLastName";
+			fr.Controls["__EVENTTARGET"].Value = "FormView1:InsertButton";
 			fr.Controls["__EVENTARGUMENT"].Value = "";
 			t.Request = fr;
 			pageHTML = t.Run ();			
@@ -1628,12 +1751,26 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 		}		
 
 		[Test]
-		[Category ("NotWorking")]
+		[Category ("NotDotNet")] // becaue Naming container: use "FormView1$DeleteButton" for DotNet
+		[Category ("NunitWeb")]
 		public void FormView_DeleteAndEmptyTemplatePostback ()
 		{
 			WebTest t = new WebTest ("FormViewInsertEditDelete.aspx");
 			string pageHTML = t.Run ();
+			
 			//Before Delete
+			//
+			// The following line fails, it returns "false" instead of true, because
+			// the page actually contains the value "1002", not 1001.
+			//
+//Failures: 1) MonoTests.System.Web.UI.WebControls.FormViewTest.FormView_DeleteAndEmptyTemplatePostback : BeforeDelete1 ^M
+//        expected:<True>^M
+//         but was:<False>^M
+//  at MonoTests.System.Web.UI.WebControls.FormViewTest.FormView_DeleteAndEmptyTemplatePostback () [0x00018] in /home/cvs/mcs/class/System.Web/Test/System.Web.UI.WebControls/FormViewTest.cs:1769
+//  at <0x00000> <unknown method>
+//  at (wrapper managed-to-native) System.Reflection.MonoMethod:InternalInvoke (object,object[])
+//  at System.Reflection.MonoMethod.Invoke (System.Object obj, BindingFlags invokeAttr, System.Reflection.Binder binder, System.Object[] parameters, System.Globalization.CultureInfo culture) [0x00056] in /home/cvs/mcs/class/corlib/System.Reflection/MonoMethod.cs:143
+//			
 			Assert.AreEqual (true, pageHTML.Contains ("1001"), "BeforeDelete1");
 			Assert.AreEqual (true, pageHTML.Contains ("Mahesh"), "BeforeDelete2");
 			Assert.AreEqual (true, pageHTML.Contains ("Chand"), "BeforeDelete3");
@@ -1642,7 +1779,7 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 			FormRequest fr = new FormRequest (t.Response, "form1");
 			fr.Controls.Add ("__EVENTTARGET");
 			fr.Controls.Add ("__EVENTARGUMENT");
-			fr.Controls["__EVENTTARGET"].Value = "FormView1$DeleteButton";
+			fr.Controls["__EVENTTARGET"].Value = "FormView1:DeleteButton";
 			fr.Controls["__EVENTARGUMENT"].Value = "";
 			t.Request = fr;
 			pageHTML = t.Run ();			
@@ -1657,7 +1794,7 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 			fr = new FormRequest (t.Response, "form1");
 			fr.Controls.Add ("__EVENTTARGET");
 			fr.Controls.Add ("__EVENTARGUMENT");
-			fr.Controls["__EVENTTARGET"].Value = "FormView1$DeleteButton";
+			fr.Controls["__EVENTTARGET"].Value = "FormView1:DeleteButton";
 			fr.Controls["__EVENTARGUMENT"].Value = "";
 			t.Request = fr;
 			pageHTML = t.Run ();
@@ -1671,22 +1808,155 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 			fr = new FormRequest (t.Response, "form1");
 			fr.Controls.Add ("__EVENTTARGET");
 			fr.Controls.Add ("__EVENTARGUMENT");
-			fr.Controls["__EVENTTARGET"].Value = "FormView1$DeleteButton";
+			fr.Controls["__EVENTTARGET"].Value = "FormView1:DeleteButton";
 			fr.Controls["__EVENTARGUMENT"].Value = "";
 			t.Request = fr;
 			pageHTML = t.Run ();			
 			Assert.AreEqual (true, pageHTML.Contains ("FormView1_Label1"), "EmptyTemplateTest1"); 
 			Assert.AreEqual (true, pageHTML.Contains ("The Database is empty"), "EmptyTemplateTest2");
 		}
-		
-		
 
+		[Test]
+		public void FormView_CurrentMode () {
+			FormView view = new FormView ();
+			view.DefaultMode = FormViewMode.Insert;
+			Assert.AreEqual (FormViewMode.Insert, view.CurrentMode, "FormView_CurrentMode#1");
+			view.ChangeMode (FormViewMode.Edit);
+			Assert.AreEqual (FormViewMode.Edit, view.CurrentMode, "FormView_CurrentMode#2");
+		}
 
+		[Test]
+#if TARGET_JVM //BUG #6484
+		[Category("NotWorking")]
+#endif
+		public void FormView_CreateDataSourceSelectArguments2 () {
+			DataSourceView view;
+			Page p = new Page ();
+
+			Poker dv = new Poker ();
+			p.Controls.Add (dv);
+
+			ObjectDataSource data = new ObjectDataSource ();
+			data.TypeName = typeof (DataSourceObject).AssemblyQualifiedName;
+			data.SelectMethod = "GetList";
+			data.SortParameterName = "sortExpression";
+			DataSourceSelectArguments arg;
+			p.Controls.Add (data);
+
+			dv.DataSource = data;
+			dv.DataBind ();
+
+			arg = dv.DoCreateDataSourceSelectArguments ();
+			Assert.IsTrue (arg.Equals (DataSourceSelectArguments.Empty), "Default");
+
+			dv.AllowPaging = true;
+			dv.PageIndex = 2;
+			arg = dv.DoCreateDataSourceSelectArguments ();
+			view = dv.DoGetData ();
+			Assert.IsFalse (view.CanPage);
+			Assert.IsTrue (view.CanRetrieveTotalRowCount);
+			Assert.IsTrue (arg.Equals (DataSourceSelectArguments.Empty), "AllowPaging = true, CanPage = false, CanRetrieveTotalRowCount = true");
+
+			// make DataSourceView.CanPage = true
+			data.EnablePaging = true;
+
+			arg = dv.DoCreateDataSourceSelectArguments ();
+			view = dv.DoGetData ();
+			Assert.IsTrue (view.CanPage);
+			Assert.IsFalse (view.CanRetrieveTotalRowCount);
+			Assert.IsTrue (arg.Equals (new DataSourceSelectArguments (2, -1)), "AllowPaging = true, CanPage = true, CanRetrieveTotalRowCount = false");
+
+			dv.AllowPaging = false;
+			arg = dv.DoCreateDataSourceSelectArguments ();
+			Assert.IsTrue (arg.Equals (DataSourceSelectArguments.Empty), "AllowPaging = false, CanPage = true, CanRetrieveTotalRowCount = false");
+
+			// make DataSourceView.CanRetrieveTotalRowCount = true
+			data.SelectCountMethod = "GetCount";
+
+			arg = dv.DoCreateDataSourceSelectArguments ();
+			Assert.IsTrue (arg.Equals (DataSourceSelectArguments.Empty), "AllowPaging = false, CanPage = true, CanRetrieveTotalRowCount = true");
+
+			dv.AllowPaging = true;
+			arg = dv.DoCreateDataSourceSelectArguments ();
+			DataSourceSelectArguments arg1 = new DataSourceSelectArguments (2, 1);
+			arg1.RetrieveTotalRowCount = true;
+			view = dv.DoGetData ();
+			Assert.IsTrue (view.CanPage);
+			Assert.IsTrue (view.CanRetrieveTotalRowCount);
+			Assert.IsTrue (arg.Equals (arg1), "AllowPaging = true, CanPage = true, CanRetrieveTotalRowCount = true");
+		}
+
+		[Test]
+#if TARGET_JVM //BUG #6484
+		[Category ("NotWorking")]
+#endif
+		public void FormView_GetPostBackOptions () {
+			FormView fv = new FormView ();
+			fv.Page = new Page ();
+			IButtonControl btn = new Button ();
+			btn.CausesValidation = false;
+			Assert.IsFalse (btn.CausesValidation);
+			Assert.AreEqual (String.Empty, btn.CommandName);
+			Assert.AreEqual (String.Empty, btn.CommandArgument);
+			Assert.AreEqual (String.Empty, btn.PostBackUrl);
+			Assert.AreEqual (String.Empty, btn.ValidationGroup);
+			PostBackOptions options = ((IPostBackContainer) fv).GetPostBackOptions (btn);
+			Assert.IsFalse (options.PerformValidation);
+			Assert.IsFalse (options.AutoPostBack);
+			Assert.IsFalse (options.TrackFocus);
+			Assert.IsTrue (options.ClientSubmit);
+			Assert.IsTrue (options.RequiresJavaScriptProtocol);
+			Assert.AreEqual ("$", options.Argument);
+			Assert.AreEqual (null, options.ActionUrl);
+			Assert.AreEqual (null, options.ValidationGroup);
+			Assert.IsTrue (object.ReferenceEquals (options.TargetControl, fv));
+
+			btn.ValidationGroup = "VG";
+			btn.CommandName = "CMD";
+			btn.CommandArgument = "ARG";
+			btn.PostBackUrl = "Page.aspx";
+			Assert.IsFalse (btn.CausesValidation);
+			Assert.AreEqual ("CMD", btn.CommandName);
+			Assert.AreEqual ("ARG", btn.CommandArgument);
+			Assert.AreEqual ("Page.aspx", btn.PostBackUrl);
+			Assert.AreEqual ("VG", btn.ValidationGroup);
+			options = ((IPostBackContainer) fv).GetPostBackOptions (btn);
+			Assert.IsFalse (options.PerformValidation);
+			Assert.IsFalse (options.AutoPostBack);
+			Assert.IsFalse (options.TrackFocus);
+			Assert.IsTrue (options.ClientSubmit);
+			Assert.IsTrue (options.RequiresJavaScriptProtocol);
+			Assert.AreEqual ("CMD$ARG", options.Argument);
+			Assert.AreEqual (null, options.ActionUrl);
+			Assert.AreEqual (null, options.ValidationGroup);
+		}
+
+		[Test]
+		[ExpectedException (typeof (InvalidOperationException))]
+		public void FormView_GetPostBackOptions_CausesValidation () {
+			FormView fv = new FormView ();
+			fv.Page = new Page ();
+			IButtonControl btn = new Button ();
+			Assert.IsTrue (btn.CausesValidation);
+			Assert.AreEqual (String.Empty, btn.CommandName);
+			Assert.AreEqual (String.Empty, btn.CommandArgument);
+			Assert.AreEqual (String.Empty, btn.PostBackUrl);
+			Assert.AreEqual (String.Empty, btn.ValidationGroup);
+			PostBackOptions options = ((IPostBackContainer) fv).GetPostBackOptions (btn);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentNullException))]
+		public void FormView_GetPostBackOptions_Null_Argument () {
+			FormView fv = new FormView ();
+			fv.Page = new Page ();
+			PostBackOptions options = ((IPostBackContainer) fv).GetPostBackOptions (null);
+		}
 	}
-
+	
 	public class TestMyData
 	{
-		static IList<int> str = (IList<int>) new int[] { 1, 2, 3, 4, 5, 6 };
+		static IList<int> str = new List<int> (new int[] { 1, 2, 3, 4, 5, 6 });
 
 
 		public static IList<int> GetMyList ()
@@ -1712,7 +1982,7 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 		}
 
 	}
-
+	
 	public class MyTemplate : ITemplate
 	{
 
@@ -1732,6 +2002,140 @@ CommandEventArgs cargs = new CommandEventArgs ("Page", "Prev");
 
 #endregion
 	}
+
+public class FormViewDataObject
+	{
+
+		public static DataTable ds = CreateDataTable();
+
+		public static DataTable Select()
+		{
+
+			return ds;
+
+		}
+
+
+
+		public static DataTable Delete(string ID, string FName, string LName)
+		{
+
+			DataRow dr = ds.Rows.Find(ID);
+
+			if (dr != null)
+			{
+
+				ds.Rows.Remove(dr);
+
+			}
+
+			return ds;
+
+		}
+
+
+
+		public static DataTable Insert(string ID, string FName, string LName)
+		{
+
+			DataRow dr = ds.NewRow();
+			dr["ID"] = ID;
+			dr["FName"] = FName;
+			dr["LName"] = LName;
+			ds.Rows.Add(dr);
+			return ds;
+		}
+
+
+
+		public static DataTable Update(string ID, string FName, string LName)
+		{
+			DataRow dr = ds.Rows.Find(ID);
+			if (dr == null)
+			{
+				Label lbl = new Label();
+				lbl.Text = "ID doesn't exist ";
+				return ds;
+			}
+			dr["FName"] = FName;
+			dr["LName"] = LName;
+			return ds;
+
+		}
+
+
+
+		public static DataTable CreateDataTable()
+		{
+
+			DataTable aTable = new DataTable("A");
+			DataColumn dtCol;
+			DataRow dtRow;
+
+			// Create ID column and add to the DataTable.
+
+			dtCol = new DataColumn();
+			dtCol.DataType = Type.GetType("System.Int32");
+			dtCol.ColumnName = "ID";
+			dtCol.AutoIncrement = true;
+			dtCol.Caption = "ID";
+			dtCol.ReadOnly = true;
+			dtCol.Unique = true;
+			aTable.Columns.Add(dtCol);
+
+
+
+			// Create Name column and add to the table
+
+			dtCol = new DataColumn();
+			dtCol.DataType = Type.GetType("System.String");
+			dtCol.ColumnName = "FName";
+			dtCol.AutoIncrement = false;
+			dtCol.Caption = "First Name";
+			dtCol.ReadOnly = false;
+			dtCol.Unique = false;
+			aTable.Columns.Add(dtCol);
+
+
+
+			// Create Last Name column and add to the table.
+
+			dtCol = new DataColumn();
+			dtCol.DataType = Type.GetType("System.String");
+			dtCol.ColumnName = "LName";
+			dtCol.AutoIncrement = false;
+			dtCol.Caption = "Last Name";
+			dtCol.ReadOnly = false;
+			dtCol.Unique = false;
+			aTable.Columns.Add(dtCol);
+
+
+
+			// Create three rows to the table
+
+			dtRow = aTable.NewRow();
+			dtRow["ID"] = 1001;
+			dtRow["FName"] = "Mahesh";
+			dtRow["LName"] = "Chand";
+			aTable.Rows.Add(dtRow);
+
+			dtRow = aTable.NewRow();
+			dtRow["ID"] = 1002;
+			dtRow["FName"] = "Melanie";
+			dtRow["LName"] = "Talmadge";
+			aTable.Rows.Add(dtRow);
+
+			dtRow = aTable.NewRow();
+			dtRow["ID"] = 1003;
+			dtRow["FName"] = "Vinay";
+			dtRow["LName"] = "Bansal";
+			aTable.Rows.Add(dtRow);
+
+			aTable.PrimaryKey = new DataColumn[] { aTable.Columns["ID"] };
+			return aTable;
+
+		}
+	}
 }
 
 
