@@ -186,7 +186,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void ObjectDataSource_NotWorkingDefaultProperties ()
 		{
 			ObjectDataSourcePoker ods = new ObjectDataSourcePoker ();
@@ -246,7 +245,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void ObjectDataSource_NotWorkingAssignToDefaultProperties ()
 		{
 			ObjectDataSourcePoker ods = new ObjectDataSourcePoker ();
@@ -267,11 +265,10 @@ namespace MonoTests.System.Web.UI.WebControls
 		//ViewState
 
 		[Test]
-		[Category ("NotWorking")]
 		public void ObjectDataSource_ViewState ()
 		{
 			ObjectDataSourcePoker  ods = new ObjectDataSourcePoker ();
-			InitObjectDataSource (ods,"");	
+			//InitObjectDataSource (ods,"");	
 			ObjectDataSourcePoker copy = new ObjectDataSourcePoker ();
 			FormParameter p = new FormParameter ("ID", "TextBox1");
 			p.DefaultValue = "1002";
@@ -296,10 +293,30 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		[Category ("NunitWeb")]
+		[Category("NotWorking")]			
 		public void ObjectDataSource_ConvertNullToDBNull ()
 		{
+// 7) MonoTests.System.Web.UI.WebControls.ObjectDataSourceTest.ObjectDataSource_ConvertNullToDBNull : NUnit.Framework.AssertionException: ConvertNullToDBNull ^M
+//         expected:<True>^M
+//          but was:<False>
+//   at NUnit.Framework.Assert.Fail (System.String message, System.Object[] args) [0x00000]
+//   at NUnit.Framework.Assert.Fail (System.String message) [0x00000]
+//   at NUnit.Framework.Assert.FailNotEquals (System.Object expected, System.Object actual, System.String message, System.Object[] args) [0x00000]
+//   at NUnit.Framework.Assert.AreEqual (System.Object expected, System.Object actual, System.String message, System.Object[] args) [0x00000]
+//   at NUnit.Framework.Assert.AreEqual (System.Object expected, System.Object actual, System.String message) [0x00000]
+//   at MonoTests.System.Web.UI.WebControls.ObjectDataSourceTest.ConvertNullToDBNull (System.Web.UI.Page p) [0x00000]
+//   at (wrapper delegate-invoke) System.MulticastDelegate:invoke_void_Page (System.Web.UI.Page)
+//   at MonoTests.SystemWeb.Framework.PageInvoker.Invoke (MonoTests.SystemWeb.Framework.PageDelegate callback) [0x00000]
+//   ----> NUnit.Framework.AssertionException : ConvertNullToDBNull ^M
+//         expected:<True>^M
+//          but was:<False>^M
+//   at System.Runtime.Remoting.Proxies.RealProxy.PrivateInvoke (System.Runtime.Remoting.Proxies.RealProxy rp, IMessage msg, System.Exception& exc, System.Object[]& out_args) [0x00188] in /home/cvs/mcs/class/corlib/System.Runtime.Remoting.Proxies/RealProxy.cs:221
+// --AssertionException
+//   at MonoTests.System.Web.UI.WebControls.ObjectDataSourceTest.ConvertNullToDBNull (System.Web.UI.Page p) [0x00000]
+//   at (wrapper delegate-invoke) System.MulticastDelegate:invoke_void_Page (System.Web.UI.Page)
+//   at MonoTests.SystemWeb.Framework.PageInvoker.Invoke (MonoTests.SystemWeb.Framework.PageDelegate callback) [0x00000]
+// 
 			string html = new WebTest (PageInvoker.CreateOnLoad (
 				new PageDelegate (ConvertNullToDBNull))).Run ();
 		}
@@ -315,14 +332,15 @@ namespace MonoTests.System.Web.UI.WebControls
 				ods.Delete ();
 			}
 			catch (Exception ex) {
-				Assert.AreEqual (true,ex.Message.Contains ("type 'System.DBNull' cannot be converted to type 'System.String'"), "ConvertNullToDBNull");
+				Assert.AreEqual (true,
+					ex.Message.Contains ("type 'System.DBNull' cannot be converted to type 'System.String'") || // dotnet
+					ex.Message.Contains ("Value cannot be null."), "ConvertNullToDBNull"); // mono
 				dbnull = true;
 			}
 			Assert.AreEqual (true, dbnull, "ConvertNullToDBNull2");
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		[Category ("NunitWeb")]
 		public void ObjectDataSource_FilterExpression ()
 		{
@@ -358,7 +376,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		[Category ("NunitWeb")]
 		public void ObjectDataSource_FilterParameter ()
 		{
@@ -397,7 +414,6 @@ namespace MonoTests.System.Web.UI.WebControls
 
 
 		[Test]
-		[Category ("NotWorking")]
 		[Category ("NunitWeb")]
 		public void ObjectDataSource_EnablePaging ()
 		{
@@ -458,7 +474,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		//public methods
 
 		[Test]
-		[Category ("NotWorking")]
 		[Category ("NunitWeb")]
 		public void ObjectDataSource_Delete ()
 		{
@@ -512,7 +527,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		[Category ("NunitWeb")]
 		public void ObjectDataSource_Select ()
 		{
@@ -541,7 +555,32 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
+		[Category ("NunitWeb")]
+		public void ObjectDataSource_Select_Cached ()
+		{
+			string html = new WebTest (PageInvoker.CreateOnLoad (
+				new PageDelegate (SelectMethodCached))).Run ();
+		}
+
+
+		public static void SelectMethodCached (Page p)
+		{
+			MyTableObject.ds = MyTableObject.CreateDataTable ();
+			ObjectDataSourcePoker ods = new ObjectDataSourcePoker ();
+			ods.EnableCaching = true;
+			InitObjectDataSource (ods, "");
+			p.Controls.Add (ods);
+			ods.Selecting += new ObjectDataSourceSelectingEventHandler (odc_Selecting);
+
+			selecting = false;
+			IEnumerable table = (IEnumerable) ods.Select ();
+			Assert.AreEqual (true, selecting, "AfterSelectingEvent");
+
+			selecting = false;
+			IEnumerable table2 = (IEnumerable) ods.Select ();
+			Assert.AreEqual (false, selecting, "AfterSelectingEvent");
+		}
+		[Test]
 		[Category ("NunitWeb")]
 		public void ObjectDataSource_Insert ()
 		{
@@ -570,7 +609,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		[Category ("NunitWeb")]
 		public void ObjectDataSource_Update ()
 		{
@@ -621,7 +659,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		// Tests for events Select,Update,Delete and Insert include in Select,Update,Delete and Insert methods tests.
 
 		[Test]
-		[Category ("NotWorking")]
 		[Category ("NunitWeb")]
 		public void ObjectDataSource_Events ()
 		{
@@ -719,7 +756,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		//Excpetions
 
 		[Test]
-		[Category ("NotWorking")]
 		[ExpectedException (typeof (NotSupportedException))]
 		[Category ("NunitWeb")]
 		public void ObjectDataSource_EnableCachingException ()
@@ -742,7 +778,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		[ExpectedException (typeof (NotSupportedException))]
 		[Category ("NunitWeb")]
 		public void ObjectDataSource_FilterExpressionException ()
@@ -905,4 +940,4 @@ namespace MonoTests.System.Web.UI.WebControls
 	}
 	
 }
-#endif
\ No newline at end of file
+#endif
