@@ -191,7 +191,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void DefaultsNotWorking ()
 		{
 			ObjectDataSource ds = new ObjectDataSource ();
@@ -253,7 +252,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void ViewState ()
 		{
 			// Note :
@@ -345,6 +343,27 @@ namespace MonoTests.System.Web.UI.WebControls
 			Assert.IsFalse (sql.CanUpdate, "A3");
 		}
 
+		[Test]
+		public void CanRetrieveTotalRowCount () {
+			ObjectDataSource ds = new ObjectDataSource ();
+			ObjectDataSourceView view = new ObjectDataSourceView (ds, "DefaultView", null);
+
+			Assert.IsFalse (view.CanPage, "CanPage#1");
+			Assert.IsTrue (view.CanRetrieveTotalRowCount, "CanRetrieveTotalRowCount#1");
+
+			view.EnablePaging = true;
+			Assert.IsTrue (view.CanPage, "CanPage#2");
+			Assert.IsFalse (view.CanRetrieveTotalRowCount, "CanRetrieveTotalRowCount#2");
+
+			view.SelectCountMethod = "SelectCountMethod";
+			Assert.IsTrue (view.CanPage, "CanPage#3");
+			Assert.IsTrue (view.CanRetrieveTotalRowCount, "CanRetrieveTotalRowCount#3");
+
+			view.EnablePaging = false;
+			Assert.IsFalse (view.CanPage, "CanPage#4");
+			Assert.IsTrue (view.CanRetrieveTotalRowCount, "CanRetrieveTotalRowCount#4");
+		}
+
 		[Test]
 		public void OldValuesParameterFormatString ()
 		{
@@ -370,13 +389,11 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		[Test]
 		[Category("NunitWeb")]
-		[Category ("NotWorking")]
 		public void DeleteMethod ()
 		{
 			WebTest t = new WebTest (PageInvoker.CreateOnLoad (delete));
 			string html = t.Run ();
-			string origin = @"<div></div>";
-			HtmlDiff.AssertAreEqual (origin, HtmlDiff.GetControlFromPageHtml (html), "ObjectDataSourceViewDelete");
+			Assert.AreEqual (-1, html.IndexOf("Yonik"), "ObjectDataSourceViewDelete");
 			WebTest.Unload ();
 		}
 
@@ -410,6 +427,8 @@ namespace MonoTests.System.Web.UI.WebControls
 			view = (ObjectDataSourceView) ds.DoGetView ("DefaultView");
 			view.Deleting += new ObjectDataSourceMethodEventHandler (Event);
 
+			DataSourceObject.InitDS ();
+
 			Hashtable table = new Hashtable ();
 			table.Add ("ID", "1001");
 			table.Add ("FName", "Yonik");
@@ -430,6 +449,26 @@ namespace MonoTests.System.Web.UI.WebControls
 		[Category ("NotWorking")]
 		public void SelectMethod ()
 		{
+// 8) MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.SelectMethod : NUnit.Framework.AssertionException: SelectedRowsCount ^M
+//         expected:<1>^M
+//          but was:<0>
+//   at NUnit.Framework.Assert.Fail (System.String message, System.Object[] args) [0x00000]
+//   at NUnit.Framework.Assert.Fail (System.String message) [0x00000]
+//   at NUnit.Framework.Assert.FailNotEquals (System.Object expected, System.Object actual, System.String message, System.Object[] args) [0x00000]
+//   at NUnit.Framework.Assert.AreEqual (Int32 expected, Int32 actual, System.String message, System.Object[] args) [0x00000]
+//   at NUnit.Framework.Assert.AreEqual (Int32 expected, Int32 actual, System.String message) [0x00000]
+//   at MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.select (System.Web.UI.Page p) [0x00000]
+//   at (wrapper delegate-invoke) System.MulticastDelegate:invoke_void_Page (System.Web.UI.Page)
+//   at MonoTests.SystemWeb.Framework.PageInvoker.Invoke (MonoTests.SystemWeb.Framework.PageDelegate callback) [0x00000]
+//   ----> NUnit.Framework.AssertionException : SelectedRowsCount ^M
+//         expected:<1>^M
+//          but was:<0>^M
+//   at System.Runtime.Remoting.Proxies.RealProxy.PrivateInvoke (System.Runtime.Remoting.Proxies.RealProxy rp, IMessage msg, System.Exception& exc, System.Object[]& out_args) [0x00188] in /home/cvs/mcs/class/corlib/System.Runtime.Remoting.Proxies/RealProxy.cs:221
+// --AssertionException
+//   at MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.select (System.Web.UI.Page p) [0x00000]
+//   at (wrapper delegate-invoke) System.MulticastDelegate:invoke_void_Page (System.Web.UI.Page)
+//   at MonoTests.SystemWeb.Framework.PageInvoker.Invoke (MonoTests.SystemWeb.Framework.PageDelegate callback) [0x00000]
+// 
 			WebTest t = new WebTest (PageInvoker.CreateOnLoad (select));
 			string html = t.Run ();
 			string origin = @"<div>
@@ -466,6 +505,8 @@ namespace MonoTests.System.Web.UI.WebControls
 			view = (ObjectDataSourceView) ds.DoGetView ("DefaultView");
 			view.Selecting += new ObjectDataSourceSelectingEventHandler (view_Selecting);
 
+			DataSourceObject.InitDS ();
+
 			DataView view1 = (DataView) view.Select (new DataSourceSelectArguments ());
 			Assert.AreEqual (1, view1.Count, "SelectedRowsCount");
 			Assert.AreEqual (1001, view1[0].Row["ID"], "SelectedRowsValue#1");
@@ -479,6 +520,20 @@ namespace MonoTests.System.Web.UI.WebControls
 		[Category ("NotWorking")]
 		public void SelectCountMethod ()
 		{
+// 9) MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.SelectCountMethod : ObjectDataSourceViewSelectCount ^M
+//         String lengths differ.  Expected length=389, but was length=11.^M
+//         Strings differ at index 6.^M
+//         ^M
+//         expected:<"<div><table cellspacing="0" rules="all" s...">^M
+//          but was:<"<div></div>">^M
+//         -----------------^^M
+//         ^M
+//   at MonoTests.stand_alone.WebHarness.HtmlDiff.AssertAreEqual (System.String origin, System.String derived, System.String msg) [0x0001b] in /home/cvs/mcs/class/System.Web/Test/mainsoft/MainsoftWebTest/NunitWebTest.cs:98
+//   at MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.SelectCountMethod () [0x00024] in /home/cvs/mcs/class/System.Web/Test/System.Web.UI.WebControls/ObjectDataSourceViewTest.cs:516
+//   at <0x00000> <unknown method>
+//   at (wrapper managed-to-native) System.Reflection.MonoMethod:InternalInvoke (object,object[])
+//   at System.Reflection.MonoMethod.Invoke (System.Object obj, BindingFlags invokeAttr, System.Reflection.Binder binder, System.Object[] parameters, System.Globalization.CultureInfo culture) [0x00056] in /home/cvs/mcs/class/corlib/System.Reflection/MonoMethod.cs:143
+// 
 			// This method will render grid view with paging 
 			// Note : ObjectDataSource will return page counter 5 hard coded
 			WebTest t = new WebTest (PageInvoker.CreateOnLoad (selectcount));
@@ -535,7 +590,6 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		[Test]
 		[Category ("NunitWeb")]
-		[Category ("NotWorking")]
 		public void InsertMethod ()
 		{
 			WebTest t = new WebTest (PageInvoker.CreateOnLoad (insert));
@@ -601,9 +655,23 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		[Test]
 		[Category ("NunitWeb")]
-		[Category ("NotWorking")]
+		[Category("NotWorking")]
 		public void UpdateMethod ()
 		{
+// 10) MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.UpdateMethod : ObjectDataSourceViewUpdate ^M
+//         String lengths are both 4.^M
+//         Strings differ at index 3.^M
+//         ^M
+//         expected:<"1001">^M
+//          but was:<"1000">^M
+//         --------------^^M
+//         ^M
+//   at MonoTests.stand_alone.WebHarness.HtmlDiff.AssertAreEqual (System.String origin, System.String derived, System.String msg) [0x0001b] in /home/cvs/mcs/class/System.Web/Test/mainsoft/MainsoftWebTest/NunitWebTest.cs:98
+//   at MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.UpdateMethod () [0x00024] in /home/cvs/mcs/class/System.Web/Test/System.Web.UI.WebControls/ObjectDataSourceViewTest.cs:631
+//   at <0x00000> <unknown method>
+//   at (wrapper managed-to-native) System.Reflection.MonoMethod:InternalInvoke (object,object[])
+//   at System.Reflection.MonoMethod.Invoke (System.Object obj, BindingFlags invokeAttr, System.Reflection.Binder binder, System.Object[] parameters, System.Globalization.CultureInfo culture) [0x00056] in /home/cvs/mcs/class/corlib/System.Reflection/MonoMethod.cs:143
+// 
 			WebTest t = new WebTest (PageInvoker.CreateOnLoad (update));
 			string html = t.Run ();
 			string origin = @"<div>
@@ -651,6 +719,8 @@ namespace MonoTests.System.Web.UI.WebControls
 			view = (ObjectDataSourceView) ds.DoGetView ("defaultView");
 			view.Updating += new ObjectDataSourceMethodEventHandler (Event);
 
+			DataSourceObject.InitDS ();
+
 			Hashtable table = new Hashtable ();
 			table.Add ("ID", "1001");
 			table.Add ("FName", "Yonik");
@@ -807,6 +877,10 @@ namespace MonoTests.System.Web.UI.WebControls
 	{
 		private static int maximumRows;
 		public static DataTable ds = CreateDataTable ();
+		public static void InitDS ()
+		{
+			ds = CreateDataTable ();
+		}
 		public static DataTable Select ()
 		{
 			return ds;
