@@ -641,7 +641,6 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		[Test]
 		[Category ("NunitWeb")]
-		[Category ("NotWorking")]
 		public void Wizard_SideBarRendering ()
 		{
 			WebTest t = new WebTest (PageInvoker.CreateOnPreInit (_SideBarRendering));
@@ -1182,7 +1181,6 @@ namespace MonoTests.System.Web.UI.WebControls
 
 
 		[Test]
-		[Category ("NotWorking")]
 		public void Wizard_AllowNavigationToStep ()
 		{
 			PokerWizard wizard = new PokerWizard ();
@@ -1229,7 +1227,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		public void Wizard_ControlState ()
 		{
 			PokerWizard wizard = new PokerWizard ();
@@ -1328,16 +1325,6 @@ namespace MonoTests.System.Web.UI.WebControls
 			eventassert ("FinishButtonClick");
 		}
 
-		[Test]
-		[Category ("NotWorking")]
-		public void Wizard_Init ()
-		{
-			PokerWizard wizard = new PokerWizard ();
-			wizard.Init += new EventHandler (wizard_handler);
-			wizard.DoOnInit (new EventArgs ());
-			eventassert ("OnInit");
-		}
-
 		[Test]
 		public void Wizard_NextButtonClick ()
 		{
@@ -1768,7 +1755,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		[ExpectedException (typeof (InvalidOperationException))]
 		public void Wizard_ActiveStepException1 ()
 		{
@@ -1778,7 +1764,6 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		[Category ("NotWorking")]
 		[ExpectedException (typeof (ArgumentOutOfRangeException))]
 		public void Wizard_ActiveStepException2 ()
 		{
@@ -1822,4 +1807,4 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 	}
 }
-#endif
\ No newline at end of file
+#endif
