@@ -85,6 +85,7 @@ namespace Mono.Xml.XPath
 			case XmlNodeType.Document:
 				current = ((XmlDocument) parent).CreateDocumentFragment ();
 				break;
+			case XmlNodeType.DocumentFragment:
 			case XmlNodeType.Element:
 				current = parent.OwnerDocument.CreateDocumentFragment ();
 				break;
@@ -451,6 +452,10 @@ namespace Mono.Xml.XPath
 			get { return navigator.BaseURI; }
 		}
 
+		public override bool CanEdit {
+			get { return true; }
+		}
+
 		public override bool IsEmptyElement {
 			get { return navigator.IsEmptyElement; }
 		}
