@@ -42,13 +42,6 @@ namespace Mono.Xml.XPath
 	{
 		public XPathNavigatorReader (XPathNavigator nav)
 		{
-			switch (nav.NodeType) {
-			case XPathNodeType.Element:
-			case XPathNodeType.Root:
-				break;
-			default:
-				throw new InvalidOperationException (String.Format ("NodeType {0} is not supported to read as a subtree of an XPathNavigator.", nav.NodeType));
-			}
 			current = nav.Clone ();
 		}
 
@@ -454,19 +447,20 @@ namespace Mono.Xml.XPath
 			return false;
 		}
 
-		/*
-		public override void MoveToAttribute (int i)
-		{
-			if (!MoveToAttributeNavigator (i))
-				throw new ArgumentOutOfRangeException ();
-		}
-		*/
-
 		public override bool MoveToFirstAttribute ()
 		{
-			if (CheckAttributeMove (current.MoveToFirstNamespace (XPathNamespaceScope.Local)))
-				return true;
-			return CheckAttributeMove (current.MoveToFirstAttribute ());
+			switch (current.NodeType) {
+			case XPathNodeType.Element:
+				if (CheckAttributeMove (current.MoveToFirstNamespace (XPathNamespaceScope.Local)))
+					return true;
+				return CheckAttributeMove (current.MoveToFirstAttribute ());
+			case XPathNodeType.Namespace:
+			case XPathNodeType.Attribute:
+				current.MoveToParent ();
+				goto case XPathNodeType.Element;
+			default:
+				return false;
+			}
 		}
 
 		public override bool MoveToNextAttribute ()
