@@ -57,7 +57,7 @@ namespace System.Xml.Schema
 		internal override int Compile(ValidationEventHandler h, XmlSchema schema)
 		{
 			// If this is already compiled this time, simply skip.
-			if (this.IsComplied (schema.CompilationId))
+			if (CompilationId == schema.CompilationId)
 				return 0;
 
 			if(Content == null)
