@@ -78,7 +78,7 @@ namespace System.Xml.Schema
 		internal override int Compile(ValidationEventHandler h, XmlSchema schema)
 		{
 			// If this is already compiled this time, simply skip.
-			if (this.IsComplied (schema.CompilationId))
+			if (CompilationId == schema.CompilationId)
 				return 0;
 
 #if NET_2_0
@@ -140,7 +140,7 @@ namespace System.Xml.Schema
 			if (MemberTypes != null) {
 				foreach (XmlQualifiedName memberTypeName in MemberTypes) {
 					object type = null;
-					XmlSchemaType xstype = schema.SchemaTypes [memberTypeName] as XmlSchemaSimpleType;
+					XmlSchemaType xstype = schema.FindSchemaType (memberTypeName) as XmlSchemaSimpleType;
 					if (xstype != null) {
 						errorCount += xstype.Validate (h, schema);
 						type = xstype;
