@@ -28,19 +28,20 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0
 
 using System;
 using System.Xml.Schema;
 
 namespace System.Xml.Serialization
 {
-	public enum XmlMappingAccess
+	[Flags]
+#if NET_2_0
+	public
+#endif
+	enum XmlMappingAccess
 	{
 		None,
 		Read,
 		Write
 	}
 }
-
-#endif
