@@ -106,6 +106,48 @@ namespace System.Xml.Serialization {
 			XmlReflectionMember [] members,
 			bool hasWrapperElement)
 		{
+			return ImportMembersMapping (elementName, ns, members, hasWrapperElement, true);
+		}
+
+#if NET_2_0
+		[MonoTODO]
+		public
+#endif
+		XmlMembersMapping ImportMembersMapping (string elementName, 
+			string ns, 
+			XmlReflectionMember[] members, 
+			bool hasWrapperElement, 
+			bool writeAccessors)
+		{
+			return ImportMembersMapping (elementName, ns, members, hasWrapperElement, writeAccessors, true);
+		}
+
+#if NET_2_0
+		[MonoTODO]
+		public
+#endif
+		XmlMembersMapping ImportMembersMapping (string elementName, 
+			string ns, 
+			XmlReflectionMember[] members, 
+			bool hasWrapperElement, 
+			bool writeAccessors, 
+			bool validate)
+		{
+			return ImportMembersMapping (elementName, ns, members, hasWrapperElement, writeAccessors, validate, XmlMappingAccess.Read | XmlMappingAccess.Write);
+		}
+
+#if NET_2_0
+		[MonoTODO] // FIXME: handle writeAccessors, validate, and mapping access
+		public
+#endif
+		XmlMembersMapping ImportMembersMapping (string elementName, 
+			string ns, 
+			XmlReflectionMember[] members, 
+			bool hasWrapperElement, 
+			bool writeAccessors, 
+			bool validate,
+			XmlMappingAccess access)
+		{
 //			Reset ();	Disabled. See ChangeLog
 
 			XmlMemberMapping[] mapping = new XmlMemberMapping[members.Length];
@@ -124,29 +166,6 @@ namespace System.Xml.Serialization {
 			return mps;
 		}
 
-#if NET_2_0
-		[MonoTODO]
-		public XmlMembersMapping ImportMembersMapping (string elementName, 
-			string ns, 
-			XmlReflectionMember[] members, 
-			bool hasWrapperElement, 
-			bool rpc)
-		{
-			throw new NotImplementedException ();
-		}
-
-		[MonoTODO]
-		public XmlMembersMapping ImportMembersMapping (string elementName, 
-			string ns, 
-			XmlReflectionMember[] members, 
-			bool hasWrapperElement, 
-			bool rpc, 
-			bool openModel)
-		{
-			throw new NotImplementedException ();
-		}
-#endif
-
 		public XmlTypeMapping ImportTypeMapping (Type type)
 		{
 			return ImportTypeMapping (type, null, null);
@@ -223,7 +242,7 @@ namespace System.Xml.Serialization {
 			string elementName;
 			bool includeInSchema = true;
 			XmlAttributes atts = null;
-			bool nullable = true;
+			bool nullable = CanBeNull (typeData);
 
 			if (defaultXmlType == null) defaultXmlType = typeData.XmlType;
 
@@ -331,7 +350,7 @@ namespace System.Xml.Serialization {
 					XmlTypeMapMember mem = CreateMapMember (type, rmember, ns);
 					mem.CheckOptionalValueType (type);
 					classMap.AddMember (mem);
-				} catch (InvalidOperationException ex) {
+				} catch (Exception ex) {
 					throw new InvalidOperationException (string.Format (
 						CultureInfo.InvariantCulture, "There was an error" +
 						" reflecting field '{0}'.", rmember.MemberName), ex);
@@ -456,12 +475,16 @@ namespace System.Xml.Serialization {
 
 			foreach (XmlArrayItemAttribute att in atts.XmlArrayItems)
 			{
+				if (att.Namespace != null && att.Form == XmlSchemaForm.Unqualified)
+					throw new InvalidOperationException ("XmlArrayItemAttribute.Form must not be Unqualified when it has an explicit Namespace value.");
 				if (att.NestingLevel != nestingLevel) continue;
 				Type elemType = (att.Type != null) ? att.Type : itemType;
 				XmlTypeMapElementInfo elem = new XmlTypeMapElementInfo (null, TypeTranslator.GetTypeData(elemType, att.DataType));
 				elem.Namespace = att.Namespace != null ? att.Namespace : defaultNamespace;
 				if (elem.Namespace == null) elem.Namespace = "";
 				elem.Form = att.Form;
+				if (att.Form == XmlSchemaForm.Unqualified)
+					elem.Namespace = string.Empty;
 				elem.IsNullable = att.IsNullable && CanBeNull (elem.TypeData);
 				elem.NestingLevel = att.NestingLevel;
 
@@ -761,6 +784,13 @@ namespace System.Xml.Serialization {
 			XmlAttributes atts = rmember.XmlAttributes;
 			TypeData typeData = TypeTranslator.GetTypeData (rmember.MemberType);
 
+			if (atts.XmlArray != null) {
+				if (atts.XmlArray.Namespace != null && atts.XmlArray.Form == XmlSchemaForm.Unqualified)
+					throw new InvalidOperationException ("XmlArrayAttribute.Form must not be Unqualified when it has an explicit Namespace value.");
+				if (typeData.SchemaType != SchemaTypes.Array)
+					throw new InvalidOperationException ("XmlArrayAttribute can be applied to members of array or collection type.");
+			}
+
 			if (atts.XmlAnyAttribute != null)
 			{
 				if ( (rmember.MemberType.FullName == "System.Xml.XmlAttribute[]") ||
@@ -837,7 +867,10 @@ namespace System.Xml.Serialization {
 				{
 					// A flat list
 
-					// TODO: check that it does not have XmlArrayAttribute
+					// check that it does not have XmlArrayAttribute
+					if (atts.XmlArray != null)
+						throw new InvalidOperationException ("XmlArrayAttribute cannot be used with members which also attributed with XmlElementAttribute or XmlTextAttribute.");
+
 					XmlTypeMapMemberFlatList member = new XmlTypeMapMemberFlatList ();
 					member.ListMap = new ListMap ();
 					member.ListMap.ItemInfo = ImportElementInfo (declaringType, XmlConvert.EncodeLocalName (rmember.MemberName), defaultNamespace, typeData.ListItemType, member, atts);
@@ -855,10 +888,17 @@ namespace System.Xml.Serialization {
 					member.ElementInfo = new XmlTypeMapElementInfoList();
 					XmlTypeMapElementInfo elem = new XmlTypeMapElementInfo (member, typeData);
 					elem.ElementName = XmlConvert.EncodeLocalName((atts.XmlArray != null && atts.XmlArray.ElementName.Length != 0) ? atts.XmlArray.ElementName : rmember.MemberName);
+					// note that it could be changed below (when Form is Unqualified)
 					elem.Namespace = (atts.XmlArray != null && atts.XmlArray.Namespace != null) ? atts.XmlArray.Namespace : defaultNamespace;
 					elem.MappedType = ImportListMapping (rmember.MemberType, null, elem.Namespace, atts, 0);
 					elem.IsNullable = (atts.XmlArray != null) ? atts.XmlArray.IsNullable : false;
 					elem.Form = (atts.XmlArray != null) ? atts.XmlArray.Form : XmlSchemaForm.Qualified;
+					// This is a bit tricky, but is done
+					// after filling descendant members, so
+					// that array items could be serialized
+					// with proper namespace.
+					if (atts.XmlArray != null && atts.XmlArray.Form == XmlSchemaForm.Unqualified)
+						elem.Namespace = String.Empty;
 
 					member.ElementInfo.Add (elem);
 					mapMember = member;
@@ -873,7 +913,7 @@ namespace System.Xml.Serialization {
 				mapMember = member;
 			}
 
-			mapMember.DefaultValue = atts.XmlDefaultValue;
+			mapMember.DefaultValue = GetDefaultValue (typeData, atts.XmlDefaultValue);
 			mapMember.TypeData = typeData;
 			mapMember.Name = rmember.MemberName;
 			mapMember.IsReturnValue = rmember.IsReturnValue;
@@ -935,8 +975,8 @@ namespace System.Xml.Serialization {
 				if (elem.Form != XmlSchemaForm.Unqualified)
 					elem.Namespace = (att.Namespace != null) ? att.Namespace : defaultNamespace;
 				elem.IsNullable = att.IsNullable;
-				
-				if (elem.IsNullable && elem.TypeData.IsValueType)
+
+				if (elem.IsNullable && !elem.TypeData.IsNullable)
 					throw new InvalidOperationException ("IsNullable may not be 'true' for value type " + elem.TypeData.FullTypeName + " in member '" + defaultName + "'");
 					
 				if (elem.TypeData.IsComplexType)
@@ -1035,7 +1075,11 @@ namespace System.Xml.Serialization {
 		
 		bool CanBeNull (TypeData type)
 		{
-			return (type.SchemaType != SchemaTypes.Primitive || type.Type == typeof (string));
+#if !NET_2_0	// idiotic compatibility
+			if (type.Type == typeof (XmlQualifiedName))
+				return false;
+#endif
+			return !type.Type.IsValueType;
 		}
 		
 		public void IncludeType (Type type)
@@ -1063,6 +1107,32 @@ namespace System.Xml.Serialization {
 				IncludeType (at.Type);
 		}
 
+		private object GetDefaultValue (TypeData typeData, object defaultValue)
+		{
+			if (defaultValue == DBNull.Value || typeData.SchemaType != SchemaTypes.Enum)
+				return defaultValue;
+
+			// get string representation of enum value
+			string namedValue = Enum.Format (typeData.Type, defaultValue, "g");
+			// get decimal representation of enum value
+			string decimalValue = Enum.Format (typeData.Type, defaultValue, "d");
+
+			// if decimal representation matches string representation, then
+			// the value is not defined in the enum type (as the "g" format
+			// will return the decimal equivalent of the value if the value
+			// is not equal to a combination of named enumerated constants
+			if (namedValue == decimalValue) {
+				string msg = string.Format (CultureInfo.InvariantCulture,
+					"Value '{0}' cannot be converted to {1}.", defaultValue,
+					defaultValue.GetType ().FullName);
+				throw new InvalidOperationException (msg);
+			}
+
+			// XmlSerializer expects integral enum value
+			//return namedValue.Replace (',', ' ');
+			return defaultValue;
+		}
+
 		#endregion // Methods
 	}
 }
