@@ -30,9 +30,15 @@
 //
 
 using System.Xml;
+#if !TARGET_JVM
 using System.CodeDom.Compiler;
+#endif
 using System.Xml.Schema;
 using System.Collections;
+#if NET_2_0 && CONFIGURATION_DEP
+using System.Configuration;
+using System.Xml.Serialization.Configuration;
+#endif
 
 namespace System.Xml.Serialization 
 {
@@ -82,6 +88,8 @@ namespace System.Xml.Serialization
 		{
 			this.schemas = schemas;
 			typeIdentifiers = new CodeIdentifiers ();
+
+			InitializeExtensions ();
 		}
 
 		public XmlSchemaImporter (XmlSchemas schemas, CodeIdentifiers typeIdentifiers)
@@ -91,6 +99,7 @@ namespace System.Xml.Serialization
 		}
 		
 #if NET_2_0
+#if !TARGET_JVM
 		[MonoTODO]
 		public XmlSchemaImporter (XmlSchemas schemas, CodeGenerationOptions options, CodeDomProvider codeProvider, ImportContext context)
 		{
@@ -102,8 +111,11 @@ namespace System.Xml.Serialization
 			}
 			else
 				typeIdentifiers = new CodeIdentifiers ();
+
+			InitializeExtensions ();
 		}
-		
+#endif
+
 		public XmlSchemaImporter (XmlSchemas schemas, CodeGenerationOptions options, ImportContext context)
 		{
 			this.schemas = schemas;
@@ -114,6 +126,8 @@ namespace System.Xml.Serialization
 			}
 			else
 				typeIdentifiers = new CodeIdentifiers ();
+
+			InitializeExtensions ();
 		}
 		
 
@@ -122,8 +136,10 @@ namespace System.Xml.Serialization
 			this.typeIdentifiers = typeIdentifiers;
 			this.schemas = schemas;
 			this.options = options;
+
+			InitializeExtensions ();
 		}
-		
+
 		void InitSharedData (ImportContext context)
 		{
 			if (context.ShareTypes) {
@@ -144,6 +160,19 @@ namespace System.Xml.Serialization
 
 		#region Methods
 
+		void InitializeExtensions ()
+		{
+#if NET_2_0 && CONFIGURATION_DEP
+			SerializationSectionGroup root = ConfigurationManager.GetSection ("system.xml.serialization") as SerializationSectionGroup;
+			if (root == null)
+				return;
+
+			foreach (SchemaImporterExtensionElement element in
+				 root.SchemaImporterExtensions.SchemaImporterExtensions)
+				Extensions.Add (element.Name, element.Type);
+#endif
+		}
+
 		public XmlMembersMapping ImportAnyType (XmlQualifiedName typeName, string elementName)
 		{
 			if (typeName == XmlQualifiedName.Empty)
@@ -417,12 +446,22 @@ namespace System.Xml.Serialization
 			XmlSchemaType stype;
 
 			XmlSchemaElement elem = (XmlSchemaElement) schemas.Find (name, typeof (XmlSchemaElement));
-			if (!LocateElement (elem, out qname, out stype)) return null;
+			if (!LocateElement (elem, out qname, out stype))
+				throw new InvalidOperationException (String.Format ("'{0}' is missing.", name));
 			
 			if (stype == null) {
-				// Importing a primitive type
-				TypeData td = TypeTranslator.GetPrimitiveTypeData (qname.Name);
-				return ReflectType (td, name.Namespace);
+				if (qname == anyType) {
+					// Importing anyType.
+					XmlTypeMapping xmap = GetTypeMapping (TypeTranslator.GetTypeData (typeof (object)));
+					// This also means, all complexTypes
+					// are imported as well.
+					BuildPendingMaps ();
+					return xmap;
+				} else {
+					// Importing a primitive type
+					TypeData td = TypeTranslator.GetPrimitiveTypeData (qname.Name);
+					return ReflectType (td, name.Namespace);
+				}
 			}
 			
 			XmlTypeMapping map = GetRegisteredTypeMapping (qname);
@@ -457,6 +496,11 @@ namespace System.Xml.Serialization
 				stype = elem.SchemaType;
 				qname = elem.QualifiedName;
 			}
+			else if (elem.ElementType == XmlSchemaComplexType.AnyType)
+			{
+				qname = anyType;
+				return true;
+			}
 			else
 			{
 				if (elem.SchemaTypeName.IsEmpty) return false;
@@ -647,6 +691,8 @@ namespace System.Xml.Serialization
 		
 		void ImportAttributes (XmlQualifiedName typeQName, ClassMap cmap, XmlSchemaObjectCollection atts, XmlSchemaAnyAttribute anyat, CodeIdentifiers classIds)
 		{
+			atts = CollectAttributeUsesNonOverlap (atts, cmap);
+
 			if (anyat != null)
 			{
     			XmlTypeMapMemberAnyAttribute member = new XmlTypeMapMemberAnyAttribute ();
@@ -688,6 +734,20 @@ namespace System.Xml.Serialization
 			}
 		}
 
+		// Attributes might be redefined, so there is an existing attribute for the same name, skip it.
+		// FIXME: this is nothing more than just a hack.
+		// Basically it should use
+		// XmlSchemaComplexType.AttributeUses.
+		XmlSchemaObjectCollection CollectAttributeUsesNonOverlap (
+			XmlSchemaObjectCollection src, ClassMap map)
+		{
+			XmlSchemaObjectCollection atts = new XmlSchemaObjectCollection ();
+			foreach (XmlSchemaAttribute a in src)
+				if (map.GetAttribute (a.QualifiedName.Name, a.QualifiedName.Namespace) == null)
+					atts.Add (a);
+			return atts;
+		}
+
 		ListMap BuildArrayMap (XmlQualifiedName typeQName, XmlSchemaComplexType stype, out TypeData arrayTypeData)
 		{
 			if (encodedFormat)
@@ -869,9 +929,13 @@ namespace System.Xml.Serialization
 						{
 							// It is a simple list (space separated list).
 							// Since this is not supported, map as a single item value
-							// TODO: improve this
 							member = new XmlTypeMapMemberElement ();
+#if NET_2_0
+							// In MS.NET those types are mapped to a string
+							typeData = TypeTranslator.GetTypeData(typeof(string));
+#else
 							typeData = typeData.ListItemTypeData;
+#endif
 						}
 						else
 							member = new XmlTypeMapMemberList ();
@@ -1045,7 +1109,10 @@ namespace System.Xml.Serialization
 				choiceMember.ElementInfo.Add (CreateElementInfo (typeQName.Namespace, choiceMember, choiceMember.Name, choiceMember.TypeData, false, XmlSchemaForm.None));
 				cmap.AddMember (choiceMember);
 			}
-
+			
+			if (typeData == null)
+				return;
+	
 			if (multiValue)
 				typeData = typeData.ListTypeData;
 
@@ -1154,7 +1221,7 @@ namespace System.Xml.Serialization
 
 		TypeData FindBuiltInType (XmlQualifiedName qname, XmlSchemaSimpleType st)
 		{
-			if (CanBeEnum (st))
+			if (CanBeEnum (st) && qname != null)
 				return ImportType (qname, null, true).TypeData;
 
 			if (st.Content is XmlSchemaSimpleTypeRestriction) {
@@ -1168,23 +1235,9 @@ namespace System.Xml.Serialization
 			else if (st.Content is XmlSchemaSimpleTypeList) {
 				return FindBuiltInType (GetContentBaseType (st.Content)).ListTypeData;
 			}
-			else if (st.Content is XmlSchemaSimpleTypeUnion)
-			{
-				// Check if all types of the union are equal. If not, then will use anyType.
-				XmlSchemaSimpleTypeUnion uni = (XmlSchemaSimpleTypeUnion) st.Content;
-				TypeData utype = null;
-
-				// Anonymous types are unique
-				if (uni.BaseTypes.Count != 0 && uni.MemberTypes.Length != 0)
-					return FindBuiltInType (anyType);
-
-				foreach (XmlQualifiedName mt in uni.MemberTypes)
-				{
-					TypeData qn = FindBuiltInType (mt);
-					if (utype != null && qn != utype) return FindBuiltInType (anyType);
-					else utype = qn;
-				}
-				return utype;
+			else if (st.Content is XmlSchemaSimpleTypeUnion) {
+				// MS.NET always import simple unions as string
+				return FindBuiltInType (new XmlQualifiedName ("string", XmlSchema.Namespace));
 			}
 			else
 				return null;
@@ -1310,7 +1363,11 @@ namespace System.Xml.Serialization
 			// It is an extension of a primitive or known type
 			
 			TypeData typeData = FindBuiltInType (typeQName, stype);
-			return GetTypeMapping (typeData);
+			XmlTypeMapping rmap = GetTypeMapping (typeData);
+			
+			// The resulting map must be a simple type. It needs to be explicitely set for arrays
+			rmap.IsSimpleType = true;
+			return rmap;
 		}
 
 		bool CanBeEnum (XmlSchemaSimpleType stype)
@@ -1592,7 +1649,7 @@ namespace System.Xml.Serialization
 			}
 			else
 			{
-				ns = typeQName.Namespace;
+				ns = attr.ParentIsSchema ? typeQName.Namespace : String.Empty;
 				return attr;
 			}
 		}
