@@ -40,6 +40,10 @@ using System.Reflection;
 
 namespace System.Xml.Serialization 
 {
+#if NET_2_0
+	[MonoTODO]
+	// FIXME: provide expected elements/attributes on unknown elements/attributs
+#endif
 	public abstract class XmlSerializationReader 
 #if NET_2_0
 		: XmlSerializationGeneratedCode
@@ -284,13 +288,13 @@ namespace System.Xml.Serialization
 			return result;
 		}
 
-		[MonoTODO ("Implement")]
+		[MonoTODO]
 		protected void FixupArrayRefs (object fixup)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO ("Implement")]
+		[MonoTODO]
 		protected int GetArrayLength (string name, string ns)
 		{
 			throw new NotImplementedException ();
@@ -479,7 +483,16 @@ namespace System.Xml.Serialization
 			string id = Reader.GetAttribute ("id");
 			object ob;
 
-			if (qname == arrayQName)
+			// it takes precedence over xsi:type.
+			// Sometimes there are array types in WSDL,
+			// which are not reflected in client proxies.
+			// In SOAP messages, they are marked
+			// soap-env:arrayType, so use it (this could coexist
+			// with xsi:type, which indicates the type in WSDL).
+			// See bug #79057.
+			string arrayTypeVal = Reader.GetAttribute (arrayType, soapNS);
+
+			if (qname == arrayQName || arrayTypeVal != null && arrayTypeVal.Length > 0)
 			{
 				CollectionFixup fixup = (collFixups != null) ? (CollectionFixup) collFixups[id] : null;
 				if (ReadList (out ob))
@@ -717,7 +730,7 @@ namespace System.Xml.Serialization
 				XmlNode node = Document.ReadNode (reader);
 				
 				if (reportUnknown)
-					OnUnknownNode (node, null);
+					OnUnknownNode (node, null, null);
 
 				if (node.ChildNodes.Count == 0 && node.Attributes.Count == 0)
 					return new Object ();
@@ -786,25 +799,32 @@ namespace System.Xml.Serialization
 
 		protected byte[] ToByteArrayBase64 (bool isNull)
 		{
-			return Convert.FromBase64String (Reader.ReadString());
+			if (isNull) {
+				Reader.ReadString ();
+				return null;
+			}
+			else
+				return ToByteArrayBase64 (Reader.ReadString ());
 		}
 
-		[MonoTODO ("Implement")]
 		protected static byte[] ToByteArrayBase64 (string value)
 		{
-			throw new NotImplementedException ();
+			return Convert.FromBase64String (value);
 		}
 
-		[MonoTODO ("Implement")]
 		protected byte[] ToByteArrayHex (bool isNull)
 		{
-			throw new NotImplementedException ();
+			if (isNull) {
+				Reader.ReadString ();
+				return null;
+			}
+			else
+				return ToByteArrayHex (Reader.ReadString ());
 		}
 
-		[MonoTODO ("Implement")]
 		protected static byte[] ToByteArrayHex (string value)
 		{
-			throw new NotImplementedException ();
+			return XmlConvert.FromBinHexString (value);
 		}
 
 		protected static char ToChar (string value)
@@ -874,6 +894,14 @@ namespace System.Xml.Serialization
 		}
 
 		protected void UnknownAttribute (object o, XmlAttribute attr)
+		{
+			UnknownAttribute (o, attr, null);
+		}
+
+#if NET_2_0
+		protected
+#endif
+		void UnknownAttribute (object o, XmlAttribute attr, string qnames)
 		{
 			int line_number, line_position;
 			
@@ -885,11 +913,24 @@ namespace System.Xml.Serialization
 				line_position = 0;
 			}
 
+			XmlAttributeEventArgs args = new XmlAttributeEventArgs (attr, line_number, line_position, o);
+#if NET_2_0
+			args.ExpectedAttributes = qnames;
+#endif
+
 			if (eventSource != null)
-				eventSource.OnUnknownAttribute (new XmlAttributeEventArgs (attr, line_number, line_position, o));
+				eventSource.OnUnknownAttribute (args);
 		}
 
 		protected void UnknownElement (object o, XmlElement elem)
+		{
+			UnknownElement (o, elem, null);
+		}
+
+#if NET_2_0
+		protected
+#endif
+		void UnknownElement (object o, XmlElement elem, string qnames)
 		{
 			int line_number, line_position;
 			
@@ -900,17 +941,30 @@ namespace System.Xml.Serialization
 				line_number = 0;
 				line_position = 0;
 			}
-	
+
+			XmlElementEventArgs args = new XmlElementEventArgs (elem, line_number, line_position, o);
+#if NET_2_0
+			args.ExpectedElements = qnames;
+#endif
+
 			if (eventSource != null)
-				eventSource.OnUnknownElement (new XmlElementEventArgs (elem, line_number, line_position,o));
+				eventSource.OnUnknownElement (args);
 		}
 
 		protected void UnknownNode (object o)
 		{
-			OnUnknownNode (ReadXmlNode (false), o);
+			UnknownNode (o, null);
+		}
+
+#if NET_2_0
+		protected
+#endif
+		void UnknownNode (object o, string qnames)
+		{
+			OnUnknownNode (ReadXmlNode (false), o, qnames);
 		}
 		
-		void OnUnknownNode (XmlNode node, object o)
+		void OnUnknownNode (XmlNode node, object o, string qnames)
 		{
 			int line_number, line_position;
 			
@@ -924,12 +978,12 @@ namespace System.Xml.Serialization
 	
 			if (node is XmlAttribute)
 			{
-				UnknownAttribute (o, (XmlAttribute)node);
+				UnknownAttribute (o, (XmlAttribute)node, qnames);
 				return;
 			}
 			else if (node is XmlElement)
 			{
-				UnknownElement (o, (XmlElement) node);
+				UnknownElement (o, (XmlElement) node, qnames);
 				return;
 			}
 			else
@@ -1092,12 +1146,6 @@ namespace System.Xml.Serialization
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
-		protected IXmlSerializable ReadSerializable (IXmlSerializable serializable, bool wrapped)
-		{
-			throw new NotImplementedException ();
-		}
-		
 		[MonoTODO]
 		protected string ReadString (string value, bool trim)
 		{
