@@ -73,7 +73,7 @@ namespace System.Xml.Serialization
 			public Type WriterType;
 			public MethodInfo WriterMethod;
 			public GenerationBatch Batch;
-			public IXmlSerializerImplementation Implementation = null;
+			public XmlSerializerImplementation Implementation = null;
 			
 			public XmlSerializationReader CreateReader () {
 				if (ReaderType != null)
@@ -360,8 +360,8 @@ namespace System.Xml.Serialization
 					return serializerData.ReaderMethod.Invoke (reader, null);
 			} catch (Exception ex) {
 				if (ex is InvalidOperationException || ex is InvalidCastException)
-					throw new InvalidOperationException ("There was an error generating" +
-						" the XML document.", ex);
+					throw new InvalidOperationException ("There is an error in"
+						+ " XML document.", ex);
 				throw;
 			}
 		}
@@ -454,8 +454,13 @@ namespace System.Xml.Serialization
 
 				if (namespaces == null || namespaces.Count == 0) {
 					namespaces = new XmlSerializerNamespaces ();
+#if NET_2_0
+					namespaces.Add ("xsi", XmlSchema.InstanceNamespace);
+					namespaces.Add ("xsd", XmlSchema.Namespace);
+#else
 					namespaces.Add ("xsd", XmlSchema.Namespace);
 					namespaces.Add ("xsi", XmlSchema.InstanceNamespace);
+#endif
 				}
 
 				xsWriter.Initialize (writer, namespaces);
@@ -528,29 +533,6 @@ namespace System.Xml.Serialization
 		}
 #endif
 
-		[MonoTODO]
-		[Obsolete]
-		public static Assembly GenerateSerializer (Type[] types, 
-			XmlMapping[] mappings, 
-			string codePath, 
-			bool debug, 
-			bool keepFiles)
-		{
-			throw new NotImplementedException ();
-		}
-		
-		[MonoTODO]
-		[Obsolete]
-		public static Assembly GenerateSerializer (Type[] types, 
-			XmlMapping[] mappings, 
-			string codePath, 
-			bool debug, 
-			bool keepFiles, 
-			string compilerOptions)
-		{
-			throw new NotImplementedException ();
-		}
-
 		public static string GetXmlSerializerAssemblyName (Type type)
 		{
 			return type.Assembly.GetName().Name + ".XmlSerializers";
