@@ -89,11 +89,6 @@ namespace System.Xml.XPath
 			if (!MoveNextCore ())
 				return false;
 			position++;
-			if (Current != null &&
-				Current.NodeType == XPathNodeType.Whitespace &&
-				_nsm is XsltContext &&
-				!((XsltContext) _nsm).PreserveWhitespace (Current))
-				return MoveNext ();
 			return true;
 		}
 
@@ -522,7 +517,7 @@ namespace System.Xml.XPath
 					return true;
 				}
 				if (!_nav.MoveToParent ())	// should NEVER fail!
-					throw new XPathException ("There seems some bugs on the XPathNavigator implementation class.");
+					throw new XPathException ("Current node is removed while it should not be, or there are some bugs in the XPathNavigator implementation class: " + _nav.GetType ());
 				_depth --;
 			}
 			_finished = true;
@@ -569,7 +564,7 @@ namespace System.Xml.XPath
 					return true;
 				}
 				if (!_nav.MoveToParent ())	// should NEVER fail!
-					throw new XPathException ("There seems some bugs on the XPathNavigator implementation class.");
+					throw new XPathException ("Current node is removed while it should not be, or there are some bugs in the XPathNavigator implementation class: " + _nav.GetType ());
 				_depth --;
 			}
 			_finished = true;
@@ -743,14 +738,14 @@ namespace System.Xml.XPath
 		private SimpleIterator _iter;
 		private NodeTest _test;
 			
-		string name, ns;
-		XPathNodeType matchType;
+		//string name, ns;
+		//XPathNodeType matchType;
 
 		public AxisIterator (SimpleIterator iter, NodeTest test) : base (iter.NamespaceManager)
 		{
 			_iter = iter;
 			_test = test;
-			test.GetInfo (out name, out ns, out matchType, NamespaceManager);
+			//test.GetInfo (out name, out ns, out matchType, NamespaceManager);
 //			if (name != null)
 //				name = Current.NameTable.Add (name);
 
@@ -762,9 +757,9 @@ namespace System.Xml.XPath
 		{
 			_iter = (SimpleIterator) other._iter.Clone ();
 			_test = other._test;
-			name = other.name;
-			ns = other.ns;
-			matchType = other.matchType;
+			//name = other.name;
+			//ns = other.ns;
+			//matchType = other.matchType;
 		}
 		public override XPathNodeIterator Clone () { return new AxisIterator (this); }
 
@@ -1057,22 +1052,17 @@ namespace System.Xml.XPath
 
 		public ListIterator (BaseIterator iter, IList list) : base (iter.NamespaceManager)
 		{
-			if (!(list is ICloneable))
-				throw new ArgumentException ("Target enumerator must be cloneable.");
 			_list = list;
 		}
 		
 		public ListIterator (IList list, NSResolver nsm) : base (nsm)
 		{
-			if (!(list is ICloneable))
-				throw new ArgumentException ("Target enumerator must be cloneable.");
 			_list = list;
 		}
 
 		private ListIterator (ListIterator other) : base (other)
 		{
-			ICloneable listClone = other._list as ICloneable;
-			_list = (IList) listClone.Clone ();
+			_list = other._list;
 		}
 		public override XPathNodeIterator Clone () { return new ListIterator (this); }
 
