@@ -651,13 +651,11 @@ namespace System.Xml.XPath
 			return Clone ();
 		}
 
-		[MonoTODO]
 		public virtual object Evaluate (string xpath, IXmlNamespaceResolver nsResolver)
 		{
 			return Evaluate (Compile (xpath), null, nsResolver);
 		}
 
-		[MonoTODO]
 		public virtual IDictionary<string, string> GetNamespacesInScope (XmlNamespaceScope scope)
 		{
 			IDictionary<string, string> table = new Dictionary<string, string> ();
@@ -834,7 +832,13 @@ namespace System.Xml.XPath
 
 		public virtual XmlReader ReadSubtree ()
 		{
-			return new XPathNavigatorReader (this);
+			switch (NodeType) {
+			case XPathNodeType.Element:
+			case XPathNodeType.Root:
+				return new XPathNavigatorReader (this);
+			default:
+				throw new InvalidOperationException (String.Format ("NodeType {0} is not supported to read as a subtree of an XPathNavigator.", NodeType));
+			}
 		}
 
 		public virtual XPathNodeIterator Select (string xpath, IXmlNamespaceResolver nsResolver)
@@ -874,9 +878,48 @@ namespace System.Xml.XPath
 			writer.WriteNode (this, false);
 		}
 
-		[MonoTODO]
+		static readonly char [] escape_text_chars =
+				new char [] {'&', '<', '>'};
+		static readonly char [] escape_attr_chars =
+				new char [] {'"', '&', '<', '>', '\r', '\n'};
+
+		static string EscapeString (string value, bool attr)
+		{
+			StringBuilder sb = null;
+			char [] escape = attr ? escape_attr_chars : escape_text_chars;
+			if (value.IndexOfAny (escape) < 0)
+				return value;
+			sb = new StringBuilder (value, value.Length + 10);
+			if (attr)
+				sb.Replace ("\"", "&quot;");
+			sb.Replace ("<", "&lt;");
+			sb.Replace (">", "&gt;");
+			if (attr) {
+				sb.Replace ("\r\n", "&#10;");
+				sb.Replace ("\r", "&#10;");
+				sb.Replace ("\n", "&#10;");
+			}
+			return sb.ToString ();
+		}
+
 		public virtual string InnerXml {
 			get {
+				switch (NodeType) {
+				case XPathNodeType.Element:
+				case XPathNodeType.Root:
+					break;
+				case XPathNodeType.Attribute:
+				case XPathNodeType.Namespace:
+					return EscapeString (Value, true);
+				case XPathNodeType.Text:
+				case XPathNodeType.Whitespace:
+				case XPathNodeType.SignificantWhitespace:
+					return String.Empty;
+				case XPathNodeType.ProcessingInstruction:
+				case XPathNodeType.Comment:
+					return Value;
+				}
+
 				XmlReader r = ReadSubtree ();
 				r.Read (); // start
 				// skip the element itself (or will reach to 
@@ -886,7 +929,10 @@ namespace System.Xml.XPath
 				if (NodeType != XPathNodeType.Root)
 					r.Read ();
 				StringWriter sw = new StringWriter ();
-				XmlWriter xtw = XmlWriter.Create (sw);
+				XmlWriterSettings s = new XmlWriterSettings ();
+				s.ConformanceLevel = ConformanceLevel.Fragment;
+				s.OmitXmlDeclaration = true;
+				XmlWriter xtw = XmlWriter.Create (sw, s);
 				while (!r.EOF && r.Depth > depth)
 					xtw.WriteNode (r, false);
 				return sw.ToString ();
@@ -907,16 +953,42 @@ namespace System.Xml.XPath
 
 		public virtual string OuterXml {
 			get {
+				switch (NodeType) {
+				case XPathNodeType.Attribute:
+					return String.Concat (
+						Prefix,
+						Prefix.Length > 0 ? ":" : String.Empty,
+						LocalName,
+						"=\"",
+						EscapeString (Value, true),
+						"\"");
+					break;
+				case XPathNodeType.Namespace:
+					return String.Concat (
+						"xmlns",
+						LocalName.Length > 0 ? ":" : String.Empty,
+						LocalName,
+						"=\"",
+						EscapeString (Value, true),
+						"\"");
+					break;
+				case XPathNodeType.Text:
+					return EscapeString (Value, false);
+				case XPathNodeType.Whitespace:
+				case XPathNodeType.SignificantWhitespace:
+					return Value;
+				}
+
 				XmlWriterSettings s = new XmlWriterSettings ();
 				s.Indent = true;
 				s.OmitXmlDeclaration = true;
+				s.ConformanceLevel = ConformanceLevel.Fragment;
 				StringBuilder sb = new StringBuilder ();
 				using (XmlWriter w = XmlWriter.Create (sb, s)) {
 					WriteSubtree (w);
 				}
 				return sb.ToString ();
 			}
-			[MonoTODO]
 			set {
 				switch (NodeType) {
 				case XPathNodeType.Root:
@@ -931,14 +1003,12 @@ namespace System.Xml.XPath
 			}
 		}
 
-		[MonoTODO]
 		public virtual IXmlSchemaInfo SchemaInfo {
 			get {
 				return null;
 			}
 		}
 
-		[MonoTODO]
 		public override object TypedValue {
 			get {
 				switch (NodeType) {
@@ -989,7 +1059,6 @@ namespace System.Xml.XPath
 			}
 		}
 
-		[MonoTODO]
 		public override XmlSchemaType XmlType {
 			get {
 				if (SchemaInfo != null)
@@ -1032,7 +1101,6 @@ namespace System.Xml.XPath
 			w.Close ();
 		}
 
-		[MonoTODO]
 		public virtual void AppendChild (
 			XPathNavigator nav)
 		{
@@ -1056,7 +1124,6 @@ namespace System.Xml.XPath
 		}
 
 		// must override it.
-		[MonoTODO ("needs tests")]
 		public virtual XmlWriter CreateAttributes ()
 		{
 			throw new NotSupportedException ();
@@ -1108,7 +1175,6 @@ namespace System.Xml.XPath
 			}
 		}
 
-		[MonoTODO]
 		public virtual void InsertAfter (XPathNavigator nav)
 		{
 			InsertAfter (new XPathNavigatorReader (nav));
@@ -1131,7 +1197,6 @@ namespace System.Xml.XPath
 			}
 		}
 
-		[MonoTODO]
 		public virtual void InsertBefore (XPathNavigator nav)
 		{
 			InsertBefore (new XPathNavigatorReader (nav));
@@ -1174,7 +1239,6 @@ namespace System.Xml.XPath
 			}
 		}
 
-		[MonoTODO]
 		public virtual void PrependChild (XPathNavigator nav)
 		{
 			PrependChild (new XPathNavigatorReader (nav));
@@ -1199,7 +1263,6 @@ namespace System.Xml.XPath
 			throw new NotSupportedException ();
 		}
 
-		[MonoTODO]
 		public virtual void ReplaceSelf (XPathNavigator navigator)
 		{
 			ReplaceSelf (new XPathNavigatorReader (navigator));
@@ -1217,7 +1280,6 @@ namespace System.Xml.XPath
 			throw new NotSupportedException ();
 		}
 
-		[MonoTODO]
 		private void DeleteChildren ()
 		{
 			switch (NodeType) {
