@@ -127,8 +127,12 @@ namespace System.Xml
 
 		// This is used in XmlReader.Create() to indicate that string
 		// argument is uri, not an xml fragment.
-		internal XmlTextReader (bool dummy, string url, XmlNodeType fragType, XmlParserContext context)
+		internal XmlTextReader (bool dummy, XmlResolver resolver, string url, XmlNodeType fragType, XmlParserContext context)
 		{
+			if (resolver == null)
+				resolver = new XmlUrlResolver ();
+
+			this.XmlResolver = resolver;
 			string uriString;
 			Stream stream = GetStreamFromUrl (url, out uriString);
 			this.InitializeContext (uriString, context, new XmlStreamReader (stream), fragType);
@@ -681,7 +685,7 @@ namespace System.Xml
 		}
 
 #if NET_2_0
-		[MonoTODO ("Implement for performance reason")]
+		[MonoTODO] // FIXME: Implement, for performance improvement
 		public override void Skip ()
 		{
 			base.Skip ();
