@@ -129,9 +129,9 @@ namespace System.Xml
 			source = new XmlTextReaderImpl (baseURI, xmlFragment, fragType, context);
 		}
 
-		internal XmlTextReader (bool dummy, string url, XmlNodeType fragType, XmlParserContext context)
+		internal XmlTextReader (bool dummy, XmlResolver resolver, string url, XmlNodeType fragType, XmlParserContext context)
 		{
-			source = new XmlTextReaderImpl (dummy, url, fragType, context);
+			source = new XmlTextReaderImpl (dummy, resolver, url, fragType, context);
 		}
 
 		private XmlTextReader (XmlTextReaderImpl entityContainer, bool insideAttribute)
@@ -591,8 +591,9 @@ namespace System.Xml
 			else {
 				if (source.NodeType != XmlNodeType.EntityReference)
 					throw new InvalidOperationException ("The current node is not an Entity Reference");
-				XmlTextReaderImpl entReader = 
-					ParserContext.Dtd.GenerateEntityContentReader (source.Name, ParserContext);
+				XmlTextReaderImpl entReader = null;
+				if (ParserContext.Dtd != null)
+					entReader = ParserContext.Dtd.GenerateEntityContentReader (source.Name, ParserContext);
 				if (entReader == null)
 					throw new XmlException (this as IXmlLineInfo, this.BaseURI, String.Format ("Reference to undeclared entity '{0}'.", source.Name));
 				if (entityNameStack == null)
@@ -619,7 +620,7 @@ namespace System.Xml
 			base.Skip ();
 		}
 
-		[MonoTODO ("Check how expanded entity is handled here.")]
+		[MonoTODO] // FIXME: Check how expanded entity is handled here.
 		public TextReader GetRemainder ()
 		{
 			if (entity != null) {
@@ -635,7 +636,7 @@ namespace System.Xml
 			return true;
 		}
 
-		[MonoTODO ("Check how expanded entity is handled here.")]
+		[MonoTODO] // FIXME: Check how expanded entity is handled here.
 		public int ReadBase64 (byte [] buffer, int offset, int length)
 		{
 			if (entity != null)
@@ -644,7 +645,7 @@ namespace System.Xml
 				return source.ReadBase64 (buffer, offset, length);
 		}
 
-		[MonoTODO ("Check how expanded entity is handled here.")]
+		[MonoTODO] // FIXME: Check how expanded entity is handled here.
 		public int ReadBinHex (byte [] buffer, int offset, int length)
 		{
 			if (entity != null)
@@ -653,7 +654,7 @@ namespace System.Xml
 				return source.ReadBinHex (buffer, offset, length);
 		}
 
-		[MonoTODO ("Check how expanded entity is handled here.")]
+		[MonoTODO] // FIXME: Check how expanded entity is handled here.
 		public int ReadChars (char [] buffer, int offset, int length)
 		{
 			if (entity != null)
@@ -663,7 +664,7 @@ namespace System.Xml
 		}
 
 
-		[MonoTODO ("Check how expanded entity is handled here.")]
+		[MonoTODO] // FIXME: Check how expanded entity is handled here.
 		public override int ReadContentAsBase64 (byte [] buffer, int offset, int length)
 		{
 			if (entity != null)
@@ -672,7 +673,7 @@ namespace System.Xml
 				return source.ReadContentAsBase64 (buffer, offset, length);
 		}
 
-		[MonoTODO ("Check how expanded entity is handled here.")]
+		[MonoTODO] // FIXME: Check how expanded entity is handled here.
 		public override int ReadContentAsBinHex (byte [] buffer, int offset, int length)
 		{
 			if (entity != null)
@@ -681,7 +682,7 @@ namespace System.Xml
 				return source.ReadContentAsBinHex (buffer, offset, length);
 		}
 
-		[MonoTODO ("Check how expanded entity is handled here.")]
+		[MonoTODO] // FIXME: Check how expanded entity is handled here.
 		public override int ReadElementContentAsBase64 (byte [] buffer, int offset, int length)
 		{
 			if (entity != null)
@@ -690,7 +691,7 @@ namespace System.Xml
 				return source.ReadElementContentAsBase64 (buffer, offset, length);
 		}
 
-		[MonoTODO ("Check how expanded entity is handled here.")]
+		[MonoTODO] // FIXME: Check how expanded entity is handled here.
 		public override int ReadElementContentAsBinHex (byte [] buffer, int offset, int length)
 		{
 			if (entity != null)
