@@ -3,10 +3,33 @@
 //
 // Author:
 //   Gert Driesen (drieseng@users.sourceforge.net)
+//   Atsushi Enomoto (atsushi@ximian.com)
 //
-// (C) 2005 Novell
+// (C) 2005 Gert Driesen
+// Copyright (C) 2006 Novell, Inc.
 // 
 
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.CodeDom;
 using System.Collections;
@@ -993,5 +1016,103 @@ namespace MonoTests.System.XmlSerialization
 
 			return qnames;
 		}
+
+		[Test]
+		[ExpectedException (typeof (InvalidOperationException))]
+		public void ImportTypeMappingNonExistent ()
+		{
+			XmlSchemas xss = new XmlSchemas ();
+			xss.Add (new XmlSchema ());
+			XmlSchemaImporter imp = new XmlSchemaImporter (xss);
+			imp.ImportTypeMapping (new XmlQualifiedName ("foo"));
+		}
+
+		[Test]
+		public void AnyTypeTopLevelElementImportsAllComplexTypes ()
+		{
+			string xsd = @"
+<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>
+  <xs:element name='Root' type='xs:anyType' />
+  <xs:complexType name='FooType'>
+    <xs:sequence>
+      <xs:element name='Child1' type='xs:string' />
+      <xs:element name='Child2' type='xs:string' />
+      <xs:element name='Child3' type='xs:string' />
+    </xs:sequence>
+  </xs:complexType>
+  <xs:complexType name='BarType' />
+</xs:schema>";
+			XmlSchemas xss = new XmlSchemas ();
+			xss.Add (XmlSchema.Read (new XmlTextReader (new StringReader (xsd)), null));
+			XmlSchemaImporter imp = new XmlSchemaImporter (xss);
+			CodeNamespace cns = new CodeNamespace ();
+			XmlCodeExporter exp = new XmlCodeExporter (cns);
+			exp.ExportTypeMapping (imp.ImportTypeMapping (new XmlQualifiedName ("Root")));
+			bool foo = false, bar = false;
+			foreach (CodeTypeDeclaration td in cns.Types) {
+				if (td.Name == "FooType")
+					foo = true;
+				else if (td.Name == "BarType")
+					bar = true;
+			}
+			Assert.IsTrue (foo, "FooType not found");
+			Assert.IsTrue (bar, "BarType not found");
+		}
+
+		[Test]
+		public void DefaultTypeTopLevelElementImportsAllComplexTypes ()
+		{
+			string xsd = @"
+<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>
+  <xs:element name='Root' />
+  <xs:complexType name='FooType'>
+    <xs:sequence>
+      <xs:element name='Child1' type='xs:string' />
+      <xs:element name='Child2' type='xs:string' />
+      <xs:element name='Child3' type='xs:string' />
+    </xs:sequence>
+  </xs:complexType>
+  <xs:complexType name='BarType' />
+</xs:schema>";
+			XmlSchemas xss = new XmlSchemas ();
+			xss.Add (XmlSchema.Read (new XmlTextReader (new StringReader (xsd)), null));
+			XmlSchemaImporter imp = new XmlSchemaImporter (xss);
+			CodeNamespace cns = new CodeNamespace ();
+			XmlCodeExporter exp = new XmlCodeExporter (cns);
+			exp.ExportTypeMapping (imp.ImportTypeMapping (new XmlQualifiedName ("Root")));
+			bool foo = false, bar = false;
+			foreach (CodeTypeDeclaration td in cns.Types) {
+				if (td.Name == "FooType")
+					foo = true;
+				else if (td.Name == "BarType")
+					bar = true;
+			}
+			Assert.IsTrue (foo, "FooType not found");
+			Assert.IsTrue (bar, "BarType not found");
+		}
+
+		[Test]
+		public void ImportComplexDerivationByExtension ()
+		{
+			string xsd = @"<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>
+  <xs:element name='Root' type='DerivedType' />
+  <xs:complexType name='DerivedType'>
+    <xs:complexContent>
+      <xs:extension base='BaseType'>
+        <xs:attribute name='Foo' type='xs:string' />
+      </xs:extension>
+    </xs:complexContent>
+  </xs:complexType>
+  <xs:complexType name='BaseType'>
+    <xs:attribute name='Foo' type='xs:string' />
+  </xs:complexType>
+</xs:schema>";
+			XmlSchemas xss = new XmlSchemas ();
+			xss.Add (XmlSchema.Read (new XmlTextReader (new StringReader (xsd)), null));
+			XmlSchemaImporter imp = new XmlSchemaImporter (xss);
+			CodeNamespace cns = new CodeNamespace ();
+			XmlCodeExporter exp = new XmlCodeExporter (cns);
+			exp.ExportTypeMapping (imp.ImportTypeMapping (new XmlQualifiedName ("Root")));
+		}
 	}
 }
