@@ -97,35 +97,33 @@ namespace MonoTests.System.Xml
 		[Test]
 		public void CreateNodeNodeTypeNameEmptyParams ()
 		{
-			XmlNode node;
-
 			try {
-				node = document.CreateNode (null, null, null);
+				document.CreateNode (null, null, null);
 				Fail ("Expected an ArgumentException to be thrown.");
 			} catch (ArgumentException) {}
 
 			try {
-				node = document.CreateNode ("attribute", null, null);
+				document.CreateNode ("attribute", null, null);
 				Fail ("Expected a NullReferenceException to be thrown.");
 			} catch (NullReferenceException) {}
 
 			try {
-				node = document.CreateNode ("attribute", "", null);
+				document.CreateNode ("attribute", "", null);
 				Fail ("Expected an ArgumentException to be thrown.");
 			} catch (ArgumentException) {}
 
 			try {
-				node = document.CreateNode ("element", null, null);
+				document.CreateNode ("element", null, null);
 				Fail ("Expected a NullReferenceException to be thrown.");
 			} catch (NullReferenceException) {}
 
 			try {
-				node = document.CreateNode ("element", "", null);
+				document.CreateNode ("element", "", null);
 				Fail ("Expected an ArgumentException to be thrown.");
 			} catch (ArgumentException) {}
 
 			try {
-				node = document.CreateNode ("entityreference", null, null);
+				document.CreateNode ("entityreference", null, null);
 				Fail ("Expected a NullReferenceException to be thrown.");
 			} catch (NullReferenceException) {}
 		}
@@ -1164,5 +1162,26 @@ namespace MonoTests.System.Xml
 			AssertEquals (1,
 				doc.DocumentElement.FirstChild.ChildNodes.Count);
 		}
+
+		[Test]
+		public void LoadTreatsFixedAttributesAsIfItExisted ()
+		{
+			string xml = @"<!DOCTYPE foo [<!ELEMENT foo EMPTY><!ATTLIST foo xmlns CDATA #FIXED 'urn:foo'>]><foo />";
+			XmlDocument doc = new XmlDocument ();
+			doc.Load (new StringReader (xml));
+			AssertEquals ("urn:foo", doc.DocumentElement.NamespaceURI);
+		}
+
+		[Test]
+		public void Bug79468 () // XmlNameEntryCache bug
+		{
+			string xml = "<?xml version='1.0' encoding='UTF-8'?>"
+				+ "<ns0:DebtAmountRequest xmlns:ns0='http://whatever'>"
+				+ "  <Signature xmlns='http://www.w3.org/2000/09/xmldsig#' />"
+				+ "</ns0:DebtAmountRequest>";
+			XmlDocument doc = new XmlDocument ();
+			doc.LoadXml (xml);
+			XmlNodeList nodeList = doc.GetElementsByTagName ("Signature");
+		}
 	}
 }
