@@ -10,6 +10,7 @@
 #if NET_2_0
 using System;
 using System.IO;
+using System.Net;
 using System.Text;
 using System.Xml;
 using System.Xml.Schema;
@@ -306,6 +307,57 @@ namespace MonoTests.System.Xml
 			// but don't reject because of that fact.
 			XmlReader r = XmlReader.Create (dr, settings);
 		}
+
+		[Test]
+		public void NullResolver ()
+		{
+			XmlReaderSettings settings = new XmlReaderSettings ();
+			settings.XmlResolver = null;
+			using (XmlReader xr = XmlReader.Create ("Test/XmlFiles/simple.xml", settings)) {
+				while (!xr.EOF)
+					xr.Read ();
+			}
+		}
+
+		class ThrowExceptionResolver : XmlResolver
+		{
+			public override ICredentials Credentials {
+				set { }
+			}
+
+			public override object GetEntity (Uri uri, string type, Type expected)
+			{
+				throw new ApplicationException ("error");
+			}
+		}
+
+		[Test]
+		[ExpectedException (typeof (ApplicationException))]
+		public void CustomResolverUsedForXmlStream ()
+		{
+			XmlReaderSettings settings = new XmlReaderSettings ();
+			settings.XmlResolver = new ThrowExceptionResolver ();
+			using (XmlReader xr = XmlReader.Create ("Test/XmlFiles/simple.xml", settings)) {
+				while (!xr.EOF)
+					xr.Read ();
+			}
+		}
+
+		[Test]
+		[ExpectedException (typeof (ApplicationException))]
+		public void ValidationEventHandler ()
+		{
+			XmlReaderSettings settings = new XmlReaderSettings ();
+			settings.Schemas.Add (new XmlSchema ());
+			settings.ValidationType = ValidationType.Schema;
+			settings.ValidationEventHandler += delegate (object o, ValidationEventArgs e) {
+				throw new ApplicationException ();
+			};
+			XmlReader r = XmlReader.Create (
+				new StringReader ("<root/>"), settings);
+			while (!r.EOF)
+				r.Read ();
+		}
 	}
 }
 #endif
