@@ -9,6 +9,7 @@
 //
 using System;
 using System.IO;
+using System.Net;
 using System.Xml;
 using System.Xml.Schema;
 using NUnit.Framework;
@@ -319,5 +320,47 @@ namespace MonoTests.System.Xml
 			while (reader.Read ())
 				;
 		}
+
+		[Test] // bug #79650
+#if NET_2_0
+		// annoyance
+		[ExpectedException (typeof (XmlSchemaValidationException))]
+#else
+		[ExpectedException (typeof (XmlSchemaException))]
+#endif
+		public void EnumerationFacetOnAttribute ()
+		{
+			string xml = "<test mode='NOT A ENUMERATION VALUE' />";
+			XmlSchema schema = XmlSchema.Read (new XmlTextReader ("Test/XmlFiles/xsd/79650.xsd"), null);
+			XmlValidatingReader xvr = new XmlValidatingReader (xml, XmlNodeType.Document, null);
+			xvr.ValidationType = ValidationType.Schema;
+			xvr.Schemas.Add (schema);
+			while (!xvr.EOF)
+				xvr.Read ();
+		}
+
+		class XmlErrorResolver : XmlResolver
+		{
+			public override ICredentials Credentials {
+				set { }
+			}
+
+			public override object GetEntity (Uri uri, string role, Type type)
+			{
+				throw new Exception ();
+			}
+		}
+
+		[Test] // bug #79924
+		public void ValidationTypeNoneIgnoreSchemaLocations ()
+		{
+			string xml = "<project xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xsi:noNamespaceSchemaLocation='nosuchschema.xsd'/>";
+			XmlValidatingReader vr = new XmlValidatingReader (
+				new XmlTextReader (new StringReader (xml)));
+			vr.ValidationType = ValidationType.None;
+			vr.XmlResolver = new XmlErrorResolver ();
+			while (!vr.EOF)
+				vr.Read ();
+		}
 	}
 }
