@@ -38,6 +38,7 @@ using System.Runtime.InteropServices;
 namespace System.Collections.Generic
 {
 	[ComVisible (false)]
+	[Serializable]
 	public class Stack <T> : IEnumerable <T>, ICollection, IEnumerable {
 		
 		T [] data;
@@ -49,6 +50,9 @@ namespace System.Collections.Generic
 
 		public Stack ()
 		{
+#if TARGET_JVM //to work around resize (null) bug
+			data = new T [INITIAL_SIZE];
+#endif
 			defaultCapacity = INITIAL_SIZE;
 		}
 		
@@ -184,6 +188,7 @@ namespace System.Collections.Generic
 			return GetEnumerator ();
 		}
 		
+		[Serializable]
 		public struct Enumerator : IEnumerator <T>, IEnumerator, IDisposable {
 			const int NOT_STARTED = -2;
 			
