@@ -399,12 +399,29 @@ public sealed class TypeDescriptor
 		return GetEditor (component, editorBaseType, false);
 	}
 
-	[MonoTODO]
 	public static object GetEditor (object component, Type editorBaseType, bool noCustomTypeDesc)
 	{
+		if (component == null)
+			throw new ArgumentNullException ("component");
+		if (editorBaseType == null)
+			throw new ArgumentNullException ("editorBaseType");
+		
 		if (!noCustomTypeDesc && (component is ICustomTypeDescriptor))
 			return ((ICustomTypeDescriptor) component).GetEditor (editorBaseType);
-		throw new NotImplementedException ();
+
+		object [] atts = component.GetType ().GetCustomAttributes (typeof (EditorAttribute), true);
+		if (atts.Length == 0)
+			return null;
+		string target = editorBaseType.AssemblyQualifiedName;
+		
+		foreach (EditorAttribute ea in atts){
+			if (ea.EditorBaseTypeName == target){
+				Type t = Type.GetType (ea.EditorTypeName, true);
+
+				return Activator.CreateInstance (t);
+			}
+		}
+		return null;
 	}
 
 	public static EventDescriptorCollection GetEvents (object component)
@@ -705,8 +722,13 @@ public sealed class TypeDescriptor
 			bool cache = true;
 			object[] ats = _infoType.GetCustomAttributes (true);
 			Hashtable t = new Hashtable ();
-			foreach (Attribute at in ats)
-				t [at.TypeId] = at;
+
+			// Filter the custom attributes, so that only the top
+			// level of the same type are left.
+			//
+			for (int i = ats.Length -1; i >= 0; i--) {
+				t [((Attribute) ats[i]).TypeId] = ats[i];
+			}
 					
 			if (comp != null && comp.Site != null) 
 			{
