@@ -82,6 +82,9 @@ namespace System.Diagnostics
 				try {
 					logKey = eventLogKey.OpenSubKey (sourceData.LogName, true);
 					if (logKey == null) {
+						ValidateCustomerLogName (sourceData.LogName, 
+							sourceData.MachineName);
+
 						logKey = eventLogKey.CreateSubKey (sourceData.LogName);
 						logKey.SetValue ("Sources", new string [] { sourceData.LogName,
 							sourceData.Source });
@@ -353,28 +356,22 @@ namespace System.Diagnostics
 			}
 		}
 
-		public override EventLog [] GetEventLogs (string machineName)
-		{
-			using (RegistryKey eventLogKey = GetEventLogKey (machineName, false)) {
-				if (eventLogKey == null) {
-					throw new InvalidOperationException ("TODO");
-				}
-				string [] logNames = eventLogKey.GetSubKeyNames ();
-				EventLog [] eventLogs = new EventLog [logNames.Length];
-				for (int i = 0; i < logNames.Length; i++) {
-					EventLog eventLog = new EventLog (logNames [i], machineName);
-					eventLogs [i] = eventLog;
-				}
-				return eventLogs;
-			}
-		}
-
 		[MonoTODO]
 		protected override string GetLogDisplayName ()
 		{
 			return CoreEventLog.Log;
 		}
 
+		protected override string [] GetLogNames (string machineName)
+		{
+			using (RegistryKey eventLogKey = GetEventLogKey (machineName, true)) {
+				if (eventLogKey == null)
+					return new string [0];
+
+				return eventLogKey.GetSubKeyNames ();
+			}
+		}
+
 		public override string LogNameFromSourceName (string source, string machineName)
 		{
 			using (RegistryKey logKey = FindLogKeyBySource (source, machineName, false)) {
@@ -619,8 +616,7 @@ namespace System.Diagnostics
 
 			try {
 				for (int i = 0; i < replacementStrings.Length; i++) {
-					// TODO: use StringToHGlobalAuto once bug #79117 is fixed
-					arguments [i] = Marshal.StringToHGlobalUni (
+					arguments [i] = Marshal.StringToHGlobalAuto (
 						replacementStrings [i]);
 				}
 
@@ -628,8 +624,7 @@ namespace System.Diagnostics
 					FormatMessageFlags.FromHModule | FormatMessageFlags.AllocateBuffer,
 					msgDllHandle, messageID, 0, ref lpMsgBuf, 0, arguments);
 				if (ret != 0) {
-					// TODO: use PtrToStringAuto once bug #79117 is fixed
-					string sRet = Marshal.PtrToStringUni (lpMsgBuf);
+					string sRet = Marshal.PtrToStringAuto (lpMsgBuf);
 					lpMsgBuf = PInvoke.LocalFree (lpMsgBuf);
 					// remove trailing whitespace (CRLF)
 					return sRet.TrimEnd (null);
@@ -646,6 +641,13 @@ namespace System.Diagnostics
 					}
 				}
 			} finally {
+				// release unmanaged memory allocated for replacement strings
+				for (int i = 0; i < arguments.Length; i++) {
+					IntPtr argument = arguments [i];
+					if (argument != IntPtr.Zero)
+						Marshal.FreeHGlobal (argument);
+				}
+
 				PInvoke.FreeLibrary (msgDllHandle);
 			}
 			return null;
@@ -676,19 +678,21 @@ namespace System.Diagnostics
 			if (hEventLog == IntPtr.Zero) {
 				throw new InvalidOperationException (string.Format (
 					CultureInfo.InvariantCulture, "Event Log '{0}' on computer"
-					+ " '{1}' cannot be opened."), new Win32Exception ());
+					+ " '{1}' cannot be opened.", logName, CoreEventLog.MachineName),
+					new Win32Exception ());
 			}
 			return hEventLog;
 		}
 
 		private IntPtr RegisterEventSource ()
 		{
-			IntPtr hEventLog = PInvoke.OpenEventLog (CoreEventLog.MachineName,
-				CoreEventLog.Source);
+			IntPtr hEventLog = PInvoke.RegisterEventSource (
+				CoreEventLog.MachineName, CoreEventLog.Source);
 			if (hEventLog == IntPtr.Zero) {
 				throw new InvalidOperationException (string.Format (
-					CultureInfo.InvariantCulture, "Event Log '{0}' on computer"
-					+ " '{1}' cannot be opened."), new Win32Exception ());
+					CultureInfo.InvariantCulture, "Event source '{0}' on computer"
+					+ " '{1}' cannot be opened.", CoreEventLog.Source,
+					CoreEventLog.MachineName), new Win32Exception ());
 			}
 			return hEventLog;
 		}
@@ -722,7 +726,7 @@ namespace System.Diagnostics
 			[DllImport ("kernel32.dll", SetLastError=true)]
 			public static extern IntPtr LocalFree (IntPtr hMem);
 
-			[DllImport ("advapi32.dll", CharSet=CharSet.Auto, SetLastError=true)]
+			[DllImport ("advapi32.dll", SetLastError=true)]
 			public static extern bool LookupAccountSid (
 				string lpSystemName,
 				[MarshalAs (UnmanagedType.LPArray)] byte [] Sid,
@@ -738,7 +742,7 @@ namespace System.Diagnostics
 			[DllImport ("advapi32.dll", SetLastError=true)]
 			public static extern IntPtr RegisterEventSource (string machineName, string sourceName);
 
-			[DllImport ("Advapi32.dll", SetLastError = true)]
+			[DllImport ("Advapi32.dll", SetLastError=true)]
 			public static extern int ReportEvent (IntPtr hHandle, ushort wType,
 				ushort wCategory, uint dwEventID, IntPtr sid, ushort wNumStrings,
 				uint dwDataSize, string [] lpStrings, byte [] lpRawData);
