@@ -45,7 +45,7 @@ namespace System.IO.Compression {
 			Z_BLOCK         = 5,
 		};
 
-
+#if !TARGET_JVM
 		[DllImport("MonoPosixHelper")]
 		static extern IntPtr create_z_stream(CompressionMode compress, bool gzip);
 		[DllImport("MonoPosixHelper")]
@@ -64,7 +64,7 @@ namespace System.IO.Compression {
 		static extern ZReturnConsts z_stream_inflate(IntPtr z_stream, ref int avail_out);
 		[DllImport("MonoPosixHelper")]
 		static extern ZReturnConsts z_stream_deflate(IntPtr z_stream, ZFlushConsts flush, IntPtr next_out, ref int avail_out);
-
+#endif
 		delegate int  ReadMethod (byte[] array, int offset, int count);
 		delegate void WriteMethod(byte[] array, int offset, int count);
 
@@ -87,7 +87,7 @@ namespace System.IO.Compression {
 			this.sized_buffer = Marshal.AllocHGlobal (BUFFER_SIZE);
 			this.z_stream = create_z_stream (mode, gzip);
 			if (z_stream == IntPtr.Zero) {
-				throw new OutOfMemoryException ();
+				throw new NotImplementedException ("Failed to initialize zlib. You probably have an old zlib installed. Version 1.2.0.4 or later is required.");
 			}
 			this.open = true;
 			if (mode == CompressionMode.Compress) {
