@@ -51,7 +51,11 @@ namespace System.Net.Sockets
 			Receive,
 			ReceiveFrom,
 			Send,
-			SendTo
+			SendTo,
+			UsedInManaged1,
+			UsedInManaged2,
+			UsedInProcess,
+			UsedInConsole2
 		}
 
 		[StructLayout (LayoutKind.Sequential)]
@@ -285,8 +289,8 @@ namespace System.Net.Sockets
 			{
 				try {
 					if (!result.Sock.Blocking) {
-						result.Sock.Poll (-1, SelectMode.SelectWrite);
-						int success = (int)result.Sock.GetSocketOption (SocketOptionLevel.Socket, SocketOptionName.Error);
+						int success;
+						result.Sock.Poll (-1, SelectMode.SelectWrite, out success);
 						if (success == 0) {
 							result.Sock.connected = true;
 						} else {
@@ -429,12 +433,12 @@ namespace System.Net.Sockets
 
 			sockets.Add (null);
 		}
-
+#if !TARGET_JVM
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static void Select_internal (ref Socket [] sockets,
 							int microSeconds,
 							out int error);
-
+#endif
 		public static void Select (IList checkRead, IList checkWrite, IList checkError, int microSeconds)
 		{
 			ArrayList list = new ArrayList ();
@@ -483,7 +487,12 @@ namespace System.Net.Sockets
 				}
 
 				if (currentList != null) {
-					sock.connected = true;
+					if (currentList == checkWrite) {
+						if ((int)sock.GetSocketOption (SocketOptionLevel.Socket, SocketOptionName.Error) == 0) {
+							sock.connected = true;
+						}
+					}
+					
 					currentList.Add (sock);
 				}
 			}
@@ -524,14 +533,14 @@ namespace System.Net.Sockets
 			socket=sock;
 			connected=true;
 		}
-		
+#if !TARGET_JVM
 		// Creates a new system socket, returning the handle
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern IntPtr Socket_internal(AddressFamily family,
 						      SocketType type,
 						      ProtocolType proto,
 						      out int error);
-		
+#endif		
 		public Socket(AddressFamily family, SocketType type,
 			      ProtocolType proto) {
 			address_family=family;
@@ -551,11 +560,12 @@ namespace System.Net.Sockets
 				return(address_family);
 			}
 		}
-
+#if !TARGET_JVM
 		// Returns the amount of data waiting to be read on socket
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static int Available_internal(IntPtr socket,
 							     out int error);
+#endif
 		
 		public int Available {
 			get {
@@ -573,12 +583,12 @@ namespace System.Net.Sockets
 				return(ret);
 			}
 		}
-
+#if !TARGET_JVM
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static void Blocking_internal(IntPtr socket,
 							     bool block,
 							     out int error);
-
+#endif
 		public bool Blocking {
 			get {
 				return(blocking);
@@ -610,12 +620,12 @@ namespace System.Net.Sockets
 				return(socket);
 			}
 		}
-
+#if !TARGET_JVM
 		// Returns the local endpoint details in addr and port
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static SocketAddress LocalEndPoint_internal(IntPtr socket, out int error);
-
-		[MonoTODO("Support non-IP endpoints")]
+#endif
+		// Wish:  support non-IP endpoints.
 		public EndPoint LocalEndPoint {
 			get {
 				if (disposed && closed)
@@ -653,7 +663,9 @@ namespace System.Net.Sockets
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static SocketAddress RemoteEndPoint_internal(IntPtr socket, out int error);
 
-		[MonoTODO("Support non-IP endpoints")]
+		//
+		// Wish: Support non-IP endpoints
+		//
 		public EndPoint RemoteEndPoint {
 			get {
 				if (disposed && closed)
@@ -1341,13 +1353,51 @@ namespace System.Net.Sockets
 			if (error != 0)
 				throw new SocketException (error);
 
-			if (result == true) {
+			if (mode == SelectMode.SelectWrite && result == true) {
 				/* Update the connected state; for
 				 * non-blocking Connect()s this is
 				 * when we can find out that the
 				 * connect succeeded.
 				 */
-				connected = true;
+				if ((int)GetSocketOption (SocketOptionLevel.Socket, SocketOptionName.Error) == 0) {
+					connected = true;
+				}
+			}
+			
+			return result;
+		}
+
+		/* This overload is needed as the async Connect method
+		 * also needs to check the socket error status, but
+		 * getsockopt(..., SO_ERROR) clears the error.
+		 */
+		private bool Poll (int time_us, SelectMode mode,
+				   out int socket_error)
+		{
+			if (disposed && closed)
+				throw new ObjectDisposedException (GetType ().ToString ());
+
+			if (mode != SelectMode.SelectRead &&
+			    mode != SelectMode.SelectWrite &&
+			    mode != SelectMode.SelectError)
+				throw new NotSupportedException ("'mode' parameter is not valid.");
+
+			int error;
+			bool result = Poll_internal (socket, mode, time_us, out error);
+			if (error != 0)
+				throw new SocketException (error);
+
+			socket_error = (int)GetSocketOption (SocketOptionLevel.Socket, SocketOptionName.Error);
+			
+			if (mode == SelectMode.SelectWrite && result == true) {
+				/* Update the connected state; for
+				 * non-blocking Connect()s this is
+				 * when we can find out that the
+				 * connect succeeded.
+				 */
+				if (socket_error == 0) {
+					connected = true;
+				}
 			}
 			
 			return result;
