@@ -305,7 +305,11 @@ namespace System.Net
 
 			if (e == null) { // At least we now where it comes from
 				try {
+#if TARGET_JVM
+					throw new Exception ();
+#else
 					throw new Exception (new System.Diagnostics.StackTrace ().ToString ());
+#endif
 				} catch (Exception e2) {
 					e = e2;
 				}
@@ -425,20 +429,7 @@ namespace System.Net
 			certsAvailable = (server != null);
 		}
 
-		delegate void MyDelegate (object o);
-
 		internal static void InitRead (object state)
-		{
-			WebConnection cnc = (WebConnection) state;
-			if (!cnc.ssl) {
-				InitRead2 (state);
-			} else {
-				MyDelegate d = new MyDelegate (InitRead2);		
-				d.BeginInvoke (state, null, null);
-			}
-		}
-
-		static void InitRead2 (object state)
 		{
 			WebConnection cnc = (WebConnection) state;
 			Stream ns = cnc.nstream;
@@ -818,7 +809,7 @@ namespace System.Net
 			try {
 				nstream.EndWrite (result);
 				return true;
-			} catch (Exception e) {
+			} catch {
 				status = WebExceptionStatus.SendFailure;
 				return false;
 			}
