@@ -67,73 +67,125 @@ namespace System.Net
 		protected WebRequest (SerializationInfo serializationInfo, StreamingContext streamingContext) 
 		{
 		}
+
+		static Exception GetMustImplement ()
+		{
+			return new NotImplementedException ("This method must be implemented in derived classes");
+		}
 		
 		// Properties
 		
 		public virtual string ConnectionGroupName { 
-			get { throw new NotImplementedException (); }
-			set { throw new NotImplementedException (); }
+			get { throw GetMustImplement (); }
+			set { throw GetMustImplement (); }
 		}
 		
 		public virtual long ContentLength { 
-			get { throw new NotImplementedException (); }
-			set { throw new NotImplementedException (); }
+			get { throw GetMustImplement (); }
+			set { throw GetMustImplement (); }
 		}
 		
 		public virtual string ContentType { 
-			get { throw new NotImplementedException (); }
-			set { throw new NotImplementedException (); }
+			get { throw GetMustImplement (); }
+			set { throw GetMustImplement (); }
 		}
 		
 		public virtual ICredentials Credentials { 
-			get { throw new NotImplementedException (); }
-			set { throw new NotImplementedException (); }
+			get { throw GetMustImplement (); }
+			set { throw GetMustImplement (); }
 		}
 		
 		public virtual WebHeaderCollection Headers { 
-			get { throw new NotImplementedException (); }
-			set { throw new NotImplementedException (); }
+			get { throw GetMustImplement (); }
+			set { throw GetMustImplement (); }
 		}
 		
 		public virtual string Method { 
-			get { throw new NotImplementedException (); }
-			set { throw new NotImplementedException (); }
+			get { throw GetMustImplement (); }
+			set { throw GetMustImplement (); }
 		}
 		
 		public virtual bool PreAuthenticate { 
-			get { throw new NotImplementedException (); }
-			set { throw new NotImplementedException (); }
+			get { throw GetMustImplement (); }
+			set { throw GetMustImplement (); }
 		}
 		
 		public virtual IWebProxy Proxy { 
-			get { throw new NotImplementedException (); }
-			set { throw new NotImplementedException (); }
+			get { throw GetMustImplement (); }
+			set { throw GetMustImplement (); }
 		}
 		
 		public virtual Uri RequestUri { 
-			get { throw new NotImplementedException (); }
+			get { throw GetMustImplement (); }
 		}
 		
 		public virtual int Timeout { 
-			get { throw new NotImplementedException (); }
-			set { throw new NotImplementedException (); }
+			get { throw GetMustImplement (); }
+			set { throw GetMustImplement (); }
 		}
 		
+#if NET_2_0
+		volatile static IWebProxy proxy;
+		static readonly object lockobj = new object ();
+		
+		public static IWebProxy DefaultWebProxy {
+			get {
+				lock (lockobj) {
+					if (proxy == null)
+						proxy = GetDefaultWebProxy ();
+					return proxy;
+				}
+			}
+			set {
+				if (value == null)
+					throw new ArgumentNullException ("WebRequest.DefaultWebProxy",
+							"null IWebProxy not allowed.");
+				proxy = value;
+			}
+		}
+		
+		[MonoTODO("Needs to respect Module, Proxy.AutoDetect, and Proxy.ScriptLocation config settings")]
+		static IWebProxy GetDefaultWebProxy ()
+		{
+			WebProxy p = null;
+			
+#if CONFIGURATION_DEP
+			DefaultProxySection sec = ConfigurationManager.GetSection ("system.net/defaultProxy") as DefaultProxySection;
+			if (sec == null)
+				return GetSystemWebProxy ();
+			
+			ProxyElement pe = sec.Proxy;
+			
+			if ((pe.UseSystemDefault != ProxyElement.UseSystemDefaultValues.False) && (pe.ProxyAddress == null))
+				p = (WebProxy) GetSystemWebProxy ();
+			else
+				p = new WebProxy ();
+			
+			if (pe.ProxyAddress != null)
+				p.Address = pe.ProxyAddress;
+			
+			if (pe.BypassOnLocal != ProxyElement.BypassOnLocalValues.Unspecified)
+				p.BypassProxyOnLocal = (pe.BypassOnLocal == ProxyElement.BypassOnLocalValues.True);
+#endif
+			return p;
+		}		
+#endif
+
 		// Methods
 		
 		public virtual void Abort()
 		{
-			throw new NotImplementedException ();
+			throw GetMustImplement ();
 		}
 		
 		public virtual IAsyncResult BeginGetRequestStream (AsyncCallback callback, object state) 
 		{
-			throw new NotImplementedException ();
+			throw GetMustImplement ();
 		}
 		
 		public virtual IAsyncResult BeginGetResponse (AsyncCallback callback, object state)
 		{
-			throw new NotImplementedException ();
+			throw GetMustImplement ();
 		}
 
 		public static WebRequest Create (string requestUriString) 
@@ -159,24 +211,39 @@ namespace System.Net
 
 		public virtual Stream EndGetRequestStream (IAsyncResult asyncResult)
 		{
-			throw new NotImplementedException ();
+			throw GetMustImplement ();
 		}
 		
 		public virtual WebResponse EndGetResponse (IAsyncResult asyncResult)
 		{
-			throw new NotImplementedException ();
+			throw GetMustImplement ();
 		}
 		
 		public virtual Stream GetRequestStream()
 		{
-			throw new NotImplementedException ();
+			throw GetMustImplement ();
 		}
 		
 		public virtual WebResponse GetResponse()
 		{
-			throw new NotImplementedException ();
+			throw GetMustImplement ();
 		}
 		
+#if NET_2_0
+		[MonoTODO("Look in other places for proxy config info")]
+		public static IWebProxy GetSystemWebProxy ()
+		{
+			string address = Environment.GetEnvironmentVariable ("http_proxy");
+			if (address != null) {
+				try {
+					WebProxy p = new WebProxy (address);
+					return p;
+				} catch (UriFormatException) {}
+			}
+			return new WebProxy ();
+		}
+#endif
+
 		void ISerializable.GetObjectData (SerializationInfo serializationInfo,
 		   				  StreamingContext streamingContext)
 		{
