@@ -32,6 +32,19 @@ namespace MonoTests.System.ComponentModel
 		public void Dispose () { }
 	}
 
+	class MyOtherDesigner: IDesigner
+	{
+		public MyOtherDesigner()
+		{
+		}
+
+		public IComponent Component {get {return null; } }
+		public DesignerVerbCollection Verbs { get {return null; } }
+		public void DoDefaultAction () { }
+		public void Initialize (IComponent component) { }
+		public void Dispose () { }
+	}
+	
 	class MySite: ISite
 	{ 
 		public IComponent Component { get {  return null; } }
@@ -136,7 +149,7 @@ namespace MonoTests.System.ComponentModel
 		}
 		
 		[DescriptionAttribute ("test")]
-		public string TestProperty
+		public virtual string TestProperty
 		{
 			get { return prop; }
 			set { prop = value; }
@@ -149,6 +162,29 @@ namespace MonoTests.System.ComponentModel
 		}
 	}
 
+	[DescriptionAttribute ("my test derived component")]
+	[DesignerAttribute (typeof(MyOtherDesigner))]
+	public class MyDerivedComponent: MyComponent
+	{
+		string prop;
+		
+		public MyDerivedComponent  ()
+		{
+		}
+		
+		public MyDerivedComponent (ISite site) : base (site)
+		{
+		}
+		
+		[DescriptionAttribute ("test derived")]
+		public override string TestProperty
+		{
+			get { return prop; }
+			set { prop = value; }
+		}
+	}
+	
+
 	[DefaultProperty("AnotherProperty")]
 	[DefaultEvent("AnotherEvent")]
 	[DescriptionAttribute ("my test component")]
@@ -198,7 +234,84 @@ namespace MonoTests.System.ComponentModel
 	{
 		public int TestVal;
 	}
-	
+
+	public class TestCustomTypeDescriptor : ICustomTypeDescriptor
+	{
+		public string methods_called = "";
+
+		public void ResetMethodsCalled ()
+		{
+			methods_called = "";
+		}
+
+		public TypeConverter GetConverter()
+		{
+			return new StringConverter();
+		}
+
+		public EventDescriptorCollection GetEvents(Attribute[] attributes)
+		{
+			methods_called += "1";
+			return null;
+		}
+
+		public EventDescriptorCollection GetEvents()
+		{
+			methods_called += "2";
+			return null;
+		}
+
+		public string GetComponentName()
+		{
+			return "MyComponentnName";
+		}
+
+		public object GetPropertyOwner(PropertyDescriptor pd)
+		{
+			return this;
+		}
+
+		public AttributeCollection GetAttributes()
+		{
+			methods_called += "3";
+			return null;
+		}
+
+		public PropertyDescriptorCollection GetProperties(Attribute[] attributes)
+		{
+			methods_called += "4";
+			return new PropertyDescriptorCollection(new PropertyDescriptor[0]);
+		}
+
+		public PropertyDescriptorCollection GetProperties()
+		{
+			methods_called += "5";
+			return new PropertyDescriptorCollection(new PropertyDescriptor[0]);
+		}
+
+		public object GetEditor(Type editorBaseType)
+		{
+			return null;
+		}
+
+		public PropertyDescriptor GetDefaultProperty()
+		{
+			methods_called += "6";
+			return null;
+		}
+
+		public EventDescriptor GetDefaultEvent()
+		{
+			methods_called += "7";
+			return null;
+		}
+
+		public string GetClassName()
+		{
+			return this.GetType().Name;
+		}
+	}
+
 	[TestFixture]
 	public class TypeDescriptorTests: Assertion
 	{
@@ -206,6 +319,48 @@ namespace MonoTests.System.ComponentModel
 		MyComponent sitedcom = new MyComponent (new MySite ());
 		AnotherComponent anothercom = new AnotherComponent ();
 		
+		[Test]
+		public void TestICustomTypeDescriptor ()
+		{
+			TestCustomTypeDescriptor test = new TestCustomTypeDescriptor ();
+
+			PropertyDescriptorCollection props;
+			PropertyDescriptor prop;
+			EventDescriptorCollection events;
+
+			test.ResetMethodsCalled ();
+			props = TypeDescriptor.GetProperties (test);
+			AssertEquals ("t1", "5", test.methods_called);
+
+			test.ResetMethodsCalled ();
+			props = TypeDescriptor.GetProperties (test, new Attribute[0]);
+			AssertEquals ("t2", "4", test.methods_called);
+
+			test.ResetMethodsCalled ();
+			props = TypeDescriptor.GetProperties (test, new Attribute[0], false);
+			AssertEquals ("t3", "4", test.methods_called);
+
+			test.ResetMethodsCalled ();
+			props = TypeDescriptor.GetProperties (test, false);
+			AssertEquals ("t4", "5", test.methods_called);
+
+			test.ResetMethodsCalled ();
+			prop = TypeDescriptor.GetDefaultProperty (test);
+			AssertEquals ("t5", "6", test.methods_called);
+
+			test.ResetMethodsCalled ();
+			events = TypeDescriptor.GetEvents (test);
+			AssertEquals ("t6", "2", test.methods_called);
+
+			test.ResetMethodsCalled ();
+			events = TypeDescriptor.GetEvents (test, new Attribute[0]);
+			AssertEquals ("t7", "1", test.methods_called);
+
+			test.ResetMethodsCalled ();
+			events = TypeDescriptor.GetEvents (test, false);
+			AssertEquals ("t8", "2", test.methods_called);
+		}
+
 		[Test]
 		public void TestCreateDesigner ()
 		{
@@ -256,6 +411,13 @@ namespace MonoTests.System.ComponentModel
 			Assert ("t10", col[typeof(DescriptionAttribute)] != null);
 			Assert ("t11", col[typeof(DesignerAttribute)] != null);
 			Assert ("t12", col[typeof(EditorAttribute)] != null);
+
+			col = TypeDescriptor.GetAttributes (typeof (MyDerivedComponent));
+			Assert ("t13", col[typeof(DesignerAttribute)] != null);
+			Assert ("t14", col[typeof(DescriptionAttribute)] != null);
+			DesignerAttribute attribute = col[typeof(DesignerAttribute)] as DesignerAttribute;
+			Assert ("t15", attribute != null);
+			AssertEquals ("t16", attribute.DesignerTypeName, typeof(MyOtherDesigner).AssemblyQualifiedName);
 		}
 		
 		[Test]
