@@ -1088,6 +1088,14 @@ namespace MonoTests.System
 			Assert ("#2 known to fail with ms.net 1.x", !uri1.Equals (uri2));
 		}
 
+		[Test]
+		public void Equals3 ()
+		{
+			Uri uri1 = new Uri ("svn+ssh://atsushi@mono-cvs.ximian.com");
+			Uri uri2 = new Uri ("svn+ssh://anonymous@mono-cvs.ximian.com");
+			Assert (uri1.Equals (uri2));
+		}
+
 		[Test]
 		public void TestEquals2 ()
 		{
