@@ -32,14 +32,20 @@ using System;
 
 namespace System.Diagnostics {
 
-	[AttributeUsageAttribute(AttributeTargets.Class | AttributeTargets.Struct | AttributeTargets.Assembly)]	
+	[AttributeUsageAttribute(AttributeTargets.Class |
+				 AttributeTargets.Struct |
+				 AttributeTargets.Enum |
+				 AttributeTargets.Field |
+				 AttributeTargets.Delegate |
+				 AttributeTargets.Property |
+				 AttributeTargets.Assembly, AllowMultiple=true)]	
 #if NET_2_0
 	public sealed class DebuggerDisplayAttribute : Attribute
 #else
 	internal sealed class DebuggerDisplayAttribute : Attribute
 #endif
 	{
-		string value;
+		string value, type, name;
 		string target_type_name;
 		Type target_type;
 
@@ -71,6 +77,26 @@ namespace System.Diagnostics {
 				target_type_name = value;
 			}
 		}
+
+		public string Type {
+			get {
+				return type;
+			}
+
+			set {
+				type = value;
+			}
+		}
+
+		public string Name {
+			get {
+				return name;
+			}
+
+			set {
+				type = name;
+			}
+		}
 	}
 
 }
