@@ -43,7 +43,7 @@ using System.IO;
 /// </para>
 /// </remarks>
 [Serializable]
-[MonoTODO ("Fix serialization compatibility with MS.NET")]
+[MonoTODO ("Serialization format not compatible with.NET")]
 public class HebrewCalendar : Calendar {
 	/// <summary>
 	/// Constructor.
