@@ -42,7 +42,7 @@ using System;
 /// </para>
 /// </remarks>
 [Serializable]
-[MonoTODO ("Fix serialization compatibility with MS.NET")]
+[MonoTODO ("Serialization format not compatible with .NET")]
 public class JulianCalendar : Calendar {
 	/// <summary>
 	/// Default constructor.
@@ -478,7 +478,7 @@ public class JulianCalendar : Calendar {
 			hour, minute, second, milliseconds);
 	}
 
-	[MonoTODO]
+	[MonoTODO("Not supported")]
 	public override int ToFourDigitYear(int year)
 	{
 		throw new NotImplementedException();
