@@ -47,6 +47,9 @@
 
 namespace System.Globalization {
 
+#if NET_2_0
+	[System.Runtime.InteropServices.ComVisible(false)]
+#endif
 	[Serializable]
 	public sealed class NumberFormatInfo : ICloneable, IFormatProvider {
 		private bool isReadOnly;
@@ -866,10 +869,7 @@ namespace System.Globalization {
 
 		public object GetFormat (Type formatType) 
 		{
-			// work around http://bugzilla.ximian.com/show_bug.cgi?id=55978
-			// the comparison fails because formatType likely comes from another domain
-			//return (formatType == typeof (NumberFormatInfo)) ? this : null;
-			return this;
+			return (formatType == typeof (NumberFormatInfo)) ? this : null;
 		}
 		
 		public object Clone () 
