@@ -68,9 +68,6 @@ namespace System.Globalization {
 		[NonSerialized]
 		readonly CultureInfo ci;
 
-		[NonSerialized]
-		readonly CultureInfo parentCulture;
-
 		[NonSerialized]
 		readonly bool handleDotI;
 
@@ -89,9 +86,8 @@ namespace System.Globalization {
 			}
 
 			CultureInfo tmp = ci;
-			while (tmp.Parent != null && tmp.Parent != tmp && tmp.Parent.LCID != 0x7F)
+			while (tmp.Parent != null && tmp.Parent.LCID != 0x7F && tmp.Parent != tmp)
 				tmp = tmp.Parent;
-			parentCulture = tmp;
 
 			if (tmp != null) {
 				switch (tmp.LCID) {
@@ -112,7 +108,6 @@ namespace System.Globalization {
 			m_listSeparator = textInfo.ListSeparator;
 			customCultureName = textInfo.CultureName;
 			ci = textInfo.ci;
-			parentCulture = textInfo.parentCulture;
 			handleDotI = textInfo.handleDotI;
 			data = textInfo.data;
 		}
