@@ -71,6 +71,11 @@ namespace System.IO
 
 			if (path.Trim ().Length == 0)
 				throw new ArgumentException ("Only blank characters in path");
+
+#if NET_2_0
+			if (File.Exists(path))
+				throw new IOException ("Cannot create " + path + " because a file with the same name already exists.");
+#endif
 			
 			// LAMESPEC: with .net 1.0 version this throw NotSupportedException and msdn says so too
 			// but v1.1 throws ArgumentException.
@@ -127,8 +132,19 @@ namespace System.IO
 				throw new NotSupportedException ("Only ':' In path");
 
 			MonoIOError error;
+			bool success;
 			
-			if (!MonoIO.RemoveDirectory (path, out error)) {
+			if (MonoIO.ExistsSymlink (path, out error)) {
+				/* RemoveDirectory maps to rmdir()
+				 * which fails on symlinks (ENOTDIR)
+				 */
+				success = MonoIO.DeleteFile (path, out error);
+			} else {
+				success = MonoIO.RemoveDirectory (path,
+								  out error);
+			}
+			
+			if (!success) {
 				/*
 				 * FIXME:
 				 * In io-layer/io.c rmdir returns error_file_not_found if directory does not exists.
@@ -143,8 +159,15 @@ namespace System.IO
 
 		static void RecursiveDelete (string path)
 		{
-			foreach (string dir in GetDirectories (path))
-				RecursiveDelete (dir);
+			MonoIOError error;
+			
+			foreach (string dir in GetDirectories (path)) {
+				if (MonoIO.ExistsSymlink (dir, out error)) {
+					MonoIO.DeleteFile (dir, out error);
+				} else {
+					RecursiveDelete (dir);
+				}
+			}
 
 			foreach (string file in GetFiles (path))
 				File.Delete (file);
