@@ -63,6 +63,14 @@ namespace System.IO
 			get;
 		}
 
+#if NET_2_0
+		public virtual bool CanTimeout {
+			get {
+				return false;
+			}
+		}
+#endif
+
 		public abstract long Length
 		{
 			get;
@@ -75,26 +83,56 @@ namespace System.IO
 		}
 
 
-		public virtual void Close ()
-		{
-		}
-
-		void IDisposable.Dispose ()
+#if NET_2_0
+		// 2.0 version of Dispose.
+		public void Dispose ()
 		{
 			Close ();
 		}
 
-#if NET_2_0
-		public void Dispose ()
+		// 2.0 version of Dispose.
+		protected virtual void Dispose (bool disposing)
+		{
+			// nothing.
+		}
+
+		//
+		// 2.0 version of Close (): calls Dispose (true)
+		//
+		public virtual void Close ()
 		{
 			Dispose (true);
 		}
 
-		protected virtual void Dispose (bool disposing)
+		public virtual int ReadTimeout {
+			get {
+				throw new InvalidOperationException ("Timeouts are not supported on this stream.");
+			}
+			set {
+				throw new InvalidOperationException ("Timeouts are not supported on this stream.");
+			}
+		}
+
+		public virtual int WriteTimeout {
+			get {
+				throw new InvalidOperationException ("Timeouts are not supported on this stream.");
+			}
+			set {
+				throw new InvalidOperationException ("Timeouts are not supported on this stream.");
+			}
+		}
+#else
+		// 1.1 version of Close
+		public virtual void Close ()
+		{
+			// nothing
+		}
+
+#endif
+		void IDisposable.Dispose ()
 		{
 			Close ();
 		}
-#endif
 
 		protected virtual WaitHandle CreateWaitHandle()
 		{
