@@ -784,7 +784,7 @@ namespace System.Reflection.Emit {
 			emit_int (token_gen.GetToken (type));
 		}
 
-		[MonoTODO ("Do something about varargs method")]
+		[MonoTODO ("vararg methods are not supported")]
 		public void EmitCall (OpCode opcode, MethodInfo methodinfo, Type[] optionalParamTypes)
 		{
 			if (methodinfo == null)
@@ -951,7 +951,7 @@ namespace System.Reflection.Emit {
 			Emit (OpCodes.Throw);
 		}
 
-		[MonoTODO]
+		[MonoTODO("Not implemented")]
 		public void UsingNamespace (String usingNamespace)
 		{
 			throw new NotImplementedException ();
