@@ -170,7 +170,6 @@ namespace System.Reflection.Emit {
 			return false;
 		}*/
 
-		[MonoTODO]
 		public override Type UnderlyingSystemType {
 			get {
 
@@ -502,7 +501,8 @@ namespace System.Reflection.Emit {
 			check_not_created ();
 			if (IsInterface && (
 				!((attributes & MethodAttributes.Abstract) != 0) || 
-				!((attributes & MethodAttributes.Virtual) != 0)))
+				!((attributes & MethodAttributes.Virtual) != 0)) &&
+				!(((attributes & MethodAttributes.Static) != 0)))
 				throw new ArgumentException ("attributes", "Interface method must be abstract and virtual.");
 
 			if (returnType == null)
@@ -1363,10 +1363,11 @@ namespace System.Reflection.Emit {
 			}
 		}
 		public void SetParent (Type parentType) {
-			if (parentType == null)
-				throw new ArgumentNullException ("parentType");
 			check_not_created ();
 
+			if (parentType == null && (attrs & TypeAttributes.Interface) == 0)
+				throw new ArgumentNullException ("parentType");
+			
 			parent = parentType;
 			// will just set the parent-related bits if called a second time
 			setup_internal_class (this);
