@@ -126,7 +126,9 @@ namespace System.Reflection
 				return null;
 			}
 
-			[MonoTODO]
+			//
+			// FIXME: There was a MonoTODO, but it does not explain what the problem is
+			// 
 			public override MethodBase BindToMethod (BindingFlags bindingAttr, MethodBase[] match, ref object[] args, ParameterModifier[] modifiers, CultureInfo culture, string[] names, out object state)
 			{
 				Type[] types;
@@ -148,16 +150,17 @@ namespace System.Reflection
 
 			void ReorderParameters (string [] names, ref object [] args, MethodBase selected)
 			{
+				object [] newArgs = new object [args.Length];
+				Array.Copy (args, newArgs, args.Length);
 				ParameterInfo [] plist = selected.GetParameters ();
 				for (int n = 0; n < names.Length; n++)
 					for (int p = 0; p < plist.Length; p++) {
 						if (names [n] == plist [p].Name) {
-							object o = args [n];
-							args [n] = args [p];
-							args [p] = o;
+							newArgs [p] = args [n];
+							break;
 						}
-						break;
 					}
+				Array.Copy (newArgs, args, args.Length);
 			}
 
 			static bool IsArrayAssignable (Type object_type, Type target_type)
@@ -190,7 +193,7 @@ namespace System.Reflection
 				return null;
 			}
 
-			[MonoTODO]
+			[MonoTODO ("This method does not do anything in Mono")]
 			public override void ReorderArgumentArray (ref object[] args, object state)
 			{
 				//do nothing until we support named arguments
