@@ -81,6 +81,14 @@ namespace System.Reflection {
 			return get_base_definition (this);
 		}
 
+#if NET_2_0 || BOOTSTRAP_NET_2_0
+		public override ParameterInfo ReturnParameter {
+			get {
+				return new ParameterInfo (ReturnType, this, MonoMethodInfo.get_retval_marshal (mhandle));
+			}
+		}
+#endif
+
 		public override Type ReturnType {
 			get {
 				MonoMethodInfo info;
@@ -230,11 +238,14 @@ namespace System.Reflection {
 				if (i > 0)
 					parms = parms + ", ";
 				Type pt = p[i].ParameterType;
+				bool byref = pt.IsByRef;
+				if (byref)
+					pt = pt.GetElementType ();
 				if (pt.IsClass && pt.Namespace != "")
 					parms = parms + pt.Namespace + "." + pt.Name;
 				else
 					parms = parms + pt.Name;
-				if (pt.IsByRef)
+				if (byref)
 					parms += " ByRef";
 			}
 			if (ReturnType.IsClass && ReturnType.Namespace != "")
