@@ -73,7 +73,25 @@ namespace System.Reflection
 			this.AttrsImpl = ParameterAttributes.Retval;
 			this.marshalAs = marshalAs;
 		}
-		
+
+		public override string ToString() {
+			Type elementType = ClassImpl;
+			while (elementType.HasElementType) {
+					elementType = elementType.GetElementType();
+			}
+			bool useShort = elementType.IsPrimitive || ClassImpl == typeof(void)
+				|| ClassImpl.Namespace == MemberImpl.DeclaringType.Namespace;
+			string result = useShort
+				? ClassImpl.Name
+				: ClassImpl.FullName;
+			// MS.NET seems to skip this check and produce an extra space for return types
+			if (!IsRetval) {
+				result += ' ';
+				result += NameImpl;
+			}
+			return result;
+		}
+
 		public virtual Type ParameterType {
 			get {return ClassImpl;}
 		}
@@ -85,23 +103,53 @@ namespace System.Reflection
 		}
 
 		public bool IsIn {
-			get {return (AttrsImpl & ParameterAttributes.In) != 0;}
+			get {
+#if NET_2_0
+				return (Attributes & ParameterAttributes.In) != 0;
+#else
+				return (AttrsImpl & ParameterAttributes.In) != 0;
+#endif
+			}
 		}
 
 		public bool IsLcid {
-			get {return (AttrsImpl & ParameterAttributes.Lcid) != 0;}
+			get {
+#if NET_2_0
+				return (Attributes & ParameterAttributes.Lcid) != 0;
+#else
+				return (AttrsImpl & ParameterAttributes.Lcid) != 0;
+#endif
+			}
 		}
 
 		public bool IsOptional {
-			get {return (AttrsImpl & ParameterAttributes.Optional) != 0;}
+			get {
+#if NET_2_0
+				return (Attributes & ParameterAttributes.Optional) != 0;
+#else
+				return (AttrsImpl & ParameterAttributes.Optional) != 0;
+#endif
+			}
 		}
 
 		public bool IsOut {
-			get {return (AttrsImpl & ParameterAttributes.Out) != 0;}
+			get {
+#if NET_2_0
+				return (Attributes & ParameterAttributes.Out) != 0;
+#else
+				return (AttrsImpl & ParameterAttributes.Out) != 0;
+#endif
+			}
 		}
 
 		public bool IsRetval {
-			get {return (AttrsImpl & ParameterAttributes.Retval) != 0;}
+			get {
+#if NET_2_0
+				return (Attributes & ParameterAttributes.Retval) != 0;
+#else
+				return (AttrsImpl & ParameterAttributes.Retval) != 0;
+#endif
+			}
 		}
 
 		public virtual MemberInfo Member {
@@ -185,12 +233,12 @@ namespace System.Reflection
 			}
 		}
 
-		[MonoTODO]
+		[MonoTODO("Not implemented")]
 		public virtual Type[] GetOptionalCustomModifiers () {
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO("Not implemented")]
 		public virtual Type[] GetRequiredCustomModifiers () {
 			throw new NotImplementedException ();
 		}
