@@ -73,24 +73,15 @@ namespace System.Runtime.Remoting
 		public static string ApplicationId 
 		{
 			get 
-			{ 
-				applicationID = AppDomain.CurrentDomain.SetupInformation.ApplicationName; 
+			{
+				applicationID = ApplicationName; 
 				return applicationID;
 			}
 		}
 		
 		public static string ApplicationName 
 		{
-			get { 
-				try {
-					applicationName = AppDomain.CurrentDomain.SetupInformation.ApplicationName; 
-				}
-				catch (Exception e) {
-					throw e;
-				}
-				// We return null if the application name has not been set.
-				return null;
-			}
+			get { return applicationName; }
 			set { applicationName = value; }
 		}
 		
@@ -134,7 +125,7 @@ namespace System.Runtime.Remoting
 			}
 			catch (Exception ex)
 			{
-				throw new RemotingException ("Configuration file '" + filename + "' could not be loaded: " + ex.Message);
+				throw new RemotingException ("Configuration file '" + filename + "' could not be loaded: " + ex.Message, ex);
 			}
 		}
 		
@@ -376,10 +367,16 @@ namespace System.Runtime.Remoting
 
 		internal static void SetCustomErrorsMode (string mode)
 		{
-			if (mode != "on" && mode != "off" && mode != "remoteOnly")
+			if (mode == null)
+				throw new RemotingException ("mode attribute is required");
+
+			// the mode is case insensitive
+			string m = mode.ToLower ();
+
+			if (m != "on" && m != "off" && m != "remoteonly")
 				throw new RemotingException ("Invalid custom error mode: " + mode);
 				
-			_errorMode = mode;
+			_errorMode = m;
 		}
 	}
 
@@ -439,7 +436,7 @@ namespace System.Runtime.Remoting
 			}
 			catch (Exception ex)
 			{
-				throw new RemotingException ("Error in element " + name + ": " + ex.Message);
+				throw new RemotingException ("Error in element " + name + ": " + ex.Message, ex);
 			}
 		}
 		
@@ -581,8 +578,8 @@ namespace System.Runtime.Remoting
 			if (currentProviderData != null)
 			{
 				currentProviderData.Pop ();
-				if (currentProviderData.Count > 0) return;
-				currentProviderData = null;
+				if (currentProviderData.Count == 0) 
+					currentProviderData = null;
 			}
 			
 			currentXmlPath = currentXmlPath.Substring (0, currentXmlPath.Length - name.Length - 1);
