@@ -49,6 +49,7 @@ using System.Security.Permissions;
 
 #if NET_2_0
 using System.Runtime.ConstrainedExecution;
+using System.Runtime.Serialization.Formatters;
 #endif
 
 namespace System.Runtime.Remoting
@@ -78,6 +79,10 @@ namespace System.Runtime.Remoting
 			StreamingContext context = new StreamingContext (StreamingContextStates.Remoting, null);
 			_serializationFormatter = new BinaryFormatter (surrogateSelector, context);
 			_deserializationFormatter = new BinaryFormatter (null, context);
+#if NET_2_0
+			_serializationFormatter.AssemblyFormat = FormatterAssemblyStyle.Full;
+			_deserializationFormatter.AssemblyFormat = FormatterAssemblyStyle.Full;
+#endif
 			
 			RegisterInternalChannels ();
 			app_id = Guid.NewGuid().ToString().Replace('-', '_') + "/";
@@ -321,7 +326,7 @@ namespace System.Runtime.Remoting
 		static string NewUri ()
 		{
 			int n = Interlocked.Increment (ref next_id);
-			return app_id + Environment.TickCount + "_" + n + ".rem";
+			return app_id + Environment.TickCount.ToString("x") + "_" + n + ".rem";
 		}
 
 #if NET_2_0
@@ -503,7 +508,10 @@ namespace System.Runtime.Remoting
 	
 		internal static object CreateClientProxy (Type objectType, string url, object[] activationAttributes)
 		{
-			string activationUrl = url + "/RemoteActivationService.rem";
+			string activationUrl = url;
+			if (!activationUrl.EndsWith ("/"))
+				activationUrl += "/";
+			activationUrl += "RemoteActivationService.rem";
 
 			string objectUri;
 			GetClientChannelSinkChain (activationUrl, null, out objectUri);
