@@ -262,6 +262,9 @@ namespace System.Runtime.Serialization.Formatters.Binary
 			else
 #endif
 				objectInstance = FormatterServices.GetUninitializedObject (metadata.Type);
+#if NET_2_0
+			_manager.RaiseOnDeserializingEvent (objectInstance);
+#endif
 				
 			info = metadata.NeedsSerializationInfo ? new SerializationInfo(metadata.Type, new FormatterConverter()) : null;
 
@@ -395,7 +398,17 @@ namespace System.Runtime.Serialization.Formatters.Binary
 
 				case TypeCode.DateTime: {
 					DateTime[] arr = new DateTime [length];
-					for (int n = 0; n < length; n++) arr [n] = new DateTime (reader.ReadInt64());
+					for (int n = 0; n < length; n++) {
+						ulong nr = reader.ReadUInt64 ();
+						const ulong mask = (1ul << 62) - 1;
+						long ticks = (long) (nr & mask);
+#if NET_2_0
+						DateTimeKind kind = (DateTimeKind) (nr >> 62);
+						arr [n] = new DateTime (ticks, kind);
+#else
+						arr [n] = new DateTime (ticks);
+#endif
+					}
 					val = arr;
 					break;
 				}
