@@ -221,6 +221,9 @@ namespace System.Runtime.Serialization.Formatters.Binary
 		FormatterTypeStyle _typeFormat;
 		byte[] arrayBuffer;
 		int ArrayBufferLength = 4096;
+#if NET_2_0
+		SerializationObjectManager _manager;
+#endif
 		
 		class MetadataReference
 		{
@@ -240,6 +243,9 @@ namespace System.Runtime.Serialization.Formatters.Binary
 			_context = context;
 			_assemblyFormat = assemblyFormat;
 			_typeFormat = typeFormat;
+#if NET_2_0
+			_manager = new SerializationObjectManager (context);
+#endif
 		}
 
 		public void WriteObjectGraph (BinaryWriter writer, object obj, Header[] headers)
@@ -249,6 +255,9 @@ namespace System.Runtime.Serialization.Formatters.Binary
 			QueueObject (obj);
 			WriteQueuedObjects (writer);
 			WriteSerializationEnd (writer);
+#if NET_2_0
+			_manager.RaiseOnSerializedEvent ();
+#endif
 		}
 
 		public void QueueObject (object obj)
@@ -376,6 +385,10 @@ namespace System.Runtime.Serialization.Formatters.Binary
 
 			BinaryCommon.CheckSerializable (instanceType, _surrogateSelector, _context);
 
+#if NET_2_0
+			_manager.RegisterObject (obj);
+#endif
+
 			ISerializable ser = obj as ISerializable;
 
 			if (ser != null) 
@@ -546,8 +559,13 @@ namespace System.Runtime.Serialization.Formatters.Binary
 					break;
 
 				case TypeCode.DateTime: 
-					foreach (DateTime item in (DateTime[]) array)
-						writer.Write (item.Ticks);
+					foreach (DateTime item in (DateTime[]) array) {
+						ulong val = (ulong) item.Ticks;
+#if NET_2_0
+						val |= ((ulong) item.Kind) << 62;
+#endif
+						writer.Write (val);
+					}
 					break;
 
 				case TypeCode.Decimal:
