@@ -32,7 +32,8 @@ namespace System.Security.AccessControl {
 
 	[MonoTODO ("required for NativeObjectSecurity - implementation is missing")]
 	public abstract class CommonObjectSecurity : ObjectSecurity {
-		protected CommonObjectSecurity ()
+
+		internal CommonObjectSecurity ()
 		{
 			/* Give it a 0-param constructor */
 		}
