@@ -7,7 +7,7 @@
 //
 // (C) 2002, 2003 Motus Technologies Inc. (http://www.motus.com)
 // Copyright (C) Tim Coleman, 2004
-// Copyright (C) 2004-2005 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2004-2006 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -206,6 +206,23 @@ public class CryptoConfig {
 	private const string urlKeyValueRSA = urlXmlDsig + " KeyValue/RSAKeyValue";	// space is required
 	private const string urlRetrievalMethod = urlXmlDsig + " RetrievalMethod";	// space is required
 
+#if NET_2_0
+	// new (2.0) X509 certificate extensions
+	private const string oidX509SubjectKeyIdentifier = "2.5.29.14";
+	private const string oidX509KeyUsage = "2.5.29.15";
+	private const string oidX509BasicConstraints = "2.5.29.19";
+	private const string oidX509EnhancedKeyUsage = "2.5.29.37";
+
+	private const string nameX509SubjectKeyIdentifier = defaultNamespace + "X509Certificates.X509SubjectKeyIdentifierExtension, " + Consts.AssemblySystem;
+	private const string nameX509KeyUsage = defaultNamespace + "X509Certificates.X509KeyUsageExtension, " + Consts.AssemblySystem;
+	private const string nameX509BasicConstraints = defaultNamespace + "X509Certificates.X509BasicConstraintsExtension, " + Consts.AssemblySystem;
+	private const string nameX509EnhancedKeyUsage = defaultNamespace + "X509Certificates.X509EnhancedKeyUsageExtension, " + Consts.AssemblySystem;
+
+	// new (2.0) X509 Chain
+	private const string nameX509Chain = "X509Chain";
+	private const string defaultX509Chain = defaultNamespace + "X509Certificates.X509Chain, " + Consts.AssemblySystem;
+#endif
+
 	static CryptoConfig () 
 	{
 		// lock(this) is bad
@@ -215,7 +232,7 @@ public class CryptoConfig {
 
 	private static void Initialize () 
 	{
-		Hashtable algorithms = new Hashtable ();
+		Hashtable algorithms = new Hashtable (new CaseInsensitiveHashCodeProvider (), new CaseInsensitiveComparer ());
 		// see list @ http://msdn.microsoft.com/library/en-us/cpref/html/
 		// frlrfSystemSecurityCryptographyCryptoConfigClassTopic.asp
 		algorithms.Add (nameSHA1a, defaultSHA1);
@@ -311,7 +328,17 @@ public class CryptoConfig {
 		algorithms.Add (urlKeyValueRSA, defaultKeyValueRSA);
 		algorithms.Add (urlRetrievalMethod, defaultRetrievalMethod);
 
-		Hashtable oid = new Hashtable ();
+#if NET_2_0
+		// note: X.509 extensions aren't part of OID but names
+		algorithms.Add (oidX509SubjectKeyIdentifier, nameX509SubjectKeyIdentifier);
+		algorithms.Add (oidX509KeyUsage, nameX509KeyUsage);
+		algorithms.Add (oidX509BasicConstraints, nameX509BasicConstraints);
+		algorithms.Add (oidX509EnhancedKeyUsage, nameX509EnhancedKeyUsage);
+		// note: the default X.509Chain can also be created this way
+		algorithms.Add (nameX509Chain, defaultX509Chain);
+#endif
+
+		Hashtable oid = new Hashtable (new CaseInsensitiveHashCodeProvider (), new CaseInsensitiveComparer ());
 		// comments here are to match with MS implementation (but not with doc)
 		// LAMESPEC: only HashAlgorithm seems to have their OID included
 		oid.Add (defaultSHA1, oidSHA1);
@@ -339,10 +366,7 @@ public class CryptoConfig {
 		oid.Add (name3DESKeyWrap, oid3DESKeyWrap);
 
 #if NET_2_0
-//		oid.Add (nameRSAa, oidRSA);
-		oid.Add (nameDSAa, oidDSA);
 		oid.Add (nameDESa, oidDES);
-		oid.Add (name3DESa, oid3DES);
 		oid.Add (name3DESb, oid3DES);
 		oid.Add (nameRC2a, oidRC2);
 #endif
@@ -362,62 +386,14 @@ public class CryptoConfig {
 		if (!File.Exists (filename))
 			return;
 
-		SecurityParser sp = new SecurityParser ();
-		StreamReader sr = new StreamReader (filename);
 		try {
-			sp.LoadXml (sr.ReadToEnd ());
-		}
-		finally {
-			sr.Close ();
-		}
-
-		try {
-			SecurityElement root = sp.ToXml ();
-			SecurityElement mscorlib = root.SearchForChildByTag ("mscorlib");
-			if (mscorlib == null)
-				return;
-			SecurityElement cryptographySettings = mscorlib.SearchForChildByTag ("cryptographySettings");
-			if (cryptographySettings == null)
-				return;
-
-			// algorithms
-			SecurityElement cryptoNameMapping = cryptographySettings.SearchForChildByTag ("cryptoNameMapping");
-			if (cryptoNameMapping != null) {
-				SecurityElement cryptoClasses = cryptoNameMapping.SearchForChildByTag ("cryptoClasses");
-				if (cryptoClasses != null) {
-					foreach (SecurityElement nameEntry in cryptoNameMapping.Children) {
-						if (nameEntry.Tag == "nameEntry") {
-							string name = (string) nameEntry.Attributes ["name"];
-							string clas = (string) nameEntry.Attributes ["class"];
-							foreach (SecurityElement cryptoClass in cryptoClasses.Children) {
-								string fullname = (string) cryptoClass.Attributes [clas];
-								if (fullname != null) {
-									if (algorithms.ContainsKey (name)) 
-										algorithms.Remove (name);
-									algorithms.Add (name, fullname);
-									break;
-								}
-							}
-						}
-					}
-				}
-			}
-
-			// oid
-			SecurityElement oidMap = cryptographySettings.SearchForChildByTag ("oidMap");
-			if (oidMap == null)
-				return;
-			foreach (SecurityElement oidEntry in oidMap.Children) {
-				if (oidEntry.Tag == "oidEntry") {
-					string oids = (string) oidEntry.Attributes ["OID"];
-					if (oid.ContainsKey (oids)) 
-						oid.Remove (oids);
-					oid.Add (oids, oidEntry.Attributes ["name"]);
-				}
+			using (TextReader reader = new StreamReader (filename)) {
+				CryptoHandler handler = new CryptoHandler (algorithms, oid);
+				SmallXmlParser parser = new SmallXmlParser ();
+				parser.Parse (reader, handler);
 			}
 		}
 		catch {
-			// there's no error/warning in case the machine.config contains bad entries
 		}
 	}
 
@@ -561,6 +537,130 @@ public class CryptoConfig {
 
 		return (string)oid [name];
 	}
-}
 
+	class CryptoHandler: SmallXmlParser.IContentHandler {
+
+		private Hashtable algorithms;
+		private Hashtable oid;
+		private Hashtable names;
+		private Hashtable classnames;
+		int level;
+
+		public CryptoHandler (Hashtable algorithms, Hashtable oid)
+		{
+			this.algorithms = algorithms;
+			this.oid = oid;
+			// temporary tables to reconstruct algorithms
+			names = new Hashtable ();
+			classnames = new Hashtable ();
+		}
+
+		public void OnStartParsing (SmallXmlParser parser)
+		{
+			// don't care
+		}
+
+		public void OnEndParsing (SmallXmlParser parser)
+		{
+			foreach (DictionaryEntry de in names) {
+				try {
+					algorithms.Add (de.Key, classnames[de.Value]);
+				}
+				catch {
+				}
+			}
+			// matching is done, data no more required
+			names.Clear ();
+			classnames.Clear ();
+		}
+
+		private string Get (SmallXmlParser.IAttrList attrs, string name)
+		{
+			for (int i = 0; i < attrs.Names.Length; i++) {
+				if (attrs.Names[i] == name)
+					return attrs.Values[i];
+			}
+			return String.Empty;
+		}
+
+		public void OnStartElement (string name, SmallXmlParser.IAttrList attrs)
+		{
+			switch (level) {
+			case 0:
+				if (name == "configuration")
+					level++;
+				break;
+			case 1:
+				if (name == "mscorlib")
+					level++;
+				break;
+			case 2:
+				if (name == "cryptographySettings")
+					level++;
+				break;
+			case 3:
+				if (name == "oidMap")
+					level++;
+				else if (name == "cryptoNameMapping")
+					level++;
+				break;
+			case 4:
+				if (name == "oidEntry") {
+					oid.Add (Get (attrs, "OID"), Get (attrs, "name"));
+				} else if (name == "nameEntry") {
+					names.Add (Get (attrs, "name"), Get (attrs, "class"));
+				} else if (name == "cryptoClasses") {
+					level++;
+				}
+				break;
+			case 5:
+				if (name == "cryptoClass")
+					classnames.Add (attrs.Names[0], attrs.Values[0]);
+				break;
+			}
+		}
+
+		public void OnEndElement (string name)
+		{
+			// parser will make sure the XML structure is respected
+			switch (level) {
+			case 1:
+				if (name == "configuration")
+					level--;
+				break;
+			case 2:
+				if (name == "mscorlib")
+					level--;
+				break;
+			case 3:
+				if (name == "cryptographySettings")
+					level--;
+				break;
+			case 4:
+				if ((name == "oidMap") || (name == "cryptoNameMapping"))
+					level--;
+				break;
+			case 5:
+				if (name == "cryptoClasses")
+					level--;
+				break;
+			}
+		}
+
+		public void OnProcessingInstruction (string name, string text)
+		{
+			// don't care
+		}
+
+		public void OnChars (string text)
+		{
+			// don't care
+		}
+
+		public void OnIgnorableWhitespace (string text)
+		{
+			// don't care
+		}
+	}
+}
 }
