@@ -61,6 +61,7 @@ namespace System.Security {
 		private PolicyLevel _policyLevel;
 		private bool _declsec;
 		private bool _readOnly;
+		private bool[] _ignored; // for asserts and non-CAS permissions
 
 		// constructors
 
@@ -72,9 +73,7 @@ namespace System.Security {
 
 		public PermissionSet (PermissionState state) : this ()
 		{
-			if (!Enum.IsDefined (typeof (PermissionState), state))
-				throw new System.ArgumentException ("state");
-			this.state = state;
+			this.state = CodeAccessPermission.CheckPermissionState (state, true);
 		}
 
 		public PermissionSet (PermissionSet permSet) : this ()
@@ -117,7 +116,11 @@ namespace System.Security {
 
 		// methods
 
+#if NET_2_0
+		public IPermission AddPermission (IPermission perm)
+#else
 		public virtual IPermission AddPermission (IPermission perm)
+#endif
 		{
 			if ((perm == null) || _readOnly)
 				return perm;
@@ -151,10 +154,13 @@ namespace System.Security {
 		}
 
 		[MonoTODO ("Imperative mode isn't supported")]
+		[SecurityPermission (SecurityAction.Demand, Assertion = true)]
+#if NET_2_0
+		public void Assert ()
+#else
 		public virtual void Assert ()
+#endif
 		{
-			new SecurityPermission (SecurityPermissionFlag.Assertion).Demand ();
-
 			int count = this.Count;
 
 			// we (current frame) must have the permission to assert it to others
@@ -202,33 +208,40 @@ namespace System.Security {
 			}
 		}
 
-		[MonoTODO ("Imperative Assert, Deny and PermitOnly aren't yet supported")]
+#if NET_2_0
+		public void Demand ()
+#else
 		public virtual void Demand ()
+#endif
 		{
 			// Note: SecurityEnabled only applies to CAS permissions
 			// so we're not checking for it (yet)
 			if (IsEmpty ())
 				return;
 
-			PermissionSet cas = this;
-			// avoid copy (if possible)
-			if (ContainsNonCodeAccessPermissions ()) {
-				// non CAS permissions (e.g. PrincipalPermission) do not requires a stack walk
-				cas = this.Copy ();
-				foreach (IPermission p in list) {
-					Type t = p.GetType ();
-					if (!t.IsSubclassOf (typeof (CodeAccessPermission))) {
-						p.Demand ();
-						// we wont have to process this one in the stack walk
-						cas.RemovePermission (t);
-					}
+			int n = list.Count;
+			if ((_ignored == null) || (_ignored.Length != n)) {
+				_ignored = new bool [n];
+			}
+
+			bool call_cas_only = this.IsUnrestricted ();
+			// non CAS permissions (e.g. PrincipalPermission) do not requires a stack walk
+			for (int i = 0; i < n; i++) {
+				CodeAccessPermission p = (CodeAccessPermission) list [i];
+				Type t = p.GetType ();
+				if (t.IsSubclassOf (typeof (CodeAccessPermission))) {
+					_ignored [i] = false;
+					call_cas_only = true;
+				} else {
+					_ignored [i] = true;
+					p.Demand ();
 				}
 			}
 
 			// don't start the stack walk if
 			// - the permission set only contains non CAS permissions; or
 			// - security isn't enabled (applis only to CAS!)
-			if (!cas.IsEmpty () && SecurityManager.SecurityEnabled)
+			if (call_cas_only && SecurityManager.SecurityEnabled)
 				CasOnlyDemand (_declsec ? 5 : 3);
 		}
 
@@ -241,6 +254,11 @@ namespace System.Security {
 			Assembly current = null;
 			AppDomain domain = null;
 
+			if (_ignored == null) {
+				// special case when directly called from CodeAccessPermission.Demand
+				_ignored = new bool [list.Count];
+			}
+
 			ArrayList frames = SecurityFrame.GetStack (skip);
 			if ((frames != null) && (frames.Count > 0)) {
 				SecurityFrame first = ((SecurityFrame) frames [0]);
@@ -248,8 +266,10 @@ namespace System.Security {
 				domain = first.Domain;
 				// skip ourself, Demand and other security runtime methods
 				foreach (SecurityFrame sf in frames) {
-					if (ProcessFrame (sf, ref current, ref domain))
-						return; // reached Assert
+					if (ProcessFrame (sf, ref current, ref domain)) {
+						if (AllIgnored ())
+							return; // reached Assert
+					}
 				}
 				SecurityFrame last = ((SecurityFrame) frames [frames.Count - 1]);
 				CheckAssembly (current, last);
@@ -260,14 +280,20 @@ namespace System.Security {
 			CompressedStack stack = Thread.CurrentThread.GetCompressedStack ();
 			if ((stack != null) && !stack.IsEmpty ()) {
 				foreach (SecurityFrame frame in stack.List) {
-					if (ProcessFrame (frame, ref current, ref domain))
-						return; // reached Assert
+					if (ProcessFrame (frame, ref current, ref domain)) {
+						if (AllIgnored ())
+							return; // reached Assert
+					}
 				}
 			}
 		}
 
 		[MonoTODO ("Imperative mode isn't supported")]
+#if NET_2_0
+		public void Deny ()
+#else
 		public virtual void Deny ()
+#endif
 		{
 			if (!SecurityManager.SecurityEnabled)
 				return;
@@ -319,41 +345,65 @@ namespace System.Security {
 			}
 		}
 
+#if NET_2_0
+		public IEnumerator GetEnumerator ()
+#else
 		public virtual IEnumerator GetEnumerator ()
+#endif
 		{
 			return list.GetEnumerator ();
 		}
 
-		public virtual bool IsSubsetOf (PermissionSet target)
+#if NET_2_0
+		public bool IsSubsetOf (PermissionSet target)
 		{
 			// if target is empty we must be empty too
 			if ((target == null) || (target.IsEmpty ()))
 				return this.IsEmpty ();
 
-			// TODO - non CAS permissions must be evaluated for unrestricted
-
-			// if target is unrestricted then we are a subset
-			if (!this.IsUnrestricted () && target.IsUnrestricted ())
+			// all permissions support unrestricted in 2.0
+			if (target.IsUnrestricted ())
 				return true;
-			// else target isn't unrestricted.
-			// so if we are unrestricted, the we can't be a subset
-			if (this.IsUnrestricted () && !target.IsUnrestricted ())
+			if (this.IsUnrestricted ())
+				return false;
+#else
+		public virtual bool IsSubsetOf (PermissionSet target)
+		{
+#endif
+			if (this.IsUnrestricted () && ((target == null) || !target.IsUnrestricted ()))
 				return false;
 
 			// if each of our permission is (a) present and (b) a subset of target
 			foreach (IPermission p in list) {
-				// for every type in both list
-				IPermission i = target.GetPermission (p.GetType ());
-				if (i == null)
-					return false; // not present (condition a)
-				if (!p.IsSubsetOf (i))
-					return false; // not a subset (condition b)
+#if !NET_2_0
+				if (target == null) {
+					if (!p.IsSubsetOf (null))
+						return false;
+				} else
+#endif
+				{
+					// non CAS permissions must be evaluated for unrestricted
+					Type t = p.GetType ();
+					IPermission i = null;
+					if (target.IsUnrestricted () && (p is CodeAccessPermission) && (p is IUnrestrictedPermission)) {
+						i = (IPermission) Activator.CreateInstance (t, psUnrestricted);
+					} else {
+						i = target.GetPermission (t);
+					}
+
+					if (!p.IsSubsetOf (i))
+						return false; // not a subset (condition b)
+				}
 			}
 			return true;
 		}
 
 		[MonoTODO ("Imperative mode isn't supported")]
+#if NET_2_0
+		public void PermitOnly ()
+#else
 		public virtual void PermitOnly ()
+#endif
 		{
 			if (!SecurityManager.SecurityEnabled)
 				return;
@@ -368,9 +418,11 @@ namespace System.Security {
 
 		public bool ContainsNonCodeAccessPermissions () 
 		{
-			foreach (IPermission p in list) {
-				if (! p.GetType ().IsSubclassOf (typeof (CodeAccessPermission)))
-					return true;
+			if (list.Count > 0) {
+				foreach (IPermission p in list) {
+					if (! p.GetType ().IsSubclassOf (typeof (CodeAccessPermission)))
+						return true;
+				}
 			}
 			return false;
 		}
@@ -447,17 +499,28 @@ namespace System.Security {
 			throw new SerializationException (String.Format (Locale.GetText ("Unknown output format {0}."), outFormat));
 		}
 
-		public virtual IPermission GetPermission (Type permClass) 
+#if NET_2_0
+		public IPermission GetPermission (Type permClass)
+#else
+		public virtual IPermission GetPermission (Type permClass)
+#endif
 		{
+			if ((permClass == null) || (list.Count == 0))
+				return null;
+
 			foreach (object o in list) {
-				if (o.GetType ().Equals (permClass))
+				if ((o != null) && o.GetType ().Equals (permClass))
 					return (IPermission) o;
 			}
 			// it's normal to return null for unrestricted sets
 			return null;
 		}
 
-		public virtual PermissionSet Intersect (PermissionSet other) 
+#if NET_2_0
+		public PermissionSet Intersect (PermissionSet other)
+#else
+		public virtual PermissionSet Intersect (PermissionSet other)
+#endif
 		{
 			// no intersection possible
 			if ((other == null) || (other.IsEmpty ()) || (this.IsEmpty ()))
@@ -467,20 +530,32 @@ namespace System.Security {
 			if (this.IsUnrestricted () && other.IsUnrestricted ())
 				state = PermissionState.Unrestricted;
 
-			PermissionSet interSet = new PermissionSet (state);
+			PermissionSet interSet = null;
+#if NET_2_0
+			// much simpler with 2.0
+			if (state == PermissionState.Unrestricted) {
+				interSet = new PermissionSet (state);
+			} else if (this.IsUnrestricted ()) {
+				interSet = other.Copy ();
+			} else if (other.IsUnrestricted ()) {
+				interSet = this.Copy ();
+			} else {
+				interSet = new PermissionSet (state);
+				InternalIntersect (interSet, this, other, false);
+			}
+#else
+			interSet = new PermissionSet (state);
 			if (state == PermissionState.Unrestricted) {
 				InternalIntersect (interSet, this, other, true);
 				InternalIntersect (interSet, other, this, true);
-			}
-			else if (this.IsUnrestricted ()) {
+			} else if (this.IsUnrestricted ()) {
 				InternalIntersect (interSet, this, other, true);
-			}
-			else if (other.IsUnrestricted ()) {
+			} else if (other.IsUnrestricted ()) {
 				InternalIntersect (interSet, other, this, true);
-			}
-			else {
+			} else {
 				InternalIntersect (interSet, this, other, false);
 			}
+#endif
 			return interSet;
 		}
 
@@ -505,7 +580,11 @@ namespace System.Security {
 			}
 		}
 
-		public virtual bool IsEmpty () 
+#if NET_2_0
+		public bool IsEmpty ()
+#else
+		public virtual bool IsEmpty ()
+#endif
 		{
 			// note: Unrestricted isn't empty
 			if (state == PermissionState.Unrestricted)
@@ -521,12 +600,20 @@ namespace System.Security {
 			return true;
 		}
 
-		public virtual bool IsUnrestricted () 
+#if NET_2_0
+		public bool IsUnrestricted ()
+#else
+		public virtual bool IsUnrestricted ()
+#endif
 		{
 			return (state == PermissionState.Unrestricted);
 		}
 
-		public virtual IPermission RemovePermission (Type permClass) 
+#if NET_2_0
+		public IPermission RemovePermission (Type permClass)
+#else
+		public virtual IPermission RemovePermission (Type permClass)
+#endif
 		{
 			if ((permClass == null) || _readOnly)
 				return null;
@@ -540,7 +627,11 @@ namespace System.Security {
 			return null;
 		}
 
-		public virtual IPermission SetPermission (IPermission perm) 
+#if NET_2_0
+		public IPermission SetPermission (IPermission perm)
+#else
+		public virtual IPermission SetPermission (IPermission perm)
+#endif
 		{
 			if ((perm == null) || _readOnly)
 				return perm;
@@ -580,14 +671,23 @@ namespace System.Security {
 			return se;
 		}
 
+#if NET_2_0
+		public PermissionSet Union (PermissionSet other)
+#else
 		public virtual PermissionSet Union (PermissionSet other)
+#endif
 		{
 			if (other == null)
 				return this.Copy ();
 
-			PermissionSet copy = this.Copy ();
+			PermissionSet copy = null;
 			if (this.IsUnrestricted () || other.IsUnrestricted ()) {
-				// so we keep the "right" type
+#if NET_2_0
+				// there are no child elements in unrestricted permission sets
+				return new PermissionSet (PermissionState.Unrestricted);
+#else
+				copy = this.Copy ();
+				// so we keep the "right" type (e.g. NamedPermissionSet)
 				copy.Clear ();
 				copy.state = PermissionState.Unrestricted;
 				// copy all permissions that do not implement IUnrestrictedPermission
@@ -599,8 +699,9 @@ namespace System.Security {
 					if (!(p is IUnrestrictedPermission))
 						copy.AddPermission (p);
 				}
-			}
-			else {
+#endif
+			} else {
+				copy = this.Copy ();
 				// PermissionState.None -> copy all permissions
 				foreach (IPermission p in other.list) {
 					copy.AddPermission (p);
@@ -632,7 +733,7 @@ namespace System.Security {
 			set { _declsec = value; }
 		}
 
-		[MonoTODO()]
+		[MonoTODO ("may not be required")]
 		void IDeserializationCallback.OnDeserialization (object sender) 
 		{
 		}
@@ -671,10 +772,9 @@ namespace System.Security {
 			return (list.Count == 0) ? (int) state : base.GetHashCode ();
 		}
 
-		[MonoTODO ("what's it doing here?")]
+		[MonoTODO ("(2.0) what's it doing here? There's probably a reason this was added here.")]
 		static public void RevertAssert ()
 		{
-			// FIXME: There's probably a reason this was added here ?
 			CodeAccessPermission.RevertAssert ();
 		}
 #endif
@@ -691,6 +791,19 @@ namespace System.Security {
 			_readOnly = value;
 		}
 
+		private bool AllIgnored ()
+		{
+			if (_ignored == null)
+				throw new NotSupportedException ("bad bad bad");
+
+			for (int i=0; i < _ignored.Length; i++) {
+				if (!_ignored [i])
+					return false;
+			}
+			// everything is ignored (i.e. non-CAS permission or asserted permission).
+			return true;
+		}
+
 		internal bool ProcessFrame (SecurityFrame frame, ref Assembly current, ref AppDomain domain)
 		{
 			if (IsUnrestricted ()) {
@@ -698,7 +811,7 @@ namespace System.Security {
 				if (frame.Deny != null) {
 					// but have restrictions (some denied permissions)
 					CodeAccessPermission.ThrowSecurityException (this, "Deny", frame, SecurityAction.Demand, null);
-				} else if (frame.PermitOnly != null) {
+				} else if ((frame.PermitOnly != null) && !frame.PermitOnly.IsUnrestricted ()) {
 					// but have restrictions (only some permitted permissions)
 					CodeAccessPermission.ThrowSecurityException (this, "PermitOnly", frame, SecurityAction.Demand, null);
 				}
@@ -706,9 +819,13 @@ namespace System.Security {
 
 			// skip next steps if no Assert, Deny or PermitOnly are present
 			if (frame.HasStackModifiers) {
-				foreach (CodeAccessPermission cap in list) {
-					if (cap.ProcessFrame (frame))
-						return true; // Assert reached - abort stack walk!
+				for (int i = 0; i < list.Count; i++) {
+					CodeAccessPermission cap = (CodeAccessPermission) list [i];
+					if (cap.ProcessFrame (frame)) {
+						_ignored [i] = true; // asserted
+						if (AllIgnored ())
+							return true; // no more, abort stack walk!
+					}
 				}
 			}
 
@@ -788,14 +905,13 @@ namespace System.Security {
 
 		static object[] action = new object [1] { (SecurityAction) 0 };
 
-		// TODO: add support for arrays and enums
+		// TODO: add support for arrays and enums (2.0)
 		internal static IPermission ProcessAttribute (byte[] data, ref int position)
 		{
 			int clen = ReadEncodedInt (data, ref position);
 			string cnam = Encoding.UTF8.GetString (data, position, clen);
 			position += clen;
 
-			// TODO: Unification
 			Type secattr = Type.GetType (cnam);
 			SecurityAttribute sa = (Activator.CreateInstance (secattr, action) as SecurityAttribute);
 			if (sa == null)
@@ -837,7 +953,7 @@ namespace System.Security {
 				object[] arrayIndex = null;
 				for (int i = 0; i < arrayLength; i++) {
 					if (array) {
-						// TODO - setup index
+						// TODO - setup index (2.0)
 					}
 
 					// sadly type values doesn't match ther TypeCode enum :(
