@@ -46,7 +46,7 @@ namespace System.Text {
 #if NET_2_0
 	[ComVisible (true)]
 #endif
-	[MonoTODO ("Fix serialization compatibility with MS.NET")]
+        [MonoTODO ("Serialization format not compatible with .NET")]
 	public sealed class StringBuilder
 #if NET_2_0
 		: ISerializable
@@ -106,6 +106,11 @@ namespace System.Text {
 
 		public StringBuilder (string value)
 		{
+			/*
+			 * This is an optimization to avoid allocating the internal string
+			 * until the first Append () call.
+			 * The runtime pinvoke marshalling code needs to be aware of this.
+			 */
 			if (null == value)
 				value = "";
 			
