@@ -4,7 +4,7 @@
  *
  * Copyright (c) 2001, 2002  Southern Storm Software, Pty Ltd
  * Copyright (C) 2003, 2004 Novell, Inc.
- * Copyright (C) 2006 Kornl Pl <http://www.kornelpal.hu/>
+ * Copyright (C) 2006 Kornél Pál <http://www.kornelpal.hu/>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the "Software"),
@@ -35,7 +35,7 @@ using System.Runtime.InteropServices;
 #if NET_2_0
 [ComVisible (true)]
 #endif
-[MonoTODO ("Fix serialization compatibility with MS.NET")]
+[MonoTODO ("Serialization format not compatible with .NET")]
 public class UnicodeEncoding : Encoding
 {
 	// Magic numbers used by Windows for Unicode.
