@@ -419,7 +419,6 @@ namespace System {
 				refusedPermissions, false);
 		}
 
-		[MonoTODO ("FIXME: examine all other parameters")]
 		public AssemblyBuilder DefineDynamicAssembly (AssemblyName name, AssemblyBuilderAccess access, string dir,
 		                                              Evidence evidence,
 		                                              PermissionSet requiredPermissions,
@@ -457,7 +456,7 @@ namespace System {
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		public extern int ExecuteAssembly (string assemblyFile, Evidence assemblySecurity, string[] args);
 
-		[MonoTODO]
+		[MonoTODO ("No support for ExecuteAssembly")]
 		public int ExecuteAssembly (string assemblyFile, Evidence assemblySecurity, string[] args, byte[] hashValue, AssemblyHashAlgorithm hashAlgorithm)
 		{
 			throw new NotImplementedException ();
@@ -706,7 +705,7 @@ namespace System {
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		private static extern AppDomain createDomain (string friendlyName, AppDomainSetup info);
 
-		[MonoTODO ("allow setup in the other domain")]
+		[MonoTODO ("Currently it does not allow the setup in the other domain")]
 		[SecurityPermission (SecurityAction.Demand, ControlAppDomain = true)]
 		public static AppDomain CreateDomain (string friendlyName, Evidence securityInfo, AppDomainSetup info)
 		{
@@ -1015,7 +1014,7 @@ namespace System {
 
 		// methods
 
-		[MonoTODO ("what's the policy affecting names ?")]
+		[MonoTODO ("This routine only returns the parameter currently")]
 		[ComVisible (false)]
 		public string ApplyPolicy (string assemblyName)
 		{
@@ -1071,7 +1070,6 @@ namespace System {
 			return Object.ReferenceEquals (this, DefaultDomain);
 		}
 
-		[MonoTODO ("see Assembly.ReflectionOnlyLoad")]
 		public Assembly[] ReflectionOnlyGetAssemblies ()
 		{
 			return GetAssemblies (true);
