@@ -65,7 +65,7 @@ namespace System
 			}
 		}
 
-		private static TextWriter stdout;
+		internal static TextWriter stdout;
 		private static TextWriter stderr;
 		private static TextReader stdin;
 
@@ -108,12 +108,23 @@ namespace System
 			((StreamWriter)stderr).AutoFlush = true;
 			stderr = TextWriter.Synchronized (stderr, true);
 
-			stdout = new UnexceptionalStreamWriter (OpenStandardOutput (0), outputEncoding);
-			((StreamWriter)stdout).AutoFlush = true;
-			stdout = TextWriter.Synchronized (stdout, true);
+#if NET_2_0
+			if (!Environment.IsRunningOnWindows && ConsoleDriver.IsConsole) {
+				StreamWriter w = new CStreamWriter (OpenStandardOutput (0), outputEncoding);
+				w.AutoFlush = true;
+				stdout = w;
+				stdin = new CStreamReader (OpenStandardInput (0), inputEncoding);
+			} else {
+#endif
+				stdout = new UnexceptionalStreamWriter (OpenStandardOutput (0), outputEncoding);
+				((StreamWriter)stdout).AutoFlush = true;
+				stdout = TextWriter.Synchronized (stdout, true);
+				stdin = new UnexceptionalStreamReader (OpenStandardInput (0), inputEncoding);
+				stdin = TextReader.Synchronized (stdin);
+#if NET_2_0
+			}
+#endif
 
-			stdin = new UnexceptionalStreamReader (OpenStandardInput (0), inputEncoding);
-			stdin = TextReader.Synchronized (stdin);
 			GC.SuppressFinalize (stdout);
 			GC.SuppressFinalize (stderr);
 			GC.SuppressFinalize (stdin);
@@ -148,7 +159,7 @@ namespace System
 			return OpenStandardError (0);
 		}
 
-		// calling any FileStream constructor with an handle normally
+		// calling any FileStream constructor with a handle normally
 		// requires permissions UnmanagedCode permissions. In this 
 		// case we assert this permission so the console can be used
 		// in partial trust (i.e. without having UnmanagedCode).
@@ -167,7 +178,7 @@ namespace System
 			return OpenStandardInput (0);
 		}
 
-		// calling any FileStream constructor with an handle normally
+		// calling any FileStream constructor with a handle normally
 		// requires permissions UnmanagedCode permissions. In this 
 		// case we assert this permission so the console can be used
 		// in partial trust (i.e. without having UnmanagedCode).
@@ -186,7 +197,7 @@ namespace System
 			return OpenStandardOutput (0);
 		}
 
-		// calling any FileStream constructor with an handle normally
+		// calling any FileStream constructor with a handle normally
 		// requires permissions UnmanagedCode permissions. In this 
 		// case we assert this permission so the console can be used
 		// in partial trust (i.e. without having UnmanagedCode).
@@ -448,27 +459,30 @@ namespace System
 		}
 #endif
 
+#if NET_2_0
+		public static int Read ()
+		{
+			if ((stdin is CStreamReader) && ConsoleDriver.IsConsole) {
+				return ConsoleDriver.Read ();
+			} else {
+				return stdin.Read ();
+			}
+		}
+
+		public static string ReadLine ()
+		{
+			if ((stdin is CStreamReader) && ConsoleDriver.IsConsole) {
+				return ConsoleDriver.ReadLine ();
+			} else {
+				return stdin.ReadLine ();
+			}
+		}
+#else
 		public static int Read ()
 		{
 			return stdin.Read ();
 		}
 
-#if NET_2_0
-		public static string ReadLine ()
-		{
-			bool prevEcho = false;
-			if (ConsoleDriver.Initialized) {
-				prevEcho = ConsoleDriver.Echo;
-				ConsoleDriver.Echo = true;
-			}
-
-			string ret = stdin.ReadLine ();
-			if (ConsoleDriver.Initialized)
-				ConsoleDriver.Echo = prevEcho;
-
-			return ret;
-		}
-#else
 		public static string ReadLine ()
 		{
 			return stdin.ReadLine ();
@@ -581,14 +595,14 @@ namespace System
 			ConsoleDriver.Clear ();
 		}
 
-		[MonoTODO]
+		[MonoTODO ("Not implemented")]
 		public static void MoveBufferArea (int sourceLeft, int sourceTop, int sourceWidth, int sourceHeight,
 						int targetLeft, int targetTop)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
+		[MonoTODO("Not implemented")]
 		public static void MoveBufferArea (int sourceLeft, int sourceTop, int sourceWidth, int sourceHeight,
 						int targetLeft, int targetTop, Char sourceChar,
 						ConsoleColor sourceForeColor, ConsoleColor sourceBackColor)
@@ -631,8 +645,44 @@ namespace System
 			ConsoleDriver.SetWindowSize (width, height);
 		}
 
-		[MonoTODO ("Implement add/remove hooks")]
-		public static event ConsoleCancelEventHandler CancelKeyPress;
+		static ConsoleCancelEventHandler cancel_event;
+		public static event ConsoleCancelEventHandler CancelKeyPress {
+			add {
+				if (ConsoleDriver.Initialized == false)
+					ConsoleDriver.Init ();
+
+				cancel_event += value;
+			}
+			remove {
+				if (ConsoleDriver.Initialized == false)
+					ConsoleDriver.Init ();
+
+				cancel_event -= value;
+			}
+		}
+
+		delegate void InternalCancelHandler ();
+		static InternalCancelHandler cancel_handler = new InternalCancelHandler (DoConsoleCancelEvent);
+
+		internal static void DoConsoleCancelEvent ()
+		{
+			bool exit = true;
+			if (cancel_event != null) {
+				ConsoleCancelEventArgs args = new ConsoleCancelEventArgs (ConsoleSpecialKey.ControlC);
+				Delegate [] delegates = cancel_event.GetInvocationList ();
+				foreach (ConsoleCancelEventHandler d in delegates){
+					try {
+						// Sender is always null here.
+						d (null, args);
+					} catch {} // Ignore any exception.
+				}
+				exit = !args.Cancel;
+			}
+
+			if (exit)
+				Environment.Exit (58);
+		}
 #endif
 	}
 }
+
