@@ -269,25 +269,24 @@ namespace System
 #endif
 		}
 
-		[MonoTODO ("Calendar is unused")]
+		[MonoTODO ("The Calendar is not taken into consideration")]
 		public DateTime (int year, int month, int day, Calendar calendar)
 			: this (year, month, day, 0, 0, 0, 0, calendar)
 		{
 		}
 		
-		[MonoTODO ("Calendar is unused")]
+		[MonoTODO ("The Calendar is not taken into consideration")]
 		public DateTime (int year, int month, int day, int hour, int minute, int second, Calendar calendar)
 			: this (year, month, day, hour, minute, second, 0, calendar)
 		{
 		}
 
-		[MonoTODO ("Calendar is unused")]
+		[MonoTODO ("The Calendar is not taken into consideration")]
 		public DateTime (int year, int month, int day, int hour, int minute, int second, int millisecond, Calendar calendar)
 			: this (year, month, day, hour, minute, second, millisecond) 
 		{
 			if (calendar == null)
 				throw new ArgumentNullException ("calendar");
-			// FIXME: we're not using the calendar anywhere???
 		}
 
 		internal DateTime (bool check, TimeSpan value)
@@ -418,11 +417,34 @@ namespace System
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		internal static extern long GetNow ();
 
+		//
+		// To reduce the time consumed by DateTime.Now, we keep
+		// the difference to map the system time into a local
+		// time into `to_local_time_span', we record the timestamp
+		// for this in `last_now'
+		//
+		static object to_local_time_span_object;
+		static long last_now;
+		
 		public static DateTime Now 
 		{
 			get	
 			{
-				return new DateTime (GetNow ()).ToLocalTime ();
+				long now = GetNow ();
+				DateTime dt = new DateTime (now);
+
+				if ((now - last_now) > TimeSpan.TicksPerMinute){
+					to_local_time_span_object = TimeZone.CurrentTimeZone.GetLocalTimeDiff (dt);
+					last_now = now;
+
+				}
+
+				// This is boxed, so we avoid locking.
+				DateTime ret = dt + (TimeSpan) to_local_time_span_object;
+#if NET_2_0
+				ret.kind = DateTimeKind.Local;
+#endif
+				return ret;
 			}
 		}
 
@@ -438,7 +460,11 @@ namespace System
 		{
 			get {
 				DateTime now = Now;
-				return new DateTime (now.Year, now.Month, now.Day);
+				DateTime today = new DateTime (now.Year, now.Month, now.Day);
+#if NET_2_0
+				today.kind = now.kind;
+#endif
+				return today;
 			}
 		}
 
@@ -760,7 +786,6 @@ namespace System
 			return Parse (s, fp, DateTimeStyles.AllowWhiteSpaces);
 		}
 
-		[MonoTODO ("see the comments inline")]
 		public static DateTime Parse (string s, IFormatProvider fp, DateTimeStyles styles)
 		{
 			// This method should try only expected patterns. 
