@@ -61,7 +61,6 @@ namespace System {
 		 */
 		private const int mono_corlib_version = 54;
 		
-		[MonoTODO]
 		public enum SpecialFolder
 		{	// TODO: Determine if these windoze style folder identifiers 
 			//       have unix/linux counterparts
@@ -233,7 +232,7 @@ namespace System {
 		/// <summary>
 		/// Gets a flag indicating whether the process is in interactive mode
 		/// </summary>
-		[MonoTODO]
+		[MonoTODO ("Currently always returns false, regardless of interactive state")]
 		public static bool UserInteractive {
 			get {
 				return false;
@@ -261,7 +260,7 @@ namespace System {
 		/// <summary>
 		/// Get the amount of physical memory mapped to process
 		/// </summary>
-		[MonoTODO]
+		[MonoTODO ("Currently always returns zero")]
 		public static long WorkingSet {
 			[EnvironmentPermission (SecurityAction.Demand, Unrestricted=true)]
 			get { return 0; }
@@ -475,7 +474,10 @@ namespace System {
 #endif
 			case SpecialFolder.DesktopDirectory:
 				return Path.Combine (home, "Desktop");
-			
+
+			case SpecialFolder.MyMusic:
+				return Path.Combine (home, "Music");
+				
 			// these simply dont exist on Linux
 			// The spec says if a folder doesnt exist, we
 			// should return ""
@@ -484,7 +486,6 @@ namespace System {
 			case SpecialFolder.SendTo:
 			case SpecialFolder.StartMenu:
 			case SpecialFolder.Startup:
-			case SpecialFolder.MyMusic:
 			case SpecialFolder.MyPictures:
 			case SpecialFolder.Templates:
 			case SpecialFolder.Cookies:
@@ -621,17 +622,13 @@ namespace System {
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		internal static extern void InternalSetEnvironmentVariable (string variable, string value);
 
-		[MonoTODO]
-		public static int ProcessorCount {
+		public static extern int ProcessorCount {
 			[EnvironmentPermission (SecurityAction.Demand, Read="NUMBER_OF_PROCESSORS")]
-			get {
-				// note: Changes to the NUMBER_OF_PROCESSORS environment variable
-				// under Windows doesn't affect the (good) value returned.
-				throw new NotImplementedException ();
-			}
+			[MethodImplAttribute (MethodImplOptions.InternalCall)]
+			get;			
 		}
 
-		[MonoTODO ("not much documented")]
+		[MonoTODO ("Not implemented")]
 		[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode=true)]
 		public static void FailFast (string message)
 		{
