@@ -35,7 +35,7 @@ namespace System {
 	public delegate void EventHandler (object sender, EventArgs e);
 		
 #if NET_2_0
-	public delegate void EventHandler <T> (object sender, T e) where T : EventArgs;
+	public delegate void EventHandler <TEventArgs> (object sender, TEventArgs e) where TEventArgs : EventArgs;
 #endif
 
 }
