@@ -41,6 +41,9 @@ namespace System
 {
 	[Serializable]
 	[ClassInterface (ClassInterfaceType.AutoDual)]
+#if NET_2_0
+	[ComVisible(true)]
+#endif
 	public class Exception : ISerializable 
 #if NET_2_0
 	, _Exception
@@ -48,7 +51,7 @@ namespace System
 	{
 		IntPtr [] trace_ips;
 		Exception inner_exception;
-		string message;
+		internal string message;
 		string help_link;
 		string class_name;
 		string stack_trace;
@@ -328,6 +331,19 @@ namespace System
 			}
 #endif
 			return mi.DeclaringType.ToString () + "." + mi.Name + generic + " (" + parms + ")";
-		}				
+		}
+
+#if NET_2_0
+		//
+		// The documentation states that this is available in 1.x,
+		// but it was not available (MemberRefing this would fail)
+		// and it states the signature is `override sealed', but the
+		// correct value is `newslot' 
+		//
+		public new Type GetType ()
+		{
+			return base.GetType ();
+		}
+#endif
 	}
 }
