@@ -45,7 +45,6 @@ namespace System
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private static extern void type_from_obj (MonoType type, Object obj);
 		
-		[MonoTODO]
 		internal MonoType (Object obj)
 		{
 			// this should not be used - lupus
@@ -299,13 +298,14 @@ namespace System
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		protected extern override bool IsPrimitiveImpl ();
 
-		protected override bool IsValueTypeImpl ()
+		public override bool IsSubclassOf (Type type)
 		{
-			return type_is_subtype_of (this, typeof (System.ValueType), false) &&
-				this != typeof (System.ValueType) &&
-				this != typeof (System.Enum);
+			if (type == null)
+				throw new ArgumentNullException ("type");
+
+			return base.IsSubclassOf (type);
 		}
-		
+
 		public override object InvokeMember (string name, BindingFlags invokeAttr,
 						     Binder binder, object target, object[] args,
 						     ParameterModifier[] modifiers,
