@@ -6,6 +6,7 @@
 //   Jeffrey Stedfast (fejj@ximian.com)
 //   Dan Lewis (dihlewis@yahoo.co.uk)
 //   Sebastien Pouliot  <sebastien@ximian.com>
+//   Marek Safar (marek.safar@seznam.cz)
 //
 // (C) 2001 Ximian, Inc.  http://www.ximian.com
 // Copyright (C) 2004-2005 Novell (http://www.novell.com)
@@ -694,12 +695,12 @@ namespace System
 #if NET_2_0
 		public int IndexOf (string value, StringComparison comparison)
 		{
-			return IndexOf (value, 0, value.Length, comparison);
+			return IndexOf (value, 0, this.Length, comparison);
 		}
 
 		public int IndexOf (string value, int startIndex, StringComparison comparison)
 		{
-			return IndexOf (value, startIndex, value.Length - startIndex, comparison);
+			return IndexOf (value, startIndex, this.Length - startIndex, comparison);
 		}
 
 		public int IndexOf (string value, int startIndex, int count, StringComparison comparison)
@@ -924,7 +925,7 @@ namespace System
 				throw new ArgumentOutOfRangeException ("startIndex - count + 1 < 0");
 
 			if (value.Length == 0)
-				return 0;
+				return startIndex;
 
 			if (startIndex == 0 && this.length == 0)
 				return -1;
@@ -1451,13 +1452,13 @@ namespace System
 
 		public unsafe static String Concat (String s1, String s2)
 		{
-			if (s1 == null) {
-				if (s2 == null)
+			if (s1 == null || s1.Length == 0) {
+				if (s2 == null || s2.Length == 0)
 					return String.Empty;
 				return s2;
 			}
 
-			if (s2 == null)
+			if (s2 == null || s2.Length == 0)
 				return s1; 
 
 			String tmp = InternalAllocateStr (s1.length + s2.length);
@@ -1478,24 +1479,24 @@ namespace System
 
 		public unsafe static String Concat (String s1, String s2, String s3)
 		{
-			if (s1 == null){
-				if (s2 == null){
-					if (s3 == null)
+			if (s1 == null || s1.Length == 0){
+				if (s2 == null || s2.Length == 0){
+					if (s3 == null || s3.Length == 0)
 						return String.Empty;
 					return s3;
 				} else {
-					if (s3 == null)
+					if (s3 == null || s3.Length == 0)
 						return s2;
 				}
 				s1 = String.Empty;
 			} else {
-				if (s2 == null){
-					if (s3 == null)
+				if (s2 == null || s2.Length == 0){
+					if (s3 == null || s3.Length == 0)
 						return s1;
 					else
 						s2 = String.Empty;
 				} else {
-					if (s3 == null)
+					if (s3 == null || s3.Length == 0)
 						s3 = String.Empty;
 				}
 			}
