@@ -4,7 +4,7 @@
 // Authors:
 //	Gonzalo Paniagua Javier (gonzalo@ximian.com)
 //
-// (C) 2005 Novell, Inc (http://www.novell.com)
+// (C) 2005,2006 Novell, Inc (http://www.novell.com)
 //
 
 //
@@ -28,8 +28,11 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 #if NET_2_0
+//#define DEBUG
 using System.Collections;
 using System.IO;
+using System.Text;
+using System.Runtime.InteropServices;
 namespace System {
 	class TermInfoDriver : IConsoleDriver {
 		static string [] locations = { "/etc/terminfo", "/usr/share/terminfo", "/usr/lib/terminfo" };
@@ -45,8 +48,10 @@ namespace System {
 		string clear;
 		string bell;
 		string term;
-		Stream stdout;
 		Stream stdin;
+		internal byte verase;
+		byte vsusp;
+		byte intr;
 
 		int windowWidth;
 		int windowHeight;
@@ -58,7 +63,6 @@ namespace System {
 		byte [] buffer;
 		int readpos;
 		int writepos;
-		string enterCA, exitCA;
 		string keypadXmit, keypadLocal;
 		bool controlCAsInput;
 		bool inited;
@@ -75,6 +79,10 @@ namespace System {
 		Hashtable keymap;
 		ByteMatcher rootmap;
 		bool home_1_1; // if true, we have to add 1 to x and y when using cursorAddress
+		int rl_startx = -1, rl_starty = -1;
+#if DEBUG
+		StreamWriter logger;
+#endif
 
 		static string SearchTerminfo (string term)
 		{
@@ -108,7 +116,7 @@ namespace System {
 			if (str == null)
 				return;
 
-			Console.Write (str);
+			((CStreamWriter) Console.stdout).InternalWriteString (str);
 		}
 
 		public TermInfoDriver ()
@@ -118,6 +126,10 @@ namespace System {
 
 		public TermInfoDriver (string term)
 		{
+#if DEBUG
+			File.Delete ("console.log");
+			logger = new StreamWriter (File.OpenWrite ("console.log"));
+#endif
 			this.term = term;
 
 			if (term == "xterm") {
@@ -137,30 +149,27 @@ namespace System {
 		public bool Initialized {
 			get { return inited; }
 		}
-		
-		void Init ()
+
+		public void Init ()
 		{
 			if (inited)
 				return;
+			
+			/* This should not happen any more, since it is checked for in Console */
+			if (!ConsoleDriver.IsConsole)
+				throw new IOException ("Not a tty.");
 
 			inited = true;
-			enterCA = reader.Get (TermInfoStrings.EnterCaMode);
-			exitCA = reader.Get (TermInfoStrings.ExitCaMode);
+			ConsoleDriver.SetEcho (false);
 			string endString = null;
-			if (enterCA != null && exitCA != null)
-				endString = exitCA;
-
 			keypadXmit = reader.Get (TermInfoStrings.KeypadXmit);
 			keypadLocal = reader.Get (TermInfoStrings.KeypadLocal);
 			if (keypadXmit != null) {
-				WriteConsole (keypadXmit);
+				WriteConsole (keypadXmit); // Needed to get the arrows working
 				if (keypadLocal != null)
 					endString += keypadLocal;
 			}
 
-			if (!ConsoleDriver.Isatty (MonoIO.ConsoleOutput) || !ConsoleDriver.Isatty (MonoIO.ConsoleInput))
-				throw new IOException ("Not a tty.");
-
 			origPair = reader.Get (TermInfoStrings.OrigPair);
 			origColors = reader.Get (TermInfoStrings.OrigColors);
 			setafcolor = MangleParameters (reader.Get (TermInfoStrings.SetAForeground));
@@ -169,13 +178,9 @@ namespace System {
 			if (resetColors != null)
 				endString += resetColors;
 
-			if (!ConsoleDriver.TtySetup (endString))
+			if (!ConsoleDriver.TtySetup (endString, out verase, out vsusp, out intr))
 				throw new IOException ("Error initializing terminal.");
 
-			if (enterCA != null && exitCA != null)
-				WriteConsole (enterCA);
-
-			stdout = Console.OpenStandardOutput (0);
 			stdin = Console.OpenStandardInput (0);
 			clear = reader.Get (TermInfoStrings.ClearScreen);
 			bell = reader.Get (TermInfoStrings.Bell);
@@ -189,7 +194,7 @@ namespace System {
 				csrVisible = reader.Get (TermInfoStrings.CursorVisible);
 
 			csrInvisible = reader.Get (TermInfoStrings.CursorInvisible);
-			if (term == "cygwin" || term == "linux" || term.StartsWith ("xterm") ||
+			if (term == "cygwin" || term == "linux" || (term != null && term.StartsWith ("xterm")) ||
 				term == "rxvt" || term == "dtterm") {
 				titleFormat = "\x1b]0;{0}\x7"; // icon + window title
 			} else if (term == "iris-ansi") {
@@ -204,6 +209,17 @@ namespace System {
 				home_1_1 = (cursorAddress != result);
 				cursorAddress = MangleParameters (result);
 			}
+
+			GetCursorPosition ();
+#if DEBUG
+			logger.WriteLine ("noGetPosition: {0} left: {1} top: {2}", noGetPosition, cursorLeft, cursorTop);
+			logger.Flush ();
+#endif
+			if (noGetPosition) {
+				WriteConsole (clear);
+				cursorLeft = 0;
+				cursorTop = 0;
+			}
 		}
 
 		static string MangleParameters (string str)
@@ -249,29 +265,111 @@ namespace System {
 			return 0;
 		}
 
+		void IncrementX ()
+		{
+			cursorLeft++;
+			if (cursorLeft >= WindowWidth) {
+				cursorTop++;
+				cursorLeft = 0;
+				if (cursorTop >= WindowHeight) {
+					// Writing beyond the initial screen
+					if (rl_starty != -1) rl_starty--;
+					cursorTop--;
+				}
+			}
+		}
+
+		public bool NotifyWrite (ConsoleKeyInfo key)
+		{
+			if (!inited)
+				return true;
+
+			switch (key.Key) {
+			case ConsoleKey.Backspace:
+				if (cursorLeft > 0) {
+					if (cursorLeft <= rl_startx && cursorTop == rl_starty)
+						return false;
+					cursorLeft--;
+					SetCursorPosition (cursorLeft, cursorTop);
+					WriteConsole (" ");
+					SetCursorPosition (cursorLeft, cursorTop);
+				}
+#if DEBUG
+				logger.WriteLine ("BS left: {0} top: {1}", cursorLeft, cursorTop);
+				logger.Flush ();
+#endif
+				return false;
+			case ConsoleKey.Tab:
+				int n = 8 - (cursorLeft % 8);
+				for (int i = 0; i < n; i++) IncrementX ();
+				return true;
+			case ConsoleKey.Clear:
+				return false;
+			case ConsoleKey.Enter:
+				cursorLeft = 0;
+				cursorTop++;
+				if (cursorTop >= WindowHeight) {
+					cursorTop--;
+					//TODO: scroll up
+				}
+#if DEBUG
+				logger.WriteLine ("ENTER left: {0} top: {1}", cursorLeft, cursorTop);
+				logger.Flush ();
+#endif
+				return true;
+			}
+			IncrementX ();
+#if DEBUG
+			logger.WriteLine ("left: {0} top: {1}", cursorLeft, cursorTop);
+			logger.Flush ();
+#endif
+			return true;
+		}
+
+		public bool NotifyWrite (char val)
+		{
+			return NotifyWrite (CreateKeyInfoFromInt (val));
+		}
+
 		public ConsoleColor BackgroundColor {
-			get { return bgcolor; }
+			get {
+				if (!inited) {
+					Init ();
+				}
+
+				return bgcolor;
+			}
 			set {
-				Init ();
+				if (!inited) {
+					Init ();
+				}
+
 				bgcolor = value;
 				WriteConsole (String.Format (setabcolor, TranslateColor (value)));
 			}
 		}
 
 		public ConsoleColor ForegroundColor {
-			get { return fgcolor; }
+			get {
+				if (!inited) {
+					Init ();
+				}
+
+				return fgcolor;
+			}
 			set {
-				Init ();
+				if (!inited) {
+					Init ();
+				}
+
 				fgcolor = value;
 				WriteConsole (String.Format (setafcolor, TranslateColor (value)));
 			}
 		}
 
+		// Only used once.
 		void GetCursorPosition ()
 		{
-			if (noGetPosition)
-				return;
-
 			int row = 0, col = 0;
 			bool prevEcho = Echo;
 			Echo = false;
@@ -330,36 +428,61 @@ namespace System {
 
 		public int BufferHeight {
 			get {
-				GetWindowDimensions ();
+				if (!inited) {
+					Init ();
+				}
+
 				return bufferHeight;
 			}
 			set {
+				if (!inited) {
+					Init ();
+				}
+
 				throw new NotSupportedException ();
 			}
 		}
 
 		public int BufferWidth {
 			get {
-				GetWindowDimensions ();
+				if (!inited) {
+					Init ();
+				}
+
 				return bufferWidth;
 			}
 			set {
+				if (!inited) {
+					Init ();
+				}
+
 				throw new NotSupportedException ();
 			}
 		}
 
 		public bool CapsLock {
-			get { throw new NotSupportedException (); }
+			get {
+				if (!inited) {
+					Init ();
+				}
+
+				throw new NotSupportedException ();
+			}
 		}
 
 		public int CursorLeft {
 			get {
-				Init ();
-				GetCursorPosition ();
+				if (!inited) {
+					Init ();
+				}
+
 				return cursorLeft;
 			}
 			set {
-				Init ();
+				if (!inited) {
+					Init ();
+				}
+
 				SetCursorPosition (value, CursorTop);
 				cursorLeft = value;
 			}
@@ -367,12 +490,17 @@ namespace System {
 
 		public int CursorTop {
 			get {
-				Init ();
-				GetCursorPosition ();
+				if (!inited) {
+					Init ();
+				}
+
 				return cursorTop;
 			}
 			set {
-				Init ();
+				if (!inited) {
+					Init ();
+				}
+
 				SetCursorPosition (CursorLeft, value);
 				cursorTop = value;
 			}
@@ -380,11 +508,17 @@ namespace System {
 
 		public bool CursorVisible {
 			get {
-				Init ();
+				if (!inited) {
+					Init ();
+				}
+
 				return cursorVisible;
 			}
 			set {
-				Init ();
+				if (!inited) {
+					Init ();
+				}
+
 				cursorVisible = value;
 				WriteConsole ((value ? csrVisible : csrInvisible));
 			}
@@ -393,24 +527,46 @@ namespace System {
 		// we have CursorNormal vs. CursorVisible...
 		[MonoTODO]
 		public int CursorSize {
-			get { return 1; }
-			set {}
+			get {
+				if (!inited) {
+					Init ();
+				}
+				return 1;
+			}
+			set {
+				if (!inited) {
+					Init ();
+				}
+			}
+
 		}
 
 		public bool Echo {
-			get { return !noEcho; }
+			get {
+				if (!inited) {
+					Init ();
+				}
+
+				return !noEcho;
+			}
 			set {
+				if (!inited) {
+					Init ();
+				}
+
 				if (value != noEcho)
 					return;
 
 				noEcho = !value;
-				ConsoleDriver.SetEcho (value);
 			}
 		}
 
 		public bool KeyAvailable {
 			get {
-				Init ();
+				if (!inited) {
+					Init ();
+				}
+
 				return (writepos > readpos || ConsoleDriver.InternalKeyAvailable (0) > 0);
 			}
 		}
@@ -425,35 +581,56 @@ namespace System {
 		}
 
 		public bool NumberLock {
-			get { throw new NotSupportedException (); }
+			get {
+				if (!inited) {
+					Init ();
+				}
+
+				throw new NotSupportedException ();
+			}
 		}
 
 		public string Title {
-			get { return title; }
+			get {
+				if (!inited) {
+					Init ();
+				}
+				return title;
+			}
 			
 			set {
-				Init ();
+				if (!inited) {
+					Init ();
+				}
+
 				title = value;
 				WriteConsole (String.Format (titleFormat, value));
 			}
 		}
 
 		public bool TreatControlCAsInput {
-			get { return controlCAsInput; }
+			get {
+				if (!inited) {
+					Init ();
+				}
+				return controlCAsInput;
+			}
 			set {
+				if (!inited) {
+					Init ();
+				}
+
 				if (controlCAsInput == value)
 					return;
 
-				Init ();
 				ConsoleDriver.SetBreak (value);
 				controlCAsInput = value;
 			}
 		}
 
+		// TODO: use this at the beginning and on every SIGWINCH
 		void GetWindowDimensions ()
 		{
-			//TODO: Handle SIGWINCH
-
 			/* Try the ioctl first */
 			if (!ConsoleDriver.GetTtySize (MonoIO.ConsoleOutput, out windowWidth, out windowHeight)) {
 				windowWidth = reader.Get (TermInfoNumbers.Columns);
@@ -475,61 +652,97 @@ namespace System {
 				}
 			}
 
-			//windowTop = 0;
-			//windowLeft = 0;
 			bufferHeight = windowHeight;
 			bufferWidth = windowWidth;
 		}
 
 		public int WindowHeight {
 			get {
+				if (!inited) {
+					Init ();
+				}
+
 				GetWindowDimensions ();
 				return windowHeight;
 			}
 			set {
+				if (!inited) {
+					Init ();
+				}
+
 				throw new NotSupportedException ();
 			}
 		}
 
 		public int WindowLeft {
 			get {
+				if (!inited) {
+					Init ();
+				}
+
 				//GetWindowDimensions ();
 				return 0;
 			}
 			set {
+				if (!inited) {
+					Init ();
+				}
+
 				throw new NotSupportedException ();
 			}
 		}
 
 		public int WindowTop {
 			get {
+				if (!inited) {
+					Init ();
+				}
+
 				//GetWindowDimensions ();
 				return 0;
 			}
 			set {
+				if (!inited) {
+					Init ();
+				}
+
 				throw new NotSupportedException ();
 			}
 		}
 
 		public int WindowWidth {
 			get {
+				if (!inited) {
+					Init ();
+				}
+
 				GetWindowDimensions ();
 				return windowWidth;
 			}
 			set {
+				if (!inited) {
+					Init ();
+				}
+
 				throw new NotSupportedException ();
 			}
 		}
 
 		public void Clear ()
 		{
-			Init ();
+			if (!inited) {
+				Init ();
+			}
+
 			WriteConsole (clear);
 		}
 
 		public void Beep (int frequency, int duration)
 		{
-			Init ();
+			if (!inited) {
+				Init ();
+			}
+
 			WriteConsole (bell);
 		}
 
@@ -537,6 +750,10 @@ namespace System {
 					int targetLeft, int targetTop, Char sourceChar,
 					ConsoleColor sourceForeColor, ConsoleColor sourceBackColor)
 		{
+			if (!inited) {
+				Init ();
+			}
+
 			throw new NotImplementedException ();
 		}
 
@@ -611,6 +828,10 @@ namespace System {
 
 		ConsoleKeyInfo ReadKeyInternal ()
 		{
+			if (!inited) {
+				Init ();
+			}
+
 			InitKeys ();
 			object o = null;
 			while (o == null) {
@@ -633,7 +854,9 @@ namespace System {
 
 		public ConsoleKeyInfo ReadKey (bool intercept)
 		{
-			Init ();
+			if (!inited)
+				Init ();
+
 			bool prevEcho = Echo;
 			if (prevEcho == intercept)
 				Echo  = !intercept;
@@ -645,20 +868,62 @@ namespace System {
 			return key;
 		}
 
+		public string ReadLine ()
+ 		{
+			if (!inited)
+				Init ();
+
+			bool prevEcho = Echo;
+			if (prevEcho == false)
+				Echo  = true;
+			StringBuilder builder = new StringBuilder ();
+			bool exit = false;
+			CStreamWriter writer = Console.stdout as CStreamWriter;
+			rl_startx = cursorLeft;
+			rl_starty = cursorLeft;
+			do {
+				ConsoleKeyInfo key = ReadKeyInternal ();
+				char c = key.KeyChar;
+				exit = (c == '\n');
+				if (!exit)
+					builder.Append (c);
+				if (writer != null)
+					writer.WriteKey (key);
+				else
+					Console.stdout.Write (c);
+			} while (!exit);
+			if (prevEcho == false)
+				Echo = prevEcho;
+			rl_startx = -1;
+			rl_starty = -1;
+			return builder.ToString ();
+ 		}
+
 		public void ResetColor ()
 		{
-			Init ();
+			if (!inited) {
+				Init ();
+			}
+
 			string str = (origPair != null) ? origPair : origColors;
 			WriteConsole (str);
 		}
 
 		public void SetBufferSize (int width, int height)
 		{
+			if (!inited) {
+				Init ();
+			}
+
 			throw new NotImplementedException (String.Empty);
 		}
 
 		public void SetCursorPosition (int left, int top)
 		{
+			if (!inited) {
+				Init ();
+			}
+
 			if (bufferWidth == 0)
 				GetWindowDimensions ();
 
@@ -668,8 +933,6 @@ namespace System {
 			if (top < 0 || top >= bufferHeight)
 				throw new ArgumentOutOfRangeException ("top", "Value must be positive and below the buffer height.");
 
-			Init ();
-
 			// Either CursorAddress or nothing.
 			// We might want to play with up/down/left/right/home when ca is not available.
 			if (cursorAddress == null)
@@ -683,12 +946,20 @@ namespace System {
 
 		public void SetWindowPosition (int left, int top)
 		{
+			if (!inited) {
+				Init ();
+			}
+
 			// No need to throw exceptions here.
 			//throw new NotSupportedException ();
 		}
 
 		public void SetWindowSize (int width, int height)
 		{
+			if (!inited) {
+				Init ();
+			}
+
 			// No need to throw exceptions here.
 			//throw new NotSupportedException ();
 		}
