@@ -3,11 +3,11 @@
 //
 // Authors:
 //   Sebastien Pouliot <sebastien@ximian.com>
-//   Kornl Pl <http://www.kornelpal.hu/>
+//   Kornél Pál <http://www.kornelpal.hu/>
 //   Jonathan Chambers <joncham@gmail.com>
 //
 // Copyright (C) 2004 Novell (http://www.novell.com)
-// Copyright (C) 2005 Kornl Pl
+// Copyright (C) 2005 Kornél Pál
 // Copyright (C) 2006 Jonathan Chambers
 //
 
@@ -188,7 +188,7 @@ namespace System
 		{
 			get
 			{
-				IntPtr pUnk = GetInterface (typeof (IDispatchMono));
+				IntPtr pUnk = GetInterface (typeof (IDispatch));
 				if (pUnk == IntPtr.Zero)
 					throw new InvalidComObjectException ("COM object that has been separated from its underlying RCW cannot be used.");
 				return pUnk;
@@ -211,11 +211,6 @@ namespace System
 			}
 		}
 
-		[Guid ("00020400-0000-0000-C000-000000000046")]
-		internal interface IDispatchMono
-		{
-		}
-
 		public override bool Equals (object obj)
 		{
 			if (obj == null)
